/*
 * Khoros: $Id: fileobj.h,v 1.7 1996/05/20 10:57:56 mirage Exp $
 */


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: file structure to keep track of all different
   >>>>		       forms a file can appear as.  This is
   >>>>		       the actual structure definition of the kobject
   >>>>		       with the type KOBJ_CMSFILE
   >>>>
   >>>>    Written By: Steven Jorgensen
   >>>>
   >>>>          Date: Apr 14, 1993 01:26
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _kcms_fileobj_h_
#define _kcms_fileobj_h_


	/*-------------------------------------*
	|       #include
	---------------------------------------*/

	/*-------------------------------------*
	|       #defines
	---------------------------------------*/

#define KCMS_FOBJ_MAX		 14L    /* number of attributes in table */

/*-- common attributes -----------------------------------------------*/

#define KCMS_FOBJ_BASENAME	1L
				/* KCMS_TYPE is #2 */
#define KCMS_FOBJ_FULLNAME	3L
#define KCMS_FOBJ_PARENTOBJ	4L
				/* KCMS_DATE is #2 */


/*-- file-specific attributes ----------------------------------------*/


/*------------------------------------------------------------------------
| Attribute:	KCMS_FOBJ_TYPE
| Type:		long
| Description:	The type of the the file object.
| Legal Values:	KCMS_FOBJ_TYPE_MISC
|		KCMS_FOBJ_TYPE_SRC
|		KCMS_FOBJ_TYPE_MAN
|		KCMS_FOBJ_TYPE_HELP
|		KCMS_FOBJ_TYPE_MANUAL
|		KCMS_FOBJ_TYPE_UIS
|		KCMS_FOBJ_TYPE_SCRIPT
|		KCMS_FOBJ_TYPE_DBM
|		KCMS_FOBJ_TYPE_INFO
|		KCMS_FOBJ_TYPE_CONFIG
| Access:	read/write
| Persistence:	stored
------------------------------------------------------------------------*/
#define KCMS_FOBJ_TYPE			6L

/*------------------------------------------------------------------------
| Attribute:	KCMS_FOBJ_SUBTYPE
| Type:		long
| Description:	The subtype of the file object.  Valid types are determined
|		by the file type of the object, which is specified with
|		the
|		.symbol - KCMS_FOBJ_TYPE
|		attribute.
| Access:	read/write
| Persistence:	stored
------------------------------------------------------------------------*/
#define KCMS_FOBJ_SUBTYPE		7L

/*------------------------------------------------------------------------
| Attribute:	KCMS_FOBJ_GENERATED
| Type:		long
| Description:	Specifies whether this file was auto-generated, and the
|		type of generated file.
| Legal Values:	KCMS_FOBJ_GEN_NONE
|		KCMS_FOBJ_GEN_MAIN
|		KCMS_FOBJ_GEN_INCL
|		KCMS_FOBJ_GEN_LFILE
|		KCMS_FOBJ_GEN_MAN1
|		KCMS_FOBJ_GEN_MAN3
|		KCMS_FOBJ_GEN_HELP
|		KCMS_FOBJ_GEN_GWFILE
|		KCMS_FOBJ_GEN_LINCL
|		KCMS_FOBJ_GEN_DOFILE
|		KCMS_FOBJ_GEN_COND
|		KCMS_FOBJ_GEN_SCRIPT
|		KCMS_FOBJ_GEN_STRUCT
| Access:	read/write
| Persistence:	stored
------------------------------------------------------------------------*/
#define KCMS_FOBJ_GENERATED		8L

/*------------------------------------------------------------------------
| Attribute:	KCMS_FOBJ_ACCESS
| Type:		long
| Description:	The access permissions on the file itself.
| Access:	read/write
| Persistence:	stored
------------------------------------------------------------------------*/
#define KCMS_FOBJ_ACCESS		9L

/*------------------------------------------------------------------------
| Attribute:	KCMS_FOBJ_DIRNAME
| Type:		kstring
| Description:	The fullpath of the directory which contains this file.
| Access:	read/write
| Persistence:	stored
------------------------------------------------------------------------*/
#define KCMS_FOBJ_DIRNAME		10L

/*------------------------------------------------------------------------
| Attribute:	KCMS_FOBJ_DIRNAME_TKN
| Type:		ktoken
| Description:	The token associated with the value of the
|		.symbol - KCMS_FOBJ_DIRNAME
|		attribute.
| Access:	read only
| Persistence:	transient
------------------------------------------------------------------------*/
#define KCMS_FOBJ_DIRNAME_TKN		11L

/*------------------------------------------------------------------------
| Attribute:	KCMS_FOBJ_BASENAME_TKN
| Type:		ktoken
| Description:	The token associated with the value of the KCMS_NAME
|		attribute, the basename of the file.
| Access:	read only
| Persistence:	transient
------------------------------------------------------------------------*/
#define KCMS_FOBJ_BASENAME_TKN		12L

/*------------------------------------------------------------------------
| Attribute:	KCMS_FOBJ_FULLNAME_TKN
| Type:		ktoken
| Description:	The token associated with the full path (KCMS_PATH) of
|		the file.
| Access:	read only
| Persistence:	transient
------------------------------------------------------------------------*/
#define KCMS_FOBJ_FULLNAME_TKN		13L

/*------------------------------------------------------------------------
| Attribute:	KCMS_FOBJ_MODIFIED
| Type:		int
| Description:	A boolean attribute which specifies whether the file
|		has been modified.
| Access:	read only
| Persistence:	transient
------------------------------------------------------------------------*/
#define KCMS_FOBJ_MODIFIED		14L


/* KCMS_FOBJ_TYPE can be one of: */
#define KCMS_FOBJ_TYPE_MISC	0L	/* misc files, no subtypes          */
#define KCMS_FOBJ_TYPE_SRC	1L	/* src file, see src subtypes       */
#define KCMS_FOBJ_TYPE_MAN	2L	/* man file, see man subtypes       */
#define KCMS_FOBJ_TYPE_HELP	3L	/* help file, see help subtypes     */
#define KCMS_FOBJ_TYPE_MANUAL	4L	/* manual file, see manual subtypes */
#define KCMS_FOBJ_TYPE_UIS	5L	/* UIS file, see manual subtypes    */
#define KCMS_FOBJ_TYPE_SCRIPT	6L	/* script file, see script subtypes */
#define KCMS_FOBJ_TYPE_DBM	7L	/* database file, see dbm subtypes  */
#define KCMS_FOBJ_TYPE_INFO	8L	/* object info: todo, bugs & done   */
#define KCMS_FOBJ_TYPE_CONFIG	9L	/* configuration, imake and similar */

/* KCMS_FOBJ_SUBTYPE can be one of: */

/* IFF KCMS_FOBJ_TYPE == KCMS_FOBJ_TYPE_MISC   */
#define KCMS_FOBJ_SUBTYPE_NONE	   -1L	/* no subtype                        */

/* IFF KCMS_FOBJ_TYPE == KCMS_FOBJ_TYPE_SRC    */
#define KCMS_FOBJ_SUBTYPE_C	    0L	/* C src file, suffix (.c)           */
#define KCMS_FOBJ_SUBTYPE_INCL	    1L	/* header file, suffix (.h)          */
#define KCMS_FOBJ_SUBTYPE_FORTRAN   2L	/* FORTRAN file, suffix (.f)         */
#define KCMS_FOBJ_SUBTYPE_CPLUSPLUS 3L	/* C++ file, suffixes (.cxx, .C .cc) */
#define KCMS_FOBJ_SUBTYPE_OBJC	    4L	/* Objective-C file, suffix (.m)     */
#define KCMS_FOBJ_SUBTYPE_YACC	    5L	/* YACC or bison file, suffix (.y)   */
#define KCMS_FOBJ_SUBTYPE_LEX	    6L	/* (f)lex file, suffixes (.l, .lex)  */
#define KCMS_FOBJ_SUBTYPE_STRUCT    7L	/* Structure Spec file, suffix (.x)  */

/* IFF KCMS_FOBJ_TYPE == KCMS_FOBJ_TYPE_MAN    */
#define KCMS_FOBJ_SUBTYPE_MAN1	   0L	/* program man file, in $TB/man1 dir */
#define KCMS_FOBJ_SUBTYPE_MAN3	   1L	/* library man file, in $TB/manoname */
#define KCMS_FOBJ_SUBTYPE_CAT1	   2L	/* program cat file, in $TB/cat1 dir */
#define KCMS_FOBJ_SUBTYPE_CAT3	   3L	/* library cat file, in $TB/catoname */

/* IFF KCMS_FOBJ_TYPE == KCMS_FOBJ_TYPE_HELP   */
#define KCMS_FOBJ_SUBTYPE_DOC	    0L	/* user help file, suffix (.doc)     */
#define KCMS_FOBJ_SUBTYPE_HELP	    1L	/* generated help file, suffix (.hlp)*/
#define KCMS_FOBJ_SUBTYPE_FORMATTED 2L	/* formatted doc or help file        */

/* IFF KCMS_FOBJ_TYPE == KCMS_FOBJ_TYPE_MANUAL */
#define KCMS_FOBJ_SUBTYPE_MS	   0L	/* generated ms manual file (.ms)   */
#define KCMS_FOBJ_SUBTYPE_MAN	   1L	/* user written manual file (.man)  */
#define KCMS_FOBJ_SUBTYPE_SEC	   2L	/* user written section file (.sec) */
#define KCMS_FOBJ_SUBTYPE_PS	   3L	/* generated postscript file (.ps)  */

/* IFF KCMS_FOBJ_TYPE == KCMS_FOBJ_TYPE_UIS    */
#define KCMS_FOBJ_SUBTYPE_FORM		0L
#define KCMS_FOBJ_SUBTYPE_PANE		1L
#define KCMS_FOBJ_SUBTYPE_MISC		2L

/* IFF KCMS_FOBJ_TYPE == KCMS_FOBJ_TYPE_SCRIPT */
#define KCMS_FOBJ_SUBTYPE_SH	   0L	/* Bourne Shell file (.sh)  */
#define KCMS_FOBJ_SUBTYPE_PERL	   1L	/* PERL script file (.perl) */
#define KCMS_FOBJ_SUBTYPE_CSH	   2L	/* C-Shell file (.csh)      */
#define KCMS_FOBJ_SUBTYPE_TCSH	   3L	/* Cornell's C-Shell (.csh) */

/* IFF KCMS_FOBJ_TYPE == KCMS_FOBJ_TYPE_DBM	 */
#define KCMS_FOBJ_SUBTYPE_CM	    0L	/* Config Management file (.cm)   */
#define KCMS_FOBJ_SUBTYPE_CI	    1L	/* Category Info file (.ci)       */
#define KCMS_FOBJ_SUBTYPE_SDBM_DIR  2L	/* A SDBM Dir file (.dir)         */
#define KCMS_FOBJ_SUBTYPE_SDBM_PAG  3L	/* A SDBM Page file (.pag)        */
#define KCMS_FOBJ_SUBTYPE_WORKSPACE 4L	/* A saved workspace file	*/

/* IFF KCMS_FOBJ_TYPE == KCMS_FOBJ_TYPE_INFO */
#define KCMS_FOBJ_SUBTYPE_TODO		0L
#define KCMS_FOBJ_SUBTYPE_BUGS		1L
#define KCMS_FOBJ_SUBTYPE_DONE		2L
#define KCMS_FOBJ_SUBTYPE_CHANGELOG	3L
#define KCMS_FOBJ_SUBTYPE_DESCRIPTION	4L

/* IFF KCMS_FOBJ_TYPE == KCMS_FOBJ_TYPE_CONFIG */
#define KCMS_FOBJ_SUBTYPE_IMAKE_CONFIG	0L
#define KCMS_FOBJ_SUBTYPE_IMAKEFILE	1L
#define KCMS_FOBJ_SUBTYPE_MAKEFILE	2L
#define KCMS_FOBJ_SUBTYPE_ALIAS_FILE	3L
#define KCMS_FOBJ_SUBTYPE_APP_DEFAULTS	4L

/* KCMS_FOBJ_GENERATED can be one of: (NOTE first six have keys) */
#define KCMS_FOBJ_GEN_NONE	   -1L   /* wasn't generated            */
#define KCMS_FOBJ_GEN_MAIN	   0L    /* *.c                         */
#define KCMS_FOBJ_GEN_INCL	   1L    /* *.h                         */
#define KCMS_FOBJ_GEN_LFILE	   2L    /* library file                */
#define KCMS_FOBJ_GEN_MAN1	   3L    /* *.1                         */
#define KCMS_FOBJ_GEN_MAN3	   4L    /* *.3                         */
#define KCMS_FOBJ_GEN_HELP	   5L    /* *.hlp                       */
#define KCMS_FOBJ_GEN_GWFILE	   6L    /* usage.c                     */
#define KCMS_FOBJ_GEN_LINCL	   7L    /* lib's public include file   */
#define KCMS_FOBJ_GEN_DOFILE	   8L    /* do_{name}.c                 */
#define KCMS_FOBJ_GEN_COND	   9L    /* form_info.h, form_info.c,   */
				         /* form_drv.c,  form_init.c    */
#define KCMS_FOBJ_GEN_SCRIPT	  10L    /* *.sh                        */
#define KCMS_FOBJ_GEN_STRUCT	  11L    /* *.x generated file io*.[ch] */

/* KCMS_FOBJ_ACCESS can be one of: */
#define KCMS_FOBJ_ACCESS_RDWR	   0L	/* Read/Write Access */
#define KCMS_FOBJ_ACCESS_RDONLY	   1L	/* Read Only  Access */

	/*-------------------------------------*
	|       typedefs
	---------------------------------------*/

	/*-------------------------------------*
	|       global variable declarations
	---------------------------------------*/


	/*-------------------------------------*
	|       macros
	---------------------------------------*/


	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/


#endif /* _kcms_fileobj_h_ */
/* Don't add after this point */
