/*
 * Khoros: $Id: klibm.h,v 1.4 1996/04/04 22:38:54 fas Exp $
 */


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Definitions and prototypes for library klibm
   >>>>
   >>>>       Toolbox: bootstrap
   >>>>
   >>>>    Written By: Jeremy Worley & Mark Young
   >>>>
   >>>>          Date: Apr 04, 1994
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _klibm_klibm_h_
#define _klibm_klibm_h_


	/*-------------------------------------*
	|       #include
	---------------------------------------*/

#include <math.h>
#include <klibm/kdata.h>


	/*-------------------------------------*
	|       #defines
	---------------------------------------*/

/*
 * Standard data type defines
 *	KANYTYPE   all ones
 *	KBIT	   1
 *	KBYTE      2
 *	KUBYTE     4
 *	KSHORT     8
 *	KUSHORT    16
 *	KINT       32
 *	KUINT      64
 *	KLONG      128
 *	KULONG     256
 *	KFLOAT     512
 *	KDOUBLE    1024
 *	KCOMPLEX   2048
 *	KDCOMPLEX  4096
 *	KSTRING    8192
 *	KSTRUCT    16384
 *	KLOGICAL   32768
 *	KBIT_LSB   65536
 */
#define KANYTYPE	(~0)
#define KUNDEFINED	(0)
#define KBIT		(1<<0)
#define KBYTE		(1<<1)
#define KUBYTE		(1<<2)
#define KSHORT		(1<<3)
#define KUSHORT		(1<<4)
#define KINT		(1<<5)
#define KUINT		(1<<6)
#define KLONG		(1<<7)
#define KULONG		(1<<8)
#define KFLOAT		(1<<9)
#define KDOUBLE		(1<<10)
#define KCOMPLEX	(1<<11)
#define KDCOMPLEX	(1<<12)
#define KSTRING		(1<<13)
#define KSTRUCT		(1<<14)
#define KBIT_LSB	(1<<15)
#define KLOGICAL	(1<<16)

        /*
         *  Definitions for some possibly machine dependent constants, more
         *  dependable than math.h since math.h is often missing (Way to go
         *  SYS V!).  Some installations may want to put values in here that
         *  more closely match the machine.
         */
#define KMAXFLOAT 1.0e+38       /* Good enough for most applications */
#define KMAXLINT  2147483647    /* Assuming 32 bit 2's complement */
#define KMAXSINT  32767         /* Assuming 16 bit 2's complement */
#define KGAMMA    0.57721566490153286060
#define KDEG      57.29577951308232087680
#define KPHI      1.61803398874989484820
#define KE        2.71828182845904523536
#define KLOG2E    1.4426950408889634074
#define KLOG10E   0.43429448190325182765
#define KLN2      0.69314718055994530942
#define KLN10     2.30258509299404568402
#define KPI       3.14159265358979323846
#define K2PI      (KPI * 2)
#define KPI_2     1.57079632679489661923
#define KPI_4     0.78539816339744830962
#define K1_PI     0.31830988618379067154
#define K2_PI     0.63661977236758134308
#define KSQRTPI   1.77245385090551602730
#define KSQRT2    1.41421356237309504880
#define KSQRT1_2  0.70710678118654752440
#define KEPSILON  0.00001

#define KDATAOP_NOP       0
#define KDATAOP_SCALE     1
#define KDATAOP_NORMALIZE 2

#define KLIN_DETERMINATE  0x1
#define KLIN_INVERSE      0x2

	/*-------------------------------------*
	|       typedefs
	---------------------------------------*/

	/*-------------------------------------*
	|       global variable declarations
	---------------------------------------*/

	/*-------------------------------------*
	|       macros
	---------------------------------------*/

/************************************************************
*
*  Routine Name: kmax - return the greater of two values.
*       Purpose: The kmax function obtains the larger of the two
*		 input arguments.
*		 This is a macro, so any data type is supported.
*         Input: x - a variable of any base data type.
*	         y - a variable of any base data type.
*        Output: none
*       Returns: the larger value of the two input arguments.
*  Restrictions:
*    Written By: Mark Young
*          Date: Aug 10, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: kmax(x, y)
*
*************************************************************/
#define kmax(x,y) (((x) > (y)) ? (x) : (y))

/************************************************************
*
*  Routine Name: kmin - return the lessor of two values.
*       Purpose: The kmin function obtains the smaller of the two input
*		 arguments.
*		 This is a macro, so any data type is supported.
*         Input: x - a variable of any base data type.
*	         y - a variable of any base data type.
*        Output: none
*       Returns: the smaller value of the two input arguments.
*  Restrictions:
*    Written By: Mark Young
*          Date: Aug 10, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: kmin(x, y)
*
*************************************************************/
#define kmin(x,y) (((x) < (y)) ? (x) : (y))

/************************************************************
*
*  Routine Name: krange - return a ranged value
*       Purpose: The krange function obtains the range value, given a
*		 value, and a lower and upper bound.  The value returned
*		 will be lower >= value <= upper.
*         Input: lower - a variable of any base data type.
*	         value - a variable of any base data type.
*	         upper - a variable of any base data type.
*        Output: none
*       Returns: the range value of the three input arguments.
*  Restrictions:
*    Written By: Mark Young
*          Date: Aug 10, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: krange(lower, value, upper)
*
*************************************************************/
#define krange(x, w, y) (kmax(x, kmin(w, y)))

/************************************************************
*
*  Routine Name: kmin - return the lessor of two values.
*       Purpose: The kmin function obtains the smaller of the two input
*		 arguments.
*		 This is a macro, so any data type is supported.
*         Input: x - a variable of any base data type.
*	         y - a variable of any base data type.
*        Output: none
*       Returns: the smaller value of the two input arguments.
*  Restrictions:
*    Written By: Mark Young
*          Date: Aug 10, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: kmin(x, y)
*
*************************************************************/
#define kmin3(x0, x1, x2) ((kmin(x0,x1) < x2) ? kmin(x0,x1) : x2)

/************************************************************
*
*  Routine Name: kmin - return the lessor of two values.
*       Purpose: The kmin function obtains the smaller of the two input
*		 arguments.
*		 This is a macro, so any data type is supported.
*         Input: x - a variable of any base data type.
*	         y - a variable of any base data type.
*        Output: none
*       Returns: the smaller value of the two input arguments.
*  Restrictions:
*    Written By: Mark Young
*          Date: Aug 10, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: kmin(x, y)
*
*************************************************************/
#define kmax3(x0, x1, x2) ((kmax(x0,x1) > x2) ? kmax(x0,x1) : x2)

/************************************************************
*
*  Routine Name: kmin - return the lessor of two values.
*       Purpose: The kmin function obtains the smaller of the two input
*		 arguments.
*		 This is a macro, so any data type is supported.
*         Input: x - a variable of any base data type.
*	         y - a variable of any base data type.
*        Output: none
*       Returns: the smaller value of the two input arguments.
*  Restrictions:
*    Written By: Mark Young
*          Date: Aug 10, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: kmin(x, y)
*
*************************************************************/
#define kmin4(x0, x1, x2, x3) ((kmin(x0,x1) < kmin(x2,x3)) \
                                          ? kmin(x0,x1) : kmin(x2,x3))

/************************************************************
*
*  Routine Name: kmin - return the lessor of two values.
*       Purpose: The kmin function obtains the smaller of the two input
*		 arguments.
*		 This is a macro, so any data type is supported.
*         Input: x - a variable of any base data type.
*	         y - a variable of any base data type.
*        Output: none
*       Returns: the smaller value of the two input arguments.
*  Restrictions:
*    Written By: Mark Young
*          Date: Aug 10, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: kmin(x, y)
*
*************************************************************/
#define kmax4(x0, x1, x2, x3) ((kmax(x0,x1) > kmax(x2,x3)) \
                                          ? kmax(x0,x1) : kmax(x2,x3))

/************************************************************
*
*  Routine Name: kmin - return the lessor of two values.
*       Purpose: The kmin function obtains the smaller of the two input
*		 arguments.
*		 This is a macro, so any data type is supported.
*         Input: x - a variable of any base data type.
*	         y - a variable of any base data type.
*        Output: none
*       Returns: the smaller value of the two input arguments.
*  Restrictions:
*    Written By: Mark Young
*          Date: Aug 10, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: kmin(x, y)
*
*************************************************************/
#define kabs(x) (((x) < 0) ? (-(x)) : (x))

/************************************************************
*
*  Routine Name: kmin - return the lessor of two values.
*       Purpose: The kmin function obtains the smaller of the two input
*		 arguments.
*		 This is a macro, so any data type is supported.
*         Input: x - a variable of any base data type.
*	         y - a variable of any base data type.
*        Output: none
*       Returns: the smaller value of the two input arguments.
*  Restrictions:
*    Written By: Mark Young
*          Date: Aug 10, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: kmin(x, y)
*
*************************************************************/
#define ksqr(x) ((x)*(x))

/************************************************************
*
*  Routine Name: kmin - return the lessor of two values.
*       Purpose: The kmin function obtains the smaller of the two input
*		 arguments.
*		 This is a macro, so any data type is supported.
*         Input: x - a variable of any base data type.
*	         y - a variable of any base data type.
*        Output: none
*       Returns: the smaller value of the two input arguments.
*  Restrictions:
*    Written By: Mark Young
*          Date: Aug 10, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: kmin(x, y)
*
*************************************************************/
#define kdegrees_radians(degrees) ((KPI/180.0)*(degrees))

/************************************************************
*
*  Routine Name: kmin - return the lessor of two values.
*       Purpose: The kmin function obtains the smaller of the two input
*		 arguments.
*		 This is a macro, so any data type is supported.
*         Input: x - a variable of any base data type.
*	         y - a variable of any base data type.
*        Output: none
*       Returns: the smaller value of the two input arguments.
*  Restrictions:
*    Written By: Mark Young
*          Date: Aug 10, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: kmin(x, y)
*
*************************************************************/
#define kradians_degrees(radians) ((180.0/KPI)*(radians))
 
/*
 *   The following macro is used to interpolate the intercept from a vector
 *   described by the two points (x0,y0) & (x1,y1) and a value along the
 *   vector, y2, the macro will return the corresponding x2 value.
 */
/************************************************************
*
*  Routine Name: kintercept - interpolate the intercept
*       Purpose: The following macro is used to interpolate the intercept
*		 from a vector described by the two points (x0,y0) & (x1,y1)
*		 and a value along the vector, y2, the macro will return the
*		 corresponding x2 value.
*		 This is a macro, so any data type is supported.
*         Input: x0 - a variable of any base data type.
*	         x1 - a variable of any base data type.
*	         y0 - a variable of any base data type.
*	         y1 - a variable of any base data type.
*	         y2 - a variable of any base data type.
*        Output: none
*       Returns: the intercept point
*  Restrictions:
*    Written By: Mark Young
*          Date: Aug 10, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: kintercept(x0, x1, y0, y1, y2)
*
*************************************************************/
#define kintercept(x0,x1,y0,y1,y2)   ((y1) != (y0) \
        ? ((x0) + (((x1) - (x0))*((y2) - (y0)))/((y1) - (y0))) \
        : (((x0) + (x1)) / 2.0))


	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

/* klibm.c */
extern double (*kacos)     PROTO((double));
extern double (*kasin)     PROTO((double));
extern double (*katan2)    PROTO((double, double));
extern double (*katan)     PROTO((double));
extern double (*ksinh)     PROTO((double));
extern double (*kcosh)     PROTO((double));
extern double (*ktanh)     PROTO((double));
extern double (*ksin)      PROTO((double));
extern double (*kcos)      PROTO((double));
extern double (*ktan)      PROTO((double));
extern double (*kceil)     PROTO((double));
extern double (*kfloor)    PROTO((double));
extern double (*kexp)      PROTO((double));
extern double (*klog)      PROTO((double));
extern double (*klog10)    PROTO((double));
extern double (*kpow)      PROTO((double, double));
extern double (*ksqrt)     PROTO((double));
extern double (*kfabs)     PROTO((double));
extern double (*kfmod)     PROTO((double, double));
extern double (*kfrexp)    PROTO((double, int *));
extern double (*kldexp)    PROTO((double, int));
extern double (*kmodf)     PROTO((double, double *));
extern double (*khypot)    PROTO((double, double));
extern double (*ktrunc)    PROTO((double));
extern double (*kgamma)    PROTO((double));
extern double (*kcbrt)     PROTO((double));
extern double (*kj0)       PROTO((double));
extern double (*kj1)       PROTO((double));
extern double (*kjn)       PROTO((int, double));
extern double (*ky0)       PROTO((double));
extern double (*ky1)       PROTO((double));
extern double (*kyn)       PROTO((int, double));
extern double (*kerf)      PROTO((double));
extern double (*kerfc)     PROTO((double));
extern double (*kasinh)    PROTO((double));
extern double (*kacosh)    PROTO((double));
extern double (*katanh)    PROTO((double));
extern double (*kexpm1)    PROTO((double));
extern double (*kexp2)     PROTO((double));
extern double (*kexp10)    PROTO((double));
extern double (*klog1p)    PROTO((double));
extern double (*klog2)     PROTO((double));

extern long   (*krandom)   PROTO((void));
extern void   (*ksrandom)  PROTO((int));


/* kcomplex.c */
kdcomplex kdcadd	PROTO((kdcomplex, kdcomplex));
kdcomplex kdcsub	PROTO((kdcomplex, kdcomplex));
kdcomplex kdcmult	PROTO((kdcomplex, kdcomplex));
kdcomplex kdcdiv	PROTO((kdcomplex, kdcomplex));
kdcomplex kdcsqrt	PROTO((kdcomplex));
kdcomplex kdcexp	PROTO((kdcomplex));
kdcomplex kdclog	PROTO((kdcomplex));
kdcomplex kdcpow	PROTO((kdcomplex, kdcomplex));
kdcomplex kdccomp	PROTO((double, double));
kdcomplex kdcconj	PROTO((kdcomplex));
kdcomplex kcomp2dcomp	PROTO((kcomplex));
kdcomplex kdcr2p	PROTO((kdcomplex));
kdcomplex kdcp2r	PROTO((kdcomplex));
kcomplex  kcadd		PROTO((kcomplex, kcomplex));
kcomplex  kcsub		PROTO((kcomplex, kcomplex));
kcomplex  kcmult	PROTO((kcomplex, kcomplex));
kcomplex  kcdiv		PROTO((kcomplex, kcomplex));
kcomplex  kcsqrt	PROTO((kcomplex));
kcomplex  kcexp		PROTO((kcomplex));
kcomplex  kclog		PROTO((kcomplex));
kcomplex  kcpow		PROTO((kcomplex, kcomplex));
kcomplex  kccomp	PROTO((double, double));
kcomplex  kcconj	PROTO((kcomplex));
kcomplex  kdcomp2comp	PROTO((kdcomplex));
kcomplex  kcr2p		PROTO((kcomplex));
kcomplex  kcp2r		PROTO((kcomplex));
double	  kdcmagsq	PROTO((kdcomplex));
double	  kdcmag	PROTO((kdcomplex));
double	  kdclogmagp1	PROTO((kdcomplex));
double	  kdclogmagsqp1	PROTO((kdcomplex));
double	  kdclogmagsq	PROTO((kdcomplex));
double	  kdclogmag	PROTO((kdcomplex));
double	  kdcang	PROTO((kdcomplex));
double	  kdcreal	PROTO((kdcomplex));
double	  kdcimag	PROTO((kdcomplex));
float	  kcmagsq	PROTO((kcomplex));
float	  kcmag		PROTO((kcomplex));
float	  kclogmagp1	PROTO((kcomplex));
float	  kclogmagsqp1	PROTO((kcomplex));
float	  kclogmagsq	PROTO((kcomplex));
float	  kclogmag	PROTO((kcomplex));
float	  kcang		PROTO((kcomplex));
float	  kcreal	PROTO((kcomplex));
float	  kcimag	PROTO((kcomplex));
int	  kdcomplex_to_arrays PROTO((kdcomplex *, int, double **, double **));

/* extensions.c */
int    kpowtwo	 PROTO((int));
double kfact	 PROTO((double));
double kimpulse  PROTO((double));
double kstep	 PROTO((double));
double ksign	 PROTO((double));
double ksinc	 PROTO((double));
double krecip	 PROTO((double));
double kneg	 PROTO((double));
double kfraction PROTO((double));
unsigned long   kset	 PROTO((unsigned long));
unsigned long   kclear	 PROTO((unsigned long));
unsigned long   knot	 PROTO((unsigned long));
long   klogical_not PROTO((long));
double klogn     PROTO((double, double));

/* flin.c */
void kblas_sscal PROTO((float *, float, int, int));
void kblas_saxpy PROTO((float *, float *, float, int, int, int));
void kblas_sswap PROTO((float *, float *, int, int, int));
int  klin_sgefa PROTO((float *, int, int, int *));
int  klin_sgedi PROTO((float *, int, int, int *, int, float *, float *));

/* dlin.c */
void kblas_dscal PROTO((double *, double, int, int));
void kblas_daxpy PROTO((double *, double *, double, int, int, int));
void kblas_dswap PROTO((double *, double *, int, int, int));
int  klin_dgefa PROTO((double *, int, int, int *));
int  klin_dgedi PROTO((double *, int, int, int *, int, double *, double *));
   
/* fmatrix.c */
float  kfmatrix_inner_prod  PROTO((float *, float *, int));
int    kfmatrix_vector_prod PROTO((float *, float *, int, int, float *));
int    kfmatrix_princ_axis  PROTO((float *, int, float *));
int    kfmatrix_multiply    PROTO((float *, int, int, 
				   float *, int, int, float *));
int    kfmatrix_clear	    PROTO((int, int, float *));
int    kfmatrix_identity    PROTO((int, int, float *));
int    kfmatrix_inverse	    PROTO((float *, int, float *));

/* dmatrix.c */
double kdmatrix_inner_prod  PROTO((double *, double *, int));
int    kdmatrix_vector_prod PROTO((double *, double *, int, int, double *));
int    kdmatrix_princ_axis  PROTO((double *, int, double *));
int    kdmatrix_multiply    PROTO((double *, int, int, 
				   double *, int, int, double *));
int    kdmatrix_clear	    PROTO((int, int, double *));
int    kdmatrix_identity    PROTO((int, int, double *));
int    kdmatrix_inverse	    PROTO((double *, int, double *));

/* noise.c */
double kurng PROTO((void));
int    kgen_gauss    PROTO((int, double, double, double *));
int    kgen_expon    PROTO((int, double, double *));
int    kgen_unif     PROTO((int, double, double, double *));
int    kgen_rayleigh PROTO((int, double, double *));
int    kgen_poisson  PROTO((int, double, double, double *));

/* signal.c */
int kgen_sine	PROTO((int, double, double, double, double, double *));
int kgen_linear PROTO((int, double, double, double, double, double, double, double, double *));
int kgen_sinec	PROTO((int, double, double, double, int, double *));

#endif /* _klibm_klibm_h_ */
/* Don't add after this point */
