/*
 * Khoros: $Id: karray.h,v 1.6 1996/04/04 22:38:58 fas Exp $
 */


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose:  To provide function declarations and
   >>>>			definitions for the kutils array utilities
   >>>>
   >>>>    Written By:  Mark Young
   >>>>
   >>>>          Date:  Nov 29, 1992 09:56
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _kutils_karray_h_
#define _kutils_karray_h_

        /*-------------------------------------*
        |       #include
        ---------------------------------------*/

        /*-------------------------------------*
        |       defines
        ---------------------------------------*/

/*
 *  Defines used for karray_dirlist for it's list mode
 */
#define KPATH    (1 << 0)
#define KFILE    (1 << 1)
#define KDIR     (1 << 2)
#define KDOT     (1 << 3)
#define KLINK    (1 << 4)
#define KSOCK    (1 << 5)
#define KREAD    (1 << 6)
#define KWRITE   (1 << 7)
#define KEXEC    (1 << 8)
#define KRECURSE (1 << 9)


        /*-------------------------------------*
        |       typedefs
        ---------------------------------------*/


        /*-------------------------------------*
        |       global variable declarations
        ---------------------------------------*/


        /*-------------------------------------*
        |       macros
        ---------------------------------------*/

/************************************************************
*
*  Routine Name: karray_add - add an entry into the array list
*
*       Purpose: Adds an entry to the array list.  This is done by
*                adding the entry to the end of the array list
*                (if the array currently exists).  If the array is
*                currently NULL then the new item is returned as the
*                first entry of the array list.  If the array list is
*                not NULL then the entry is added to the end of the
*                list and the original array is passed back to the calling
*                routine.
*
*                The routine first scans the array to make sure the
*                entry is not already on the list, if so then we
*                don't change original array.
*
*         Input: array      - The current array in which we will be adding the
*                             entry to (if NULL then return the newly
*                             malloc'ed array list).
*                entry      - The entry identifier to be added to the array
*                             list
*                num        - The number of entry in the list; ignored if
*                             list is NULL
*
*       Returns: returns the modified array list.
*    Written By: Mark Young
*          Date: Nov 28, 1992
*   Declaration: <type> *karray_add(
*		 !   <type> *array,
*		 !   int     type,
*		 !   <type>  entry,
*		 !   size_t  num)
*
*************************************************************/

#define karray_add(array, type, entry, num) \
		karray_insert(array, type, entry, num, KLIST_TAIL, FALSE)

/************************************************************
*
*  Routine Name: karray_copy - copy an array of strings
*
*       Purpose: This module is used to copy the input array
*                of strings into a new array.  If the "copy_entries"
*                parameter is TRUE, then each entry will also be
*                copied rather than just copying the pointers.
*
*         Input: array         - the array that is to be sorted
*                num          - the number of entries in the array
*                copy_entries - whether to kmalloc and copy each entry or not
*
*       Returns: a pointer to the new array, or NULL on error
*    Written By: Mark Young
*          Date: Jul 03, 1992
*  Side Effects: kmallocs the space for the new array
*   Declaration: <type> *karray_copy(
*		 !   <type> *array,
*		 !   int     type,
*		 !   size_t  num,
*		 !   int     copy_entries)
*
*************************************************************/
 
#define karray_copy(array, type, num, copy_entries) \
		karray_merge(array, NULL, type, num, (size_t) 0, copy_entries)


        /*-------------------------------------*
        |       routine definitions
        ---------------------------------------*/

void   *karray_insert	 PROTO((void *, int, kvalist));
void   *karray_delete	 PROTO((void *, int, kvalist));
ssize_t karray_locate	 PROTO((void *, int, kvalist));
void   *karray_sort	 PROTO((void *, int, size_t, int));
void   *karray_merge	 PROTO((void *, void *, int, size_t, size_t, int));
char   *karray_to_string PROTO((char **, int, size_t, char *));
void    karray_free	 PROTO((void *, int, size_t,void (*) PROTO((void *))));
klist  *karray_to_list	 PROTO((void *, int, size_t));
char  **karray_filelist  PROTO((char *, char *, int, size_t *));
char  **karray_dirlist	 PROTO((char *, char *, char *, int, int, size_t *));

#endif /* _kutils_karray_h_ */
/* Don't add after this point. */
