/*
 * Khoros: $Id: lib.c,v 1.4 1996/04/04 22:41:04 fas Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lib.c,v 1.4 1996/04/04 22:41:04 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  Static:
   >>>>  Public:
   >>>>             ReadLine()
   >>>>             ProcessLine()
   >>>>             WriteLine()
   >>>>             mycpp()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "kgenmake.h"

static int   token_last = -1;
static int   token_next = 1;
static int   token_tablesize = 0;
static char  **token_table = NULL;
static kilist **hash_table = NULL;

char *kstrcpy(KPARAM(char *, istr1), KPARAM(char *, istr2))
   KDECL(char *istr1;)
   KDECL(char *istr2;)
{
	char *string = istr1;
	if (istr1 == NULL)
	   return(NULL);
	else if (istr2 == NULL)
	   return(istr1);

	/*EMPTY*/
	while ((*istr1++ = *istr2++) != '\0');
	return(string);
}

/*-----------------------------------------------------------
|
|  Routine Name: kilist_head
|       Purpose: find the head of a list
|         Input: list - list to search
|       Returns: the head of list on success, NULL otherwise
|    Written By: Mark Young
|          Date: Mar 06, 1995
| Modifications: simplified version of klist_head from kutils (SJ)
|
------------------------------------------------------------*/
kilist *kilist_head(KPARAM(kilist *, list))
   KDECL(kilist *list;)
{
   if (list == NULL)
      return NULL;

   while (list->prev != NULL && (int) list->head == FALSE)
      list = list->prev;

   return list;
}

/*-----------------------------------------------------------
|
|  Routine Name: kilist_tail
|       Purpose: find the tail of a list
|         Input: list - list to search
|       Returns: the tail of list on success, NULL otherwise
|    Written By: Mark Young
|          Date: Mar 06, 1995
| Modifications: simplified version of klist_tail from kutils (SJ)
|
------------------------------------------------------------*/
kilist *kilist_tail(KPARAM(kilist *, list))
   KDECL(kilist *list;)
{
   if (list == NULL)
      return NULL;

   while (list->next != NULL && (int) list->next->head == FALSE)
      list = list->next;

   return list;
}

/*-----------------------------------------------------------
|
|  Routine Name: kilist_locate
|       Purpose: search for a list entry
|         Input: list - list to search
|                identifier - number to search for
|       Returns: the items kilist entry on success, NULL otherwise
|    Written By: Mark Young
|          Date: Mar 06, 1995
| Modifications: simplified version of klist_locate from kutils (SJ)
|
------------------------------------------------------------*/
kilist *kilist_locate( KPARAM(kilist *,list), KPARAM(int, identifier))
   KDECL(kilist *list;)
   KDECL(int    identifier;)
{
   if (list == NULL)
      return NULL;
   do
   {
      if (list->identifier == identifier)
         return list;

      list = list->next;
   } while (list != NULL && (int) list->head == FALSE);
   return NULL;
}

/*-----------------------------------------------------------
|
|  Routine Name: kilist_add
|       Purpose: add an entry to the end of a list
|         Input: list - list to add to
|		 identifier - number to identify the data with
|		 client_data - string to store.
|       Returns: the head of list on success, NULL otherwise
|    Written By: Mark Young
|          Date: Mar 06, 1995
| Modifications: simplified version of klist_insert from kutils (SJ)
|
------------------------------------------------------------*/
kilist *kilist_add(
	KPARAM( kilist *, list),
	KPARAM(int, identifier),
	KPARAM(char *, client_data))
   KDECL(kilist *list;)
   KDECL(int   identifier;)
   KDECL(char  *client_data;)
{
   kilist *entry;

   entry = (kilist *) kcalloc(1, sizeof(kilist));
   entry->identifier = identifier;
   entry->client_data = client_data;

   if (list == NULL)
   {
      entry->head = TRUE;
      list = entry;
      list->next  =
      list->prev  = NULL;
      return list;
   }

   list = kilist_tail(list);

   if (list->next != NULL)
      list->next->prev = entry;
   entry->next = list->next;
   list->next  = entry;
   entry->prev = list;
   return kilist_head(entry);
}

/*-----------------------------------------------------------
|
|  Routine Name: kilist_delete
|       Purpose: delete an entry from the list
|         Input: list - list to delete from
|                identifier - number to identify the data with
|                client_data - string to store.
|       Returns: the head of list on success, NULL otherwise
|    Written By: Mark Young
|          Date: Mar 06, 1995
| Modifications: simplified version of klist_insert from kutils (SJ)
|
------------------------------------------------------------*/
kilist *kilist_delete(KPARAM(kilist *, list), KPARAM(int, identifier))
   KDECL(kilist *list;)
   KDECL(int    identifier;)
{
   kilist *entry;

   if ((entry = kilist_locate(list, identifier)) == NULL)
      return list;

   if (entry->next != NULL)
   {
      entry->next->prev = entry->prev;
      if ((int) entry->head == TRUE)
         entry->next->head = TRUE;
   }
   if (entry->prev != NULL)
      entry->prev->next = entry->next;

   if (entry == list && entry->next != NULL)
      list = entry->next;
   else if (entry == list)
      list = entry->prev;

   kfree_and_NULL(entry);
   return list;
}

/*-----------------------------------------------------------
|
|  Routine Name: kstring_3cat
|       Purpose: concat 3 strings together
|         Input: istr1 - input string 1
|		 istr2 - input string 2
|		 istr3 - input string 3
|		 ostr  - output string
|       Returns: ostr if it is non-NULL, the kmalloc'ed
|                string if ostr is NULL, or NULL on error.
|    Written By: Steven Jorgensen & Nick Ruprecht
|          Date: Jul 03, 1992
| Modifications: simplified version of kstring_3cat from kutils (SJ)
|
------------------------------------------------------------*/
char *kstring_3cat(
	KPARAM(char *, istr1),
	KPARAM(char *, istr2),
	KPARAM(char *, istr3),
	KPARAM(char *, ostr) )
   KDECL(char *istr1;)
   KDECL(char *istr2;)
   KDECL(char *istr3;)
   KDECL(char *ostr;)
{
   int n1   = kstrlen(istr1),
       n2   = kstrlen(istr2),
       n3   = kstrlen(istr3),
       nout = n1+n2+n3;

   if (nout <= 0)
      return NULL;
   if (!ostr && (ostr = kmalloc(nout+1)) == NULL)
   {
      return(NULL);
   }
   if (n1 > 0 && ostr != istr1)
      kstrcpy(ostr, istr1);
   if (n2 > 0)
      kstrcpy(ostr + n1, istr2);
   if (n3 > 0)
      kstrcpy(ostr + n1 + n2, istr3);

   ostr[nout] = '\0';
   return ostr;
}

/*-----------------------------------------------------------
|
|  Routine Name: _cleanup_string
|       Purpose: cleanup a string of beginning and ending white space
|         Input: istr - input string
|                ostr - output string
|       Returns: ostr if it is not NULL, a pointer to the
|                resulting malloc'ed string if it is NULL.  NULL on
|                an error.
|    Written By: Mark Young
|          Date: Jul 03, 1992
| Modifications: simplified version of kstring_3cat from kutils (SJ)
--------------------------------------------------------------*/
static char *_cleanup_string(
	KPARAM(register char *, string),
	KPARAM(int, cleanall))
   KDECL(register char *string;)
   KDECL(int cleanall;)
{
   register char *temp;

   if (cleanall)
   {
      while (isspace(*string))
         string++;
   }

   if (*string == '\0')
      return NULL;

   temp = string + kstrlen(string);
   while (isspace(*--temp))
      ;
   *(temp+1) = '\0';

   return string;
}

/*-----------------------------------------------------------
|
|  Routine Name: kstring_cleanup
|       Purpose: cleanup a string of beginning and ending white space
|         Input: istr - input string
|                ostr - output string
|       Returns: ostr if it is not NULL, a pointer to the
|		 resulting malloc'ed string if it is NULL.  NULL on
|		 an error.
|    Written By: Mark Young
|          Date: Jul 03, 1992
| Modifications: simplified version of kstring_3cat from kutils (SJ)
| ------------------------------------------------------------*/
char *kstring_cleanup(
	KPARAM(char *, istr),
	KPARAM(int, cleanall),
	KPARAM(char *, ostr))
	KDECL(char *istr;)
	KDECL(int  cleanall;)
	KDECL(char *ostr;)
{
	char *string, *temp;

	if (istr == NULL || strlen(istr) == 0)
	{
	   if (ostr) ostr[0] = '\0';
	   return(NULL);
	}

	if (ostr == NULL)
	   string = kstrdup(istr);
	else if (istr != ostr)
	   string = kstrcpy(ostr, istr);
	else
	   string = ostr;

	if ((temp = _cleanup_string(string, cleanall)) == NULL)
	{
	   if (ostr != NULL) ostr[0] = '\0';
	      else kfree_and_NULL(string);

	   return(NULL);
	}
	else if (temp != string)
	{
	   kstrcpy(string, temp);
	}
	return(string);
}

/*-----------------------------------------------------------
|
|  Routine Name: khash - polynomial conversion
|       Purpose: polynomial conversion ignoring overflows
|		 [this seems to work remarkably well, in fact better
|		 then the ndbm hash function. Replace at your own risk]
|
|		 ! 	use:  65599	nice.
|		 ! 	      65587   even better. 
|
|         Input: data - the data in which to create the has for
|		 length - the length of the hash to be used.  If -1,
|			  then the length is computed using kstrlen().
|       Returns: returns the associated hash for the given data.
|    Written By: Ozan S. Yigit (oz@nexus.yorku.ca)
|          Date: 1991 
| Modifications: Integrated into Khoros 2.0 (MY) 4/93
------------------------------------------------------------*/
long khash( KPARAM(register char *, data),KPARAM(register int, length) )
   KDECL(register char *data;)
   KDECL(register int  length;)
{
   register unsigned long n = 0;

#define HASHC	n = *data++ + 65599 * n
   if (length <= -1)
      length = kstrlen(data);

   switch (length % 8)
   {
      case 7: HASHC;
      case 6: HASHC;
      case 5: HASHC;
      case 4: HASHC;
      case 3: HASHC;
      case 2: HASHC;
      case 1: HASHC;
   }

   length >>= 3;
   while (length-- > 0)
   {
      HASHC; HASHC; HASHC; HASHC;
      HASHC; HASHC; HASHC; HASHC;
   }
   return n;
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoken_find - static routine to find a token
|       Purpose: This routine checks to see if the string has been
|		 token'ized.
|         Input: string - the string to check if token'ized
|         	 hash   - if -1 then re-compute the hash from the string
|        Output: indx   - the hash index into the hash_table
|       Returns: the token or 0 if it doesn't exist
|    Written By: Mark Young
|          Date: Nov 16, 1993
------------------------------------------------------------*/
int ktoken_find(
	KPARAM(char *, string),
	KPARAM(int, hash),
	KPARAM(int *,indx))
   KDECL(char *string;)
   KDECL(int  hash;)
   KDECL(int  *indx;)
{
   kilist *list;

   if (!string)
      return 0;

   if (token_last != -1 && strcmp(string, token_table[token_last]) == 0)
      return token_last;

   if (!hash_table)
   {
      hash_table = (kilist **) kcalloc(TABLESIZE, sizeof(kilist *));
      token_table = (char **) kcalloc(TABLESIZE, sizeof(char *));
      token_tablesize = TABLESIZE;
   }

   if (hash == -1)
      hash = khash(string, -1);
   hash = kabs(hash) % TABLESIZE;
   if (indx)
      *indx = hash;

   for (list = hash_table[hash]; list != NULL; list = list->next)
   {
      if (strcmp(string, list->client_data) == 0)
      {
         token_last = (int) list->identifier;
         return token_last;
      }
   }
   return 0;
}

/*-----------------------------------------------------------
|
|  Routine Name: kstring_to_token - convert a string to a unique integer
|
|       Purpose: Uses a hashing function to convert a string to a unique
|		 integer
|         Input: string - input string
|       Returns: token on success, or 0 on error
|    Written By: Mark Young
|          Date: Jan 10, 1995
------------------------------------------------------------*/
int kstring_to_token(KPARAM(char *, string))
   KDECL(char *string;)
{
   int indx, token;

   if ((token = ktoken_find(string, -1, &indx)) != 0 || !string)
      return token;

   if ((token = token_next++) == token_tablesize)
   {
      token_tablesize += TABLESIZE;
      token_table = (char **) krealloc(token_table, token_tablesize *
				sizeof(char *));
   }
   token_last = token;
   token_table[token] = kstrdup(string);
   hash_table[indx] = kilist_add(hash_table[indx], token, token_table[token]);
   return token;
}


/*-----------------------------------------------------------
|
|  Routine Name: ktoken_to_string - return the string associated with
|				 the specified token
|       Purpose: Returns the string that is associated with the supplied
|		 token.  The idea is that given a unique token for each
|		 string an application uses ktoken_to_string(token) in order
|		 to get the unique string back.  The kstring_to_token() routine
|		 should be used to create the unique token for the specified
|		 string.
|	  Input: token - the token which we will be returning the string for
|       Returns: the string or NULL if the token does not exist
|    Written By: Mark Young
|          Date: Nov 16, 1992
------------------------------------------------------------*/

char *ktoken_to_string(KPARAM(int, token))
   KDECL(int token;)
{
   return (token < 0 || token >= token_next) ? NULL : token_table[token];
}

/*-----------------------------------------------------------
|
|  Routine Name: kstrstr - find a substring within a string
|       Purpose: This routine is a replacement for the system
|		 library routine strstr.  This routine scans the
|		 input string for the first occurance
|		 of a substring specified by sub_str.
|         Input: istr - the string to search
|		 sub_str   - the sub string to look for.
|       Returns: If the sub_str is found in istr, a pointer indicating the
|		 address of the first character of the substring within the
|		 istr string is returned.  If the substring is not
|		 a part of the input string, a NULL is returned.
|
|		 This routine also protects against NULL on both the input and
|		 substring variables.  If either is NULL, the value of NULL is
|		 returned. 
|    Written By: Hideo Noro & Mark Young & Tom Sauer
|          Date: Jul 03, 1992 14:05
| Modifications: Converted from vstrstr in Khoros 1.0 (SJ)
------------------------------------------------------------*/
char *kstrstr(KPARAM(char *, istr), KPARAM(char *, sub_str))
   KDECL(char *istr;)
   KDECL(char *sub_str;)
{
   char *string;
   int  len;

   if (istr == NULL || sub_str == NULL)
      return(NULL);

   len = kstrlen(sub_str);
   string = istr;
   while (*string != '\0')
   {
      if (strncmp(string, sub_str, (size_t) len) == 0)
         return string;
      string++;
   }
   return NULL;
}

/*-----------------------------------------------------------
|
|  Routine Name: kstring_replace - replace one substring with another.
|       Purpose: This function performs a global change of text on
|                the input string.  It returns a string where every
|                occurrence of the scan string is replaced with the
|                replacement string.  If ostr is sent in as NULL,
|                the result will be kmalloc'ed for you.  If ostr
|                is provided as non-null, the result will be stored in it.
|                Note that if the latter use is chosen, ostr must
|                be previously allocated with a sufficient amount of storage
|                space before this routine is called.
|
|                For example, if we call:
|                new_string = kstring_replace("Welcome to Khoros", "Khoros",
|                                             "the Twilight Zone", NULL);
|                the result will be:
|                new_string = "Welcome to the Twilight Zone".
|         Input: istr  - string to be changed
|                scan_str      - string to be replaced
|                replace_str   - string to replace the scan_str
|        Output: ostr - string that holds the converted string.
|                               If it's NULL, it kmallocs the space necessary.
|       Returns: ostr if it is not NULL, a pointer to the
|                resulting kmalloc'ed string if it is NULL.  NULL on
|                an error.
|    Written By: Mark Young
|          Date: Jul 03, 1992 14:04
|  Side Effects: Mallocs the space for the output if ostr is NULL
| Modifications: Converted from vreplace_string in Khoros 1.0 (SJ)
------------------------------------------------------------*/
char *kstring_replace(
	KPARAM(char *, istr),
	KPARAM(char *, scan_str),
	KPARAM(char *, replace_str),
	KPARAM(char *, ostr))
   KDECL(char *istr;)
   KDECL(char *scan_str;)
   KDECL(char *replace_str;)
   KDECL(char *ostr;)
{
    char    *string, *temp = NULL, *save = istr;
    int ilen, rlen, size = 0, length, numchar;
 
    if (istr == NULL)
       return(NULL);
 
    if (scan_str == NULL)
       return(NULL);
 
    ilen = kstrlen(scan_str);
    rlen = kstrlen(replace_str);
    length = kmax(rlen, LENGTH);
 
    if ((save == ostr && rlen > ilen) || (ostr==NULL))
    {
       size = length;
       if ((temp = (char *) kmalloc((unsigned) sizeof(char)*size)) == NULL)
           return(NULL);
       string = temp;
    }
    else
       string = ostr;
 
    while (*istr != '\0')
    {
       if (temp != NULL)
       {
          numchar = string - temp;
          if (numchar >= (size-rlen-1))
          {
             size += length;
             if ((temp = (char *) krealloc(temp, size)) == NULL)
                return(NULL);
             string = temp + numchar;
          }
       }
 
       if (strncmp(istr, scan_str, (size_t) ilen) == 0)
       {
          strncpy(string, replace_str, (size_t) rlen);
          string += rlen;
          istr += ilen;
       }
       else
          *string++ = *istr++;
    }
    *string = '\0';
 
    if (ostr == NULL)
    {
       ostr = kstrdup(temp);
       kfree_and_NULL(temp);
    }
    else if (save == ostr && rlen > ilen)
    {
       kstrcpy(ostr, temp);
       kfree_and_NULL(temp);
    }
    return(ostr);
}
