/*
 * Khoros: $Id: line.c,v 1.7 1996/08/10 03:13:48 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: line.c,v 1.7 1996/08/10 03:13:48 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  Static:
   >>>>  Public:
   >>>>             ReadLine()
   >>>>             ProcessLine()
   >>>>             WriteLine()
   >>>>             mycpp()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "kgenmake.h"
#include "kimake.h"

/* static int   token_last = -1;
static int   token_next = 1;
static int   token_tablesize = 0;
static char  **token_table = NULL;
static kilist **hash_table = NULL;*/
static kilist *symbol_list = NULL;

static int   saw_first_target = FALSE;

/*-----------------------------------------------------------
|
|  Routine Name: KReadLine - Read a line from the Makefile
|
|       Purpose: Similiar in function to a kfgets, but it deals with
|		 the fact that Makefiles can have line continuations via
|		 a '\' at the end of a line.
|
|         Input: infile - FILE stream to read from
|       Returns: a pointer to the read line, or NULL on end of file
|
|    Written By: Steven Jorgensen
|          Date: Jan 10, 1995
| Modifications:
------------------------------------------------------------*/
char *KReadLine(KPARAM(FILE *, infile))
	KDECL(FILE *infile;)
{
   int    num = 0, size = 0;
   char   *tmp, *line = NULL;

   static char *data = NULL;
   static int initialized = FALSE;
#if defined(FIXUP_CPP_WHITESPACE)
   static int InRule = FALSE;
   int        last;
#else
   static int InRule = TRUE;
#endif
 
   if (infile == NULL)
      return NULL;
 
   if (initialized == FALSE)
   {
      while (!feof(infile))
      {
	 if (num+LENGTH >= size)
	 {
	    size += LENGTH;
            data = krealloc(data, size);
	 }
         num += fread(&data[num], (size_t)1, (size_t)LENGTH, infile);
      }
      if (num == size)
	 data = krealloc(data, size+1);
      data[num] = '\0';
      kstring_replace(data, "@@", "\n", data);
      initialized = TRUE;
   }

   while (data != NULL)
   {
      if ((tmp = strchr(data, '\n')) != NULL)
      {
	 line = tmp;
	 while (line != data && isspace(*line))
	    line--;

	 if (*line == '\\')
	 {
	    *tmp = *line = ' ';
	    continue;
	 }
	 else if (line != tmp)
	    line[1] = '\0';

	 *tmp++ = '\0';
	 if (line == data)
	 {
#if defined(FIXUP_CPP_WHITESPACE)
	    InRule = FALSE;
#endif
	    data = tmp;
	    continue;
	 }
      }
      line = data, data = tmp;

#if defined(FIXUP_CPP_WHITESPACE)
      if (!InRule)
      {
	 InRule = (strchr(line, '=') == NULL && (tmp = strchr(line, ':')) != NULL);
         while (*line == ' ') line++;
      }
#endif
      if (*line == '#')
	 continue;
      else if (InRule && *line == ' ')
	 *line = '\t';

      return(line);
   }
   return(NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: make_ret_str - make a string with all the symbols in it
|
|       Purpose:
|         Input:
|        Output:
|       Returns:
|    Written By: Steven Jorgensen
|          Date: Mar 28, 1995
| Modifications:
|
------------------------------------------------------------*/
static char *make_ret_str(KPARAM(char *, str))
   KDECL(char *str;)
{
   char *tmp1 = NULL, *tmp2 = NULL;
   kilist *head;
   for (head = kilist_head(symbol_list); head != NULL; head=head->next)
   {
      if (head->client_data == NULL)
	 continue;
      tmp1 = kstring_3cat(tmp2, ktoken_to_string(head->identifier), " = ",
			     NULL);
      kfree_and_NULL(tmp2);
      tmp2 = kstring_3cat(tmp1, head->client_data, "\n", NULL);
      kfree_and_NULL(tmp1);
   }
   tmp1 = kstring_3cat(tmp2, str, NULL, NULL);
   kfree_and_NULL(tmp2);
   return(tmp1);
}

/*-----------------------------------------------------------
|
|  Routine Name: KProcessLine - Process a line from the Makefile
|
|       Purpose: This routine processes lines that set macro
|		 values.  Macro lines set with the = symbol
|		 remembers both the variable name and its value.
|		 += lines are removed for make programs that do
|		 not support this extention (i.e:
|
|		 VARIABLE = first_val
|		 (other lines)
|		 VARAIBLE += append_val1
|		 (other lines)
|		 VARAIBLE += append_val2
|
|		 becomes:
|		 
|		 VARIABLE = first_val
|		 (other lines)
|		 VARAIBLE = first_val append_val1
|		 (other lines)
|		 VARAIBLE = first_val append_val1 append_val2
|
|		 On systems that have a make which supports the +=
|		 feature, no processing is done on the line.
|
|         Input: str - char string to read from
|       Returns: a pointer to the new line, or NULL on error
|    Written By: Steven Jorgensen
|          Date: Jan 10, 1995
------------------------------------------------------------*/
char *KProcessLine(KPARAM(char *, str))
	KDECL(char *str;)
{
   char        assignment[LENGTH];
   char        varname[LENGTH];
   char        sepstr[LENGTH];
   char       *tmp, *tmp1, *temp;
   char       *valname;
   kilist     *sltmp = NULL;
   int         var_token = 0;
   int         mset, prepend;
   int         i;
   static int  re_compiled = FALSE;

   if (!re_compiled)
   {
      if ((tmp = kre_comp("^\\s*([\\w+]+)\\s*(((\\+>|\\+<):?)?=)(.*)$")) != NULL)
      {
	 fprintf(stderr, "kre_comp failed.  Error returned is\n\n'%s'\n\n",
		 tmp);
	 return NULL;
      }
      re_compiled = TRUE;
   }

   if (kre_exec(str) == FALSE)
   {
      if (saw_first_target == TRUE)
         return kstrdup(str);
      re_compiled = FALSE;
      if ((tmp = kre_comp("^[$()\\w_]+:.*$")) != NULL)
      {
	 fprintf(stderr, "kre_comp failed.  Error returned is\n\n'%s'\n\n",
		 tmp);
	 return NULL;
      }
      saw_first_target = kre_exec(str);
      if (saw_first_target == TRUE)
	 return make_ret_str(str);
      return kstrdup(str);
   }
   if ((temp = (char *) kmalloc(kstrlen(str))) == NULL)
   {
      fprintf(stderr, "Could not malloc space for macro value\n");
      return NULL;
   }
   assignment[0] = '\0';
   kre_subs("\\1", varname);
   kre_subs("\\2", assignment);
   kre_subs("\\5", temp);
   valname = kstring_cleanup(temp, TRUE, NULL);
   kfree_and_NULL(temp);
   mset = (strchr(assignment, '+') == NULL) ? FALSE : TRUE;
   prepend = (strchr(assignment, '<') == NULL) ? FALSE: TRUE;
   if (strchr(assignment, ':') == NULL)
      kstrcpy(sepstr, " \\\n \t\t");
   else
      kstrcpy(sepstr, ":");
   var_token = kstring_to_token(varname);
   sltmp = kilist_locate(symbol_list, var_token);
   i = kstrlen(valname);
   if (i != 0 && (!mset || sltmp == NULL))
   {
      if (sltmp != NULL)
         kfree_and_NULL(sltmp->client_data);
      symbol_list = kilist_delete(symbol_list, var_token);
      symbol_list = kilist_add(symbol_list, var_token, valname);
   }
   else if (i != 0)
   {
      tmp = kstrstr(sltmp->client_data, valname);
      if (tmp == NULL || !isspace(tmp[i]) || tmp[i] != sepstr[0])
      {
         tmp = sltmp->client_data;
         if (prepend)
            sltmp->client_data = kstring_3cat(valname, sepstr, tmp, NULL);
         else
	    sltmp->client_data = kstring_3cat(tmp, sepstr, valname, NULL);
         kfree_and_NULL(valname);
         kfree_and_NULL(tmp);
      }
   }
   if (saw_first_target == FALSE)
      return NULL;

   sltmp = kilist_locate(symbol_list, var_token);
   if (sltmp == NULL || sltmp->client_data == NULL)
      return NULL;
   tmp = kstring_3cat(varname, " = ", sltmp->client_data, NULL);
   tmp1 = kstring_cat(tmp, "\n", NULL);
   kfree_and_NULL(tmp);
   return tmp1;
}



/*-----------------------------------------------------------
|
|  Routine Name: mycpp - Remove the += Symbols from the Makefile
|
|       Purpose: This routine parses the Makefile looking for lines
|		 with += in them.  It replaces the += syntax with a
|		 correctly appended = version.
|
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Steven Jorgensen
|          Date: Mar 06, 1995
| Modifications:
|
------------------------------------------------------------*/
extern char *Makefile;

int kcpp(KPARAM(FILE *, infile))
   KDECL(FILE *infile;)
{

   FILE *outfile = NULL;
   char *line    = NULL;
   char *outline    = NULL;

   if ((outfile = fopen(Makefile,"w")) == NULL)
   {
      fprintf(stderr, "Cannot open '%s' for output\n", Makefile);
      return FALSE;
   }

   while((line = KReadLine(infile)) != NULL)
   {
      outline = KProcessLine(line);
      if (outline != NULL)
      {
         fprintf(outfile, "%s\n", outline);
         kfree_and_NULL(outline);
      }
   }
   fclose(outfile);
   return TRUE;
}
