/* $XConsortium: imake.c,v 1.87 94/04/17 20:10:28 rws Exp $ */

/*****************************************************************************
 *                                                                           *
 *                                Porting Note                               *
 *                                                                           *
 * Add the value of BOOTSTRAPCFLAGS to the cpp_argv table so that it will be *
 * passed to the template file.                                              *
 *                                                                           *
 *****************************************************************************/

/*
 * 
Copyright (c) 1985, 1986, 1987  X Consortium

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the X Consortium shall not be
used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from the X Consortium.
 * 
 * Original Author:
 *	Todd Brunhoff
 *	Tektronix, inc.
 *	While a guest engineer at Project Athena, MIT
 *
 * imake: the include-make program.
 *
 * Usage: imake [-Idir] [-Ddefine] [-T template] [-f imakefile ] [-C Imakefile.c ] [-s] [-e] [-v] [make flags]
 *
 * Imake takes a template file (Imake.tmpl) and a prototype (Imakefile)
 * and runs cpp on them producing a Makefile.  It then optionally runs make
 * on the Makefile.
 * Options:
 *		-D	define.  Same as cpp -D argument.
 *		-I	Include directory.  Same as cpp -I argument.
 *		-T	template.  Designate a template other
 *			than Imakefile
 *		-f	specify the Imakefile file
 *		-C	specify the name to use instead of Imakefile.c
 *		-s[F]	show.  Show the produced makefile on the standard
 *			output.  Make is not run in this case.  If a file
 *			argument is provided, the output is placed there.
 *		-v	verbose.  Show the make command line executed.
 *
 * Environment variables:
 *		
 *		IMAKEINCLUDE	Include directory to use in addition to "."
 *		IMAKECPP	Cpp to use instead of /lib/cpp
 *		IMAKEMAKE	make program to use other than what is
 *				found by searching the $PATH variable.
 * Other features:
 *	imake reads the entire cpp output into memory and then scans it
 *	for occurences of "@@".  If it encounters them, it replaces it with
 *	a newline.  It also trims any trailing white space on output lines
 *	(because make gets upset at them).  This helps when cpp expands
 *	multi-line macros but you want them to appear on multiple lines.
 *	It also changes occurences of "KCOM" to "#", to avoid problems
 *	with treating commands as invalid preprocessor commands.
 *
 *	The macros MAKEFILE and MAKE are provided as macros
 *	to make.  MAKEFILE is set to imake's makefile (not the constructed,
 *	preprocessed one) and MAKE is set to argv[0], i.e. the name of
 *	the imake program.
 *
 * Theory of operation:
 *   1. Determine the name of the imakefile from the command line (-f)
 *	or from the content of the current directory (Imakefile or imakefile).
 *	Call this <imakefile>.  This gets added to the arguments for
 *	make as MAKEFILE=<imakefile>.
 *   2. Determine the name of the template from the command line (-T)
 *	or the default, Imake.tmpl.  Call this <template>
 *   3. Determine the name of the imakeCfile from the command line (-C)
 *	or the default, Imakefile.c.  Call this <imakeCfile>
 *   3. Store three lines of input into <imakeCfile>:
 *		#define IMAKE_TEMPLATE		" <template> "
 *		#define INCLUDE_IMAKEFILE	< <imakefile> >
 *		#include IMAKE_TEMPLATE
 *	Start up cpp and provide it with this file.
 *	Note that the define for INCLUDE_IMAKEFILE is intended for
 *	use in the template file.  This implies that the imake is
 *	useless unless the template file contains at least the line
 *		#include INCLUDE_IMAKEFILE
 *   4. Gather the output from cpp, and clean it up, expanding @@ to
 *	newlines, stripping trailing white space, cpp control lines,
 *	and extra blank lines, and changing KCOM to #.  This cleaned
 *	output is placed in a new file, default "Makefile", but can
 *	be specified with -s or -e options.
 *   5. Optionally start up make on the resulting file.
 *
 * The design of the template makefile should therefore be:
 *	<set global macros like CFLAGS, etc.>
 *	<include machine dependent additions>
 *	#include INCLUDE_IMAKEFILE
 *	<add any global targets like 'clean' and long dependencies>
 */
#include "kgenmake.h"
#include "kimake.h"

/*
 *     The following table contains the flags that should be passed
 *     whenever a Makefile is being generated.
 */
char *cpp_argv[LENGTH] = {
	"cc",		/* replaced by the actual program to exec */
	"-I.",		/* add current directory to include path */
	NULL
};

typedef	unsigned char	boolean;

char	*Imakefile = NULL;
char	*Makefile = "Makefile";
char	*Template = "Imakefile"; /*"Imake.tmpl";*/
char	*tmpImakefile = "/tmp/IIf.XXXXXX";
char	*ImakefileC = "Imakefile.c";
boolean haveImakefileC = FALSE;
char	*program;
char    command[LENGTH*20];

boolean	verbose = FALSE;
boolean	show = TRUE;
boolean cmdln_template = 0;	 /* Added to see of -T was specified on */
				 /* command line.  SJ */

void wrapup PROTO((void));
void catch PROTO((int));
void init PROTO((void));
void AddCppArg PROTO((char *));
void SetOpts PROTO((int, char **));
char *FindImakefile PROTO((char *));
void LogFatal PROTO((char *, char *));
void cppit PROTO((char *, char *, char *, char *));
void sdirs PROTO((int));

void khoros_incl PROTO((void));
int  khoros_cpps PROTO((FILE *));
void khoros_tbinit PROTO((void));

/*--------------------------------------------------------------
 | Main routine to generate a new Makefile
 +--------------------------------------------------------------*/
void process_make(KPARAM(int, argc), KPARAM(char **, argv))
	KDECL(int	argc;)
	KDECL(char	**argv;)
{
	program = argv[0];
	init();
	SetOpts(argc, argv);

	Imakefile = FindImakefile(Imakefile);
	cppit(Imakefile, Template, ImakefileC, Makefile);
	wrapup();
}

/*--------------------------------------------------------------
 | Clean up the Imakefile.c used by cpp to generate the Makefile
 +--------------------------------------------------------------*/
void wrapup(KNONE)
{
	if (haveImakefileC)
	   unlink(ImakefileC);
}

/*--------------------------------------------------------------
 | Signal handling routine to clear up.
 +--------------------------------------------------------------*/
#ifdef SIGNALRETURNSINT
int
#else
void
#endif
catch(KPARAM(int, sig))
	KDECL(int	sig;)
{
	char sigstr[LENGTH];
	sprintf(sigstr, "%d", sig);
	errno = 0;
	LogFatal("Signal %s.", sigstr);
}

/*--------------------------------------------------------------
 | Initialize some variables.
 +--------------------------------------------------------------*/
void init(KNONE)
{
	int	 i;
	register char	*p;
        char str[LENGTH];

	command[0] = '\0';

        /*--------------------------------------------------------------
	 | See if the standard include directory is different than
	 | the default.  Or if cpp is not the default.  Or if the make
	 | found by the PATH variable is not the default.
         +--------------------------------------------------------------*/
	if ((p = getenv("IMAKEINCLUDE")) != NULL) {
		if (*p != '-' || *(p+1) != 'I')
			LogFatal("Environment var IMAKEINCLUDE %s\n",
				"must begin with -I");
		AddCppArg(p);
		for (; *p; p++)
			if (*p == ' ') {
				*p++ = '\0';
				AddCppArg(p);
			}
	}

	if ((p = getenv("IMAKECPP")) != NULL)
		strcpy(command, p);
	else
		strcpy(command, DEFAULT_CPP);

	i = 1;
	while (cpp_argv[i] != NULL)
                AddCppArg(cpp_argv[i++]);
	sprintf(str, "-D%s", KARCH_NAME);
        AddCppArg(str);
	sprintf(str, "-D%s", KOPSYS_NAME);
        AddCppArg(str);

	if (signal(SIGINT, SIG_IGN) != SIG_IGN)
		signal(SIGINT, catch);

	/*
	 * Added Khoros Initialize in order to initialize the khoros toolbox
	 * list.  This is used during the cppit() phase to define the paths
	 * to all the toolbox config files...
	 */
	(void) khoros_tbinit();
}

/*--------------------------------------------------------------
 | Add an argument to the cpp command string.
 +--------------------------------------------------------------*/
void AddCppArg(KPARAM(char *, arg))
	KDECL(char	*arg;)
{
	errno = 0;
	kstring_3cat(command, " ", arg, command);
}

/*--------------------------------------------------------------
 | Process the command line options
 +--------------------------------------------------------------*/
void SetOpts(KPARAM(int, argc), KPARAM(char **, argv))
	KDECL(int	argc;)
	KDECL(char	**argv;)
{
	errno = 0;

	/*
	 * Now gather the arguments for make
	 */
	for(argc--, argv++; argc; argc--, argv++) {
	/*
	     * We intercept these flags.
	     */
	    if (argv[0][0] == '-') {
		if (argv[0][1] == 'D') {
		    AddCppArg(argv[0]);
		} else if (argv[0][1] == 'I') {
		    AddCppArg(argv[0]);
		} else if (argv[0][1] == 'f') {
		    if (argv[0][2])
			Imakefile = argv[0]+2;
		    else {
			argc--, argv++;
			if (! argc)
			    LogFatal("No description arg after -f flag\n", "");
			Imakefile = argv[0];
		    }
		} else if (argv[0][1] == 's') {
		    if (argv[0][2])
			Makefile = ((argv[0][2] == '-') && !argv[0][3]) ?
			    NULL : argv[0]+2;
		    else {
			argc--, argv++;
			if (!argc)
			    LogFatal("No description arg after -s flag\n", "");
			Makefile = ((argv[0][0] == '-') && !argv[0][1]) ?
			    NULL : argv[0];
		    }
		    show = TRUE;
		} else if (argv[0][1] == 'T') {
		    if (argv[0][2])
			Template = argv[0]+2;
		    else {
			argc--, argv++;
			if (! argc)
			    LogFatal("No description arg after -T flag\n", "");
			Template = argv[0];
		    cmdln_template = 1;
		    }
		} else if (argv[0][1] == 'C') {
		    if (argv[0][2])
			ImakefileC = argv[0]+2;
		    else {
			argc--, argv++;
			if (! argc)
			    LogFatal("No imakeCfile arg after -C flag\n", "");
			ImakefileC = argv[0];
		    }
		} else if (argv[0][1] == 'v') {
		    verbose = TRUE;
	          }
	    }
	}
}

/*--------------------------------------------------------------
 | Search for an Imakefile
 +--------------------------------------------------------------*/
char *FindImakefile(KPARAM(char *, Ifile))
	KDECL(char	*Ifile;)
{
	if (Ifile) {
		if (access(Ifile, R_OK) < 0)
			LogFatal("Cannot find %s.", Ifile);
	} else {
		if (access("Imakefile", R_OK) < 0)
			if (access("imakefile", R_OK) < 0)
				LogFatal("No description file.", "");
			else
				Ifile = "imakefile";
		else
			Ifile = "Imakefile";
	}
	return(Ifile);
}

/*--------------------------------------------------------------
 | Error notification handling routine
 +--------------------------------------------------------------*/
extern char     *sys_errlist[];
void
LogFatal(KPARAM(char *, x0), KPARAM(char *, x1))
	KDECL(char *x0;)
	KDECL(char *x1;)
{
	static boolean	entered = FALSE;

	if (entered)
		return;
	entered = TRUE;

	fprintf(stderr, "%s: ", program);
	if (errno)
		fprintf(stderr, "%s: ", sys_errlist[ errno ]);
	fprintf(stderr, x0,x1);
	fprintf(stderr, "  Stop.\n");
	wrapup();
	exit(1);
}

#define TmplDef "#define IMAKE_TEMPLATE"
#define ImakeDef "#define INCLUDE_IMAKEFILE"
#define TmplInc "#include IMAKE_TEMPLATE"

/*--------------------------------------------------------------
 | Do that cpp thang
 +--------------------------------------------------------------*/
void cppit(
	KPARAM(char *, imakefile),
	KPARAM(char *, template),
	KPARAM(char *, masterc),
	KPARAM(char *, outfname))
	KDECL(char	*imakefile;)
	KDECL(char	*template;)
	KDECL(char	*masterc;)
	KDECL(char	*outfname;)
{
	FILE	*inFile;

	haveImakefileC = TRUE;

	if ((inFile = fopen(masterc, "w"))==NULL)  /*-- Create Imakefile.c --*/
		LogFatal("Cannot open %s for output.", masterc);

	khoros_incl();	/* Khoros stuff that we need */

	/*-- Insert important things to include and define --*/
	if (fprintf(inFile, "%s \"%s\"\n", TmplDef, template) < 0 ||
	    fprintf(inFile, "%s <%s>\n", ImakeDef, imakefile) < 0 ||
	    /*-- ...Add Khoros defines into the imake stream...  MY --*/
	    khoros_cpps(inFile) < 0 ||
	    fprintf(inFile, "%s\n", TmplInc) < 0 ||
	    fclose(inFile))
	{
	    LogFatal("Cannot write to %s.", masterc);
	}

	/*-- Show the cpp command in verbose mode --*/	
	if (verbose == TRUE)
	   printf("command: '%s'\n", command);

	/*---------------------------------------------------------------
	 |  take Imakefile.c and run it through cpp.  That output
         |  is piped to our routines to post process the Makefile
	 |  before the final version is written out
         +--------------------------------------------------------------*/
	inFile = (FILE *) popen(command, "r");
	kcpp(inFile);
	pclose(inFile);
}


/*--------------------------------------------------------------
 | Variables needed to store information taken out of the
 | Khoros toolbox system.
 +--------------------------------------------------------------*/
int     num_toolboxes = 0,		/*-- Number of toolboxes --*/
	bootstrap = -1,			/*-- index of bootstrap toolbox --*/
	current = -1,			/*-- index of current toolbox --*/
	manual = 0;			/*-- Imakefile of a manual type?  --*/
int	*toolbox_tkn = NULL;		/*-- array of toolbox tokens --*/
int	*toolbox_mach = NULL;		/*-- array of toolbox mach values --*/
int	*toolbox_gdir = NULL;		/*-- array of toolbox gdir values --*/
int     *toolbox_order = NULL;		/*-- array of toolbox indices --*/
int      toolbox_cnt = 0;		/*-- count for above array --*/
char	*global_bindir = NULL;		/*-- Global bin set in bootstrap --*/
char	*global_libdir = NULL;		/*-- Global lib set in bootstrap --*/
char	**toolbox_bindir = NULL;	/*-- array of toolbox bindirs --*/
char	**toolbox_libdir = NULL;	/*-- array of toolbox libdirs --*/
char	**toolboxes = NULL;		/*-- array of toolbox names --*/
char	**toolbox_paths = NULL;		/*-- array of toolbox paths --*/
char	**toolbox_incl = NULL;		/*-- array of toolbox *.def incls --*/
char    current_tb[1024];		/*-- all caps string of current tb --*/
char	dirfile[1024];			/*-- path to dirfile --*/
char 	sitefile[1024];			/*-- path to site include file --*/
char 	macrofile[1024];		/*-- path to macro include file --*/

/*--------------------------------------------------------------
 | Convert a string to its uppercase version
 | Khoros toolbox system.
 +--------------------------------------------------------------*/
void kstrup(KPARAM(char *, str))
	KDECL(char *str;)
{
   while (*str)
   {
      if (islower(*str))
         *str = toupper(*str);
	 str++;
   }
}

/*--------------------------------------------------------------
 | Do a strncmp skipping whitespace first.
 +--------------------------------------------------------------*/
int kspacencmp(KPARAM(char *, str1), KPARAM(char *, str2), KPARAM(int, n))
	KDECL(char *str1;)
	KDECL(char *str2;)
	KDECL(int n;)
{
   while (*str1 && isspace(*str1))
      str1++;
   while (*str2 && isspace(*str2))
      str2++;
   while (--n && *str1 && *str2 && *str1 == *str2)
   {
      str1++;
      str2++;
   }
   return (*str1 - *str2);
   
}

/*--------------------------------------------------------------
 | Get the value in a VAR=value line.
 +--------------------------------------------------------------*/
char *kgetval(KPARAM(char *, str))
	KDECL(char *str;)
{
	 char *tmp;
	 str = strchr(str, '=');
         if (str == NULL)
            return NULL;
	 else
	    str++;
         while(*str && isspace(*str))
            str++;
         tmp = str;
         while(*tmp && (*tmp == '_' || isalnum(*tmp)))
            tmp++;
         *tmp = '\0';
	 return str;
}

/*--------------------------------------------------------------
 | Initialize the khoros variables and options
 +--------------------------------------------------------------*/
void khoros_tbinit(KNONE)
{
   char path[1024], temp[1024], temp2[1024], *ftmp, *ftmp1, *tmp, *tmp1;
   char name[1024];
   FILE *file;

   
   /*--Initialize the toolbox array to hold all tb names and their paths --*/
   if ((ftmp = getenv("KHOROS_TOOLBOX")) == NULL)
      LogFatal("The environment variable KHOROS_TOOLBOX is not set\n Please use setenv to set the environment variable\nKHOROS_TOOLBOX to a valid  toolbox file", NULL);
   strcpy(path, ftmp);
   ftmp = path;

   /*-- This loop works on each tb file defined in KHOROS_TOOLBOX --*/
   do
   {
      ftmp1 = strchr(ftmp, ':');
      if (ftmp1 != NULL)
         *ftmp1 = '\0';
      if (access(ftmp, F_OK) == -1)
      {
         fprintf(stderr, "kgenmake: The environment variable KHOROS_TOOLBOX\n");	 fprintf(stderr, "is pointing to a file,\n  '%s',\n", ftmp);
         fprintf(stderr, "that does not exist.  Ignoring this entry\n");
         if (ftmp1 == NULL)
	    ftmp = NULL;
         else
            ftmp = ftmp1+1;
         continue;
      }

      if ((file = fopen(ftmp, "r")) == NULL)
         LogFatal("The environment variable KHOROS_TOOLBOX\npoints to a file\n  '%s'\n that cannot be opened.  Please set the environment variable\nKHOROS_TOOLBOX to a valid set toolbox files", ftmp);

      /*-- This loop works on each line in a toolboxes file. --*/
      while (fgets(temp, 1024, file) != NULL)
      {
         if (temp[0] == '#')
            continue;
         tmp = strchr(temp, ':');
         if (tmp == NULL)
            continue;

         *tmp++ = '\0';
         tmp1 = strrchr(tmp, ';');
         if (tmp1 != NULL)
            *tmp1++ = '\0';
         if ((num_toolboxes % 256) == 0)
	 {
            if (num_toolboxes == 0)
            {
               toolboxes = (char **) malloc(256 * sizeof (char *));
               toolbox_paths = (char **) malloc(256 * sizeof (char *));
               toolbox_bindir = (char **) malloc(256 * sizeof (char *));
               toolbox_libdir = (char **) malloc(256 * sizeof (char *));
               toolbox_tkn = (int *) malloc(256 * sizeof (int));
               toolbox_mach = (int *) malloc(256 * sizeof (int));
               toolbox_gdir = (int *) malloc(256 * sizeof (int));
               toolbox_incl = (char **) malloc(256 * sizeof (char *));
	       if (toolboxes == NULL || toolbox_paths == NULL ||
		   toolbox_tkn == NULL || toolbox_incl == NULL)
                  LogFatal("Ran out of memory while initializing toolboxes.  Bye.", NULL);
	       
            }
	    else
	    {
		toolboxes = (char **) realloc(toolboxes,
				   256 * (num_toolboxes/256+1)*sizeof(char *));
		toolbox_paths = (char **) realloc(toolbox_paths,
				   256 * (num_toolboxes/256+1)*sizeof(char *));
		toolbox_bindir = (char **) realloc(toolbox_bindir,
				   256 * (num_toolboxes/256+1)*sizeof(char *));
		toolbox_libdir = (char **) realloc(toolbox_bindir,
				   256 * (num_toolboxes/256+1)*sizeof(char *));
		toolbox_tkn = (int *) realloc(toolbox_tkn,
				   256 * (num_toolboxes/256+1)*sizeof(int));
		toolbox_mach = (int *) realloc(toolbox_mach,
				   256 * (num_toolboxes/256+1)*sizeof(int));
		toolbox_gdir = (int *) realloc(toolbox_gdir,
				   256 * (num_toolboxes/256+1)*sizeof(int));
		toolbox_incl = (char **) realloc(toolbox_incl,
				   256 * (num_toolboxes/256+1)*sizeof(char *));
	       if (toolboxes == NULL || toolbox_paths == NULL)
                  LogFatal("Ran out of memory while initializing toolboxes.  Bye.", NULL);
	    }
	  }
	  toolboxes[num_toolboxes] = malloc(sizeof(char) * strlen(temp)+1);
	  toolbox_paths[num_toolboxes] = malloc(sizeof(char) * strlen(tmp)+1);
	  if (toolboxes[num_toolboxes]     == NULL ||
              toolbox_paths[num_toolboxes] == NULL)
		LogFatal("Ran out of memory while initializing toolboxes.  Bye.", NULL);
	  strcpy(toolboxes[num_toolboxes], temp);
	  kstrup(toolboxes[num_toolboxes]);
	  strcpy(toolbox_paths[num_toolboxes], tmp);
	  toolbox_mach[num_toolboxes] = FALSE;
	  toolbox_gdir[num_toolboxes] = FALSE;
	  toolbox_bindir[num_toolboxes] = NULL;
	  toolbox_libdir[num_toolboxes] = NULL;

	  /*-- This loop checks for machdefs and globaldir set to true --*/
	  while (tmp1 != NULL)
          {
	    if (tmp1 != NULL && sscanf(tmp1, "%[^=]=%[^,\n]", name, temp) == 2)
            {
               kstring_cleanup(temp, TRUE, temp);
               kstrup(temp);
               kstring_cleanup(name, TRUE, name);
               kstrup(name);
               if (strcmp(temp, "YES") == 0 || strcmp(temp, "TRUE") == 0 || 
                   strcmp(temp, "ON") == 0  || strcmp(temp, "1") == 0)
	       {
		  if (strcmp(name, "MACHDEFS") == 0)
                     toolbox_mach[num_toolboxes] = TRUE;
		  else
		     toolbox_gdir[num_toolboxes] = TRUE;
	       }
            }
	    tmp = tmp1;
	    tmp1 = strchr(tmp, ',');
	    if (tmp1 != NULL) tmp1++;
	  } /*-- end machdefs and globaldir loop --*/
	  tmp = toolbox_paths[num_toolboxes];
	  tmp1 = tmp+strlen(tmp)-1;
	  if (*tmp1 == '\n')
	     *tmp1 = '\0';
	  toolbox_tkn[num_toolboxes] = kstring_to_token(toolboxes[num_toolboxes]);
	  sprintf(temp2, "-I%s/config", toolbox_paths[num_toolboxes]);
	  toolbox_incl[num_toolboxes] = kstrdup(temp2);
	  num_toolboxes++;
      } /*-- end toolboxes file loop --*/

      if (ftmp1 == NULL)
	 ftmp = NULL;
      else
         ftmp = ftmp1+1;
      fclose(file);
   }
   while (ftmp != NULL); /*-- end KHOROS_TOOLBOX file loop --*/
}

/*--------------------------------------------------------------
 | Defines that we need to make Khoros work.
 +--------------------------------------------------------------*/
#define KhorosDef1 "#define KCOM #"
#define KhorosDef2 "#define YES 1"
#define KhorosDef3 "#define NO  0"
#define KhorosDef4 "#define MacroIncludeFile"
#define KhorosDef5 "#define SiteIncludeFile"

#include <sys/utsname.h>

/*--------------------------------------------------------------
 | Routine to define dirfile, sitefile, macrofile
 +--------------------------------------------------------------*/
void vardefs(KPARAM(int, cur))
	KDECL(int cur;)
{
	static char *karch = NULL, *kopsys = NULL, *kwidset = NULL;
	static int initialized = FALSE;

	if (initialized == FALSE) /*-- Only want to do these getenv's once --*/
	{
	   karch     = getenv("KARCH");
	   kopsys    = getenv("KOPSYS");
	   kwidset   = getenv("KWIDSET");
 	   initialized = TRUE;
	}

	if (toolbox_mach[cur] == TRUE && karch != NULL)
	{
	   /*-- Machdefs is true for this toolbox, use the environ vars --*/
#ifdef KDEBUG
	   printf("karch = '%s'\n", (karch ? karch : "NULL"));
	   printf("kopsys = '%s'\n", (kopsys ? kopsys : "NULL"));
	   printf("kwidset = '%s'\n", (kwidset ? kwidset : "NULL"));
#endif
	   if ((kwidset == NULL && kopsys == NULL) ||
	       (kwidset != NULL && kopsys == NULL))
	   {
	      sprintf(sitefile, "Site.%s", karch);
	      sprintf(macrofile, "%s.cf", karch);
	      sprintf(dirfile, "Dir.%s", karch);
	   }
	   else if (kopsys != NULL && kwidset == NULL)
	   {
	      sprintf(sitefile, "Site.%s-%s", karch, kopsys);
	      sprintf(macrofile, "%s-%s.cf", karch, kopsys);
	      sprintf(dirfile, "Dir.%s-%s", karch, kopsys);
	   }
	   else if (kopsys != NULL && kwidset != NULL)
	   {
	      sprintf(sitefile, "Site.%s-%s-%s", karch, kopsys, kwidset);
	      sprintf(macrofile, "%s-%s-%s.cf", karch, kopsys, kwidset);
	      sprintf(dirfile, "Dir.%s-%s-%s", karch, kopsys, kwidset);
	   }
	   else
	   {
	      strcpy(sitefile, SiteIncludeFile);
	      strcpy(macrofile, MacroIncludeFile);
	      strcpy(dirfile, "Dir.def");
	   }
	}
	else
	{
#ifdef KDEBUG
	   fprintf(stderr, "\n");
	   fprintf(stderr, "The machdefs flag is set to true in the Toolboxes file, but KARCH\n");
	   fprintf(stderr, "not set in environment.  This could cause kgenmake to work incorrectly.\n");
	   fprintf(stderr, "If you have problems contact the person that installed Khoros at your\n");
	   fprintf(stderr, "site to determine how to set up your environment to run Khoros.\n");
	   fprintf(stderr, "\n");
#endif
	   /*-- Use the system defaults --*/
	   strcpy(sitefile, SiteIncludeFile);
	   strcpy(macrofile, MacroIncludeFile);
	   strcpy(dirfile, "Dir.def");
	}
}

/*--------------------------------------------------------------
 | Routine to insert cpp directives into the stream being processed
 | by cpp.
 +--------------------------------------------------------------*/
int khoros_cpps(KPARAM(FILE *, inFile))
	KDECL(FILE *inFile;)
{
	int i;
	struct utsname uts;
	char   *khostname, *temp;
	char   buffer[1024];


	for (i = 0 ; i < toolbox_cnt; i++)
	{
	   vardefs(toolbox_order[i]);
	   sprintf(buffer, "%s/config/%s", toolbox_paths[toolbox_order[i]],
		   sitefile);
	   if (access(buffer, R_OK) == 0)
	      break;
	}
	if (fprintf(inFile, "#undef bsdi\n") < 0 ||
	    fprintf(inFile, "#undef vax\n") < 0 ||
	    fprintf(inFile, "#undef mips\n") < 0 ||
	    fprintf(inFile, "#undef alpha\n") < 0 ||
	    fprintf(inFile, "#undef bsd\n") < 0 ||
	    fprintf(inFile, "#undef solaris\n") < 0 ||
	    fprintf(inFile, "#undef sparc\n") < 0 ||
	    fprintf(inFile, "#undef sun\n") < 0 ||
	    fprintf(inFile, "#undef dg\n") < 0 ||
	    fprintf(inFile, "#undef convex\n") < 0 ||
	    fprintf(inFile, "#undef exemplar\n") < 0 ||
	    fprintf(inFile, "#undef next\n") < 0 ||
	    fprintf(inFile, "#undef hpux\n") < 0 ||
	    fprintf(inFile, "#undef apollo\n") < 0 ||
	    fprintf(inFile, "#undef sony\n") < 0 ||
	    fprintf(inFile, "#undef macII\n") < 0 ||
	    fprintf(inFile, "#undef cray\n") < 0 ||
	    fprintf(inFile, "#undef sgi\n") < 0 ||
	    fprintf(inFile, "#undef luna\n") < 0 ||
	    fprintf(inFile, "#undef linux\n") < 0 ||
	    fprintf(inFile, "#undef i386\n") < 0 ||
	    fprintf(inFile, "#undef ibm\n") < 0 ||
	    fprintf(inFile, "#undef aix\n") < 0 ||
	    fprintf(inFile, "%s\n", KhorosDef1) < 0 ||
	    fprintf(inFile, "%s\n", KhorosDef2) < 0 ||
	    fprintf(inFile, "%s\n", KhorosDef3) < 0 ||
	    fprintf(inFile, "KARCH_DEFINE = -D%s\n", KARCH_NAME) < 0 ||
	    fprintf(inFile, "KOPSYS_DEFINE = -D%s\n", KOPSYS_NAME) < 0 ||
	    fprintf(inFile, "%s \"%s\"\n", KhorosDef4, macrofile) < 0 ||
	    fprintf(inFile, "%s \"%s\"\n", KhorosDef5, sitefile) < 0)
	{
	   return(-1);
	}

	uname(&uts); khostname = uts.nodename;
	if ((temp = strchr(khostname, '.')) != NULL)
	   *temp = '\0';

	if (fprintf(inFile, "#define kmach_%s\n", khostname) < 0)
	   return(-1);

#ifdef KDEBUG
	fprintf(stderr, "#define IsManualType %s\n",
		(manual ? "TRUE" : "FALSE"));
#endif
	if (fprintf(inFile, "#define IsManualType %d\n", manual) < 0)
	   return(-1);

        for (i=0; i < num_toolboxes; i++)
	{
	   /*--------------------------------------------------------------
	    | These variables are defined if we depend on that toolbox.
	    | So, just skip all toolboxes we don't depend on to make
	    | the generated Makefile smaller.
            +--------------------------------------------------------------*/
	   if (toolbox_libdir[i] == NULL || toolbox_bindir[i] == NULL)
	      continue;
#ifdef KDEBUG
	   fprintf(stderr, "Adding vars for tb '%s'\n", toolboxes[i]);
	   fprintf(stderr, "%s_PATH = %s\n", toolboxes[i], toolbox_paths[i]);
	   fprintf(stderr, "%s_LIBDIR = %s\n", toolboxes[i],
		   toolbox_libdir[i]);
	   fprintf(stderr, "%s_BINDIR = %s\n", toolboxes[i],
		   toolbox_bindir[i]);
	   fprintf(stderr,
	     "#define %s_INCLUDE <%s/repos/config/imake_config/toolbox.def>\n",
	     toolboxes[i], toolbox_paths[i]);
#endif
	   fprintf(inFile, "%s_PATH = %s\n", toolboxes[i], toolbox_paths[i]);
	   fprintf(inFile, "%s_LIBDIR = %s\n", toolboxes[i],
		   toolbox_libdir[i]);
	   fprintf(inFile, "%s_BINDIR = %s\n", toolboxes[i],
		   toolbox_bindir[i]);
	   fprintf(inFile,
	     "#define %s_INCLUDE <%s/repos/config/imake_config/toolbox.def>\n",
	     toolboxes[i], toolbox_paths[i]);
	}
	fprintf(inFile, "TB_NAME = $(%s_PATH)\n\n", toolboxes[current]);
	fprintf(inFile, "TOOLBOX_PATH = %s\n", toolbox_paths[current]);
	if (toolbox_bindir[current] == NULL && toolbox_libdir[current] == NULL)
	    LogFatal("The binary and library directory are NULL for the current\ntoolbox '%s'.  This can only occur if the Imakefile for this\ndirectory has a corrupted or incorrect value in the TOOLBOX_NAME macro of the Imakefile.\nPlease fix this problem and rerun kgenmake", (toolboxes[current] == NULL) ? "Unknown Toolbox" : toolboxes[current]);
	fprintf(inFile, "BINDIR = %s\n", toolbox_bindir[current]);
	fprintf(inFile, "LIBDIR = %s\n", toolbox_libdir[current]);

	return(0);
}

/*-------------------------------------------------------------------------
| Search for bin and lib path information..
-------------------------------------------------------------------------*/
void khoros_incl(KNONE)
{
   char line[1024], *tmp;
   FILE *file;
   int i, found_tb = 0, found_type = 0;

   if ((file = fopen(Imakefile, "r")) == NULL)
      LogFatal("Cannot open the Imakefile.", NULL);
   current_tb[0] = '\0';
   while ( (!found_tb || !found_type) && fgets(line, 1024, file) != NULL )
   {
      tmp = NULL;
      if (!found_tb && kspacencmp("TOOLBOX_NAME", line, 12) == 0)
      {
	 tmp = kgetval(line);
         if (tmp != NULL && ((int) strlen(tmp) > 0))
         {
            strcpy(current_tb, tmp);
            kstrup(current_tb);
	    found_tb = 1;
         }
      }
      if (!found_type && kspacencmp("OBJECT_TYPE", line, 11) == 0)
      {
	 tmp = kgetval(line);
         if (tmp != NULL && ((int) strlen(tmp) > 0))
         {
	    if (strcmp(tmp,"manual") == 0    || strcmp(tmp,"mindex") == 0 ||
		strcmp(tmp,"mglossary") == 0 || strcmp(tmp,"mdir") == 0)
	       manual = 1;
	    found_type = 1;
	 }
      }
   }
   fclose(file);
   if (strlen(current_tb) == 0)
      LogFatal("No Toolbox variable set in Imakefile", NULL);
   
   for (i=0; i < num_toolboxes; i++)
   {
      if (strcmp(toolboxes[i], current_tb) == 0)
	 current = i;
      if (strcmp(toolboxes[i], "BOOTSTRAP") == 0)
         bootstrap = i;
   }
	 
   if ( current == -1)
      LogFatal("Cannot determine the path to the %s toolbox", current_tb);
   if ( bootstrap == -1)
      LogFatal("Cannot determine the path to the BOOTSTRAP toolbox", NULL);

   AddCppArg("-DKHOROS_TEMPLATE=\"Imakefile\"");

   sdirs(-1);

   sprintf(line,"-I%s/config", toolbox_paths[bootstrap]);
   AddCppArg(kstrdup(line));
   if (manual)
   {
      sprintf(line,"-I%s/repos/config/doc_config", toolbox_paths[bootstrap]);
      AddCppArg(kstrdup(line));
   }
   sprintf(line,"-I%s/repos/config/imake_config", toolbox_paths[bootstrap]);
   AddCppArg(kstrdup(line));

   AddCppArg(ImakefileC);
}

/*-------------------------------------------------------------------------
| Search for bin and lib path information..
-------------------------------------------------------------------------*/
static void search_the_site_include_file(KPARAM(int, cur))
	KDECL(int cur;)
{
   char temp[1024], temp2[1024];
   char *ptr;
   FILE *tbfile;
   int bin = FALSE, lib = FALSE;
   /*
    * Base case is the Imakefile.. This isn't a toolbox, so don't search
    * for a Dir.* file, and don't set bindir and libdir.
    */
   if (cur == -1) return;
   /*
    * vardefs sets dirfile correctly for current toolbox.
    */
   vardefs(cur);
   /*
    * set our work variable to the Dir.* file.
    */
   sprintf(temp, "%s/config/%s", toolbox_paths[cur], dirfile);
   /*
    * If machdefs is true, but no def.arch file exists, fall back
    * on to Dir.def.
    */
   if (toolbox_mach[cur] == TRUE && access(temp, R_OK) != 0)
      sprintf(temp, "%s/config/Dir.def", toolbox_paths[cur]);
#ifdef KDEBUG
   printf("Dirfile = '%s'\n", temp);
#endif
   /*
    * If no Dir.* file, and we are NOT using global dirs, set
    * bin and lib dirs to toolbox defaults
    */
   if (toolbox_gdir[cur] == FALSE && (tbfile = fopen(temp, "r")) == NULL)
   {
      sprintf(temp, "%s/bin\n", toolbox_paths[cur]);
      toolbox_bindir[cur] = kstrdup(temp);
      sprintf(temp, "%s/lib\n", toolbox_paths[cur]);
      toolbox_libdir[cur] = kstrdup(temp);
      if (cur == bootstrap)
      {
         global_bindir = toolbox_bindir[cur];
         global_libdir = toolbox_libdir[cur];
      }
      return;
   }
   /*
    * Global dir is true, and no Dir.* exists, so drop back to global
    * dir defined as we bounce back up the recursion tree.
    */
   else if ((tbfile = fopen(temp, "r")) == NULL)
      return;
   /*
    * Ok, we got a Dir.* files, parse it for the information
    */
   while (fgets(temp, 1024, tbfile) != NULL && (!bin || !lib))
   {
      if ((ptr = strstr(temp, "BinDir")) != NULL)
      {
         if (sscanf(ptr, "BinDir %s", temp2) == 1)
	    toolbox_bindir[cur] = kstrdup(temp2), bin = TRUE;
	 continue;
      }
      if ((ptr = strstr(temp, "LibDir")) != NULL)
      {
         if (sscanf(ptr, "LibDir %s", temp2) == 1)
	    toolbox_libdir[cur] = kstrdup(temp2), lib = TRUE;
	 continue;
      }
   }
   fclose(tbfile);
   /*
    * if the current toolbox is our bootstrap one, set global bin and
    * lib to end part of recursive routine works correct
    */
   if (cur == bootstrap)
   {
      global_bindir = toolbox_bindir[cur];
      global_libdir = toolbox_libdir[cur];
   }
}

static int *get_tb_indx(KPARAM(int, cur))
	KDECL(int cur;)
{
   char temp[1024], *ptr, *tmp;
   int *ret, tkn, i, size = 0;
   FILE *tbfile;

   search_the_site_include_file(cur);

   if (cur == -1)
      sprintf(temp, "Imakefile");
   else
      sprintf(temp, "%s/repos/config/imake_config/toolbox.def",
	      toolbox_paths[cur]);
   if ((tbfile = fopen(temp, "r")) == NULL)
      LogFatal("Cannot open '%s'\n", temp);
   ret = (int *) kcalloc(256, sizeof(int));
   ret[0] = -1;
   while (fgets(temp, 1024, tbfile) != NULL)
   {
      ptr = temp;
      while (isspace(*ptr))
	 ptr++;
      if (*ptr != '#')  /* not a line starting with # */
	 continue;
      ptr++;
      while (isspace(*ptr))
	 ptr++;
      if (strncmp(ptr, "include", (size_t) 7) != 0)
         continue;
      ptr += 7;
      while (isspace(*ptr))
	 ptr++;
      if ((tmp = strrchr(ptr, '_')) == NULL)
	 continue;
      if (strncmp(tmp, "_INCLUDE", (size_t) 8) != 0)
	 continue;
      *tmp = '\0';
      tkn = kstring_to_token(ptr);
      for (i = 0; i < num_toolboxes; i++)
	 if (tkn == toolbox_tkn[i])
	    break;
      if (i >= num_toolboxes)
	 LogFatal("Unreferenced toolbox '%s'\n", temp);	
      ret[size] = i;
      size++;
      ret[size] = -1;
   }
   fclose(tbfile);
   return(ret);
}

void sdirs(KPARAM(int, cur))
	KDECL(int cur;)
{
    int i, j = 0, *tb_indxs;

    tb_indxs = get_tb_indx(cur);

    for (i = 0; tb_indxs[i] != -1; i++)
    {
      j = tb_indxs[i];
      if (toolbox_incl[j] != NULL)
      {
	 if ((toolbox_cnt % 256) == 0)
	 {
	    if (toolbox_order == NULL)
               toolbox_order = (int *) malloc(256 * sizeof (int));
	    else
               toolbox_order = (int *) realloc(toolbox_order,
		                 256 * (toolbox_cnt/ 256 + 1 ) * sizeof (int));
	 }
	 toolbox_order[toolbox_cnt++] = j;
         AddCppArg(toolbox_incl[j]);
	 free(toolbox_incl[j]);
	 toolbox_incl[j] = NULL;
      }
    }
    for (i = 0; tb_indxs[i] != -1; i++)
    {
      j = tb_indxs[i];
      sdirs(j);
      if (toolbox_bindir[j] == NULL && toolbox_gdir[j] == TRUE)
        toolbox_bindir[j] = global_bindir;
      if (toolbox_libdir[j] == NULL && toolbox_gdir[j] == TRUE)
        toolbox_libdir[j] = global_libdir;
    }
    free(tb_indxs);
}
