#!/bin/sh

sed=/usr/bin/sed
if [ ! -x $sed ]; then
	sed=/bin/sed
	if [ ! -x $sed ]; then
		sed=sed
	fi
fi

FindEchoN()
{
        # build a list of potential echos, starting with sh built-in
        echoEnd=""
        elist="echo"
        for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
                [ -x "$dir/echo" ] && elist="$elist $dir/echo"
        done
 
        # work through list, see if one of them groks -n
        for echo in $elist; do
                result=`$echo -n hello`
                if [ "$result" != "-n hello" ]; then
                        echoN="$echo -n"
                        break
                fi
        done
        if [ "$echoN" = "" ]; then
                echoN=echo
                echoEnd="\c"
                result=`$echoN "testing"$echoEnd`
                if [ "$result" != "testing" ]; then
                        unset echoEnd
                fi
        fi
}

# split config.guess machine-vendor-OS
SplitGuess()
{
        if [ "$#" -eq 0 ]; then
                kopsysguess="none"
                return 0
        fi

        index=0
        for elements in `echo $1 | $sed -e 's/-/ /g'`; do
                if [ $index -eq 0 ]; then
                        karchguess=$elements
                fi

                if [ $index -eq 1 ]; then
                        kvendor=$elements
                fi

#
# Clean up the kopsysguess string to get rid of the actual numbers and for
# DGUX the R.  Can not use [0-9.R] as string1 to tr since DGUX does not like
# it.
#
                if [ $index -eq 2 ]; then
                        kopsysguess=`echo $elements | tr -d "0123456789.R"`
                fi

                index=`expr $index + 1`
        done

#
# Make a distinction between the Little Endian DECStation Mips.  All other
# Mips boxes are Big Endian, right ???
#
        if [ karchguess = "mips" -a $kvendor = "dec" ]; then
                karchguess="mipsel"
        fi

#
# Just use convex and not c32, c2 and cX.
#
        if [ $kvendor = "convex" ]; then
                karchguess="convex"
        fi

#
# No idea what apollo really returns so use this.  I think however the
# early m68k apollo's will not be indicated correctly, but both the
# DN10000 and m68k's are Big Endian IEEE so nothing is broken.
#
        if [ $kvendor = "apollo" ]; then
                karchguess="apollo"
        fi

        return 0
}

echon()
{
        $echoN "$* $echoEnd"
}

FindEchoN

# # ========================================================================
# #
# # guess what OS & machine architecture we are on
# #
# # ========================================================================

guess=`../../../script/kbuild/src/config.guess`
SplitGuess $guess

# # ========================================================================
# #
# # get details of the OS we're building on
# #
# # ========================================================================
needkopsys=1
while [ $needkopsys = 1 ]; do
        needkopsys=0
        cat <<"	EndOpSysList"
        What Operatings System (OS) are you building Khoros on?

        sunos:       SunOS 4.1.X                (supported)
        solaris:     Solaris 2.x                (supported)
        osf:         OSF/1                      (supported)
        aix:         AIX 3.2                    (partially ported)
        dgux:        DGUX                       (partially ported)
        hpux:        HPUX                       (partially ported)
        irix:        IRIX 4.0.4 or 5.2          (partially ported)
        bsdi:        BSDI's BSD/OS              (partially ported)
        osf1:        Paragon's OSF1             (partially ported)
        linux:       Linux                      (partially ported)
        dynix:       Dynix                      (partially ported)
        ultrix:      Ultrix                     (partially ported)
        unicos:      Unicos 8                   (partially ported)
        freebsd:     FreeBSD                    (partially ported)
        exemplar:    Convex Exemplar HPUX       (partially ported)
        dap:         CPP-DAP                    (not ported)
        netbsd:      NetBSD                     (not ported)
        convex:      ConvexOS                   (not ported)
        newos:       NewOS                      (not ported)
        interactive: Interactive 386/486        (not ported)

        `q' to QUIT the installation
	EndOpSysList
        echon  "'q' to quit [$kopsysguess]: "
        read kopsys

        if [ "$kopsys" = "q" -o "$kopsys" = "Q" ]; then
                $mv Makefile.org Makefile
                echo ""
                echo "Exiting the Khoros installation script.  Bye..."
                echo ""
                exit 0
        fi

        if [ "$kopsys" = ""  ]; then
                kopsys=$kopsysguess
        fi

        case $kopsys in

            sunos*)       defopsys="SUNOS" ;;      # Sun (SunOS 4.1.X)

            solaris*)     defopsys="SOLARIS" ;;    # Sparc (Solaris 2.1)

            osf*)         defopsys="OSF" ;;        # DEC Alpha

            dgux*)        defopsys="DGUX" ;;       # Data General

            ultrix*)      defopsys="ULTRIX" ;;     # DEC 3100/5000

            netbsd*)      defopsys="NETBSD" ;;     # NetBSD

            hpux*)        defopsys="HPUX" ;;       # HP 9000

            aix*)         defopsys="AIX" ;;        # IBM RS/6000

            irix*)        defopsys="IRIX" ;;       # SGI 4.0.4 or 5.2

            newsos*)      defopsys="NEWSOS" ;;     # Sony

            interactive*) defopsys="I386" ;;       # 386/486

            unicos*)      defopsys="UNICOS" ;;     # Cray

            convex*)      defopsys="CONVEX" ;;     # Convex

            bsdi*)        defopsys="BSDI" ;;       # BSD/386

            linux*)       defopsys="LINUX" ;;      # Linux

            freebsd*)     defopsys="FREEBSD" ;;    # FreeBSd 1.1

            osf1*)        defopsys="OSF1" ;;       # Paragon

            dap*)         defopsys="DAP" ;;        # CPP-DAP

            dynix*)       defopsys="DYNIX" ;;      # Dynix

            exemplar*)    defopsys="EXEMPLAR" ;;   # HP Convex Exemplar

            *)  echo ""
                echo "You entered an invalid machine type, please re-enter"
                echo ""
                needkopsys=1 ;;

        esac

done

defname=`echo $defopsys | tr "[A-Z]" "[a-z]"`

# # ========================================================================
# #
# # get details of the machine architecture we're building on
# #
# # ========================================================================
needkarch=1
while [ $needkarch = 1 ]; do
        needkarch=0
        needkarch=0
        cat <<"	EndArchList"
        What machine architecture are you building Khoros on?

        sparc:   Sun 4, Sparc
        alpha:   Dec Alpha
        i386:    386/486 PC, Sequent
        i486:    486 PC
        mips:    SGI Mips, Mips
        rs6000:  IBM RS/6000
        powerpc: IBM PowerPC
        hppa:    HP 9000 (s700, s800), Convex Exemplar
        ymp:     Cray
        convex:  Convex
        i860:    Paragon
        ns32k:   Sequent, Encore
        m88k:    DG, Luna, Motorola 88K
        dap:     CPP-DAP
        vax:     Vax
        mipsel:  DEC 3100/5000
        apollo:  APOLLO DN10000
        m68k:    Sun 3, Motorola 68K, HP 9000 (s300, s400), NeXT, Mac, Sony

        `q' to QUIT the installation
	EndArchList
        echon  "'q' to quit [$karchguess]: "
        read karch

        if [ "$karch" = "q" -o "$karch" = "Q" ]; then
                $mv Makefile.org Makefile
                echo ""
                echo "Exiting the Khoros installation script.  Bye..."
                echo ""
                exit 0
        fi

        if [ "$karch" = ""  ]; then
                karch=$karchguess
        fi

        case $karch in

            sparc*) defarch="SPARC" ;;

            alpha*) defarch="ALPHA" ;;

            i[34]86*) defarch="I80x86" ;;

            mips*) defarch="MIPSEB" ;;

            rs6000*) defarch="RS6000" ;;

            powerpc*) defarch="RS6000" ;;

            hppa*) defarch="PARISC" ;;

            ymp*) defarch="CRAY" ;;

            convex*) defarch="CONVEX" ;;

            i860*) defarch="I860" ;;

            ns32k*) defarch="NS32000" ;;

            m88k*) defarch="M88x00" ;;

            dap*)  defarch="SPARC" ;;

            vax*)  defarch="VAX" ;;

            mipsel*) defarch="MIPSEL" ;;

            apollo*) defarch="APOLLO" ;;

            m68k*) defarch="MC680x0" ;;

            *)  echo ""
                echo "You entered an invalid machine type, please re-enter"
                echo ""
                needkarch=1 ;;

        esac

done

defarchname=`echo $defarch | tr "[A-Z]" "[a-z]"`

needcc=1
while [ $needcc = 1 ]; do
        needcc=0
        echon "Enter C-compiler [cc]: "
        read resp
        if [ "$resp" = "" ]; then
                ccomp=cc
        else
                ccomp="$resp"
        fi

        foundcc=""
        for ccdir in `echo $PATH | $sed -e 's/:/ /g'`; do
            if [ -x "$ccdir/$ccomp" ]; then
               foundcc="$ccdir/$ccomp"
               break
            fi
        done
done

make -f Makefile.ini clean 
make CC=$ccomp BOOTSTRAPCFLAGS="-DKARCH_$defarch -DKOPSYS_$defopsys" -f Makefile.ini
make -f Makefile.ini Makefile
make install
