/*
 * Khoros: $Id: utilities.c,v 1.1 1996/05/16 20:36:46 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: utilities.c,v 1.1 1996/05/16 20:36:46 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  Static:
   >>>>             _static_routines()
   >>>>  Public:
   >>>>             public_routines()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "kset.h"


/*-----------------------------------------------------------
| Routine Name: function_name - short description of function
|
| Purpose:	This function is used to add a misc file to an
|		software object or toolbox.
|
| Input:	object    - kcms object reference for a toolbox or
|			    software object.
|		misc_file - the name or path to the misc file.
|			    If this is not a full path, then we
|			    look for the file in the misc directory
|			    of the object.
|
| Written By:	Neil Bowers
| Date:		16-May-96
------------------------------------------------------------*/
void
kset_add_misc_file(
   kobject  object,
   kstring  misc_file)
{
   kstring  routine = "kset_add_misc_file()";
   kstring  path    = NULL;


   /*-- get the path to the top of the object ---------------------*/
   if (!kcms_get_attribute(object, KCMS_PATH, &path))
   {
      errno = KCMS_EINTERNAL;
      kerror(NULL, routine,
	     "Could not determine object's directory (KCMS_PATH)\n");
      return;
   }

   /*-- filename with no path, look for it in $OBJECT/misc/ ----------*/
   if (kstrchr(misc_file, '/') == NULL)
   {
      kstring  misc_path;


      /*-- get the path to the top of the object ---------------------*/
      if ((misc_path = kstring_3cat(path, "/misc/", misc_file, NULL)) == NULL)
      {
	 errno = KCMS_EINTERNAL;
	 kerror(NULL, routine,
		"Could not build a string for the misc file's path :-(\n");
	 return;
      }

      /*-- does the misc file live in the default misc directory? ----*/
      if (kaccess(misc_path, R_OK) == -1)
      {
	 kerror(NULL, routine,
		"I could not find the misc file in the default location:\n"
		"\n"
		"\t%s\n"
		"\n"
		"Please give an explicit toolbox relative path.\n",
		misc_path);
	 kfree(misc_path);
	 return;
      }

      kcms_create_fileobj(object, misc_path, NULL,
			  KCMS_FOBJ_TYPE_MISC, KCMS_FOBJ_SUBTYPE_NONE,
			  KCMS_FOBJ_GEN_NONE, KCMS_FOBJ_ACCESS_RDWR);
      kfree(misc_path);
   }
   else if (kaccess(misc_file, R_OK) == 0)
   {
      kcms_create_fileobj(object, misc_file, NULL,
			  KCMS_FOBJ_TYPE_MISC, KCMS_FOBJ_SUBTYPE_NONE,
			  KCMS_FOBJ_GEN_NONE, KCMS_FOBJ_ACCESS_RDWR);
   }
   else
   {
      kstring misc_path;


      if ((misc_path = kstring_3cat(path, "/", misc_file, NULL)) == NULL)
      {
	 errno = KCMS_EINTERNAL;
	 kerror(NULL, routine,
		"Could not build a string for the misc file's path :-(\n");
	 return;
      }

      /*-- does the misc file live in the default misc directory? ----*/
      if (kaccess(misc_path, R_OK) == -1)
      {
	 kerror(NULL, routine,
		"I could not find the misc file; I guessed you had given "
		"an object relative path\n"
		"\n"
		"\t%s\n"
		"\n",
		misc_path);
	 kfree(misc_path);
	 return;
      }

      kcms_create_fileobj(object, misc_path, NULL,
			  KCMS_FOBJ_TYPE_MISC, KCMS_FOBJ_SUBTYPE_NONE,
			  KCMS_FOBJ_GEN_NONE, KCMS_FOBJ_ACCESS_RDWR);
      kfree(misc_path);
   }
}

