/*
 * Khoros: $Id: file.c,v 1.11 1996/05/19 23:57:24 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: file.c,v 1.11 1996/05/19 23:57:24 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		CMS File Utility Routines
   >>>>		
   >>>>  Private:
   >>>>                 kcms_fobj_free()
   >>>>                 kcms_fobj_duplicate()
   >>>>                 kcms_fobj_destroy()
   >>>>   Static:
   >>>>                 _kcms_fobj_initialize()
   >>>>   Public:
   >>>>                 kcms_open_fileobj()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include "fileP.h"

static kobject _kcms_fobj_initialize PROTO((kobject, char *, char *));

/************************************************************
* Routine Name:	kcms_open_fileobj - create a file object for an existing file
*
* Purpose:	This routine allocates and initializes a kcms file object
*		structure for an existing file on disk.
*
* Input:	parent	 - The parent kobject for the new file object.
*		filename - path in cm or Imakefile if prefix is NULL
*		           otherwise assumed to be fullname path
*		           beginning with $TOOLBOX/.../filename
*		path	 - The $TOOLBOX/... part of the fullname path;
*		           set to NULL if name uses the fullname path.
*		ftype	 - The file type.  Preprocessor symbols for
*		           legal file types can be found in the include
*		           file "kcms/fileobj.h"
*
*		           Note, if type is set to -1, this routine will
*		           try to figure out the type and subtype given
*		           the path and relative path.
*
*		subtype	 - The subtype of the file depending on type.
*		           Preprocessor symbols for legal file subtypes
*		           can be found in the include
*		           file "kcms/fileobj.h"
*
*		           Note, if type is set to -1, this value is
*		           ignored.
*
*		source	 - A flag indicating which tool generated it.
*		           Preprocessor symbols for legal source flags
*		           can be can be found in the include
*		           file "kcms/fileobj.h"
*
*		perms	 - A flag indicating the user permissions on
*		           the file it can be one of:
*		           ! KCMS_FACCESS_RDWR - User can read and write the
*		                                 file
*		           ! KCMS_FACCESS_RDONLY - User can only read the file
* Output:	
* Returns:	The opened file object, or NULL on failure.
* Written By:	Steven Jorgensen and Neil Bowers
* Date:		13-aug-94
*************************************************************/
kobject
kcms_open_fileobj(
   kobject  parent,
   char    *name,
   char    *path,
   long     type,
   long     subtype,
   long     source,
   long     perms)
{
   kstring  routine = "Open File Object";
   kobject  fileobj;


   if (name == NULL)
   {
      kinfo(KSYSLIB, "kcms_open_fileobj: name parameter is NULL");
      errno = KCMS_NULLNAME;
      return NULL;
   }

   if (!kcms_legal_object(parent, routine)
       || (fileobj = _kcms_fobj_initialize(parent, name, path)) == NULL)
      return NULL;

   /*UPDATE: there is some kinda kludge here, set file_attr.c for details --*/
   if (source < KCMS_FOBJ_GEN_NONE || source > KCMS_FOBJ_GEN_STRUCT)
   {
      errno = KCMS_RANGEERR;
      return NULL;
   }

   fileobj->generated = source;
    
   if (!kcms_set_attributes(fileobj,
			    KCMS_FOBJ_TYPE,      type,
			    KCMS_FOBJ_SUBTYPE,   subtype,
			    KCMS_FOBJ_ACCESS,    perms,
			    KCMS_END))
   {
      return NULL;
   }

   return fileobj;
}

/*-----------------------------------------------------------
| Routine Name:	_kcms_fobj_initialize - initialize the CMS file object
|
| Purpose:	This routine will initialize the CMS file object.
|
| Input:	name   - pathname given in the CM file or fullpath name.
|		prefix - prefix to prepend to "name"
|		parent - parent kobject of the file
| Output:	
| Returns:	new data object on success, NULL otherwise
| Written By:	Steven Jorgensen
| Date:		Apr 21, 1993 13:53
------------------------------------------------------------*/
static kobject
_kcms_fobj_initialize(
   kobject  parent,
   char    *name,
   char    *prefix)
{
   kstring  routine         = "_kcms_fobj_initialize()";
   kobject  fileobj;
   char     nname[KLENGTH];
   kstring  tmp;
   char     cmname[KLENGTH];
   char     temp[KLENGTH];


   errno = KCMS_OK;
   if ((fileobj = (kobject) kcalloc((size_t)1,sizeof(struct _kobject)))==NULL)
   {
      errno = KCMS_CANNOTMALLOCOBJ;
      kerror(KCMS, routine, "Unable to kmalloc (%ld) bytes.\n",
	     (long) sizeof(struct _kobject));
      return NULL;
   }

   if (prefix != NULL)
   {
      if (((unsigned int)prefix[kstrlen(prefix) - 1]) != '/')
	 kstring_3cat(prefix, "/", name, nname);
      else
	 kstring_cat(prefix, name, nname);
      name = nname;
   }
   tmp = kstrchr(name, '/');
   if (tmp != NULL)
      kstring_cleanup(tmp + 1, cmname);
   else
      kstring_cleanup(name, cmname);
   if (((unsigned int)name[0]) != '$')
   {
      errno = KCMS_INVALIDPATH;
      kerror(KCMS, routine, "Path does not begin with $TOOLBOX/.\n"
	     "File objects must exist within a toolbox");
      kfree_and_NULL(fileobj);
      return NULL;
   }
   kstring_cleanup(name, temp);
   fileobj->type          = KOBJ_CMSFILE;
   fileobj->fullname      = kstring_to_token(temp);
   fileobj->cm_name       = kstring_to_token(cmname);
   fileobj->attributes    = kcms_fobj_attr_tbl();
   fileobj->attribute_cnt = KCMS_FOBJ_MAX;

   fileobj->add_sub_obj   = NULL;
   fileobj->del_sub_obj   = NULL;
   fileobj->mod_sub_obj   = NULL;
   fileobj->parent        = parent;

   fileobj->callbacks     = NULL;

   return fileobj;
}


/*-----------------------------------------------------------
| Routine Name:	kcms_fobj_free - free memory associated with a file object
|
| Purpose:	This routine frees the memory associated with a
|		CMS file object
|
| Input:	object - the object to free
| Output:	TRUE (1) on success, FALSE (0) on failure.
| Written By:	Steven Jorgensen
| Date:		Apr 21, 1993 22:46
------------------------------------------------------------*/
int
kcms_fobj_free(
   kobject fileobj)
{
   if (fileobj == NULL)
      return FALSE;

   kfree_and_NULL(fileobj);
   return TRUE;
}


/*-----------------------------------------------------------
| Routine Name:	kcms_fobj_duplicate - duplicate a file object in one
|		parent object into another.
|
| Purpose:	This routine copies the physical file on disk,
|		as well as the file information structure to
|		a new parent object.  The new parent must
|		be of the same type as the old one. (i.e
|		if the old parent is a program object, you can't
|		copy it to a toolbox object.)  Finally, it
|		informs the parent object that it has arrived.
|
| Input:	src  - the object to copy
|		dest - the parent object destination
|
| Returns:	the kobject duplicated on success, NULL on error
|
| Written By:	Steven Jorgensen and Neil Bowers
| Date:		24-aug-94
------------------------------------------------------------*/
kobject
kcms_fobj_duplicate(
   kobject  src,
   kobject  dest)
{
   kobject  otb;
   kobject  ntb;
   kobject  parent;
   kobject  newfobj;
   kstring  new_tbname;
   char     temp[KLENGTH];
   char    *tmp1;
   char    *fullname = NULL;
   long     gen;
   long     acc;
   long     type;
   long     subtype;
   int      itmp;


   if (kcms_get_attribute(src, KCMS_PARENT, &parent) == FALSE)
      return NULL;
   if (parent->type != dest->type)
   {
      errno = KCMS_OBJTYPEMISMATCH;
      return NULL;
   }
   if (parent->type == KOBJ_CMSOBJ)
   {
      if (!kcms_get_attribute(parent, KCMS_PARENT, &otb))
	 return NULL;

      if (!kcms_get_attribute(dest, KCMS_PARENT, &ntb))
	 return NULL;
   }
   else
   {
      otb = parent;
      ntb = dest;
   }


   if (kcms_get_attribute(src, KCMS_PATH, &fullname) == FALSE)
      return NULL;

   if (kcms_get_attribute(ntb, KCMS_NAME, &new_tbname) == FALSE)
      return NULL;

   /*UPDATE: we could check to see if reparenting in the same tb -- */
   ksprintf(temp, "$%s/%s", new_tbname, kstrchr(fullname, '/') + 1);

   if (kcms_get_attributes(src,
			   KCMS_FOBJ_TYPE,      &type,
			   KCMS_FOBJ_SUBTYPE,   &subtype,
			   KCMS_FOBJ_GENERATED, &gen,
			   KCMS_FOBJ_ACCESS,    &acc,
			   KCMS_END) == FALSE)
      return NULL;

   if ((newfobj = kcms_create_fileobj(dest, temp, NULL, type, subtype,
                                  gen, acc)) == NULL)
      return NULL;

   newfobj->update_routine = src->update_routine;

   if (kcms_get_attribute(newfobj, KCMS_PATH, &tmp1) == FALSE)
   {
      itmp = errno;
      kcms_destroy(newfobj);
      errno = itmp;
      return NULL;
   }

   if (kcms_copyfile(otb, ntb, fullname, tmp1) == FALSE)
   {
      kcms_destroy(newfobj);
      errno = KCMS_PERMISSION;
      return NULL;
   }
   return newfobj;
}

/*-----------------------------------------------------------
| Routine Name:	kcms_fobj_destroy - destroy a file object and it's associated
|		file.
|
| Purpose:	This routine kunlinks the file represented by the cms file
|		object, and then destroy's the file object info structure.
|
| Input:	object - the object to destroy
| Output:	
| Returns:	TRUE (1) on success, FALSE (0) otherwise
| Written By:	Steven Jorgensen & Neil Bowers
| Date:		10-jul-94
------------------------------------------------------------*/
int
kcms_fobj_destroy(
   kobject  fileobj)
{
   kstring  routine   = "Destroy File Object";
   kstring  tmp;
   int      result    = FALSE;
   long     file_gen;
   


   if (!kcms_get_attributes(fileobj,
			   KCMS_PATH,           &tmp,
			   KCMS_FOBJ_GENERATED, &file_gen,
			   KCMS_END))
      return FALSE;

   /*------------------------------------------------------------------
   | if the file is from an associated library, don't unlink the file,
   | just let the cmobj code break the association
   ------------------------------------------------------------------*/
   if (file_gen != KCMS_FOBJ_GEN_LINCL && file_gen != KCMS_FOBJ_GEN_LFILE)
   {
      /*-- if the file is there, unlink (delete) it ------------------*/
      if (kaccess(tmp, R_OK) == 0 && kunlink(tmp) == -1)
         kerror(KCMS, routine, "Unable to unlink (kunlink) file %s.", tmp);
   }

   /*-- remove the file object from its parent --------------------*/
   if (fileobj->parent->del_sub_obj(fileobj->parent, fileobj))
   {
      kcms_fobj_free(fileobj);
      result = TRUE;
   }

   return result;
}
