/*
 * Khoros: $Id: attrib_misc.c,v 1.10.2.1 1996/12/06 20:27:27 fas Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: attrib_misc.c,v 1.10.2.1 1996/12/06 20:27:27 fas Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>  Private:
   >>>>                 kvf_get_logic_val()
   >>>>                 kvf_set_logic_val()
   >>>>                 kvf_get_logic_def()
   >>>>                 kvf_set_logic_def()
   >>>>                 kvf_get_logic_label()
   >>>>                 kvf_set_logic_label()
   >>>>                 kvf_get_list_index()
   >>>>                 kvf_set_list_index()
   >>>>                 kvf_get_list_label()
   >>>>                 kvf_set_list_label()
   >>>>                 kvf_get_list_size()
   >>>>                 kvf_set_list_size()
   >>>>                 kvf_get_list_val()
   >>>>                 kvf_set_list_val()
   >>>>                 kvf_get_list_item()
   >>>>                 kvf_set_list_item()
   >>>>                 kvf_get_list_start()
   >>>>                 kvf_set_list_start()
   >>>>                 kvf_get_list_contents()
   >>>>                 kvf_set_list_contents()
   >>>>                 kvf_get_list_doubleclick()
   >>>>                 kvf_set_list_doubleclick()
   >>>>                 kvf_get_exectype()
   >>>>                 kvf_set_exectype()
   >>>>                 kvf_get_routine()
   >>>>                 kvf_set_routine()
   >>>>                 kvf_get_help_path()
   >>>>                 kvf_set_help_path()
   >>>>                 kvf_get_me_subforms()
   >>>>                 kvf_set_me_subforms()
   >>>>                 kvf_set_list_add()
   >>>>                 kvf_set_list_delete()
   >>>>                 kvf_set_list_deleteall()
   >>>>                 kvf_get_val_labels()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"


/*-----------------------------------------------------------
|
|       Routine: kvf_get_logic_val
|       Purpose: Gets the value of the logical from 
|                the (-l) UIS line of a logical selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LOGIC_VAL 
|        Output: calldata    - passes back logical value (TRUE or FALSE)
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_logic_val(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info    line_info;
	int          *logic_val;

        /* set the pointer to be returned */
        logic_val = (int *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	if (line_info.logical_val)
	    *logic_val = TRUE;
	else *logic_val = FALSE;

	kvf_free_line_info_strings(&line_info);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_logic_val
|       Purpose: Sets the value of the logical on 
|                the (-l) UIS line of a logical selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LOGIC_VAL 
|                calldata    - provides boolean value of TRUE or FALSE
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_logic_val(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	int          *logic_val;
	Line_Info    line_info;

	/* the value to which KVF_LOGIC_VAL is to be set */
        logic_val  = (int *) calldata;
	if (!(kvf_check_boolean(*logic_val)))
        {
	    errno = KINVALID_PARAMETER;
            kerror("kforms", "kvf_set_logic_val", 
		   "'kvf_logic_val' may only be set to 0 or 1");
	    return(FALSE);
        }

	/*	
	 * change default of logical
	 */
	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	line_info.logical_val = *logic_val;
	kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_get_logic_def
|       Purpose: Gets the value of the logical default field
|                from the (-l) UIS line of a logical selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LOGIC_DEF 
|        Output: calldata    - passes back logical default (TRUE or FALSE)
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_logic_def(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info    line_info;
	int          *logic_def;

        /* set the pointer to be returned */
        logic_def = (int *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	if (line_info.logical_def)
	    *logic_def = TRUE;
	else *logic_def = FALSE;

	kvf_free_line_info_strings(&line_info);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_logic_def
|       Purpose: Sets the value of the logical default field
|                on the (-l) UIS line of a logical selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LOGIC_DEF 
|                calldata    - provides boolean value of TRUE or FALSE
|        Output: none
|
|          Date: Nov 12, 1992
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_logic_def(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	int          *logic_def;
	Line_Info    line_info;

	/* the value to which KVF_LOGIC_DEF is to be set */
        logic_def  = (int *) calldata;
	if (!(kvf_check_boolean(*logic_def)))
        {
	    errno = KINVALID_PARAMETER;
            kerror("kforms", "kvf_set_logic_def", 
		   "'kvf_logic_def' may only be set to 0 or 1");
	    return(FALSE);
        }

	/*	
	 * change default of logical
	 */
	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	line_info.logical_def = *logic_def;
	kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	kvf_set_attribute(kformstruct, KVF_LOGIC_VAL, *logic_def);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_get_logic_label
|       Purpose: Gets the label associated with a value of 0 or 1
|                from the (-l) UIS line of a logical selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LOGIC_1LABEL 
|                              KVF_LOGIC_0LABEL 
|                              KVF_LOGIC_LABEL  
|        Output: calldata    - passes back title for value of 0 or 1 on logical
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_logic_label(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info    line_info;
	char         **val_label;
	int          logic_val;

        /* set the pointer to be returned */
        val_label = (char **) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);


	if (kstrcmp(attribute, KVF_LOGIC_1LABEL) == 0)
	{
	    if (kstrcmp(line_info.val_labels[1], " ") == 0)
	 	*val_label = NULL;
	    else *val_label = kstrdup(line_info.val_labels[1]);
	}

	else if (kstrcmp(attribute, KVF_LOGIC_0LABEL) == 0)
	{
	    if (kstrcmp(line_info.val_labels[0], " ") == 0)
		*val_label = NULL;
	    else *val_label = kstrdup(line_info.val_labels[0]);
	}

	else if (kstrcmp(attribute, KVF_LOGIC_LABEL) == 0)
	{
	     kvf_get_attribute(kformstruct, KVF_LOGIC_VAL, &logic_val);
	     if (kstrcmp(line_info.val_labels[logic_val], " ") == 0)
                *val_label = NULL;
             else *val_label = kstrdup(line_info.val_labels[logic_val]);
	}

	kvf_free_line_info_strings(&line_info);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_logic_label
|       Purpose: Sets the label associated with a value of 0 or 1
|                on the (-l) UIS line of a logical selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LOGIC_1LABEL 
|                              KVF_LOGIC_0LABEL 
|                              KVF_LOGIC_LABEL  
|                calldata    - provides string for value of 0 or 1 for logical
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_logic_label(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	char         **val_label;
	Line_Info    line_info;
	int          logic_val;

	/* value to which KVF_LOGIC_1LABEL or KVF_LOGIC_0LABEL is to be set */
        val_label  = (char **) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);


	if (kstrcmp(attribute, KVF_LOGIC_1LABEL) == 0)
	{
	    kfree_and_NULL(line_info.val_labels[1]);
	    if (kstrlen(*val_label) > 0)
	        line_info.val_labels[1] = kstrdup(*val_label);
	    else line_info.val_labels[1] = kstrdup(" ");
	}
	else if (kstrcmp(attribute, KVF_LOGIC_0LABEL) == 0)
	{
	    kfree_and_NULL(line_info.val_labels[0]);
	    if (kstrlen(*val_label) > 0)
	        line_info.val_labels[0] = kstrdup(*val_label);
	    else line_info.val_labels[0] = kstrdup(" ");
	}
	else if (kstrcmp(attribute, KVF_LOGIC_LABEL) == 0)
        {
	    kvf_get_attribute(kformstruct, KVF_LOGIC_VAL, &logic_val);
            kfree_and_NULL(line_info.val_labels[logic_val]);
            if (kstrlen(*val_label) > 0)
                line_info.val_labels[logic_val] = kstrdup(*val_label);
            else line_info.val_labels[logic_val] = kstrdup(" ");
        }

	else return(FALSE);

	kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}
/*-----------------------------------------------------------
|
|       Routine: kvf_get_list_index
|       Purpose: Gets the value of the value index field
|                from the (-l), (-x), (-z), or (-c) UIS line 
|                of a logical, list, displaylist, or cycle selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_INDEX 
|        Output: calldata    - passes back integer value of the list index
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_list_index(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info    line_info;
	int          *list_index;

        /* set the pointer to be returned */
        list_index = (int *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	if (kformstruct->Selptr->type == KUIS_STRINGLIST)
	    *list_index = line_info.list_val - 1;
	else *list_index = line_info.list_val - line_info.int_val;

	kvf_free_line_info_strings(&line_info);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_list_index
|       Purpose: Sets the value of the value index field
|                on the (-l), (-x), (-z), or (-c) UIS line of a 
|                logical, list, displaylist, or cycle selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_INDEX 
|                calldata    - provides integer value of list_index
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_list_index(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	int          *list_index;
	Line_Info    line_info;

	/* the value to which KVF_VAL_INDEX is to be set */
        list_index  = (int *) calldata;

	if (*list_index == -1)
	{
	    if (kformstruct->flag != KUIS_DISPLAYLIST)
	    {
		kerror("kforms", "kvf_set_list_index",
		       "Cannot set list index to -1, except on displaylist selections");
		return(FALSE);
	    }
	}
	else if (*list_index < 0)
	{
	    kerror("kforms", "kvf_set_list_index",
		   "Attempt to set list index to invalid negative value");
	    return(FALSE);
	}

	/*	
	 * change val index
	 */
	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	if (kformstruct->Selptr->type == KUIS_STRINGLIST)
	    line_info.list_val = *list_index + 1;
	else line_info.list_val = *list_index + line_info.int_val;
	kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_get_list_label
|       Purpose: Gets the label associated with a value of n from the
|                (-l), (-x), (-z), or (-c) UIS line of a 
|                logical, list, displaylist, or cycle selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_LABEL 
|        Output: calldata    - passes back title for value of n on logical
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_list_label(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info    line_info;
	char         **list_label;
	int          list_index;

        /* set the pointer to be returned */
        list_label = (char **) calldata;

	if (!(kvf_get_attribute(kformstruct, KVF_LIST_INDEX, &list_index)))
	    return(FALSE);

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	if (kformstruct->Selptr->type == KUIS_STRINGLIST)
            list_index = line_info.list_val - 1;
        else list_index = line_info.list_val - line_info.int_val;

	if (list_index == -1)
	    *list_label = NULL;
	else if (line_info.val_labels == NULL)
	    *list_label = NULL;
	else if (kstrcmp(line_info.val_labels[list_index], " ") == 0)
	    *list_label = NULL;
	else *list_label = kstrdup(line_info.val_labels[list_index]);

	kvf_free_line_info_strings(&line_info);
	return(TRUE);
 
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_list_label
|       Purpose: Sets the label associated with a value of N on the
|                (-l), (-x), (-z), or (-c) UIS line of a 
|                logical, list, displaylist, or cycle selection.
|                Note that the list value must be set to N first,
|                before the label for that value can be assigned.
|
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_LABEL 
|                calldata    - provides string value of title for 
|                            value of n on logical
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_list_label(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	char         **list_label;
	int          list_index;
	Line_Info    line_info;

	/* the value to which KVF_VAL_LABEL is to be set */
        list_label  = (char **) calldata;

	/* can't set list label to NULL */
	if (*list_label == NULL)
	{
	    kerror("kforms", "kvf_set_list_label",
		   "Attempt to set list label to invalid NULL value");
	    return(FALSE);
	}

	/* get the current index, to see which label we are setting */
	if (!(kvf_get_attribute(kformstruct, KVF_LIST_INDEX, &list_index)))
            return(FALSE);

	/* on displaylists, index can be set to -1, indicating no value.
           if this is the case, cannot change the label */
	if (list_index == -1)
	{
	    kerror("kforms", "kvf_set_list_label",
		   "Invalid attempt to set label on displaylist selection when index is set to -1 (indicating NO current value)");
	    return(FALSE);
	}

	/* set the label */
	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	/* make sure label isn't a duplicate */
	if (karray_locate(line_info.val_labels, KSTRING, *list_label,
                          line_info.val_labelnum) != -1)
        {
            kerror("kforms", "kvf_set_list_label",
                   "There is already an item in list '%s' with label '%s'; cannot use label '%s' again, sorry.",
                    line_info.title, *list_label, *list_label);
            kvf_free_line_info_strings(&line_info);
            return(FALSE);
        }

        kfree(line_info.val_labels[list_index]);
	if (kstrlen(*list_label) > 0)
            line_info.val_labels[list_index] = kstrdup(*list_label);
	else line_info.val_labels[list_index] = kstrdup(" ");
        kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_get_list_size
|       Purpose: Gets the number of list choices from the 
|                (-l), (-x), (-z), or (-c) UIS line of a 
|                logical, list, displaylist, or cycle selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_SIZE 
|        Output: calldata    - passes back number of choices for list
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_list_size(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info  line_info;
	int      *list_size;

        /* set the pointer to be returned */
        list_size = (int *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

        *list_size = (int) line_info.list_num;

        kvf_free_line_info_strings(&line_info);
	return(TRUE);

}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_list_size
|       Purpose: Sets the number of list choices on the 
|                (-l), (-x), (-z), or (-c) UIS line of a 
|                logical, list, displaylist, or cycle selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_SIZE 
|                calldata    - provides number of choices for list
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_list_size(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	int         *list_size;
	Line_Info    line_info;
	kselection  *selection;
	char         temp[KLENGTH];

        /* the value to which KVF_LIST_SIZE is to be set */
        list_size = (int *) calldata;

	selection = kformstruct->Selptr;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	/*
	 *  update list num, and reallocate val labels
	 */
	line_info.list_num = *list_size;

	if (line_info.list_num > line_info.val_labelnum)
	{
	   while (line_info.val_labelnum < line_info.list_num)
	   {
		ksprintf(temp, "Choice %d", line_info.val_labelnum + 1);
		line_info.val_labels = karray_insert(line_info.val_labels, 
					             KSTRING, kstrdup(temp),
			                             line_info.val_labelnum++,
						     KLIST_TAIL, TRUE);
	   }
	}

	/*
	 *  recalculate the current list value
	 */
	if (line_info.list_val > (line_info.int_val + line_info.list_num - 1))
	    line_info.list_val = line_info.int_val + line_info.list_num - 1;
	else if (line_info.list_val < line_info.int_val)
	    line_info.list_val = line_info.int_val;

	if ((line_info.list_val - line_info.int_val == -1) &&
	    (selection->type != KUIS_DISPLAYLIST))
	    line_info.list_val =  line_info.int_val;

        kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_get_list_val
|       Purpose: Gets the current list choice (by integer value) from the 
|                (-l), (-x), (-z), or (-c) UIS line of a 
|                logical, list, displaylist, or cycle selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_VAL 
|        Output: calldata    - passes back default choice for list
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_list_val(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        Line_Info  line_info;
        int        *list_val;

        /* set the pointer to be returned */
        list_val = (int *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

        *list_val = line_info.list_val;

        kvf_free_line_info_strings(&line_info);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_list_val
|       Purpose: Sets the current list choice (by integer value) on the 
|                (-l), (-x), (-z), or (-c) UIS line of a 
|                logical, list, displaylist, or cycle selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_VAL 
|                calldata    - provides default choice for list
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_list_val(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info    line_info;
        int          i, *list_val; 
        kselection   *selection;
	char         temp[KLENGTH];
	char         mesg[KLENGTH];

        /* set the pointer to be returned */
        list_val = (int *) calldata;

	selection = kformstruct->Selptr;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	if ((*list_val < line_info.int_val) &&
	    (selection->type != KUIS_DISPLAYLIST))
	{
	    mesg[0] = '\0';
	    ksprintf(mesg, "list value for %s (%s) must be >= start (%d); legal values include:\n",
                   line_info.title, line_info.variable, line_info.int_val);
	    for (i = 0; i < line_info.list_num; i++)
            {
                if (i < line_info.list_num-1)
		{
		     ksprintf(temp, "%d (\"%s\"), ", 
			      i+line_info.int_val, line_info.val_labels[i]);
                     kstring_cat(mesg, temp, mesg);
		}
                else 
		{
		    ksprintf(temp, "or %d (\"%s\")\n", 
			     i+line_info.int_val, line_info.val_labels[i]);
                    kstring_cat(mesg, temp, mesg);
		}
            }
	    kerror("kforms", "kvf_set_list_val", mesg);
	    kvf_free_line_info_strings(&line_info);
	    return(FALSE);
	}
	else if ((*list_val < line_info.int_val - 1) &&
            (selection->type == KUIS_DISPLAYLIST))
	{
	    kerror("kforms", "kvf_set_list_val",
		   "list default must be >= start (%d) to indicate selected item, or (%d) to indicate that NO item is selected.", line_info.int_val, line_info.int_val - 1);
	    kvf_free_line_info_strings(&line_info);
	    return(FALSE);
	}

	else if (*list_val > (line_info.int_val + line_info.list_num - 1))
	{
	    mesg[0] = '\0';
            ksprintf(mesg, "list value for %s (%s) must be <= %d; legal values include:\n",
		     line_info.title, line_info.variable, 
		     line_info.int_val + line_info.list_num - 1);
            for (i = 0; i < line_info.list_num; i++)
            {
                if (i < line_info.list_num-1)
                {
                     ksprintf(temp, "%d (\"%s\"), ",
                              i+line_info.int_val, line_info.val_labels[i]);
                     kstring_cat(mesg, temp, mesg);
                }
                else
                {
                    ksprintf(temp, "or %d (\"%s\")",
                             i+line_info.int_val, line_info.val_labels[i]);
                    kstring_cat(mesg, temp, mesg);
                }
            }
	    kerror("kforms", "kvf_set_list_val", mesg);
	    kvf_free_line_info_strings(&line_info);
	    return(FALSE);
	}
	else line_info.list_val = *list_val;

        kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_get_list_item
|       Purpose: Gets the current list choice (by string value) from the 
|                (-l), (-x), (-z), or (-c) UIS line of a 
|                logical, list, displaylist, or cycle selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_VAL 
|        Output: calldata    - passes back default choice for list
|
|          Date: March 31, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_list_item(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        Line_Info  line_info;
	char      **list_item;
	int        indx;

        /* set the pointer to be returned */
        list_item = (char **) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	indx =  line_info.list_val - line_info.int_val;

	if (line_info.val_labels == NULL)
	    *list_item = NULL;
	else if (indx < 0)
	    *list_item = NULL;
	else if (kstrcmp(line_info.val_labels[indx], " ") == 0)
	    *list_item = NULL;
	else *list_item = kstrdup(line_info.val_labels[indx]);

        kvf_free_line_info_strings(&line_info);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_list_item
|       Purpose: Sets the current list choice (by string value) on the 
|                (-l), (-x), (-z), or (-c) UIS line of a 
|                logical, list, displaylist, or cycle selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_ITEM 
|                calldata    - provides default choice for list
|        Output: none
|
|          Date: March 31, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_list_item(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        int          i; 
	Line_Info    line_info;
	char         **list_item;

        /* set the pointer to be returned */
        list_item = (char **) calldata;

	if (*list_item == NULL)
	{	
	    kerror("kforms", "kvf_set_list_item",
		   "Cannot set value to NULL");
	    return(FALSE);
	}

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	if (line_info.val_labels == NULL)
	    return(FALSE);

	/*
	 *  they want to set the list value to the item with the 
         *  string they specify.  first, find the string in the list.
         *  if it's not there, return false.
	 */
	i = 0;
        while ((i < line_info.list_num) &&
               (kstrncasecmp(line_info.val_labels[i], 
			     *list_item, kstrlen(*list_item)) != 0))
             i++;
	if (i >= line_info.list_num)
             return(FALSE);

        kvf_free_line_info_strings(&line_info);

	kvf_set_attribute(kformstruct, KVF_LIST_INDEX, i);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_get_list_start
|       Purpose: Gets the integer start value from the 
|                (-l), (-x), (-z), or (-c) UIS line of a 
|                logical, list, displaylist, or cycle selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_START 
|        Output: calldata    - passes back starting integer for list
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_list_start(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info    line_info;
        int          *list_start;

        /* set the pointer to be returned */
        list_start = (int *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

        *list_start = line_info.int_val;

        kvf_free_line_info_strings(&line_info);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_list_start
|       Purpose: Sets the integer start value on the 
|                (-l), (-x), (-z), or (-c) UIS line of a 
|                logical, list, displaylist, or cycle selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_START 
|                calldata    - provides starting integer for list
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_list_start(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        int          old_index, list_val;
	int          *list_start;
	Line_Info    line_info;

        /* the value to which KVF_LIST_START is to be set */
        list_start = (int *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	old_index = line_info.list_val - line_info.int_val;

	if (!(kvf_get_attribute(kformstruct, KVF_LIST_VAL, &list_val)))
	{
	    kvf_free_line_info_strings(&line_info);
	    return(FALSE);
	}

        /*
         * current list val less than new list start, putting the
         * list value L of range.  scoot list value up to list start.
         */
        if (list_val < *list_start)
            line_info.list_val = *list_start;

        /*
         * current list value greater than list_start + item num, putting the
         * list value R of range.  scoot list value back to highest value.
         */
        else if (list_val > (*list_start + line_info.list_num - 1))
            line_info.list_val = *list_start + line_info.list_num - 1;

	line_info.int_val = *list_start;
        kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	
	/*
	 *  current list value is within range now, but relative offset probably
         *  changed.  (ie, if the 2nd item was selected before, want the
         *  second item selected now, even though the start value changed.
	 */
	if (line_info.list_num > 0)
	    kvf_set_attribute(kformstruct, KVF_LIST_INDEX, old_index);


	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_get_list_contents
|       Purpose: Gets the list of strings from the 
|                (-l), (-x), (-z), or (-c) UIS line of a
|                logical, list, displaylist, or cycle selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_CONTENTS 
|        Output: calldata    - passes back array of strings making up list
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_list_contents(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info    line_info;
        char         ***val_labels;

        /* set the pointer to be returned */
        val_labels = (char ***) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	*val_labels = karray_copy(line_info.val_labels, KSTRING,
                                  (size_t) line_info.list_num, TRUE);

        kvf_free_line_info_strings(&line_info);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_list_contents
|       Purpose: Sets the list of string on the 
|                (-l), (-x), (-z), or (-c) UIS line of a
|                logical, list, displaylist, or cycle selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_CONTENTS 
|                calldata    - provides array of strings for list
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications: 
| ------------------------------------------------------------*/

/* ARGSUSED */
int kvf_set_list_contents(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	int          i;
	kselection   *selection;
	char         ***val_labels;
	Line_Info    line_info;

        /* the value to which KVF_LIST_CONTENTS is to be set */
        val_labels = (char ***) calldata;

	selection = kformstruct->Selptr;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);


        /*
         *  move new list contents into line_info.val_labels array
         */
        for (i = 0; i < line_info.list_num; i++)
        {
            kfree_and_NULL(line_info.val_labels[i]);
	    if ( (*val_labels)[i] == NULL )
	    {
		kerror("kforms", "kvf_set_list_contents",
		       "You must enter something in each field.");
		return(FALSE);
	    }
            line_info.val_labels[i] = kstrdup((*val_labels)[i]);
        }

	if (selection->type == KUIS_DISPLAYLIST)
        {
	   if ((line_info.list_num > 0) || (*val_labels != NULL))
		line_info.list_val = line_info.int_val - 1;
	}
        kvf_gen_deparse_kformstruct(kformstruct, &line_info);
	return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_get_list_doubleclick
|       Purpose: Gets the value of the doubleclick field
|                from the (-z) DISPLAYLIST UIS line 
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_DOUBLECLICK 
|        Output: calldata    - passes back boolean value of the list index
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_list_doubleclick(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info    line_info;
	int          *doubleclick;

        /* set the pointer to be returned */
        doubleclick = (int *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	*doubleclick = line_info.special;

	kvf_free_line_info_strings(&line_info);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_list_doubleclick
|       Purpose: Sets the value of the doubleclick field
|                from the (-z) DISPLAYLIST UIS line
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_DOUBLECLICK 
|                calldata    - provides integer value of list_index
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_list_doubleclick(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	int          *doubleclick;
	Line_Info    line_info;

	/* the value to which KVF_VAL_INDEX is to be set */
        doubleclick  = (int *) calldata;

        if (!(kvf_check_boolean(*doubleclick)))
        {
            errno = KINVALID_PARAMETER;
            kerror("kforms", "kvf_set_list_doubleclick",
                   "KVF_LIST_DOUBLECLICK may only be set to 0 or 1");
            return(FALSE);
        }

	/*	
	 * change doubleclick flag
	 */
	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	line_info.special = *doubleclick;
	kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_get_exectype
|       Purpose: Gets the execution type from the (-R) UIS line 
|                of a routine selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_EXECTYPE 
|        Output: calldata    - passes back execution type for Routine button
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_exectype(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info    line_info;
        int          *exectype;

        /* set the pointer to be returned */
        exectype = (int *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

        *exectype = line_info.exec_type;

        kvf_free_line_info_strings(&line_info);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_exectype
|       Purpose: Sets the execution type on the (-R) UIS line 
|                of a routine selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_EXECTYPE 
|                calldata    - provides execution type for Routine button
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications: 
| ------------------------------------------------------------*/

/* ARGSUSED */
int kvf_set_exectype(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	int        *exectype;
	Line_Info  line_info;

        /* the value to which KVF_EXECTYPE is to be set */
        exectype = (int *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	line_info.exec_type = *exectype;
        kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_get_routine
|       Purpose: Gets the value of the routine field from 
|                the (-R) UIS line of a routine selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute - KVF_ROUTINE 
|        Output: calldata  - passes back string value of routine
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_routine(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info    line_info;
	char         **routine;

        /* set the pointer to be returned */
        routine = (char **) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	if (line_info.routine == NULL)
	{
	    *routine = NULL;
	    return(FALSE);
	}
	else *routine = kstrdup(line_info.routine);

	kvf_free_line_info_strings(&line_info);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_routine
|       Purpose: Sets the value of the routine field on 
|                the (-R) UIS line of a routine selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_ROUTINE 
|                calldata    - provides string value of routine
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
-------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_routine(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	char         **routine;
	Line_Info    line_info;

	/* the value to which KVF_ROUTINE is to be set */
        routine  = (char **) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);


        kfree_and_NULL(line_info.routine);
        if (*routine == NULL)
            *routine = kstrdup(" ");
        else line_info.routine = kstrdup(*routine);

        kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_get_help_path
|       Purpose: Gets the value of the help path from 
|                the (-H) UIS line of a Help selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_HELPPATH 
|        Output: calldata    - passes back string value of help_path
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_help_path(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info    line_info;
	char         **help_path;

        /* set the pointer to be returned */
        help_path = (char **) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	if (line_info.help_file == NULL)
	{
	    *help_path = NULL;
	    return(FALSE);
	}
	else *help_path = kstrdup(line_info.help_file);

	kvf_free_line_info_strings(&line_info);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_help_path
|       Purpose: Sets the value of the help path on 
|                the (-H) UIS line of a Help selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_HELPPATH 
|                calldata    - provides string value of help_path
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
-------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_help_path(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	char         **help_path;
	Line_Info    line_info;

	/* the value to which KVF_HELPPATH is to be set */
        help_path  = (char **) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	kfree_and_NULL(line_info.help_file);
	line_info.help_file = kstrdup(*help_path);

	kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_get_me_subforms
|       Purpose: Gets the TRUE/FALSE value of whether or not multiple
|                subforms on a master are supposed to be mutually 
|                exclusive (ie, only one allowed to map at any one time)
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_ME_SUBFORMS 
|        Output: calldata    - passes back boolean value of me_subform:
|                              TRUE  - subforms are mutually exclusive
|                              FALSE - multiple subforms may be mapped
|          Date: April 7, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_me_subforms(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info    line_info;
	int          *me_subforms;

        /* set the pointer to be returned */
        me_subforms = (int *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	*me_subforms = line_info.logical_val;

	kvf_free_line_info_strings(&line_info);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_me_subforms
|       Purpose: Sets the TRUE/FALSE value of whether or not multiple
|                subforms on a master are supposed to be mutually 
|                exclusive (ie, only one allowed to map at any one time)
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_ME_SUBFORMS 
|                calldata    - passes in integer value of scope: 
|                              TRUE  - subforms are mutually exclusive
|                              FALSE - multiple subforms may be mapped
|        Output: none
|          Date: April 7, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
-------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_me_subforms(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	int          *me_subforms;
	Line_Info    line_info;

	/* the value to which KVF_ME_SUBFORMS is to be set */
        me_subforms  = (int *) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	line_info.logical_val = *me_subforms;

	kvf_gen_deparse_kformstruct(kformstruct, &line_info);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|       Routine: kvf_set_list_add
|       Purpose: Adds an item to a list of choices on the
|                (-l), (-x), (-z), or (-c) UIS line of a
|                logical, list, displaylist, or cycle selection
|                This is an action attribute routine.
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_ADD 
|                calldata    - provides label for new item in list
|                              (index is determined by length of list;
|                               new item is always added at the end>)
|        Output: none
|
|          Date: Jan 31, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_list_add(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	char         **new_choice;
        Line_Info    line_info;
	kselection   *selection;

	new_choice = (char **) calldata;
	
        kvf_clear_line_info(&line_info);
        selection = kformstruct->Selptr;
	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	/*
	 *  update number of choices; save choice label
	 */

	if (kstrlen(*new_choice) < 1)
	{
	    kerror("kforms", "kvf_set_list_add",
		   "Cannot add item of length 0 to list (ie, a NULL or empty string)");
	    kvf_free_line_info_strings(&line_info);
	    return(FALSE);
	}

	line_info.list_num++;
	line_info.val_labels = karray_insert(line_info.val_labels, 
                                             KSTRING, kstrdup(*new_choice),
		                             line_info.val_labelnum++,
					     KLIST_TAIL, TRUE);

	kvf_gen_deparse(&line_info, &selection->line);
	kvf_free_line_info_strings(&line_info);
	return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_set_list_delete
|       Purpose: Deletes an item from the list of choices on the
|                (-l), (-x), (-z), or (-c) UIS line of a
|                logical, list, displaylist, or cycle selection
|                This is an action attribute routine.
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_DELETE 
|                calldata    - provides index of choice to be deleted
|        Output: none
|
|          Date: Jan 31, 1994
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_list_delete(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
        kselection   *selection;
        Line_Info    line_info;
        int          reset = FALSE; 
	char         **old_choice;

	old_choice = (char **) calldata;

        selection = kformstruct->Selptr;
	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);


	if (*old_choice == NULL)
	{
            kvf_free_line_info_strings(&line_info);
            return(FALSE);
	}

	if (line_info.list_num == 0)
	{
	    kerror("kforms", "kvf_change_listdel",
		   "list selection has no choices; cannot delete choice '%s'",
		    *old_choice); 
	    kvf_free_line_info_strings(&line_info);
	    return(FALSE);
	}

	if (karray_locate(line_info.val_labels, KSTRING, *old_choice,
			  line_info.val_labelnum) != -1)
	{
	    line_info.val_labels = karray_delete(line_info.val_labels,
					          KSTRING, *old_choice, 
				                  line_info.val_labelnum--);
	    line_info.list_num--;
	    if (line_info.list_val > line_info.int_val+line_info.list_num - 1)
	    {
		line_info.list_val = line_info.int_val + line_info.list_num - 1;
	        reset = TRUE;
	    }
	}
	else
	{
	    kerror("kforms", "kvf_set_list_delete", 
		   "Cannot delete item '%s' from list '%s'; no such item",	
		   *old_choice, line_info.title);
	    kvf_free_line_info_strings(&line_info);
	    return(FALSE);
	}

	kvf_gen_deparse(&line_info, &selection->line);

	if ((line_info.list_num > 0) && (reset))
	    kvf_set_attribute(kformstruct, KVF_LIST_VAL, 
	    	 	      line_info.int_val + line_info.list_num - 1);

	kvf_free_line_info_strings(&line_info);
	return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_set_list_deleteall
|       Purpose: Deletes all items from the list of choices on the
|                (-l), (-x), (-z), or (-c) UIS line of a
|                logical, list, displaylist, or cycle selection
|                This is an action attribute routine.
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_LIST_DELETEALL 
|                calldata    - provides integer value of TRUE (ignored)
|        Output: none
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_set_list_deleteall(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info    line_info;
	kselection   *selection;

        kvf_clear_line_info(&line_info);
        selection = kformstruct->Selptr;
	kvf_gen_parse(selection->line, &line_info);

	kfree_and_NULL(line_info.string_val);

	line_info.list_num = 0;
	line_info.list_val = line_info.int_val;

	karray_free(line_info.val_labels, KSTRING, 
		    line_info.val_labelnum, NULL);
	line_info.val_labelnum = 0;
	line_info.val_labels   = NULL;

        kvf_gen_deparse(&line_info, &selection->line);
	kvf_free_line_info_strings(&line_info);

	return(TRUE);
}


/*-----------------------------------------------------------
|
|       Routine: kvf_get_val_labels
|       Purpose: Gets the val_labels array of strings from the
|                (-l), (-x), (-z), or (-c) UIS line of a
|                logical, list, displaylist, or cycle selection
|
|         Input: kformstruct - generic kformstruct holding ptr to selection
|                attribute   - KVF_VAL_LABELS 
|        Output: calldata    - passes back array of strings making up list
|
|          Date: March 22, 1994
|    Written By: Danielle Argiro
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int kvf_get_val_labels(
   kform_struct *kformstruct,
   char         *attribute,
   kaddr        calldata)
{
	Line_Info line_info;
	char      ***val_labels;

        /* the value to which KVF_VAL_LABELS is to be set */
        val_labels = (char ***) calldata;

	if (!(kvf_gen_parse_kformstruct(kformstruct, &line_info)))
	    return(FALSE);

	/* DON'T call kvf_free_line_info_strings after this...
	   it is the only get routine that doesn't calloc */
	*val_labels = line_info.val_labels;

	kfree_and_NULL(line_info.title);
        kfree_and_NULL(line_info.description);
        kfree_and_NULL(line_info.variable);
        kfree_and_NULL(line_info.routine);
        kfree_and_NULL(line_info.help_file);
        kfree_and_NULL(line_info.filename);
        kfree_and_NULL(line_info.string_val);
        kfree_and_NULL(line_info.string_def);
        kfree_and_NULL(line_info.file_def);
        kfree_and_NULL(line_info.literal);
        kfree_and_NULL(line_info.lib_call);

	return(TRUE);
}
