/*
 * Khoros: $Id: link.c,v 1.4 1996/04/04 22:47:11 fas Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: link.c,v 1.4 1996/04/04 22:47:11 fas Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		<<<<
   >>>>		General Utilities for Form Tree Linking            <<<<
   >>>>		<<<<
   >>>>  Private:
   >>>>                 kvf_fastlink_subform()
   >>>>                 kvf_fastlink_guide()
   >>>>                 kvf_fastlink_selection()
   >>>>                 kvf_fastlink_group()
   >>>>                 kvf_fastlink_toggle()
   >>>>                 kvf_fastlink_submenu()
   >>>>                 kvf_link_selection()
   >>>>                 kvf_link_toggle()
   >>>>                 kvf_link_group()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"


/*-------------------------------------------------------------
|
|  Routine Name: kvf_fastlink_subform
|
|       Purpose: Links subform into linked list of subforms
|                This routine is used only by createtree.c routines;
|                it is more efficient because it passes back the *last
|                pointer so we don't have to search for the last subform
|
|         Input: master  - pointer to the master
|                subform - subform to link in
|                last    - calling routine declares last
|        Output: none
|       Returns: nothing
|          Date: May 13, 1992
|    Written By: Danielle Argiro 
| Modifications:
|
---------------------------------------------------------------*/
void kvf_fastlink_subform(
   kcontrol *master,
   ksubform *subform,
   ksubform **last_subform)
{
	if (master->subform_list == NULL)
	{
	    master->subform_list = subform;
	    *last_subform = subform;
	}
	else
	{
	    (*last_subform)->next = subform;
	    *last_subform = subform;
	}
}

/*-------------------------------------------------------------
|
|  Routine Name: kvf_fastlink_guide
|
|       Purpose: Links guide into linked list of guides
|                This routine is used only by createtree.c routines;
|                it is more efficient because it passes back the *last
|                pointer so we don't have to search for the last guide
|
|         Input: guidepane - pointer to the guidepane
|                guide     - guide to link in
|                last      - calling routine declares last
|        Output: none
|       Returns: nothing
|          Date: May 13, 1992
|    Written By: Danielle Argiro 
| Modifications:
|
---------------------------------------------------------------*/
void kvf_fastlink_guide(
   kcontrol *guidepane,
   kguide   *guide,
   kguide   **last)
{
	if (guidepane->guide_list == NULL) 
	{ 
	    guidepane->guide_list = guide;
	    *last = guide;
	}
	else
	{
	    (*last)->next = guide;
	    *last = guide;
	}
}

/*-------------------------------------------------------------
|
|  Routine Name: kvf_fastlink_selection 
|
|       Purpose: Links selection into linked list of selections.
|                This routine is used only by createtree.c routines;
|                it is more efficient because it passes back the *last
|                pointer so we don't have to search for the last selection
|
|         Input: control   - pointer to the master, guidepane, or pane
|                selection - selection to link in
|                last      - calling routine declares last
|        Output: 
|       Returns:
|          Date: May 13, 1992
|    Written By: Danielle Argiro 
| Modifications:
|
---------------------------------------------------------------*/
void kvf_fastlink_selection (
   kcontrol   *control,
   kselection *selection,
   kselection **last)
{
	if (control->sel_list == NULL) 
	{ 
	    control->sel_list = selection;
	    *last = selection;
	}
	else
	{
	    (*last)->next = selection;
	    *last = selection;
	}
}

/*-------------------------------------------------------------
|
|  Routine Name: kvf_fastlink_group
|
|       Purpose: Links a group member into linked list of members
|                This routine is used only by createtree.c routines;
|                it is more efficient because it passes back the *last
|                pointer so we don't have to search for the last group member
|
|         Input: group_start  - pointer to the group header
|                member       - group member to link in
|                last         - calling routine declares last
|        Output: none
|       Returns: nothing
|          Date: Oct 07, 1992
|    Written By: Danielle Argiro
| Modifications:
|
---------------------------------------------------------------*/
void kvf_fastlink_group(
   kselection *group_start,
   kselection *member,
   kselection **last)
{
	if (group_start->group_next == NULL)
        {
            group_start->group_next = member;
            *last = member;
        }
        else
        {
            (*last)->next = member;
            *last = member;
        }
}

/*-------------------------------------------------------------
|
|  Routine Name: kvf_fastlink_toggle
|
|       Purpose: Links a toggle member into linked list of toggle members
|                This routine is used only by createtree.c routines;
|                it is more efficient because it passes back the *last
|                pointer so we don't have to search for the last toggle member
|
|         Input: group_start  - pointer to the toggle header
|                member       - toggle member to link in
|                last         - calling routine declares last
|        Output: none
|       Returns: nothing
|          Date: Oct 07, 1992
|    Written By: Danielle Argiro
| Modifications:
|
---------------------------------------------------------------*/
void kvf_fastlink_toggle(
   kselection *toggle_start,
   kselection *member,
   kselection **last)
{
	if (toggle_start->toggle_next == NULL)
        {
            toggle_start->toggle_next = member;
            *last = member;
        }
        else
        {
            (*last)->next = member;
            (*last)->next = member;
            *last = member;
        }
}


/*-------------------------------------------------------------
|
|  Routine Name: kvf_fastlink_submenu
|
|       Purpose: Links kformstruct into linked list of submenu items
|                This routine is used only by createtree.c routines;
|                it is more efficient because it passes back the *last
|                pointer so we don't have to search for the last submenu
|
|         Input: submenu_start - pointer to the submenu (-D) 
|                kformstruct     - submenu member to link in
|                last          - calling routine declares last
|        Output: none
|       Returns: nothing
|          Date: May 13, 1992
|    Written By: Danielle Argiro 
| Modifications:
|
---------------------------------------------------------------*/
void kvf_fastlink_submenu(
   kselection   *submenu_start,
   kform_struct *member,
   kform_struct **last_member)
{
	if (submenu_start->submenu_next == NULL)
	{
	    submenu_start->submenu_next = member;
	    *last_member = member;
	}
	else
	{
	    (*last_member)->next = member;
	    *last_member = member;
	}
}


/*-------------------------------------------------------------
|
|  Routine Name: kvf_link_selection 
|       Purpose: Links selection into linked list of selections.
|                This routine is used creation.c routines;
|                it is less efficient because it has to search for
|                the last selection in the list each time since the
|                sel_list is not doubly linked) but guise and xvlang
|                are not in a position to save the *last pointer like
|                the routines in createtree.c are.
|
|         Input: control   - pointer to the master, guidepane, or pane
|                selection - selection to link in
|        Output:
|       Returns:
|          Date: June 5, 1994
|    Written By: Danielle Argiro
| Modifications:
|
---------------------------------------------------------------*/
void kvf_link_selection(
   kcontrol   *control,
   kselection *selection)
{
        kselection *last;

        if (control->sel_list == NULL)
            control->sel_list = selection;
        else
        {
            last = control->sel_list;
            while(last->next != NULL)
                last = last->next;

            last->next = selection;
        }
}

/*-------------------------------------------------------------
|
|  Routine Name: kvf_link_toggle
|       Purpose: Links a toggle member into linked list of toggle members
|                This routine is used creation.c routines;
|                it is less efficient because it has to search for
|                the last selection in the list each time since the
|                sel_list is not doubly linked) but guise and xvlang
|                are not in a position to save the *last pointer like
|                the routines in createtree.c are.
|
|         Input: control   - pointer to the master, guidepane, or pane
|                selection - selection to link in
|        Output:
|       Returns:
|          Date: June 5, 1994
|    Written By: Danielle Argiro
| Modifications:
|
---------------------------------------------------------------*/
void kvf_link_toggle(
   kselection *toggle_start,
   kselection *member)
{
        kselection *last;

        if (toggle_start->toggle_next == NULL)
            toggle_start->toggle_next = member;
        else
        {
            last = toggle_start->toggle_next;
            while(last->next != NULL)
                last = last->next;

            last->next = member;
        }
}

/*-------------------------------------------------------------
|
|  Routine Name: kvf_link_group
|       Purpose: Links a group member into linked list of group members
|                This routine is used creation.c routines;
|                it is less efficient because it has to search for
|                the last selection in the list each time since the
|                sel_list is not doubly linked) but guise and xvlang
|                are not in a position to save the *last pointer like
|                the routines in createtree.c are.
|
|         Input: control   - pointer to the master, guidepane, or pane
|                selection - selection to link in
|        Output:
|       Returns:
|          Date: June 5, 1994
|    Written By: Danielle Argiro
| Modifications:
|
---------------------------------------------------------------*/
void kvf_link_group(
   kselection *group_start,
   kselection *member)
{
        kselection *last;

        if (group_start->group_next == NULL)
            group_start->group_next = member;
        else
        {
            last = group_start->group_next;
            while(last->next != NULL)
                last = last->next;

            last->next = member;
        }
}

