/*
 * Khoros: $Id: Shm.c,v 1.16.2.1 1996/12/06 20:27:52 fas Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Shm.c,v 1.16.2.1 1996/12/06 20:27:52 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		UNIX Pipe Transport Routines
   >>>>		
   >>>>   Static:
   >>>>                 shm_ftok()
   >>>>                 shm_shmget()
   >>>>                 shm_reopen()
   >>>>                 shm_initialize()
   >>>>                 shm_tempnam()
   >>>>                 shm_open()
   >>>>                 shm_close()
   >>>>                 shm_read()
   >>>>                 shm_write()
   >>>>                 shm_lseek()
   >>>>                 shm_access()
   >>>>                 shm_unlink()
   >>>>                 shm_lock()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include <machine/cdefs.h>
#ifdef KOPSYS_AIX
#define   _XOPEN_SOURCE
#else
#ifdef KOPSYS_DGUX
#define   _DGUX_SOURCE
#else
#ifdef KOPSYS_HPUX
#ifndef _HPUX_SOURCE
#define   _HPUX_SOURCE
#endif
#else
#undef   _POSIX_SOURCE
#endif
#endif
#endif
#include "internals.h"	

#if !defined(KSHM_DEF)
TransportInformation shm_transport[] = { {NULL} };
#else

#include <sys/ipc.h>
#include <sys/shm.h>

static int shm_tempnam	 PROTO((const char *, const char *, char *));
static int shm_open	 PROTO((const char *, int, mode_t, kfile *));
static int shm_close	 PROTO((kfile *));
static ssize_t shm_read	 PROTO((kfile *, kaddr, size_t));
static ssize_t shm_write PROTO((kfile *, kaddr, size_t));
static off_t shm_lseek	 PROTO((kfile *, off_t, int));
static int shm_access	 PROTO((const char *, const char *, int));
static int shm_unlink	 PROTO((const char *, const char *));
static int shm_lock	 PROTO((kfile *, int));

#if KOPSYS_SUNOS
char *shmat		PROTO((int, char *, int));
#endif

TransportInformation shm_transport[] =
{
    {
	"Shared Memory",
	"shm",
	TRUE,
	shm_tempnam,
	shm_open,
	shm_close,
	shm_read,
	shm_write,
	shm_lseek,
	shm_access,
	shm_unlink,
	shm_lock,
	NULL,
    }
};

/*
 *  Internal resource structures for "standard file" transport
 */
typedef struct
{
	int    shmid;
	key_t  key;
	off_t  offset;
	int    shmflag;
	size_t size;
	char   *addr;
} ResourceStruct;


/*-------------------------------------------------------------------*
|  
|		  Private Shared Memory Routines
|  
--------------------------------------------------------------------*/
/*ARGSUSED*/
static key_t shm_ftok(
   char *filename,
   int  id)
{
        struct stat buf;

        return(stat(filename, &buf) == -1 ? (key_t) -1 : (key_t)
                ((id << 24) | ((key_t) buf.st_dev << 16) | (key_t) buf.st_ino));
}

/*ARGSUSED*/
static int shm_shmget(
   key_t key,
   int   size,
   int   flags)
{
	return(shmget(key, (size_t) size, flags));
}

/*ARGSUSED*/
static int shm_reopen(
   kfile *file,
   size_t num_bytes)
{
	int	status = 0;
	char	*temp, *addr = NULL;
	ResourceStruct *resources = (ResourceStruct *) file->resources;

	addr = (char *) kmalloc(resources->size);
	if (addr != NULL)
	   kmemcpy(addr, resources->addr, resources->size);

	if (resources->addr != NULL)
	{
	   status = shmdt(resources->addr);
	   if (shmctl(resources->shmid, IPC_RMID, NULL) < 0)
	   {
	      kfree_and_NULL(addr);
	      return(-1);
	   }
	}

	if ((resources->shmid = shm_shmget(resources->key, (int) num_bytes,
			resources->shmflag)) < 0)
	{
	   kfree_and_NULL(addr);
	   return(-1);
	}

	/*
	 *  Attach to file the appropriate memory map space...
	 */
        if ((temp = shmat(resources->shmid, 0, SHM_RND)) == (char *) -1)
	{
	   kfree_and_NULL(addr);
           return(-1);
	}

	/*
	 *  copy data back into the new shared memory segment.
	 */
	if (addr != NULL)
	   kmemcpy(temp, addr, resources->size);

	resources->addr = temp;
	resources->size = num_bytes;

	kfree_and_NULL(addr);
	return(status);
}

/*ARGSUSED*/
static int shm_initialize(
   key_t key,
   int shmflg,
   int *size)
{
	int	shmid;
	struct  shmid_ds stats;


	/*
	 *  Get the shared memory id.  We pass in 1024 just in case
	 *  we are going to be creating this segment.
	 */
	if ((shmid = shm_shmget(key, 1, shmflg)) < 0)
	   return(-1);

	/*
	 *  Get the size of the segment in case anyone wants to know
	 */
	if (size != NULL)
	{
	   if (shmctl(shmid, IPC_STAT, &stats) < 0)
	      return(-1);

	   *size = stats.shm_segsz;
	}
	return(shmid);
}


/*-------------------------------------------------------------------*
|  
|		  Shared Memory Routines
|  
--------------------------------------------------------------------*/

/*-------------------------------------------------------------
|  
|  Routine Name: shm_tempnam
|  
|       Purpose: This function initializes a tempnam for a "file"
|	         type transport.  Given a request with a filename of
|	         the following syntax:
|  
|		  	"shm=XXXXXXXXXXX"
|		  	     ...etc....
|  
|         Input:  identifier - the transport identifier
|		  template   - the filename template to be modeled
|        Output:  result     - the newly created transport name
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int shm_tempnam(
   const char *identifier,
   const char *template,
   char *result)
{
	key_t shmkey;
	char  path[KLENGTH], *filename;

	/*
	 *  Make a tempnam from the template
	 */
	(void) kstrcpy(path, template);
	if ((filename = ktmpnam(path)) == NULL ||
	    creat(filename, (mode_t) 0666) == -1)
	   return(-1);

	/*
	 *  Initialize the Shared memory id to something unqiue by using
	 *  tempnam and the shm_ftok() routine.
	 */
	if ((shmkey = shm_ftok(filename, 0)) == -1)
	{
	   /*
	    *  Opps...  We failed to create a unique key.  Error and return
	    *  failure.
	    */
	   kinfo(KDEBUG,"shm_tempnam: Unable to get unique shared memory key.");
	   return(-1);
	}

	/*
	 *  Initialize and create
	 */
	if (shm_initialize(shmkey, 0664 | IPC_CREAT, NULL) < -1)
	   return(-1);

	/*
	 *  Store the unique key so that we can use it later.
	 */
	if (identifier == NULL)
	   (void) kstrcpy(result, filename);
	else
	   (void) sprintf(result,"%s=%s", identifier, filename);

	return(0);
}

/*-------------------------------------------------------------
|  
|  Routine Name: 	shm_open
|  
|       Purpose: This function opens a "shared memory" connection.  It is the
|	         internal driver to open a file, which is called by
|	         the user when they call kopen().  kopen() calls when
|	         it gets an open request with a filename of the following
|	         syntax:
|  
|		  	"shm=XXXXXXXXX"
|         Input: 
|        Output:  returns 0 or -1 if an error occurs
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int shm_open(
   const char *path,
   int    flags,
   mode_t mode,
   kfile  *file)
{
	char	  *addr;
	ResourceStruct *resources;
	char	  filename[KLENGTH];
	int	  size, shmid, shmflag, shmkey, prot = SHM_RND;

	/*
	 *  Get shared memory (shmat) protections from the open permission
	 *  flags...
	 */
	if (!kfile_iswrite(file))
	{
	   prot &= SHM_RDONLY;
	   mode = 0400;
	}
	else
	   mode = 0664;

	/*
	 *  Get the size of the current segment.  If the file is to be
	 *  created, etc...
	 */
	shmflag = mode;
	if (flags & KOPEN_EXCL) shmflag |= IPC_EXCL;
	if (flags & KOPEN_CREAT) shmflag |= IPC_CREAT;

	/*
	 *  Open the shared memory segment...
	 */
	if (ksscanf(path, "%d", &shmkey) == 0)
	{
	   shmkey = -1;
	   if (ksscanf(path, "%s", filename) == 1 &&
	       (shmkey = shm_ftok(filename, 0)) == -1)
	   {
	      if (kfile_iswrite(file) && access(filename, F_OK) == -1)
	      {
		 creat(filename, (mode_t) 0666);
	         shmkey = shm_ftok(filename, 0);
	      }
	   }
	}

	if (shmkey == -1 || (shmid = shm_initialize(shmkey,shmflag,&size)) < 0)
	   return(kfile_rename(file, "file="));

	/*
	 *  Attach to file the appropriate memory map space...
	 */
	if ((addr = shmat(shmid, 0, prot)) == (char *) -1)
	   return(kfile_rename(file, "file="));

	if ((resources = (ResourceStruct *) kcalloc((size_t) 1,
			sizeof(ResourceStruct))) == NULL)
	{
	   kinfo(KSYSLIB, "shm_open: Not enough memory to kcalloc (%d) bytes "
	      	"for the khoros shm structure.\n", sizeof(ResourceStruct));
	   return(-1);
	}
	resources->addr    = addr;
	resources->shmid   = shmid;
	resources->shmflag = shmflag;
	resources->key     = shmkey;
	resources->size    = (flags & KOPEN_TRUNC) ? 0 : size;
	resources->offset = (flags & KOPEN_APPEND) ? size : 0;

	file->resources = (kaddr) resources;
	return(0);
}

/*-------------------------------------------------------------
|  
|  Routine Name: shm_close
|  
|       Purpose: This function closes a "shared memory".  It is the internal
|	         driver to close a data shm segment, which is called by the user
|	         when they call kclose().  kclose() calls the "shared memory"
|	         internal drivers by the nature of the transport, which is
|	         dictated by the initial kopen().
|  
|         Input:  file - the kfile structure.
|        Output:  returns whether we were able to close the file
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int shm_close(
   kfile *file)
{
	char	  *addr;
	int	  status;
	ResourceStruct *resources = (ResourceStruct *) file->resources;

	/*
	 *  Simply call shmdt() to detach from shared memory segment
	 */
	addr = resources->addr;

	status = shmdt(addr);
	return(status);
}

/*-------------------------------------------------------------
|  
|  Routine Name: shm_read
|  
|       Purpose: This function reads a "shared memory".  It is the internal
|	         driver to read data from a shm, which is called by the user
|	         when they call kread().  kread() calls the "shared memory"
|	         internal drivers by the nature of the transport, which is
|	         dictated by the kopen().
|  
|         Input:  file   - the kfile structure.
|		  ptr    - the pointer to store the data into.
|		  nbytes - the number of bytes to read.
|        Output:  returns the number of bytes read from the shm
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static ssize_t shm_read(
   kfile  *file,
   kaddr  ptr,
   size_t nbytes)
{
	char   *addr;
	off_t  offset;
	size_t size;
	struct shmid_ds stats;
	ResourceStruct *resources = (ResourceStruct *) file->resources;

	/*
	 *  Simply call kmemcpy() since "kfile" is just a regular 
	 *  memory pointer.
	 */
	addr   = resources->addr;
	size   = resources->size;
	offset = resources->offset;

	if (offset >= size)
	   return(EOF);
	else if (addr == ptr || addr+offset == ptr)
	{
	   if (offset+nbytes > size)
	      nbytes = size-offset;
	}
	else if (offset+nbytes > size)
	{
	   kmemcpy(ptr, addr+offset, (size_t) (size-offset));
	   nbytes = size-offset;
	}
	else
	   kmemcpy(ptr, addr+offset, nbytes);

	resources->offset = offset+nbytes;
	return((ssize_t) nbytes);
}

/*-------------------------------------------------------------
|  
|  Routine Name: shm_write
|  
|       Purpose: This function writes to a "shared memory".  It is the internal
|	         driver to write data from the supplied data array to a shm,
|	         which is called by the user when they call kwrite().  kwrite()
|	         calls the "shared memory" internal drivers by the nature of
|	         the transport, which is dictated by the kopen().
|  
|         Input:  file   - the kfile structure.
|		  ptr    - the pointer to store the data into.
|		  nbytes - the number of bytes to read.
|        Output:  returns the number of bytes written to the kfile
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static ssize_t shm_write(
   kfile  *file,
   kaddr  ptr,
   size_t nbytes)
{
	char    *addr;
	off_t    offset;
	size_t   size;
	ResourceStruct *resources = (ResourceStruct *) file->resources;

	/*
	 *  Simply call kmemcpy() since "kfile" is just a regular 
	 *  memory pointer.
	 */
	addr   = resources->addr;
	size   = resources->size;
	offset = resources->offset;

	/*
	 *  Make sure that we aren't trying to read more bytes than we
	 *  currently have.  If so then try re-opening the file to see
	 *  if it's gotten bigger.
	 */
	if (offset+nbytes > size)
	{
	   shm_reopen(file, (size_t) (nbytes+offset));
	   addr   = resources->addr;
	   size   = resources->size;
	}

	if (offset == size)
	   return(EOF);
	else if (addr == ptr || addr+offset == ptr)
	{
	   if (offset+nbytes > size)
	      nbytes = size-offset;
	}
	else if (offset+nbytes > size)
	{
	   kmemcpy(addr+offset, ptr, (size_t) (size-offset));
	   nbytes = size-offset;
	}
	else
	   kmemcpy(addr+offset, ptr, nbytes);

	resources->offset = offset+nbytes;
	return((ssize_t) nbytes);
}

/*-------------------------------------------------------------
|  
|  Routine Name: shm_lseek
|  
|       Purpose: This function is used to do a "lseek".  It is the internal
|	         driver to rewind to a specific point so that the data can be
|	         skipped or re-read.  This is called when the user calls
|	         klseek().  klseek() calls the "shared memory" internal drivers
|	         by the nature of the transport, which is dictated by the
|		 kopen().
|  
|         Input:  file   - the kfile structure.
|		  offset - the offset in which to seek
|		  whence - the control of how the offset will be applied
|        Output:  returns the number of bytes written to the kfile
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static off_t shm_lseek(
   kfile *file,
   off_t  offset,
   int    whence)
{
	ResourceStruct *resources = (ResourceStruct *) file->resources;

	/*
	 *  Simply call lseek() since "kfile" is just a regular shm
	 */
	if (whence == KSEEK_SET)
	   resources->offset = offset;
	else if (whence == KSEEK_CUR)
	   resources->offset += offset;
	else if (whence == KSEEK_END)
	   resources->offset = resources->size + offset;
	else
	{
	   errno = EINVAL;
	   kinfo(KSYSLIB, "shm_lseek:  Invalid whence argument value for "
                          "file '%s'", file->path);
	   return(-1);
	}

	if (resources->offset > resources->size)
	/*EMPTY*/
	{
	   /* not sure as of yet */
	}
	return(resources->offset);
}

/*-------------------------------------------------------------
|  
|  Routine Name: shm_access
|  
|       Purpose: This function initializes a tempnam for a "shm"
|	         type transport.  Given a request with a filename of
|	         the following syntax:
|  
|		  	"shm=XXXXXXXXXX"
|  
|         Input:  name - the original path name passed to kaccess
|		  path - the filename that they wish to access
|		  mode - the access mode information
|        Output:  returns 0 or -1 depending whether we
|		  sucessfully created the template
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int shm_access(
   const char *name,
   const char *path,
   int  mode)
{
	int   shmkey;
	char  filename[KLENGTH];

	/*
	 *  Retrieve the shared memory key from the filename.  If it's not
	 *  a valid key then
	 */
	if (ksscanf(path, "%d", &shmkey) != 1)
	{
	   if (ksscanf(path, "%s", filename) == 0 ||
	       (shmkey = shm_ftok(filename, 0)) == -1)
	   {
	      errno = ENOTDIR;
	      return(-1);
	   }
	   name = filename;
	}

	/*
	 *  Check to see if the key (filename) exists
	 */
	if (shm_initialize(shmkey, 0400, NULL) < 0 && name == NULL)
	   return(-1);
	else
	   return(0);
}

/*-------------------------------------------------------------
|  
|  Routine Name: shm_unlink
|  
|       Purpose: This function initializes a tempnam for a "shm"
|	         type transport.  Given a request with a filename of
|	         the following syntax:
|  
|		  	"shm=XXXXXXXXXX"
|
|         Input:  name - the original path name passed to kunlink
|		  path - the filename that they wish to unlink
|        Output:  returns 0 or -1 depending whether we
|		  sucessfully created the template
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int shm_unlink(
   const char *name,
   const char *path)
{
	char  filename[KLENGTH];
	int   shmid, shmkey, status = -1;

	/*
	 *  Retrieve the shared memory key from the filename.  If it's not
	 *  a valid key then
	 */
	if (ksscanf(path, "%d", &shmkey) != 1)
	{
	   if (ksscanf(path, "%s", filename) == 0 ||
	       (shmkey = shm_ftok(filename, 0)) == -1)
	   {
	      errno = ENOTDIR;
	      return(-1);
	   }
	   status = unlink(filename);
	}

	/*
	 *  Retrieve the shared memeory id so that we can delete it
	 *  from shared memory.
	 */
	if ((shmid = shm_initialize(shmkey, 0400, NULL)) < 0)
	   return(status);

	if (shmctl(shmid, IPC_RMID, NULL) < 0)
	   return(-1);

	return(0);
}

/*-------------------------------------------------------------
|  
|  Routine Name: shm_lock
|  
|       Purpose: This function locks a "shm" transport.  Given
|	         a request with a filename of the following syntax:
|  
|		  	"shm=key"
|         Input: 
|        Output:  returns 0 or -1 depending whether we
|		  sucessfully created the template
|    Written By:  Mark Young
|  
-------------------------------------------------------------*/
/*ARGSUSED*/
static int shm_lock(
   kfile *file,
   int   operation)
{
	return(-1);
}

#endif  /* KSHM_DEF */
/* don't add after the endif */
