/*
 * Khoros: $Id: internals.h,v 1.15 1996/08/14 00:59:58 fas Exp $
 */


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose:  This include file contains the header
   >>>>			and define information required by the
   >>>>			internal routines in kutils
   >>>>
   >>>>    Written By:  Mark Young
   >>>>
   >>>>          Date: Jul 14, 1992 10:32
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _kutils_internals_h_
#define _kutils_internals_h_

        /*-------------------------------------*
        |       #include 
        ---------------------------------------*/

#include <bootstrap.h>
#include <kutils/kfileP.h>
#include <machine/kopsys.h>

        /*-------------------------------------*
        |       #defines 
        ---------------------------------------*/


#define KUTILS       "kutils"
#define TEMPLATE     "xvtemp"
#define ALLOCSIZE    512000
#define MAX_ARGS     200
#define KMAXENTRIES  25

#define CANNOT_CREATE_NEW_DOT_FILE	-1
#define CANNOT_READ_EXISTING_DOT_FILE	-2
#define CANNOT_OPEN_TEMP_DOTFILE	-3
#define CANNOT_WRITE_EXISTING_DOT_FILE	-4
#define	LICENSED			1
#define	UNLICENSED			2
#define	KEEP_OLD_LIC_FIELD		3
#define	REGISTERED			4
#define	UNREGISTERED			5
#define	KEEP_OLD_REG_FIELD		6
#define NOT_CURRENT_VERSION  		7

/*
 *  Default Page Size
 */
#define DefaultPageSize	4096

#if KVFORK_DEF
#define kfork() vfork()
#else
#define kfork() fork()
#endif


	/*-------------------------------------*
	|       typedefs
	---------------------------------------*/


	/*-------------------------------------*
	|       macros
	---------------------------------------*/


	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

#ifdef _POSIX_SOURCE
extern int flock	PROTO ((int, int));
#if !defined(KOPSYS_EXEMPLAR) && !defined(KOPSYS_HPUX)
extern int gethostname  PROTO ((char *, int));
#endif
extern int lstat	PROTO ((const char *, struct stat *));
extern pid_t vfork	PROTO ((void));
#ifndef KOPSYS_OSF
extern char *tempnam	PROTO ((const char *, const char *));
#endif
#ifndef KOPSYS_IRIX
extern void *memccpy    PROTO ((void *, const void *, int, size_t));
#endif
extern int ioctl	PROTO ((int, int, ...));
#endif

/* kfile.c */
void kfile_initialize PROTO((void));
kfile *kfile_checkid PROTO((int));
kfile *kfile_checkfile PROTO((kfile *));
int kfile_add PROTO((kfile *));
int kfile_delete PROTO((kfile *));
int kfile_free PROTO((kfile *));
int kfile_lock PROTO((kfile *, int));
ssize_t kfile_read PROTO((kfile *, char *, size_t));
ssize_t kfile_write PROTO((kfile *, char *, size_t));
int kfile_close PROTO((kfile *));
kfile *kfile_open PROTO((const char *, int, mode_t));
int kfile_access PROTO((const char *, int));
int kfile_rename PROTO((kfile *, const char *));
off_t kfile_seek PROTO((kfile *, off_t, int));
long int kfile_tell PROTO((kfile *));
int kfile_dup PROTO((int, int));
char  *kfile_readstring	PROTO((kfile *, int, char *, int));
TransportInformation *ktransport_routines PROTO((const char *));

/* kconvert.c */
unsigned char *convert_float PROTO((unsigned char *, unsigned char *,
				    unsigned int, unsigned int, size_t));
unsigned char *convert_double PROTO((unsigned char *, unsigned char *,
				    unsigned int, unsigned int, size_t));
unsigned char *convert_complex PROTO((unsigned char *, unsigned char *,
				    unsigned int, unsigned int, size_t));
unsigned char *convert_dcomplex PROTO((unsigned char *, unsigned char *,
				    unsigned int, unsigned int, size_t));
unsigned char *convert_short PROTO((unsigned char *, unsigned char *,
				    unsigned int, unsigned int, size_t));
unsigned char *convert_ushort PROTO((unsigned char *, unsigned char *,
				    unsigned int, unsigned int, size_t));
unsigned char *convert_int PROTO((unsigned char *, unsigned char *,
				    unsigned int, unsigned int, size_t));
unsigned char *convert_uint PROTO((unsigned char *, unsigned char *,
				    unsigned int, unsigned int, size_t));
unsigned char *convert_long PROTO((unsigned char *, unsigned char *,
				    unsigned int, unsigned int, size_t));
unsigned char *convert_ulong PROTO((unsigned char *, unsigned char *,
				    unsigned int, unsigned int, size_t));
unsigned char *convert_byte PROTO((unsigned char *, unsigned char *,
				    unsigned int, unsigned int, size_t));
unsigned char *convert_ubyte PROTO((unsigned char *, unsigned char *,
				    unsigned int, unsigned int, size_t));

/* fildes.c */
int ksend_fildes PROTO((kfile *, kfile *, char *, int));
int krecv_fildes PROTO((kfile *, kfile *, char *, int *));

/* utilities.c */
char *kalias_replace PROTO((const char *, char *));
char *_cleanup_string PROTO((char *));
char *kexpand_variable PROTO((const char *, char *));
char *kexpand_tilda PROTO((const char *, char *));
char *ktmpnam PROTO((const char *));

/* gnuscan.c */
int kdoscan PROTO((kfile *, const char *, const char *, kva_list));

/* errors.c */
void kutils_init_errors PROTO((void));

#endif /* _kutils_internals_h_ */
/* don't add after the endif */
