/*
== Toolbox:	$BOOTSTRAP
== Object:	kcms
== Testsuite:	01.kcms
== Testing:	kcms_create_toolbox()
== Description:
*/

#include <bootstrap.h>
#include <ktestutils/ktestutils.h>

main(
   int    argc,
   char  *argv[])
{
   kobject  toolbox;
   kstring  workdir;
   char     path[KLENGTH];
   char     filepath[KLENGTH];
   char     command[KLENGTH];


   /*-- working directory is passed as first argument ----------------*/
   if (argc != 2)
   {
      kprintf("testsuite FAILED\n"
	      "incorrect number of command-line arguments (%d)\n", argc);
      kexit(1);
   }
   workdir = argv[1];

   ktest_begin("01.kcms", "kcms_create_toolbox()");

   /*-----------------------------------------------------------------*/
   /*-----------------------------------------------------------------*/
   ktest_case("NULL toolbox name");
   toolbox = kcms_create_toolbox(NULL, NULL);
   ktest_result((toolbox == NULL) ? KTEST_PASS : KTEST_FAIL);

   /*-----------------------------------------------------------------*/
   /*-----------------------------------------------------------------*/
   ktest_case("toolbox name is empty string");
   toolbox = kcms_create_toolbox("", NULL);
   ktest_result((toolbox == NULL) ? KTEST_PASS : KTEST_FAIL);

   /*-----------------------------------------------------------------*/
   /*-----------------------------------------------------------------*/
   ktest_case("toolbox name doesn't start with letter");
   toolbox = kcms_create_toolbox("42", NULL);
   ktest_result((toolbox == NULL) ? KTEST_PASS : KTEST_FAIL);

   /*-----------------------------------------------------------------*/
   /*-----------------------------------------------------------------*/
   ktest_case("toolbox name contains non-alphanumeric");
   toolbox = kcms_create_toolbox("bogus*&!", NULL);
   ktest_result((toolbox == NULL) ? KTEST_PASS : KTEST_FAIL);

   /*-----------------------------------------------------------------*/
   /*-----------------------------------------------------------------*/
   ktest_case("not allowed to have a toolbox called `toolbox'");
   ksprintf(path, "%s/kcmstest", workdir);
   toolbox = kcms_create_toolbox("toolbox", NULL);
   ktest_result((toolbox == NULL) ? KTEST_PASS : KTEST_FAIL);

   /*-----------------------------------------------------------------*/
   /*-----------------------------------------------------------------*/
   ktest_case("toolbox path is in non-existent directory");
   toolbox = kcms_create_toolbox("kcmstest", "/foo/bar/kcmstest");
   ktest_result((toolbox == NULL) ? KTEST_PASS : KTEST_FAIL);

   /*-----------------------------------------------------------------*/
   /*-----------------------------------------------------------------*/
   ktest_case("don't have write permission on toolbox directory");
   if (chmod(workdir, 0) == -1)
   {
      ktest_result(KTEST_FAIL);
      /* kprintf() the reason why we failed */
   }
   else
   {
      ksprintf(path, "%s/kcmstest", workdir);
      toolbox = kcms_create_toolbox("kcmstest", path);
      ktest_result((toolbox == NULL)
		 ? KTEST_PASS
		 : KTEST_FAIL);
      if (chmod(workdir, S_IRUSR | S_IWUSR | S_IXUSR) == -1)
         kprintf("      could not chmod directory back to user writable - FAILED\n");
   }

   /*-----------------------------------------------------------------*/
   /*-----------------------------------------------------------------*/
   ktest_case("don't have write permission on toolbox file(s)");

   /*-- need to ensure the toolboxes file is there --*/
   ksprintf(filepath, "%s/Toolboxes", workdir);
   ksprintf(command, "touch %s", filepath);
   ksystem(command);

   if (chmod(filepath, S_IRUSR) == -1)
   {
      ktest_result(KTEST_FAIL);
      /* kprintf() the reason why we failed */
   }
   else
   {
      ksprintf(path, "%s/kcmstest", workdir);
      toolbox = kcms_create_toolbox("kcmstest", path);
      ktest_result((toolbox == NULL)
		 ? KTEST_PASS
		 : KTEST_FAIL);
      if (toolbox != NULL)
      {
	 kcms_set_bit(toolbox, KCMS_TB_FLAGS, KCMS_BIT_TB_FORCE);
	 kcms_destroy(toolbox);
      }
      if (chmod(filepath, S_IRUSR | S_IWUSR) == -1)
         kprintf("      could not chmod Toolboxes file back to user writable - FAILED\n");
   }
   ksprintf(command, "/bin/rm -rf %s", path);

   /*-----------------------------------------------------------------*/
   /*-----------------------------------------------------------------*/
   ktest_case("toolbox path is non-empty directory");
   ksprintf(path, "%s/kcmstest", workdir);
   if (kmkdir(path, 0755) != 0)
   {
      ktest_result(KTEST_FAIL);
      kprintf("          failed to create directory \"%s\"\n", path);
   }
   else
   {
      kfile  *file;

      ksprintf(filepath, "%s/foofile", path);
      file = kfopen(filepath, "w");
      if (file == NULL)
      {
         ktest_result(KTEST_FAIL);
         kprintf("          failed to write to file \"%s\"\n", filepath);
      }
      else
      {
         kfclose(file);
         toolbox = kcms_create_toolbox("kcmstest", path);
         ktest_result((toolbox == NULL) ? KTEST_PASS : KTEST_FAIL);
         if (toolbox != NULL)
	 {
	    kcms_set_bit(toolbox, KCMS_TB_FLAGS, KCMS_BIT_TB_FORCE);
	    kcms_destroy(toolbox);
	 }
	 kunlink(filepath);
      }
      if (kremove_dir(path) == -1)
      {
	 kprintf("          failed to remove directory %s\n", path);
      }
   }

   /*-----------------------------------------------------------------*/
   /*-----------------------------------------------------------------*/
   ktest_case("toolbox should create ok");
   ksprintf(path, "%s/kcmstest", workdir);
   toolbox = kcms_create_toolbox("kcmstest", path);
   ktest_result((toolbox != NULL) ? KTEST_PASS : KTEST_FAIL);
   if (toolbox != NULL)
      kcms_close(toolbox);

   /*-----------------------------------------------------------------*/
   /*-----------------------------------------------------------------*/
   ktest_case("toolbox name same as an existing toolbox");
   ksprintf(path, "%s/kcmstest", workdir);
   toolbox = kcms_create_toolbox("kcmstest", path);
   ktest_result((toolbox == NULL) ? KTEST_PASS : KTEST_FAIL);

   /*-----------------------------------------------------------------*/
   ktest_case("toolbox path same as that of an existing toolbox");
   ksprintf(path, "%s/kcmstest", workdir);
   toolbox = kcms_create_toolbox("kcmstesttwo", path);
   ktest_result((toolbox == NULL) ? KTEST_PASS : KTEST_FAIL);

   ktest_end();
}
