/*
== Toolbox:	$BOOTSTRAP
== Object:	kcms
== Testsuite:	04.kcms
== Testing:	kcms_query_toolboxes()
== Description:
*/

#include <bootstrap.h>
#include <ktestutils/ktestutils.h>

static void print_toolbox_list(kstring *, size_t);

main(
   int    argc,
   char  *argv[])
{
   int       test_id;
   kstring  *toolboxes;
   size_t    countToolboxes;
   kobject   toolbox;
   int       i;


   /*-- working directory is passed as first argument ----------------*/
   if (argc != 2 && argc != 4)
   {
      kprintf("incorrect number of command-line arguments (%d)\n", argc);
      kexit(1);
   }
   test_id = atoi(argv[1]);

   switch (test_id)
   {
      /*----------------------------------------------------------------
      | CASE 1
      |
      |	legal call to kcms_query_toolboxes()
      |		exit with failure if we get back NULL
      | print out the list of toolboxes
      ----------------------------------------------------------------*/
      case 1:
	 toolboxes = kcms_query_toolboxes(&countToolboxes);
	 if (toolboxes == NULL)
	 {
	    kprintf("Error: kcms_query_toolboxes() returned NULL!\n");
	    kexit(KEXIT_FAILURE);
	 }
	 print_toolbox_list(toolboxes, countToolboxes);
	 break;


      /*----------------------------------------------------------------
      | CASE 2
      |
      |	legal call to kcms_query_toolboxes()
      |		exit with failure if we get back NULL
      | print out the list of toolboxes
      | call kcms_query_toolboxes() again
      |		exit with failure if we get back NULL
      | print out the list of toolboxes
      ----------------------------------------------------------------*/
      case 2:
	 toolboxes = kcms_query_toolboxes(&countToolboxes);
	 if (toolboxes == NULL)
	 {
	    kprintf("Error: kcms_query_toolboxes() returned NULL!\n");
	    kexit(KEXIT_FAILURE);
	 }
	 print_toolbox_list(toolboxes, countToolboxes);

	 toolboxes = kcms_query_toolboxes(&countToolboxes);
	 if (toolboxes == NULL)
	 {
	    kprintf("Error: kcms_query_toolboxes() returned NULL!\n");
	    kexit(KEXIT_FAILURE);
	 }
	 print_toolbox_list(toolboxes, countToolboxes);

	 break;


      /*----------------------------------------------------------------
      | CASE 3
      |
      |	legal call to kcms_query_toolboxes()
      |		exit with failure if we get back NULL
      | print out the list of toolboxes
      | create a toolbox
      |		name and path are passed as command-line arguments
      | call kcms_query_toolboxes() again
      |		exit with failure if we get back NULL
      | print out the list of toolboxes
      | delete the toolbox
      | call kcms_query_toolboxes() again
      | print out the list of toolboxes
      ----------------------------------------------------------------*/
      case 3:
	 /*-- get the list of toolboxes -----------*/
	 toolboxes = kcms_query_toolboxes(&countToolboxes);
	 if (toolboxes == NULL)
	 {
	    kprintf("Error: kcms_query_toolboxes() returned NULL!\n");
	    kexit(KEXIT_FAILURE);
	 }
	 print_toolbox_list(toolboxes, countToolboxes);

	 /*-- create a new toolbox ----------------*/
	 toolbox = kcms_create_toolbox(argv[2], argv[3]);
	 if (toolbox == NULL)
	 {
	    kprintf("Failed to create toolbox `%s', path %s\n",
		    argv[2], argv[3]);
	    kexit(KEXIT_FAILURE);
	 }
	 kcms_sync(toolbox);

	 /*-- get the list of toolboxes -----------*/
	 toolboxes = kcms_query_toolboxes(&countToolboxes);
	 if (toolboxes == NULL)
	 {
	    kprintf("Error: kcms_query_toolboxes() returned NULL!\n");
	    kexit(KEXIT_FAILURE);
	 }
	 print_toolbox_list(toolboxes, countToolboxes);

	 /*-- destroy the toolbox -----------------*/
	 if (!kcms_set_bit(toolbox, KCMS_TB_FLAGS, KCMS_BIT_TB_FORCE))
	 {
	    kprintf("Error: failed to set the FORCE bit on toolbox\n");
	    kexit(KEXIT_FAILURE);
	 }
	 kcms_destroy(toolbox);

	 /*-- get the list of toolboxes -----------*/
	 toolboxes = kcms_query_toolboxes(&countToolboxes);
	 if (toolboxes == NULL)
	 {
	    kprintf("Error: kcms_query_toolboxes() returned NULL!\n");
	    kexit(KEXIT_FAILURE);
	 }
	 print_toolbox_list(toolboxes, countToolboxes);

	 break;


      /*----------------------------------------------------------------
      | CASE 4
      |
      |	legal call to kcms_query_toolboxes()
      |		exit with failure if we get back NULL
      | print out the list of toolboxes
      | exit with success
      ----------------------------------------------------------------*/
      case 4:
	 toolboxes = kcms_query_toolboxes(NULL);
	 if (toolboxes != NULL || errno != KCMS_EINTERNAL)
	 {
	    kprintf("Error: kcms_query_toolboxes() returned non-NULL!\n");
	    kexit(KEXIT_FAILURE);
	 }
	 break;

      /*----------------------------------------------------------------
      | bomb out with an error message otherwise
      ----------------------------------------------------------------*/
      default:
	 printf("incorrect test id (%s) -- should be in the range 1 .. 4\n",
		argv[1]);
	 kexit(KEXIT_FAILURE);
   }

   kexit(KEXIT_SUCCESS);
}

static void
print_toolbox_list(
   kstring  *toolboxes,
   size_t    countToolboxes)
{
   int  i;


   kprintf("BEGIN TOOLBOX LIST:\n");
   for (i = 0; i < countToolboxes; i++)
      kprintf("\t%s\n", toolboxes[i]);
   kprintf("END TOOLBOX LIST\n");
}
