/*
== Toolbox:	$BOOTSTRAP
== Object:	kcms
== Testsuite:	06.get_objects
== Testing:	kcms_create_manual(
==			kobject  	toolbox,
==			unsigned long	reqtype)
== Description:
*/

#include <bootstrap.h>
#include <ktestutils/ktestutils.h>

main(
   int    argc,
   char  *argv[])
{
   int       test_id;
   kobject   toolbox;
   char      manual_name[KLENGTH];


   /*-- working directory is passed as first argument ----------------*/
   if (argc != 2 && argc != 3)
   {
      kprintf("incorrect number of command-line arguments (%d)\n", argc);
      kexit(1);
   }
   test_id = atoi(argv[1]);
   kprog_set_toolbox("BOOTSTRAP");
   kprog_set_program("ktest");

   switch (test_id)
   {
      /*----------------------------------------------------------------
      | CASE 1
      |
      |	toolbox is NULL
      ----------------------------------------------------------------*/
      case 1:
	 if (kcms_get_objects(NULL, 0L) != NULL)
	    ktest_fatal("Error: kcms_get_objects(NULL, 0L) returned TRUE!\n");

         if (errno != KCMS_ENULLTB)
	    ktest_fatal("Error: unexpected errno (%d)\n", errno);

	 kprintf("kcms_get_objects() case 1 OK\n");
	 break;


      /*----------------------------------------------------------------
      | CASE 2
      |
      |	valid toolbox, no object types requested
      ----------------------------------------------------------------*/
      case 2:
	 toolbox = kcms_open_toolbox(argv[2]);
	 if (toolbox == NULL)
	    ktest_fatal("Error: failed to open toolbox `%s'\n", argv[2]);

	 if (kcms_get_objects(toolbox, 0L) != NULL)
	    ktest_fatal("Error: kcms_get_objects(toolbox, 0L) "
			"returned non-NULL!\n");

	 if (!kcms_close(toolbox))
	    ktest_fatal("Error: kcms_close() returned FALSE after "
			"call to kcms_get_objects(toolbox, 0L)\n");

	 kprintf("kcms_get_objects() case 2 OK\n");
	 break;


      /*----------------------------------------------------------------
      | CASE 3
      |
      |	valid toolbox, all five standard object types requested.
      | should be NULL, since the toolbox will have just been created.
      ----------------------------------------------------------------*/
      case 3:
	 toolbox = kcms_open_toolbox(argv[2]);
	 if (toolbox == NULL)
	    ktest_fatal("Error: failed to open toolbox `%s'\n", argv[2]);

	 if (kcms_get_objects(toolbox, (KCMS_LIBRARY|KCMS_KROUTINE|
					KCMS_XVROUTINE|KCMS_PANE|
					KCMS_SCRIPT)) != NULL)
	    ktest_fatal("Error: kcms_get_objects(toolbox, <all-objects>) "
			"returned non-NULL!\n");

	 if (!kcms_close(toolbox))
	    ktest_fatal("Error: kcms_close() returned FALSE after "
			"call to kcms_get_objects(toolbox, <all-objects>)\n");

	 kprintf("kcms_get_objects() case 3 OK\n");
	 break;


      /*----------------------------------------------------------------
      | bomb out with an error message otherwise
      ----------------------------------------------------------------*/
      default:
	 ktest_fatal("incorrect test id (%s) -- "
		     "should be in the range 1 .. 3\n",
		     argv[1]);
	 break;
   }

   kexit(KEXIT_SUCCESS);
}

