/*
== Toolbox:	$BOOTSTRAP
== Object:	kcms
== Testsuite:	27.kcms
== Testing:	unsigned long kcms_attr_int2string(
==				kcms_attr	attribute,
==				kstring		string)
== Description:	attribute can be one of:
==			KCMS_CMOBJ_TYPE		type of software object
==			KCMS_CMOBJ_LANGTYPE	language type of object
*/

#include <bootstrap.h>
#include <ktestutils/ktestutils.h>

typedef struct
{
   kstring		label;
   kcms_attr		attribute;
   unsigned long	value;
   kstring		string;
} TestInfo;

static TestInfo test_cases[] =
{
   {"Software object type: none",
	KCMS_CMOBJ_TYPE, KCMS_NONE, "unknown"},
   {"Software object type: kroutine",
	KCMS_CMOBJ_TYPE, KCMS_KROUTINE,  "kroutine"},
   {"Software object type: xvroutine",
	KCMS_CMOBJ_TYPE, KCMS_XVROUTINE, "xvroutine"},
   {"Software object type: example",
	KCMS_CMOBJ_TYPE, KCMS_EXAMPLE,   "example"},
   {"Software object type: xexample",
	KCMS_CMOBJ_TYPE, KCMS_XEXAMPLE,  "xexample"},
   {"Software object type: testsuite",
	KCMS_CMOBJ_TYPE, KCMS_TESTSUITE, "testsuite"},
   {"Software object type: library",
	KCMS_CMOBJ_TYPE, KCMS_LIBRARY,   "library"},
   {"Software object type: script",
	KCMS_CMOBJ_TYPE, KCMS_SCRIPT,    "script"},
   {"Software object type: pane",
	KCMS_CMOBJ_TYPE, KCMS_PANE,      "pane"},
   {"Software object type: manual",
	KCMS_CMOBJ_TYPE, KCMS_MANUAL,    "manual"},
   {"Software object type: mindex",
	KCMS_CMOBJ_TYPE, KCMS_MINDEX,    "mindex"},
   {"Software object type: mglossary",
	KCMS_CMOBJ_TYPE, KCMS_MGLOSSARY, "mglossary"},
   {"Software object type: dir",
	KCMS_CMOBJ_TYPE, KCMS_DIR,       "dir"},
   {"Software object type: mdir",
	KCMS_CMOBJ_TYPE, KCMS_MDIR,      "mdir"},

   {"Language type: C",
	KCMS_CMOBJ_LANGTYPE, KCMS_LANG_C, "C"},
   {"Language type: fortran",
	KCMS_CMOBJ_LANGTYPE, KCMS_LANG_FORTRAN, "fortran"},
   {"Language type: perl",
	KCMS_CMOBJ_LANGTYPE, KCMS_LANG_PERL,    "perl"},
   {"Language type: csh",
	KCMS_CMOBJ_LANGTYPE, KCMS_LANG_CSH,    "csh"},
   {"Language type: sh",
	KCMS_CMOBJ_LANGTYPE, KCMS_LANG_SH,    "sh"},
   {"Language type: ksh",
	KCMS_CMOBJ_LANGTYPE, KCMS_LANG_KSH,    "ksh"},
   {"Language type: C++",
	KCMS_CMOBJ_LANGTYPE, KCMS_LANG_CPLUSPLUS,    "C++"},
   {"Language type: ada",
	KCMS_CMOBJ_LANGTYPE, KCMS_LANG_ADA,    "ada"},
   {"Language type: roff",
	KCMS_CMOBJ_LANGTYPE, KCMS_LANG_ROFF,    "roff"},
   {"Language type: tex",
	KCMS_CMOBJ_LANGTYPE, KCMS_LANG_TEX,    "tex"},
   {"Language type: uis",
	KCMS_CMOBJ_LANGTYPE, KCMS_LANG_KHOROS_UIS,    "uis"},
   {"Language type: html",
	KCMS_CMOBJ_LANGTYPE, KCMS_LANG_HTML,    "html"},
};

main(
   int    argc,
   char  *argv[])
{
   unsigned long  result;
   int        	  i;


   /*-- working directory is passed as first argument ----------------*/
   if (argc != 1)
   {
      kprintf("incorrect number of command-line arguments (%d)\n", argc);
      kexit(KEXIT_FAILURE);
   }

   ktest_begin("27.kcms", "kcms_attr_string2int()");

   /*-----------------------------------------------------------------*/
   /*-----------------------------------------------------------------*/
   for (i = 0; i < knumber(test_cases); i++)
   {
      ktest_case(test_cases[i].label);
      result = kcms_attr_string2int(test_cases[i].attribute,
				     test_cases[i].string);
      ktest_result((result == test_cases[i].value) ? KTEST_PASS : KTEST_FAIL);
   }
   /*-----------------------------------------------------------------*/
   /*-----------------------------------------------------------------*/

   ktest_end();
}
