#include <bootstrap.h>
#include <float.h>

#define zerosky(a) \
  ((a > 0.0 - FLT_EPSILON && a < 0.0 + FLT_EPSILON) ? +0.0 : a)

#define cbi_test(funcname) \
   kprintf("testing %s ...\n",STRINGARG(funcname)); \
   for (s=-1; s<=+1; s++) \
     { \
       for (t=-1; t<=+1; t++) \
         { \
           a.r = s; a.i = t; \
           b = funcname(a); \
           kprintf("In: (% 4.4f,% 4.4f) " \
                   "Out: (% 4.4f,% 4.4f)\n", \
                   a.r, a.i, zerosky(b.r), zerosky(b.i)); \
         } \
     }

#define cbi_test_real(funcname) \
   kprintf("testing %s ...\n",STRINGARG(funcname)); \
   for (s=-1; s<=+1; s++) \
     { \
       for (t=-1; t<=+1; t++) \
         { \
           a.r = s; a.i = t; \
           b.r = funcname(a); \
           kprintf("In: (% 4.4f,% 4.4f) " \
                   "Out: % 4.4f\n", \
                   a.r, a.i, zerosky(b.r)); \
         } \
     }

#define dcbi_test(funcname) \
   kprintf("testing %s ...\n",STRINGARG(funcname)); \
   for (s=-1; s<=+1; s++) \
     { \
       for (t=-1; t<=+1; t++) \
         { \
           da.r = s; da.i = t; \
           db = funcname(da); \
           kprintf("In: (% 4.4f,% 4.4f) " \
                   "Out: (% 4.4f,% 4.4f)\n", \
                   da.r, da.i, zerosky(db.r), zerosky(db.i)); \
         } \
     }

#define dcbi_test_real(funcname) \
   kprintf("testing %s ...\n",STRINGARG(funcname)); \
   for (s=-1; s<=+1; s++) \
     { \
       for (t=-1; t<=+1; t++) \
         { \
           da.r = s; da.i = t; \
           db.r = funcname(da); \
           kprintf("In: (% 4.4f,% 4.4f) " \
                   "Out: % 4.4f\n", \
                   da.r, da.i, zerosky(db.r)); \
         } \
     }

#define cquad_test(funcname) \
   kprintf("testing %s ...\n",STRINGARG(funcname)); \
   for (s=-1; s<=+1; s++) \
     { \
       for (t=-1; t<=+1; t++) \
         { \
           for (u=-1; u<=+1; u++) \
             { \
               for (v=-1; v<=+1; v++) \
                 { \
                   a.r = s; a.i = t; \
                   b.r = u; b.i = v; \
                   c = funcname(a,b); \
                   kprintf("In: (% 4.4f,% 4.4f) (% 4.4f,% 4.4f) " \
                           "Out: (% 4.4f,% 4.4f)\n", \
                            a.r, a.i, b.r, b.i, zerosky(c.r), zerosky(c.i)); \
                 } \
             } \
         } \
     }

#define cquad_test_div(funcname) \
   kprintf("testing %s ...\n",STRINGARG(funcname)); \
   for (s=-1; s<=+1; s++) \
     { \
       for (t=-1; t<=+1; t++) \
         { \
           for (u=-1; u<=+1; u++) \
             { \
               for (v=-1; v<=+1; v++) \
                 { \
                   if (u == 0.0 && v == 0.0) break; \
                   a.r = s; a.i = t; \
                   b.r = u; b.i = v; \
                   c = funcname(a,b); \
                   kprintf("In: (% 4.4f,% 4.4f) (% 4.4f,% 4.4f) " \
                           "Out: (% 4.4f,% 4.4f)\n", \
                            a.r, a.i, b.r, b.i, zerosky(c.r), zerosky(c.i)); \
                 } \
             } \
         } \
     }

#define dcquad_test(funcname) \
   kprintf("testing %s ...\n",STRINGARG(funcname)); \
   for (s=-1; s<=+1; s++) \
     { \
       for (t=-1; t<=+1; t++) \
         { \
           for (u=-1; u<=+1; u++) \
             { \
               for (v=-1; v<=+1; v++) \
                 { \
                   da.r = s; da.i = t; \
                   db.r = u; db.i = v; \
                   dc = funcname(da,db); \
                   kprintf("In: (% 4.4f,% 4.4f) (% 4.4f,% 4.4f) " \
                           "Out: (% 4.4f,% 4.4f)\n", \
                            da.r, da.i, db.r, db.i, zerosky(dc.r), zerosky(dc.i)); \
                 } \
             } \
         } \
     }

#define dcquad_test_div(funcname) \
   kprintf("testing %s ...\n",STRINGARG(funcname)); \
   for (s=-1; s<=+1; s++) \
     { \
       for (t=-1; t<=+1; t++) \
         { \
           for (u=-1; u<=+1; u++) \
             { \
               for (v=-1; v<=+1; v++) \
                 { \
                   if (u == 0.0 && v == 0.0) break; \
                   da.r = s; da.i = t; \
                   db.r = u; db.i = v; \
                   dc = funcname(da,db); \
                   kprintf("In: (% 4.4f,% 4.4f) (% 4.4f,% 4.4f) " \
                           "Out: (% 4.4f,% 4.4f)\n", \
                            da.r, da.i, db.r, db.i, zerosky(dc.r), zerosky(dc.i)); \
                 } \
             } \
         } \
     }

main()
{
   kcomplex a,b,c;
   kdcomplex da,db,dc;
   float s,t,u,v;

   cquad_test(kcadd);
   cquad_test(kcsub);
   cquad_test(kcmult);
   cquad_test_div(kcdiv);

   dcquad_test(kdcadd);
   dcquad_test(kdcsub);
   dcquad_test(kdcmult);
   dcquad_test_div(kdcdiv);

   cbi_test_real(kcmag);
   cbi_test_real(kcmagsq);
   cbi_test_real(kcang);
   cbi_test(kcconj);

   dcbi_test_real(kdcmag);
   dcbi_test_real(kdcmagsq);
   dcbi_test_real(kdcang);
   dcbi_test(kdcconj);

   kexit(KEXIT_SUCCESS);

  }
