#include "bootstrap.h"
 
int read_list(
   int id,
   kaddr buf,
   int type)
{
	klist *entry = (klist *) buf;

	if (kread_string(id, &entry->identifier, (size_t) 1) != 1)
	   return(FALSE);
	if (kread_string(id, &entry->client_data, (size_t) 1) != 1)
	   return(FALSE);
	if (kread_ubyte(id, &entry->head, (size_t) 1) != 1)
	   return(FALSE);

	return(TRUE);
}

int write_list(
   int id,
   kaddr buf,
   int type)
{
	klist *entry = (klist *) buf;

	if (kwrite_string(id, &entry->identifier, (size_t) 1) != 1)
	   return(FALSE);
	if (kwrite_string(id, &entry->client_data, (size_t) 1) != 1)
	   return(FALSE);
	if (kwrite_ubyte(id, &entry->head, (size_t) 1) != 1)
	   return(FALSE);

	return(TRUE);
}

void free_list(
   kaddr buf,
   int type,
   int freeself)
{
        klist *entry = (klist *) buf;
 
        kstruct_free(entry->identifier, KNONE, TRUE);
        kstruct_free(entry->client_data, KNONE, TRUE);
	if (freeself)
	   kstruct_free(entry, KNONE, TRUE);
}

void main(
   int  argc,
   char *argv[])
{
	kfile *file;
	int   i, type, status;
	ssize_t num;
	klist *list = NULL, *tmp, *next;
	char  *word1, *word2, **words, *path = "Makefile";


	if (argc > 1)
	   path = argv[1];

	if ((file = kfopen(path, "r")) == NULL)
	{
	   kprintf("Unable to open file '%s'", path);
	   kexit(KEXIT_FAILURE);
	}

	if ((words = kparse_file_scan_delimit(file, KPARSE_BOF, KPARSE_EOF,
                  KLITERAL, " \t\n", NULL, NULL, &num, &status)) == NULL)
        {
           kprintf("Cannot read contents of file '%s'\n", path);
           kexit(KEXIT_FAILURE);
        }
	kfclose(file);

	kfprintf(kstderr, "%s has %ld words\n", path, (long) num);
	for (i = 0; i < num; i++)
	   list = klist_insert(list, kstrdup(words[i]), NULL, KLIST_HEAD, TRUE);

	karray_free(words, KSTRING, (size_t) num, NULL);
	type = kstruct_define("klist", sizeof(klist), read_list,
		   write_list, NULL, free_list);

	while (list != NULL)
	{
	   tmp = kstruct_duplicate(list, type, NULL);
	   word1 = (char *) klist_identifier(list);
	   word2 = (char *) klist_identifier(tmp);
	   if (kstrcmp(word1, word2) != 0 && word1 != word2)
	   {
	      kfprintf(kstderr, "Error! lists are not identical\n");
	      kexit(KEXIT_FAILURE);
	   }
	   next = klist_next(list);
	   kstruct_free(tmp, type, TRUE);
	   kstruct_free(list, type, TRUE);
	   list = next;
	}
	kfprintf(kstderr, "Success! lists are identical\n");
	kexit(KEXIT_SUCCESS);
}
