#include "bootstrap.h"
 
int read_list(
   int id,
   kaddr buf,
   int type)
{
	klist *entry = (klist *) buf;

	if (kread_string(id, &entry->identifier, (size_t) 1) != 1)
	   return(FALSE);
	if (kread_string(id, &entry->client_data, (size_t) 1) != 1)
	   return(FALSE);
	if (kread_ubyte(id, &entry->head, (size_t) 1) != 1)
	   return(FALSE);
	if (kread_pointer(id, (kaddr *) &entry->next, type) != 1)
	   return(FALSE);
	if (kread_pointer(id, (kaddr *) &entry->prev, type) != 1)
	   return(FALSE);

	return(TRUE);
}

int write_list(
   int id,
   kaddr buf,
   int type)
{
	klist *entry = (klist *) buf;

	if (kwrite_string(id, &entry->identifier, (size_t) 1) != 1)
	   return(FALSE);
	if (kwrite_string(id, &entry->client_data, (size_t) 1) != 1)
	   return(FALSE);
	if (kwrite_ubyte(id, &entry->head, (size_t) 1) != 1)
	   return(FALSE);
	if (kwrite_pointer(id, (kaddr *) &entry->next, type) != 1)
	   return(FALSE);
	if (kwrite_pointer(id, (kaddr *) &entry->prev, type) != 1)
	   return(FALSE);

	return(TRUE);
}

void free_list(
   kaddr buf,
   int type,
   int freeself)
{
	klist *entry = (klist *) buf;

	kstruct_free(entry->identifier, KNONE, TRUE);
	kstruct_free(entry->client_data, KNONE, TRUE);
	kstruct_free(entry->next, type, TRUE);
	kstruct_free(entry->prev, type, TRUE);
	if (freeself)
	   kstruct_free(entry, KNONE, TRUE);
}

void main(
   int  argc,
   char *argv[])
{
	kfile *file;
	int   i, type, status;
	ssize_t num;
	klist *tmp1, *tmp2, *list1 = NULL, *list2 = NULL;
	char  *word1, *word2, **words, *path = "Makefile";


	if (argc > 1)
	   path = argv[1];

	if ((file = kfopen(path, "r")) == NULL)
	{
	   kprintf("Unable to open file '%s'", path);
	   kexit(KEXIT_FAILURE);
	}

	if ((words = kparse_file_scan_delimit(file, KPARSE_BOF, KPARSE_EOF,
                  KLITERAL, " \t\n", NULL, NULL, &num, &status)) == NULL)
        {
           kprintf("Cannot read contents of file '%s'\n", path);
           kexit(KEXIT_FAILURE);
        }
	kfclose(file);

	kfprintf(kstderr, "%s has %ld words\n", path, (long) num);
	for (i = 0; i < num; i++)
	   list1 = klist_insert(list1, kstrdup(words[i]), NULL,KLIST_HEAD,TRUE);

	karray_free(words, KSTRING, (size_t) num, NULL);
	type = kstruct_define("klist", sizeof(klist), read_list,
		   write_list, NULL, free_list);

	list2 = kstruct_duplicate(list1, type, NULL);
	tmp1 = list1, tmp2 = list2;
	do
	{
	   word1 = (char *) klist_identifier(tmp1);
	   word2 = (char *) klist_identifier(tmp2);
	   if (kstrcmp(word1, word2) != 0 && word1 != word2)
	   {
	      kfprintf(kstderr, "Error! lists are not identical\n");
	      kexit(KEXIT_FAILURE);
	   }
	   tmp1 = klist_next(tmp1);
	   tmp2 = klist_next(tmp2);
	}
	while (klist_next(tmp1) != NULL && klist_next(tmp2) != NULL);

	do
	{
	   word1 = (char *) klist_identifier(tmp1);
	   word2 = (char *) klist_identifier(tmp2);
	   if (kstrcmp(word1, word2) != 0 && word1 != word2)
	   {
	      kfprintf(kstderr, "Error! lists are not identical\n");
	      kexit(KEXIT_FAILURE);
	   }
	   tmp1 = klist_prev(tmp1);
	   tmp2 = klist_prev(tmp2);
	}
	while (klist_prev(tmp1) != NULL && klist_next(tmp2) != NULL);

	if (tmp1 != list1 || tmp2 != list2)
	{
	   kfprintf(kstderr, "Error! lists are not identical\n");
	   kexit(KEXIT_FAILURE);
	}
	kstruct_free(list1, type, TRUE);
	kstruct_free(list2, type, TRUE);
	kfprintf(kstderr, "Success! lists are identical\n");
	kexit(KEXIT_SUCCESS);
}
