#include <bootstrap.h>
#include "io_test.h"

/*-- write out a strucure to a file --*/
void write_structure(char *name, kaddr s, int type)
{
   int fid = kopen(name, O_WRONLY|O_CREAT|O_TRUNC, 0644);
   kwrite_generic(fid, s, (size_t) 1, type);
   kclose(fid);
}

/*-- read in a strucure from a file --*/
void read_structure(char *name, kaddr s, int type)
{
   int fid = kopen(name, O_RDONLY, 0644);
   kread_generic(fid, s, (size_t) 1, type);
   kclose(fid);
}

/*-- simple routines --*/

pixel_p create_pixel(color c, int v)
{
   pixel_p p;
   
   p = (pixel_p) kmalloc(sizeof(struct pixel));
   p->a = c;

   p->cp = (color_p) kmalloc(sizeof(color));
   p->cs = (color_p) kmalloc(sizeof(color));

   *p->cp = c;
   *p->cs = c;

   p->value = v;

   return p;
}

int compare_pixel(pixel_p p1, pixel_p p2)
{
   if (p1 == NULL && p2 == NULL)
      return TRUE;

   if (p1 == NULL || p2 == NULL)
      return FALSE;

   if (p1->a != p2->a)
      return FALSE;

   if (p1->cp != NULL && p2->cp != NULL)
   {
      if (*p1->cp != *p2->cp)
	 return FALSE;
   }
   else if (p1->cp != NULL || p2->cp != NULL)
      return FALSE;

   if (p1->cs != NULL && p2->cs != NULL)
   {
      if (*p1->cs != *p2->cs)
	 return FALSE;
   }
   else if (p1->cs != NULL || p2->cs != NULL)
      return FALSE;

   if (p1->value != p2->value)
      return FALSE;

   return TRUE;
}

void
main()
{
   pixel_p o_p;
   pixel_p i_p;

   /*-- this should recursively define everything we need --*/
   kdefine_pixel_p();

   /*-- initialize pixel --*/
   o_p = (pixel_p) create_pixel(RED, 7);
   i_p = (pixel_p) kmalloc(sizeof(struct pixel));

   write_structure("./t.data", (kaddr) o_p, KTYPE_PIXEL);
   read_structure("./t.data", (kaddr) i_p, KTYPE_PIXEL);

   if (compare_pixel(o_p, i_p))
      kprintf("pixel is identical!\n");
   else
      kprintf("pixel is different!\n");

   if (!kstruct_compare(o_p, i_p, KTYPE_PIXEL_P))
      kprintf("flaky compare for pixel_p!\n");

   /*-- tidy up --*/
   kstruct_free(o_p, KTYPE_PIXEL_P, TRUE);
   kstruct_free(i_p, KTYPE_PIXEL_P, TRUE);

   kexit(KEXIT_SUCCESS);
}
