 /*
  * Khoros: $Id: klinearop.c,v 1.13 1996/07/05 02:03:44 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: klinearop.c,v 1.13 1996/07/05 02:03:44 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for klinearop
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "klinearop.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Performs linear operations (convolution/correlation)
|
|       Purpose: main program for klinearop
|
|         Input:
|		char *clui_info->i1_file; {Input object }
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->i2_file; {Input kernel}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		char *clui_info->o_file; {output object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		Required M.E. group:
|		    M.I. group:
|			int clui_info->l_flag; {TRUE if -l specified}
|
|			double clui_info->real_double; {Real part of pad constant}
|			int    clui_info->real_flag; {TRUE if -real specified}
|
|			double clui_info->imag_double; {Imaginary part of pad constant}
|			int    clui_info->imag_flag; {TRUE if -imag specified}
|
|		    int clui_info->c_flag; {TRUE if -c specified}
|
|		int clui_info->corr_logic; {Select correlation (default) or convolution}
|		int clui_info->corr_flag; {TRUE if -corr specified}
|
|		int clui_info->upcast_flag; {TRUE if -upcast specified}
|
|		int clui_info->wc_int; {Width coordinate of kernel hotspot}
|		int clui_info->wc_flag; {TRUE if -wc specified}
|
|		int clui_info->hc_int; {Height coordinate of kernel hotspot}
|		int clui_info->hc_flag; {TRUE if -hc specified}
|
|		int clui_info->dc_int; {Depth coordinate of kernel hotspot}
|		int clui_info->dc_flag; {TRUE if -dc specified}
|
|		int clui_info->tc_int; {Time coordinate of kernel hotspot}
|		int clui_info->tc_flag; {TRUE if -tc specified}
|
|		int clui_info->ec_int; {Element coordinate of kernel hotspot}
|		int clui_info->ec_flag; {TRUE if -ec specified}
|
|		int clui_info->woff_int; {Width offset for subregion}
|		int clui_info->woff_flag; {TRUE if -woff specified}
|
|		int clui_info->hsize_int; {Height size for subregion}
|		int clui_info->hsize_flag; {TRUE if -hsize specified}
|
|		int clui_info->hoff_int; {Height offset for subregion}
|		int clui_info->hoff_flag; {TRUE if -hoff specified}
|
|		int clui_info->wsize_int; {Width size for subregion}
|		int clui_info->wsize_flag; {TRUE if -wsize specified}
|
|		int clui_info->doff_int; {Depth offset for subregion}
|		int clui_info->doff_flag; {TRUE if -doff specified}
|
|		int clui_info->dsize_int; {Depth size for subregion}
|		int clui_info->dsize_flag; {TRUE if -dsize specified}
|
|		int clui_info->toff_int; {Time offset for subregion}
|		int clui_info->toff_flag; {TRUE if -toff specified}
|
|		int clui_info->tsize_int; {Time size for subregion}
|		int clui_info->tsize_flag; {TRUE if -tsize specified}
|
|		int clui_info->eoff_int; {Elements offset for subregion}
|		int clui_info->eoff_flag; {TRUE if -eoff specified}
|
|		int clui_info->esize_int; {Elements size for subregion}
|		int clui_info->esize_flag; {TRUE if -esize specified}
|
|        Output:
|       Returns:
|
|    Written By: Jonio Roberto de Hollanda Cavalcanti, Ramiro Jordan
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
        kobject src = NULL;	/* source object */
        kobject kernel = NULL;	/* kernel object */
        int *offset = NULL;	/* indicates new kernel offset values */
        int filtering_type = 0; /* linear or circular */
        float real_pad_value;   /* real value for padding object */
        float img_pad_value;    /* imaginary value for padding object */
        int  *sub_size;
        int  *sub_pos;
        char *prog = "main";    /* main/library name (for error reporting) */
        char *rtn = "klinearop";  /* routine name (for error reporting) */
        int w,h,d,t,e;
        int upcast;
        kobject dest=NULL;           /* destination object */
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "DATAMANIP");
	kexit_handler(klinearop_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "DATAMANIP", "klinearop",
		klinearop_usage_additions, klinearop_get_args,
		klinearop_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        /*
         * Open the source, kernel and destination data objects using the
         * kpds_open_input_object and kpds_open_output_object calls.
         * These calls will open the source, kernel and
         * destination data objects with the flags set correctly.  If an open
         * fails, report the error using kerror, close opened objects if
         * necessary, and exit the program using the kexit call.
         */
 
        if ((src = kpds_open_input_object(clui_info ->i1_file) ) 
		== KOBJECT_INVALID)
        {
            kerror( rtn, prog, "Cannot open source object.");
            kexit(KEXIT_FAILURE);
        }
 
        if ((kernel = kpds_open_input_object(clui_info ->i2_file) )
		== KOBJECT_INVALID)
        {
            kerror( rtn, prog, "Cannot open kernel object.");
            kpds_close_object(src);
            kexit(KEXIT_FAILURE);
        }

        if ((dest = kpds_open_output_object(clui_info ->o_file) ) 
		 == KOBJECT_INVALID)
        {
            kerror( rtn, prog, "Cannot open destination object.");
            kpds_close_object(src);
            kexit(KEXIT_FAILURE);
        }

        /*
         * The lklinearop library should have no side effects on the
         * destination object.  It will only manipulate the value data,
         * the mask, if it exists, and the attributes of the destination
         * data object that are related to the filtering operation.
         *
         * Since the source object may contain additional attributes
         * that may or may not be part of the data model,
         * the source object needs to be copied to the destination object
         * so that the user is not surprised by unwanted side effects.
         * Example: removal of map or location data.
         *
         * NOTE: The user can use the kpds_copy_object_data and
         * kpds_copy_object_attr functions to copy data and attributes
         * between source and destination objects separatedly.
         *
         * If the copy function fails, report the error, close the source
         * and destination objects, and exit.
         */
 
        if (!kpds_copy_object_attr(src, dest))
        {
            kerror( rtn, prog, "kpds_copy_object failed.");
            kpds_close_object(src);
            kpds_close_object(dest);
            kexit(KEXIT_FAILURE);
        }

        /*
        * Assign variables.  The clui_info structure is created by
        * the code generators using the user interface specification file
        * (kfilter.pane), and this structure is located in the kfilter.h
        * file.
        *
        * Below, we have assigned the structure variables to local
        * variables.  In this example, this step is not required, since
        * nothing is done to the variables before they are passed to the
        * library lklinearop.
        *
        * The filtering_type variable receives the user selection on the
        * method to perform the filtering operation.  There are two options:
        * 1) linear (default); padding is required (default values for
        *    the real and imaginary parts are 0)
        * 2) circular; no padding is required 
	*
        * In addition, if new parameters for the origin ("sweet spot") of
        * the kernel are passed by the user, use new values for origin. 
        * Default values are read from the kernel object.
	*                     wval (width)
        *                     hval (height) 
	*                     dval (depth)
        *                     tval (time) 
        *                     eval (elements) 
        *
        * Reflection process is determined.  Two options:
	* 1) Flip kernel (eg. convolution) is default.
	* 2) No flip (eg. correlation)
        *
        * Finally set up the filtering directions. There are five
        * directions available.
        *
        *      w - width.
        *      h - height.
        *      d - depth.
        *      t - time.
        *      e - element.
        * 
        * If the number of directions is smaller than the kernel dimension
        * print error message.
        */

	real_pad_value = 0.0;
	img_pad_value = 0.0;
        if (clui_info->l_flag == 1) {
	   filtering_type = 0;
	   real_pad_value = clui_info->real_double;
	   img_pad_value = clui_info->imag_double;
        }
        else if (clui_info->c_flag == 1)
             filtering_type = 1;

        if (!kpds_get_attribute(kernel, KPDS_VALUE_SIZE,&w,&h,&d,&t,&e)) {
            kerror( rtn, prog, "kpds_get_attribute failed for kernel.");
            kexit(KEXIT_FAILURE);
        }

       /*
	* Set the default sweet spot for the kernel. 
        * It will be obtained from the KPDS_KERNEL_ORIGIN attribute
        * of the kernel in the lklinearop function if there is no 
        * sweet spot selection from the pane. Any sweet spot location
        * not explictily given is left at zero.
        */	
        if (clui_info->wc_flag || clui_info->hc_flag || clui_info->dc_flag ||
            clui_info->tc_flag || clui_info->ec_flag)
        {
         offset = (int *) kmalloc((size_t) 5 * sizeof(int));
	 if (offset == NULL){
             kerror( rtn, prog, "Cannot allocate variable offset");
             kexit(KEXIT_FAILURE);
	 }
         offset[0] = offset[1] = offset[2] =  0;
         offset[3] = offset[4] = 0;
	 if (clui_info->wc_flag ) offset[0]=clui_info->wc_int;
	 if (clui_info->hc_flag ) offset[1]=clui_info->hc_int;
	 if (clui_info->dc_flag ) offset[2]=clui_info->dc_int;
	 if (clui_info->tc_flag ) offset[3]=clui_info->tc_int;
	 if (clui_info->ec_flag ) offset[4]=clui_info->ec_int;
        }

       /*
        * Select sub-region size for processing. Default is the whole object.
        */
        sub_size= (int *) kmalloc((size_t)5 * sizeof(int));
	if (sub_size== NULL){
            kerror( rtn, prog, "Cannot allocate sub_size.");
            kexit(KEXIT_FAILURE);
	}
	if (!kpds_get_attribute(src, KPDS_VALUE_SIZE,&sub_size[0],&sub_size[1],
                                     &sub_size[2],&sub_size[3],&sub_size[4])) {
            kerror( rtn, prog, "kpds_get_attribute failed for src.");
            kexit(KEXIT_FAILURE);
	}

        /* Override the default subregion size if the pane parameters indicate
           that the user has supplied specific values */
        if (clui_info->wsize_flag) sub_size[0] = clui_info->wsize_int;
        if (clui_info->hsize_flag) sub_size[1] = clui_info->hsize_int; 
        if (clui_info->dsize_flag) sub_size[2] = clui_info->dsize_int;
        if (clui_info->tsize_flag) sub_size[3] = clui_info->tsize_int;
        if (clui_info->esize_flag) sub_size[4] = clui_info->esize_int;

       /*
        * Select subregion position for processing. 
        * Default is the top left corner.
        */
        sub_pos= (int *) kmalloc((size_t)5 * sizeof(int));
	if (sub_pos== NULL){
            kerror( rtn, prog, "Cannot allocate sub_pos.");
            kexit(KEXIT_FAILURE);
	}
        sub_pos[0]=sub_pos[1]=0;
        sub_pos[2]=sub_pos[3]=sub_pos[4]=0;
        if (clui_info->woff_flag) sub_pos[0] = clui_info->woff_int;
        if (clui_info->hoff_flag) sub_pos[1] = clui_info->hoff_int; 
        if (clui_info->doff_flag) sub_pos[2] = clui_info->doff_int;
        if (clui_info->toff_flag) sub_pos[3] = clui_info->toff_int;
        if (clui_info->eoff_flag) sub_pos[4] = clui_info->eoff_int;

       /*
        * Set the upcast option. If this option is set true
        * the data type of the kernel and the source/destination
        * will set to DOUBLE.
        */
        upcast = clui_info->upcast_flag;
/* -main_before_lib_call_end */

/* -main_library_call */
        /*
         * Once all of the preliminary work has been done, pass the source,
         * kernel, and destination objects.  Also, pass  other parameters to
         * the library.
         * After execution of lklinearop close the objects and exit the
         * kfilter routine.
         */
          if (!lklinearop(src, kernel, filtering_type,
                        real_pad_value, img_pad_value, offset,
                        !(clui_info->corr_logic),
                        sub_size, sub_pos, upcast, dest))
          {
           kpds_close_object(src);
           kpds_close_object(kernel);
           kpds_close_object(dest);
	   kerror(rtn, prog, "lklinearop failed.");
           kexit(KEXIT_FAILURE);
          }
/* -main_library_call_end */

/* -main_after_lib_call */
        /* add history to the output object */
        if (!kpds_set_attribute(dest, KPDS_HISTORY, kpds_history_string()))
        {
           kerror("klinearop", "main",
                  "Unable to set history on the destination object");
            kexit(KEXIT_FAILURE);
        }
 
       /*
        * If the library call succeeded, close the source, kernel and
        * destination data objects using kpds_close_object and exit
        * using kexit.
        */
        kpds_close_object(src);
        kpds_close_object(kernel);
        kpds_close_object(dest);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: klinearop_usage_additions
| 
|       Purpose: Prints usage additions in klinearop_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname klinearop
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void klinearop_usage_additions(void)
{
	kfprintf(kstderr, "\tPerforms linear operations (convolution/correlation)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: klinearop_free_args
| 
|       Purpose: Frees CLUI struct allocated in klinearop_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname klinearop
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
klinearop_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
