 /*
  * Khoros: $Id: lkarith1.c,v 1.12 1996/07/05 02:10:38 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lkarith1.c,v 1.12 1996/07/05 02:10:38 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for karith1
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lkarith1
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/*
 * Function structure declaration
 */
struct _function
{
        char       *name;
	kfunc_void *routine;
};

/*
 * Prototypes for local function wrappers. This is necessary to deal
 * with the fact that each may be either an actual function or a macro, thus
 * making it impossible to obtain a function pointer. Wrapping them in a real
 * function makes this possible.
 */
static double ka1_kfabs(double);
static double ka1_ksqrt(double);
static double ka1_kcbrt(double);
static double ka1_kneg(double);
static double ka1_krecip(double);
static unsigned long ka1_knot(unsigned long);
static unsigned long ka1_kclear(unsigned long);
static unsigned long ka1_kset(unsigned long);
static double ka1_ksin(double);
static double ka1_kasin(double);
static double ka1_kcos(double);
static double ka1_kacos(double);
static double ka1_ktan(double);
static double ka1_katan(double);
static double ka1_ksinh(double);
static double ka1_kcosh(double);
static double ka1_ktanh(double);
static double ka1_kasinh(double);
static double ka1_kacosh(double);
static double ka1_katanh(double);
static double ka1_ksinc(double);
static double ka1_kceil(double);
static double ka1_kfloor(double);
static double ka1_ktrunc(double);
static double ka1_kfraction(double);
static double ka1_kerf(double);
static double ka1_kerfc(double);
static double ka1_kgamma(double);

/*
 * Function declaration list
 */
static struct _function functions[] =
{
    { "abs",    (kfunc_void *) &ka1_kfabs },
    { "sqrt",   (kfunc_void *) &ka1_ksqrt },
    { "cbrt",   (kfunc_void *) &ka1_kcbrt },
    { "neg",    (kfunc_void *) &ka1_kneg },
    { "recip",  (kfunc_void *) &ka1_krecip },
    { "not",    (kfunc_void *) &ka1_knot },     /* Will convert to long later */
    { "clear",  (kfunc_void *) &ka1_kclear },   /* Will convert to long later */
    { "set",    (kfunc_void *) &ka1_kset },     /* Will convert to long later */
    { "sin",    (kfunc_void *) &ka1_ksin  },
    { "asin",   (kfunc_void *) &ka1_kasin },
    { "cos",    (kfunc_void *) &ka1_kcos  },
    { "acos",   (kfunc_void *) &ka1_kacos },
    { "tan",    (kfunc_void *) &ka1_ktan  },
    { "atan",   (kfunc_void *) &ka1_katan },
    { "sinh",   (kfunc_void *) &ka1_ksinh },
    { "cosh",   (kfunc_void *) &ka1_kcosh },
    { "tanh",   (kfunc_void *) &ka1_ktanh },
    { "asinh",  (kfunc_void *) &ka1_kasinh },
    { "acosh",  (kfunc_void *) &ka1_kacosh },
    { "atanh",  (kfunc_void *) &ka1_katanh },
    { "sinc",   (kfunc_void *) &ka1_ksinc },
    { "ceil",   (kfunc_void *) &ka1_kceil },
    { "floor",  (kfunc_void *) &ka1_kfloor },
    { "trunc",  (kfunc_void *) &ka1_ktrunc },
    { "fract",  (kfunc_void *) &ka1_kfraction },
    { "trunc",  (kfunc_void *) &ka1_ktrunc },
    { "erf",    (kfunc_void *) &ka1_kerf },
    { "erfc",   (kfunc_void *) &ka1_kerfc },
    { "lgamma",  (kfunc_void *) &ka1_kgamma },
};
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lkarith1 - perform single operand arithmetic on data object
* 
*       Purpose: The following is the library routine that performs
*		 a unary function on a data object.  The routine takes
*		 an input data object, a function name, and an output
*		 data object in which to store the result of the
*		 function.
*
*		 The supported functions are:
*                .TS
*                center tab(:) ;
*                l l .
*		 abs   :  Output is Absolute Value of input
*		 sqrt  :  Output is Square Root of input
*		 cbrt  :  Output is Cube Root of input
*		 neg   :  Output is Negative of input
*		 recip :  Output is Reciprocal of input
*		 not   :  Bitwise NOT operation
*		 clear :  Bitwise CLEAR - set all valid bits to 0
*		 set   :  Bitwise SET - set all valid bits to 1
*		 sin   :  Output is the sine() of the input
*		 asin  :  Output is the arc sine() of the input
*		 cos   :  Output is the cosine() of the input
*		 acos  :  Output is arc cosine() of the input
*		 tan   :  Output is tangent() of the input
*		 atan  :  Output is arc tangent() of the input
*		 sinh  :  Output is hyperbolic sine() of the input
*		 asinh :  Output is hyperbolic arc sine() of the input
*		 cosh  :  Output is hyperbolic cosine() of the input
*		 acosh :  Output is arc hyperbolic cosine() of the input
*		 tanh  :  Output is hyperbolic tangent() of the input
*		 atanh :  Output is hyperbolic tangent() of the input
*		 sinc  :  Output is sin(x)/x of the input
*		 ceil  :  Output is ceiling of the input
*		 floor :  Output is floor of the input
*		 trunc :  Output is interger truncate of the input
*		 fract :  Output is fractional part of the input
*		 lgamma:  Output is the lgamma function of the input
*		 erfc  :  Output is 1.0-erf(x)
*		 erf   :  Output is error function of x where 
*		       :  erf(x)=2/sqrt(pi)*integralfrom0
*                .TE
*
*		 If lkarith1() were called with:
*
*		 !    lkarith1(i, "sin", o);
*
*		 then the following would be performed:
*
*		 !    o = sin(i);
*
*         Input: src_obj  - the input object to be processed
*		 function   - the function to be applied
*        Output: dst_obj - the output object
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*  Restrictions: 
*    Written By: Scott Wilson, Donna Koechner, Mark Young, Ashish Malhotra
*          Date: Jul 04, 1996
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int lkarith1(
   kobject src_obj,
   char    *function,
   kobject dst_obj)
/* -library_def_end */

/* -library_code */
{
	kfunc_long   lroutine;  /* set, clear, and not functions are long */
	kfunc_double routine;   /* all other functions are double */
	char *lib = "kdatamanip", *rtn = "lkarith1";
	long  num_pts;
        int   num_rgns, w,h,d,t,e,type,in_type;
        int   j, num, error = FALSE;
	int   map_exists, val_exists;
	char *region, *opt_region, *datasize, *datatype, *region_size;
	double *data = NULL;	 	/* array for non-complex data */
	unsigned long *ldata = NULL; 	/* array for long data */
	int   func;			/* function number (converted from 
					   function string variable) */
        klist *objlist=NULL;

/*
	 *  Check which segments (VALUE, MAP) are available for lkarith1.
	 *
	 *  If there is a MAP, operate directly on the map (Don't worry 
	 *  about VALUE data).  
	 *
	 *  If there is no map, operate on the VALUE data.  If operating on 
	 *  VALUE data and there is a MASK, ignore the MASK.  
	 *
	 *  If there is no MAP or VALUE data, fail.
	 */
	map_exists  = kpds_query_map(src_obj);
	val_exists  = kpds_query_value(src_obj);

	if (!map_exists && !val_exists)
	{
	   kerror(lib, rtn,
		  "Input data object must have either Value or Map data.");
	   return(FALSE);
	}

	/* 
	 *  Create a reference to the source data object passed in, so
	 *  that the library (lkarith1) will have no unexpected side
	 *  effects on it (such as changing the position attribute.
	 */
        KCALL((src_obj = kpds_reference_object(src_obj)) != KOBJECT_INVALID);
        objlist = klist_add(objlist,src_obj,"KOBJECT");

	for (num = 0, func = -1; num < knumber(functions) && func == -1; num++)
	{
	   if (kstrcmp(function, functions[num].name) == 0)
	   {
	      func = num;
	      break;
	   }
	}

	if (func == -1)
	{
	   kerror(lib, rtn, "Sorry, but the function '%s' is not in "
                            "the list of callable functions", function);
           (void)klist_free(objlist, (kfunc_void)lkcall_free);
	   return(FALSE);
	}
	else if (functions[func].routine == NULL)
	{
	   kerror(lib, rtn, "Sorry, but karith1 is unable to run "
                            "the function '%s' on data objects at this time.",
                            function);
           (void)klist_free(objlist, (kfunc_void)lkcall_free);
	   return(FALSE);
	}
	else
	{
	   routine = (kfunc_double)functions[func].routine;
	   lroutine = (kfunc_long)functions[func].routine;
	}

/*
	 *  Because operations in karith1 are pointwise, if there
	 *  is a map, we can work on it directly without having to
	 *  map the value data first.  Set val_exists and mask_exists
	 *  to false since they should not be use.
	 */
	if (map_exists)
	{
	   region       = KPDS_MAP_REGION;
	   region_size  = KPDS_MAP_REGION_SIZE;
	   opt_region   = KPDS_MAP_OPTIMAL_REGION_SIZE;
	   datasize     = KPDS_MAP_SIZE;
	   datatype     = KPDS_MAP_DATA_TYPE;
	   val_exists   = FALSE;
	}
	else
	{
	   region       = KPDS_VALUE_REGION;
	   region_size  = KPDS_VALUE_REGION_SIZE;
	   opt_region   = KPDS_VALUE_OPTIMAL_REGION_SIZE;
	   datasize     = KPDS_VALUE_SIZE;
	   datatype     = KPDS_VALUE_DATA_TYPE;
	}

        KCALL(kpds_get_attributes(src_obj, datasize, &w,&h,&d,&t,&e, 
				     datatype, &type, NULL));
        in_type = type;
/*
	 *  Complex processing not supported at this time, so if the 
	 *  data type of the source object is double complex, print
	 *  message and return.
	 */
	if ( (type == KCOMPLEX) || (type == KDCOMPLEX) ) 
	{
	   kerror(lib, rtn, "Sorry, but karith1 operations are not defined "
                            "for complex data at this time.");
           (void)klist_free(objlist, (kfunc_void)lkcall_free);
	   return(FALSE);
	}

/*
	 *  Set the size and datatype of the output data object to be 
	 *  the same as the input data object.
	 */
	if (map_exists)	
	{
	   if (!kpds_query_map(dst_obj))
	   {
              KCALL(kpds_create_map(dst_obj));
	   }
	}	
	else
	{
	   if (!kpds_query_value(dst_obj))
	   {
              KCALL(kpds_create_value(dst_obj));
	   }
	}	
        KCALL(kpds_set_attributes(dst_obj,datasize,
                                          w,h,d,t,e,datatype,type,NULL));
	
	/* 
	 *  After all of the segments and attributes that need to be 
	 *  transferred to the destination object have been set or created, 
	 *  make a reference to the object so that further attribute changes 
	 *  will only affect how the data is presented.
	 */
        KCALL((dst_obj = kpds_reference_object(dst_obj)) != KOBJECT_INVALID);
        objlist = klist_add(objlist,dst_obj,"KOBJECT");

/*
         *  Set source and destination presentation data types as either
	 *  KDOUBLE or KULONG (KULONG only if the operator chosen is a 
	 *  bitwise operator.
         */

        type = kdatatype_cast_process(type, type, (KULONG | KDOUBLE));
	if ( (kstrcmp(function, "set") == 0) || 
	     (kstrcmp(function, "not") == 0) ||
	     (kstrcmp(function, "clear") == 0) )
	{
	   if (type == KDOUBLE)
	   {
	      kinfo(KSTANDARD, 
		"Warning: lkarith1 casts floating point input data to unsigned "
                "long before performing bitwise operation - precision may be "
                "lost.");
	   }
	   type = KULONG;
	}
	else
	   type = KDOUBLE;

	KCALL(kpds_set_attribute(src_obj, datatype, type));
	KCALL(kpds_set_attribute(dst_obj, datatype, type));

/*
         *  We will process the data by optimal regions, so get the optimal
	 *  region size and the number of optimal regions in src_obj.
	 *  Do this after setting the data type because data type is a 
	 *  in determining optimal region size.
         */
        KCALL(kpds_get_attribute(src_obj,opt_region,
                                         &w,&h,&d,&t,&e, &num_rgns));
	num_pts = w*h*d*t*e;

        KCALL(kpds_set_attribute(src_obj, region_size, w,h,d,t,e));
        KCALL(kpds_set_attribute(dst_obj, region_size, w,h,d,t,e));

/*
	 *  Do the processing by requesting regions of data from src_obj, 
	 *  evaluating the expression, and then storing the result into 
	 *  the destination object.
	 */
	if (type == KDOUBLE)
	{
           KCALL(!((data = (double *)
                        kmalloc((size_t)(w*h*d*t*e*sizeof(double)))) == NULL));
           objlist = klist_add(objlist,data,"KMALLOC");
	   for (num = 0; num < num_rgns; num++)
	   {
              KCALL(!((data = (double *)
                       kpds_get_data(src_obj,region,(kaddr) data)) == NULL));

              for (j = 0; j < num_pts; j++) data[j] = routine(data[j]);
   
              KCALL(kpds_put_data(dst_obj,region,(kaddr)data));
	   }
	}
	else if (type == KULONG)
	{ 
	   /* type should only be KULONG if a bitwise function (set, 
	    * clear, or not) is called.
	    */
           KCALL(!((ldata=(unsigned long *)
                 kmalloc((size_t)(w*h*d*t*e*sizeof(unsigned long)))) == NULL));
           objlist = klist_add(objlist,ldata,"KMALLOC");
	   for (num = 0; num < num_rgns; num++)
	   {
              KCALL(!((ldata=(unsigned long *)
                          kpds_get_data(src_obj,region,(kaddr) data)) == NULL));

	      for (j = 0; j < num_pts; j++) ldata[j] = lroutine(ldata[j]);

              /* If original data was BIT, need to trim off all but bit0
                 so that the output cast works */
              if (in_type == KBIT)
                {
	          for (j = 0; j < num_pts; j++)
                    {
                      ldata[j] = ldata[j] & 0x1;
                    }
                }
 
              KCALL(kpds_put_data(dst_obj,region,(kaddr)ldata));
	   }
	}
	else 
	{
	   kerror(lib, rtn, "Unrecognized processing datatype %d",type);
	   error = TRUE;
	}
	
        (void)klist_free(objlist, (kfunc_void)lkcall_free);

        if (error)
        {
	   return(FALSE);
        }
        else
        {
           return(TRUE);
        }
}

/* Function wrappers to isolate us from the problem of being a macro! */
static double ka1_kfabs(double x) { return(kfabs(x)); }
static double ka1_ksqrt(double x) { return(ksqrt(x)); }
static double ka1_kcbrt(double x) { return(kcbrt(x)); }
static double ka1_kneg(double x) { return(kneg(x)); }
static double ka1_krecip(double x) { return(krecip(x)); }
static unsigned long ka1_knot(unsigned long x) { return(knot(x)); }
static unsigned long ka1_kclear(unsigned long x) { return(kclear(x)); }
static unsigned long ka1_kset(unsigned long x) { return(kset(x)); }
static double ka1_ksin(double x) { return(ksin(x)); }
static double ka1_kasin(double x) { return(kasin(x)); }
static double ka1_kcos(double x) { return(kcos(x)); }
static double ka1_kacos(double x) { return(kacos(x)); }
static double ka1_ktan(double x) { return(ktan(x)); }
static double ka1_katan(double x) { return(katan(x)); }
static double ka1_ksinh(double x) { return(ksinh(x)); }
static double ka1_kcosh(double x) { return(kcosh(x)); }
static double ka1_ktanh(double x) { return(ktanh(x)); }
static double ka1_kasinh(double x) { return(kasinh(x)); }
static double ka1_kacosh(double x) { return(kacosh(x)); }
static double ka1_katanh(double x) { return(katanh(x)); }
static double ka1_ksinc(double x) { return(ksinc(x)); }
static double ka1_kceil(double x) { return(kceil(x)); }
static double ka1_kfloor(double x) { return(kfloor(x)); }
static double ka1_ktrunc(double x) { return(ktrunc(x)); }
static double ka1_kfraction(double x) { return(kfraction(x)); }
static double ka1_kerf(double x) { return(kerf(x)); }
static double ka1_kerfc(double x) { return(kerfc(x)); }
static double ka1_kgamma(double x) { return(kgamma(x)); }
/* -library_code_end */
