 /*
  * Khoros: $Id: lklinearop.c,v 1.17 1996/07/05 02:11:10 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lklinearop.c,v 1.17 1996/07/05 02:11:10 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for klinearop
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lklinearop
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
#include "internals.h"

/* Uncomment the next line for excruciating detail */
/*
#define KLINEAROP_SUPER_DEBUG
*/
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lklinearop - perform linear operations (convolution/correlation) on a data object
* 
*       Purpose: Library routine for klinearop.  Performs convolution or
*                correlation operations on a data object.
*
*         Input: i1 (src)       - source data object
*                i2 (kernel)    - kernel data object
*                filtering_type - linear (0) or circular (1)
*                real_pad_value - real value for padding (only used if needed)
*                img_pad_value  - imaginary value for padding (only used if needed)
*                offset         - array of kernel offset values (location of 
*                                 hot-spot), these 5 values (w,h,d,t,e) override
*                                 those stored in the kernel object. If offset
*                                 is NULL, then the values stored with 
*                                 the kernel object are used (KPDS_KERNEL_ORIGIN).
*                reflection     - reflect (for convolution) (1) or do not 
*                                 reflect (for correlation) (0) the kernel
*                                 before processing
*                sub_size       - array specifying the subregion size; these 
*                                 5 values (w,h,d,t,e) specify a region of 
*                                 the input object on which to operate.
*                                 if sub_size is NULL, then the entire data object
*                                 is processed. The corresponding region in the output
*                                 object will be update with the results of the
*                                 linear operation; the data outside this region is
*                                 copied directly from the source object and is not
*                                 modified.
*                sub_pos        - array specifying the subregion position, 
*                                 these 5 values specify the origin of the 
*                                 subregion in the input object (w,h,d,t,e).
*                                 if sub_pos is NULL, then the subregion is assumed
*                                 to begin at (0,0,0,0,0).
*                upcast         - force upcast of input data type to KDOUBLE 
*                                 or KDCOMPLEX before processing (1), or allow 
*                                 upcast of data types to the best fitting 
*                                 type in {KUBYTE, KLONG, KDOUBLE, KDCOMPLEX} 
*                                 (0) (look out for loss of precision)
*
*        Output: o (dest)    - destination data object 
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: 
*    Written By: Jonio Roberto de Hollanda Cavalcanti, Ramiro Jordan
*          Date: Jul 04, 1996
*      Verified: Jan 26, 1994
*  Side Effects: 
*      Examples: 
* Modifications: Scott Wilson (major rewrites, Nov '95)
****************************************************************/
/* -library_def */
int lklinearop( kobject src,
                kobject kernel,
                int filtering_type,
                double real_pad_value,
                double img_pad_value,
                int  *offset,
                int  reflection,
                int  *sub_size,
                int  *sub_pos,
                int  upcast,
                kobject dest )
/* -library_def_end */

/* -library_code */
{
   klist *objlist=NULL;
   kobject src_ref=NULL;
   kobject dest_ref=NULL;
   char *lib = "kdatamanip";     /* library name (for error reporting) */
   char *rtn = "lklinearop";     /* routine name (for error reporting) */

   int data_type;                /* source object data type */
   int kernel_data_type;         /* kernel object data type */
   int pdata_type;               /* data type for processing */
   int w,h,d,t,e;                /* No. of data elements in each direction */
			         /* of the source object */
   int wk,hk,dk,tk,ek;           /* No. of data elements in each direction */
                                 /* of the kernel object */
   int wt,ht,dt,tt,et;           /* No. of total data elements in each */
                                 /* direction for processing */ 
   int sub_length;               /* No. of data points in output region */
   int i      ;                  /* loop counters */
   int region_size[5];           /* Size of the unit of data to be processed */
   unsigned char *bdata=NULL,*beval=NULL,*bkdata=NULL,tempb; /* Pointers for handling KUBYTE data */
   long          *ldata=NULL,*leval=NULL,*lkdata=NULL,templ; /* Pointers for handling KLONG data */
   double        *ddata=NULL,*deval=NULL,*dkdata=NULL,tempd; /* Pointers for handling KDOUBLE data */
   kdcomplex     *cdata=NULL,*ceval=NULL,*ckdata=NULL,tempc; /* Pointers for handling KDCOMPLEX data */
   int grid_type = KNONE, has_time = FALSE, transfer_loc; /* Stuff for location segment handling */
   int i0,i1,i2,i3,i4;           /* Loop counters used for scanning the data */
   int k0,k1,k2,k3,k4;           /* Loop counters used for scanning the kernel */
   int cnts0,cnts1,cnts2,cnts3,cnts4; /* Strides to get around in the source data region */
   int cntd0,cntd1,cntd2,cntd3,cntd4; /* Strides to get around in the dest data region */
   int cntk0,cntk1,cntk2,cntk3,cntk4; /* Strides to get around in the kernel region */
   int kern_index,data_index;    /* Intermediate variables used to index the arrays */
   int off[5];                   /* Local array to contain kernel hotspot (may be modified) */
   int pos[5];                   /* Local array to contain subregion position (may be modified) */
   int size[5];                  /* Local array to contain subregion size (may be modified) */
   int repeat[5];		 /* Markers for those object dimensions that should be looped
                                    through independently */
   int reps[5];                  /* Local array to contain number of reps along each dimension */
   int base_src[5];              /* Local array to contain src displacement vector */
   int base_dst[5];              /* Local array to contain dst displacement vector */
   int rw,rh,rd,rt,re;           /* Loop variables for independent region processing */

  /*
   * Print out the full set of arguments coming in to assist in debugging.
   */
   kinfo(KVERBOSE,"Parameters passed into klinearop:");
   kinfo(KVERBOSE,"  Filtering type: %d",filtering_type);
   kinfo(KVERBOSE,"  Pad value: %f %f",real_pad_value,img_pad_value);
   if (offset != NULL)
     kinfo(KVERBOSE,"  Kernel offset: w:%d h:%d d:%d t:%d e:%d",
                    offset[0],offset[1],offset[2],offset[3],offset[4]);
   kinfo(KVERBOSE,"  Reflect: %d",reflection);
   if (sub_size != NULL)
     kinfo(KVERBOSE,"  Subregion size (sub_size): w:%d h:%d d:%d t:%d e:%d",
                    sub_size[0],sub_size[1],sub_size[2],sub_size[3],sub_size[4]);
   if (sub_pos != NULL)
     kinfo(KVERBOSE,"  Subregion posn (sub_pos): w:%d h:%d d:%d t:%d e:%d",
                    sub_pos[0],sub_pos[1],sub_pos[2],sub_pos[3],sub_pos[4]);
   kinfo(KVERBOSE,"  Upcasting: %d",upcast);
   kinfo(KVERBOSE,"End of parameters passed into klinearop\n");

  /*
   * Only one object may have time data.
   */
   has_time = kapu_multi_input_time(2, src, kernel);

  /*
   * Only one object may have location data.
   */
   transfer_loc = kapu_multi_input_loc(&grid_type, 2, src, kernel);

  /*
   * If kernel object has MASK data exit lklinearop.
   */
   if (kpds_query_mask(kernel)) 
     {
       kerror(lib, rtn, "Kernel object cannot contain MASK data.");
       return(FALSE);
     }

  /*
   * If source object has MASK data exit lklinearop.
   */
   if (kpds_query_mask(src))
     {
       kerror(lib, rtn, "Source object cannot contain MASK data.");
       return(FALSE);
     }

  /*
   * Both source and destination objects must have VALUE data.
   */ 
   if (kpds_query_value(src) == 0 || kpds_query_value(kernel) == 0 )
     {
       kerror(lib, rtn, "Source and kernel objects must have VALUE data.");
       return(FALSE);
     }
   if (!kpds_query_value(dest)) KCALL(kpds_create_value(dest));

  /*
   *  Set a reference for the source object so that the library does
   *  not have any side effects on the source object's attributes.
   */
   KCALL((src_ref = kpds_reference_object(src)) != KOBJECT_INVALID);
   objlist = klist_add(objlist,src_ref,"KOBJECT");

  /*
   * Set up a reference to the kernel object to facilitate processing.
   * Do not want to introduce side effects on the kernel atributes.
   */
   KCALL((kernel = kpds_reference_object(kernel)) != KOBJECT_INVALID);
   objlist = klist_add(objlist,kernel,"KOBJECT");

  /*
   * Make sure we have a defined subregion position and size.
   * Position defaults to the origin and size to the whole data object.
   */
   if (sub_pos == NULL)
     {
       for (i=0; i<5; i++) pos[i] = 0;
     }
   else
     {
       for (i=0; i<5; i++) pos[i] = sub_pos[i];
     }
   if (sub_size == NULL)
     {
       KCALL(kpds_get_attribute(src_ref, KPDS_VALUE_SIZE, &w, &h, &d, &t, &e));
       size[0] = w;
       size[1] = h;
       size[2] = d;
       size[3] = t;
       size[4] = e;
     }
   else
     {
       for (i=0; i<5; i++) size[i] = sub_size[i];
     }

  /*
   * If source object has MAP data un-map it. Be sure to fix up the sub_size
   * value for the elements direction by multiplying it by the number of 
   * columns in the map BEFORE turning mapping mode on (or else this information
   * becomes permanently unavailable!)
   */
   if (kpds_query_map(src_ref))
     {
       KCALL(kpds_get_attribute(src_ref, KPDS_MAP_SIZE, &w, &h, &d, &t, &e));
       size[4] *= w;
       KCALL(kpds_set_attribute(src_ref, KPDS_MAPPING_MODE, KMAPPED));
       kinfo(KVERBOSE,"After mapping, new region size (after mapping):"
                    " w:%d h:%d d:%d t:%d e:%d",
                    size[0],size[1],size[2],size[3],size[4]);
     }

  /*
   * If kernel object has MAP data un-map it. 
   */
   if (kpds_query_map(kernel)) 
     KCALL(kpds_set_attribute(kernel, KPDS_MAPPING_MODE, KMAPPED));

  /*
   * If the destination object has a map, remove it (it wont't make sense).
   */
   if (kpds_query_map(dest)) KCALL(kpds_destroy_map(dest));

  /*
   * Set the data type on the actual output object to match that of
   * the *mapped* input object. The destination data type may be overridden
   * later when we handle the upcast option.
   */
   KCALL(kpds_get_attribute(src_ref, KPDS_VALUE_DATA_TYPE, &data_type));
   KCALL(kpds_set_attribute(dest,    KPDS_VALUE_DATA_TYPE, data_type));

  /*
   * See how big the source object is
   */
   KCALL(kpds_get_attribute(src_ref, KPDS_VALUE_SIZE, &w, &h, &d, &t, &e));
   kinfo(KVERBOSE,"source object is (%d,%d,%d,%d,%d)",w,h,d,t,e);

  /*
   * If offset == NULL get the default hotspot from the kernel
   * via the KPDS_KERNEL_ORIGIN attribute. Otherwise we'll use
   * the offset values provided. In either case, the values are
   * copied into a local offset array so we can modify them if
   * need be.
   */
   if (offset == NULL)
     {
       kinfo(KVERBOSE,"Reading kernel hotspot from KPDS_KERNEL_ORIGIN");
       KCALL(kpds_get_attribute(kernel,KPDS_KERNEL_ORIGIN,&off[0],
                             &off[1],&off[2],&off[3],&off[4]));
     }
   else
     {
       for (i=0; i<5; i++) off[i] = offset[i];
     }
   kinfo(KVERBOSE,"Kernel origin offset: (%d,%d,%d,%d,%d)",
                   off[0],off[1],off[2],off[3],off[4]);

  /*
   * See how big the kernel is
   */
   KCALL(kpds_get_attribute(kernel,KPDS_VALUE_SIZE,&wk,&hk,&dk,&tk,&ek));
   kinfo(KVERBOSE,"kernel object is (%d,%d,%d,%d,%d)",wk,hk,dk,tk,ek);

  /*
   * See if the hotspot location makes sense.
   * We can't have a hot-spot that lies outside the kernel.
   */ 
   if (off[0]<0 || off[0]>(wk-1) ||
       off[1]<0 || off[1]>(hk-1) ||
       off[2]<0 || off[2]>(dk-1) ||
       off[3]<0 || off[3]>(tk-1) ||
       off[4]<0 || off[4]>(ek-1))
     {
       kerror(lib, rtn, "Kernel hotspot lies outside kernel.");
       (void)klist_free(objlist, (kfunc_void)lkcall_free);
       return(FALSE);
     }

  /* 
   * See if the kernel dimensions make sense given the input object.
   * For exmple, it would not be cool to try to convolve a kernel with
   * a depth dimension of 5 against data that only has depth of 1.
   */
   if (wk > w || hk > h || dk > d || tk > t || ek > e)
     {
       if (wk > w) kerror(lib, rtn, "Kernel width inconsistent with data width.");
       if (hk > h) kerror(lib, rtn, "Kernel height inconsistent with data height.");
       if (dk > d) kerror(lib, rtn, "Kernel depth inconsistent with data depth.");
       if (tk > t) kerror(lib, rtn, "Kernel time inconsistent with data time.");
       if (ek > e) kerror(lib, rtn, "Kernel elements inconsistent with data elements.");
       (void)klist_free(objlist, (kfunc_void)lkcall_free);
       return(FALSE);
     }

  /* 
   * If a subregion has been specified, then copy all source value data onto
   * the dest object. Later, we will overwrite only those areas that
   * lie within the subregion.
   */
   if (sub_size != NULL || sub_pos != NULL)
     {
       KCALL(kpds_copy_value_data(src_ref,dest,1));
     }

  /*
   * Figure out what size of data region we will be working with.
   * We want a region that is the size of the region of support.
   */
   region_size[0]=size[0]+wk-1;
   region_size[1]=size[1]+hk-1;
   region_size[2]=size[2]+dk-1;
   region_size[3]=size[3]+tk-1;
   region_size[4]=size[4]+ek-1;
   kinfo(KVERBOSE,"Source data region size (region_size): (%d,%d,%d,%d,%d)",
                   region_size[0],region_size[1],region_size[2],
                   region_size[3],region_size[4]);

  /*
   * If convolution type of operations are to be performed, flip the
   * kernel object. Also adjust the kernel hotspot location since it
   * flips too.
   */
   if (reflection == 1)
     {
       int   num_dim;
       long *dim_list;

       KCALL(!((dim_list = (long *)kmalloc((size_t)5*sizeof(long))) == NULL));
       objlist = klist_add(objlist,dim_list,"KMALLOC");

       num_dim = 0;
       if (wk > 1) dim_list[num_dim++] = KWIDTH;
       if (hk > 1) dim_list[num_dim++] = KHEIGHT;
       if (dk > 1) dim_list[num_dim++] = KDEPTH;
       if (tk > 1) dim_list[num_dim++] = KTIME;
       if (ek > 1) dim_list[num_dim++] = KELEMENTS;
       
       KCALL(lkflip(kernel, num_dim, dim_list, kernel));
       objlist = klist_delete(objlist,dim_list);
       kinfo(KVERBOSE,"Kernel flipped..."); 
       off[0] = wk-off[0]-1;
       off[1] = hk-off[1]-1;
       off[2] = dk-off[2]-1;
       off[3] = tk-off[3]-1;
       off[4] = ek-off[4]-1;
       kinfo(KVERBOSE,"Kernel origin offset flipped, now: (%d,%d,%d,%d,%d)",
                   off[0],off[1],off[2],off[3],off[4]);
     }
   
  /*
   * Determine the region of support of the
   * kernel when run over the whole dataset.
   */
   wt = (w + wk - 1);
   ht = (h + hk - 1);
   dt = (d + dk - 1);
   tt = (t + tk - 1);
   et = (e + ek - 1);
   kinfo(KVERBOSE,"Region of support is (%d,%d,%d,%d,%d)",wt,ht,dt,tt,et);

  /*
   * Set up the object dimensions before processing.
   */
   KCALL(kpds_set_attribute(dest, KPDS_VALUE_SIZE,w,h,d,t,e));

  /*
   * Pad the source object.
   * This pad value will be used to process the boundary points
   * in both cases truncated and expanded. 
   *
   * Based on the filtering type selected, perform zero (or constant)
   * padding or circular wrapping of data elements.  That is:
   * 0) linear filtering: zero or constant value pad
   * 1) circular filtering: wraparound of data elements
   */
   if (filtering_type == 0) 
     {
       KCALL(kpds_set_attribute(src_ref, KPDS_VALUE_INTERPOLATE, KPAD));
       KCALL(kpds_set_attribute(src_ref, KPDS_VALUE_PAD_VALUE,real_pad_value,img_pad_value));
     }
   else
     {
       KCALL(kpds_set_attribute(src_ref, KPDS_VALUE_INTERPOLATE, KWRAP));
     }

  /*
   * The upcast variable set to TRUE indicates that the data type
   * will be forced to DOUBLE or KDCOMPLEX.
   * If upcast is FALSE, select the appropriate common data type for 
   * processing the data.
   *
   * Want to minimize any corruption on the data integrity.
   * Recommended data types are:
   *      - KUBYTE = unsigned char 
   *      - KLONG = long integer
   *      - KDOUBLE = double precision floating point
   *      - KDCOMPLEX = double precision complex
   */
   KCALL(kpds_get_attribute(kernel,KPDS_VALUE_DATA_TYPE,&kernel_data_type));
   KCALL(kpds_get_attribute(src_ref,KPDS_VALUE_DATA_TYPE,&data_type));
   if (upcast == 0)
     {
       pdata_type = kdatatype_cast_process(data_type,kernel_data_type,
                            (KUBYTE | KLONG | KDOUBLE | KDCOMPLEX));
     }
   else 
     {
       pdata_type = kdatatype_cast_process(data_type,kernel_data_type,
                            (KDOUBLE | KDCOMPLEX));
       KCALL(kpds_set_attribute(dest, KPDS_VALUE_DATA_TYPE, pdata_type));
     }
   kinfo(KVERBOSE,"Process data type is: %d",pdata_type);

  /*
   * Set a reference for the destination object. All further work
   * will be done on the reference.
   */
   KCALL((dest_ref = kpds_reference_object(dest)) != KOBJECT_INVALID);
   objlist = klist_add(objlist,dest_ref,"KOBJECT");

  /*
   * If data is byte or ubyte representable then process it as ubyte (KUBYTE).
   * If data is ulong, long, ushort  or short representable then 
   * process it as long (KLONG).
   * If data is float representable then process it as double (KDOUBLE).
   * If data is COMPLEX representable then process it as double 
   * complex (KDCOMPLEX).  In the destination object, the data type 
   * will be automatically cast to the appropriate data type.
   */
   if ( (pdata_type == KUBYTE) || (pdata_type == KLONG) || 
        (pdata_type == KDOUBLE) || (pdata_type == KDCOMPLEX) )
     {
       KCALL(kpds_set_attribute(src_ref, KPDS_VALUE_DATA_TYPE,pdata_type));
       KCALL(kpds_set_attribute(kernel, KPDS_VALUE_DATA_TYPE, pdata_type));
       KCALL(kpds_set_attribute(dest_ref, KPDS_VALUE_DATA_TYPE, pdata_type));
     }

  /*
   * See if we can figure out which dimensions in the input object need to be
   * treated as repetitions. This occurs if a src dimension is greater than
   * the corresponding kernel dimension and that kernel dimension
   * is 1. For example, consider a (3,3,1,1,1) kernel to be used against an
   * input object of size (5,5,3,1,1). Here, the kernel should be convolved
   * against three WxH planes independently. 
   */
   for (i=0; i<5; i++) repeat[i] = 0;
   if (size[0]>wk && wk==1) repeat[0]=1;
   if (size[1]>hk && hk==1) repeat[1]=1;
   if (size[2]>dk && dk==1) repeat[2]=1;
   if (size[3]>tk && tk==1) repeat[3]=1;
   if (size[4]>ek && ek==1) repeat[4]=1;
   /* Compute number of repetitions along each dimension */
   for (i=0; i<5; i++) reps[i] = 1;
   for (i=0; i<5; i++) if (repeat[i] == 1) reps[i] = size[i];

   kinfo(KVERBOSE,"  Axes needing repetitions : (%d,%d,%d,%d,%d)",
                  repeat[0],repeat[1],repeat[2],repeat[3],repeat[4]);
   kinfo(KVERBOSE,"  Repetition counts        : (%d,%d,%d,%d,%d)",
                  reps[0],reps[1],reps[2],reps[3],reps[4]);
   
   kinfo(KVERBOSE,"src region size: (%d,%d,%d,%d,%d)",
                          region_size[0],region_size[1],
                          region_size[2],region_size[3],
                          region_size[4]);
   kinfo(KVERBOSE,"dst region size: (%d,%d,%d,%d,%d)",
                          size[0],size[1],size[2],size[3],size[4]);

  /*
   * Now we need to go back and fix up region_size and size so that they
   * reflect the repetition structure.
   */
   for (i=0; i<5; i++)
     {
       if (repeat[i])
         {
           size[i] = 1;
           region_size[i] = 1;
         }
     }

   kinfo(KVERBOSE,"accounting for repetitions, src region size: (%d,%d,%d,%d,%d)",
                          region_size[0],region_size[1],
                          region_size[2],region_size[3],
                          region_size[4]);
   kinfo(KVERBOSE,"accounting for repetitions, dst region size: (%d,%d,%d,%d,%d)",
                          size[0],size[1],size[2],size[3],size[4]);

  /*
   * Set up the object size attributes. Src will be set up to retrieve
   * an area that covers the region of support of the subregion. The
   * destination will be set up to write into the subregion only.
   */
   KCALL(kpds_set_attribute(src_ref,
                          KPDS_VALUE_REGION_SIZE,
                          region_size[0],region_size[1],
                          region_size[2],region_size[3],
                          region_size[4]));
   KCALL(kpds_set_attribute(dest_ref,
                          KPDS_VALUE_REGION_SIZE,
                          size[0],size[1],size[2],size[3],size[4]));

  /*
   * Compute size of data buffers for use in reading and writing to the
   * data objects.
   */
   sub_length = size[0]*size[1]*size[2]*size[3]*size[4];

  /*
   * Set up stride constants to be used in the source data loops.
   */
   cnts0 = 1;                               /* Size of a single point */
   cnts1 = cnts0*region_size[0];            /* size of W plane */
   cnts2 = cnts1*region_size[1];            /* size of WxH plane */
   cnts3 = cnts2*region_size[2];            /* size of WxHxD volume */
   cnts4 = cnts3*region_size[3];            /* size of WxHxDxT hyperspace */
   kinfo(KVERBOSE,"source strides: (%d,%d,%d,%d,%d)",cnts0,cnts1,cnts2,cnts3,cnts4);

  /*
   * Set up stride constants to be used in the dest data mapping.
   */
   cntd0 = 1;                            /* Size of a single point */
   cntd1 = cntd0*size[0];            /* size of W plane */
   cntd2 = cntd1*size[1];            /* size of WxH plane */
   cntd3 = cntd2*size[2];            /* size of WxHxD volume */
   cntd4 = cntd3*size[3];            /* size of WxHxDxT hyperspace */
   kinfo(KVERBOSE,"dest   strides: (%d,%d,%d,%d,%d)",cntd0,cntd1,cntd2,cntd3,cntd4);

  /*
   * Set up stride constants to be used in the kernel loops.
   */
   cntk0 = 1;                             /* Size of a single point */
   cntk1 = cntk0*wk;                      /* size of W line */
   cntk2 = cntk1*hk;                      /* size of WxH plane */
   cntk3 = cntk2*dk;                      /* size of WxHxD volume */
   cntk4 = cntk3*tk;                      /* size of WxHxDxT hyperspace */
   kinfo(KVERBOSE,"kernel strides: (%d,%d,%d,%d,%d)",cntk0,cntk1,cntk2,cntk3,cntk4);

  /*
   * A general note on how the internal loops work. The loops are set up
   * to scan over all locations in the input data that correspond to data in 
   * the sub_region*. The kernel hotspot will be placed at each of those 
   * locations and the weighted multiply-accumulate will be
   * performed. The result is stored in the output array at the proper location
   * for the *sub_region*. The various larger sizes and offsets for accessing the
   * data are just a way to get around being forced to check the indices for
   * out-of-bounds accesses at the innermost loop (should also accomplish wrapping
   * if that ever starts working in data services).
   *
   * The code is currently set up to always operate in 5D, even if the data isn't
   * of that dimensionality. This is done for ease of maintenance and testing.
   * The cost of always operating in 5D is an additional integer multiply and add
   * for each unused axis at the inner loop (it shows up in the indexing
   * calculation. We'll just live with that overhead for now; perhaps later
   * the additional indexing multiplies can be moved outside the appropriate loops;
   * the additions will still be there).
   *
   * The data is read from the source object and written to the dest object at
   * an offset that is computed for each dimensionally (kernel-wise) consistent
   * chunk of data (a repetition). This position must be manually computed because we
   * are not processing data in contiguous, consecutive regions (which is supported by
   * the autoincrement code in dataservices).
   */

  /*
   * Process KDOUBLE data.
   */
   if (pdata_type == KDOUBLE) {

     /* Read all kernel data */
     dkdata = NULL;
     ddata = NULL;
     KCALL(!((dkdata = (double *)kpds_get_data(kernel,KPDS_VALUE_ALL,(kaddr)dkdata)) == NULL));

     /* Get space for output array */
     KCALL(!((deval = (double *)kmalloc((size_t)sub_length*sizeof(double))) == NULL));
     objlist = klist_add(objlist,deval,"KMALLOC");

     /* Loop over independently processed regions */
     for (re=0; re<reps[4]; re++)
       {
         for (rt=0; rt<reps[3]; rt++)
           {
             for (rd=0; rd<reps[2]; rd++)
               {
                 for (rh=0; rh<reps[1]; rh++)
                   {
                     for (rw=0; rw<reps[0]; rw++)
                       {
                         kinfo(KVERBOSE,"Processing region # (%d,%d,%d,%d,%d)",rw,rh,rd,rt,re);
                         /* Set up src position so that we grab the right chunk of data */
                         base_src[0] = rw;
                         base_src[1] = rh;
                         base_src[2] = rd;
                         base_src[3] = rt;
                         base_src[4] = re;
                         kinfo(KVERBOSE,"  src region origin is at (%d,%d,%d,%d,%d)",
                                        base_src[0],base_src[1],base_src[2],base_src[3],base_src[4]);
                         KCALL(kpds_set_attribute(src_ref,KPDS_VALUE_POSITION,
                                   base_src[0]+pos[0]-off[0],base_src[1]+pos[1]-off[1],
                                   base_src[2]+pos[2]-off[2],base_src[3]+pos[3]-off[3],
                                   base_src[4]+pos[4]-off[4]));
                         /* Set up dst position so that we write the result at the right place */
                         base_dst[0] = rw;
                         base_dst[1] = rh;
                         base_dst[2] = rd;
                         base_dst[3] = rt;
                         base_dst[4] = re;
                         kinfo(KVERBOSE,"  dst region origin is at (%d,%d,%d,%d,%d)",
                                        base_dst[0],base_dst[1],base_dst[2],base_dst[3],base_dst[4]);
                         KCALL(kpds_set_attribute(dest_ref,KPDS_VALUE_POSITION,
                                   base_dst[0]+pos[0],base_dst[1]+pos[1],base_dst[2]+pos[2],
                                   base_dst[3]+pos[3],base_dst[4]+pos[4]));
                         kinfo(KVERBOSE,"src: at (%d,%d,%d,%d,%d)",
                                   base_src[0]+pos[0]-off[0],base_src[1]+pos[1]-off[1],
                                   base_src[2]+pos[2]-off[2],base_src[3]+pos[3]-off[3],
                                   base_src[4]+pos[4]-off[4]);
                         kinfo(KVERBOSE,"dst: at (%d,%d,%d,%d,%d)",
                                   base_dst[0]+pos[0],base_dst[1]+pos[1],base_dst[2]+pos[2],
                                   base_dst[3]+pos[3],base_dst[4]+pos[4]);

                         /* Read the source object extended data region */
                         KCALL(!((ddata = (double *)kpds_get_data(src_ref,KPDS_VALUE_REGION,(kaddr)ddata)) == NULL));

                         /* Loop over the data sub_region */
                         for (i4=0; i4<size[4]; i4++)
                           {
                             for (i3=0; i3<size[3]; i3++) 
                               {
                                 for (i2=0; i2<size[2]; i2++)
                                   {
                                     for (i1=0; i1<size[1]; i1++)
                                       {
                                         for (i0=0; i0<size[0]; i0++)
                                           {
#ifdef KLINEAROP_SUPER_DEBUG
                                             kinfo(KVERBOSE,"Kernel array origin at: (%d,%d,%d,%d,%d)",i0,i1,i2,i3,i4);
#endif
                                             /* Loop over the kernel and accumulate to the output point */
                                             tempd = 0.0;
                                             for (k4=0; k4<ek; k4++)
                                               {
                                                 for (k3=0; k3<tk; k3++)
                                                   {
                                                     for (k2=0; k2<dk; k2++)
                                                       {
                                                         for (k1=0; k1<hk; k1++)
                                                           {
                                                             for (k0=0; k0<wk; k0++)
                                                               {
                                                                 kern_index = k4*cntk4+k3*cntk3+k2*cntk2+
                                                                              k1*cntk1+k0*cntk0;
                                                                 data_index = (i4+k4)*cnts4+(i3+k3)*cnts3+(i2+k2)*cnts2+
                                                                              (i1+k1)*cnts1+(i0+k0)*cnts0;
#ifdef KLINEAROP_SUPER_DEBUG
                                                                 kinfo(KVERBOSE,"Multiplying image data %g at"
                                                                                " (%d,%d,%d,%d,%d) "
                                                                                "with kernel data %g at"
                                                                                " (%d,%d,%d,%d,%d)",
                                                                                ddata[data_index],
                                                                                (i0+k0),(i1+k1),(i2+k2),(i3+k3),
                                                                                (i4+k4),dkdata[kern_index],
                                                                                k0,k1,k2,k3,k4);
#endif
                                                                 tempd += dkdata[kern_index]*ddata[data_index];
                                                               }
                                                           }
                                                       }
                                                   }
                                               }
                                             data_index = i4*cntd4+i3*cntd3+i2*cntd2+i1*cntd1+i0*cntd0;
#ifdef KLINEAROP_SUPER_DEBUG
                                             kinfo(KVERBOSE,"Storing result %g at (%d,%d,%d,%d,%d) in result array",
                                                            tempd,i0,i1,i2,i3,i4);
#endif
                                             deval[data_index] = tempd;
                                           } 
                                       }  
                                   } 
                               }
                           }
                         KCALL(kpds_put_data(dest_ref, KPDS_VALUE_REGION, (kaddr)deval));
                       }
                   }
               }
           }
       }
     if (ddata != NULL) kfree_and_NULL(ddata);
   }

  /*
   * Process KLONG data.
   */
   if (pdata_type == KLONG) {

     /* Read all kernel data */
     lkdata = NULL;
     ldata = NULL;
     KCALL(!((lkdata = (long *)kpds_get_data(kernel,KPDS_VALUE_ALL,(kaddr)lkdata)) == NULL));

     /* Get space for output array */
     KCALL(!((leval = (long *)kmalloc((size_t)sub_length*sizeof(long))) == NULL));
     objlist = klist_add(objlist,leval,"KMALLOC");

     /* Loop over independently processed regions */
     for (re=0; re<reps[4]; re++)
       {
         for (rt=0; rt<reps[3]; rt++)
           {
             for (rd=0; rd<reps[2]; rd++)
               {
                 for (rh=0; rh<reps[1]; rh++)
                   {
                     for (rw=0; rw<reps[0]; rw++)
                       {
                         kinfo(KVERBOSE,"Processing region # (%d,%d,%d,%d,%d)",rw,rh,rd,rt,re);
                         /* Set up src position so that we grab the right chunk of data */
                         base_src[0] = rw;
                         base_src[1] = rh;
                         base_src[2] = rd;
                         base_src[3] = rt;
                         base_src[4] = re;
                         kinfo(KVERBOSE,"  src region origin is at (%d,%d,%d,%d,%d)",
                                        base_src[0],base_src[1],base_src[2],base_src[3],base_src[4]);
                         KCALL(kpds_set_attribute(src_ref,KPDS_VALUE_POSITION,
                                   base_src[0]+pos[0]-off[0],base_src[1]+pos[1]-off[1],
                                   base_src[2]+pos[2]-off[2],base_src[3]+pos[3]-off[3],
                                   base_src[4]+pos[4]-off[4]));
                         /* Set up dst position so that we write the result at the right place */
                         base_dst[0] = rw;
                         base_dst[1] = rh;
                         base_dst[2] = rd;
                         base_dst[3] = rt;
                         base_dst[4] = re;
                         kinfo(KVERBOSE,"  dst region origin is at (%d,%d,%d,%d,%d)",
                                        base_dst[0],base_dst[1],base_dst[2],base_dst[3],base_dst[4]);
                         KCALL(kpds_set_attribute(dest_ref,KPDS_VALUE_POSITION,
                                   base_dst[0]+pos[0],base_dst[1]+pos[1],base_dst[2]+pos[2],
                                   base_dst[3]+pos[3],base_dst[4]+pos[4]));
                         kinfo(KVERBOSE,"src: at (%d,%d,%d,%d,%d)",
                                   base_src[0]+pos[0]-off[0],base_src[1]+pos[1]-off[1],
                                   base_src[2]+pos[2]-off[2],base_src[3]+pos[3]-off[3],
                                   base_src[4]+pos[4]-off[4]);
                         kinfo(KVERBOSE,"dst: at (%d,%d,%d,%d,%d)",
                                   base_dst[0]+pos[0],base_dst[1]+pos[1],base_dst[2]+pos[2],
                                   base_dst[3]+pos[3],base_dst[4]+pos[4]);

                         /* Read the source object extended data region */
                         KCALL(!((ldata = (long *)kpds_get_data(src_ref,KPDS_VALUE_REGION,(kaddr)ldata)) == NULL));

                         /* Loop over the data sub_region */
                         for (i4=0; i4<size[4]; i4++)
                           {
                             for (i3=0; i3<size[3]; i3++) 
                               {
                                 for (i2=0; i2<size[2]; i2++)
                                   {
                                     for (i1=0; i1<size[1]; i1++)
                                       {
                                         for (i0=0; i0<size[0]; i0++)
                                           {
#ifdef KLINEAROP_SUPER_DEBUG
                                             kinfo(KVERBOSE,"Kernel array origin at: (%d,%d,%d,%d,%d)",i0,i1,i2,i3,i4);
#endif
                                             /* Loop over the kernel and accumulate to the output point */
                                             templ = 0.0;
                                             for (k4=0; k4<ek; k4++)
                                               {
                                                 for (k3=0; k3<tk; k3++)
                                                   {
                                                     for (k2=0; k2<dk; k2++)
                                                       {
                                                         for (k1=0; k1<hk; k1++)
                                                           {
                                                             for (k0=0; k0<wk; k0++)
                                                               {
                                                                 kern_index = k4*cntk4+k3*cntk3+k2*cntk2+
                                                                              k1*cntk1+k0*cntk0;
                                                                 data_index = (i4+k4)*cnts4+(i3+k3)*cnts3+(i2+k2)*cnts2+
                                                                              (i1+k1)*cnts1+(i0+k0)*cnts0;
#ifdef KLINEAROP_SUPER_DEBUG
                                                                 kinfo(KVERBOSE,"Multiplying image data %d at"
                                                                                " (%d,%d,%d,%d,%d) "
                                                                                "with kernel data %d at"
                                                                                " (%d,%d,%d,%d,%d)",
                                                                                ldata[data_index],
                                                                                (i0+k0),(i1+k1),(i2+k2),(i3+k3),
                                                                                (i4+k4),lkdata[kern_index],
                                                                                k0,k1,k2,k3,k4);
#endif
                                                                 templ += lkdata[kern_index]*ldata[data_index];
                                                               }
                                                           }
                                                       }
                                                   }
                                               }
                                             data_index = i4*cntd4+i3*cntd3+i2*cntd2+i1*cntd1+i0*cntd0;
#ifdef KLINEAROP_SUPER_DEBUG
                                             kinfo(KVERBOSE,"Storing result %d at (%d,%d,%d,%d,%d) in result array",
                                                            templ,i0,i1,i2,i3,i4);
#endif
                                             leval[data_index] = templ;
                                           } 
                                       }  
                                   } 
                               }
                           }
                         KCALL(kpds_put_data(dest_ref, KPDS_VALUE_REGION, (kaddr)leval));
                       }
                   }
               }
           }
       }
     if (ldata != NULL) kfree_and_NULL(ldata);
   }

  /*
   * Process KUBYTE data.
   */
   if (pdata_type == KUBYTE) {

     /* Read all kernel data */
     bkdata = NULL;
     bdata = NULL;
     KCALL(!((bkdata = (unsigned char *)kpds_get_data(kernel,KPDS_VALUE_ALL,(kaddr)bkdata)) == NULL));

     /* Get space for output array */
     KCALL(!((beval = (unsigned char *)kmalloc((size_t)sub_length*sizeof(unsigned char))) == NULL));
     objlist = klist_add(objlist,beval,"KMALLOC");

     /* Loop over independently processed regions */
     for (re=0; re<reps[4]; re++)
       {
         for (rt=0; rt<reps[3]; rt++)
           {
             for (rd=0; rd<reps[2]; rd++)
               {
                 for (rh=0; rh<reps[1]; rh++)
                   {
                     for (rw=0; rw<reps[0]; rw++)
                       {
                         kinfo(KVERBOSE,"Processing region # (%d,%d,%d,%d,%d)",rw,rh,rd,rt,re);
                         /* Set up src position so that we grab the right chunk of data */
                         base_src[0] = rw;
                         base_src[1] = rh;
                         base_src[2] = rd;
                         base_src[3] = rt;
                         base_src[4] = re;
                         kinfo(KVERBOSE,"  src region origin is at (%d,%d,%d,%d,%d)",
                                        base_src[0],base_src[1],base_src[2],base_src[3],base_src[4]);
                         KCALL(kpds_set_attribute(src_ref,KPDS_VALUE_POSITION,
                                   base_src[0]+pos[0]-off[0],base_src[1]+pos[1]-off[1],
                                   base_src[2]+pos[2]-off[2],base_src[3]+pos[3]-off[3],
                                   base_src[4]+pos[4]-off[4]));
                         /* Set up dst position so that we write the result at the right place */
                         base_dst[0] = rw;
                         base_dst[1] = rh;
                         base_dst[2] = rd;
                         base_dst[3] = rt;
                         base_dst[4] = re;
                         kinfo(KVERBOSE,"  dst region origin is at (%d,%d,%d,%d,%d)",
                                        base_dst[0],base_dst[1],base_dst[2],base_dst[3],base_dst[4]);
                         KCALL(kpds_set_attribute(dest_ref,KPDS_VALUE_POSITION,
                                   base_dst[0]+pos[0],base_dst[1]+pos[1],base_dst[2]+pos[2],
                                   base_dst[3]+pos[3],base_dst[4]+pos[4]));
                         kinfo(KVERBOSE,"src: at (%d,%d,%d,%d,%d)",
                                   base_src[0]+pos[0]-off[0],base_src[1]+pos[1]-off[1],
                                   base_src[2]+pos[2]-off[2],base_src[3]+pos[3]-off[3],
                                   base_src[4]+pos[4]-off[4]);
                         kinfo(KVERBOSE,"dst: at (%d,%d,%d,%d,%d)",
                                   base_dst[0]+pos[0],base_dst[1]+pos[1],base_dst[2]+pos[2],
                                   base_dst[3]+pos[3],base_dst[4]+pos[4]);

                         /* Read the source object extended data region */
                         KCALL(!((bdata = (unsigned char *)kpds_get_data(src_ref,KPDS_VALUE_REGION,(kaddr)bdata)) == NULL));

                         /* Loop over the data sub_region */
                         for (i4=0; i4<size[4]; i4++)
                           {
                             for (i3=0; i3<size[3]; i3++) 
                               {
                                 for (i2=0; i2<size[2]; i2++)
                                   {
                                     for (i1=0; i1<size[1]; i1++)
                                       {
                                         for (i0=0; i0<size[0]; i0++)
                                           {
#ifdef KLINEAROP_SUPER_DEBUG
                                             kinfo(KVERBOSE,"Kernel array origin at: (%d,%d,%d,%d,%d)",i0,i1,i2,i3,i4);
#endif
                                             /* Loop over the kernel and accumulate to the output point */
                                             tempb = 0.0;
                                             for (k4=0; k4<ek; k4++)
                                               {
                                                 for (k3=0; k3<tk; k3++)
                                                   {
                                                     for (k2=0; k2<dk; k2++)
                                                       {
                                                         for (k1=0; k1<hk; k1++)
                                                           {
                                                             for (k0=0; k0<wk; k0++)
                                                               {
                                                                 kern_index = k4*cntk4+k3*cntk3+k2*cntk2+
                                                                              k1*cntk1+k0*cntk0;
                                                                 data_index = (i4+k4)*cnts4+(i3+k3)*cnts3+(i2+k2)*cnts2+
                                                                              (i1+k1)*cnts1+(i0+k0)*cnts0;
#ifdef KLINEAROP_SUPER_DEBUG
                                                                 kinfo(KVERBOSE,"Multiplying image data %d at"
                                                                                " (%d,%d,%d,%d,%d) "
                                                                                "with kernel data %d at"
                                                                                " (%d,%d,%d,%d,%d)",
                                                                                bdata[data_index],
                                                                                (i0+k0),(i1+k1),(i2+k2),(i3+k3),
                                                                                (i4+k4),bkdata[kern_index],
                                                                                k0,k1,k2,k3,k4);
#endif
                                                                 tempb += bkdata[kern_index]*bdata[data_index];
                                                               }
                                                           }
                                                       }
                                                   }
                                               }
                                             data_index = i4*cntd4+i3*cntd3+i2*cntd2+i1*cntd1+i0*cntd0;
#ifdef KLINEAROP_SUPER_DEBUG
                                             kinfo(KVERBOSE,"Storing result %d at (%d,%d,%d,%d,%d) in result array",
                                                            tempb,i0,i1,i2,i3,i4);
#endif
                                             beval[data_index] = tempb;
                                           } 
                                       }  
                                   } 
                               }
                           }
                         KCALL(kpds_put_data(dest_ref, KPDS_VALUE_REGION, (kaddr)beval));
                       }
                   }
               }
           }
       }
     if (bdata != NULL) kfree_and_NULL(bdata);
   }

  /*
   * Process KDCOMPLEX data.
   */
   if (pdata_type == KDCOMPLEX) {

     /* Read all kernel data */
     ckdata = NULL;
     cdata = NULL;
     KCALL(!((ckdata = (kdcomplex *)kpds_get_data(kernel,KPDS_VALUE_ALL,(kaddr)ckdata)) == NULL));

     /* Get space for output array */
     KCALL(!((ceval = (kdcomplex *)kmalloc((size_t)sub_length*sizeof(kdcomplex ))) == NULL));
     objlist = klist_add(objlist,ceval,"KMALLOC");

     /* Loop over independently processed regions */
     for (re=0; re<reps[4]; re++)
       {
         for (rt=0; rt<reps[3]; rt++)
           {
             for (rd=0; rd<reps[2]; rd++)
               {
                 for (rh=0; rh<reps[1]; rh++)
                   {
                     for (rw=0; rw<reps[0]; rw++)
                       {
                         kinfo(KVERBOSE,"Processing region # (%d,%d,%d,%d,%d)",rw,rh,rd,rt,re);
                         /* Set up src position so that we grab the right chunk of data */
                         base_src[0] = rw;
                         base_src[1] = rh;
                         base_src[2] = rd;
                         base_src[3] = rt;
                         base_src[4] = re;
                         kinfo(KVERBOSE,"  src region origin is at (%d,%d,%d,%d,%d)",
                                        base_src[0],base_src[1],base_src[2],base_src[3],base_src[4]);
                         KCALL(kpds_set_attribute(src_ref,KPDS_VALUE_POSITION,
                                   base_src[0]+pos[0]-off[0],base_src[1]+pos[1]-off[1],
                                   base_src[2]+pos[2]-off[2],base_src[3]+pos[3]-off[3],
                                   base_src[4]+pos[4]-off[4]));
                         /* Set up dst position so that we write the result at the right place */
                         base_dst[0] = rw;
                         base_dst[1] = rh;
                         base_dst[2] = rd;
                         base_dst[3] = rt;
                         base_dst[4] = re;
                         kinfo(KVERBOSE,"  dst region origin is at (%d,%d,%d,%d,%d)",
                                        base_dst[0],base_dst[1],base_dst[2],base_dst[3],base_dst[4]);
                         KCALL(kpds_set_attribute(dest_ref,KPDS_VALUE_POSITION,
                                   base_dst[0]+pos[0],base_dst[1]+pos[1],base_dst[2]+pos[2],
                                   base_dst[3]+pos[3],base_dst[4]+pos[4]));
                         kinfo(KVERBOSE,"src: at (%d,%d,%d,%d,%d)",
                                   base_src[0]+pos[0]-off[0],base_src[1]+pos[1]-off[1],
                                   base_src[2]+pos[2]-off[2],base_src[3]+pos[3]-off[3],
                                   base_src[4]+pos[4]-off[4]);
                         kinfo(KVERBOSE,"dst: at (%d,%d,%d,%d,%d)",
                                   base_dst[0]+pos[0],base_dst[1]+pos[1],base_dst[2]+pos[2],
                                   base_dst[3]+pos[3],base_dst[4]+pos[4]);

                         /* Read the source object extended data region */
                         KCALL(!((cdata = (kdcomplex *)kpds_get_data(src_ref,KPDS_VALUE_REGION,(kaddr)cdata)) == NULL));

                         /* Loop over the data sub_region */
                         for (i4=0; i4<size[4]; i4++)
                           {
                             for (i3=0; i3<size[3]; i3++) 
                               {
                                 for (i2=0; i2<size[2]; i2++)
                                   {
                                     for (i1=0; i1<size[1]; i1++)
                                       {
                                         for (i0=0; i0<size[0]; i0++)
                                           {
#ifdef KLINEAROP_SUPER_DEBUG
                                             kinfo(KVERBOSE,"Kernel array origin at: (%d,%d,%d,%d,%d)",i0,i1,i2,i3,i4);
#endif
                                             /* Loop over the kernel and accumulate to the output point */
                                             tempc.r = 0.0;
                                             tempc.i = 0.0;
                                             for (k4=0; k4<ek; k4++)
                                               {
                                                 for (k3=0; k3<tk; k3++)
                                                   {
                                                     for (k2=0; k2<dk; k2++)
                                                       {
                                                         for (k1=0; k1<hk; k1++)
                                                           {
                                                             for (k0=0; k0<wk; k0++)
                                                               {
                                                                 kern_index = k4*cntk4+k3*cntk3+k2*cntk2+
                                                                              k1*cntk1+k0*cntk0;
                                                                 data_index = (i4+k4)*cnts4+(i3+k3)*cnts3+(i2+k2)*cnts2+
                                                                              (i1+k1)*cnts1+(i0+k0)*cnts0;
#ifdef KLINEAROP_SUPER_DEBUG
                                                                 kinfo(KVERBOSE,"Multiplying image data (%g,%g) at"
                                                                                " (%d,%d,%d,%d,%d) "
                                                                                "with kernel data (%g,%g) at"
                                                                                " (%d,%d,%d,%d,%d)",
                                                                                cdata[data_index].r,cdata[data_index].i,
                                                                                (i0+k0),(i1+k1),(i2+k2),(i3+k3),
                                                                                (i4+k4),ckdata[kern_index].r,
                                                                                ckdata[kern_index].i,
                                                                                k0,k1,k2,k3,k4);
#endif
                                                                 tempc = kdcadd(tempc,kdcmult(ckdata[kern_index],cdata[data_index]));
                                                               }
                                                           }
                                                       }
                                                   }
                                               }
                                             data_index = i4*cntd4+i3*cntd3+i2*cntd2+i1*cntd1+i0*cntd0;
#ifdef KLINEAROP_SUPER_DEBUG
                                             kinfo(KVERBOSE,"Storing result (%g,%g) at (%d,%d,%d,%d,%d) in result array",
                                                            tempc.r,tempc.i,i0,i1,i2,i3,i4);
#endif
                                             ceval[data_index] = tempc;
                                           } 
                                       }  
                                   } 
                               }
                           }
                         KCALL(kpds_put_data(dest_ref, KPDS_VALUE_REGION, (kaddr)ceval));
                       }
                   }
               }
           }
       }
     if (cdata != NULL) kfree_and_NULL(cdata);
   }

  /*
   * Make sure that unmanaged memory gets free'd.
   */
   if (bdata != NULL) kfree_and_NULL(bdata);
   if (ldata != NULL) kfree_and_NULL(ldata);
   if (ddata != NULL) kfree_and_NULL(ddata);
   if (cdata != NULL) kfree_and_NULL(cdata);
   if (dkdata != NULL) kfree_and_NULL(dkdata);
   if (bkdata != NULL) kfree_and_NULL(bkdata);
   if (lkdata != NULL) kfree_and_NULL(lkdata);
   if (ckdata != NULL) kfree_and_NULL(ckdata);

  /*
   * Make a copy of the remaining data from source data object
   * into destination object. We do that to keep the integrity
   * of the source object (concerning location and time segments
   * for instance).
   */
   KCALL(kpds_copy_remaining_data(src_ref,dest_ref));

  /*
   * Free objects being managed by the klist mechanism.
   */
   (void)klist_free(objlist, (kfunc_void)lkcall_free);
   return (TRUE);
}
/* -library_code_end */
