 /*
  * Khoros: $Id: lklogexp.c,v 1.13 1996/07/05 02:11:13 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lklogexp.c,v 1.13 1996/07/05 02:11:13 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for klogexp
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lklogexp
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lklogexp - perform log or antilog function on data object
* 
*       Purpose: lklogexp performs
*                an log/antilog function on a data object. The routine
*                takes an input data object, user specified base,
*                offset flag, and an output data object
*                in which to store the result of the operation.
*                
*                Therefore if lklogexp() were called with:
*                !       lklogexp(i, 2.0, TRUE, FALSE, o);
*		 !then
*                !       o = log2(i).
*
*                If lklogexp() were called with:
*                !       lklogexp(i, KE, FALSE, TRUE, o);
*		 !then
*                !       o = e**(i)-1.
*
*		 If the input object has a map segment, then the values
*		 in the map segment are modified directly according to the
*		 requested operation.
*
*		 If there is no map segment, then the value data is modified
*		 directly according to the requested operation. If a mask
*		 segment is also present, then only those value points with
*		 a non-zero mask are modified.
*
*         Input: i        - the input object
*                base     - the log/anti-log base
*                offset   - whether to subtract 1 after taking anti-log and add 1 before taking the log (1) or take the direct anti-log and log (0)
*                inv      - compute log (0) or anti-log (1)
*
*        Output: o        - the output object
*
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: If the argument of the logarithm evaluates to less than or
*                equal to zero, a large negative number (-KMAXFLOAT) 
*                is returned.
*
*		 A complex base is not currently supported.
*    Written By: Donna Koechner, Mark Young, & Ashish Malhotra
*          Date: Jul 04, 1996
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int lklogexp (
   kobject i,
   double  base,
   int     offset,
   int     inv,
   kobject o)
/* -library_def_end */

/* -library_code */
{
        char *lib = "kdatamanip", *rtn = "lklogexp";
        klist *objlist=NULL;
        kaddr data = NULL;
	double *d_data = NULL;
	kdcomplex *dc_data = NULL;
        unsigned char *d_mask = NULL;
        int   j, num;
	int   mask_exists,map_exists;
	int   wc,hc,dc,tc,ec,type;
	int   wr,hr,dr,tr,er,junk;
	int   numpts,numrgns;
	kdcomplex cbase,cone,cbad,ctmp;

	if (base <= 0.0)
	  {
            kerror(lib, rtn, "The base must be non-negative");
	    return(FALSE);
	  }

        if ( !inv )
          {
            base = klog(base);
          }

	cbase.r = base;
        cbase.i = 0;
	cone.r = 1;
	cone.i = 0;
	cbad.r = -KMAXFLOAT;
	cbad.i = 0;

        mask_exists=kpds_query_mask(i);
        map_exists =kpds_query_map(i);

        KCALL((i = kpds_reference_object(i)) != KOBJECT_INVALID);
        objlist = klist_add(objlist,i,"KOBJECT");

	/* Copy everything over and then we'll just tweak what needs 
           to be changed. */
	KCALL(kpds_copy_object(i,o));

        if (map_exists)
          {
            /* Process the map only */
            KCALL(kpds_get_attributes(i,KPDS_MAP_SIZE,&wc,&hc,&dc,&tc,&ec,
                                        KPDS_MAP_DATA_TYPE,&type,NULL));
	    if (type == KCOMPLEX || type == KDCOMPLEX) type = KDCOMPLEX;
            else type = KDOUBLE;
 
            KCALL(kpds_set_attributes(o,KPDS_MAP_SIZE,wc,hc,dc,tc,ec,
                                        KPDS_MAP_DATA_TYPE,type,NULL));

            KCALL(kpds_set_attribute(i,KPDS_MAP_DATA_TYPE,type));

            KCALL(kpds_get_attribute(i,KPDS_MAP_OPTIMAL_REGION_SIZE,
                                     &wr,&hr,&dr,&tr,&er,&junk));
            numpts = wr*hr*dr*tr*er;
            numrgns = wc*hc*dc*tc*ec/numpts;
 
            KCALL(kpds_set_attribute(i,KPDS_MAP_REGION_SIZE,wr,hr,dr,tr,er));
            KCALL(kpds_set_attribute(o,KPDS_MAP_REGION_SIZE,wr,hr,dr,tr,er));
 
            if (type == KDOUBLE)
              {
                KCALL(!((data=(kaddr)kmalloc((size_t)numpts*sizeof(double)))==NULL));
                d_data = (double *)data;
              }
            else 
              {
                KCALL(!((data=(kaddr*)kmalloc((size_t)numpts*
                                                  sizeof(kdcomplex)))==NULL));
                dc_data = (kdcomplex *)data;
              }
            objlist = klist_add(objlist,data,"KMALLOC");

            for (num=0; num<numrgns; num++)
              {
                KCALL(kpds_get_data(i,KPDS_MAP_REGION,(kaddr)data));
 
                if (offset)
                  {
                    if (inv)
                      {
                        for (j=0; j<numpts; j++)
                          {
                            if (type == KDOUBLE)
                              d_data[j] = kpow(base,d_data[j])-1.0;
                            else
                              dc_data[j] = kdcsub(kdcpow(cbase,dc_data[j]),cone);
                          }
                      }
                    else
                      {
                        for (j=0; j<numpts; j++)
                          {
                            if (type == KDOUBLE)
                              d_data[j] = (d_data[j]>-1.0) ?
                                klog(d_data[j] + 1)/base: (-KMAXFLOAT);
                            else
                              {
                                ctmp = kdcadd(dc_data[j],cone);
                                dc_data[j]=(kdcmag(ctmp)>0.0 ?
                                  kdclog(kdcdiv(ctmp,cbase)): cbad);
                              }
                          }
                      }
                  }
                else /* No offset */
                  {   
                    if (inv)
                      {
                        for (j=0; j<numpts; j++)
                          {
                            if (type == KDOUBLE)
                              d_data[j]=kpow(base, d_data[j]);
                            else
                              dc_data[j] = kdcpow(cbase,dc_data[j]);
                          }
                      }
                    else
                      {
                        for (j=0; j<numpts; j++)
                          {
                            if (type == KDOUBLE)
                              d_data[j] = (d_data[j] > 0.0) ?
                                klog(d_data[j])/base : (-KMAXFLOAT);
                            else
                              dc_data[j]=(kdcmag(dc_data[j])>0.0 ?
                                kdclog(kdcdiv(dc_data[j],cbase)): cbad);
                          }
                      } 
                  }
                KCALL(kpds_put_data(o,KPDS_MAP_REGION,d_data));
              }
          }
        else
          {
	    /* Process the value data */
            KCALL(kpds_get_attributes(i, KPDS_VALUE_SIZE,&wc,&hc,&dc,&tc,&ec,
                                         KPDS_VALUE_DATA_TYPE,&type,NULL));
	    if (type == KCOMPLEX || type == KDCOMPLEX) type = KDCOMPLEX;
            else type = KDOUBLE;

            KCALL(kpds_set_attributes(o, KPDS_VALUE_SIZE,wc,hc,dc,tc,ec,
                                         KPDS_VALUE_DATA_TYPE,type,NULL));

            KCALL(kpds_set_attribute(i, KPDS_VALUE_DATA_TYPE, type));
	    KCALL(kpds_get_attribute(i,KPDS_VALUE_OPTIMAL_REGION_SIZE,
                                     &wr,&hr,&dr,&tr,&er,&junk));
	    numpts = wr*hr*dr*tr*er;
  	    numrgns = wc*hc*dc*tc*ec/numpts;
	
            KCALL(kpds_set_attribute(i,KPDS_VALUE_REGION_SIZE,wr,hr,dr,tr,er));
            KCALL(kpds_set_attribute(o,KPDS_VALUE_REGION_SIZE,wr,hr,dr,tr,er));

            if (type == KDOUBLE)
              {
                KCALL(!((data=(kaddr)kmalloc((size_t)numpts*sizeof(double)))==NULL));
                d_data = (double *)data;
              }
            else 
              {
                KCALL(!((data=(kaddr*)kmalloc((size_t)numpts*
                                                  sizeof(kdcomplex)))==NULL));
                dc_data = (kdcomplex *)data;
              }
            objlist = klist_add(objlist,data,"KMALLOC");

            if (mask_exists)
              {
                KCALL(kpds_set_attribute(i, KPDS_MASK_DATA_TYPE, KUBYTE));
                KCALL(!((d_mask = (unsigned char*)kmalloc((size_t)numpts*
                                             sizeof(unsigned char)))==NULL));
                objlist = klist_add(objlist,d_mask,"KMALLOC");
                KCALL(kpds_set_attribute(i,KPDS_MASK_REGION_SIZE,
                                           wr,hr,dr,tr,er));
              }

            for (num=0; num<numrgns; num++)
              {
                KCALL(kpds_get_data(i,KPDS_VALUE_REGION,(kaddr)data));
                if (mask_exists)
                  {
                    KCALL(kpds_get_data(i,KPDS_MASK_REGION,(kaddr)d_mask));
                  }

                if (offset)
                  {
                    if (inv)
                      {
                        for (j=0; j<numpts; j++)
                          {
                            if (!mask_exists || (mask_exists && d_mask[j] != 0))
                              {
                                if (type == KDOUBLE)
                                  d_data[j] = kpow(base,d_data[j])-1.0;
                                else
                                  dc_data[j] = kdcsub(kdcpow(cbase,dc_data[j]),cone);
                              }
                          }
                      }
                    else
                      {
                        for (j=0; j<numpts; j++)
                          {
                            if (!mask_exists || (mask_exists && d_mask[j] != 0))
                              {
                                if (type == KDOUBLE)
                                  d_data[j] = (d_data[j]>-1.0) ?
                                    klog(d_data[j] + 1)/base: (-KMAXFLOAT);
                                else
                                  {
                                    ctmp = kdcadd(dc_data[j],cone);
                                    dc_data[j]=(kdcmag(ctmp)>0.0 ?
                                     kdclog(kdcdiv(ctmp,cbase)): cbad);
                                  }
                              }
                          }
                      }
                  }
                else /* No offset */
                  {   
                    if (inv)
                      {
                        for (j=0; j<numpts; j++)
                          {
                            if (!mask_exists || (mask_exists && d_mask[j] != 0))
                              {
                                if (type == KDOUBLE)
                                  d_data[j]=kpow(base, d_data[j]);
                                else
                                  dc_data[j] = kdcpow(cbase,dc_data[j]);
                              }
                          }
                      }
                    else
                      {
                        for (j=0; j<numpts; j++)
                          {
                            if (!mask_exists || (mask_exists && d_mask[j] != 0))
                              {
                                if (type == KDOUBLE)
                                  d_data[j] = (d_data[j] > 0.0) ?
                                    klog(d_data[j])/base : (-KMAXFLOAT);
                                else
                                  dc_data[j]=(kdcmag(dc_data[j])>0.0 ?
                                    kdclog(kdcdiv(dc_data[j],cbase)): cbad);
                              }
                          }
                      } 
                  }

                KCALL(kpds_put_data(o,KPDS_VALUE_REGION,data));
              }
          }

        (void)klist_free(objlist, (kfunc_void)lkcall_free);
        return(TRUE);
}
/* -library_code_end */
