#! /bin/sh

echo "running test1e.sh for karith1 ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num="$1"
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-5 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0

# ------------------------------------------------------------------------
#  set path for test data, and path for temporary directory
# ------------------------------------------------------------------------
tbpath=`kecho -tb DATAMANIP -echo path`
dpath="$tbpath/data/testdata"
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

# ------------------------------------------------------------------------
# Test  62:  set - unsigned short value data with no map 
# ------------------------------------------------------------------------
test_num=62
descr="set - unsigned short value data with no map (assume 16 bit short int)"
cat <<EOF > $tmpdir/ascii.$$
1 2
127 0
32767 65535
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 2 -hsize 3 -type 5 -o $tmpdir/ascii.$$
${BINARY_PATH}/karith1 -i $tmpdir/ascii.$$ -o $tmpdir/out.$$ -set
cat <<EOF > $tmpdir/ascii.$$
65535 65535
65535 65535
65535 65535
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 2 -hsize 3 -type 5 -o $tmpdir/result.$$
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  63:  set - unsigned int value data with no map 
# ------------------------------------------------------------------------
test_num=63
descr="set - unsigned int value data with no map (assume 32 bit int)"
cat <<EOF > $tmpdir/ascii.$$
1 2
127 0
32767 65535
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 2 -hsize 3 -type 7 -o $tmpdir/ascii.$$
${BINARY_PATH}/karith1 -i $tmpdir/ascii.$$ -o $tmpdir/out.$$ -set
cat <<EOF > $tmpdir/ascii.$$
4294967295 4294967295
4294967295 4294967295
4294967295 4294967295
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 2 -hsize 3 -type 7 -o $tmpdir/result.$$
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

exit $status

