#! /bin/sh

# Replace the oname in the following line with the name of the program
# you are testing.

object_name="karith2"

# ---------------------------------------------------------------
# ADD TESTS BETWEEN the  "Add tests below this" and "End of test 
# suite" comment lines (line 59).
# ---------------------------------------------------------------

file_name=`echo $0 | sed -e 's/^.*\///'`
echo "running "$file_name" for "$object_name" ... process ($$)"

# report utility
# ---------------------------------------------------------------
# Usage:  report  test_num test_obj cntrl_obj test_descr tol
#
# The report function uses ksegcmp to compare the test output 
# data object ($2) against the correct result ($3), and reports 
# success or failure of the test.  If the result of ksegcmp is 
# false, status is set to 1.
#
# Input Arguments: test_num - test number
#                  test_obj - the object generated by the test
#                  cntrl_obj - known result to compare against
#                  test_descr - a string describing the test
#                  tol - tolerance allowed when comparing object 
#                        data values. Recommended tolerance 1e-5
# ---------------------------------------------------------------
report()
{
   test_num="$1"
   test_obj="$2"
   cntrl_obj="$3"
   test_descr="$4"
   tol="$5"

   ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol "$tol" -sat 1 -s -rt 1 -all
   if [ $? = 0 ]
   then
      echo "Passed:  Test  $test_num --> "$test_descr""
   else
      echo "FAILED:  Test  $test_num --> "$test_descr""
      status=1
   fi
}

# Initialize status to be success (ie. 0).  Upon failure, status 
# is initialized to 1.  Set the tmpdir variable.  Script will 
# use /tmp if $TMPDIR is not defined.
status=0
tmpdir=${TMPDIR:-/tmp}

# ---------------------------------------------------------------
#  Add tests below this
# ---------------------------------------------------------------
tbpath=`kecho -tb DATAMANIP -echo path`
dpath="$tbpath/data/testdata"
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

test_num=1
descr="real input 1 added to complex constant"

cat <<EOF > $tmpdir/ascii.$$
0 1
127 127
32767 32767
65535 65535
2147483647 4294967295
EOF

${BINARY_PATH}/karith2 -i1 $tmpdir/ascii.$$ -imag 4 -o $tmpdir/out.$$ -add

cat <<EOF > $tmpdir/ascii.$$
(0, 4) (1, 4)
(127, 4) (127, 4)
(32767, 4) (32767, 4)
(65535, 4) (65535, 4)
(2147483647, 4) (4294967295, 4)
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/ascii.$$" "$descr" "1e-5"

/bin/rm -f $tmpdir/out.$$ $tmpdir/ascii.$$

test_num=2
descr="single input with location data"

${BINARY_PATH}/karith2 -i1 $dpath/w6h5d4.fval.floc.viff -add -real 0.0 -o $tmpdir/out.$$
report $test_num "$tmpdir/out.$$" "$dpath/w6h5d4.fval.floc.viff" "$descr" "1e-5"

/bin/rm -f $tmpdir/out.$$

test_num=3
descr="single input with value and map"

cat <<EOF > $tmpdir/ascii.$$
0 1 2 3 
3 2 1 0
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 2 -type byte -o $tmpdir/val.$$
cat <<EOF > $tmpdir/ascii.$$
32 33 34
34 35 36
36 37 38
38 39 40
EOF
kasc2map -i1 $tmpdir/ascii.$$ -i2 $tmpdir/val.$$ -wsize 3 -hsize 4 -type byte -o $tmpdir/in.$$
${BINARY_PATH}/karith2 -i1 $tmpdir/in.$$ -add -o $tmpdir/out.$$ -real 1.0 -add

cat <<EOF > $tmpdir/ascii.$$
33 34 35
35 36 37
37 38 39
39 40 41
EOF
kasc2map -i1 $tmpdir/ascii.$$ -i2 $tmpdir/val.$$ -wsize 3 -hsize 4 -type byte -o $tmpdir/answer.$$

report $test_num "$tmpdir/out.$$" "$tmpdir/answer.$$" "$descr" "1e-5"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/in.$$ $tmpdir/val.$$ $tmpdir/answer.$$

# ---------------------------------------------------------------
#  End of test suite
# ---------------------------------------------------------------

exit $status
