#! /bin/sh

echo "running test2.sh for kclip ... process ($$)"


# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-5 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0


# ------------------------------------------------------------------------
#  set path for test data, and path for temporary directory
# ------------------------------------------------------------------------
tbpath=`kecho -tb DATAMANIP -echo path`
dpath="$tbpath/data/testdata"
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

# ------------------------------------------------------------------------
# Test  1:  kthreshabove: double, default levels and outputs
# ------------------------------------------------------------------------
test_num=1
descr="kthreshabove, default levels and outputs"
cat > $tmpdir/asc.$$ <<EOF
0 10 20 40 60 80 100 120 140 160 180 200 220 240
EOF
${BINARY_PATH}/kthreshabove -i $tmpdir/asc.$$ -o $tmpdir/out.$$
cat > $tmpdir/asc.$$ <<EOF
0  0  0  0  0  0  0    0   1   1   1   1   1   1
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/asc.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  2:  kthreshabove: double, non-default levels and outputs
# ------------------------------------------------------------------------
test_num=2
descr="kthreshabove, non-default levels and outputs"
cat > $tmpdir/asc.$$ <<EOF
0 10 20 40 60 80 100 120 140 160 180 200 220 240
EOF
${BINARY_PATH}/kthreshabove -i $tmpdir/asc.$$ -o $tmpdir/out.$$ -uc 50 \
                            -fval 3 -tval 11
cat > $tmpdir/asc.$$ <<EOF
11  11  11  11  3  3  3   3   3   3   3   3   3   3
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/asc.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  3:  kthreshbelow: double, default levels and outputs
# ------------------------------------------------------------------------
test_num=3
descr="kthreshbelow, default levels and outputs"
cat > $tmpdir/asc.$$ <<EOF
0 10 20 40 60 80 100 120 140 160 180 200 220 240
EOF
${BINARY_PATH}/kthreshbelow -i $tmpdir/asc.$$ -o $tmpdir/out.$$
cat > $tmpdir/asc.$$ <<EOF
1 1 1 1 1 1 1 1 0 0 0 0 0 0
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/asc.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  4:  kthreshbelow: double, non-default levels and outputs
# ------------------------------------------------------------------------
test_num=4
descr="kthreshbelow, non-default levels and outputs"
cat > $tmpdir/asc.$$ <<EOF
0 10 20 40 60 80 100 120 140 160 180 200 220 240
EOF
${BINARY_PATH}/kthreshbelow -i $tmpdir/asc.$$ -o $tmpdir/out.$$ -lc 50 \
                            -fval 3 -tval 11
cat > $tmpdir/asc.$$ <<EOF
3  3  3  3  11  11  11   11   11   11   11   11   11   11
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/asc.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  5:  kmegathresh: double, lower cutoff level
# ------------------------------------------------------------------------
test_num=5
descr="kmegathresh, lower cutoff level"
cat > $tmpdir/asc.$$ <<EOF
0 10 20 40 60 80 100 120 140 160 180 200 220 240
EOF
${BINARY_PATH}/kmegathresh -i $tmpdir/asc.$$ -o $tmpdir/out.$$ -lc 64
cat > $tmpdir/asc.$$ <<EOF
0  0  0  0  0  1  1   1   1   1   1   1   1   1
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/asc.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  6:  kmegathresh: double, upper cutoff level
# ------------------------------------------------------------------------
test_num=6
descr="kmegathresh, upper cutoff level"
cat > $tmpdir/asc.$$ <<EOF
0 10 20 40 60 80 100 120 140 160 180 200 220 240
EOF
${BINARY_PATH}/kmegathresh -i $tmpdir/asc.$$ -o $tmpdir/out.$$ -uc 64
cat > $tmpdir/asc.$$ <<EOF
1  1  1  1  1  0  0   0   0   0   0   0   0   0
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/asc.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  7:  kmegathresh: double, upper and lower cutoff levels, output levels
# ------------------------------------------------------------------------
test_num=7
descr="kmegathresh, upper and lower cutoff levels, output levels"
cat > $tmpdir/asc.$$ <<EOF
0 10 20 40 60 80 100 120 140 160 180 200 220 240
EOF
${BINARY_PATH}/kmegathresh -i $tmpdir/asc.$$ -o $tmpdir/out.$$ -lc 30 -uc 64 \
                           -tval 13 -fval 19
cat > $tmpdir/asc.$$ <<EOF
19  19  19  13  13  19  19   19   19   19   19   19   19   19
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/asc.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/asc.$$


exit $status
