#! /bin/sh

# Replace the oname in the following line with the name of the program
# you are testing.

object_name="kconvert"

# ---------------------------------------------------------------
# ADD TESTS BETWEEN the  "Add tests below this" and "End of test 
# suite" comment lines (line 60).
# ---------------------------------------------------------------

file_name=`echo $0 | sed -e 's/^.*\///'`
echo "running "$file_name" for "$object_name" ... process ($$)"

# report utility
# ---------------------------------------------------------------
# Usage:  report  test_num test_obj cntrl_obj test_descr tol
#
# The report function uses ksegcmp to compare the test output 
# data object ($2) against the correct result ($3), and reports 
# success or failure of the test.  If the result of ksegcmp is 
# false, status is set to 1.
#
# Input Arguments: test_num - test number
#                  test_obj - the object generated by the test
#                  cntrl_obj - known result to compare against
#                  test_descr - a string describing the test
#                  tol - tolerance allowed when comparing object 
#                        data values. Recommended tolerance 1e-5
# ---------------------------------------------------------------
report()
{
   test_num="$1"
   test_obj="$2"
   cntrl_obj="$3"
   test_descr="$4"
   tol="$5"

   ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol "$tol" -sat 1 -s -rt 1 -all
   if [ $? = 0 ]
   then
      echo "Passed:  Test  $test_num --> "$test_descr""
   else
      echo "FAILED:  Test  $test_num --> "$test_descr""
      status=1
   fi
}

# Initialize status to be success (ie. 0).  Upon failure, status 
# is initialized to 1.  Set the tmpdir variable.  Script will 
# use /tmp if $TMPDIR is not defined.
status=0
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

# ---------------------------------------------------------------
#  Add tests below this
# ---------------------------------------------------------------

echo "=== testing conversion from unsigned int === "
echo "         bit=1 byte=2 ubyte=3 short=4 ushort=5 int=6 uint=7 long=8 "
echo "         ulong=9 float=10 double=11 complex=12 dcomplex=13 propagate=0"
cat <<EOF > $tmpdir/ascii.$$
7 127 1 255 0 1 0 1 0 1
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 2 -hsize 5 -type "unsigned int" \
	 -o $tmpdir/input.$$

# --------- test propagate case ---------
test_num=0
descr="convert uint data to uint using -type 0 (propagate)"
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -val -type 0
report $test_num "$tmpdir/result.$$" "$tmpdir/input.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/result.$$ 

# --------- test all non-complex cases ---------
for type in 1 2 3 4 5 6 7 8 9 10 11
do
   test_num=$type
   descr="convert uint data (type 7) to type $type"
   ${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -val -type $type
   kasc2val -i1 $tmpdir/ascii.$$ -wsize 2 -hsize 5 -type $type \
        -o $tmpdir/answer.$$
   report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"
# cp $tmpdir/result.$$ result.$type
# cp $tmpdir/answer.$$ answer.$type
   /bin/rm -f $tmpdir/result.$$ $tmpdir/answer.$$
done
 
# --------- test complex cases ---------
cat <<EOF > $tmpdir/ascii.$$
(7, 0) (127, 0) (1, 0) (255, 0) (0, 0) (1, 0) (0, 0) (1, 0) (0, 0) (1, 0)
EOF
for type in 12 13
do
   test_num=$type
   descr="convert uint data (type 7) to type $type"
   ${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -val -type $type
   kasc2val -i1 $tmpdir/ascii.$$ -wsize 2 -hsize 5 -type $type \
        -o $tmpdir/answer.$$
   report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"
# cp $tmpdir/result.$$ result.$type
# cp $tmpdir/answer.$$ answer.$type
   /bin/rm -f $tmpdir/result.$$ $tmpdir/answer.$$
done
 
/bin/rm -f $tmpdir/ascii.$$ $tmpdir/input.$$

# ---------------------------------------------------------------
#  End of test suite
# ---------------------------------------------------------------

/bin/rm -f $tmpdir/ascii.$$
exit $status
