#! /bin/sh 

# Replace the oname in the following line with the name of the program
# you are testing.

object_name="kconvert"

# ---------------------------------------------------------------
# ADD TESTS BETWEEN the  "Add tests below this" and "End of test 
# suite" comment lines (line 60).
# ---------------------------------------------------------------

file_name=`echo $0 | sed -e 's/^.*\///'`
echo "running "$file_name" for "$object_name" ... process ($$)"

# report utility
# ---------------------------------------------------------------
# Usage:  report  test_num test_obj cntrl_obj test_descr tol
#
# The report function uses ksegcmp to compare the test output 
# data object ($2) against the correct result ($3), and reports 
# success or failure of the test.  If the result of ksegcmp is 
# false, status is set to 1.
#
# Input Arguments: test_num - test number
#                  test_obj - the object generated by the test
#                  cntrl_obj - known result to compare against
#                  test_descr - a string describing the test
#                  tol - tolerance allowed when comparing object 
#                        data values. Recommended tolerance 1e-5
# ---------------------------------------------------------------
report()
{
   test_num="$1"
   test_obj="$2"
   cntrl_obj="$3"
   test_descr="$4"
   tol="$5"

   ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol "$tol" -sat 1 -s -rt 1 -all
   if [ $? = 0 ]
   then
      echo "Passed:  Test  $test_num --> "$test_descr""
   else
      echo "FAILED:  Test  $test_num --> "$test_descr""
      status=1
   fi
}

# Initialize status to be success (ie. 0).  Upon failure, status 
# is initialized to 1.  Set the tmpdir variable.  Script will 
# use /tmp if $TMPDIR is not defined.
status=0
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

# ---------------------------------------------------------------
#  Add tests below this
# ---------------------------------------------------------------


# ---------------------------------------------------------------
# tests:
#       test polymorphic segments except value 
#      	test for segment name typed in 
#	test for improperly spelled segment name typed in 
# ---------------------------------------------------------------

echo "=== cursory test of segment conversion === "

# test polymorphic segments except value (test typed in name as well)

# --------- map segment ---------
test_num=1a
descr="convert map from byte to complex (-map)"
cat <<EOF > $tmpdir/ascii.$$
0 1 2 3 4 5 -8 2 0 1
EOF
kasc2map -i1 $tmpdir/ascii.$$ -wsize 10 -type byte -o $tmpdir/input.$$
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -map -type complex
cat <<EOF > $tmpdir/ascii.$$
(0, 0) (1, 0) (2, 0) (3, 0) (4, 0) (5, 0) (-8, 0) (2, 0) (0, 0) (1, 0)
EOF
kasc2map -i1 $tmpdir/ascii.$$ -wsize 10 -type complex -o $tmpdir/answer.$$
report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"

test_num=1b
descr="convert map from byte to complex (-segment map)"
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -segment map \
	-type complex
report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/input.$$ $tmpdir/answer.$$

# --------- mask segment ---------
test_num=2a
descr="convert mask from bit to float (-mask)"
cat <<EOF > $tmpdir/ascii.$$
0 1 0 1 1 0 1 1 1 1
EOF
kasc2mask -i1 $tmpdir/ascii.$$ -wsize 10 -type bit -o $tmpdir/input.$$
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -mask -type float
kasc2mask -i1 $tmpdir/ascii.$$ -wsize 10 -type float -o $tmpdir/answer.$$
report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"

test_num=2b
descr="convert mask from bit to float (-segment mask)"
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -segment mask \
	-type float
report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/input.$$ $tmpdir/answer.$$

# --------- time segment ---------
test_num=3a
descr="convert time from unsigned long to float (-time)"
cat <<EOF > $tmpdir/ascii.$$
0 1 2 3 4 5 6 7 8 9 
EOF
kasc2time -i1 $tmpdir/ascii.$$ -tsize 10 -type "unsigned long" \
	-o $tmpdir/input.$$
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -time -type float
kasc2time -i1 $tmpdir/ascii.$$ -tsize 10 -type float -o $tmpdir/answer.$$
report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"

test_num=3b
descr="convert time from unsigned long to float (-segment time)"
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -segment time \
	-type float
report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/input.$$ $tmpdir/answer.$$

# --------- location segment (uniform, rectilinear, curvilinear) ---------

test_num=4a
descr="scale and offset uniform float location (propagate)"
kgenloc -xbegin 12 -xend 47 -ybegin 13 -yend 18 -zbegin 1 \
	-zend 3 -grid 1 -type float -o $tmpdir/input.$$
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -loc -scale 0.5 -roff 10
kgenloc -xbegin 16 -xend 33.5 -ybegin 16.5 -yend 19 -zbegin 10.5 \
	-zend 11.5 -grid 1 -type float -o $tmpdir/answer.$$
report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"
test_num=4b
descr="scale and offset uniform float location and convert to int"
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -loc \
	-scale 0.5 -roff 10 -type int
kgenloc -xbegin 16 -xend 33.5 -ybegin 16.5 -yend 19 -zbegin 10.5 \
	-zend 11.5 -grid 1 -type int -o $tmpdir/answer.$$
report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/input.$$ $tmpdir/answer.$$

test_num=5
descr="convert rectilinear location from double to int"
kgenloc -xbegin 12 -xend 47 -ybegin 13 -yend 18 -zbegin 1 \
	-zend 3 -grid 2 -type double -o $tmpdir/input.$$ \
	-wsize 10 -hsize 3 -dsize 2
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -loc -type int
kgenloc -xbegin 12 -xend 47 -ybegin 13 -yend 18 -zbegin 1 \
	-zend 3 -grid 2 -type int -o $tmpdir/answer.$$ \
	-wsize 10 -hsize 3 -dsize 2
report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/input.$$ $tmpdir/answer.$$

test_num=6a
descr="convert curvilinear location from short to int (-loc)"
kgenloc -xbegin 12 -xend 47 -ybegin 13 -yend 18 -zbegin 1 \
	-zend 3 -grid 3 -type short -o $tmpdir/input.$$ \
	-wsize 10 -hsize 3 -dsize 2
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -loc -type int
kgenloc -xbegin 12 -xend 47 -ybegin 13 -yend 18 -zbegin 1 \
	-zend 3 -grid 3 -type int -o $tmpdir/answer.$$ \
	-wsize 10 -hsize 3 -dsize 2
report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"
/bin/rm -f tmpdir/result.$$ 

test_num=6b
descr="convert curvilinear from short to int (-segment loc)"
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -segment loc \
	-type int
report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"

/bin/rm -f tmpdir/result.$$ 

test_num=6c
descr="convert curvilinear from short to int (-segment location)"
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ \
	-segment location -type int
report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/input.$$ $tmpdir/answer.$$
# ---------------------------------------------------------------
#  End of test suite
# ---------------------------------------------------------------

exit $status
