#! /bin/sh

# Replace the oname in the following line with the name of the program
# you are testing.

object_name="kconvert"

# ---------------------------------------------------------------
# ADD TESTS BETWEEN the  "Add tests below this" and "End of test 
# suite" comment lines (line 60).
# ---------------------------------------------------------------

file_name=`echo $0 | sed -e 's/^.*\///'`
echo "running "$file_name" for "$object_name" ... process ($$)"

# report utility
# ---------------------------------------------------------------
# Usage:  report  test_num test_obj cntrl_obj test_descr tol
#
# The report function uses ksegcmp to compare the test output 
# data object ($2) against the correct result ($3), and reports 
# success or failure of the test.  If the result of ksegcmp is 
# false, status is set to 1.
#
# Input Arguments: test_num - test number
#                  test_obj - the object generated by the test
#                  cntrl_obj - known result to compare against
#                  test_descr - a string describing the test
#                  tol - tolerance allowed when comparing object 
#                        data values. Recommended tolerance 1e-5
# ---------------------------------------------------------------
report()
{
   test_num="$1"
   test_obj="$2"
   cntrl_obj="$3"
   test_descr="$4"
   tol="$5"

   ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol "$tol" -sat 1 -s -rt 1 -all
   if [ $? = 0 ]
   then
      echo "Passed:  Test  $test_num --> "$test_descr""
   else
      echo "FAILED:  Test  $test_num --> "$test_descr""
      status=1
   fi
}

# Initialize status to be success (ie. 0).  Upon failure, status 
# is initialized to 1.  Set the tmpdir variable.  Script will 
# use /tmp if $TMPDIR is not defined.
status=0
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

# ---------------------------------------------------------------
#  Add tests below this
# ---------------------------------------------------------------

echo "=== testing scale and offset options === "

# --------- float map scaled by .9 (propagate) ---------
test_num=1
descr="float map scaled by .9 (propagate)"
cat <<EOF > $tmpdir/ascii.$$
-1 200 8
EOF
kasc2map -i1 $tmpdir/ascii.$$ -hsize 3 -type "float" -o $tmpdir/input.$$
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -map -scale  0.9
cat <<EOF > $tmpdir/ascii.$$
-0.9 180 7.2
EOF
kasc2map -i1 $tmpdir/ascii.$$ -hsize 3 -type "float" -o $tmpdir/answer.$$
report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"

# --------- int location scaled by 5 offset by 2 (-type float) ---------
test_num=2
descr="int map scaled by 5 offset by 2 (-type float)"
cat <<EOF > $tmpdir/ascii.$$
-1 200 8
EOF
kasc2map -i1 $tmpdir/ascii.$$ -hsize 3 -type "int" -o $tmpdir/input.$$
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -map \
		-roff 2 -scale 5 -type float
cat <<EOF > $tmpdir/ascii.$$
-3 1002 42
EOF
kasc2map -i1 $tmpdir/ascii.$$ -hsize 3 -type "float" -o $tmpdir/answer.$$
report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"

# --------- short value offset by -3 (-type byte) ---------
test_num=3
descr="short value offset by -3 (-type byte)"
cat <<EOF > $tmpdir/ascii.$$
1 200 8
EOF
kasc2val -i1 $tmpdir/ascii.$$ -hsize 3 -type "short" -o $tmpdir/input.$$
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -val \
		-roff -3 -type byte
cat <<EOF > $tmpdir/ascii.$$
-2 197 5
EOF
kasc2val -i1 $tmpdir/ascii.$$ -hsize 3 -type "byte" -o $tmpdir/answer.$$
report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"

# --------- unsigned byte mask scaled by .5 offset by 1 (propagate) ---------
test_num=4
descr="unsigned byte mask scaled by .5 offset by 1 (propagate)"
cat <<EOF > $tmpdir/ascii.$$
8 4 10
EOF
kasc2mask -i1 $tmpdir/ascii.$$ -hsize 3 -type "unsigned byte" \
		-o $tmpdir/input.$$
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -mask -scale .5 \
		-roff 1 
cat <<EOF > $tmpdir/ascii.$$
5 3 6
EOF
kasc2mask -i1 $tmpdir/ascii.$$ -hsize 3 -type "unsigned byte" \
		-o $tmpdir/answer.$$
report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"

# --------- complex time scaled by .25 offset by (2, -2) (propagate) ---------
test_num=5
descr="complex time scaled by .25 offset by (2, -2) (propagate)"
cat <<EOF > $tmpdir/ascii.$$
(1, 2) (4.7, 8) (-5, 0)
EOF
kasc2time -i1 $tmpdir/ascii.$$ -tsize 3 -type complex -o $tmpdir/input.$$
${BINARY_PATH}/kconvert -i $tmpdir/input.$$ -o $tmpdir/result.$$ -time \
		-scale .25 -roff 2.0 -ioff -2.0
cat <<EOF > $tmpdir/ascii.$$
(2.25, -1.5) (3.175, 0.0) (0.75, -2)
EOF
kasc2time -i1 $tmpdir/ascii.$$ -tsize 3 -type complex -o $tmpdir/answer.$$
report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/answer.$$ $tmpdir/result.$$ $tmpdir/input.$$
# ---------------------------------------------------------------
#  End of test suite
# ---------------------------------------------------------------

exit $status
