#! /bin/sh

echo "running test.sh for kflip ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-5 -sat 1 -s -rt 1 -all -oat 1
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0


# ------------------------------------------------------------------------
#  set data path for image data, and path for temporary directory
# ------------------------------------------------------------------------
tbpath=`kecho -tb DATAMANIP -echo path`
dpath="$tbpath/testsuite/kflip/test4.inputs"
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

# ------------------------------------------------------------------------
#  This testsuite tests all possible combinations of kflip and 
#  operates on integer data only since other testsuites test
#  the gamut of datatypes.  Specifically, this testsuite tests to
#  ensure that if the attribute KPDS_KERNEL_ORIGIN was originally
#  set, that the kernel origin is flipped as well.
#
#  W   H   D   T   E
#  0   0   0   0   1     ---->  Flipped along e
#  0   0   0   1   0     ---->  Flipped along t
#  0   0   0   1   1     ---->  Flipped along t, e
#  0   0   1   0   0     ---->  Flipped along d
#  0   0   1   0   1     ---->  Flipped along d, e
#  0   0   1   1   0     ---->  Flipped along d, t
#  0   0   1   1   1     ---->  Flipped along d, t, e
#  0   1   0   0   0     ---->  Flipped along h
#  0   1   0   0   1     ---->  Flipped along h, e
#  0   1   0   1   0     ---->  Flipped along h, t
#  0   1   0   1   1     ---->  Flipped along h, t, e
#  0   1   1   0   0     ---->  Flipped along h, d
#  0   1   1   0   1     ---->  Flipped along h, d, e
#  0   1   1   1   0     ---->  Flipped along h, d, t
#  0   1   1   1   1     ---->  Flipped along h, d, t, e
#  1   0   0   0   0     ---->  Flipped along w
#  1   0   0   0   1     ---->  Flipped along w, e
#  1   0   0   1   0     ---->  Flipped along w, t
#  1   0   0   1   1     ---->  Flipped along w, t, e
#  1   0   1   0   0     ---->  Flipped along w, d
#  1   0   1   0   1     ---->  Flipped along w, d, e
#  1   0   1   1   0     ---->  Flipped along w, d, t
#  1   0   1   1   1     ---->  Flipped along w, d, t, e
#  1   1   0   0   0     ---->  Flipped along w, h
#  1   1   0   0   1     ---->  Flipped along w, h, e
#  1   1   0   1   0     ---->  Flipped along w, h, t
#  1   1   0   1   1     ---->  Flipped along w, h, t, e
#  1   1   1   0   0     ---->  Flipped along w, h, d
#  1   1   1   0   1     ---->  Flipped along w, h, d, e
#  1   1   1   1   0     ---->  Flipped along w, h, d, t
#  1   1   1   1   1     ---->  Flipped along w, h, d, t, e
# ------------------------------------------------------------------------

# ------------------------------------------------------------------------
# Test  51:  5D int value data flipped along elements only with kernel origin
# ------------------------------------------------------------------------
test_num=51
descr="5D int value data flipped along elements only with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -e

kimportasc -i1 $dpath/test_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_e.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_e.$$ -o $tmpdir/test_e_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 0

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_e_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_e.$$ $tmpdir/test_e_ko.$$ 

# ------------------------------------------------------------------------
# Test  52:  5D int value data flipped along time only with kernel origin
# ------------------------------------------------------------------------
test_num=52
descr="5D int value data flipped along time only with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -t

kimportasc -i1 $dpath/test_t.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_t.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_t.$$ -o $tmpdir/test_t_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 2 -ekernel 2

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_t_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_t.$$ $tmpdir/test_t_ko.$$ 

# ------------------------------------------------------------------------
# Test  53:  5D int value data flipped along time & elements with kernel origin
# ------------------------------------------------------------------------
test_num=53
descr="5D int value data flipped along time & elements with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -t -e

kimportasc -i1 $dpath/test_t_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_t_e.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_t_e.$$ -o $tmpdir/test_t_e_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 2 -ekernel 0

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_t_e_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_t_e.$$ $tmpdir/test_t_e_ko.$$ 

# ------------------------------------------------------------------------
# Test  54:  5D int value data flipped along depth only with kernel origin
# ------------------------------------------------------------------------
test_num=54
descr="5D int value data flipped along depth only with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -d

kimportasc -i1 $dpath/test_d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_d.$$ -o $tmpdir/test_d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 2 -tkernel 0 -ekernel 2

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_d_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_d.$$ $tmpdir/test_d_ko.$$ 

# ------------------------------------------------------------------------
# Test  55:  5D int value data flipped along depth & elements with kernel origin
# ------------------------------------------------------------------------
test_num=55
descr="5D int value data flipped along depth & elements with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -d -e

kimportasc -i1 $dpath/test_d_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_d_e.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_d_e.$$ -o $tmpdir/test_d_e_ko.$$ -wkernel 0 -hkernel 2 -dkernel 2 -tkernel 0 -ekernel 0

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_d_e_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_d_e.$$ $tmpdir/test_d_e_ko.$$ 

# ------------------------------------------------------------------------
# Test  56:  5D int value data flipped along depth & time with kernel origin
# ------------------------------------------------------------------------
test_num=56
descr="5D int value data flipped along depth & time with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -d -t

kimportasc -i1 $dpath/test_d_t.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_d_t.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_d_t.$$ -o $tmpdir/test_d_t_ko.$$ -wkernel 0 -hkernel 2 -dkernel 2 -tkernel 2 -ekernel 2

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_d_t_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_d_t.$$ $tmpdir/test_d_t_ko.$$ 

# ------------------------------------------------------------------------
# Test  57:  5D int value data flipped along depth, time, & elements with 
#            kernel origin
# ------------------------------------------------------------------------
test_num=57
descr="5D int value data flipped along depth, time, & elements with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -d -t -e

kimportasc -i1 $dpath/test_d_t_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_d_t_e.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_d_t_e.$$ -o $tmpdir/test_d_t_e_ko.$$ -wkernel 0 -hkernel 2 -dkernel 2 -tkernel 2 -ekernel 0

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_d_t_e_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_d_t_e.$$ $tmpdir/test_d_t_e_ko.$$ 

# ------------------------------------------------------------------------
# Test  58:  5D int value data flipped along height only with 
#            kernel origin
# ------------------------------------------------------------------------
test_num=58
descr="5D int value data flipped along height only with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -h

kimportasc -i1 $dpath/test_h.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_h.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_h.$$ -o $tmpdir/test_h_ko.$$ -wkernel 2 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_h_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_h.$$ $tmpdir/test_h_ko.$$ 

# ------------------------------------------------------------------------
# Test  59:  5D int value data flipped along height & elements with 
#            kernel origin
# ------------------------------------------------------------------------
test_num=59
descr="5D int value data flipped along height & elements with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -h -e

kimportasc -i1 $dpath/test_h_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_h_e.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_h_e.$$ -o $tmpdir/test_h_e_ko.$$ -wkernel 2 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 0

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_h_e_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_h_e.$$ $tmpdir/test_h_e_ko.$$ 

# ------------------------------------------------------------------------
# Test  60:  5D int value data flipped along height & time with 
#            kernel origin
# ------------------------------------------------------------------------
test_num=60
descr="5D int value data flipped along height & time with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -h -t

kimportasc -i1 $dpath/test_h_t.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_h_t.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_h_t.$$ -o $tmpdir/test_h_t_ko.$$ -wkernel 2 -hkernel 2 -dkernel 0 -tkernel 2 -ekernel 2

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_h_t_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_h_t.$$ $tmpdir/test_h_t_ko.$$ 

# ------------------------------------------------------------------------
# Test  61:  5D int value data flipped along height, time, & elements with 
#            kernel origin
# ------------------------------------------------------------------------
test_num=61
descr="5D int value data flipped along height, time, & elements with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -h -t -e

kimportasc -i1 $dpath/test_h_t_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_h_t_e.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_h_t_e.$$ -o $tmpdir/test_h_t_e_ko.$$ -wkernel 2 -hkernel 2 -dkernel 0 -tkernel 2 -ekernel 0

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_h_t_e_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_h_t_e.$$ $tmpdir/test_h_t_e_ko.$$ 

# ------------------------------------------------------------------------
# Test  62:  5D int value data flipped along height & depth with 
#            kernel origin
# ------------------------------------------------------------------------
test_num=62
descr="5D int value data flipped along height & depth with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -h -d

kimportasc -i1 $dpath/test_h_d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_h_d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_h_d.$$ -o $tmpdir/test_h_d_ko.$$ -wkernel 2 -hkernel 2 -dkernel 2 -tkernel 0 -ekernel 2

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_h_d_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_h_d.$$ $tmpdir/test_h_d_ko.$$ 

# ------------------------------------------------------------------------
# Test  63:  5D int value data flipped along height, depth, & elements with 
#            kernel origin
# ------------------------------------------------------------------------
test_num=63
descr="5D int value data flipped along height, depth, & elements with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -h -d -e

kimportasc -i1 $dpath/test_h_d_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_h_d_e.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_h_d_e.$$ -o $tmpdir/test_h_d_e_ko.$$ -wkernel 2 -hkernel 2 -dkernel 2 -tkernel 0 -ekernel 0

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_h_d_e_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_h_d_e.$$ $tmpdir/test_h_d_e_ko.$$ 

# ------------------------------------------------------------------------
# Test  64:  5D int value data flipped along height, depth, & time with 
#            kernel origin
# ------------------------------------------------------------------------
test_num=64
descr="5D int value data flipped along height, depth, & time with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -h -d -t

kimportasc -i1 $dpath/test_h_d_t.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_h_d_t.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_h_d_t.$$ -o $tmpdir/test_h_d_t_ko.$$ -wkernel 2 -hkernel 2 -dkernel 2 -tkernel 2 -ekernel 2

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_h_d_t_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_h_d_t.$$ $tmpdir/test_h_d_t_ko.$$ 

# ------------------------------------------------------------------------
# Test  65:  5D int value data flipped along height, depth, time, & elements 
#            with kernel origin
# ------------------------------------------------------------------------
test_num=65
descr="5D int value data flipped along height, depth, time, & elements with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -h -d -t -e

kimportasc -i1 $dpath/test_h_d_t_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_h_d_t_e.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_h_d_t_e.$$ -o $tmpdir/test_h_d_t_e_ko.$$ -wkernel 2 -hkernel 2 -dkernel 2 -tkernel 2 -ekernel 0

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_h_d_t_e_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_h_d_t_e.$$ $tmpdir/test_h_d_t_e_ko.$$ 

# ------------------------------------------------------------------------
# Test  66:  5D int value data flipped along width only with kernel origin
# ------------------------------------------------------------------------
test_num=66
descr="5D int value data flipped along width only with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -w

kimportasc -i1 $dpath/test_w.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_w.$$ -o $tmpdir/test_w_ko.$$ -wkernel 0 -hkernel 0 -dkernel 0 -tkernel 0 -ekernel 2

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_w_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_w.$$ $tmpdir/test_w_ko.$$ 

# ------------------------------------------------------------------------
# Test  67:  5D int value data flipped along width & elements with kernel origin
# ------------------------------------------------------------------------
test_num=67
descr="5D int value data flipped along width & elements with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -w -e

kimportasc -i1 $dpath/test_w_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_e.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_w_e.$$ -o $tmpdir/test_w_e_ko.$$ -wkernel 0 -hkernel 0 -dkernel 0 -tkernel 0 -ekernel 0

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_w_e_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_w_e.$$ $tmpdir/test_w_e_ko.$$ 

# ------------------------------------------------------------------------
# Test  68:  5D int value data flipped along width & time with kernel origin
# ------------------------------------------------------------------------
test_num=68
descr="5D int value data flipped along width & time with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -w -t

kimportasc -i1 $dpath/test_w_t.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_t.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_w_t.$$ -o $tmpdir/test_w_t_ko.$$ -wkernel 0 -hkernel 0 -dkernel 0 -tkernel 2 -ekernel 2

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_w_t_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_w_t.$$ $tmpdir/test_w_t_ko.$$ 

# ------------------------------------------------------------------------
# Test  69:  5D int value data flipped along width, time, & elements with 
#            kernel origin
# ------------------------------------------------------------------------
test_num=69
descr="5D int value data flipped along width, time, & elements  with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -w -t -e

kimportasc -i1 $dpath/test_w_t_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_t_e.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_w_t_e.$$ -o $tmpdir/test_w_t_e_ko.$$ -wkernel 0 -hkernel 0 -dkernel 0 -tkernel 2 -ekernel 0

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_w_t_e_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_w_t_e.$$ $tmpdir/test_w_t_e_ko.$$ 

# ------------------------------------------------------------------------
# Test  70:  5D int value data flipped along width & depth with 
#            kernel origin
# ------------------------------------------------------------------------
test_num=70
descr="5D int value data flipped along width & depth with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -w -d

kimportasc -i1 $dpath/test_w_d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_w_d.$$ -o $tmpdir/test_w_d_ko.$$ -wkernel 0 -hkernel 0 -dkernel 2 -tkernel 0 -ekernel 2

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_w_d_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_w_d.$$ $tmpdir/test_w_d_ko.$$ 

# ------------------------------------------------------------------------
# Test  71:  5D int value data flipped along width, depth, & elements  with 
#            kernel origin
# ------------------------------------------------------------------------
test_num=71
descr="5D int value data flipped along width, depth, & elements with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -w -d -e

kimportasc -i1 $dpath/test_w_d_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_d_e.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_w_d_e.$$ -o $tmpdir/test_w_d_e_ko.$$ -wkernel 0 -hkernel 0 -dkernel 2 -tkernel 0 -ekernel 0

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_w_d_e_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_w_d_e.$$ $tmpdir/test_w_d_e_ko.$$ 

# ------------------------------------------------------------------------
# Test  72:  5D int value data flipped along width, depth, & time with 
#            kernel origin
# ------------------------------------------------------------------------
test_num=72
descr="5D int value data flipped along width, depth, & time with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -w -d -t

kimportasc -i1 $dpath/test_w_d_t.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_d_t.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_w_d_t.$$ -o $tmpdir/test_w_d_t_ko.$$ -wkernel 0 -hkernel 0 -dkernel 2 -tkernel 2 -ekernel 2

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_w_d_t_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_w_d_t.$$ $tmpdir/test_w_d_t_ko.$$ 

# ------------------------------------------------------------------------
# Test  73:  5D int value data flipped along width, depth, time, & elements 
#	     with kernel origin
# ------------------------------------------------------------------------
test_num=73
descr="5D int value data flipped along width, depth, time, & elements with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -w -d -t -e

kimportasc -i1 $dpath/test_w_d_t_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_d_t_e.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_w_d_t_e.$$ -o $tmpdir/test_w_d_t_e_ko.$$ -wkernel 0 -hkernel 0 -dkernel 2 -tkernel 2 -ekernel 0

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_w_d_t_e_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_w_d_t_e.$$ $tmpdir/test_w_d_t_e_ko.$$ 


# ------------------------------------------------------------------------
# Test  74:  5D int value data flipped along width & height
#	     with kernel origin
# ------------------------------------------------------------------------
test_num=74
descr="5D int value data flipped along width & height with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -w -h

kimportasc -i1 $dpath/test_w_h.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_h.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_w_h.$$ -o $tmpdir/test_w_h_ko.$$ -wkernel 2 -hkernel 0 -dkernel 0 -tkernel 0 -ekernel 2

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_w_h_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_w_h.$$ $tmpdir/test_w_h_ko.$$ 

# ------------------------------------------------------------------------
# Test  75:  5D int value data flipped along width, height, & elements
#	     with kernel origin
# ------------------------------------------------------------------------
test_num=75
descr="5D int value data flipped along width, height, & elements with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -w -h -e

kimportasc -i1 $dpath/test_w_h_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_h_e.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_w_h_e.$$ -o $tmpdir/test_w_h_e_ko.$$ -wkernel 2 -hkernel 0 -dkernel 0 -tkernel 0 -ekernel 0

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_w_h_e_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_w_h_e.$$ $tmpdir/test_w_h_e_ko.$$ 

# ------------------------------------------------------------------------
# Test  76:  5D int value data flipped along width, height, & time
#	     with kernel origin
# ------------------------------------------------------------------------
test_num=76
descr="5D int value data flipped along width, height, & time with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -w -h -t

kimportasc -i1 $dpath/test_w_h_t.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_h_t.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_w_h_t.$$ -o $tmpdir/test_w_h_t_ko.$$ -wkernel 2 -hkernel 0 -dkernel 0 -tkernel 2 -ekernel 2

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_w_h_t_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_w_h_t.$$ $tmpdir/test_w_h_t_ko.$$ 

# ------------------------------------------------------------------------
# Test  77:  5D int value data flipped along width, height, time, & elements
#	     with kernel origin
# ------------------------------------------------------------------------
test_num=77
descr="5D int value data flipped along width, height, time, & elements with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -w -h -t -e

kimportasc -i1 $dpath/test_w_h_t_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_h_t_e.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_w_h_t_e.$$ -o $tmpdir/test_w_h_t_e_ko.$$ -wkernel 2 -hkernel 0 -dkernel 0 -tkernel 2 -ekernel 0

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_w_h_t_e_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_w_h_t_e.$$ $tmpdir/test_w_h_t_e_ko.$$ 

# ------------------------------------------------------------------------
# Test  78:  5D int value data flipped along width, height, & depth
#	     with kernel origin
# ------------------------------------------------------------------------
test_num=78
descr="5D int value data flipped along width, height, & depth with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -w -h -d

kimportasc -i1 $dpath/test_w_h_d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_h_d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_w_h_d.$$ -o $tmpdir/test_w_h_d_ko.$$ -wkernel 2 -hkernel 0 -dkernel 2 -tkernel 0 -ekernel 2

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_w_h_d_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_w_h_d.$$ $tmpdir/test_w_h_d_ko.$$ 

# ------------------------------------------------------------------------
# Test  79:  5D int value data flipped along width, height, depth, & elements
#	     with kernel origin
# ------------------------------------------------------------------------
test_num=79
descr="5D int value data flipped along width, height, depth, & elements with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -w -h -d -e

kimportasc -i1 $dpath/test_w_h_d_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_h_d_e.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_w_h_d_e.$$ -o $tmpdir/test_w_h_d_e_ko.$$ -wkernel 2 -hkernel 0 -dkernel 2 -tkernel 0 -ekernel 0

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_w_h_d_e_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_w_h_d_e.$$ $tmpdir/test_w_h_d_e_ko.$$ 

# ------------------------------------------------------------------------
# Test  80:  5D int value data flipped along width, height, depth, & time
#	     with kernel origin
# ------------------------------------------------------------------------
test_num=80
descr="5D int value data flipped along width, height, depth, & time with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -w -h -d -t

kimportasc -i1 $dpath/test_w_h_d_t.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_h_d_t.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_w_h_d_t.$$ -o $tmpdir/test_w_h_d_t_ko.$$ -wkernel 2 -hkernel 0 -dkernel 2 -tkernel 2 -ekernel 2

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_w_h_d_t_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_w_h_d_t.$$ $tmpdir/test_w_h_d_t_ko.$$ 

# ------------------------------------------------------------------------
# Test  81:  5D int value data flipped along width, height, depth, time,
#	     & elements with kernel origin
# ------------------------------------------------------------------------
test_num=81
descr="5D int value data flipped along width, height, depth, time, & elements with kernel origin"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/input_5d.$$ -o $tmpdir/input_5d_ko.$$ -wkernel 0 -hkernel 2 -dkernel 0 -tkernel 0 -ekernel 2

${BINARY_PATH}/kflip -i $tmpdir/input_5d_ko.$$ -o $tmpdir/kflip_out_ko.$$ -w -h -d -t -e

kimportasc -i1 $dpath/test_w_h_d_t_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_h_d_t_e.$$

${BINARY_PATH}/ksetdattr -i $tmpdir/test_w_h_d_t_e.$$ -o $tmpdir/test_w_h_d_t_e_ko.$$ -wkernel 2 -hkernel 0 -dkernel 2 -tkernel 2 -ekernel 0

report $test_num "$tmpdir/kflip_out_ko.$$" "$tmpdir/test_w_h_d_t_e_ko.$$" "$descr"

/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/input_5d_ko.$$ $tmpdir/kflip_out_ko.$$ 
/bin/rm -f $tmpdir/test_w_h_d_t_e.$$ $tmpdir/test_w_h_d_t_e_ko.$$ 

exit $status
