#! /bin/sh
 
echo "running test1.sh for klinearop (2D hotspot testing) ... process ($$)"
echo " "
 
# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"
 
ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-4 -sat 1 -rt 1 -all -s
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}
 
#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0
 
# ------------------------------------------------------------------------
#  set path for test data, and path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
bpath=`kecho -tb DATAMANIP -echo binpath`
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/"

# ------------------------------------------------------------------------
# Test  1: correlation, Double 2D - WxH, (0,0) hotspot 
# ------------------------------------------------------------------------
test_num=1
descr="correlation, Double 2D - WxH, (0,0) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 1 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
4 5 6
7 8 9
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -l -real 0 -imag 0

cat > $tmpdir/out2.$$ << EOF
0 0 0 0 0 0 0 0
0 9 8 7 0 0 0 0
0 6 5 4 0 0 0 0
0 3 2 1 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  2: convolution, Double 2D - WxH, (0,0) hotspot
# ------------------------------------------------------------------------
test_num=2
descr="convolution, Double 2D - WxH, (0,0) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 1 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
4 5 6
7 8 9
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0

cat > $tmpdir/out2.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 1 2 3 0 0
0 0 0 4 5 6 0 0
0 0 0 7 8 9 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  3: correlation, Double 2D - WxH, (0,0) hotspot, mean removed
# ------------------------------------------------------------------------
test_num=3
descr="correlation, Double 2D - WxH, (0,0) hotspot, mean removed"

cat > $tmpdir/imp.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 -4 -3 -2 0 0
0 0 0 -1 0 1 0 0
0 0 0 2 3 4 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
-4 -3 -2
-1 0 1
2 3 4
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -l -real 0 -imag 0 

cat > $tmpdir/out2.$$ << EOF
         0   0   0   0   0   0  0  0
         0 -16 -24 -25 -12  -4  0  0
         0  -8  -6   4   6   4  0  0
         0  15  36  60  36  15  0  0
         0   4   6   4  -6  -8  0  0
         0  -4 -12 -25 -24 -16  0  0
         0   0   0   0   0   0  0  0
         0   0   0   0   0   0  0  0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  4: correlation, Double 2D - WxH, (1,1) hotspot 
# ------------------------------------------------------------------------
test_num=4
descr="correlation, Double 2D - WxH, (1,1) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 1 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
4 5 6
7 8 9
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -l -real 0 -imag 0 -wc 1 -hc 1

cat > $tmpdir/out2.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 9 8 7 0 0 0
0 0 6 5 4 0 0 0
0 0 3 2 1 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  5: correlation, Double 2D - WxH, (2,0) hotspot 
# ------------------------------------------------------------------------
test_num=5
descr="correlation, Double 2D - WxH, (2,0) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 1 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
4 5 6
7 8 9
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -l -real 0 -imag 0 -wc 2 

cat > $tmpdir/out2.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 9 8 7 0 0
0 0 0 6 5 4 0 0
0 0 0 3 2 1 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$

# ------------------------------------------------------------------------
# Test  6: correlation, Double 2D - WxH, (0,2) hotspot 
# ------------------------------------------------------------------------
test_num=6
descr="correlation, Double 2D - WxH, (0,2) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 1 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
4 5 6
7 8 9
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -l -real 0 -imag 0 -hc 2

cat > $tmpdir/out2.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 9 8 7 0 0 0 0
0 6 5 4 0 0 0 0
0 3 2 1 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$

# ------------------------------------------------------------------------
# Test  7: convolution, Double 2D - WxH, (1,1) hotspot
# ------------------------------------------------------------------------
test_num=7
descr="convolution, Double 2D - WxH, (1,1) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 1 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
4 5 6
7 8 9
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -wc 1 -hc 1

cat > $tmpdir/out2.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 1 2 3 0 0 0
0 0 4 5 6 0 0 0
0 0 7 8 9 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  8: convolution, Double 2D - WxH, (2,0) hotspot
# ------------------------------------------------------------------------
test_num=8
descr="convolution, Double 2D - WxH, (2,0) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 1 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
4 5 6
7 8 9
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -wc 2

cat > $tmpdir/out2.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 1 2 3 0 0 0 0
0 4 5 6 0 0 0 0
0 7 8 9 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  9: convolution, Double 2D - WxH, (0,2) hotspot
# ------------------------------------------------------------------------
test_num=9
descr="convolution, Double 2D - WxH, (0,2) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 1 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
4 5 6
7 8 9
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -hc 2

cat > $tmpdir/out2.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 1 2 3 0 0
0 0 0 4 5 6 0 0
0 0 0 7 8 9 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

exit $status
