#! /bin/sh 
 
echo "running test3.sh for klinearop (1D testing on all 5 axes) ... process ($$)"
echo " "
 
# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"
 
ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-4 -sat 1 -rt 1 -all -s
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}
 
#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0
 
# ------------------------------------------------------------------------
#  set path for test data, and path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
bpath=`kecho -tb DATAMANIP -echo binpath`
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/"

# ------------------------------------------------------------------------
# Test  1: correlation, Double 1D - W, (0) hotspot 
# ------------------------------------------------------------------------
test_num=1
descr="correlation, Double 1D - W, (0) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -l -real 0 -imag 0

cat > $tmpdir/out2.$$ << EOF
0 3 2 1 0 0 0 0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  2: convolution, Double 1D - W, (0) hotspot
# ------------------------------------------------------------------------
test_num=2
descr="convolution, Double 1D - W, (0) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0

cat > $tmpdir/out2.$$ << EOF
0 0 0 1 2 3 0 0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  3: correlation, Double 1D - H, (0) hotspot 
# ------------------------------------------------------------------------
test_num=3
descr="correlation, Double 1D - H, (0) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0
0
0
1
0
0
0
0
EOF

cat > $tmpdir/patt.$$ << EOF
1
2
3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -l -real 0 -imag 0

cat > $tmpdir/out2.$$ << EOF
0
3
2
1
0
0
0
0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  4: convolution, Double 1D - H, (0) hotspot
# ------------------------------------------------------------------------
test_num=4
descr="convolution, Double 1D - H, (0) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0
0
0
1
0
0
0
0
EOF

cat > $tmpdir/patt.$$ << EOF
1
2
3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0

cat > $tmpdir/out2.$$ << EOF
0
0
0
1
2
3
0
0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  5: correlation, Double 1D - D, (1) hotspot 
# ------------------------------------------------------------------------
test_num=5
descr="correlation, Double 1D - D, (1) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF
korient -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -wd -hh -dw -tt -ee

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF
korient -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -wd -hh -dw -tt -ee

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -l -real 0 -imag 0 -dc 1

cat > $tmpdir/out2.$$ << EOF
0 0 3 2 1 0 0 0
EOF
korient -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -wd -hh -dw -tt -ee

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  6: correlation, Double 1D - D, (2) hotspot 
# ------------------------------------------------------------------------
test_num=6
descr="correlation, Double 1D - D, (2) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF
korient -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -wd -hh -dw -tt -ee

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF
korient -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -wd -hh -dw -tt -ee

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -l -real 0 -imag 0 -dc 2

cat > $tmpdir/out2.$$ << EOF
0 0 0 3 2 1 0 0
EOF
korient -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -wd -hh -dw -tt -ee

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$

# ------------------------------------------------------------------------
# Test  7: convolution, Double 1D - D, (1) hotspot
# ------------------------------------------------------------------------
test_num=7
descr="convolution, Double 1D - D, (1) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF
korient -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -wd -hh -dw -tt -ee

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF
korient -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -wd -hh -dw -tt -ee

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -dc 1

cat > $tmpdir/out2.$$ << EOF
0 0 1 2 3 0 0 0
EOF
korient -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -wd -hh -dw -tt -ee

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  8: convolution, Double 1D - D, (2) hotspot
# ------------------------------------------------------------------------
test_num=8
descr="convolution, Double 1D - D, (2) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF
korient -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -wd -hh -dw -tt -ee

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF
korient -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -wd -hh -dw -tt -ee

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -dc 2

cat > $tmpdir/out2.$$ << EOF
0 1 2 3 0 0 0 0
EOF
korient -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -wd -hh -dw -tt -ee

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  9: correlation, Double 1D - T, (1) hotspot 
# ------------------------------------------------------------------------
test_num=9
descr="correlation, Double 1D - T, (1) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF
korient -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -wt -hh -dd -tw -ee

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF
korient -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -wt -hh -dd -tw -ee

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -l -real 0 -imag 0 -tc 1

cat > $tmpdir/out2.$$ << EOF
0 0 3 2 1 0 0 0
EOF
korient -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -wt -hh -dd -tw -ee

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test 10: correlation, Double 1D - T, (2) hotspot 
# ------------------------------------------------------------------------
test_num=10
descr="correlation, Double 1D - T, (2) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF
korient -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -wt -hh -dd -tw -ee

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF
korient -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -wt -hh -dd -tw -ee

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -l -real 0 -imag 0 -tc 2

cat > $tmpdir/out2.$$ << EOF
0 0 0 3 2 1 0 0
EOF
korient -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -wt -hh -dd -tw -ee

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$

# ------------------------------------------------------------------------
# Test 11: convolution, Double 1D - T, (1) hotspot
# ------------------------------------------------------------------------
test_num=11
descr="convolution, Double 1D - T, (1) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF
korient -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -wt -hh -dd -tw -ee

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF
korient -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -wt -hh -dd -tw -ee

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -tc 1

cat > $tmpdir/out2.$$ << EOF
0 0 1 2 3 0 0 0
EOF
korient -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -wt -hh -dd -tw -ee

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test 12: convolution, Double 1D - T, (2) hotspot
# ------------------------------------------------------------------------
test_num=12
descr="convolution, Double 1D - T, (2) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF
korient -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -wt -hh -dd -tw -ee

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF
korient -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -wt -hh -dd -tw -ee

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -tc 2

cat > $tmpdir/out2.$$ << EOF
0 1 2 3 0 0 0 0
EOF
korient -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -wt -hh -dd -tw -ee

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test 13: correlation, Double 1D - E, (1) hotspot 
# ------------------------------------------------------------------------
test_num=13
descr="correlation, Double 1D - E, (1) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF
korient -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -we -hh -dd -tt -ew

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF
korient -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -we -hh -dd -tt -ew

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -l -real 0 -imag 0 -ec 1

cat > $tmpdir/out2.$$ << EOF
0 0 3 2 1 0 0 0
EOF
korient -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -we -hh -dd -tt -ew

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test 14: correlation, Double 1D - E, (2) hotspot 
# ------------------------------------------------------------------------
test_num=14
descr="correlation, Double 1D - E, (2) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF
korient -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -we -hh -dd -tt -ew

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF
korient -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -we -hh -dd -tt -ew

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -l -real 0 -imag 0 -ec 2

cat > $tmpdir/out2.$$ << EOF
0 0 0 3 2 1 0 0
EOF
korient -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -we -hh -dd -tt -ew

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$

# ------------------------------------------------------------------------
# Test 15: convolution, Double 1D - E, (1) hotspot
# ------------------------------------------------------------------------
test_num=15
descr="convolution, Double 1D - E, (1) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF
korient -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -we -hh -dd -tt -ew

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF
korient -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -we -hh -dd -tt -ew

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -ec 1

cat > $tmpdir/out2.$$ << EOF
0 0 1 2 3 0 0 0
EOF
korient -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -we -hh -dd -tt -ew

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test 16: convolution, Double 1D - E, (2) hotspot
# ------------------------------------------------------------------------
test_num=16
descr="convolution, Double 1D - E, (2) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF
korient -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -we -hh -dd -tt -ew

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF
korient -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -we -hh -dd -tt -ew

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -ec 2

cat > $tmpdir/out2.$$ << EOF
0 1 2 3 0 0 0 0
EOF
korient -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -we -hh -dd -tt -ew

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

exit $status
