#! /bin/sh 
 
echo "running test5.sh for klinearop (multidimensional repetition testing) ... process ($$)"
echo " "
 
# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"
 
ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-4 -sat 1 -rt 1 -all -s
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}
 
#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0
 
# ------------------------------------------------------------------------
#  set path for test data, and path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
bpath=`kecho -tb DATAMANIP -echo binpath`
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/"

# ------------------------------------------------------------------------
# Test  1: convolution, Double 2D - WxH, W kernel, (1) hotspot
# ------------------------------------------------------------------------
test_num=1
descr="convolution, Double 2D - WxH, W kernel, (1) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
0 0 0 2 0 0 0 0
0 0 0 3 0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -wc 1 

cat > $tmpdir/out2.$$ << EOF
0 0 1 2 3 0 0 0
0 0 2 4 6 0 0 0
0 0 3 6 9 0 0 0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  2: convolution, Double 2D - WxH, H kernel, (1) hotspot
# ------------------------------------------------------------------------
test_num=2
descr="convolution, Double 2D - WxH, H kernel, (1) hotspot" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 0 0 0 0 0
1 2 3 1 2 3 1 2
0 0 0 0 0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
1
2
3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -hc 1

cat > $tmpdir/out2.$$ << EOF
1 2 3 1 2 3 1 2
2 4 6 2 4 6 2 4
3 6 9 3 6 9 3 6
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  3: convolution, Double 3D - WxHxD, WxH kernel, (1,1) hotspot
# ------------------------------------------------------------------------
test_num=3
descr="convolution, Double 3D - WxHxD, WxH kernel, (1,1) hotspot" 

cat > $tmpdir/imp.$$ << EOF
1 2 3
EOF
korient -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -wd -hh -dw -tt -ee
kgconst -o $tmpdir/in.$$ -wsize 7 -hsize 7 -dsize 3 -type 11
kinset -i1 $tmpdir/in.$$ -i2 $tmpdir/imp.$$ -o $tmpdir/in.$$ -woff 3 -hoff 3

cat > $tmpdir/patt.$$ << EOF
1 2 3
4 5 6
7 8 9
EOF

$bpath/klinearop -i1 $tmpdir/in.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -wc 1 -hc 1

cat > $tmpdir/out2.$$ << EOF
0 0 0 0 0 0 0
0 0 0 0 0 0 0
0 0 1 2 3 0 0
0 0 4 5 6 0 0
0 0 7 8 9 0 0
0 0 0 0 0 0 0
0 0 0 0 0 0 0
EOF
kinset -i1 $tmpdir/in.$$ -i2 $tmpdir/out2.$$ -o $tmpdir/out3.$$ \
-woff 0 -hoff 0 -doff 0
kmul -i1 $tmpdir/out2.$$ -o $tmpdir/in.$$ -real 2
kinset -i1 $tmpdir/out3.$$ -i2 $tmpdir/in.$$ -o $tmpdir/out3.$$ \
-woff 0 -hoff 0 -doff 1
kmul -i1 $tmpdir/out2.$$ -o $tmpdir/in.$$ -real 3
kinset -i1 $tmpdir/out3.$$ -i2 $tmpdir/in.$$ -o $tmpdir/out3.$$ \
-woff 0 -hoff 0 -doff 2

report $test_num "$tmpdir/out.$$" "$tmpdir/out3.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$
/bin/rm -f $tmpdir/in.$$ $tmpdir/out3.$$

# ------------------------------------------------------------------------
# Test  4: convolution, Double 3D - WxHxD, W kernel, (1) hotspot
# ------------------------------------------------------------------------
test_num=4
descr="convolution, Double 3D - WxHxD, W kernel, (1) hotspot" 

cat > $tmpdir/imp.$$ << EOF
1 2 3
EOF
korient -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -wd -hh -dw -tt -ee
kgconst -o $tmpdir/in.$$ -wsize 7 -hsize 7 -dsize 3 -type 11
kinset -i1 $tmpdir/in.$$ -i2 $tmpdir/imp.$$ -o $tmpdir/in.$$ -woff 3 -hoff 3

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/in.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -wc 1

cat > $tmpdir/out2.$$ << EOF
0 0 0 0 0 0 0
0 0 0 0 0 0 0
0 0 0 0 0 0 0
0 0 1 2 3 0 0
0 0 0 0 0 0 0
0 0 0 0 0 0 0
0 0 0 0 0 0 0
EOF
kinset -i1 $tmpdir/in.$$ -i2 $tmpdir/out2.$$ -o $tmpdir/out3.$$ \
-woff 0 -hoff 0 -doff 0
kmul -i1 $tmpdir/out2.$$ -o $tmpdir/in.$$ -real 2
kinset -i1 $tmpdir/out3.$$ -i2 $tmpdir/in.$$ -o $tmpdir/out3.$$ \
-woff 0 -hoff 0 -doff 1
kmul -i1 $tmpdir/out2.$$ -o $tmpdir/in.$$ -real 3
kinset -i1 $tmpdir/out3.$$ -i2 $tmpdir/in.$$ -o $tmpdir/out3.$$ \
-woff 0 -hoff 0 -doff 2

report $test_num "$tmpdir/out.$$" "$tmpdir/out3.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$
/bin/rm -f $tmpdir/in.$$ $tmpdir/out3.$$

exit $status
