#! /bin/sh
 
echo "running test6.sh for klinearop (map behavior testing) ... process ($$)"
echo " "
 
# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"
 
ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-4 -sat 1 -rt 1 -all -s
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}
 
#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0
 
# ------------------------------------------------------------------------
#  set path for test data, and path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
bpath=`kecho -tb DATAMANIP -echo binpath`
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath=`kecho -tb DATAMANIP -echo path`
tdpath="$dpath/data/"

# ------------------------------------------------------------------------
# Test  1: convolution, ubyte 2D - WxH, double (w=1) map, W kernel, (1) hotspot
# ------------------------------------------------------------------------
test_num=1
descr="convolution, ubyte 2D - WxH, double (w=1) map, W kernel, (1) hotspot" 

cat > $tmpdir/asc.$$ << EOF
0 0 0 1 0 0 0 0
0 0 0 2 0 0 0 0
0 0 0 3 0 0 0 0
EOF
kimportasc -i1 $tmpdir/asc.$$ -o $tmpdir/imp.$$ \
-wsize 8 -hsize 3 -dsize 1 -tsize 1 -esize 1 -type 3 \
-windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5
cat > $tmpdir/asc.$$ << EOF
0
1
2
3
EOF
kimportasc -i1 $tmpdir/asc.$$ -i2 $tmpdir/imp.$$ -o $tmpdir/imp.$$ \
-wsize 1 -hsize 4 -dsize 1 -tsize 1 -esize 1 -segment map -type 11 \
-windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -wc 1

cat > $tmpdir/out2.$$ << EOF
0 0 1 2 3 0 0 0
0 0 2 4 6 0 0 0
0 0 3 6 9 0 0 0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$
/bin/rm -f $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  2: convolution, ubyte 2D - WxH, double (w=3) map, W kernel, (1) hotspot
# ------------------------------------------------------------------------
test_num=2
descr="convolution, ubyte 2D - WxH, double (w=3) map, W kernel, (1) hotspot"

cat > $tmpdir/asc.$$ << EOF
0 0 0 1 0 0 0 0
0 0 0 2 0 0 0 0
0 0 0 3 0 0 0 0
EOF
kimportasc -i1 $tmpdir/asc.$$ -o $tmpdir/imp.$$ \
-wsize 8 -hsize 3 -dsize 1 -tsize 1 -esize 1 -type 3 \
-windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5
cat > $tmpdir/asc.$$ << EOF
0 0 0 
0 1 0
0 2 0
0 3 0
EOF
kimportasc -i1 $tmpdir/asc.$$ -i2 $tmpdir/imp.$$ -o $tmpdir/imp.$$ \
-wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -segment map -type 11 \
-windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -wc 1 

cat > $tmpdir/asc.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 1 2 3 0 0 0
0 0 2 4 6 0 0 0
0 0 3 6 9 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF
kimportasc -i1 $tmpdir/asc.$$ -o $tmpdir/out2.$$ \
-wsize 8 -hsize 3 -dsize 1 -tsize 1 -esize 3 -type 11 \
-windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$
/bin/rm -f $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  3: convolution, ubyte 2D - WxH, ubyte (w=3) map, W kernel, (1) hotspot
# ------------------------------------------------------------------------
test_num=3
descr="convolution, ubyte 2D - WxH, ubyte (w=3) map, W kernel, (1) hotspot"
 
cat > $tmpdir/asc.$$ << EOF
0 0 0 1 0 0 0 0
0 0 0 2 0 0 0 0
0 0 0 3 0 0 0 0
EOF
kimportasc -i1 $tmpdir/asc.$$ -o $tmpdir/imp.$$ \
-wsize 8 -hsize 3 -dsize 1 -tsize 1 -esize 1 -type 3 \
-windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5
cat > $tmpdir/asc.$$ << EOF
0 0 0
0 1 0
0 2 0
0 3 0
EOF
kimportasc -i1 $tmpdir/asc.$$ -i2 $tmpdir/imp.$$ -o $tmpdir/imp.$$ \
-wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -segment map -type 3 \
-windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5
 
cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF
 
$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -wc 1
 
cat > $tmpdir/asc.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 1 2 3 0 0 0
0 0 2 4 6 0 0 0
0 0 3 6 9 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF
kimportasc -i1 $tmpdir/asc.$$ -o $tmpdir/out2.$$ \
-wsize 8 -hsize 3 -dsize 1 -tsize 1 -esize 3 -type 3 \
-windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5
 
report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"
 
/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$
/bin/rm -f $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  4: convolution, ubyte 2D - WxH, ubyte (w=3) map, avg2x2 kernel
# ------------------------------------------------------------------------
test_num=4
descr="convolution, ubyte 2D - WxH, ubyte (w=3) map, avg2x2 kernel"
 
cat > $tmpdir/asc.$$ << EOF
0 0 0 1 0 0 0 0
0 0 0 2 0 0 0 0
0 0 0 3 0 0 0 0
EOF
kimportasc -i1 $tmpdir/asc.$$ -o $tmpdir/imp.$$ \
-wsize 8 -hsize 3 -dsize 1 -tsize 1 -esize 1 -type 3 \
-windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5
cat > $tmpdir/asc.$$ << EOF
0 0 0
0 1 0
0 2 0
0 3 0
EOF
kimportasc -i1 $tmpdir/asc.$$ -i2 $tmpdir/imp.$$ -o $tmpdir/imp.$$ \
-wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -segment map -type 3 \
-windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5
 
$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 kernel:avg2x2  -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -wc 1

cat > $tmpdir/asc.$$ << EOF
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0

0 0 0 0 0 0 0 0
0 0 1 1 0 0 0 0
0 0 2 2 0 0 0 0

0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
EOF
kimportasc -i1 $tmpdir/asc.$$ -o $tmpdir/out2.$$ \
-wsize 8 -hsize 3 -dsize 1 -tsize 1 -esize 3 -type 3 \
-windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5
 
report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"
 
/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$
/bin/rm -f $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  5: convolution, ubyte 2D - WxH, ubyte (w=3) map, avg2x2 kernel 
#          (special test) . This is a special test to make sure a specific
#          bug never returns. The input file was found by Roberto Lotufo
#          to cause data services to segfault under exercise by klinearop.
#          Note that this test is essentially identical to Test #4 above,
#          but that test PASSED!
# ------------------------------------------------------------------------
test_num=5
descr="convolution, ubyte 2D - WxH, ubyte (w=3) map, avg2x2 kernel (special test)"
 
$bpath/klinearop -i1 $tdpath/testdata/grabbed.kdf -i2 kernel:avg2x2 \
-o $tmpdir/out.$$ -corr 0 -l -real 0 -imag 0 -wc 1
 
report $test_num "$tmpdir/out.$$" "test_results/grabbed.result" "$descr"
 
/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$
/bin/rm -f $tmpdir/asc.$$

exit $status
