#! /bin/sh
 
echo "running test8.sh for klinearop (circular convolution) ... process ($$)"
echo " "
 
# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"
 
ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-4 -sat 1 -rt 1 -all -s
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}
 
#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0
 
# ------------------------------------------------------------------------
#  set path for test data, and path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
bpath=`kecho -tb DATAMANIP -echo binpath`
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/"

# ------------------------------------------------------------------------
# Test  1: circular correlation, Double 1D - W, (0) hotspot (general)
# ------------------------------------------------------------------------
test_num=1
descr="circular correlation, Double 1D - W, (0) hotspot (general)" 

cat > $tmpdir/imp.$$ << EOF
1 0 0 1 0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -c

cat > $tmpdir/out2.$$ << EOF
1 3 2 1 0 0 3 2
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  2: circular correlation, Double 1D - W, (0) hotspot (constant)
# ------------------------------------------------------------------------
test_num=2
descr="circular correlation, Double 1D - W, (0) hotspot (constant)"

cat > $tmpdir/imp.$$ << EOF
1 1 1 1 1 1 1 1
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -c

cat > $tmpdir/out2.$$ << EOF
6 6 6 6 6 6 6 6
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  3: circular correlation, Double 1D - W, (0) hotspot (end boundaries)
# ------------------------------------------------------------------------
test_num=3
descr="circular correlation, Double 1D - W, (0) hotspot (end boundaries)"

cat > $tmpdir/imp.$$ << EOF
1 0 0 0 0 0 0 1
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -c

cat > $tmpdir/out2.$$ << EOF
1 0 0 0 0 3 5 3
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  4: circular convolution, Double 1D - W, (0) hotspot (constant)
# ------------------------------------------------------------------------
test_num=4
descr="circular convolution, Double 1D - W, (0) hotspot (constant)"

cat > $tmpdir/imp.$$ << EOF
1 1 1 1 1 1 1 1
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -c

cat > $tmpdir/out2.$$ << EOF
6 6 6 6 6 6 6 6
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  5: circular convolution, Double 1D - W, (0) hotspot (start)
# ------------------------------------------------------------------------
test_num=5
descr="circular convolution, Double 1D - W, (0) hotspot (start)"

cat > $tmpdir/imp.$$ << EOF
1 0 0 0 0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -c

cat > $tmpdir/out2.$$ << EOF
1 2 3 0 0 0 0 0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

# ------------------------------------------------------------------------
# Test  6: circular convolution, Double 1D - W, (0) hotspot (end)
# ------------------------------------------------------------------------
test_num=6
descr="circular convolution, Double 1D - W, (0) hotspot (end)"

cat > $tmpdir/imp.$$ << EOF
0 0 0 0 0 0 0 1
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -c

cat > $tmpdir/out2.$$ << EOF
2 3 0 0 0 0 0 1
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

# ------------------------------------------------------------------------
# Test  7: circular convolution, Double 1D - W, (0) hotspot (general)
# ------------------------------------------------------------------------
test_num=7
descr="circular convolution, Double 1D - W, (0) hotspot (general)" 

cat > $tmpdir/imp.$$ << EOF
1 0 0 1 0 0 0 1
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -c

# 1 2 3 0 0 0 0 0
# 0 0 0 1 2 3 0 0
# 2 3 0 0 0 0 0 1
cat > $tmpdir/out2.$$ << EOF
3 5 3 1 2 3 0 1
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  8: circular correlation, Double 1D - W, (1) hotspot (general)
# ------------------------------------------------------------------------
test_num=8
descr="circular correlation, Double 1D - W, (1) hotspot (general)"

cat > $tmpdir/imp.$$ << EOF
1 0 0 1 0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -c -wc 1

#2 1 0 0 0 0 0 3
#0 0 3 2 1 0 0 0
cat > $tmpdir/out2.$$ << EOF
2 1 3 2 1 0 0 3
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  9: circular correlation, Double 1D - W, (2) hotspot (general)
# ------------------------------------------------------------------------
test_num=9
descr="circular correlation, Double 1D - W, (2) hotspot (general)"

cat > $tmpdir/imp.$$ << EOF
1 0 0 1 0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -c -wc 2

#3 2 1 0 0 0 0 0
#0 0 0 3 2 1 0 0
cat > $tmpdir/out2.$$ << EOF
3 2 1 3 2 1 0 0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  10: circular convolution, Double 1D - W, (1) hotspot (general)
# ------------------------------------------------------------------------
test_num=10
descr="circular convolution, Double 1D - W, (1) hotspot (general)"

cat > $tmpdir/imp.$$ << EOF
1 0 0 1 0 0 0 1
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -c -wc 1

# 2 3 0 0 0 0 0 1
# 0 0 1 2 3 0 0 0
# 3 0 0 0 0 0 1 2
cat > $tmpdir/out2.$$ << EOF
5 3 1 2 3 0 1 3
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  11: circular convolution, Double 1D - W, (2) hotspot (general)
# ------------------------------------------------------------------------
test_num=11
descr="circular convolution, Double 1D - W, (2) hotspot (general)"

cat > $tmpdir/imp.$$ << EOF
1 0 0 1 0 0 0 1
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -c -wc 2

# 3 0 0 0 0 0 1 2
# 0 1 2 3 0 0 0 0
# 0 0 0 0 0 1 2 3
cat > $tmpdir/out2.$$ << EOF
3 1 2 3 0 1 3 5
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  12: circular convolution, Double 2D - WxH (1,1) hotspot
# ------------------------------------------------------------------------
test_num=12
descr="circular convolution, Double 2D - WxH (1,1) hotspot"

cat > $tmpdir/imp.$$ << EOF
1 0 0 0
0 0 0 0
0 0 0 0
0 0 0 0
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
4 5 6
7 8 9
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -c -wc 1 -hc 1

cat > $tmpdir/out2.$$ << EOF
5 6 0 4
8 9 0 7
0 0 0 0
2 3 0 1
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

exit $status
