#! /bin/sh

# Replace the oname in the following line with the name of the program
# you are testing.

object_name="kpad"

# ---------------------------------------------------------------
# ADD TESTS BETWEEN the  "Add tests below this" and "End of test 
# suite" comment lines (line 59).
# ---------------------------------------------------------------

file_name=`echo $0 | sed -e 's/^.*\///'`
echo "running "$file_name" for "$object_name" ... process ($$)"

# report utility
# ---------------------------------------------------------------
# Usage:  report  test_num test_obj cntrl_obj test_descr tol
#
# The report function uses ksegcmp to compare the test output 
# data object ($2) against the correct result ($3), and reports 
# success or failure of the test.  If the result of ksegcmp is 
# false, status is set to 1.
#
# Input Arguments: test_num - test number
#                  test_obj - the object generated by the test
#                  cntrl_obj - known result to compare against
#                  test_descr - a string describing the test
#                  tol - tolerance allowed when comparing object 
#                        data values. Recommended tolerance 1e-5
# ---------------------------------------------------------------
report()
{
   test_num="$1"
   test_obj="$2"
   cntrl_obj="$3"
   test_descr="$4"
   tol="$5"

   ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol "$tol" -sat 1 -s -rt 1 -all
   if [ $? = 0 ]
   then
      echo "Passed:  Test  $test_num --> "$test_descr""
   else
      echo "FAILED:  Test  $test_num --> "$test_descr""
      status=1
   fi
}

# Initialize status to be success (ie. 0).  Upon failure, status 
# is initialized to 1.  Set the tmpdir variable.  Script will 
# use /tmp if $TMPDIR is not defined.
status=0
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb datamanip -echo binpath`}

# ---------------------------------------------------------------
#  Add tests below this
# ---------------------------------------------------------------

# Set the test number and the description of the test
test_num=1a
descr="Tests padding of BIT data, pad=1, no offset"

# Create a temporary ascii file containing the test data values
cat <<EOF > $tmpdir/asc.$$
1 2
3 4
5 6
EOF

# Import the data as a 2x3x1x1x1 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 2 -hsize 3 -type bit -o $tmpdir/data.$$

# Perform the kpad operation
${BINARY_PATH}/kpad -i $tmpdir/data.$$ -wsize 5 -hsize 5 -real 1 -o $tmpdir/out.$$

# Create a temporary ascii file containing the result values
cat <<EOF > $tmpdir/asc.$$
1 2 1 1 1
3 4 1 1 1
5 6 1 1 1
1 1 1 1 1
1 1 1 1 1
EOF

# Import the data as a 5x5x1x1x1 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 5 -hsize 5 -type bit -o $tmpdir/result.$$

# Compare the results of the kpad operation against the
# correct results using the report utility
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr" "1e-5"

# Remove the temporary files created by this test
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/asc.$$
/bin/rm -f $tmpdir/data.$$

test_num=1b
descr="Tests padding of BIT data, pad=0, no offset"

# Create a temporary ascii file containing the test data values
cat <<EOF > $tmpdir/asc.$$
1 2
3 4
5 6
EOF

# Import the data as a 2x3x1x1x1 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 2 -hsize 3 -type bit -o $tmpdir/data.$$

# Perform the kpad operation
${BINARY_PATH}/kpad -i $tmpdir/data.$$ -wsize 5 -hsize 5 -o $tmpdir/out.$$

# Create a temporary ascii file containing the result values
cat <<EOF > $tmpdir/asc.$$
1 2 0 0 0
3 4 0 0 0
5 6 0 0 0
0 0 0 0 0
0 0 0 0 0
EOF

# Import the data as a 5x5x1x1x1 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 5 -hsize 5 -type bit -o $tmpdir/result.$$

# Compare the results of the kpad operation against the
# correct results using the report utility
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr" "1e-5"

# Remove the temporary files created by this test
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/asc.$$
/bin/rm -f $tmpdir/data.$$

test_num=1c
descr="Tests padding of BYTE data, pad=1, +w offset"

# Create a temporary ascii file containing the test data values
cat <<EOF > $tmpdir/asc.$$
1 2
3 4
5 6
EOF

# Import the data as a 2x3x1x1x1 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 2 -hsize 3 -type byte -o $tmpdir/data.$$

# Perform the kpad operation
${BINARY_PATH}/kpad -i $tmpdir/data.$$ -wsize 5 -hsize 5 -real 1 -woff 2 -o $tmpdir/out.$$

# Create a temporary ascii file containing the result values
cat <<EOF > $tmpdir/asc.$$
1 1 1 2 1
1 1 3 4 1
1 1 5 6 1
1 1 1 1 1
1 1 1 1 1
EOF

# Import the data as a 5x5x1x1x1 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 5 -hsize 5 -type byte -o $tmpdir/result.$$

# Compare the results of the kpad operation against the
# correct results using the report utility
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr" "1e-5"

# Remove the temporary files created by this test
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/asc.$$
/bin/rm -f $tmpdir/data.$$

test_num=1d
descr="Tests padding of BYTE data, pad=0, -w offset"

# Create a temporary ascii file containing the test data values
cat <<EOF > $tmpdir/asc.$$
1 2
3 4
5 6
EOF

# Import the data as a 2x3x1x1x1 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 2 -hsize 3 -type byte -o $tmpdir/data.$$

# Perform the kpad operation
${BINARY_PATH}/kpad -i $tmpdir/data.$$ -wsize 5 -hsize 5 -woff -1 -o $tmpdir/out.$$

# Create a temporary ascii file containing the result values
cat <<EOF > $tmpdir/asc.$$
2 0 0 0 0
4 0 0 0 0
6 0 0 0 0
0 0 0 0 0
0 0 0 0 0
EOF

# Import the data as a 5x5x1x1x1 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 5 -hsize 5 -type byte -o $tmpdir/result.$$

# Compare the results of the kpad operation against the
# correct results using the report utility
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr" "1e-5"

# Remove the temporary files created by this test
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/asc.$$
/bin/rm -f $tmpdir/data.$$

test_num=1e
descr="Tests padding of Short data, pad=1, multiple offsets"

# Create a temporary ascii file containing the test data values
cat <<EOF > $tmpdir/asc.$$
1 2 3 4 5 6
7 8  9 10 11 12
13 14 15 16 17 18
19 20 21 22 23 24
EOF

# Import the data as a 2x3x2x1x2 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 2 -hsize 3 -dsize 2 -tsize 2 -type "Short" -o $tmpdir/data.$$

# Perform the kpad operation
${BINARY_PATH}/kpad -i $tmpdir/data.$$ -wsize 2 -hsize 3 -dsize 4 -esize 2 -real 1 -o $tmpdir/out.$$

# Create a temporary ascii file containing the result values
cat <<EOF > $tmpdir/asc.$$
1 2 3 4 5 6 7 8 9 10 11 12 1 1 1 1 1 1 1 1 1 1 1 1
13 14 15 16 17 18 19 20 21 22 23 24 1 1 1 1 1 1 1 1 1 1 1 1
1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1
1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1
EOF

# Import the data as a 2x3x4x2x2 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 2 -hsize 3 -dsize 4 -esize 2 -tsize 2 -type "Short" -o $tmpdir/result.$$

# Compare the results of the kpad operation against the
# correct results using the report utility
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr" "1e-5"

# Remove the temporary files created by this test
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/asc.$$
/bin/rm -f $tmpdir/data.$$

test_num=1f
descr="Tests padding of Short data, pad=0, -h multiple offsets"

# Create a temporary ascii file containing the test data values
cat <<EOF > $tmpdir/asc.$$
1 2 3 4 5 6
7 8  9 10 11 12
13 14 15 16 17 18
19 20 21 22 23 24
EOF
 
# Import the data as a 2x3x2x1x2 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 2 -hsize 3 -dsize 2 -tsize 2 -type "Short" -o $tmpdir/data.$$

# Perform the kpad operation
${BINARY_PATH}/kpad -i $tmpdir/data.$$ -wsize 2 -hsize 3 -dsize 4 -esize 2 -real 1 -hoff -1 -o $tmpdir/out.$$

# Create a temporary ascii file containing the result values
cat <<EOF > $tmpdir/asc.$$
3 4 5 6 1 1 9 10 11 12 1 1 1 1 1 1 1 1 1 1 1 1 1 1
15 16 17 18 1 1 21 22 23 24 1 1 1 1 1 1 1 1 1 1 1 1 1 1
1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 
1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1
EOF

# Import the data as a 2x3x2x1x2 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 2 -hsize 3 -dsize 4 -esize 2 -tsize 2 -type "Short" -o $tmpdir/result.$$

# Compare the results of the kpad operation against the
# correct results using the report utility
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr" "1e-5"

# Remove the temporary files created by this test
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/asc.$$
/bin/rm -f $tmpdir/data.$$

test_num=1g
descr="Tests padding of Long data, pad=1, no offset"

# Create a temporary ascii file containing the test data values
cat <<EOF > $tmpdir/asc.$$
1 2
3 4
5 6
EOF

# Import the data as a 2x3x1x1x1 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 2 -hsize 3 -type long -o $tmpdir/data.$$

# Perform the kpad operation
${BINARY_PATH}/kpad -i $tmpdir/data.$$ -wsize 5 -hsize 5 -real 1 -o $tmpdir/out.$$

# Create a temporary ascii file containing the result values
cat <<EOF > $tmpdir/asc.$$
1 2 1 1 1
3 4 1 1 1
5 6 1 1 1
1 1 1 1 1
1 1 1 1 1
EOF

# Import the data as a 5x5x1x1x1 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 5 -hsize 5 -type long -o $tmpdir/result.$$

# Compare the results of the kpad operation against the
# correct results using the report utility
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr" "1e-5"

# Remove the temporary files created by this test
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/asc.$$
/bin/rm -f $tmpdir/data.$$

test_num=1h
descr="Tests padding of Long data, pad=0, no offset"

# Create a temporary ascii file containing the test data values
cat <<EOF > $tmpdir/asc.$$
1 2
3 4
5 6
EOF

# Import the data as a 2x3x1x1x1 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 2 -hsize 3 -type long -o $tmpdir/data.$$

# Perform the kpad operation
${BINARY_PATH}/kpad -i $tmpdir/data.$$ -wsize 5 -hsize 5 -o $tmpdir/out.$$

# Create a temporary ascii file containing the result values
cat <<EOF > $tmpdir/asc.$$
1 2 0 0 0
3 4 0 0 0
5 6 0 0 0
0 0 0 0 0
0 0 0 0 0
EOF

# Import the data as a 5x5x1x1x1 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 5 -hsize 5 -type long -o $tmpdir/result.$$

# Compare the results of the kpad operation against the
# correct results using the report utility
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr" "1e-5"

# Remove the temporary files created by this test
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/asc.$$
/bin/rm -f $tmpdir/data.$$

test_num=1i
descr="Tests padding of Unsigned Integer data, pad=1, no offset"

# Create a temporary ascii file containing the test data values
cat <<EOF > $tmpdir/asc.$$
1 2
3 4
5 6
EOF

# Import the data as a 2x3x1x1x1 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 2 -hsize 3 -type "Unsigned Integer" -o $tmpdir/data.$$

# Perform the kpad operation
${BINARY_PATH}/kpad -i $tmpdir/data.$$ -wsize 5 -hsize 5 -real 1 -o $tmpdir/out.$$

# Create a temporary ascii file containing the result values
cat <<EOF > $tmpdir/asc.$$
1 2 1 1 1
3 4 1 1 1
5 6 1 1 1
1 1 1 1 1
1 1 1 1 1
EOF

# Import the data as a 5x5x1x1x1 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 5 -hsize 5 -type "Unsigned Integer" -o $tmpdir/result.$$

# Compare the results of the kpad operation against the
# correct results using the report utility
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr" "1e-5"

# Remove the temporary files created by this test
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/asc.$$
/bin/rm -f $tmpdir/data.$$

test_num=1j
descr="Tests padding of Unsigned Integer data, pad=0, no offset"

# Create a temporary ascii file containing the test data values
cat <<EOF > $tmpdir/asc.$$
1 2
3 4
5 6
EOF

# Import the data as a 2x3x1x1x1 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 2 -hsize 3 -type "Unsigned Integer" -o $tmpdir/data.$$

# Perform the kpad operation
${BINARY_PATH}/kpad -i $tmpdir/data.$$ -wsize 5 -hsize 5 -real 0 -o $tmpdir/out.$$

# Create a temporary ascii file containing the result values
cat <<EOF > $tmpdir/asc.$$
1 2 0 0 0
3 4 0 0 0
5 6 0 0 0
0 0 0 0 0
0 0 0 0 0
EOF

# Import the data as a 5x5x1x1x1 bit data set using kasc2val
kasc2val -i1 $tmpdir/asc.$$ -wsize 5 -hsize 5 -type "Unsigned Integer" -o $tmpdir/result.$$

# Compare the results of the kpad operation against the
# correct results using the report utility
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr" "1e-5"

# Remove the temporary files created by this test
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/asc.$$
/bin/rm -f $tmpdir/data.$$

# ---------------------------------------------------------------
#  End of test suite
# ---------------------------------------------------------------

exit $status
