#! /bin/sh

echo "running test3.sh for kresample ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"
tolerance="$5"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol "$tolerance" -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0


# ------------------------------------------------------------------------
#  set path for test data, and path for temporary directory
# ------------------------------------------------------------------------
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/images"
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

# ------------------------------------------------------------------------
# Test  1:  BIT resampling test (2x7), wmag=.75, hmag=2
# ------------------------------------------------------------------------
test_num=1
descr="BIT resampling test (2x7), wmag=.75, hmag=2"
cat > $tmpdir/asc.$$ <<EOF
1 1 0 0 1 1 0
1 1 0 1 0 1 0
EOF
kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -type bit -val
${BINARY_PATH}/kresample -i $tmpdir/asc.$$ -o $tmpdir/out1.$$ -wmag .75 -hmag 2
kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -type byte -val
${BINARY_PATH}/kresample -i $tmpdir/asc.$$ -o $tmpdir/out2.$$ -wmag .75 -hmag 2
kconvert -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -type bit -val
 
report $test_num "$tmpdir/out1.$$" "$tmpdir/out2.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/out1.$$ $tmpdir/out2.$$ $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  2:  BIT resampling test (2x7), wmag=1.33, hmag=2.61
# ------------------------------------------------------------------------
test_num=2
descr="BIT resampling test (2x7), wmag=1.33, hmag=2.61"
cat > $tmpdir/asc.$$ <<EOF
1 1 0 0 1 1 0
1 1 0 1 0 1 0
EOF
kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -type bit -val
${BINARY_PATH}/kresample -i $tmpdir/asc.$$ -o $tmpdir/out1.$$ -wmag 1.33 \
   -hmag 2.61
kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -type byte -val
${BINARY_PATH}/kresample -i $tmpdir/asc.$$ -o $tmpdir/out2.$$ -wmag 1.33 \
   -hmag 2.61
kconvert -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -type bit -val
 
report $test_num "$tmpdir/out1.$$" "$tmpdir/out2.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/out1.$$ $tmpdir/out2.$$ $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  3:  BIT resampling test (7x12), wmag=1.33, hmag=.61
# ------------------------------------------------------------------------
test_num=3
descr="BIT resampling test (7x12), wmag=1.33, hmag=.61"
cat > $tmpdir/asc.$$ <<EOF
1 1 0 0 1 1 0 1 1 0 1 1
1 1 0 1 0 1 0 0 0 1 0 1
1 0 0 1 1 1 1 1 0 0 1 0
0 0 0 0 0 0 1 0 0 0 0 0
1 0 0 0 0 1 0 1 0 0 0 1
0 1 0 0 1 0 1 0 1 0 1 0
1 0 1 0 1 0 1 0 1 0 1 0
EOF
kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -type bit -val
${BINARY_PATH}/kresample -i $tmpdir/asc.$$ -o $tmpdir/out1.$$ -wmag 1.33 \
   -hmag .61
kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -type byte -val
${BINARY_PATH}/kresample -i $tmpdir/asc.$$ -o $tmpdir/out2.$$ -wmag 1.33 \
   -hmag .61
kconvert -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -type bit -val
 
report $test_num "$tmpdir/out1.$$" "$tmpdir/out2.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/out1.$$ $tmpdir/out2.$$ $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  4:  BIT resampling test (7x12), wmag=133.2176, hmag=70.9999
# ------------------------------------------------------------------------
test_num=4
descr="BIT resampling test (7x12), wmag=133.2176, hmag=70.9999"
cat > $tmpdir/asc.$$ <<EOF
1 1 0 0 1 1 0 1 1 0 1 1
1 1 0 1 0 1 0 0 0 1 0 1
1 0 0 1 1 1 1 1 0 0 1 0
0 0 0 0 0 0 1 0 0 0 0 0
1 0 0 0 0 1 0 1 0 0 0 1
0 1 0 0 1 0 1 0 1 0 1 0
1 0 1 0 1 0 1 0 1 0 1 0
EOF
kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -type bit -val
${BINARY_PATH}/kresample -i $tmpdir/asc.$$ -o $tmpdir/out1.$$ -wmag 133.2176 \
   -hmag 70.9999
kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -type byte -val
${BINARY_PATH}/kresample -i $tmpdir/asc.$$ -o $tmpdir/out2.$$ -wmag 133.2176 \
   -hmag 70.9999
kconvert -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -type bit -val
 
report $test_num "$tmpdir/out1.$$" "$tmpdir/out2.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/out1.$$ $tmpdir/out2.$$ $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  5:  BIT resampling test (7x11), wmag=8, hmag=8
# ------------------------------------------------------------------------
test_num=5
descr="BIT resampling test (7x11), wmag=8, hmag=8"
cat > $tmpdir/asc.$$ <<EOF
1 1 0 0 1 1 0 1 1 0 1
1 1 0 1 0 1 0 0 0 1 0
1 0 0 1 1 1 1 1 0 0 1
0 0 0 0 0 0 1 0 0 0 0
1 0 0 0 0 1 0 1 0 0 0
0 1 0 0 1 0 1 0 1 0 1
1 0 1 0 1 0 1 0 1 0 1
EOF
kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -type bit -val
${BINARY_PATH}/kresample -i $tmpdir/asc.$$ -o $tmpdir/out1.$$ -wmag 8 \
   -hmag 8
kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -type byte -val
${BINARY_PATH}/kresample -i $tmpdir/asc.$$ -o $tmpdir/out2.$$ -wmag 8 \
   -hmag 8
kconvert -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -type bit -val
 
report $test_num "$tmpdir/out1.$$" "$tmpdir/out2.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/out1.$$ $tmpdir/out2.$$ $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  6:  BIT resampling test (1x1), wmag=8.33333, hmag=8
# ------------------------------------------------------------------------
test_num=6
descr="BIT resampling test (1x1), wmag=8.33333, hmag=8"
cat > $tmpdir/asc.$$ <<EOF
1
EOF
kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -type bit -val
${BINARY_PATH}/kresample -i $tmpdir/asc.$$ -o $tmpdir/out1.$$ -wmag 8.33333 \
   -hmag 8
kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -type byte -val
${BINARY_PATH}/kresample -i $tmpdir/asc.$$ -o $tmpdir/out2.$$ -wmag 8.33333 \
   -hmag 8
kconvert -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -type bit -val
 
report $test_num "$tmpdir/out1.$$" "$tmpdir/out2.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/out1.$$ $tmpdir/out2.$$ $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  7:  BIT resampling test (7x11), wmag=8, hmag=8, large data set
#           processing
# ------------------------------------------------------------------------
test_num=7
descr="BIT resampling test (14x11), wmag=8, hmag=8, large data set processing"
cat > $tmpdir/asc.$$ <<EOF
1 1 0 0 1 1 0 1 1 0 1 1 1 0 0 1 1 0 1 1 0 1
1 1 0 1 0 1 0 0 0 1 0 1 1 0 1 0 1 0 0 0 1 0
1 0 0 1 1 1 1 1 0 0 1 1 0 0 1 1 1 1 1 0 0 1
0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0
1 0 0 0 0 1 0 1 0 0 0 1 0 0 0 0 1 0 1 0 0 0
0 1 0 0 1 0 1 0 1 0 1 0 1 0 0 1 0 1 0 1 0 1
1 0 1 0 1 0 1 0 1 0 1 1 0 1 0 1 0 1 0 1 0 1
EOF
kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -type bit -val

set KHOROS_KDMS_BUFSIZE=1
export KHOROS_KDMS_BUFSIZE
${BINARY_PATH}/kresample -i $tmpdir/asc.$$ -o $tmpdir/out1.$$ -wmag 8 \
   -hmag 8
unset KHOROS_KDMS_BUFSIZE

kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -type byte -val
${BINARY_PATH}/kresample -i $tmpdir/asc.$$ -o $tmpdir/out2.$$ -wmag 8 \
   -hmag 8
kconvert -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -type bit -val
 
report $test_num "$tmpdir/out1.$$" "$tmpdir/out2.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/out1.$$ $tmpdir/out2.$$ $tmpdir/asc.$$

exit $status
