#! /bin/sh

echo "running test4.sh for kresample ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"
tolerance="$5"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol "$tolerance" -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0


# ------------------------------------------------------------------------
#  set path for test data, and path for temporary directory
# ------------------------------------------------------------------------
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/images"
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

# ------------------------------------------------------------------------
# Test  1:  Check simple magnification. This makes sure a bug reported by
#           Hank van Bekkem is gone.
# ------------------------------------------------------------------------
test_num=1
descr="Check simple magnification, wmag=3"
cat > $tmpdir/asc.$$ <<EOF
1 2 3 4
EOF
${BINARY_PATH}/kresample -i $tmpdir/asc.$$ -o $tmpdir/out.$$ -wmag 3 -resize 1
cat > $tmpdir/asc.$$ <<EOF
1 1 1 2 2 2 3 3 3 4 4 4
EOF
 
report $test_num "$tmpdir/out.$$" "$tmpdir/asc.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/out.$$ $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  2:  Check simple demagnification. This makes sure a bug reported by
#           Hank van Bekkem is gone.
# ------------------------------------------------------------------------
test_num=2
descr="Check simple demagnification, wmag=.333333333"
cat > $tmpdir/asc.$$ <<EOF
1 1 1 2 2 2 3 3 3 4 4 4
EOF
${BINARY_PATH}/kresample -i $tmpdir/asc.$$ -o $tmpdir/out.$$ -wmag .333333333 \
                         -resize 1
cat > $tmpdir/asc.$$ <<EOF
1 2 3 4
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/asc.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/out.$$ $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  3:  Check resample of SUN rasterfile. This makes sure that a
#           bug reported by Hank van Bekkem is gone.
# ------------------------------------------------------------------------
test_num=3
descr="Check SUN rasterfile demagnification"

kformats -i $dpath/canyon.ras -o $tmpdir/kdf.$$ -kdf
${BINARY_PATH}/kresample -i $dpath/canyon.ras -o $tmpdir/ras.$$ -wmag .1 \
                         -hmag .1 -resize 1
${BINARY_PATH}/kresample -i $tmpdir/kdf.$$ -o $tmpdir/kdf.$$ -wmag .1 \
                         -hmag .1 -resize 1

report $test_num "$tmpdir/ras.$$" "$tmpdir/kdf.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/out.$$ $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  4:  Check resample of PPM file.
# ------------------------------------------------------------------------
test_num=4
descr="Check PPM demagnification"

kformats -i $dpath/canyon.ras -o $tmpdir/kdf.$$ -kdf
${BINARY_PATH}/kresample -i $dpath/canyon.ppm -o $tmpdir/ppm.$$ -wmag .1 \
                         -hmag .1 -resize 1
${BINARY_PATH}/kresample -i $tmpdir/kdf.$$ -o $tmpdir/kdf.$$ -wmag .1 \
                         -hmag .1 -resize 1

report $test_num "$tmpdir/ppm.$$" "$tmpdir/kdf.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/out.$$ $tmpdir/asc.$$

exit $status

