 /*
  * Khoros: $Id: Arf.c,v 1.5 1996/06/08 18:39:23 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Arf.c,v 1.5 1996/06/08 18:39:23 mirage Exp $";

#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros ARF Data Service Routines
   >>>>		
   >>>>  Private:
   >>>>                 init_arf()
   >>>>                 init_arf()
   >>>>   Static:
   >>>>                 arf_check()
   >>>>                 arf_input()
   >>>>                 arf_output()
   >>>>                 arf_prep()
   >>>>                 arf_destroy()
   >>>>                 arf_prep()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

#if !defined(KARF_DEF)

void
init_arf(void) /*-- NULL arf initialization --*/
{
   return;
}

#else

#include "kdatafmt/arf.h"

kubyte arf_magic[4] = {187, 187, 186, 173};

static int _init PROTO((kobject));
static int arf_check   PROTO((int));
static int arf_input   PROTO((kobject, int, int));
static int arf_output  PROTO((kobject, int, int));
static int arf_destroy PROTO((kobject));
static int arf_prep    PROTO((kobject));


static int define_arf_colormap_attributes PROTO((void));
static int define_arf_comment_attribute   PROTO((void));
static int malloc_attribute_arrays        PROTO((unsigned long, unsigned long,
						 ARF_FOOTER_ATTRIBUTES *));
     
static int create_arf_footer_attributes PROTO((kobject, unsigned long, 
					       unsigned long));
static int set_arf_footer_attributes    PROTO((kobject, 
					       ARF_FOOTER_ATTRIBUTES *,
					       unsigned long));
static int get_arf_footer_attributes    PROTO((kobject, 
					       ARF_FOOTER_ATTRIBUTES *,
					       unsigned long));
     
static int create_arf_info_attributes PROTO((kobject, ARF_INFO *));
static int set_arf_info_attributes    PROTO((kobject, ARF_INFO *));
static int get_arf_info_attributes    PROTO((kobject, ARF_INFO *));
    
static int create_arf_ladar_param_attributes PROTO((kobject));
static int set_arf_ladar_param_attributes    PROTO((kobject, 
						    ARF_LADAR_PARAM *));
static int get_arf_ladar_param_attributes    PROTO((kobject, 
						    ARF_LADAR_PARAM *));


/*=====================================================================*/

/*-----------------------------------------------------------
|  Routine Name: init_arf
|       Purpose: initialize the arf file format
------------------------------------------------------------*/
void
init_arf(void)
{
   kdms_add_file_format("arf", 
			"Sun Arf file File Format (arf)",
			arf_check, 
			arf_input, 
			arf_output,
			arf_destroy, 
			arf_prep,
			NULL, NULL, NULL, NULL, NULL);
   return;
}

/*=====================================================================*/

/*
 *  Internal Resource Structure
 *  arf  -  ATRWG ARF format
 */

typedef struct 
{
   int              fid;
   ARF_SUBHEADER    *subh;
   ARF_FRAME_FOOTER *frame_footer;
   ARF              *arf;
   unsigned long    frame_size, info_size, colormap_size, comment_size;
   unsigned long    multi_band_size, frame_data_size, ladar_param_size;
   unsigned long    frame_foot_size, info_offset, colormap_offset;
   unsigned long    comment_offset, multi_band_offset;
   unsigned long    frame_data_offset, ladar_param_offset;
   char            *imagedata;

}  ResourceStruct;

/****    =============================================================
 ****    Format Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: _init - Creates an arf image
|
|       Purpose: This function is used to create an arf image.  Which
|		 means that we use the current dimension values in order
|		 to create a proper arf.
|
|         Input: data - initial sample of data
|                num  - number of bytes in the sample
|
|        Output: Returns: returns TRUE or FALSE if an error occurs
|
|    Written By: James A. Moulton
|          Date: Aug 11, 1993
| Modifications: Apr 11, 1994 Tim Williams - modeled after a version of
|                                            Viff.c. It looks as though
|                                            James used Rast.c as a model.
|
------------------------------------------------------------*/
static int
_init(kobject object)
{
   ResourceStruct *resources;


   /*-- create the image to the object file --*/
   resources = (ResourceStruct *) kcalloc((size_t)1, sizeof(ResourceStruct));
   if (resources == NULL)
      return FALSE;

   resources->info_size        = 0;
   resources->colormap_size    = 0;
   resources->comment_size     = 0;
   resources->multi_band_size  = 0;
   resources->frame_data_size  = 0;
   resources->ladar_param_size = 0;
   resources->frame_foot_size  = 0;
   
   /*-- default to size of ARF_MAIN_HDR --*/
   resources->info_offset      	 = 32;
   resources->colormap_offset  	 = 32;
   resources->comment_offset   	 = 32;
   resources->multi_band_offset  = 32;
   resources->frame_data_offset  = 32;
   resources->ladar_param_offset = 32;

   kdms_glue_set_resources(object, resources);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: arf_check - Checks to see if the file is
|				 an arf
|
|       Purpose: This function is used to check the first 4 bytes of
|		 the stream to see if it is an arf.
|
|         Input: file - the arf file to be checked.
|
|        Output: Returns: returns TRUE or FALSE if an error occurs
|
|    Written By: Tim Williams
|          Date: Apr 22, 1994
| Modifications: 
|
------------------------------------------------------------*/
static int
arf_check(int fid)
{
   kubyte magic[4];


   /*-- ARF is written with XDR implying big-endian, IEEE floats --*/
   kfile_setmachtype(fid, KMACH_BIG_ENDIAN_IEEE);

   /*-- read first 4 bytes for magic number --*/
   kread_generic(fid, &magic, (size_t)4, KUBYTE);

   return (kmemcmp(magic, arf_magic, (size_t) 4) == 0);
}

/*-----------------------------------------------------------
|
|  Routine Name: arf_input - Reads an arf image
|
|       Purpose: This function is used to read in the image if the
|		 supplied data is arf image.  The data was checked
|		 by the arf_check() routine in which the arf
|		 identifier indicated that the data is a valid arf.
|		 Since the data was valid we can assume simply read
|		 the header in and initialize that data segments.
|
|         Input: object - the arf object to be initialized
|
|        Output: Returns: returns TRUE or FALSE if an error occurs
|
|    Written By: James A. Moulton
|          Date: Aug 11, 1993
| Modifications: 13 June, 1994 Tim Williams - modeled after a version of
|                                            Viff.c. It looks as though
|                                            James used Rast.c as a model.
|
------------------------------------------------------------*/
static int
arf_input(kobject object, int fid, int flags)
{
   ResourceStruct *resources;
   ARF            *arf;
   ARF_SUBHEADER  *subh = NULL;
   int             offset, i;
   int             type;
   int             frame, frame_size;
   int             size[KDMS_MAX_DIM];
   kaddr           data;
   kubyte         *d;

   int val_order[5] = {KWIDTH, KHEIGHT, KELEMENTS, KTIME, KDEPTH};
   int map_order[5] = {KMAP_HEIGHT, KMAP_WIDTH, KMAP_ELEMENTS, KMAP_DEPTH,
		       KMAP_TIME};

   ARF_FOOTER_ATTRIBUTES footer_attributes;


   /*-- get the resource structure --*/
   if (!kdms_initialized(object))
      _init(object);
   
   resources = (ResourceStruct *) kdms_glue_get_resources(object);

   resources->fid = fid;
  
   /*-- ARF is written with XDR implying big-endian, IEEE floats --*/
   kfile_setmachtype(fid, KMACH_BIG_ENDIAN_IEEE);
  
   if ((arf = (ARF *)kmalloc(sizeof(ARF))) == NULL)
   {
      kerror("data services", "arf_input", 
	     "not enough memory for ARF structure");
      return FALSE;
   }
   
   resources->arf = arf;
   /*-- get 32 byte ARF main header (8 ulongs) --*/
   arf_get_main_hdr(fid, &(resources->arf->hdr));

   /*------------------------------------------------------------------
   |  create attribute to save ambiguous ARF data type
   |  arf_ten_bit, arf_twelve_bit, arf_short_word -> KUSHORT 
   \-----------------------------------------------------------------*/
   kdms_create_attribute(object, NULL, "ARF_DATATYPE", 1, 1, KINT, TRUE, TRUE);
   kdms_set_attribute(object, NULL, "ARF_DATATYPE", arf->hdr._image_type);
  
   /*-----------------------------------------------------------------------
   |   initialize all data segments based on information in arf header
   |   
   |   - arf structure contains only value, and map data
   |     plus *many* specific attributes.
   \-----------------------------------------------------------------------*/

   /*-- check for subheaders --*/

   /*--------------------------------------------------------------
   | the arf_get_ routines will read the structure requested 
   | and return the length in the file of the subheader
   \--------------------------------------------------------------*/
   if (arf->hdr._subheader_flags)
   {
      if ((subh = (ARF_SUBHEADER *)kmalloc(sizeof(ARF_SUBHEADER))) == NULL)
      {
	 kerror("data services", "arf_input",
		"not enough memory for ARF_SUBHEADER structure");
	 return FALSE;
      }
      resources->subh = subh;
      if (arf->hdr._subheader_flags & ARF_INFO_FLAG)
      {
	 resources->info_size = arf_get_info(fid, subh);
	 if (!resources->info_size)
	    return FALSE;

	 /*-- create and set attributes --*/
	 create_arf_info_attributes(object, &subh->info);
	 if (!set_arf_info_attributes(object, &subh->info))
	 {
	    kerror("data services", "arf_input",
		   "Error setting ARF_INFO attriutes");
	    return FALSE;
	 }
      }
    
      resources->colormap_offset = resources->info_offset + 
	                           resources->info_size;

      /*--------------------------------------------------------------------
      |  map data
      \--------------------------------------------------------------------*/
      if (arf->hdr._subheader_flags & ARF_COLORMAP_FLAG)
      {
	 kubyte *mapdata, *mapptr;

	 /*-- create the map segment --*/
	 kpds_create_map(object);

	 resources->colormap_size = arf_get_colormap(fid, subh);
	 size[0] = ARF_COLORMAP_SIZE;
	 size[1] = 3;
	 size[2] = size[3] = size[4] = 1;
       
	 mapdata = (unsigned char *)kmalloc(3*ARF_COLORMAP_SIZE*sizeof(char));
	 mapptr = mapdata;
	 for (i = 0; i < ARF_COLORMAP_SIZE; i++, mapptr++)
	    *mapptr = subh->colormap.red[i];
	 for (i = 0; i < ARF_COLORMAP_SIZE; i++, mapptr++)
	    *mapptr = subh->colormap.green[i];
	 for (i = 0; i < ARF_COLORMAP_SIZE; i++, mapptr++)
	    *mapptr = subh->colormap.blue[i];
       
	 define_arf_colormap_attributes();

	 /*-- want colorspace to be RGB --*/
	 kcolor_set_attribute(object, KCOLOR_COLORSPACE, KRGB);

	 /*-- store all map information --*/
	 kdms_set_segment(object, KDMS_SEGMENT_MAP, 5, size, map_order,
			  KUBYTE, mapdata);
      }


      resources->comment_offset = resources->colormap_offset + 
	                          resources->colormap_size;

      if (arf->hdr._subheader_flags & ARF_COMMENT_FLAG)
      {
	 define_arf_comment_attribute();
	 resources->comment_size = arf_get_comment(fid, subh);
	 if (resources->comment_size)
	    kdms_set_attribute(object, NULL, KDMS_COMMENT, subh->comment.bs);
	 else
	    return FALSE;
      }
      resources->multi_band_offset = resources->comment_offset + 
                                     resources->comment_size;
    
      if (arf->hdr._subheader_flags & ARF_MULTI_BAND_FLAG)
      {
	 resources->multi_band_size = arf_get_multi_band(fid, subh);
	 if (resources->multi_band_size)
	 {
	    /*-- create and set attributes --*/
            /*	
	    create_arf_multi_band_attributes(object, &subh->multi_band);
	    set_arf_multi_band_attributes(object, &subh->multi_band);
	    */
	    for (i = 0; i < (int)subh->multi_band.num_bands; i++)
	    {
	       arf2khoros_datatype(subh->multi_band.band[i].type, &type);
	       resources->frame_size += 
		  arf->hdr._num_rows * arf->hdr._num_cols * 
		  kmach_sizeof(KMACH_BIG_ENDIAN_IEEE, (unsigned int) type);
	    }
	 } /*-- ARF_MULTI_BAND_FLAG --*/
	 else
	    return FALSE;
      }
      else
      {
	 arf2khoros_datatype(arf->hdr._image_type, &type);
	 resources->frame_size = arf->hdr._num_rows * arf->hdr._num_cols * 
	                         kmach_sizeof(KMACH_BIG_ENDIAN_IEEE, 
					      (unsigned int) type);

      } /*-- not ARF_MULTI_BAND_FLAG --*/

      resources->frame_data_offset = resources->multi_band_offset + 
	                             resources->multi_band_size;
   
      if (arf->hdr._subheader_flags & ARF_FRAME_DATA_FLAG) /* footers exist */
      {
	 resources->frame_data_size = arf_get_frame_data(fid, subh);

	 /*----------------------------------------------------------------
	 | allocate footer structure, attribute arrays, and 
	 | define footer attributes
	 \---------------------------------------------------------------*/

	 /*-- create attribute for footer flags --*/
	 kdms_create_attribute(object, NULL, "ARF_FRAME_DATA_footer_flags", 
			       1, 1, KULONG, TRUE, TRUE);
	 kdms_set_attribute(object, NULL, 
			    "ARF_FRAME_DATA_footer_flags", 
			    subh->frame_data.footer_flags);

	 resources->frame_footer = 
	    (ARF_FRAME_FOOTER *) kmalloc(sizeof(ARF_FRAME_FOOTER));
	 if (resources->frame_footer == NULL)
	 {
	    kerror("data services", "arf_input",
		   "not enough memory to create footers");
	    return FALSE;
	 }

	 /*-- create footer attributes and allocate arrays --*/
	 if (!malloc_attribute_arrays(subh->frame_data.footer_flags,
				      arf->hdr._num_frames,
				      &footer_attributes))
	 {
	    kerror("data services", "arf_input", 
		   "error allocating footer attribute arrays");
	    return FALSE;
	 }
	 create_arf_footer_attributes(object, subh->frame_data.footer_flags, 
				      arf->hdr._num_frames);
       
	 /*-- read footers AFTER reading all subheaders --*/

      } /*-- ARF_FRAME_DATA_FLAG --*/

      resources->ladar_param_offset = resources->frame_data_offset + 
	                              resources->frame_data_size;

      if (arf->hdr._subheader_flags & ARF_LADAR_PARAM_FLAG)
      {
	 resources->ladar_param_size = arf_get_ladar_param(fid, subh);
       
	 /*-- create and set attributes --*/
	 create_arf_ladar_param_attributes(object);
	 set_arf_ladar_param_attributes(object, &subh->ladar_param);
      }
    
      offset = resources->ladar_param_offset + resources->ladar_param_size;
      /*-- this should match arf->hdr._image_offset --*/
      if (offset != arf->hdr._image_offset)
      {
	 kerror("data services", "arf_input",
		"image offset %d calculated from subheaders doesn't match "
		"offset in main ARF header %d", offset, 
		arf->hdr._image_offset);
	 return FALSE;
      }

   } /*-- end subheaders --*/
  
   /*-- read the footers --*/
   if (resources->frame_data_size)
   {
      int here = (int) ktell(fid);
     
      /*-- go to start of image data --*/
      klseek(fid, (off_t) resources->arf->hdr._image_offset, KSEEK_SET);
      for (frame=0; frame<resources->arf->hdr._num_frames; frame++)
      {
	 /*-- skip past the frame --*/
	 klseek(fid, (off_t) resources->frame_size, KSEEK_CUR);
	 resources->frame_foot_size = 
	    arf_get_footer(fid, subh->frame_data.footer_flags, 
			   resources->frame_footer);
	 if (subh->frame_data.footer_flags & ARF_FRAME_INFO_FLAG)
	 {
	    footer_attributes.frame_info_x[frame] = 
	       resources->frame_footer->info.x;
	    footer_attributes.frame_info_y[frame] = 
	       resources->frame_footer->info.y;
	    footer_attributes.frame_info_az[frame] = 
	       resources->frame_footer->info.azimuth;
	    footer_attributes.frame_info_el[frame] = 
	       resources->frame_footer->info.elevation;
	    footer_attributes.frame_info_irig_yy[frame] = 
	       resources->frame_footer->info.irig_time.yy;
	    footer_attributes.frame_info_irig_dd[frame] = 
	       resources->frame_footer->info.irig_time.dd;
	    footer_attributes.frame_info_irig_hh[frame] = 
	       resources->frame_footer->info.irig_time.hh;
	    footer_attributes.frame_info_irig_mm[frame] = 
	       resources->frame_footer->info.irig_time.mm;
	    footer_attributes.frame_info_irig_ss[frame] = 
	       resources->frame_footer->info.irig_time.ss;
	    footer_attributes.frame_info_irig_ms[frame] = 
	       resources->frame_footer->info.irig_time.ms;
	 }
      
	 if (subh->frame_data.footer_flags & ARF_PLATFORM_ATTITUDE_FLAG)
	 {
	    footer_attributes.platform_attitude_vel_n[frame] = 
	       resources->frame_footer->attitude.vel_north;
	    footer_attributes.platform_attitude_vel_e[frame] = 
	       resources->frame_footer->attitude.vel_east;
	    footer_attributes.platform_attitude_vel_v[frame] = 
	       resources->frame_footer->attitude.vel_vertical;
	    footer_attributes.platform_attitude_acc_n[frame] = 
	       resources->frame_footer->attitude.acc_north;
	    footer_attributes.platform_attitude_acc_e[frame] = 
	       resources->frame_footer->attitude.acc_east;
	    footer_attributes.platform_attitude_acc_v[frame] = 
	       resources->frame_footer->attitude.acc_vertical;
	    footer_attributes.platform_attitude_lat[frame] = 
	       resources->frame_footer->attitude.latitude;
	    footer_attributes.platform_attitude_long[frame] = 
	       resources->frame_footer->attitude.longitude;
	    footer_attributes.platform_attitude_alt[frame] = 
	       resources->frame_footer->attitude.altitude;
	    footer_attributes.platform_attitude_elev[frame] = 
	       resources->frame_footer->attitude.elevation;
	    footer_attributes.platform_attitude_bank[frame] = 
	       resources->frame_footer->attitude.bank;
	    footer_attributes.platform_attitude_head[frame] = 
	       resources->frame_footer->attitude.heading;
	 }
      } /* frame loop */
    
      /*-- go back to where you were --*/
      klseek(fid, (off_t) here, KSEEK_SET);
      
      /* now set the attributes defined above */
      set_arf_footer_attributes(object, &footer_attributes, 
				subh->frame_data.footer_flags);
   }
  
   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/

   /*-- create the value segment --*/
   kpds_create_value(object);

   /*-- determine size and datatype --*/
   size[0] = arf->hdr._num_cols;
   size[1] = arf->hdr._num_rows;
   if (arf->hdr._image_type == arf_multi_band)
      size[2] = subh->multi_band.num_bands;
   else
      size[2] = 1; 
   size[3] = arf->hdr._num_frames;
   size[4] = 1;

   arf2khoros_datatype(arf->hdr._image_type, &type);

   /*-- read all frames of data --*/
   frame_size = size[0] * size[1];
   data = kmalloc((size_t) kdata_size(type)*frame_size*size[2]*size[3]);

   for (i = 0, d = (kubyte *) data; 
	i < size[2] * size[3];
	i++, d += frame_size * kdata_size(type))
   {
      kread_generic(fid, (kaddr) d, (size_t) frame_size, type);
      klseek(fid, (off_t) resources->frame_foot_size, SEEK_CUR);
   }
   
   /*-- store all value information --*/
   kdms_set_segment(object, KDMS_SEGMENT_VALUE, 5, size, val_order, 
		    type, data);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: arf_output - Closes the arf image
|
|       Purpose: This function is used to close the arf image.  Which
|		 means if the image was modified we then re-write image
|		 back out.
|
|         Input: object - the arf object to be closed
|
|        Output: Returns: returns TRUE or FALSE if an error occurs
|
|    Written By: Tim Williams
|          Date: May 16, 1994
| Modifications: ARF_MULTI_BAND data returns FALSE for now - tjw
|
------------------------------------------------------------*/
static int
arf_output(kobject object, int fid, int flags)
{
   ResourceStruct *resources;
   
   ARF           *arf;
   ARF_SUBHEADER *subh;
   int  	  type, arf_type, frame, colorspace, i, frame_size = 1;
   char 	 *tmpcomment = NULL;
   kaddr          vdata = NULL;
   kubyte        *d;
   
   ARF_FOOTER_ATTRIBUTES footer_attributes;


   if (!kdms_initialized(object)) 
     _init(object);

   resources = (ResourceStruct *) kdms_glue_get_resources(object);

   resources->fid = fid;
   
   if (resources->subh == NULL) /* set up ARF_SUBHEADER structure */
   {
      resources->subh=(ARF_SUBHEADER *)kmalloc(sizeof(ARF_SUBHEADER));
      resources->subh=(ARF_SUBHEADER *)kmemset((kaddr)resources->subh, 
					       0, sizeof(ARF_SUBHEADER));
      if (!resources->subh)
      {
	 /* _kdms_traceback("arf_output");
	 _kdms_set_error(KDMS_EFMT_FAILURE); */
	 return FALSE;
      }
   }
   subh = (ARF_SUBHEADER *)resources->subh;
   
   /*-- ARF is written with XDR implying big-endian, IEEE floats --*/
   kfile_setmachtype(resources->fid, KMACH_BIG_ENDIAN_IEEE);
   
   if (resources->arf == NULL) /* set up ARF structure */
   {
      resources->arf = (ARF *)kmalloc(sizeof(ARF));
      resources->arf = (ARF *)kmemset((kaddr)resources->arf, 0, sizeof(ARF));
      if (!resources->arf)
      {
	 /* _kdms_traceback("arf_output");
	 _kdms_set_error(KDMS_EFMT_FAILURE); */
	 return FALSE;
      }
   }
   arf = resources->arf;
   
   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/
   if (kpds_query_value(object))
   {       
      kpds_get_attributes(object, 
			  KPDS_VALUE_SIZE, 
			  &arf->hdr._num_cols, &arf->hdr._num_rows, NULL,
			  &arf->hdr._num_frames, NULL, 
			  KPDS_VALUE_DATA_TYPE, &type,
			  NULL);

      frame_size = arf->hdr._num_cols * arf->hdr._num_rows;
			 
      kmemcpy(arf->hdr._magic_num, arf_magic, (size_t) 4);
      arf->hdr._version    = ARF_CURRENT_VERSION;
      khoros2arf_datatype(type, &arf->hdr._image_type);

      /*------------------------------------------------------------------
      |  resolve KUSHORT to arf_ten_bit, arf_twelve_bit, or 
      |  arf_short_word if possible
      \-----------------------------------------------------------------*/
      if ((type == KUSHORT) &&
	  (kdms_get_attribute(object, NULL, "ARF_DATATYPE", &arf_type)))
	 arf->hdr._image_type = (ARF_IMAGE_TYPE)arf_type;
      
      arf->hdr._image_offset    = 32; 
      arf->hdr._subheader_flags = 0L;

      vdata = kpds_get_data(object, KPDS_VALUE_ALL, NULL);

   }
   
   
   /*-- get ARF_INFO subheader attributes, if any --*/
   resources->info_size = get_arf_info_attributes(object, &subh->info);
   if (resources->info_size)
   {
      arf->hdr._subheader_flags |= ARF_INFO_FLAG;
      arf->hdr._image_offset += resources->info_size;
   }
   
   /*-----------------------------------------------------------------------
   |  map data
   \-----------------------------------------------------------------------*/
   if (kpds_query_map(object))
   {
      int     mw, mh;
      kaddr   data;
      kubyte *mapptr;

      kpds_get_attribute(object, KCOLOR_COLORSPACE, &colorspace);
      
      kpds_get_attribute(object, KPDS_MAP_SIZE, &mw, &mh, NULL, NULL, NULL);
      kpds_set_attribute(object, KPDS_MAP_DATA_TYPE, KUBYTE);
      
      data = kpds_get_data(object, KPDS_MAP_ALL, NULL);
     
      if (data && (mh <= ARF_COLORMAP_SIZE) && (mw == 3))
      {
	 mapptr = data;
	 
	 for (i = 0; i < mh; i++)
	 {
	    subh->colormap.red[i]   = (unsigned char)*mapptr++;
	    subh->colormap.green[i] = (unsigned char)*mapptr++;
	    subh->colormap.blue[i]  = (unsigned char)*mapptr++;
	 }
	 
	 /* 256*4*3=3072 */
	 resources->colormap_size = ARF_COLORMAP_SIZE * 4 * 3; 
	 arf->hdr._subheader_flags |= ARF_COLORMAP_FLAG;
	 arf->hdr._image_offset += resources->colormap_size;

	 kfree_and_NULL(data);
      }
   }     
   
   /*-- get ARF_COMMENT subheader attributes, if any --*/
   if (kpds_get_attribute(object, KPDS_COMMENT, &tmpcomment))
   {
      int length;
      
      kstrcpy(subh->comment.bs, tmpcomment);
      length = kstrlen(subh->comment.bs);
      if (length != 0) 
      {
	 resources->comment_size = 
	    length%4 ? (4+length+(4-length%4)) : (4+length);
	 
	 arf->hdr._subheader_flags |= ARF_COMMENT_FLAG;
	 arf->hdr._image_offset += resources->comment_size;
      }
   }
   
   /*-- get ARF_MULTI_BAND subheader attributes, if any --*/
/*
   resources->multi_band_size=
     get_arf_multi_band_attributes(object, &subh->multi_band);
   if (resources->multi_band_size != 0) 
   {
     arf->hdr._subheader_flags |= ARF_MULTI_BAND_FLAG;
     arf->hdr._image_offset += resources->multi_band_size;
     arf->hdr._image_type = arf_multi_band;
     for (i=0; i<subh->multi_band.num_bands; i++)
     {
       arf2khoros_datatype(subh->multi_band.band[i].type, &type);
       resources->frame_size += 
	 arf->hdr._num_rows *
	   arf->hdr._num_cols * 
	     kmach_sizeof(KARCH_SUN, (unsigned int) type);
     }
   }
*/

   resources->multi_band_size = 0;

   /*-- only ARF_RGB currently supported --*/
   if ((arf->hdr._num_frames == 3) && (type == KUBYTE)) 
   {
      long tmpval;
      static char *band_name[]={"red", "green", "blue"};
      
      arf->hdr._subheader_flags |= ARF_MULTI_BAND_FLAG;
      
      kstrcpy(subh->multi_band.name, "rgb");
      tmpval = kstrlen(subh->multi_band.name);
      resources->multi_band_size += tmpval%4 ? 4+tmpval+(4-tmpval%4) :4+tmpval;
      
      subh->multi_band.num_bands = arf->hdr._num_frames;
      resources->multi_band_size += 4;
      
      for (i = 0; i < arf->hdr._num_frames; i++)
      {
	 kstrcpy(subh->multi_band.band[i].name, band_name[i]);
	 tmpval = kstrlen(subh->multi_band.band[i].name);
	 resources->multi_band_size += 
	    tmpval%4 ? 4+tmpval+(4-tmpval%4) : 4+tmpval;
	 
	 subh->multi_band.band[i].type = arf->hdr._image_type;
	 resources->multi_band_size += 4;
      }
      arf2khoros_datatype(arf->hdr._image_type, &type);
      resources->frame_size += 
	 arf->hdr._num_rows *
	 arf->hdr._num_cols * 
	 kmach_sizeof(KMACH_BIG_ENDIAN_IEEE, (unsigned int) type);
      
      /*-- increment subhsize for empty bands up to 16 (ARF_MAX_BANDS) --*/
      resources->multi_band_size += 8*(ARF_MAX_BANDS-
				       subh->multi_band.num_bands);
      
      arf->hdr._image_offset +=  resources->multi_band_size;
      arf->hdr._image_type = arf_multi_band;
   }
   else
   {
      arf2khoros_datatype(arf->hdr._image_type, &type);
      resources->frame_size = 
	 arf->hdr._num_rows *
	 arf->hdr._num_cols * 
	 kmach_sizeof(KMACH_BIG_ENDIAN_IEEE, (unsigned int) type);
   }

   
   /*-- get ARF_FRAME_DATA subheader attributes, if any --*/
   if (kdms_get_attribute(object, NULL,
			  "ARF_FRAME_DATA_footer_flags",
			  &subh->frame_data.footer_flags))
   {
      resources->frame_data_size = 4;
      arf->hdr._subheader_flags |= ARF_FRAME_DATA_FLAG;
      arf->hdr._image_offset += resources->frame_data_size;
      resources->frame_footer = 
	 (ARF_FRAME_FOOTER*)kmalloc(sizeof(ARF_FRAME_FOOTER));
      if (resources->frame_footer == NULL)
      {
	 kerror("data services", "arf_output",
		"not enough memory to create footers");
	 return FALSE;
      }
   }
   
   /*-- get ARF_LADAR_PARAM subheader attributes, if any --*/
   resources->ladar_param_size=
      get_arf_ladar_param_attributes(object, &subh->ladar_param);
   if (resources->ladar_param_size != 0) 
   {
      arf->hdr._subheader_flags |= ARF_LADAR_PARAM_FLAG;
      arf->hdr._image_offset += resources->ladar_param_size;
   }

   /*-----------------------------------------------------------------------
   |  Write the arf structure out to the specified filename
   \-----------------------------------------------------------------------*/
   
   /*-- write 32 byte ARF main header (8 unsigned longs) --*/
   arf_put_main_hdr(fid, &arf->hdr);
   
   /*-- write sub-headers --*/
   if (arf->hdr._subheader_flags)
   {
      if (arf->hdr._subheader_flags & ARF_INFO_FLAG)
	 arf_put_info(fid, &resources->subh->info);
      if (arf->hdr._subheader_flags & ARF_COLORMAP_FLAG)
	 arf_put_colormap(fid, &resources->subh->colormap);
      if (arf->hdr._subheader_flags & ARF_COMMENT_FLAG)
	 arf_put_comment(fid, &resources->subh->comment);
      if (arf->hdr._subheader_flags & ARF_MULTI_BAND_FLAG)
	 arf_put_multi_band(fid, &resources->subh->multi_band);
      if (arf->hdr._subheader_flags & ARF_FRAME_DATA_FLAG)
	 arf_put_frame_data(fid, &resources->subh->frame_data);
      if (arf->hdr._subheader_flags & ARF_LADAR_PARAM_FLAG)
	 arf_put_ladar_param(fid, &resources->subh->ladar_param);
   }
   
   /*-- write data and footers --*/

   /*-- go to start of image data --*/
   klseek(fid, (off_t)resources->arf->hdr._image_offset, KSEEK_SET);

   if (/*(resources->frame_data_size != 0) &&*/ vdata != NULL)
   {
      /*-- allocate attribute arrays --*/
      if (!malloc_attribute_arrays(subh->frame_data.footer_flags,
				   arf->hdr._num_frames,
				   &footer_attributes))
      {
	 kerror("data services", "arf_output", 
		"error allocating footer attribute arrays");
	 return FALSE;
      }
      
      /*-- get attributes --*/
      get_arf_footer_attributes(object, &footer_attributes, 
				subh->frame_data.footer_flags);
      
      for (frame = 0, d = (kubyte *) vdata; 
	   frame < resources->arf->hdr._num_frames; 
	   frame++, d += frame_size * kdata_size(type))
      {
	 /*-- write out the frame... --*/
	 kwrite_generic(fid, (kaddr) d, (size_t) frame_size, type);

	 /*-- ... and write out the footer data --*/
	 if (subh->frame_data.footer_flags & ARF_FRAME_INFO_FLAG)
	 {
	    resources->frame_footer->info.x = 
	       footer_attributes.frame_info_x[frame] ;
	    resources->frame_footer->info.y = 
	       footer_attributes.frame_info_y[frame] ;
	    resources->frame_footer->info.azimuth = 
	       footer_attributes.frame_info_az[frame];
	    resources->frame_footer->info.elevation = 
	       footer_attributes.frame_info_el[frame] ;
	    resources->frame_footer->info.irig_time.yy =
	       footer_attributes.frame_info_irig_yy[frame];
	    resources->frame_footer->info.irig_time.dd =
	       footer_attributes.frame_info_irig_dd[frame];
	    resources->frame_footer->info.irig_time.hh =
	       footer_attributes.frame_info_irig_hh[frame];
	    resources->frame_footer->info.irig_time.mm = 
	       footer_attributes.frame_info_irig_mm[frame];
	    resources->frame_footer->info.irig_time.ss =
	       footer_attributes.frame_info_irig_ss[frame];
	    resources->frame_footer->info.irig_time.ms =
	       footer_attributes.frame_info_irig_ms[frame];
	 }
	 
	 if (subh->frame_data.footer_flags & ARF_PLATFORM_ATTITUDE_FLAG)
	 {
	    resources->frame_footer->attitude.vel_north =
	       footer_attributes.platform_attitude_vel_n[frame];
	    resources->frame_footer->attitude.vel_east =
	       footer_attributes.platform_attitude_vel_e[frame];
	    resources->frame_footer->attitude.vel_vertical =
	       footer_attributes.platform_attitude_vel_v[frame];
	    resources->frame_footer->attitude.acc_north =
	       footer_attributes.platform_attitude_acc_n[frame];
	    resources->frame_footer->attitude.acc_east =
	       footer_attributes.platform_attitude_acc_e[frame];
	    resources->frame_footer->attitude.acc_vertical =
	       footer_attributes.platform_attitude_acc_v[frame];
	    resources->frame_footer->attitude.latitude =
	       footer_attributes.platform_attitude_lat[frame];
	    resources->frame_footer->attitude.longitude =
	       footer_attributes.platform_attitude_long[frame];
	    resources->frame_footer->attitude.altitude =
	       footer_attributes.platform_attitude_alt[frame];
	    resources->frame_footer->attitude.elevation =
	       footer_attributes.platform_attitude_elev[frame];
	    resources->frame_footer->attitude.bank =
	       footer_attributes.platform_attitude_bank[frame];
	    resources->frame_footer->attitude.heading =
	       footer_attributes.platform_attitude_head[frame];
	 }
	 resources->frame_foot_size =
	    arf_put_footer(fid, 
			   resources->subh->frame_data.footer_flags, 
			   resources->frame_footer);
      } /* frame loop */

      kfree_and_NULL(vdata);

   } /* frame_data_size if */
   
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: arf_destroy - Frees an arf image
|
|       Purpose: This function is used to destroy an arf image.  
|
|         Input: data - initial sample of data
|                num  - number of bytes in the sample
|
|        Output: Returns: returns TRUE or FALSE if an error occurs
|
|    Written By: Tim Williams (from Mark Young's viff_destroy)
|          Date: 18 May 1994
| Modifications:
|
------------------------------------------------------------*/
static int
arf_destroy(kobject object)
{
   ResourceStruct *resources;

   resources = (ResourceStruct *) kdms_glue_get_resources(object);

   /*-- free structures --*/
   if (resources->arf)
     kfree(resources->arf);
   
   if (resources->subh)
     kfree(resources->subh);

   kfree(resources);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: arf_prep - prepare an object for output to arf
|
|       Purpose: This function is used to prepare an object for
|		 output to arf.  It will print out information 
|		 about what is not getting stored out to the arf
|		 file due to the limitations of the arf format.	
|
|         Input: object - the object to be prepared
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: May 18, 1996
|
------------------------------------------------------------*/
/*ARGSUSED2*/
static int
arf_prep(kobject object)
{
   char *warn = NULL;
   

   /*-----------------------------------------------------------------------
   |  mask, location, and time data
   \-----------------------------------------------------------------------*/
   if (kpds_query_mask(object))
      warn = kstring_append(warn, "- no support for storing mask "
			    ": mask segment ignored.\n");

   if (kpds_query_location(object))
      warn = kstring_append(warn, "- no support for storing location "
			    ": location segment ignored.\n");

   if (kpds_query_time(object))
      warn = kstring_append(warn, "- no support for storing time "
			    ": time segment ignored.\n");

   /*-- not sure what other Arf limitations should be reported --*/

   /*-- if we have any news to report, report it now --*/
   if (warn)
   {
      kinfo(KSTANDARD, "ARF format output :\n%s", warn);
      kfree_and_NULL(warn);
   }

   return TRUE;
}
 
/*-----------------------------------------------------------
|
|  Routine Name: define_arf_..._attributes
|
|       Purpose: The following functions are called to predefine
|                all application specific attributes that
|                are specifically needed by arf glue.
|
|         Input: 
|
|        Output: 
|
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By: Tim Williams (modeled after Jeremy Worley's
|                viff_define_attributes)
|          Date: 25 July 1994
| Modifications:
|
------------------------------------------------------------*/
static int define_arf_colormap_attributes(void)
{
  static int defed=FALSE;

  if (defed)
    return TRUE;

   if (!kdms_query_attribute_definition(KDMS_OBJECT, _INTERNAL_COLORSPACE))
      kdms_define_attribute(KDMS_OBJECT, _INTERNAL_COLORSPACE, 1, 1, KINT,
                            TRUE, TRUE, KCOLOR_COLORSPACE_DEFAULT);

  defed = TRUE;
  return TRUE;
}
  
static int define_arf_comment_attribute(void)
{
  static int defed=FALSE;

  if (defed)
    return TRUE;

  if (!kdms_query_attribute_definition(KDMS_OBJECT, KDMS_COMMENT))
    kdms_define_attribute(KDMS_OBJECT, KDMS_COMMENT, 1, 1, KSTRING,
			  TRUE, TRUE,
                            NULL);
  defed = TRUE;
  return TRUE;
}


/*-----------------------------------------------------------
|
|  Routine Name: malloc_attribute_arrays
|
|       Purpose: allocates space for attribute arrays
|                needed for ARF footers
|                
|
|         Input: footer_flags - footer flags
|                num_frames - number of frames in ARF image
|                footer_attributes - ARF_FOOTER_ATTRIBUTES
|                                    containing arrays of arf footer
|                                    attributes
|
|	 Output: pointers to attribute arrays
|
|       Returns: TRUE (1), FALSE (0) on failure
|
|    Written By: Tim Williams 
|          Date:  1 July 1994
| Modifications:
|                18 July 1994 - Tim Williams
|                               created ARF_FOOTER_ATTRIBUTES structure
|                               to hold footer attribute arrays
|
------------------------------------------------------------*/
static int 
malloc_attribute_arrays(unsigned long footer_flags, 
			unsigned long num_frames,
			ARF_FOOTER_ATTRIBUTES *footer_attributes)
{
  if (footer_flags & ARF_FRAME_INFO_FLAG)
  {
    footer_attributes->frame_info_x = 
       (unsigned long *)kmalloc((size_t)num_frames * sizeof(unsigned long));
    if (footer_attributes->frame_info_x == NULL) return FALSE;
    
    footer_attributes->frame_info_y = 
       (unsigned long *)kmalloc((size_t)num_frames * sizeof(unsigned long));
    if (footer_attributes->frame_info_y == NULL) return FALSE;
    
    footer_attributes->frame_info_az = 
       (float *)kmalloc((size_t)num_frames * sizeof(float));
    if (footer_attributes->frame_info_az == NULL) return FALSE;
    
    footer_attributes->frame_info_el = 
       (float *)kmalloc((size_t)num_frames * sizeof(float));
    if (footer_attributes->frame_info_el == NULL) return FALSE;
    
    footer_attributes->frame_info_irig_yy = 
       (unsigned short *)kmalloc((size_t)num_frames * sizeof(unsigned short));
    if (footer_attributes->frame_info_irig_yy == NULL) return FALSE;
    
    footer_attributes->frame_info_irig_dd = 
       (unsigned short *)kmalloc((size_t)num_frames * sizeof(unsigned short));
    if (footer_attributes->frame_info_irig_dd == NULL) return FALSE;
    
    footer_attributes->frame_info_irig_hh = 
       (unsigned short *)kmalloc((size_t)num_frames * sizeof(unsigned short));
    if (footer_attributes->frame_info_irig_hh == NULL) return FALSE;
    
    footer_attributes->frame_info_irig_mm = 
      (unsigned short *)kmalloc((size_t)num_frames * sizeof(unsigned short));
    if (footer_attributes->frame_info_irig_mm == NULL) return FALSE;
    
    footer_attributes->frame_info_irig_ss = 
      (unsigned short *)kmalloc((size_t)num_frames * sizeof(unsigned short));
    if (footer_attributes->frame_info_irig_ss == NULL) return FALSE;
    
    footer_attributes->frame_info_irig_ms = 
      (unsigned short *)kmalloc((size_t)num_frames * sizeof(unsigned short));
    if (footer_attributes->frame_info_irig_ms == NULL) return FALSE;

  } /* ARF_FRAME_INFO_FLAG */
  
  if (footer_flags & ARF_PLATFORM_ATTITUDE_FLAG)
  {
    footer_attributes->platform_attitude_vel_n = 
       (double *)kmalloc((size_t) num_frames * sizeof(double));
    if (footer_attributes->platform_attitude_vel_n == NULL) return FALSE;
    
    footer_attributes->platform_attitude_vel_e = 
       (double *)kmalloc((size_t) num_frames * sizeof(double));
    if (footer_attributes->platform_attitude_vel_e == NULL) return FALSE;
    
    footer_attributes->platform_attitude_vel_v = 
       (double *)kmalloc((size_t) num_frames * sizeof(double));
    if (footer_attributes->platform_attitude_vel_v == NULL) return FALSE;
    
    footer_attributes->platform_attitude_acc_n = 
       (double *)kmalloc((size_t) num_frames * sizeof(double));
    if (footer_attributes->platform_attitude_acc_n == NULL) return FALSE;
    
    footer_attributes->platform_attitude_acc_e = 
       (double *)kmalloc((size_t) num_frames * sizeof(double));
    if (footer_attributes->platform_attitude_acc_e == NULL) return FALSE;
    
    footer_attributes->platform_attitude_acc_v = 
      (double *)kmalloc((size_t) num_frames * sizeof(double));
    if (footer_attributes->platform_attitude_acc_v == NULL) return FALSE;
    
    footer_attributes->platform_attitude_lat = 
      (double *)kmalloc((size_t) num_frames * sizeof(double));
    if (footer_attributes->platform_attitude_lat == NULL) return FALSE;
    
    footer_attributes->platform_attitude_long = 
      (double *)kmalloc((size_t) num_frames * sizeof(double));
    if (footer_attributes->platform_attitude_long == NULL) return FALSE;
    
    footer_attributes->platform_attitude_alt = 
      (double *)kmalloc((size_t) num_frames * sizeof(double));
    if (footer_attributes->platform_attitude_alt == NULL) return FALSE;
    
    footer_attributes->platform_attitude_elev = 
      (double *)kmalloc((size_t) num_frames * sizeof(double));
    if (footer_attributes->platform_attitude_elev == NULL) return FALSE;
    
    footer_attributes->platform_attitude_bank = 
      (double *)kmalloc((size_t) num_frames * sizeof(double));
    if (footer_attributes->platform_attitude_bank == NULL) return FALSE;
    
    footer_attributes->platform_attitude_head = 
      (double *)kmalloc((size_t) num_frames * sizeof(double));
    if (footer_attributes->platform_attitude_head == NULL) return FALSE;

  } /* ARF_PLATFORM_ATTITUDE_FLAG */
  
  return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: create_arf_footer_attributes
|
|       Purpose: creates ARF footer attributes
|
|         Input: object - the ARF object containing the footers
|                footer_flags - footer flags
|                num_frames - number of frames in ARF image
|
|	 Output: None
|
|       Returns: TRUE (1), FALSE (0) on failure
|
|    Written By: Tim Williams 
|          Date:  1 July 1994
| Modifications:
|
------------------------------------------------------------*/
static int 
create_arf_footer_attributes(
   kobject       object, 
   unsigned long footer_flags, 
   unsigned long num_frames)
{
  if (footer_flags && ARF_FRAME_INFO_FLAG)
  {
    if (!kdms_create_attribute(object, NULL, "ARF_FRAME_INFO_x",
			       1, (int)num_frames, KULONG,
			       TRUE, TRUE)) 
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_FRAME_INFO_y",
			       1, (int)num_frames, KULONG,
			       TRUE, TRUE)) 
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_FRAME_INFO_az",
			       1, (int)num_frames, KFLOAT,
			       TRUE, TRUE)) 
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_FRAME_INFO_el",
			       1, (int)num_frames, KFLOAT,
			       TRUE, TRUE))  
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_FRAME_INFO_irig_yy",
			       1, (int)num_frames, KUSHORT,
			       TRUE, TRUE)) 
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_FRAME_INFO_irig_dd",
			       1, (int)num_frames, KUSHORT,
			       TRUE, TRUE))
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_FRAME_INFO_irig_hh",
			       1, (int)num_frames, KUSHORT,
			       TRUE, TRUE)) 
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_FRAME_INFO_irig_mm",
			       1, (int)num_frames, KUSHORT,
			       TRUE, TRUE)) 
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_FRAME_INFO_irig_ss",
			       1, (int)num_frames, KUSHORT,
			       TRUE, TRUE)) 
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_FRAME_INFO_irig_ms",
			       1, (int)num_frames, KUSHORT,
			       TRUE, TRUE)) 
      return FALSE;

  } /* ARF_FRAME_INFO_FLAG */
  
  if (footer_flags && ARF_PLATFORM_ATTITUDE_FLAG)
  {
    if (!kdms_create_attribute(object, NULL, "ARF_PLATFORM_ATTITUDE_vel_n",
			       1, (int)num_frames, KDOUBLE,
			       TRUE, TRUE)) 
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_PLATFORM_ATTITUDE_vel_e",
			       1, (int)num_frames, KDOUBLE,
			       TRUE, TRUE))
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_PLATFORM_ATTITUDE_vel_v",
			       1, (int)num_frames, KDOUBLE,
			       TRUE, TRUE))
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_PLATFORM_ATTITUDE_acc_n",
			       1, (int)num_frames, KDOUBLE,
			       TRUE, TRUE)) return FALSE;
    
    if (!kdms_create_attribute(object, NULL, "ARF_PLATFORM_ATTITUDE_acc_e",
			       1, (int)num_frames, KDOUBLE,
			       TRUE, TRUE)) 
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_PLATFORM_ATTITUDE_acc_v",
			       1, (int)num_frames, KDOUBLE,
			       TRUE, TRUE)) 
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_PLATFORM_ATTITUDE_lat",
			       1, (int)num_frames, KDOUBLE,
			       TRUE, TRUE)) 
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_PLATFORM_ATTITUDE_long",
			       1, (int)num_frames, KDOUBLE,
			       TRUE, TRUE)) 
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_PLATFORM_ATTITUDE_alt",
			       1, (int)num_frames, KDOUBLE,
			       TRUE, TRUE))
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_PLATFORM_ATTITUDE_elev",
			       1, (int)num_frames, KDOUBLE,
			       TRUE, TRUE)) 
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_PLATFORM_ATTITUDE_bank",
			       1, (int)num_frames, KDOUBLE,
			       TRUE, TRUE))
      return FALSE;
    if (!kdms_create_attribute(object, NULL, "ARF_PLATFORM_ATTITUDE_head",
			       1, (int)num_frames, KDOUBLE,
			       TRUE, TRUE)) 
      return FALSE;

  } /* ARF_PLATFORM_ATTITUDE_FLAG */

  return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: set_arf_footer_attributes
|
|       Purpose: sets ARF footer attributes
|
|         Input: object - the ARF object containing the footers
|                footer_attributes - ARF_FOOTER_ATTRIBUTES structure
|                                    containing footer attribute arrays
|                footer_flags - footer flags
|
|	 Output: None
|
|       Returns: TRUE (1), FALSE (0) on failure
|
|    Written By: Tim Williams 
|          Date:  18 July 1994
| Modifications:
|
------------------------------------------------------------*/
static int 
set_arf_footer_attributes(
   kobject                object,
   ARF_FOOTER_ATTRIBUTES *footer_attributes,
   unsigned long          footer_flags)
{
  if (footer_flags & ARF_FRAME_INFO_FLAG)
  {
    if (!kdms_set_attribute(object, NULL, 
			    "ARF_FRAME_INFO_x", 
			    footer_attributes->frame_info_x)) 
      return FALSE;
    if (!kdms_set_attribute(object, NULL, 
			    "ARF_FRAME_INFO_y", 
			    footer_attributes->frame_info_y)) 
      return FALSE;
    if (!kdms_set_attribute(object, NULL, 
			    "ARF_FRAME_INFO_el", 
			    footer_attributes->frame_info_el)) 
      return FALSE;
    if (!kdms_set_attribute(object, NULL, 
			    "ARF_FRAME_INFO_az", 
			    footer_attributes->frame_info_az)) 
      return FALSE;
    if (!kdms_set_attribute(object, NULL, 
			    "ARF_FRAME_INFO_irig_yy", 
			    footer_attributes->frame_info_irig_yy)) 
      return FALSE;
    if (!kdms_set_attribute(object, NULL, 
			    "ARF_FRAME_INFO_irig_dd", 
			    footer_attributes->frame_info_irig_dd)) 
      return FALSE;
    if (!kdms_set_attribute(object, NULL, 
			    "ARF_FRAME_INFO_irig_hh", 
			    footer_attributes->frame_info_irig_hh)) 
      return FALSE;
    if (!kdms_set_attribute(object, NULL, 
			    "ARF_FRAME_INFO_irig_mm",
			    footer_attributes->frame_info_irig_mm))
      return FALSE;
    if (!kdms_set_attribute(object, NULL, 
			    "ARF_FRAME_INFO_irig_ss",
			    footer_attributes->frame_info_irig_ss))
      return FALSE;
    if (!kdms_set_attribute(object, NULL, 
			    "ARF_FRAME_INFO_irig_ms",
			    footer_attributes->frame_info_irig_ms))
      return FALSE;
  }

  if (footer_flags & ARF_PLATFORM_ATTITUDE_FLAG)
  {
    if (!kdms_set_attribute(object, NULL,
			    "ARF_PLATFORM_ATTITUDE_vel_n",
			    footer_attributes->platform_attitude_vel_n))
      return FALSE;
    if (!kdms_set_attribute(object, NULL,
			    "ARF_PLATFORM_ATTITUDE_vel_e",
			    footer_attributes->platform_attitude_vel_e))
      return FALSE;
    if (!kdms_set_attribute(object, NULL,
			    "ARF_PLATFORM_ATTITUDE_vel_v",
			    footer_attributes->platform_attitude_vel_v))
      return FALSE;
    if (!kdms_set_attribute(object, NULL,
			    "ARF_PLATFORM_ATTITUDE_acc_n",
			    footer_attributes->platform_attitude_acc_n))
      return FALSE;
    if (!kdms_set_attribute(object, NULL,
			    "ARF_PLATFORM_ATTITUDE_acc_e",
			    footer_attributes->platform_attitude_acc_e))
      return FALSE;
    if (!kdms_set_attribute(object, NULL,
			    "ARF_PLATFORM_ATTITUDE_acc_v",
			    footer_attributes->platform_attitude_acc_v))
      return FALSE;
    if (!kdms_set_attribute(object, NULL, 
			    "ARF_PLATFORM_ATTITUDE_lat",
			    footer_attributes->platform_attitude_lat))
      return FALSE;
    if (!kdms_set_attribute(object, NULL, 
			    "ARF_PLATFORM_ATTITUDE_long",
			    footer_attributes->platform_attitude_long))
      return FALSE;
    if (!kdms_set_attribute(object, NULL, 
			    "ARF_PLATFORM_ATTITUDE_alt",
			    footer_attributes->platform_attitude_alt))
      return FALSE;
    if (!kdms_set_attribute(object, NULL, 
			    "ARF_PLATFORM_ATTITUDE_elev",
			    footer_attributes->platform_attitude_elev))
      return FALSE;
    if (!kdms_set_attribute(object, NULL, 
			    "ARF_PLATFORM_ATTITUDE_bank",
			    footer_attributes->platform_attitude_bank))
      return FALSE;
    if (!kdms_set_attribute(object, NULL, 
			    "ARF_PLATFORM_ATTITUDE_head",
			    footer_attributes->platform_attitude_head))
      return FALSE;
  }

  return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: get_arf_footer_attributes
|
|       Purpose: gets ARF footer attributes
|
|         Input: object - the ARF object containing the footers
|                footer_attributes - ARF_FOOTER_ATTRIBUTES structure
|                                    containing footer attribute arrays
|                footer_flags - footer flags
|
|	 Output: None
|
|       Returns: TRUE (1), FALSE (0) on failure
|
|    Written By: Tim Williams 
|          Date:  18 July 1994
| Modifications:
|
------------------------------------------------------------*/
static int 
get_arf_footer_attributes(
   kobject                object,
   ARF_FOOTER_ATTRIBUTES *footer_attributes,
   unsigned long          footer_flags)
{
  if (footer_flags & ARF_FRAME_INFO_FLAG)
  {
    if (!kdms_get_attribute(object, NULL, 
			    "ARF_FRAME_INFO_x", 
			    &footer_attributes->frame_info_x)) 
       return FALSE;
    if (!kdms_get_attribute(object, NULL, 
			    "ARF_FRAME_INFO_y", 
			    &footer_attributes->frame_info_y)) 
       return FALSE;
    if (!kdms_get_attribute(object, NULL, 
			    "ARF_FRAME_INFO_el", 
			    &footer_attributes->frame_info_el)) 
       return FALSE;
    if (!kdms_get_attribute(object, NULL, 
			    "ARF_FRAME_INFO_az", 
			    &footer_attributes->frame_info_az)) 
       return FALSE;
    if (!kdms_get_attribute(object, NULL, 
			    "ARF_FRAME_INFO_irig_yy",
			    &footer_attributes->frame_info_irig_yy)) 
      return FALSE;
    if (!kdms_get_attribute(object, NULL, 
			    "ARF_FRAME_INFO_irig_dd",
			    &footer_attributes->frame_info_irig_dd)) 
      return FALSE;
    if (!kdms_get_attribute(object, NULL, 
			    "ARF_FRAME_INFO_irig_hh",
			    &footer_attributes->frame_info_irig_hh))
      return FALSE;
    if (!kdms_get_attribute(object, NULL, 
			    "ARF_FRAME_INFO_irig_mm",
			    &footer_attributes->frame_info_irig_mm)) 
      return FALSE;
    if (!kdms_get_attribute(object, NULL, 
			    "ARF_FRAME_INFO_irig_ss",
			    &footer_attributes->frame_info_irig_ss)) 
      return FALSE;
    if (!kdms_get_attribute(object, NULL, 
			    "ARF_FRAME_INFO_irig_ms",
			    &footer_attributes->frame_info_irig_ms)) 
      return FALSE;
  }

  if (footer_flags & ARF_PLATFORM_ATTITUDE_FLAG)
  {
    if (!kdms_get_attribute(object, NULL,
			    "ARF_PLATFORM_ATTITUDE_vel_n",
			    &footer_attributes->platform_attitude_vel_n))
      return FALSE;
    if (!kdms_get_attribute(object, NULL,
			    "ARF_PLATFORM_ATTITUDE_vel_e",
			    &footer_attributes->platform_attitude_vel_e))
      return FALSE;
    if (!kdms_get_attribute(object, NULL,
			    "ARF_PLATFORM_ATTITUDE_vel_v",
			    &footer_attributes->platform_attitude_vel_v))
      return FALSE;
    if (!kdms_get_attribute(object, NULL,
			    "ARF_PLATFORM_ATTITUDE_acc_n",
			    &footer_attributes->platform_attitude_acc_n))
      return FALSE;
    if (!kdms_get_attribute(object, NULL,
			    "ARF_PLATFORM_ATTITUDE_acc_e",
			    &footer_attributes->platform_attitude_acc_e))
      return FALSE;
    if (!kdms_get_attribute(object, NULL,
			    "ARF_PLATFORM_ATTITUDE_acc_v",
			    &footer_attributes->platform_attitude_acc_v))
      return FALSE;
    if (!kdms_get_attribute(object, NULL, 
			    "ARF_PLATFORM_ATTITUDE_lat",
			    &footer_attributes->platform_attitude_lat))
      return FALSE;
    if (!kdms_get_attribute(object, NULL, 
			    "ARF_PLATFORM_ATTITUDE_long",
			    &footer_attributes->platform_attitude_long))
      return FALSE;
    if (!kdms_get_attribute(object, NULL, 
			    "ARF_PLATFORM_ATTITUDE_alt",
			    &footer_attributes->platform_attitude_alt))
      return FALSE;
    if (!kdms_get_attribute(object, NULL, 
			    "ARF_PLATFORM_ATTITUDE_elev",
			    &footer_attributes->platform_attitude_elev))
      return FALSE;
    if (!kdms_get_attribute(object, NULL, 
			    "ARF_PLATFORM_ATTITUDE_bank",
			    &footer_attributes->platform_attitude_bank))
      return FALSE;
    if (!kdms_get_attribute(object, NULL, 
			    "ARF_PLATFORM_ATTITUDE_head",
			    &footer_attributes->platform_attitude_head))
      return FALSE;
  }
  
  return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: create_arf_info_attributes
|
|       Purpose: creates ARF_INFO subheader attributes
|
|         Input: object - the ARF object containing the footers
|
|	 Output: None
|
|       Returns: TRUE (1), FALSE (0) on failure
|
|    Written By: Tim Williams 
|          Date:  18 July 1994
| Modifications:
|
------------------------------------------------------------*/
/*ARGSUSED*/
static int 
create_arf_info_attributes(
   kobject   object, 
   ARF_INFO *info)
{
  if (!kdms_create_attribute(object, NULL, "ARF_INFO_image_source", 1, 1,
			     KULONG, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_INFO_start_x", 1, 1,
			     KULONG, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_INFO_start_y", 1, 1,
			     KULONG, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_INFO_num_avg", 1, 1,
			     KULONG, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_INFO_capture_rate", 1, 1,
			     KULONG, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_INFO_capture_time_yy", 1, 1,
			     KUSHORT, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_INFO_capture_time_dd", 1, 1,
			     KUSHORT, TRUE, TRUE))
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_INFO_capture_time_hh", 1, 1,
			     KUSHORT, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_INFO_capture_time_mm", 1, 1,
			     KUSHORT, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_INFO_capture_time_ss", 1, 1,
			     KUSHORT, TRUE, TRUE)) return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_INFO_capture_time_ms", 1, 1,
			     KUSHORT, TRUE, TRUE)) return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_INFO_capture_loc", 1, 1,
			     KSTRING, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_INFO_sensor_name", 1, 1,
			     KSTRING, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_INFO_digitizer", 1, 1,
			     KSTRING, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_INFO_sensor_hor_fov", 1, 1,
			     KFLOAT, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_INFO_sensor_ver_fov", 1, 1,
			     KFLOAT, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_INFO_samples_per_dwell", 1, 1,
			     KULONG, TRUE, TRUE)) 
    return FALSE;
  
  return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: set_arf_info_attributes
|
|       Purpose: sets ARF_INFO subheader attributes
|
|         Input: object - the ARF object containing the footers
|                info   - ARF_INFO subheader
|
|	 Output: None
|
|       Returns: TRUE (1), FALSE (0) on failure
|
|    Written By: Tim Williams 
|          Date:  18 July 1994
| Modifications:
|
------------------------------------------------------------*/
static int 
set_arf_info_attributes(
   kobject object, 
   ARF_INFO *info)
{
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_INFO_image_source", info->image_source))
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_INFO_start_x", info->start_x)) 
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_INFO_start_y", info->start_y))
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_INFO_num_avg", info->num_avg)) 
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_INFO_capture_rate", info->capture_rate))
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_INFO_capture_time_yy", 
			  info->capture_time.yy)) 
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_INFO_capture_time_dd", 
			  info->capture_time.dd)) 
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_INFO_capture_time_hh", 
			  info->capture_time.hh)) 
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_INFO_capture_time_mm", 
			  info->capture_time.mm)) 
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_INFO_capture_time_ss", 
			  info->capture_time.ss)) 
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_INFO_capture_time_ms", 
			  info->capture_time.ms)) 
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_INFO_capture_loc",
			  info->capture_loc))
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_INFO_sensor_name",
			  info->sensor_name)) 
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_INFO_digitizer", 
			  info->digitizer)) 
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_INFO_sensor_hor_fov", 
			  info->sensor_hor_fov)) 
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_INFO_sensor_ver_fov", 
			  info->sensor_ver_fov))
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_INFO_samples_per_dwell", 
			  info->samples_per_dwell)) 
    return FALSE;
  
  return TRUE;
}


/*-----------------------------------------------------------
|
|  Routine Name: get_arf_info_attributes
|
|       Purpose: gets ARF_INFO subheader attributes
|
|         Input: object - the ARF object containing the footers
|                info   - ARF_INFO subheader
|
|	 Output: None
|
|       Returns: size in bytes of ARF_INFO subheader 
|                (with XDR padding on strings)
|                FALSE (0) on failure
|
|    Written By: Tim Williams 
|          Date:  18 July 1994
| Modifications:
|
------------------------------------------------------------*/
static int 
get_arf_info_attributes(
   kobject object, 
   ARF_INFO *info)
{
  unsigned long tmpval;
  int subh_length;
  char *string;
  
  if (!kdms_get_attribute(object, NULL, 
			  "ARF_INFO_image_source", &info->image_source))
    return FALSE;
  if (!kdms_get_attribute(object, NULL, 
			  "ARF_INFO_start_x", &info->start_x)) 
    return FALSE;
  if (!kdms_get_attribute(object, NULL, 
			  "ARF_INFO_start_y", &info->start_y))
    return FALSE;
  if (!kdms_get_attribute(object, NULL, 
			  "ARF_INFO_num_avg", &info->num_avg)) 
    return FALSE;
  if (!kdms_get_attribute(object, NULL, 
			  "ARF_INFO_capture_rate", &info->capture_rate))
    return FALSE;
  if (!kdms_get_attribute(object, NULL, 
			  "ARF_INFO_capture_time_yy", 
			  &info->capture_time.yy)) 
    return FALSE;
  if (!kdms_get_attribute(object, NULL, 
			  "ARF_INFO_capture_time_dd", 
			  &info->capture_time.dd)) 
    return FALSE;
  if (!kdms_get_attribute(object, NULL, 
			  "ARF_INFO_capture_time_hh", 
			  &info->capture_time.hh)) 
    return FALSE;
  if (!kdms_get_attribute(object, NULL, 
			  "ARF_INFO_capture_time_mm", 
			  &info->capture_time.mm)) 
    return FALSE;
  if (!kdms_get_attribute(object, NULL, 
			  "ARF_INFO_capture_time_ss", 
			  &info->capture_time.ss)) 
    return FALSE;
  if (!kdms_get_attribute(object, NULL, 
			  "ARF_INFO_capture_time_ms", 
			  &info->capture_time.ms)) 
    return FALSE;
  if (!kdms_get_attribute(object, NULL, 
			  "ARF_INFO_capture_loc", &string))
    return FALSE;
  else
    kstrcpy(info->capture_loc, string);
  if (!kdms_get_attribute(object, NULL, 
			  "ARF_INFO_sensor_name", &string))
    return FALSE;
  else
    kstrcpy(info->sensor_name, string);
  if (!kdms_get_attribute(object, NULL, 
			  "ARF_INFO_digitizer", &string))
    return FALSE;
  else
    kstrcpy(info->digitizer, string);
  if (!kdms_get_attribute(object, NULL, 
			  "ARF_INFO_sensor_hor_fov", 
			  &info->sensor_hor_fov)) 
    return FALSE;
  if (!kdms_get_attribute(object, NULL, 
			  "ARF_INFO_sensor_ver_fov", 
			  &info->sensor_ver_fov))
    return FALSE;
  if (!kdms_get_attribute(object, NULL, 
			  "ARF_INFO_samples_per_dwell", 
			  &info->samples_per_dwell)) 
    return FALSE;
  
  /** update the subhsize ***/
  /* 12 longs + 2 floats + 3 string lengths (ints) */
  subh_length = 68;
  
  tmpval = kstrlen(info->capture_loc);
  if (tmpval%4)
    tmpval += (4 - (tmpval%4));
  subh_length += tmpval;
  
  tmpval = kstrlen(info->sensor_name);
  if (tmpval%4) tmpval += (4 - (tmpval%4));
  subh_length += tmpval;
  
  tmpval = kstrlen(info->digitizer);
  if (tmpval%4) tmpval += (4 - (tmpval%4));
  subh_length += tmpval;
  
  return (subh_length);
}

/*-----------------------------------------------------------
|
|  Routine Name: create_arf_ladar_param_attributes
|
|       Purpose: creates ARF_LADAR_PARAM subheader attributes
|
|       Input: object - the ARF object containing the footers
|
|	 Output: None
|
|       Returns: TRUE (1), FALSE (0) on failure
|
|    Written By: Tim Williams 
|          Date:  18 July 1994
| Modifications:
|
------------------------------------------------------------*/
static int 
create_arf_ladar_param_attributes(kobject object)
{
  if (!kdms_create_attribute(object, NULL, "ARF_LADAR_PARAM_fm_scale", 1, 1,
			     KFLOAT, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, 
			     "ARF_LADAR_PARAM_resolved_range_bias",
			     1, 1,
			     KFLOAT, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_LADAR_PARAM_hor_ifov", 1, 1,
			     KFLOAT, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_LADAR_PARAM_ver_ifov", 1, 1,
			     KFLOAT, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_LADAR_PARAM_coarse_res", 1, 1,
			     KFLOAT, TRUE, TRUE)) 
    return FALSE;
  if (!kdms_create_attribute(object, NULL, "ARF_LADAR_PARAM_fine_res", 1, 1,
			     KFLOAT, TRUE, TRUE)) 
    return FALSE;
  
  return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: set_arf_ladar_param_attributes
|
|       Purpose: sets ARF_LADAR_PARAM subheader attributes
|
|         Input: object      - the ARF object containing the footers
|                ladar_param - ARF_LADAR_PARAM subheader
|
|	 Output: None
|
|       Returns: TRUE (1), FALSE (0) on failure
|
|    Written By: Tim Williams 
|          Date:  18 July 1994
| Modifications:
|
------------------------------------------------------------*/
static int 
set_arf_ladar_param_attributes(
   kobject object, 
   ARF_LADAR_PARAM *ladar_param)
{
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_LADAR_PARAM_fm_scale", 
			  ladar_param->fm_scale)) 
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_LADAR_PARAM_resolved_range_bias", 
			  ladar_param->resolved_range_bias)) 
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_LADAR_PARAM_hor_ifov", 
			  ladar_param->hor_ifov)) 
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_LADAR_PARAM_ver_ifov", 
			  ladar_param->ver_ifov)) 
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_LADAR_PARAM_coarse_res", 
			  ladar_param->coarse_res)) 
    return FALSE;
  if (!kdms_set_attribute(object, NULL, 
			  "ARF_LADAR_PARAM_fine_res", 
			  ladar_param->fine_res)) 
    return FALSE;
  
  return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: get_arf_ladar_param_attributes
|
|       Purpose: gets ARF_LADAR_PARAM subheader attributes
|
|         Input: object      - the ARF object containing the footers
|                ladar_param - ARF_LADAR_PARAM subheader
|
|	 Output: None
|
|       Returns: size in bytes of ARF_LADAR_PARAM subheader
|                FALSE (0) on failure
|
|    Written By: Tim Williams 
|          Date:  18 July 1994
| Modifications:
|
------------------------------------------------------------*/
static int 
get_arf_ladar_param_attributes(
   kobject object, 
   ARF_LADAR_PARAM *ladar_param)
{
  int subh_length;
  
  subh_length=0;
  if (kdms_get_attribute(object, NULL, 
			 "ARF_LADAR_PARAM_fm_scale", 
			  ladar_param->fm_scale)) 
    subh_length +=4;
  else
    return FALSE;

  if (kdms_get_attribute(object, NULL, 
			 "ARF_LADAR_PARAM_resolved_range_bias", 
			 ladar_param->resolved_range_bias)) 
    subh_length += 4;
  else
    return FALSE;
  
  if (kdms_get_attribute(object, NULL, 
			 "ARF_LADAR_PARAM_hor_ifov", 
			 ladar_param->hor_ifov)) 
    subh_length += 4;
  else
    return FALSE;

  if (kdms_get_attribute(object, NULL, 
			 "ARF_LADAR_PARAM_ver_ifov", 
			 ladar_param->ver_ifov)) 
    subh_length += 4;
  else
    return FALSE;
  
  if (kdms_set_attribute(object, NULL, 
			 "ARF_LADAR_PARAM_coarse_res", 
			 ladar_param->coarse_res)) 
    subh_length += 4;
  else
    return FALSE;
  
  if (kdms_set_attribute(object, NULL, 
			 "ARF_LADAR_PARAM_fine_res", 
			 ladar_param->fine_res)) 
    subh_length += 4;
  else
    return FALSE;
  
  return(subh_length);
}


#endif	/* KARF_DEF */

/* don`t add after the endif */
