 /*
  * Khoros: $Id: Gdbm.c,v 1.8 1996/06/26 14:45:56 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Gdbm.c,v 1.8 1996/06/26 14:45:56 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros GDBM Data Service Routines
   >>>>		
   >>>>  Private:
   >>>>                 init_gdbm()
   >>>>                 init_gdbm()
   >>>>                 gdbm_reader()
   >>>>                 gdbm_writer()
   >>>>                 gdbm_seeker()
   >>>>   Static:
   >>>>                 _init_cookies()
   >>>>                 _stupid()
   >>>>                 _sort_keys()
   >>>>                 _get_all_keys()
   >>>>                 _get_attribute()
   >>>>                 _compose_key()
   >>>>                 _decompose_key()
   >>>>                 gdbm_check()
   >>>>                 gdbm_input()
   >>>>                 gdbm_destroy()
   >>>>                 gdbm_locate_segment()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/*-- should move this to be an install option... --*/
#if !defined(KGDBM_DEF)
#define KGDBM_DEF 1
#endif

#if !defined(KGDBM_DEF)

void
init_gdbm(void) /*-- NULL gdbm initialization --*/
{
   return;
}

#else

static int     	gdbm_check  	    PROTO((int));
static int     	gdbm_input  	    PROTO((kobject, int, int));
static int     	gdbm_destroy        PROTO((kobject));
static int     	gdbm_locate_segment PROTO((kobject, char *));
static ssize_t 	gdbm_reader   	    PROTO((int, kaddr, size_t, int));
static ssize_t 	gdbm_writer  	    PROTO((int, kaddr, size_t, int));
static off_t   	gdbm_seeker  	    PROTO((int, off_t, int));
static long int gdbm_teller         PROTO((int));

static kdatum version_tag;
static kdatum version_val;
static kdatum cookie_tag;
static kdatum cookie_val;

#define VIFF_VERSION_TAG "Version"
#define VIFF_COOKIE_TAG  "ViffMagicCookie"
#define VIFF_VERSION_VAL "2.0b"
#define VIFF_COOKIE_VAL  "ThisIsAViff"

#define INCREMENT 10

static klist *gdb_list;  /*-- list keyed on file descriptors --*/ 

/*=====================================================================*/

/*-----------------------------------------------------------
|  Routine Name: init_gdbm
|       Purpose: initialize the gdbm file format
------------------------------------------------------------*/
void
init_gdbm(void)
{
   kdms_add_file_format("gdbm", 
			"Khoros 2.0 Visualization Image File Format (gdbm)",
			gdbm_check, 
			gdbm_input, NULL,
			gdbm_destroy, NULL,
			gdbm_locate_segment, 
			gdbm_reader, gdbm_writer, gdbm_seeker, gdbm_teller);
   return;
}

/*=====================================================================*/

/*
 *  Internal Resource Structure 
 *  gdbm  -  New Khoros 2.0 Visualization Image File Format
 */
typedef struct
{
   int    fid;
   kdbm  *db;
   kdatum datum;                /* monkey trick */
}
ResourceStruct;

/*-----------------------------------------------------------
|  Routine Name: (static) _init_cookies
|       Purpose: initialize some often-needed version cookies
|    Written By: Jeremy Worley
|          Date: November 15, 1993
------------------------------------------------------------*/
static void
_init_cookies(void)
{
   if (version_tag.dptr != NULL)
      return;

   version_tag.dptr  = kstring_copy(VIFF_VERSION_TAG, NULL);
   version_tag.dsize = kstrlen(version_tag.dptr) + 1;

   version_val.dptr  = kstring_copy(VIFF_VERSION_VAL, NULL);
   version_val.dsize = kstrlen(version_val.dptr) + 1;

   cookie_tag.dptr   = kstring_copy(VIFF_COOKIE_TAG, NULL);
   cookie_tag.dsize  = kstrlen(cookie_tag.dptr) + 1;

   cookie_val.dptr   = kstring_copy(VIFF_COOKIE_VAL, NULL);
   cookie_val.dsize  = kstrlen(cookie_val.dptr) + 1;

   return;
}

/*-----------------------------------------------------------
|  Routine Name: (static) _sort_keys
|       Purpose: Maybe this is stupid, but if we encode
|                the keys correctly, then sorting them could
|                be very beneficial
|    Written By: Jeremy Worley
|          Date: November 8, 1993
------------------------------------------------------------*/
static int
_stupid(const void *a, const void *b)
{
   kdatum *da = (kdatum *) a,
     *db = (kdatum *) b;

   return (kstrcmp((char *)da->dptr, (char *)db->dptr));
}

static int
_sort_keys(kdatum * keys, int num)
{
   (void)qsort(keys, (size_t)num, sizeof(kdatum), _stupid);
   return (TRUE);
}

/*-----------------------------------------------------------
|  Routine Name: _get_all_keys
|       Purpose: This silly thing gets a list of all keys
|                in the object.
|    Written By: Jeremy Worley
|          Date: November 8, 1993
------------------------------------------------------------*/
static kdatum *
_get_all_keys(kdbm * db, int *num)
{
   kdatum *tmp;
   size_t  amount;
   int     n = INCREMENT;

   tmp = kmalloc((size_t)n * sizeof(kdatum));

   if (tmp == NULL)
      return (FALSE);

   *num = 0;
   tmp[*num] = kdbm_firstkey(db);
   tmp[*num].dptr = kstrdup(tmp[*num].dptr);

   do
   {
      (*num)++;
      if (*num == n)
      {
         n += INCREMENT;
         tmp = (kdatum *) krealloc(tmp, (size_t)n * sizeof(kdatum));

         if (tmp == NULL)
            return (FALSE);
      }
      tmp[*num] = kdbm_nextkey(db);
      if (tmp[*num].dptr != NULL)
         tmp[*num].dptr = kstrdup(tmp[*num].dptr);
   }
   while (tmp[*num].dptr != NULL);

   amount = (size_t) (*num) * sizeof(kdatum);
   tmp = (kdatum *) krealloc(tmp, amount);
   if (tmp == NULL)
      return (FALSE);

   _sort_keys(tmp, *num);

   return (tmp);
}

/*-----------------------------------------------------------
|  Routine Name: _get_attribute
|       Purpose: get a single attribute
|    Written By: Jeremy Worley
|          Date: November 17, 1993
------------------------------------------------------------*/
static int 
_get_attribute(
   kdbm  *db,
   kdatum da,
   int 	 *num,
   int 	 *size,
   int 	 *type,
   kaddr *data)
{
   int sz;

   kdbm_read(db, da, num, 1, KINT);
   kdbm_read(db, da, size, 1, KINT);
   kdbm_read(db, da, type, 1, KINT);

   sz = (*num) * (*size);

   *data = kcalloc((size_t) 1, (size_t)(sz * kdata_size(*type)));

   kdbm_read(db, da, *data, sz, *type);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: _compose_key
|
|       Purpose: This composes a database key from the 
|		 segment and attribute name.  The possible
|		 permutations are : 
|
|                  segment:
|                  segment:attribute
|                	  :attribute
|
|         Input: segment   - segment name or NULL
|		 attribute - attribute name or NULL
|
|        Output: key - composed key, allocated for you 
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley
|          Date: November 8, 1993
| Modifications:
|
------------------------------------------------------------*/
static int
_compose_key(
   char   *segment,
   char   *attribute,
   kdatum *key)
{
   key->dptr = kstring_3cat(segment, ":", attribute, NULL);
   key->dsize = kstrlen(key->dptr) + 1;

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) _decompose_key
|
|       Purpose: This decomposes a database key back into 
|		 a segment and attribute name.  The possible
|		 permutations are : 
|
|                   segment:
|                   segment:attribute
|                	   :attribute
|
|         Input: key - key to decompose
|
|        Output: segment   - segment name that was encoded
|		 attribute - attribute name that was encoded
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley
|          Date: November 8, 1993
| Modifications:
|
------------------------------------------------------------*/
static int
_decompose_key(
   kdatum key,
   char **segment,
   char **attribute)
{
   char *i1;
   char *tmp;

   if (segment != NULL)
      *segment = NULL;

   if (attribute != NULL)
      *attribute = NULL;

   tmp = key.dptr;

   /*
    * rule 1:  if the first substring is > 0, then it is a segment
    */
   if ((i1 = kstrchr(tmp, ':')) == NULL)
      return FALSE;

   if (i1 - tmp > 0)
   {
      *segment = kstring_ncopy(tmp, (size_t)(i1 - tmp), NULL);
      tmp = i1 + 1;
   }
   else
   {
      tmp = i1 + 1;
   }

   /*
    * rule 2:  everything that is left is an attribute
    */
   if (kstrlen(tmp) > 0)
      *attribute = kstring_copy(tmp, NULL);

   return TRUE;
}


/****    =============================================================
 ****    Format Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: gdbm_check - Checks to see if the file is a gdbm
|
|       Purpose: This function is used to check the first few bytes of
|                the stream to see if it is a gdbm.
|
|         Input: fid - file id of a file to check
|        Output: none
|       Returns: returns TRUE if the file is a gdbm
|
|    Written By: Jeremy Worley
|          Date: November 8, 1993
|
------------------------------------------------------------*/
static int
gdbm_check(int fid)
{
   kdbm  *db;
   kdatum tmp;

   /*-- kdbm has this cool function that does most of my work for me --*/
   if (!kdbm_check(fid))
      return FALSE;

   db = kdbm_fdopen(fid, (int) kfile_flags(fid), (int) kfile_mode(fid));

   if (db == NULL)
      return (FALSE);

   /*------------------------------------------------------------------
   | since it definitely was a db file, we gotta have the version and
   | cookie information initialized so that we can complete the check.
   \------------------------------------------------------------------*/
   _init_cookies();

   /*-- try to fetch a couple of magic fields out of the kdbm --*/
   tmp = kdbm_fetch(db, version_tag);
   if (tmp.dptr == 0)
      return FALSE;

   tmp = kdbm_fetch(db, cookie_tag);
   if (tmp.dptr == 0)
      return FALSE;

   /*-- well, it looks like a kdbm... --*/
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: gdbm_input
|
|       Purpose: This function is used to read in the image if the
|                supplied data is a gdbm file.  We simply read in 
|		 all keys header and initialize all segments
|		 and attributes 
|
|         Input: object - object to read viff into 
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley
|          Date: November 8, 1993
|
------------------------------------------------------------*/
static int
gdbm_input(kobject object, int fid, int flags)
{
   ResourceStruct *resources;

   kdatum *keys;
   kdatum  tmp;
   int     num_keys;
   int     i, j;
   int     nargs, asize, dtype;
   kaddr   data;
   int 	   val_order[5] = {KWIDTH, KHEIGHT, KDEPTH, KTIME, KELEMENTS};
   int 	   loc_order[4] = {KWIDTH, KHEIGHT, KDEPTH, KDIMENSION};
   int 	   map_order[5] = {KMAP_WIDTH, KMAP_HEIGHT, KMAP_DEPTH, KMAP_TIME,
			   KMAP_ELEMENTS};


   /*-- get the resource structure --*/ 
   if (!kdms_initialized(object))
   {
      if ((resources = kcalloc((size_t)1, sizeof(ResourceStruct))) == NULL)
         return(FALSE);
      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   /*-- destroy needs this information --*/
   resources->fid = fid;

   /*-- gotta have some magic cookies --*/
   _init_cookies();

   /*-- open the object as a dbm file.  if this fails, then fail --*/
   if ((resources->db = kdbm_fdopen(fid, flags | KOPEN_LOCK, 0666)) == NULL)
   {
      /* _kdms_set_error(KDMS_EFMT_OPENFAIL); */
      return FALSE;
   }

   /*-- associate this db with this file descriptor --*/
   gdb_list = klist_add(gdb_list, (kaddr) (long) fid, (kaddr) resources);

   /*-- get list of keys in dbm.  these will be attributes and segments --*/
   keys = _get_all_keys(resources->db, &num_keys);

   for (i = 0; i < num_keys; i++)
   {
      char *seg, *attr;

      if (!_decompose_key(keys[i], &seg, &attr))
         continue;

      /*-- if this name a segment, and we don't have it yet, create it --*/ 
      if (kstrlen(seg) != 0 && !kdms_query_segment(object, seg))
      {
         int size[KDMS_MAX_DIM];
	 int order[KDMS_MAX_DIM];
	 int datatype;
	 int dimension;

	 /*-- get description of segment --*/
         _compose_key(seg, KDMS_DIMENSION, &tmp);
         _get_attribute(resources->db, tmp, &nargs, &asize, &dtype, &data);
         dimension = *(int *) data;
         kfree(tmp.dptr);  /* compose_key allocs */
         kfree(data);      /* get attribute allocs, but this is special case */

         _compose_key(seg, KDMS_DATA_TYPE, &tmp);
         _get_attribute(resources->db, tmp, &nargs, &asize, &dtype, &data);
         datatype = *(int *) data;
         kfree(tmp.dptr); /* compose_key allocs */
         kfree(data);     /* get attribute allocs, but this is special case */

         _compose_key(seg, KDMS_SIZE, &tmp);
         _get_attribute(resources->db, tmp, &nargs, &asize, &dtype, &data);
         for (j = 0; j < dimension; j++)
            size[j] = ((int *)data)[j];
         kfree(tmp.dptr); /* compose_key allocs */
         kfree(data);     /* get attribute allocs, but this is special case */

         _compose_key(seg, KDMS_INDEX_ORDER, &tmp);
         _get_attribute(resources->db, tmp, &nargs, &asize, &dtype, &data);
         for (j = 0; j < dimension; j++)
            order[j] = ((int *)data)[j];
         kfree(tmp.dptr); /* compose_key allocs */
         kfree(data);     /* get attribute allocs, but this is special case */

	 kdms_set_segment(object, seg, dimension, size, order, datatype, NULL);

	 /*-- for polymophic segments : set default index orders --*/
         if (kstrcmp(seg, KDMS_SEGMENT_VALUE) == 0) 
            kdms_set_attribute(object, KDMS_SEGMENT_VALUE, 
			       KDMS_INDEX_ORDER, val_order);

         else if (kstrcmp(seg, KDMS_SEGMENT_MASK) == 0) 
            kdms_set_attribute(object, KDMS_SEGMENT_MASK, /* same as value */
				       KDMS_INDEX_ORDER, val_order);

         else if (kstrcmp(seg, KDMS_SEGMENT_LOCATION) == 0)
            kdms_set_attribute(object, KDMS_SEGMENT_LOCATION, 
				       KDMS_INDEX_ORDER, loc_order);

         else if (kstrcmp(seg, KDMS_SEGMENT_MAP) == 0) 
            kdms_set_attribute(object, KDMS_SEGMENT_LOCATION, 
				       KDMS_INDEX_ORDER, map_order);
      }

      /*-- if this is an attribute, then read it in --*/
      if (kstrlen(attr) != 0)
      {                         /* (attr != NULL) {   SK & JMAS */

         /*-- don't bother if this is one of the "special" attributes --*/
         if (kstrcmp(KDMS_SIZE,        attr) &&
	     kstrcmp(KDMS_INDEX_ORDER, attr) && 
	     kstrcmp(KDMS_DATA_TYPE,   attr) &&
             kstrcmp(KDMS_DIMENSION,   attr))
         {
            _get_attribute(resources->db, keys[i], &nargs, &asize, &dtype, 
			   &data);

            if (!kdms_query_attribute(object, seg, attr, NULL, NULL, 
				      NULL, NULL))
               kdms_create_attribute(object, seg, attr, nargs, asize, dtype,
                                     TRUE, TRUE);

            kdms_set_Vattribute(object, seg, attr, data);
         }
      }

      /*--- the decompose_key function allocates this space, so free it --*/
      kfree_and_NULL(seg);
      kfree_and_NULL(attr);
   }

   /*-- the get_all_keys function allocates its space, so free it --*/
   for (i = 0; i < num_keys; i++)
      kfree_and_NULL(keys[i].dptr);

   kfree_and_NULL(keys);

   /*-- all done --*/
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: gdbm_destroy
|
|       Purpose: 
|
|         Input: 
|
|        Output: 
|
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By: Jeremy Worley
|          Date: Nov 08, 1993 11:21
| Modifications:
|
------------------------------------------------------------*/
static int
gdbm_destroy(kobject object)
{
   ResourceStruct *resources;

   resources = (ResourceStruct *) kdms_glue_get_resources(object);
   if (resources == NULL)
      return TRUE;

   kdbm_close(resources->db);

   /*-- don't need to keep track of this fid any more --*/
   gdb_list = klist_delete(gdb_list, (kaddr) (long) resources->fid);

   kfree(resources);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: gdbm_locate_segment
|
|       Purpose: This routine locates a fid at a 
|                named segment and returns it.
|
|         Input: object  - object 
|		 segment - segment
|        Output: none
|       Returns: fid set to the appropriate offset
|
|    Written By: Jeremy Worley and Steve Kubica
|          Date: January 15, 1993
| Modifications:
|
------------------------------------------------------------*/
static int
gdbm_locate_segment(kobject object, char *segment)
{
   ResourceStruct *resources;
   klist *l;

   resources = (ResourceStruct *) kdms_glue_get_resources(object);

   /*-- make sure we can get the database entry associated with the fid --*/
   if ((l = klist_locate(gdb_list, (kaddr) (long) resources->fid)) == NULL)
      return -1;

   /*-- resource datum always has the most recent located segment --*/
   if (resources->datum.dptr == NULL)
      resources->datum.dptr = (char *) kmalloc(sizeof(char) * KLENGTH);

   /*-- compose key w/o allocating every time --*/
   resources->datum.dptr  = kstring_cat(segment, ":", resources->datum.dptr);
   resources->datum.dsize = kstrlen(resources->datum.dptr) + 1;

   /*-- make sure we have this key in database --*/
   if (kdbm_checkkey(resources->db, resources->datum))
   {
      kdatum key;

      key  = resources->datum;

      /*-- seek this datum to the beginning --*/ 
      kdbm_lseek(resources->db, key, 0, KSEEK_SET);
      return resources->fid;
   }
   else
      return -1;
}

/*-----------------------------------------------------------
|
|  Routine Name: gdbm_reader
|
|       Purpose: This routine is just like kread_generic,
|		 except it reads from the gdbm file and key 
|		 associated with the fid.  The association is
|		 set up by the locate segment, which should have
|		 been called just before we were.
|
|         Input: fid   - the file id associated with the database and
|		  	 key to be read from
|		 num   - the number of data points to be read
|		 type  - data type  
|        Output: data  - the array to read the data into
|       Returns: the number of data points read
|
|    Written By: Steve Kubica
|          Date: March 11, 1996
| Modifications:
|
------------------------------------------------------------*/
static ssize_t
gdbm_reader(int fid, kaddr data, size_t num, int type)
{
   ResourceStruct *resource;
   klist *l;
   
   /*-- get the database entry associated with the fid --*/
   if ((l = klist_locate(gdb_list, (kaddr) (long) fid)) == NULL)
      return -1;

   resource = (ResourceStruct *) l->client_data;

   /*-- read from the key indicated by the resource structure --*/
   return (ssize_t) kdbm_read(resource->db, resource->datum, data, 
			      (int) num, type);
}

/*-----------------------------------------------------------
|
|  Routine Name: gdbm_writer
|
|       Purpose: This routine is just like kwrite_generic,
|		 Except it writes to the gdbm file and key 
|		 associated with the fid.  The association is
|		 set up by the locate segment, which should have
|		 been called just before we were.
|
|         Input: fid   - the file id associated with the database and
|		  	 key to be written to
|		 data  - the array of data to write
|		 num   - the number of data points to be read
|		 type  - data type  
|        Output: none
|       Returns: the number of points written, or -1 on error 
|
|    Written By: Steve Kubica
|          Date: March 7, 1996
| Modifications:
|
------------------------------------------------------------*/
static ssize_t
gdbm_writer(int fid, kaddr data, size_t num, int type)
{
   ResourceStruct *resource;
   klist *l;
   
   /*-- get the database entry associated with the fid --*/
   if ((l = klist_locate(gdb_list, (kaddr) (long) fid)) == NULL)
      return -1;

   resource = (ResourceStruct *) l->client_data;

   /*-- read from the key indicated by the resource structure --*/
   return (ssize_t) kdbm_write(resource->db, resource->datum, data, 
			       (int) num, type);
}

/*-----------------------------------------------------------
|
|  Routine Name: gdbm_seeker
|
|       Purpose: This routine is just like klseek
|		 except it seeks within a single gdbm key 
|		 associated with the fid.  The association is
|		 set up by the locate segment, which should have
|		 been called just before we were.
|
|         Input: fid    - the file id associated with the database and
|		  	  key to be seeked within
|		 offset - number of bytes to seek
|		 whence - seek flag
|        Output: none
|       Returns: the number of bytes seeked
|
|    Written By: Steve Kubica
|          Date: March 7, 1996
| Modifications:
|
------------------------------------------------------------*/
static off_t
gdbm_seeker(int fid, off_t offset, int whence)
{
   ResourceStruct *resource;
   kdatum key;
   klist *l;

   /*-- avoid seeking if we have nowhere to go --*/
   if (offset == 0 && whence == KSEEK_CUR)
      return 0;
   
   /*-- get the database entry associated with the fid --*/
   if ((l = klist_locate(gdb_list, (kaddr) (long) fid)) == NULL)
      return -1;

   resource = (ResourceStruct *) l->client_data;
   key = resource->datum;

   /*-- read from the key indicated by the resource structure --*/
   return kdbm_lseek(resource->db, key, (int) offset, whence);
}

/*-----------------------------------------------------------
|
|  Routine Name: gdbm_teller
|
|       Purpose: This routine is just like ktell
|		 except it tells the position within a single gdbm key 
|		 associated with the fid.  The association is
|		 set up by the locate segment, which should have
|		 been called just before we were.
|
|         Input: fid    - the file id associated with the database and
|		  	  key to be seeked within
|        Output: none
|       Returns: the gdbm position
|
|    Written By: Steve Kubica
|          Date: June 7, 1996
| Modifications:
|
------------------------------------------------------------*/
static long int
gdbm_teller(int fid)
{
   ResourceStruct *resource;
   kdatum key;
   klist *l;

   /*-- get the database entry associated with the fid --*/
   if ((l = klist_locate(gdb_list, (kaddr) (long) fid)) == NULL)
      return -1;

   resource = (ResourceStruct *) l->client_data;
   key = resource->datum;

   /*-- read from the key indicated by the resource structure --*/
   return (long int) kdbm_tell(resource->db, key);
}

#endif
