 /*
  * Khoros: $Id: Viff.c,v 1.11 1996/06/26 14:46:00 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Viff.c,v 1.11 1996/06/26 14:46:00 mirage Exp $";

#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros VIFF Data Service Routines
   >>>>		
   >>>>  Private:
   >>>>                 init_viff()
   >>>>                 init_viff()
   >>>>                 viff_reader()
   >>>>                 viff_writer()
   >>>>   Static:
   >>>>                 viff_check()
   >>>>                 viff_input()
   >>>>                 viff_output()
   >>>>                 viff_prep()
   >>>>                 viff_destroy()
   >>>>                 viff_locate_segment()
   >>>>                 read_compressed()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

#if !defined(KVIFF_DEF)

void
init_viff(void) /*-- NULL viff initialization --*/
{
   return;
}

#else

#include "kdatafmt/viff.h"

static int     viff_check          PROTO((int));
static int     viff_input          PROTO((kobject, int, int));
static int     viff_output         PROTO((kobject, int, int));
static int     viff_destroy        PROTO((kobject));
static int     viff_prep           PROTO((kobject));
static int     viff_locate_segment PROTO((kobject, char *));
static ssize_t viff_reader   	   PROTO((int, kaddr, size_t, int));
static ssize_t viff_writer  	   PROTO((int, kaddr, size_t, int));
static int     read_compressed     PROTO((int, char *, int, viff *));

#define KVIFF_MAP_ENABLE         "viffMapEnable"
#define KVIFF_VALUE_SUBROW_SIZE  "viffSubrowSize"

/*=====================================================================*/

/*-----------------------------------------------------------
|  Routine Name: init_viff
|       Purpose: initialize the viff file format
------------------------------------------------------------*/
void
init_viff(void)
{
   kdms_add_file_format("viff", 
			"Khoros 1.0 Data Format (viff)",
			viff_check, 
			viff_input, 
			viff_output, 
			viff_destroy,
			viff_prep,
			viff_locate_segment, 
			viff_reader, viff_writer, NULL, NULL);

   /*-- viff specific attributes --*/
   kdms_define_attribute(KDMS_OBJECT, KVIFF_VALUE_SUBROW_SIZE, 1, 1, KINT,
                         TRUE, TRUE, 0);

   kdms_define_attribute(KDMS_OBJECT, KVIFF_MAP_ENABLE, 1, 1, KINT,
                         TRUE, TRUE, KOPTIONAL);
   return;
}

/*=====================================================================*/

/*
 *  Internal Resource Structure 
 *  viff  -  Khoros 1.0 Visualization Image File Format
 */
typedef struct
{
   viff *image;
   int   fid;
}
ResourceStruct;


/****    =============================================================
 ****    Format Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: viff_check - Checks to see if the file is a viff
|
|       Purpose: This function is used to check the first few bytes 
|                of a fid to see if it is a viff file. 
|
|         Input: fid - file id of a file to check
|        Output: none
|	Returns: returns TRUE if the file is a viff
|
|    Written By: Mark Young
|          Date: July 13, 1992
|
------------------------------------------------------------*/
static int
viff_check(int fid)
{
   unsigned char value[4];

   if ((kread_ubyte(fid, value, (size_t) 4) != 4)     ||
       ((int) value[0] != (int)	XV_FILE_MAGIC_NUM)  ||
       ((int) value[1] != (int)	XV_FILE_TYPE_XVIFF) ||
       ((int) value[2] != (int)	XV_IMAGE_REL_NUM)   ||
       ((int) value[3] != (int)	XV_IMAGE_VER_NUM))
      return FALSE;  /*-- not a viff --*/

   return TRUE;  /*-- a viff --*/
}

/*-----------------------------------------------------------
|
|  Routine Name: viff_input - Read a viff image
|
|       Purpose: This function is used to read in the image if the
|                supplied data is a viff image.  We simply read in 
|		 the header and initialize the segments.
|
|         Input: object - object to read viff into 
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Mark Young and Steve Kubica
|          Date: July 13, 1992
|
------------------------------------------------------------*/
/*ARGSUSED2*/
static int
viff_input(kobject object, int fid, int flags)
{
   ResourceStruct *resources;
   viff           *image;
   int             dsize, dcount, msize, mcount, lsize, lcount;

   int size[KDMS_MAX_DIM];
   int datatype;

   /*-- viff native index orders --*/
   int val_order[5] = {KWIDTH, KHEIGHT, KELEMENTS, KDEPTH, KTIME};
   int loc_order[4] = {KWIDTH, KHEIGHT, KDIMENSION, KDEPTH};
   int map_order[5] = {KMAP_HEIGHT, KMAP_WIDTH, KMAP_ELEMENTS, KMAP_DEPTH,
		       KMAP_TIME};


   /*-- get the resource structure --*/ 
   if (!kdms_initialized(object))
   {
      if ((resources = kcalloc((size_t)1, sizeof(ResourceStruct))) == NULL)
         return(FALSE);
      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   resources->fid = fid;

   /*-- read the viff header and save in the resource structure --*/
   image = resources->image = viff_readheader(fid);

   /*-- get size of each segment contained in viff --*/
   if (!image || !viff_size(image, &dsize, &dcount, &msize, &mcount,
			    &lsize, &lcount))
   {
      kfree_and_NULL(resources->image);
      /* _kdms_set_error(KDMS_EFMT_READFAIL); */
      return (FALSE);
   }

   /*-----------------------------------------------------------------------
   |   initialize all data segments based on information in viff header
   |   
   |   - viff structure contains only value, map, and location data, 
   |     plus a few specific attributes.
   \-----------------------------------------------------------------------*/

   /*-----------------------------------------------------------------------
   |  map data
   \-----------------------------------------------------------------------*/
   if (msize > 0)
   {
      kaddr data = NULL;

      /*-- create the map segment --*/
      kpds_create_map(object);

      size[0] = image->map_col_size;
      size[1] = image->map_row_size;
      size[2] = (image->map_scheme == VFF_MS_ONEPERBAND || 
		 image->map_scheme == VFF_MS_CYCLE) ? image->num_data_bands:1;
      size[3] = size[4] = 1;

      viff2khoros_maptype(image->map_storage_type, &datatype);

      if (image->data_encode_scheme != VFF_DES_RAW)
      {
	 data = image->maps = kmalloc((size_t)msize);
	 read_compressed(fid, image->maps, msize, image);
      }

      /*-- store all map information, and possibly the data --*/
      kdms_set_segment(object, KDMS_SEGMENT_MAP, 5, size, map_order, datatype, 
		       data);
   }


   /*-----------------------------------------------------------------------
   |  location data
   \-----------------------------------------------------------------------*/
   if (lsize > 0)
   {
      kaddr data = NULL;

      /*-- create the location segment (curvilinear default) --*/
      kpds_create_location(object);
      
      size[0] = image->row_size;
      size[1] = image->col_size;
      size[2] = image->location_dim;
      size[3] = size[4] = 1;

      if (image->data_encode_scheme != VFF_DES_RAW)
      {
	 data = image->location = kmalloc((size_t)lsize);
	 read_compressed(fid, (char *)image->location, lsize, image);
      }

      /*-- store all location information, and possibly the data --*/
      kdms_set_segment(object, KDMS_SEGMENT_LOCATION, 4, size, loc_order, 
		       KFLOAT, data);
   }


   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/
   if (dsize > 0)
   {
      kaddr data = NULL;

      /*-- create the value segment --*/
      kpds_create_value(object);

      size[0] = image->row_size;
      size[1] = image->col_size;

      /*-- try to work around some ambiguities in the Xvimage definition --*/
      if (image->color_space_model == VFF_CM_NONE)
      {
         size[2] = image->num_data_bands;
         size[3] = size[4] = 1;
      }
      else if (image->num_data_bands % 3 == 0)
      {
         size[2] = 3;
         size[3] = 1;
         size[4] = image->num_data_bands / 3;
      }
      else
      {
         size[2] = size[3] = 1;
         size[4] = image->num_data_bands;
      }

      viff2khoros_datatype(image->data_storage_type, &datatype);

      if (image->data_encode_scheme != VFF_DES_RAW)
      {
	 data = image->imagedata = kmalloc((size_t)dsize);
	 read_compressed(fid, image->imagedata, dsize, image);
      }

      /*-- store all location information, and possibly the data --*/
      kdms_set_segment(object, KDMS_SEGMENT_VALUE, 5, size, val_order, 
		       datatype, data);
   }


   /*-----------------------------------------------------------------------
   |  initialize all attributes used by viff 
   \-----------------------------------------------------------------------*/
   kpds_set_attribute(object, KDMS_COMMENT, image->comment);

   if (!kpds_query_attribute(object, _INTERNAL_COLORSPACE, NULL, NULL, 
			     NULL, NULL))
      kpds_create_object_attr(object, _INTERNAL_COLORSPACE, 1, 1, KINT, 
			      TRUE, TRUE);

   kpds_set_attribute(object, _INTERNAL_COLORSPACE,
                      viff2khoros_csm(image->color_space_model));

   kpds_set_attribute(object, KVIFF_MAP_ENABLE,
                (image->map_enable == VFF_MAP_FORCE) ? KFORCED : KOPTIONAL);

   kpds_set_attribute(object, KVIFF_VALUE_SUBROW_SIZE,
                      (int)image->subrow_size);

   kpds_create_object_attr(object, "viffMapScheme", 1, 1, KULONG, TRUE, TRUE);
   kpds_set_attribute(object, "viffMapScheme", image->map_scheme);

   kpds_create_object_attr(object, "ispare1", 1, 1, KULONG, TRUE, TRUE);
   kpds_set_attribute(object, "ispare1", image->ispare1);

   kpds_create_object_attr(object, "ispare2", 1, 1, KULONG, TRUE, TRUE);
   kpds_set_attribute(object, "ispare2", image->ispare2);

   kpds_create_object_attr(object, "fspare1", 1, 1, KFLOAT, TRUE, TRUE);
   kpds_set_attribute(object, "fspare1", image->fspare1);

   kpds_create_object_attr(object, "fspare2", 1, 1, KFLOAT, TRUE, TRUE);
   kpds_set_attribute(object, "fspare2", image->fspare2);

   if (image->startx >= 0 || image->starty >= 0) 
      kpds_set_attribute(object, KPDS_SUBOBJECT_POSITION,
			 (int)image->startx, (int)image->starty, 0, 0, 0);


   /*-----------------------------------------------------------------------
   |  try to do the best thing with the pixsiz fields
   \-----------------------------------------------------------------------*/
   if (kpds_query_location(object))
   {
      /*-------------------------------------------------------------- 
      | if we already have location data, then store the 
      | pixsizx and pixsizy to the width & height image point size 
      \--------------------------------------------------------------*/
      kpds_set_attribute(object, KPDS_POINT_SIZE, 
			 (double)image->pixsizx, (double)image->pixsizy, 
			 1.0, 1.0, 1.0);
   }
   else
   {
      /*-------------------------------------------------------------- 
      | if we already don't have location data, then store the 
      | pixsizx and pixsizy as uniform begin and end values
      \--------------------------------------------------------------*/
      if ((image->pixsizx != 1.0) || (image->pixsizy != 1.0))
      {
	 kpds_set_attribute(object, KPDS_LOCATION_GRID, KUNIFORM);
	 kpds_create_location(object);

	 kpds_set_attribute(object, KPDS_LOCATION_SIZE, 
			    image->row_size, image->col_size, 1, 2);
	 kpds_set_attribute(object, KPDS_LOCATION_END, 
			    image->pixsizx * (double) image->row_size, 
			    image->pixsizy * (double) image->col_size, 1.0);

      }
   }
   
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: viff_output - Close the viff image
|
|       Purpose: This function is used to close the viff image.  
|		 It basically is a writeheader of the viff header.
|
|         Input: object - the viff object to be closed
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Mark Young and Steve Kubica
|          Date: July 13, 1992
|
------------------------------------------------------------*/
/*ARGSUSED2*/
static int
viff_output(kobject object, int fid, int flags)
{
   ResourceStruct *resources;

   viff *image;

   char *tmpcomment = NULL;
   int   itmp;
   int   grid;

   int  datatype;
   int *size;
   int *order;
   int  map_scheme;

   unsigned long ispare1 = 0, ispare2 = 0;

   double px, py;


   /*-- get the resource structure --*/ 
   if (!kdms_initialized(object))
   {
      if ((resources = kcalloc((size_t)1, sizeof(ResourceStruct))) == NULL)
         return(FALSE);
      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   resources->fid = fid;

   /*-- get the comment now, since we'll need it for initialization --*/
   kpds_get_attribute(object, KDMS_COMMENT, &tmpcomment);

   if (resources->image == NULL)
   {
      resources->image = viff_create((unsigned long)0, (unsigned long) 0,
				     (unsigned long) VFF_TYP_1_BYTE,
				     (unsigned long) 0, (unsigned long) 0,
				     (tmpcomment == NULL ? " " : tmpcomment),
				     (unsigned long) 0, (unsigned long) 0,
				     (unsigned long) VFF_MS_NONE,
				     (unsigned long) VFF_MAPTYP_1_BYTE,
                                     (unsigned long) VFF_LOC_IMPLICIT,
				     (unsigned long) 0);
      if (!resources->image)
      {
         /* _kdms_set_error(KDMS_EFMT_FAILURE); */
         return FALSE;
      }
   }
   image = resources->image;

   /*-----------------------------------------------------------------------
   |   initialize all data segments based on information in viff header
   |   
   |   - viff structure contains only value, map, and location data, 
   |     plus a few specific attributes.
   \-----------------------------------------------------------------------*/

   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/
   if (kdms_get_attributes(object, KDMS_SEGMENT_VALUE, 
			   KDMS_PHYSICAL_DATA_TYPE,   &datatype,
			   KDMS_PHYSICAL_SIZE,        &size,
			   KDMS_PHYSICAL_INDEX_ORDER, &order, NULL) == TRUE)
   {
      khoros2viff_datatype(datatype, &image->data_storage_type);
      image->imagedata = NULL;

      image->row_size       = kdas_get_order(size, order, 5, KWIDTH, 1);
      image->col_size       = kdas_get_order(size, order, 5, KHEIGHT, 1);
      image->num_data_bands = kdas_get_order(size, order, 5, KELEMENTS, 1);
      image->num_of_images  = kdas_get_order(size, order, 5, KTIME, 1) * 
                              kdas_get_order(size, order, 5, KDEPTH, 1);
   }

   /*-----------------------------------------------------------------------
   |  map data
   \-----------------------------------------------------------------------*/
   if (kdms_get_attributes(object, KDMS_SEGMENT_MAP, 
			   KDMS_PHYSICAL_DATA_TYPE,   &datatype,
			   KDMS_PHYSICAL_SIZE,        &size,
			   KDMS_PHYSICAL_INDEX_ORDER, &order, NULL) == TRUE)
   {
      khoros2viff_datatype(datatype, &image->map_storage_type);
      image->maps = NULL;

      image->map_row_size = kdas_get_order(size, order, 5, KMAP_WIDTH, 1);
      image->map_col_size = kdas_get_order(size, order, 5, KMAP_HEIGHT, 1);
      image->map_scheme   = (size[2] > 1) ? VFF_MS_ONEPERBAND : VFF_MS_SHARED;
   }
   else
      image->map_scheme = VFF_MS_NONE;

   /*-----------------------------------------------------------------------
   |  location data
   \-----------------------------------------------------------------------*/
   if (kdms_get_attributes(object, KDMS_SEGMENT_LOCATION, 
			   KDMS_PHYSICAL_SIZE,        &size,
			   KDMS_PHYSICAL_INDEX_ORDER, &order, NULL) == TRUE)
   {
      image->location = (float *)NULL;
      image->location_dim = kdas_get_order(size, order, 4, KDIMENSION, 1);
   }


   /*-- get all relevant header information --*/
   kmemcpy(image->comment, tmpcomment, sizeof(char) * 512);

   kpds_get_attribute(object, _INTERNAL_COLORSPACE , &itmp);
   image->color_space_model = khoros2viff_csm(itmp);

   if (kpds_query_attribute(object, "viffMapScheme", NULL, NULL, NULL, NULL))
   {
      kpds_get_attribute(object, "viffMapScheme", &map_scheme);

      if (image->map_scheme != map_scheme && map_scheme != VFF_MS_NONE)
	 image->map_scheme = map_scheme;
   }

   kpds_get_attribute(object, KPDS_SUBOBJECT_POSITION, 
		      &image->startx, &image->starty, NULL, NULL, NULL);

   if (image->startx == 0 && image->starty == 0) 
   {
      image->startx = -1;
      image->starty = -1;
   }

   kpds_get_attribute(object, KVIFF_VALUE_SUBROW_SIZE, &image->subrow_size);
   kpds_get_attribute(object, "ispare1", &image->ispare1);
   kpds_get_attribute(object, "ispare2", &image->ispare2);
   kpds_get_attribute(object, "fspare1", &image->fspare1);
   kpds_get_attribute(object, "fspare2", &image->fspare2);

   /*-- luxury exception for spectrum --*/
   if (kdms_get_attribute(object, NULL, "colorMapContents", &ispare1))
      image->ispare1 = ispare1;
   if (kdms_get_attribute(object, NULL, "colorMapColnum", &ispare2))
      image->ispare2 = ispare2;
   
   /*-- uniform location data can be stored as the pixsiz attribute --*/
   kpds_get_attribute(object, KPDS_LOCATION_GRID, &grid);
   if (grid == KUNIFORM)   
   {
      int    wsize, hsize;
      double xbeg, ybeg, xend, yend;

      kpds_get_attributes(object, 
			  KPDS_LOCATION_SIZE, &wsize, &hsize, NULL, NULL,
			  KPDS_LOCATION_END, &xbeg, &ybeg, NULL, 
			  KPDS_LOCATION_END, &xend, &yend, NULL, 
			  NULL);

      image->pixsizx = (float) (xend-xbeg)/wsize;
      image->pixsizy = (float) (yend-ybeg)/hsize;
   }
   else if (kpds_get_attribute(object, KPDS_POINT_SIZE, &px, &py, 
			       NULL, NULL, NULL))
   {
      image->pixsizx = (float) px;
      image->pixsizy = (float) py; 
   }
   else
   {
      image->pixsizy = 1.0;
      image->pixsizx = 1.0;
   }

   /*-- write the viff structure out to the specified filename --*/
   viff_writeheader(fid, image);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: viff_prep - prepare an object for output to viff
|
|       Purpose: This function is used to prepare an object for
|		 output to viff.  It will print out information 
|		 about what is not getting stored out to the viff
|		 file due to the limitations of the viff format.	
|
|         Input: object - the object to be prepared
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: March 20, 1996
|
------------------------------------------------------------*/
/*ARGSUSED2*/
static int
viff_prep(kobject object)
{
   char *warn = NULL;
   int   d, t, e;
   int 	 val_order[5] = {KWIDTH, KHEIGHT, KELEMENTS, KDEPTH, KTIME};
   int 	 loc_order[4] = {KWIDTH, KHEIGHT, KDIMENSION, KDEPTH};
   int 	 map_order[5] = {KMAP_HEIGHT, KMAP_WIDTH, KMAP_ELEMENTS, KMAP_DEPTH,
			 KMAP_TIME};

   /*-----------------------------------------------------------------------
   |  mask and time data
   \-----------------------------------------------------------------------*/
   if (kpds_query_mask(object))
      warn = kstring_append(warn, "- no support for storing mask "
			    ": mask segment ignored.\n");

   if (kpds_query_time(object))
       warn = kstring_append(warn, "- no support for storing time "
			    ": time segment ignored.\n");

   /*-- support for checking extraneous attributes coming soon! --*/

   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/
   if (kpds_query_value(object))
   {
      /*-- size issue : can only store 4D data plane --*/
      kpds_get_attribute(object, KPDS_VALUE_SIZE, NULL, NULL, &d, &t, NULL);

      if (d > 1 && t > 1)
	 warn = kstring_append(warn, "- combining depth and time of value "
			       "segment into the viff number of images.\n");

      /*-- reorder to viff ordering for output --*/
      kdms_set_attribute(object, KDMS_SEGMENT_VALUE, 
			 KDMS_PHYSICAL_INDEX_ORDER, val_order);
   }

   /*-----------------------------------------------------------------------
   |  map data
   \-----------------------------------------------------------------------*/
   if (kpds_query_map(object))
   {
      /*-- size issue : can only store depth # of wxh map planes --*/
      kpds_get_attribute(object, KPDS_MAP_SIZE, NULL, NULL, NULL, &t, &e);

      if (t > 1 || e > 1)
	 warn = kstring_append(warn, "- ignoring time and element planes "
			       "of map segment.\n");

      /*-- reorder to viff ordering for output --*/
      kdms_set_attribute(object, KDMS_SEGMENT_MAP, 
			 KDMS_PHYSICAL_INDEX_ORDER, map_order);
   }


   /*-----------------------------------------------------------------------
   |  location data
   \-----------------------------------------------------------------------*/
   if (kpds_query_location(object))
   {
      int grid;
      
      /*-- uniform location data can be stored as the pixsiz attribute --*/
      kpds_get_attribute(object, KPDS_LOCATION_GRID, &grid);

      if (grid == KRECTILINEAR)
	 warn = kstring_append(warn, "- no support for storing rectilinear "
			       "location : location segment ignored.\n");

      else if (grid == KUNIFORM)
	 warn = kstring_append(warn, "- only able to store uniform unit size "
			       "over width and height via the viff pixsizx "
			       "and pixsizy fields\n");

      else if (grid == KCURVILINEAR)
      {
	 kpds_get_attribute(object, KPDS_LOCATION_SIZE, NULL, NULL, &d, NULL);
	 if (d > 1)
	    warn = kstring_append(warn, "- ignoring depth plane "
			       "of location segment.\n");

	 /*-- reorder to viff ordering for output --*/
	 kdms_set_attribute(object, KDMS_SEGMENT_VALUE, 
			    KDMS_PHYSICAL_INDEX_ORDER, loc_order);
      }
   }

   /*-- if we have any news to report, report it now --*/
   if (warn)
   {
      kinfo(KSTANDARD, "VIFF format output :\n%s", warn);
      kfree_and_NULL(warn);
   }

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: viff_destroy - Frees an viff image
|
|       Purpose: This function is used to free the resources used
|		 by the viff format glue.
|
|         Input: object - object with resources to free
|        Output: none
|	Returns: TRUE on success FALSE otherwise
|
|    Written By: Mark Young and Steve Kubica
|          Date: July 13, 1992
| Modifications:
|
------------------------------------------------------------*/
static int
viff_destroy(kobject object)
{
   ResourceStruct *resources;

   resources = (ResourceStruct *) kdms_glue_get_resources(object);
   if (resources == NULL)
      return TRUE;

   /*-- free the viff structure --*/
   viff_free(resources->image);

   /*-- free the resources --*/
   kfree_and_NULL(resources);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: viff_locate_segment
|
|       Purpose: This routine locates a fid at a 
|                named segment and returns it.
|
|         Input: object  - object 
|		 segment - segment
|        Output: none
|       Returns: fid set to the appropriate offset
|
|    Written By: Jeremy Worley and Steve Kubica
|          Date: January 15, 1993
| Modifications:
|
------------------------------------------------------------*/
static int
viff_locate_segment(kobject object, char *segment)
{
   ResourceStruct *resources;
   off_t offset;
   int 	 mapsize;
   int 	 locsize;
   int 	 datatype;
   int 	*size;


   resources = (ResourceStruct *) kdms_glue_get_resources(object);

   /*-- need to compute map and location sizes since offset is cumulative --*/

   /*-- map segment size --*/
   if (kdms_get_attributes(object, KDMS_SEGMENT_MAP,
			   KDMS_PHYSICAL_DATA_TYPE, &datatype,
			   KDMS_PHYSICAL_SIZE,      &size, NULL) &&
       resources->image->map_scheme   != VFF_MS_NONE             &&
       resources->image->map_col_size != 0)
   {
      mapsize = size[0]*size[1]*size[2]*size[3]*size[4]*kdata_size(datatype);
   }
   else
      mapsize = 0;

   /*-- location segment size --*/
   if (kdms_get_attributes(object, KDMS_SEGMENT_LOCATION, 
			   KDMS_PHYSICAL_DATA_TYPE, &datatype,
			   KDMS_PHYSICAL_SIZE,      &size, NULL) &&
       resources->image->location_dim != 0)
   {
      locsize = size[0]*size[1]*size[2]*size[3]*kdata_size(datatype);
   }
   else
      locsize = 0;

   if ((kstrcmp(segment, KDMS_SEGMENT_MAP) == 0) && mapsize > 0)
      offset = (off_t) 1024;

   else if ((kstrcmp(segment, KDMS_SEGMENT_LOCATION) == 0) && locsize > 0)
      offset = (off_t) 1024 + mapsize;

   else if (kstrcmp(segment, KDMS_SEGMENT_VALUE) == 0)
      offset = (off_t) 1024 + mapsize + locsize;

   else
      return -1;
   
   klseek(resources->fid, offset, KSEEK_SET);

   return resources->fid;
}

/*-----------------------------------------------------------
|
|  Routine Name: viff_reader
|
|       Purpose: This routine is just like kread_generic,
|                except it reorders bit data before returning it.
|
|         Input: fid   - the file id to be read
|		 num   - the number of data points to be read
|		 type  - data type  
|        Output: data  - the array to read the data into
|       Returns: the number of data points read
|
|    Written By: Steve Kubica
|          Date: March 7, 1996
| Modifications:
|
------------------------------------------------------------*/
static ssize_t
viff_reader(int fid, kaddr data, size_t num, int type)
{
   if (type == KBIT)  /*-- must cast to correct bit ordering --*/
   {
      ssize_t rval;
      kaddr   idata = kcalloc((size_t) 1, (size_t) ((num+7) >> 3));

      rval = kread_bit(fid, idata, num);
      kdata_cast(idata, KBIT_LSB, KBIT, (int) num, KREAL, 1.0, 1.0, data);

      kfree(idata);
      return rval;
   }
   else
      return kread_generic(fid, data, num, type);
}

/*-----------------------------------------------------------
|
|  Routine Name: viff_writer
|
|       Purpose: This routine is just like kwrite_generic,
|                except it reorders bit data before writing it.
|
|         Input: fid   - the file id to be read
|		 data  - the array of data to write
|		 num   - the number of data points to be read
|		 type  - data type  
|        Output: none
|       Returns: the number of points written, or -1 on error 
|
|    Written By: Steve Kubica
|          Date: March 7, 1996
| Modifications:
|
------------------------------------------------------------*/
static ssize_t
viff_writer(int fid, kaddr data, size_t num, int type)
{
   if (type == KBIT)  /*-- must cast to correct bit ordering --*/
   {
      ssize_t rval;
      kaddr   idata = kcalloc((size_t) 1, (size_t) ((num+7) >> 3));

      kdata_cast(data, KBIT_LSB, KBIT, (int) num, KREAL, 1.0, 1.0, idata);
      rval = kwrite_bit(fid, idata, num);

      kfree(idata);
      return rval;
   }
   else
      return kwrite_generic(fid, data, num, type);
}

/****    =============================================================
 ****    Coping Utility for Compressed Data
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: (static) read_compressed
|
|       Purpose: read data if it is compressed.
|
|         Input: fd - the file descriptor in which to read from
|		 data - a character pointer where the bytes will be 
|			read and stored to
|		 size - an integer specifying the number of bytes to 
|			to be read.
|		 image - pointer to viff structure.
|        Output: none
|       Returns: TRUE on success, FALSE on failure
|
|    Written By: Scott Wilson 
|          Date: April 22, 1989
| Modifications: September 19, 1994 JW - upgraded to K 2.0
|
------------------------------------------------------------*/

static char *uncompr_cmd[] =
	{"This_should_never_happen",      /* DES-RAW */
         "uncompress",                    /* DES_COMPRESS */
         "unrle",                         /* DES_RLE */
         "untransform",                   /* DES_TRANSFORM */
         "unccitt",                       /* DES_CCITT */
         "unadpcm",                       /* DES_ADPCM */
         "ungencomp"};                    /* DES_GENERIC */

static int
read_compressed(int fd, char *data, int size, viff *image)
{
   char  *tmpfilename;
   char   command[255];
   char  *buf;
   kfile *file;
   int    fdtmp;
   int    len, index, n;
   
   /*-- see if the size happends to be zero - if so, just return --*/
   if (size == 0)
      return FALSE;

   /* 
    * Grab the first 8 bytes from the file descriptor and see if
    * the data should be unencoded or just copied 
    */
   if (kread(fd, command, (size_t) 8) != 8)
   {
      kerror("kdataman", "read_compressed", "couldn't read first 8 bytes!\n");
      return FALSE;
   }
   
   command[8] = 0;
   len = atoi(command);

   if (len == 0)
   {
      if (kread(fd, data, (size_t) size) != size)
      {
	 kerror("kdataman", "read_compressed", "Incorrect byte count!\n");
	 return FALSE;
      }
      return TRUE;
   }
   else
   {
      /*-- read the data --*/
      if ((buf = kmalloc((size_t)len)) == NULL)
      {
	 kerror("kdataman","read_compr", "Not enough space for buffer!\n");
	 return FALSE;
      }
      
      if (kread(fd, buf, (size_t)len) != len)
      {
	 kerror("kdataman", "read_compr", 
		"Incorrect byte count on data block!\n");
	 kfree(buf);
	 return FALSE;
      }
      
      /*-- run it thru the decompress command --*/
      tmpfilename = ktempnam(NULL, "viffXXXXXX");
      index = image->data_encode_scheme;
      ksprintf(command,"%s > %s",uncompr_cmd[index],tmpfilename);
      if ((file = kpopen(command, "w")) == NULL)
      {
	 kerror("kdataman", "read_compressed", "Unable to open process\n");
	 kfree(buf);
	 return FALSE;
      }
      if (kfwrite(buf,(size_t)1,(size_t)len,file) != len)
      {
	 kerror("kdataman","read_compressed", "Unable to write source data\n");
	 kpclose(file);
	 kfree(buf);
	 return FALSE;
      }
      kpclose(file);
      kfree_and_NULL(buf);
      
      fdtmp = kopen(tmpfilename, KOPEN_RDONLY, 0444);
      n = kread(fdtmp, data, (size_t)size);
      if (n != size)
      {
	 kerror("kdataman", "read_compressed", 
		 "Incorrect decoded byte count!\n");
	 kclose(fdtmp);
	 kunlink(tmpfilename);
	 return FALSE;
      }
      kclose(fdtmp);
      kunlink(tmpfilename);
      return TRUE;
   }
}

#endif /* KVIFF_DEF */
