 /*
  * Khoros: $Id: Xpm.c,v 1.8 1996/06/28 19:50:15 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Xpm.c,v 1.8 1996/06/28 19:50:15 mirage Exp $";

#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros XPM Data Service Routines
   >>>>		
   >>>>  Private:
   >>>>                 init_xpm()
   >>>>                 init_xpm()
   >>>>   Static:
   >>>>                 xpm_check()
   >>>>                 xpm_input()
   >>>>                 xpm_output()
   >>>>                 xpm_destroy()
   >>>>                 xpm_prep()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"


#if !defined(KXPM_DEF)

void
init_xpm(void) /*-- NULL xpm initialization --*/
{
   return;
}

#else

#include "kdatafmt/xpm.h"

static int xpm_check   PROTO((int));
static int xpm_input   PROTO((kobject, int, int));
static int xpm_output  PROTO((kobject, int, int));
static int xpm_destroy PROTO((kobject));
static int xpm_prep    PROTO((kobject));

/*=====================================================================*/

/*-----------------------------------------------------------
|  Routine Name: init_xpm
|       Purpose: initialize the xpm file format
------------------------------------------------------------*/
void
init_xpm(void)
{
   kdms_add_file_format("xpm", 
			"X11 Pixmap File Format (xpm)", 
			xpm_check, 
			xpm_input, 
			xpm_output, 
			xpm_destroy, 
			xpm_prep,
			NULL, NULL, NULL, NULL, NULL);
   return;
}

/*=====================================================================*/

/*
 *  Internal Resource Structure 
 *  xpm  -  X11 Pixmap File Format (xpm)
 */
typedef struct
{
   int  fid;
   xpm *image;
}
ResourceStruct;

/****    =============================================================
 ****    Format Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: xpm_check - Checks to see if the file is an xpm
|
|       Purpose: This function is used to check the first few bytes 
|                of a fid to see if it is an xbm file. 
|
|         Input: fid - file id of a file to check
|        Output: none
|	Returns: TRUE if the file is an xbm
|
|    Written By: Jeremy Worley
|          Date: March 14, 1994
| Modifications:
|
------------------------------------------------------------*/
static int
xpm_check(int fid)
{
   xpm *check;
   
   /*-- seems like the easiest way to do this is to check the header --*/
   if ((check = xpm_readheader(fid)) == NULL)
      return FALSE;

   xpm_free(check);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: xpm_input - Read an xpm image
|
|       Purpose: This function is used to read in the image if the
|                supplied data is an xpm image.  We can simply set up 
|                the header and initialize the data segments.
|
|         Input: object - the xpm object to be initialized
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley
|          Date: March 14, 1994
| Modifications:
|
------------------------------------------------------------*/
/*ARGSUSED */
static int
xpm_input(kobject object, int fid, int flags)
{
   ResourceStruct *resources;
   xpm *image;

   int size[KDMS_MAX_DIM];
   int i;

   /*-- get the resource structure --*/ 
   if (!kdms_initialized(object))
   {
      if ((resources = kcalloc((size_t)1, sizeof(ResourceStruct))) == NULL)
         return(FALSE);

      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   resources->fid = fid;

   if ((image = resources->image = xpm_fdread(fid)) == NULL)
   {
      kdas_set_error(KDMS_EFMT_FAILURE);
      return FALSE;
   }


   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/
   if (image->width != 0 && image->height != 0)
   {
      /*-- create the value segment --*/
      kpds_create_value(object);

      size[0] = image->width;
      size[1] = image->height;
      size[2] = size[3] = size[4] = 1;

      kdms_set_segment(object, KDMS_SEGMENT_VALUE, 5, size, NULL, 
		       KUBYTE, image->data);
      image->data = NULL;
   }

   /*-----------------------------------------------------------------------
   |  map data
   \-----------------------------------------------------------------------*/
   if (image->ncolors != 0)
   {
      unsigned short *data, *d;

      /*-- create the map segment --*/
      kpds_create_map(object);

      size[0] = 3;
      size[1] = image->ncolors;
      size[2] = size[3] = size[4] = 1;

      data = (unsigned short *) kmalloc(sizeof(unsigned short) * 
					image->ncolors * 3);

      for (i = 0, d = data; i < image->ncolors; i++)
      {
         *d++ = image->maps[i].red;
         *d++ = image->maps[i].green;
         *d++ = image->maps[i].blue;
      }

      kdms_set_segment(object, KDMS_SEGMENT_MAP, 5, size, NULL, KUSHORT, data);
   }
   return TRUE;
}



/*-----------------------------------------------------------
|
|  Routine Name: xpm_output - Closes the xpm image
|
|       Purpose: This function is used to close the xpm image.  Which
|                means if the image was modified we then re-write image
|                back out.
|
|         Input: object - the xpm object to be closed
|
|        Output: Returns: returns TRUE or FALSE if an error occurs
|
|    Written By: Jeremy Worley
|          Date: Mar 14, 1994 16:53
| Modifications:
|
------------------------------------------------------------*/
#define XPM_MAX_VAL 65535.0

/*ARGSUSED */
static int
xpm_output(kobject object, int fid, int flags)
{
   ResourceStruct *resources;
   unsigned short *data;
   xpm *image = NULL;
   int *size;
   int  i, j;
   int vorder[KDMS_MAX_DIM] = {KWIDTH, KHEIGHT, KDEPTH, KTIME, KELEMENTS};
   int morder[KDMS_MAX_DIM] = {KMAP_WIDTH, KMAP_HEIGHT, KMAP_DEPTH, KMAP_TIME,
			       KMAP_ELEMENTS};
   int begin[KDMS_MAX_DIM] = {0, 0, 0, 0, 0};
   int end[KDMS_MAX_DIM];
   
   /*-- get the resource structure --*/ 
   if (!kdms_initialized(object))
   {
      if ((resources = kcalloc((size_t)1, sizeof(ResourceStruct))) == NULL)
         return(FALSE);

      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   if (resources->image == NULL)
   {
      if ((image = resources->image = xpm_create()) == NULL)
      {
         kdas_set_error(KDMS_EFMT_FAILURE);
         return FALSE;
      }
   }
   else
      image = resources->image;

   /*
    * Get the value segment.  Xpm is limited to char data only, with a
    * specific index order. So, first we gonna set the data type and 
    * index order, then sync downward and then get the segment.
    */
   if (!kdms_set_attributes(object, KDMS_SEGMENT_VALUE, KDMS_DATA_TYPE, KBYTE,
                            KDMS_INDEX_ORDER, vorder, NULL))
      return (FALSE);

   kdms_get_attribute(object, KDMS_SEGMENT_VALUE, KDMS_SIZE, &size);
   
   end[0] = size[0] - 1;
   end[1] = size[1] - 1;
   end[2] = end[3] = end[4] = 0;
   
   image->width = (int)size[0];
   image->height = (int)size[1];

   image->data = kdms_get_data(object, KDMS_SEGMENT_VALUE, begin, end, NULL);
   
   /*
    * if the map segment exists, then set the datatype to short and 
    * normalize in the range 2^16 - 1.
    */
   if (kdms_query_segment(object, KDMS_SEGMENT_MAP))
   {
      kdms_set_attributes(object, KDMS_SEGMENT_MAP, KDMS_DATA_TYPE, KUSHORT, 
			  KDMS_INDEX_ORDER, morder, KDMS_SCALING, KNORMALIZE,
			  KDMS_NORM_MIN, 0.0, KDMS_NORM_MAX, XPM_MAX_VAL,
			  NULL);

      /* get the size and override the width to 3 */
      kdms_get_attribute(object, KDMS_SEGMENT_MAP, KDMS_SIZE, &size);
      size[0] = 3;
      kdms_set_attribute(object, KDMS_SEGMENT_MAP, KDMS_SIZE, size);

      end[0] = size[0] - 1;
      end[1] = size[1] - 1;
      end[2] = size[2] - 1;
      end[3] = size[3] - 1;
      end[4] = size[4] - 1;
      
      data = kdms_get_data(object, KDMS_SEGMENT_MAP, begin, end, NULL);
      image->maps = (xpmcolor *) kmalloc((size_t)(size[1] * sizeof(xpmcolor)));

      for (i = 0, j = 0; i < size[1]; i++)
      {
         image->maps[i].red = data[j++];
         image->maps[i].green = data[j++];
         image->maps[i].blue = data[j++];
         image->maps[i].flags = 0xff;
         image->maps[i].val = NULL;
      }
      image->ncolors = size[1];

      kfree_and_NULL(data);
   } 
   else 
   {
      image->maps = (xpmcolor *) kmalloc((size_t)256 * sizeof(xpmcolor));
      for (i = 0; i < 256; i++)
      {
	 image->maps[i].red = image->maps[i].green = image->maps[i].blue = 
	    i << 8;
	 image->maps[i].flags = 0xff;
	 image->maps[i].val = NULL;
      }
      image->ncolors = 256;
   }
   
   /*-- write the xpm structure out to the specified filename --*/
   return xpm_fdwrite(fid, image);
}

/*-----------------------------------------------------------
|
|  Routine Name: xpm_destroy - Frees an xpm image
|
|       Purpose: This function is used to create an xpm image.  Which
|                means that we use the current dimension values in order
|                to create a proper xpm.
|
|         Input: data - initial sample of data
|                num  - number of bytes in the sample
|
|        Output: Returns: returns TRUE or FALSE if an error occurs
|
|    Written By: Jeremy Worley
|          Date: March 14, 1994
| Modifications:
|
------------------------------------------------------------*/
static int
xpm_destroy(kobject object)
{
   ResourceStruct *resources;

   if ((resources = (ResourceStruct *) kdms_glue_get_resources(object))==NULL)
      return TRUE;

   /*-- free the xpm structure --*/
   xpm_free(resources->image);

   /*-- free the resources --*/
   kfree(resources);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: xpm_prep - prepare an object for output to xpm
|
|       Purpose: This function is used to prepare an object for
|		 output to xpm.  It will print out information 
|		 about what is not getting stored out to the xpm
|		 file due to the limitations of the xpm format.	
|
|         Input: object - the object to be prepared
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: May 18, 1996
|
------------------------------------------------------------*/
/*ARGSUSED2*/
static int
xpm_prep(kobject object)
{
   char *warn = NULL;
   int   w, h, d, t, e;
   int   type;
   int   has_map = FALSE;
   

   /*-----------------------------------------------------------------------
   |  mask, location, and time data
   \-----------------------------------------------------------------------*/
   if (kpds_query_mask(object))
      warn = kstring_append(warn, "- no support for storing mask "
			    ": mask segment ignored.\n");

   if (kpds_query_location(object))
      warn = kstring_append(warn, "- no support for storing location "
			    ": location segment ignored.\n");

   if (kpds_query_time(object))
      warn = kstring_append(warn, "- no support for storing time "
			    ": time segment ignored.\n");

   if (kpds_query_map(object))
   {
      has_map = TRUE;

      kpds_get_attributes(object, 
			  KPDS_MAP_DATA_TYPE, &type,
			  KPDS_MAP_SIZE,      &w, &h, &d, &t, &e, NULL);
      
      if (type > KUBYTE)  
	 warn = kstring_append(warn, "- map data cast to unsigned byte "
			       "for storage.\n");
      
      if (d > 1 || t > 1 || e > 1)  
	 warn = kstring_append(warn, "- ignoring depth, time, and element "
			       "planes of map segment.\n");
      if (w > 3)
	 warn = kstring_append(warn, "- only storing first three columns "
			       "of map segment.\n");

      if (w > 256)
	 warn = kstring_append(warn, "- only storing first 256 rows "
			       "of map segment.\n");
   }
   
   /*-- support for checking extraneous attributes coming soon! --*/

   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/
   if (kpds_query_value(object))
   {
      /*-- this gets hairy : uncouple so we don't thrash the disk --*/
      kpds_set_attribute(object, KPDS_COUPLING, KUNCOUPLED);

      /*-- size issue : can only store 2D data plane --*/
      kpds_get_attribute(object, KPDS_VALUE_SIZE, NULL, NULL, &d, &t, &e);

      if (d > 1 || t > 1)
      {
	 kpds_set_attribute(object, KPDS_VALUE_SIZE, -1, -1, 1, 1, -1);
	 warn = kstring_append(warn, "- ignoring depth and time planes of "
			       "value segment.\n");
      }

      /*-- special case : if we have only one plane of color... --*/
      if (e == 1)
      {
	 if (!has_map) /*-- ...create a greyscale map --*/
	 {
	    kcolor_set_attribute(object, KCOLOR_MAP_AUTOCOLOR, KGREYSCALE);
	    kpds_set_attribute(object, KPDS_MAPPING_MODE, KMAPPED);
	 
	    warn = kstring_append(warn, "- only one plane of value "
				  "information : creating greyscale "
				  "colormap\n");
	    warn = kstring_append(warn, "  check initial image value range if "
				  "results are poor...\n");
	 }
      }
      else if (e > 1) 
      {
	 if (has_map)
	    warn = kstring_append(warn, "- only storing first plane of "
				  "value segment.\n");

	 else /*-- gamut to create map --*/ 
	 {
	    warn = kstring_append(warn, "- only storing red plane of value "
				  "segment.\n");
	    warn = kstring_append(warn, "- use routine igamut to create "
				  "a colormap before saving.\n");
#if 0  
	    /*-- this is really cool and it works from the command-line...
	         but it crashes editimage :-( --*/ 
	    kobject gobject = kpds_create_object();
	    
	    kinfo(KSTANDARD, "XPM format output :\n\n"
		  "Creating colormap...may take awhile.\n");

	    kcolor_gamut_object(object, 255, 8, 0.5, gobject);

	    kpds_copy_value(gobject, object, FALSE);
	    kpds_copy_map(gobject, object, FALSE);
	    kpds_close_object(gobject);
#endif
	 }
      }
      
      /*-- type issue : can only store bit & unsigned byte images --*/
      kpds_get_attribute(object, KPDS_VALUE_DATA_TYPE, &type);

      if (type != KUBYTE)
	 warn = kstring_append(warn, "- casting value data to unsigned byte "
			       "for storage.\n");
   }

   /*-- if we have any news to report, report it now --*/
   if (warn)
   {
      kinfo(KSTANDARD, "XPM format output :\n%s", warn);
      kfree_and_NULL(warn);
   }

   return TRUE;
}

#endif /* KXPM_DEF */

/* don`t add after the endif */
