 /*
  * Khoros: $Id: Kdf.c,v 1.15 1996/06/26 14:47:57 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Kdf.c,v 1.15 1996/06/26 14:47:57 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros KDF Data Service Routines
   >>>>		
   >>>>  Private:
   >>>>                 init_kdf()
   >>>>                 kdf_locate_segment()
   >>>>   Static:
   >>>>                 skip_attribute()
   >>>>                 create_segment()
   >>>>                 read_attribute()
   >>>>                 read_attributes()
   >>>>                 write_attribute()
   >>>>                 write_attributes()
   >>>>                 kdf_check()
   >>>>                 kdf_input()
   >>>>                 kdf_output()
   >>>>                 kdf_destroy()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

static int kdf_check   	      PROTO((int));
static int kdf_input   	      PROTO((kobject, int, int));
static int kdf_output  	      PROTO((kobject, int, int));
static int kdf_destroy 	      PROTO((kobject));
static int kdf_locate_segment PROTO((kobject, char *));

/*=====================================================================*/

/*-----------------------------------------------------------
|  Routine Name: init_kdf
|       Purpose: initialize the kdf file format
------------------------------------------------------------*/
void
init_kdf(void)
{
   kdms_add_file_format("kdf", 
			"Khoros 2.0 Data Format (kdf)",
			kdf_check, 
			kdf_input, 
			kdf_output, 
			kdf_destroy, NULL, 
			kdf_locate_segment, NULL, NULL, NULL, NULL);
   return;
}

/*=====================================================================*/

/*
 *  Internal Resource Structure 
 *  kdf  -  New Khoros 2.0 Visualization Image Format
 */
typedef struct
{
   int      fid;			/* monkey food I  */
   long int offset;
   klist   *seginfo;
}
ResourceStruct;

static unsigned char magic[] = { 01, 03, 19, 94, 0, 2 };

static char eoa[] = {"<>"};

/*-----------------------------------------------------------
|
|  Routine Name: (static) skip_attribute
|
|       Purpose: skip attirbute data 
|
|         Input:
|    Written By: Steve Kubica 
|          Date: September 16, 1995
|
------------------------------------------------------------*/
static void
skip_attribute(int fid)
{
   char c;
   do 
   {  
      do kread_byte(fid, &c, (size_t) 1); while (c != eoa[0]);
      kread_byte(fid, &c, (size_t) 1);
   } while (c != eoa[1]);
   kread_byte(fid, &c, (size_t) 1);  /*-- consume the '\0' --*/
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) create_segment
|
|       Purpose: creates a segment and initializes it
|
|         Input: 
|    Written By: Mark Young
|          Date: June 04, 1995
|
------------------------------------------------------------*/
static void
create_segment(kobject object, ResourceStruct *resources, char *segment)
{
   char *datatype;
   int i, total, mach, dimension, type, fixed_dim, dim_index;
   int size[5], order[5];

   kread_int(resources->fid, &dimension, (size_t) 1);
   kread_string(resources->fid, &datatype, (size_t) 1);
   kread_int(resources->fid, size, (size_t) dimension);
   kread_int(resources->fid, order, (size_t) dimension);
   kread_int(resources->fid, &fixed_dim, (size_t) 1);
   kread_int(resources->fid, &dim_index, (size_t) 1);
   type = kdatatype_to_define(datatype);
   kdms_set_segment(object, segment, dimension, size, order, type, NULL);

   mach  = kfile_getmachtype(resources->fid);
   if (type == KBIT)
      total = (size[0]+7)/8;
   else
      total = size[0]*kmach_sizeof((unsigned int) mach, (unsigned int) type);

   for (i = 1; i < dimension; i++)
      total *= size[i];

   resources->seginfo = klist_add(resources->seginfo,
				  (kaddr) kstring_to_token(segment),
				  (kaddr) resources->offset);
   resources->offset += (long int) total;
   kfree_and_NULL(datatype);
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) read_attribute
|
|       Purpose: write out a single attribute
|
|         Input: 
|    Written By: Mark Young
|          Date: June 04, 1995
|
------------------------------------------------------------*/
static void
read_attribute(kobject object, ResourceStruct *resources, char *segment)
{
   kaddr data;
   char  tmp[3], *name, *datatype;
   int   num, size, type, info[2];


   kread_string(resources->fid, &name, (size_t) 1);
   kread_int(resources->fid, info, (size_t) 2);
   kread_string(resources->fid, &datatype, (size_t) 1);
   num = info[0]; size = info[1]; type = kdatatype_to_define(datatype);

   data = kmalloc((size_t)(num * size * kdata_size(type)));
   if (kread_generic(resources->fid, data, (size_t) (num*size), type) 
       != (ssize_t) -1)
   {
      kdms_create_attribute(object, segment, name, num, size, type, TRUE,TRUE);
      kdms_set_Vattribute(object, segment, name, data);
      kread(resources->fid, tmp, (size_t) 3);
   }
   else /*-- skip the attribute --*/
   {
       skip_attribute(resources->fid);
       kinfo(KSTANDARD, "Warning : \n" 
	      "\tUnable to read attribute \"%s\"\n"
	      "\tAttribute datatype \"%s\" is not defined.\n",
	      name, datatype);
   }
   kfree_and_NULL(name);
   kfree_and_NULL(datatype);
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) read_attributes
|
|       Purpose: read in attributes and segment info
|
|         Input: 
|    Written By: Mark Young
|          Date: June 04, 1995
|
------------------------------------------------------------*/
static void
read_attributes(kobject object, ResourceStruct *resources)
{
   int   i, num = 0;
   char  *segment, *orig_segment, *name;


   kread_string(resources->fid, &segment, (size_t) 1);
   kread_int(resources->fid, &num, (size_t) 1);
   orig_segment = segment;
   if ((name = kstring_cleanup(segment, segment)) != NULL)
      create_segment(object, resources, segment);
   else
      segment = KDMS_OBJECT;

   for (i = 0; i < num; i++)
      read_attribute(object, resources, name);

   kfree_and_NULL(orig_segment);
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) write_attribute
|
|       Purpose: write out a single attribute
|
|         Input: 
|    Written By: Mark Young
|          Date: June 04, 1995
|
------------------------------------------------------------*/
static void
write_attribute(kobject object, ResourceStruct *resources, char *segment,
	        char *name)
{
   kaddr tmp;
   char  *datatype;
   int   num, size, type, info[2];
   
   
   kdms_query_attribute(object, segment, name, &num, &size, &type, NULL);
   if (num <= 0 || size <= 0)
      return;
   
   kdms_get_Vattribute(object, segment, name, &tmp);
   info[0] = num; info[1] = size; datatype = kdefine_to_datatype(type);
   
   kwrite_string(resources->fid, &name, (size_t) 1);
   kwrite_int(resources->fid, info, (size_t) 2);
   kwrite_string(resources->fid, &datatype, (size_t) 1);
   kwrite_generic(resources->fid, tmp, (size_t) (num*size), type);
   kwrite(resources->fid, eoa, (size_t) 3);
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) write_attributes
|
|       Purpose: write out attributes and segment info
|
|         Input:
|    Written By: Mark Young
|          Date: June 04, 1995
|
------------------------------------------------------------*/
static void
write_attributes(kobject object, ResourceStruct *resources, char *segment)
{
   char  **names, *datatype;
   int   i, mach, total, dimension, type, *size, *order, num = 0;
   int   fixed_dim = -1, dim_index = -1;  /* poss. future multi-res support */
 

   /* Get the segment attributes names.  */
   names = kdms_get_attribute_names(object, segment, NULL, TRUE, &num);

   /*
    *  First write out the number of segment attributes.  Note: if the
    *  segment is general (KDMS_OBJECT) then we aren't going to write
    *  out the general segment information, so the number of attributes
    *  is whats returned from kdms_get_attribute_names().  Otherwise we
    *  write out the dimension, type, size, and index order information.
    */
   kwrite_string(resources->fid, &segment, (size_t) 1);
   kwrite_int(resources->fid, &num, (size_t) 1);

   /* Write out segment specific attributes if segment is not KDMS_OBJECT */
   if (kstrcmp(segment, KDMS_OBJECT) != 0)
   {
      kdms_get_attributes(object, segment,
		KDMS_PHYSICAL_DIMENSION,   &dimension,
		KDMS_PHYSICAL_DATA_TYPE,   &type,
		KDMS_PHYSICAL_SIZE,	   &size,
		KDMS_PHYSICAL_INDEX_ORDER, &order,
		NULL);
      datatype = kdefine_to_datatype(type);
      kwrite_int(resources->fid, &dimension, (size_t) 1);
      kwrite_string(resources->fid, &datatype, (size_t) 1);
      kwrite_int(resources->fid, size, (size_t) dimension);
      kwrite_int(resources->fid, order, (size_t) dimension);

      /* write out fixed_dim and dim_index for future multi-res support */
      /* currently hardwired to -1 to represent not available */
      kwrite_int(resources->fid, &fixed_dim, (size_t) 1);
      kwrite_int(resources->fid, &dim_index, (size_t) 1);

      mach  = kmach_local();
      if (type == KBIT)
         total = (size[0]+7)/8;
      else
         total = size[0]*kmach_sizeof((unsigned int) mach,(unsigned int) type);

      for (i = 1; i < dimension; i++)
         total *= size[i];

      resources->seginfo = klist_add(resources->seginfo,
				(kaddr) kstring_to_token(segment),
				(kaddr) resources->offset);
      resources->offset += (long int) total;
   }

   /* Write out general segment attributes */
   for (i = 0; i < num; i++)
      write_attribute(object, resources, segment, names[i]);

   karray_free(names, KSTRING, (size_t) num, NULL);
}

/****    =============================================================
 ****    Format Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: (static) kdf_check
|
|       Purpose: check to see if the file is really a kdf.
|
|         Input: 
|        Output: 
|       Returns: TRUE on success, FALSE otherwise
|    Written By: Mark Young
|          Date: June 04, 1995
|
------------------------------------------------------------*/
static int
kdf_check(int fid)
{
   unsigned char tmp[4];

   if ((kread_ubyte(fid, tmp, (size_t) 4) != 4) || 
        kmemcmp(magic, tmp, (size_t)4) != 0)
      return FALSE;

   /* well, it looks like a kdf to me...  */
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) kdf_input
|
|       Purpose: 
|
|         Input: 
|        Output: 
|       Returns: TRUE on success, FALSE otherwise
|    Written By: Mark Young
|          Date: June 04, 1995
|
------------------------------------------------------------*/
static int
kdf_input(kobject object, int fid, int flags)
{
   int      i, num = 0;
   unsigned char mach, tmp[6];
   ResourceStruct *resources;


   if (!kdms_initialized(object))
   {
      if ((resources = kcalloc((size_t)1, sizeof(ResourceStruct))) == NULL)
         return FALSE;
      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   resources->fid     = fid;
   resources->offset  = (long int) 0;
   resources->seginfo = NULL;

   /* read in attributes and create all segments.  */
   kread_ubyte(resources->fid, tmp, (size_t) 6);
   kread_ubyte(resources->fid, &mach, (size_t) 1);
   kfile_setmachtype(resources->fid, (int) mach);

   /* dummy read to read in the number of data sets */
   kread_int(resources->fid, &num, (size_t) 1);

   /* get number of attributes */
   kread_int(resources->fid, &num, (size_t) 1);
   for (i = 0; i < num; i++)
      read_attributes(object, resources);

   resources->offset = (long int) ktell(resources->fid);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) kdf_output
|
|       Purpose: 
|
|         Input: 
|        Output: 
|       Returns: TRUE on success, FALSE otherwise
|    Written By: Mark Young
|          Date: June 04, 1995
|
------------------------------------------------------------*/
static int
kdf_output(kobject object, int fid, int flags)
{
   klist    *list;
   int      total;
   char     *segment;
   unsigned char mach;
   ResourceStruct *resources;

   if (!kdms_initialized(object))
   {
      /* Create the image to the object file */
      if ((resources = kcalloc((size_t)1, sizeof(ResourceStruct))) == NULL)
         return FALSE;

      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   resources->fid     = fid;
   resources->offset  = (long int) 0;
   resources->seginfo = NULL;

   /* dump out the magic number and the machine architecture */
   mach = (unsigned char) kmach_local();
   kwrite_ubyte(resources->fid, magic, (size_t) 6);
   kwrite_ubyte(resources->fid, &mach, (size_t) 1);

   /* dump out total number of data sets (currently hardwired to 1) */
   total = 1;
   kwrite_int(resources->fid, &total, (size_t) 1);

   /* Write out total number of attribute blocks */
   list = kdms_get_segment_list(object);
   total = klist_size(list)+1;
   kwrite_int(resources->fid, &total, (size_t) 1);

   /* write out all attributes and segments.  */
   write_attributes(object, resources, KDMS_OBJECT);

   /*
    *  Write out segment attributes
    */
   while (list != NULL)
   {
       segment = ktoken_to_string(klist_identifier(list));
       write_attributes(object, resources, segment);
       list = klist_next(list);
   }
   resources->offset = ktell(resources->fid);
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) kdf_destroy
|
|       Purpose: 
|
|         Input: 
|        Output: 
|       Returns: TRUE on success, FALSE otherwise
|    Written By: Mark Young
|          Date: June 04, 1995
|
------------------------------------------------------------*/
static int
kdf_destroy(kobject object)
{
   ResourceStruct *resources;

   resources = (ResourceStruct *) kdms_glue_get_resources(object);

   if (resources == NULL)
      return TRUE;

   /* free the resources */
   klist_free(resources->seginfo, NULL);
   kfree(resources);
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) kdf_locate_segment
|
|       Purpose: 
|
|         Input: 
|        Output: 
|       Returns: TRUE on success, FALSE otherwise
|    Written By: Steve Kubica
|          Date: June 04, 1995
|
------------------------------------------------------------*/
static int 
kdf_locate_segment(kobject object, char *segment)
{
   ResourceStruct *resources;
   klist          *list;
   off_t           offset;
   ktoken          seg = kstring_to_token(segment);

   resources = (ResourceStruct *) kdms_glue_get_resources(object);

   if ((list = klist_locate(resources->seginfo, (kaddr) seg)) == NULL)
      return -1;

   offset = (off_t) ((long int) klist_clientdata(list) + resources->offset);
   klseek(resources->fid, offset, KSEEK_SET);

   return resources->fid;
}
