#include "dataserv.h"
#include "io_hypo.h"

#define FAIL(error)               \
        {                         \
          kinfo(KFORCE,error);    \
          kexit(KEXIT_FAILURE);   \
        }

struct hypo *generate_hypothesis(int number, int seed)
{
   struct hypo *rhypo = NULL;

   rhypo = kcalloc((size_t)1, sizeof(struct hypo));

   rhypo->Height = (float) seed++;
   rhypo->Type   = seed++;

   rhypo->Identifiers[0] = seed++;
   rhypo->Identifiers[1] = seed++;
   rhypo->Identifiers[2] = seed++;

   rhypo->Peaks.Peaks_len = 2;
   rhypo->Peaks.Peaks_val = kcalloc((size_t)2, sizeof(float));
   rhypo->Peaks.Peaks_val[0] = (float) seed++;
   rhypo->Peaks.Peaks_val[1] = (float) seed++;
  
   /*-- recursively add another hypo to the next pointer --*/
   if (number!=0)
      rhypo->next = generate_hypothesis(number-1, seed);
   else 
      rhypo->next = NULL; 

   return rhypo;
}

void fprint_hypothesis(kfile *file, struct hypo *phypo)
{
   int i;

   kfprintf(file, "\theight = %g\n", phypo->Height);
   kfprintf(file, "\ttype   = %d\n", phypo->Type);

   kfprintf(file, "\tidentifiers = %d %d %d\n", 
	 phypo->Identifiers[0], phypo->Identifiers[1], phypo->Identifiers[2]);

   kfprintf(file, "\tpeaks = "); 
   for (i = 0; i < phypo->Peaks.Peaks_len; i++)
      kfprintf(file, " %g", phypo->Peaks.Peaks_val[i]);
   kfprintf(file, "\n"); 

   /* recursively print out the next one, if there is one */
   if (phypo->next != NULL)
   {
      kfprintf(file, " ... next hypothesis ... \n");
      fprint_hypothesis(file, phypo->next);
   }

   return;
}

void
main(
   int  argc,
   char *argv[])
{
    struct hypo *a = NULL;
    struct hypo *b = NULL;
    kobject in, out;
    int     number = 2;;

    if (argc > 1)
       number = atoi(argv[1]);

    /* first, define KTYPE_HYPO as a valid datatype */
    kdefine_hypo();

    a = generate_hypothesis(number, number);

    /* second, define "hypothesis" as a defined attribute
     * this attribute will have 1 argument of size 1
     */
    kdms_define_attribute(NULL, "hypothesis", 1, 1, 
                         KTYPE_HYPO, TRUE, TRUE, NULL);

    out = kpds_open_output_object("./test.data");
    kpds_set_attribute(out, "hypothesis", a);

    kpds_create_object_attr(out, "bob!", 1, 1, KINT, TRUE, TRUE);
    kpds_set_attribute(out, "bob!", 1);
    kpds_create_value(out);
    kpds_set_attribute(out, KPDS_VALUE_SIZE, 10, 10, 1, 1, 1);
    kpds_set_attribute(out, KPDS_VALUE_DATA_TYPE, KBYTE);

    kpds_close_object(out);

    in = kpds_open_input_object("./test.data");
    if (!kpds_query_attribute(in, "hypothesis", NULL, NULL, NULL, NULL))
       FAIL("unable to find hypothesis attribute\n");
    kpds_get_attribute(in, "hypothesis", &b);

    if (!kstruct_compare(a, b, KTYPE_HYPO))
    {
       FAIL("structures do not match\n\n");
    }
    else
       kfprintf(kstdout, "structures match\n\n");

    fprint_hypothesis(kstdout, b);
    
    kstruct_free(a, KTYPE_HYPO, TRUE);

    kpds_close_object(in);

    kexit(KEXIT_SUCCESS);
} 
