#include <dataserv.h>
#include "io_test.h"

#define FAIL(error)               \
        {                         \
          kinfo(KFORCE,error);    \
          kexit(KEXIT_FAILURE);   \
        }

#define ITEMS  "items"

void define_attributes PROTO((void));

/*-- 
  -- write out a strucure to a file 
  -- . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . 
  --
  --*/
void define_attributes(void)
{
   kdms_define_attribute(NULL, ITEMS, 1, 1, KTYPE_ITEMS,
			 TRUE, TRUE, NULL);
}

/*-- 
  -- write out a strucure to a file 
  -- . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . 
  --
  --*/
void write_structure(char *name, kaddr s, int type)
{
   int fid = kopen(name, O_WRONLY|O_CREAT|O_TRUNC, 0644);
   kwrite_generic(fid, s, (size_t) 1, type);
   kclose(fid);
}

/*-- 
  -- read in a strucure from a file 
  -- . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . 
  --
  --*/
void read_structure(char *name, kaddr s, int type)
{
   int fid = kopen(name, O_RDONLY, 0644);
   kread_generic(fid, s, (size_t) 1, type);
   kclose(fid);

   /*-- tidy up --*/
   kunlink(name);
}


/*-- 
  -- items routines
  -- . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . 
  -- 
  --*/
struct items *create_items(int z)
{
   struct items *new;

   new = (struct items *) kmalloc(sizeof(struct items));

   new->_char   = (char) z;
   new->_uchar  = (unsigned char) z;
   new->_short  = (short) z;
   new->_ushort = (unsigned short) z;
   new->_int    = (int) z;
   new->_uint   = (unsigned int) z;
   new->_long   = (long) z;
   new->_ulong  = (unsigned long) z;
   new->_float  = (float)  z;     
   new->_double = (double) z;

   return new;
}

void
main()
{
   kobject i, o;
   char *filename = "t.data";

   struct items *a, *b;
   
   /*-- define needed datatypes and attributes --*/
   kdefine_items();
   define_attributes();

   /*
    *   test basic structure of items  
    */
   kprintf("\ttest: structure of items of various datatypes ... ");

   /*-- output coord list --*/
   a = create_items(3);
   o = kpds_open_output_object(filename);
   kpds_set_attribute(o, ITEMS, a);
   kpds_close_object(o);
   
   /*-- input coord list --*/
   i = kpds_open_input_object(filename);
   kpds_get_attribute(i, ITEMS, &b);

   if (!kstruct_compare(a, b, KTYPE_ITEMS))
       FAIL("official structure compare doesn't work\n");

   /*-- tidy up --*/
   kpds_close_object(i);
   kstruct_free(a, KTYPE_ITEMS, TRUE);
   kunlink(filename);
   kprintf("successful\n");

   kexit(KEXIT_SUCCESS);
}
