#include <xvinclude.h>

/*
 * This example creates a set of pixmap objects, laying them out in rows
 * by using a rowcol object as the parent.  The pixmap object, as opposed to 
 * a label displaying a pixmap, is slightly faster and designed specifically 
 * for the display of pixmaps.  
 */
main(
   int  argc,
   char *argv[])
{
	int      i;
	xvobject rowcol;
	xvobject pixmap;

 static char *filenames[] = {"backgrounds:brown_marble",
                             "backgrounds:black_marble",
                             "backgrounds:purple_marble",
                             "backgrounds:gravel_marble",
                             "backgrounds:purple_maze",
                             "backgrounds:wood_pulp",
                             "backgrounds:bw_marble"};

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "DESIGN");

	/* initialize the xvwidgets lib */
	if (!xvw_initialize(XVW_MENUS_XVFORMS))
	{
	   kerror(NULL, "main", "unable to open display");
	   kexit(KEXIT_FAILURE);
	}

	/*
	 *  create a RowCol object to serve as the backplane,
         *  so that pixmaps can be laid out automatically, without
         *  using relative or absolute layout.
	 */
	rowcol = xvw_create_rowcol(NULL, "rowcol");
	xvw_set_attributes(rowcol,
			   XVW_WIDTH,  400,
			   XVW_HEIGHT, 400,
			   NULL);

	for (i = 0; i < knumber(filenames); i++)
	{
	     /*
	      *  Create the pixmap objects and set the file defining
              *  each pixmap to be displayed.
	      */
	    pixmap = xvw_create_pixmap(rowcol, "pixmap");
	    xvw_set_attribute(pixmap, XVW_PIXMAP_FILENAME, filenames[i]);
	}

	/* display & run the program */
	xvf_run_form();
}
