#include <xvinclude.h>

static void quit_cb PROTO((xvobject, kaddr, kaddr));

/*
 *  This program demonstrates the use of the commandbar object, as well
 *  as the use of an announcement handler.
 *
 *  It creates a workspace with a commandbar and quit button,
 *  and restores the color arithmetic workspace "workspaces:ColorArith1".
 *  The commandbar object may be used to perform various operations,
 *  such as clearing the workspace, restoring another workspace, creating
 *  a loop or a procedure, deleting selected glyphs, and so on.
 *
 *  A labelstring object is created beneath the command bar which will
 *  be updated by the announcement handler which is installed using
 *  kset_announcehandler().  This allows the label string to 
 *  echo what each pixmap button on the command bar does when the pointer
 *  is moved over the command bar.
 */
static int update_status PROTO((char *, char *, char *, char *, char *));
static void quit_cb      PROTO((xvobject, kaddr, kaddr));

xvobject status;
main(
   int  argc,
   char *argv[])
{
	xvobject parent, workspace, commandbar, quit;
	char  *filename = "workspaces:ColorArith1";

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "DESIGN");

	/* initialize the xvwidgets lib */
	if (!xvw_initialize(XVW_MENUS_XVFORMS))
	{
	   kerror(NULL, "main", "unable to open display");
	   kexit(KEXIT_FAILURE);
	}

	/*
	 *  Create the manager.  Note that since the parent is NULL, a
	 *  toplevel window will be created and the manager display placed
	 *  inside.
	 */
	parent = xvw_create_manager(NULL, "Parent");

	/*
	 *  Create the quit button so that the user can quit
	 */
        quit = xvw_create_button(parent, "Quit");
        xvw_set_attributes(quit,
                XVW_LABEL, "Quit",
                XVW_BELOW, NULL,
                XVW_LEFT_OF, NULL,
                XVW_CHAR_WIDTH, 6.0,
                XVW_CHAR_HEIGHT, 1.2,
                NULL);

        /*
         *  Create the commandbar
         */
        commandbar = xvw_create_commandbar(parent, "CommandBar");
        xvw_set_attributes(commandbar,
                XVW_BELOW,    NULL,
                XVW_RIGHT_OF, NULL,
                XVW_LEFT_OF, quit,
                XVW_TACK_EDGE, KMANAGER_TACK_HORIZ,
                NULL);

	/*
	 *  Create the status labelstring
	 */
	status = xvw_create_label(parent, "Status");
        xvw_set_attributes(status,
                XVW_BORDER_WIDTH,    2,
                XVW_TACK_EDGE,       KMANAGER_TACK_HORIZ,
	        XVW_BELOW,           commandbar,
	        XVW_CHAR_MIN_HEIGHT, 1.0,
                XVW_FORCE_REDISPLAY, TRUE,
                NULL);
	/*
	 *  by using kset_announcehandler() to install the update_status()
	 *  announcement handler, all calls to kannounce() made by the
	 *  command bar object will be directed to the update_status() routine,
	 *  which in turn will update the status labelstring.
	 */
	kset_announcehandler(update_status);

	/*
	 *  Create the workspace.  The XVW_WORKSPACE_RESTORE attribute is
	 *  used to specify the workspace file to be restored.
	 */
	workspace = xvw_create_workspace(parent, "workspace");
	xvw_set_attributes(workspace,
		XVW_BELOW,             status,
		XVW_WIDTH,             750,
		XVW_HEIGHT,            750,
		XVW_WORKSPACE_RESTORE, filename,
		NULL);
	xvw_set_attribute(commandbar, XVW_COMMANDBAR_WORKSPACE, workspace);
	xvw_add_callback(quit, XVW_BUTTON_SELECT, quit_cb, parent);

	/* display & run the program */
	xvf_run_form();
}

/* ARGSUSED */
static void quit_cb(
    xvobject object,
    kaddr client_data,
    kaddr call_data)
{
        xvobject parent = (xvobject) client_data;
        xvw_destroy(parent);
}

/* ARGSUSED */
static int update_status(
   char *toolbox,
   char *program,
   char *library,
   char *routine,
   char *message)
{
        xvw_set_attribute(status, XVW_LABEL, message);
        return(TRUE);
}

