
#include <xvinclude.h>

/*
 *   This example demonstrates the use of the toolbox list, a set of
 *   three list objects which list the available operators in the Khoros 
 *   system by category / subcategory / icon name.
 *   Event handlers are installed for selection of an operator from the
 *   toolboxlist, and creation of an operator from the toolboxlist.
 *
 *   To use:
 *   1) Select a category, subcategory, or icon name from the toolboxlist.  
 *      The information associated with the operator that corresponds to
 *      the currently selected category, subcategory, and icon name 
 *      will be printed to the tty.
 *   2) Double click on an icon name in the toolboxlist.  The cursor
 *      will change to the zen sign at which time you can move the
 *      pointer into the workspace and place a newly created glyph.
 */
static void toolboxlist_select_cb PROTO((xvobject, kaddr, kaddr));
static void toolboxlist_create_cb PROTO((xvobject, kaddr, kaddr));

xvobject workspace;

main(
   int  argc,
   char *argv[])
{
	xvobject parent, toolboxlist;

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "DESIGN");

	/* initialize the xvwidgets lib */
	if (!xvw_initialize(XVW_MENUS_XVFORMS))
	{
	   kerror(NULL, "main", "Unable to open display");
	   kexit(KEXIT_FAILURE);
	}

	/*
	 * create the toolbox list object, and install event
	 * handlers to print information on selection of an operator, and
	 * to create a glyph in the workspace.
	 */
	toolboxlist = xvw_create_toolboxlist(NULL, "toolboxlist");
	xvw_set_attributes(toolboxlist,
			   XVW_WIDTH,  500,
			   XVW_HEIGHT, 300,
			   NULL);

	xvw_add_callback(toolboxlist, XVW_TOOLBOXLIST_SELECT, 
			 toolboxlist_select_cb, NULL);
	xvw_add_callback(toolboxlist, XVW_TOOLBOXLIST_CREATE, 
			 toolboxlist_create_cb, NULL);

	workspace = xvw_create_workspace(NULL, "workspace");
	xvw_set_attributes(workspace,
			   XVW_WIDTH,  500,
			   XVW_HEIGHT, 600,
			   NULL);

	/* display & run the program */
	xvf_run_form();
}

void toolboxlist_select_cb(
    xvobject object, 
    kaddr client_data, 
    kaddr call_data)

{
	xvw_program_info *program_info = (xvw_program_info *) call_data;

	kfprintf(kstderr, "Program Info:\n");
	kfprintf(kstderr, "Toolbox name = %s\n",   program_info->tbname);
	kfprintf(kstderr, "Object  name = %s\n",   program_info->oname);
	kfprintf(kstderr, "Category     = %s\n",   program_info->category);
	kfprintf(kstderr, "Subcategory  = %s\n",   program_info->subcategory);
	kfprintf(kstderr, "Icon Name    = %s\n",   program_info->icon_name);
	kfprintf(kstderr, "Pane File    = %s\n",   program_info->panefile);
	kfprintf(kstderr, "Workspace    = %s\n",   program_info->workspace);
	kfprintf(kstderr, "Short Desc   = %s\n\n", program_info->short_desc);

}

void toolboxlist_create_cb(
    xvobject object,
    kaddr client_data,
    kaddr call_data)

{
	xvobject glyph;
        xvw_program_info *program_info = (xvw_program_info *) call_data;

	if (program_info->type != XVL_OPERATOR)
	{
	    kinfo(KSTANDARD, "Sorry, but we are not dealing with procedures, loops, or conditionals in this example");
	    return;
	}
	glyph = xvw_create_glyph(workspace, "glyph");
        xvw_set_attributes(glyph,
                XVW_NODE_NAME,      program_info->icon_name,
                XVW_GLYPH_FORMFILE, program_info->panefile,
                XVW_GLYPH_TBNAME,   program_info->tbname,
                XVW_GLYPH_ONAME,    program_info->oname,
                NULL);

}
