/*
  * Khoros: $Id: GraphicsAxisMode.h,v 1.5 1996/04/04 23:21:51 fas Exp $
  */


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: #defines for graphics attributes that
   >>>>                control the axis mode (linear, log, ln).
   >>>>
   >>>>    Written By: John Salas, Danielle Argiro, Mark Young
   >>>>
   >>>>          Date: Aug 20, 1994
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvisual_GraphicsAxisMode_h_
#define _xvisual_GraphicsAxisMode_h_

	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/

/*------------------------------------------------------------------------
| Attribute:    XVW_GRAPHICS_MODE_X
| Description:  Whether the X Axis is marked as linear or log 10 scale.
| Type:         int
| Default:      .symbol index KGRAPHICS_LINEAR
| Legal Values: .symbol index KGRAPHICS_LINEAR
|               .symbol index KGRAPHICS_LOG10
| Resource:     graphicsModeX
------------------------------------------------------------------------*/
#define XVW_GRAPHICS_MODE_X             "graphicsModeX"

/*------------------------------------------------------------------------
| Attribute:    XVW_GRAPHICS_MODE_Y
| Description:  Whether the Y Axis is marked as linear or log 10 scale.
| Type:         int
| Default:      .symbol index KGRAPHICS_LINEAR
| Legal Values: .symbol index KGRAPHICS_LINEAR
|               .symbol index KGRAPHICS_LOG10
| Resource:     graphicsModeY
------------------------------------------------------------------------*/
#define XVW_GRAPHICS_MODE_Y             "graphicsModeY"

/*------------------------------------------------------------------------
| Attribute:    XVW_GRAPHICS_MODE_Z
| Description:  Whether the Z Axis is marked as linear or log 10 scale.
| Type:         int
| Default:      .symbol index KGRAPHICS_LINEAR
| Legal Values: .symbol index KGRAPHICS_LINEAR
|               .symbol index KGRAPHICS_LOG10
| Resource:     graphicsModeZ
------------------------------------------------------------------------*/
#define XVW_GRAPHICS_MODE_Z             "graphicsModeZ"

/*------------------------------------------------------------------------
| Attribute:    XVW_GRAPHICS_PROPORTIONAL
|
| Description:  When this attribute is set to 
|               .symbol index KGRAPHICS_NONPROP ,
|               the scale in each axis direction is determined by the
|               world coordinate minimums and maximums of the data. Thus,
|               if the X values range from 0 to 1, and the Y values range
|               from 1 to 100, and the physical length of the X and Y axes
|               are the same, then the scale in the X direction (1) is much
|               smaller than the scale in the Y direction (100).
|               .sp
|               In contrast, when this attribute is set to
|               .symbol index KGRAPHICS_PROP_WINDOWED 
|               or 
|               .symbol index KGRAPHICS_PROP_NONWINDOWED ,
|               it causes the scale in each axis direction to be the same.  
|               When a visual object is proportional, this means that the 
|               minimum and maximum values across being displayed as X and Y 
|               are found before the object is displayed.  Then the minimum 
|               and maximum are set on each axis so that the scale is the same.
|               .sp
|               If 
|               .symbol index KGRAPHICS_PROP_WINDOWED
|               is specified, then the center of the data is determined along 
|               each axis and the minimum and maximum values are set such that 
|               the data is in the center of the world coordinate space. If
|               .symbol index KGRAPHICS_PROP_NONWINDOWED 
|               is specified, the range of numbers covered, i.e, the minimum 
|               and maximum values are the same, by the X and Y axes are the
|               same.  The "sense of proportion" conveyed by the object is 
|               correct, but details of the object may be lost if the range 
|               spanned by X varies greatly from that spanned by Y.
|               .sp
|               In some cases, proportional display not be a good method of 
|               display.  For example, suppose a 2D plot object has data 
|               displayed as X ranges from 0 to 255, but the data displayed as 
|               Y ranges from 0 to 1.  Obviously, the plot will not be very 
|               informative, as all the points will be bunched against the X 
|               axis.  In cases like this, non-proportional display may be 
|               more useful.
|
| Type:         int
| Default:      .symbol index KGRAPHICS_NONPROP
| Legal Values: .symbol index KGRAPHICS_NONPROP
|               .symbol index KGRAPHICS_PROP_WINDOWED
|               .symbol index KGRAPHICS_PROP_NONWINDOWED
| Resource:     graphicsProportional
-----------------------------------------------------------------------*/
#define XVW_GRAPHICS_PROPORTIONAL	"graphicsProportional"

#endif /* _xvisual_GraphicsAxisMode_h_ */
/* Don't add after this point */
