/*
  * Khoros: $Id: GraphicsVP.h,v 1.5 1996/04/04 23:21:58 fas Exp $
  */


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: #defines for graphics attributes that
   >>>>                control the viewport coordinate system.
   >>>>
   >>>>    Written By: John Salas, Danielle Argiro, Mark Young
   >>>>
   >>>>          Date: Aug 20, 1994
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvisual_GraphicsVP_h_
#define _xvisual_GraphicsVP_h_

	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/

/*------------------------------------------------------------------------
| Attribute:    XVW_GRAPHICS_VIEWPORT_MIN_X
|
| Description:  This attribute specifies the minimum viewport value 
|               in the X direction.  Note that this attribute should 
|		\fIonly\fP be set on the visual object that is serving 
|		as the "controlling" visual object (see Section C.4, 
|		"Attaching the World View").
|
| Type:         double
| Default:      0.0
| Legal Values: value <
|               .symbol index XVW_GRAPHICS_VIEWPORT_MAX_X
| Resource:     graphicsViewportMinX
------------------------------------------------------------------------*/
#define XVW_GRAPHICS_VIEWPORT_MIN_X     "graphicsViewportMinX"

/*------------------------------------------------------------------------
| Attribute:    XVW_GRAPHICS_VIEWPORT_MIN_Y
|
| Description:  This attribute specifies the minimum viewport value 
|               in the Y direction.  Note that this attribute should 
|               \fIonly\fP be set on the visual object that is serving 
|               as the "controlling" visual object (see Section C.4, 
|               "Attaching the World View").
|
| Type:         double
| Default:      0.0
| Legal Values: value <
|               .symbol index XVW_GRAPHICS_VIEWPORT_MAX_Y
| Resource:     graphicsViewportMinY
------------------------------------------------------------------------*/
#define XVW_GRAPHICS_VIEWPORT_MIN_Y     "graphicsViewportMinY"

/*------------------------------------------------------------------------
| Attribute:    XVW_GRAPHICS_VIEWPORT_MIN_Z
|
| Description:  This attribute specifies the minimum viewport value 
|               in the Z direction.  Note that this attribute should 
|               \fIonly\fP be set on the visual object that is serving 
|               as the "controlling" visual object (see Section C.4, 
|               "Attaching the World View").
|               Note that this attribute is only applicable to 3D objects.
|
| Type:         double
| Default:      0.0
| Legal Values: value <
|               .symbol index XVW_GRAPHICS_VIEWPORT_MAX_Z
| Resource:     graphicsViewportMinZ
------------------------------------------------------------------------*/
#define XVW_GRAPHICS_VIEWPORT_MIN_Z     "graphicsViewportMinZ"

/*------------------------------------------------------------------------
| Attribute:    XVW_GRAPHICS_VIEWPORT_MAX_X
|
| Description:  This attribute specifies the maximum viewport value 
|               in the X direction.  Note that this attribute should 
|               \fIonly\fP be set on the visual object that is serving 
|               as the "controlling" visual object (see Section C.4, 
|               "Attaching the World View").
|
| Type:         double
| Default:      1.0
| Legal Values: value >
|               .symbol index XVW_GRAPHICS_VIEWPORT_MIN_X
| Resource:     graphicsViewportMaxX
------------------------------------------------------------------------*/
#define XVW_GRAPHICS_VIEWPORT_MAX_X     "graphicsViewportMaxX"

/*------------------------------------------------------------------------
| Attribute:    XVW_GRAPHICS_VIEWPORT_MAX_Y
|
| Description:  This attribute specifies the maximum viewport value 
|               in the Y direction.  Note that this attribute should 
|               \fIonly\fP be set on the visual object that is serving 
|               as the "controlling" visual object (see Section C.4, 
|               "Attaching the World View").
|
| Type:         double
| Default:      1.0
| Legal Values: value >
|               .symbol index XVW_GRAPHICS_VIEWPORT_MIN_Y
| Resource:     graphicsViewportMaxY
------------------------------------------------------------------------*/
#define XVW_GRAPHICS_VIEWPORT_MAX_Y     "graphicsViewportMaxY"

/*------------------------------------------------------------------------
| Attribute:    XVW_GRAPHICS_VIEWPORT_MAX_Z
|
| Description:  This attribute specifies the maximum viewport value 
|               in the Z direction.  Note that this attribute should 
|               \fIonly\fP be set on the visual object that is serving 
|               as the "controlling" visual object (see Section C.4, 
|               "Attaching the World View").
|               Note that this attribute is only applicable to 3D objects.
|
| Type:         double
| Default:      1.0
| Legal Values: value >
|               .symbol index XVW_GRAPHICS_VIEWPORT_MIN_Z
| Resource:     graphicsViewportMaxZ
------------------------------------------------------------------------*/
#define XVW_GRAPHICS_VIEWPORT_MAX_Z     "graphicsViewportMaxZ"

#endif /* _xvisual_GraphicsVP_h_ */
/* Don't add after this point */
