/*
  * Khoros: $Id: Plot3D.h,v 1.7.4.1 1996/12/06 20:28:53 fas Exp $
  */


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: #defines and #includes for creating a
   >>>>		       graphical plot3d object.
   >>>>
   >>>>    Written By: John Salas and Mark Young
   >>>>
   >>>>          Date: Aug 05, 1992 14:27
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvisual_Plot3D_h_
#define _xvisual_Plot3D_h_

    /*-------------------------------------*
    |       #defines 
    ---------------------------------------*/

/*
 *  Attribute definitions for Plot 3D Visual Object
 */

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_PLOTOBJ
|
| Description:  The data object containing the 3D plot data to be displayed.
|               Note that this attribute is mutually exclusive with
|               .symbol index XVW_PLOT3D_PLOTFILE ;
|               use one or the other, not both.
|
| Type:         kobject
| Default:      NULL
| Legal Values: any valid data object
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT3D_PLOTOBJ              "plot3DPlotobj"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_PLOTFILE
|
| Description:  The file containing the 3D plot data to be displayed.
|               Note that this attribute is mutually exclusive with
|               .symbol index XVW_PLOT3D_PLOTOBJ ;
|               use one or the other, not both.
|
| Type:         char *
| Default:      NULL
| Legal Values: any valid input file
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT3D_PLOTFILE             "plot3DPlotfile"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_COLOR_ORIGINATION
|
| Description:  This attribute is only used when the plot data is provided
|               via a kobject specified using the
|               .symbol index XVW_PLOT3D_PLOTOBJ
|               or
|               .symbol index XVW_PLOT3D_PLOTFILE
|               attributes.  It dictates whether the color of the plot is to
|               be specified by the plot foreground color or by the plot data
|               itself.
|
|               If the entire plot is to appear in a particular
|               color, set this attribute to
|               .symbol index KPLOT3D_COLOR_FROM_FOREGROUND;
|               then, specify the desired color using the
|               .symbol index XVW_FOREGROUND
|               or
|               .symbol index XVW_FOREGROUND_COLOR
|               attributes.
|
|               If each point in the data set is to have its color specified
|               individually, set this attribute to
|               .symbol index KPLOT3D_COLOR_FROM_DATA .
|               The color of each point in the plot will be dictated
|               by the values stored in the kobject; see ??? for details
|               on how the color is interpreted from the data of the kobject.
|
| Type:         int
| Default:      .symbol index KPLOT3D_COLOR_FROM_FOREGROUND
| Legal Values: .symbol index KPLOT3D_COLOR_FROM_FOREGROUND
|               or
|               symbol index KPLOT3D_COLOR_FROM_DATA
| Resource:     plot3DColorOrigination
------------------------------------------------------------------------*/
#define XVW_PLOT3D_COLOR_ORIGINATION         "plot3DColorOrigination"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_DATA_MAX_X
|
| Description:  This \fIread-only\fP attribute returns the maximum of the
|               X coordinates in the 3D plot data currently in the Coord array
|               specified with
|               .symbol index XVW_PLOT3D_POINTS .
|
| Type:         double
| Default:      1.0
| Legal Values: The maximum X world coordinate value (read only)
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT3D_DATA_MAX_X           "plot3DDataMaxX"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_DATA_MAX_Y
|
| Description:  This \fIread-only\fP attribute returns the maximum of the
|               Y coordinates in the 3D plot data currently in the Coord array
|               specified with
|               .symbol index XVW_PLOT3D_POINTS .
|
| Type:         double
| Default:      1.0
| Legal Values: The maximum Y world coordinate value (read only)
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT3D_DATA_MAX_Y           "plot3DDataMaxY"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_DATA_MAX_Z
|
| Description:  This \fIread-only\fP attribute returns the maximum of the
|               Z coordinates in the 3D plot data currently in the Coord array
|               specified with
|               .symbol index XVW_PLOT3D_POINTS .
|
| Type:         double
| Default:      1.0
| Legal Values: The maximum Z world coordinate value (read only)
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT3D_DATA_MAX_Z           "plot3DDataMaxZ"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_DATA_MIN_X
|
| Description:  This \fIread-only\fP attribute returns the minimum of the
|               X coordinates in the 3D plot data currently in the Coord array
|               specified with
|               .symbol index XVW_PLOT3D_POINTS .
|
| Type:         double
| Default:      1.0
| Legal Values: The minimum X world coordinate value (read only)
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT3D_DATA_MIN_X           "plot3DDataMinX"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_DATA_MIN_Y
|
| Description:  This \fIread-only\fP attribute returns the minimum of the
|               Y coordinates in the 3D plot data currently in the Coord array
|               specified with
|               .symbol index XVW_PLOT3D_POINTS .
|
| Type:         double
| Default:      1.0
| Legal Values: The minimum Y world coordinate value (read only)
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT3D_DATA_MIN_Y           "plot3DDataMinY"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_DATA_MIN_Z
|
| Description:  This \fIread-only\fP attribute returns the minimum of the
|               Z coordinates in the 3D plot data currently in the Coord array
|               specified with
|               .symbol index XVW_PLOT3D_POINTS .
|
| Type:         double
| Default:      1.0
| Legal Values: The minimum Z world coordinate value (read only)
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT3D_DATA_MIN_Z           "plot3DDataMinZ"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_PLOTTYPE
| Description:  This attribute indicates how the plot is to be displayed. 
|               Choices include:
|		.symbol index KPLOT3D_LINEPLOT
|		.symbol index KPLOT3D_MESH
|		.symbol index KPLOT3D_SCATTER
|		.symbol index KPLOT3D_CONTOUR_3D
|		.symbol index KPLOT3D_CONTOUR_2D
|		.symbol index KPLOT3D_IMPULSE
|		.symbol index KPLOT3D_PHONG_SHADING
|		.symbol index KPLOT3D_GHOURAUD_SHADING
|		.symbol index KPLOT3D_CONSTANT_SHADING
|		.symbol index KPLOT3D_HORIZON
|		.symbol index KPLOT3D_WIREFRAME
| Type:         int
| Default:      .symbol index KPLOT3D_LINEPLOT
| Legal Values: .symbol index KPLOT3D_LINEPLOT
|               .symbol index KPLOT3D_MESH
|               .symbol index KPLOT3D_SCATTER
|               .symbol index KPLOT3D_CONTOUR_3D
|               .symbol index KPLOT3D_CONTOUR_2D
|               .symbol index KPLOT3D_IMPULSE
|               .symbol index KPLOT3D_PHONG_SHADING
|               .symbol index KPLOT3D_GHOURAUD_SHADING
|               .symbol index KPLOT3D_CONSTANT_SHADING
|               .symbol index KPLOT3D_HORIZON
|               .symbol index KPLOT3D_WIREFRAME
| Resource:     plot3DPlotType
------------------------------------------------------------------------*/
#define XVW_PLOT3D_PLOTTYPE             "plot3DPlotType"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_NUM_CONTOUR_LEVELS
|
| Description:  The number of contour levels to be drawn when 
|               .symbol index XVW_PLOT3D_PLOTTYPE 
|               is set to 
|               .symbol index KPLOT3D_CONTOUR_3D 
|		or
|               .symbol index KPLOT3D_CONTOUR_2D .
|		If set to the default (-1) value, there will be one contour 
|		level drawn for each Z value of the plot data, where the
|		Z values of the contour levels are calculated automatically.
|		If the number of unique Z values in the plot data exceeds
|		the number of colors that can be displayed, there will be
|		one contour level drawn for each color that can be displayed.
|
|		When the
|               .symbol index KPLOT3D_CONTOUR_LEVELS
|		is set to an array of Real values that
|		specify the Z values of the contour levels,
|		.symbol index XVW_PLOT3D_NUM_CONTOUR_LEVELS
|		is used to indicate the size of the array;
|		If 
|               .symbol index KPLOT3D_CONTOUR_LEVELS
|		is set to NULL (the default value) and 
|               .symbol index XVW_PLOT3D_NUM_CONTOUR_LEVELS
|		is set to a value greater than 0,
|		the Z values of the contour levels will be automatically
|		calculated.
|
| Type:         int
| Default:      -1
| Legal Values: value = -1, or value > 0
| Resource:     plot3DContourLevelNum
------------------------------------------------------------------------*/
#define XVW_PLOT3D_NUM_CONTOUR_LEVELS  "plot3DNumContourLevels"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_CONTOUR_LEVELS
|
| Description:  This attribute may be set to an array of Real
|		values that specify the Z values of the contour levels 
|		to be drawn when 
|               .symbol index XVW_PLOT3D_PLOTTYPE 
|               is set to 
|               .symbol index KPLOT3D_CONTOUR_3D 
|               or
|               .symbol index KPLOT3D_CONTOUR_2D .
|
|               When using this attribute, the
|               .symbol index XVW_PLOT3D_NUM_CONTOUR_LEVELS
|		attribute must be used to specify the size of the Real array.
|
| Type:         Real *
| Default:      NULL
| Legal Values: any array of Real values
| Resource:     plot3DContourLevels
------------------------------------------------------------------------*/
#define XVW_PLOT3D_CONTOUR_LEVELS  "plot3DContourLevels"


/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_SHADETYPE
|
| Description:  For 3D plots that are shaded, ie, when the 
|		.symbol index XVW_PLOT3D_PLOTTYPE
|		is set to one of:
|		.symbol index KPLOT3D_PHONG_SHADING 
|               .symbol index KPLOT3D_GHOURAUD_SHADING
|		or
|               .symbol index KPLOT3D_CONSTANT_SHADING ,
|		this attribute specifies what part of the data 
|		is to dictate the shading.  
|		.br
|               .symbol index KPLOT3D_SHADE_ELEVATION
|		specifies that shading is to be done on elevation,
|		ie, using the \fIz\fP value of each data point as defined
|		by its Coord structure.
|               .br
|               .symbol index KPLOT3D_SHADE_IMAGERY
|		specifies taht shading is to be done on imagery (or color)
|		ie, using the \fId\fP value of each data point as defined
|		by its Coord structure.
|		.br
|		When
|               .symbol index KPLOT3D_SHADE_NORMAL 
|		is used, the normal to each data point is computed, and
|		shading is done using the normals.
|
| Type:         int
| Default:      .symbol index KPLOT3D_SHADE_ELEVATION
| Legal Values: .symbol index KPLOT3D_SHADE_IMAGERY
|               .symbol index KPLOT3D_SHADE_ELEVATION
|               .symbol index KPLOT3D_SHADE_NORMAL
| Resource:     plot3DShadeType
------------------------------------------------------------------------*/
#define XVW_PLOT3D_SHADETYPE            "plot3DShadeType"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_PLOTSIZE
|
| Description:  This is the number of data points contained in the Coord
|               array specified by the attribute
|               .symbol noindex XVW_PLOT3D_POINTS .
|               Note that you \fImust\fP use this attribute to specify the
|               number of points prior to specifying the data points with
|               .symbol noindex XVW_PLOT3D_POINTS .
|
| Type:         int
| Default:      0
| Legal Values: values > 0
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT3D_PLOTSIZE             "plot3DPlotsize"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_PLOTWIDTH
|
| Description:  This attribute dictates the number of data points in 
|               each \fIrow\fP to be plotted.  Thus, if there are 100
|               data points specified by
|               .symbol index XVW_PLOT3D_PLOTSIZE
|		and 
|               .symbol index XVW_PLOT3D_PLOTWIDTH
|               is set to 2, this implies that the 3D plot has 50 continuous
|               lines, each with 2 data points.  Alternatively, if 
|               .symbol index XVW_PLOT3D_PLOTWIDTH
|               was set to 25, this would imply that the same 100 data triplets
|               were to be plotted as 4 continuous lines, each with 25 data
|               points.
|
| Type:         int
| Default:      N/A
| Legal Values: 0 < value <= 
|               .symbol index XVW_PLOT3D_PLOTSIZE
| Resource:     N/A
-----------------------------------------------------------------------*/
#define XVW_PLOT3D_PLOTWIDTH            "plot3DPlotwidth"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_POINTS
|
| Description:  This is the array of coordinates defining the 3D data points
|               in the plot. It is an array of type Coord, where the Coord
|               type is defined as:
|		.begin code
|               typedef  struct
|               {
|                  Real  x, y, z;
|                  Index d;
|               }  Coord;
|               .end code
|               Note that the
|               .symbol index XVW_PLOT3D_PLOTSIZE
|               attribute \fImust\fP be set to the number of points in the
|               Coord array \fIprior\fP to setting the
|               .symbol index XVW_PLOT3D_POINTS
|               attribute.
|               .br
|		If the color of a plot point is to be dictated by either the
|               .symbol index XVW_FOREGROUND
|               or the
|               .symbol index XVW_FOREGROUND_COLOR
|               attributes,
|               set the \fId\fP value to
|               .symbol index KGRAPHICS_UNINITIALIZED .
|
|               Alternatively, you may set the \fId\fP value to the pixel
|               value which specifies the desired color of the plot point.
|               See ??? for more details on specifying plot colors.
|		Remember to mention:
|		.nf
|               .symbol index KPLOT3D_CONTOUR_2D
|               .symbol index KPLOT3D_CONTOUR_3D
|               .symbol index KPLOT3D_CONSTANT_SHADING
|               .symbol index KPLOT3D_PHONG_SHADING
|               .symbol index KPLOT3D_GHOURAUD_SHADING
|		.fi
|               It should be set to a valid color index (on a 8-bit display
|               the range is 0-255 all values outside that range are defaulted
|               to the default foreground color).
|
|
| Type:         Coord *
| Default:      NULL
| Legal Values: array of Coords defining data points for 3D plot
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT3D_POINTS               "plot3DPoints"


/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_X_ORIENTATION
|
| Description:  This attribute is only used when the plot data is provided
|               via a kobject specified using the
|               .symbol index XVW_PLOT3D_PLOTOBJ
|               or
|               .symbol index XVW_PLOT3D_PLOTFILE
|               attributes;  furthermore, it only applies when the plot data
|               is stored in the value segment of the data object.
|
|               It dictates how the X coordinates of the 3D plot data is to be 
|		extracted from the value segment of the data object.  The 3D 
|		plot data is always extracted as a surface, but that surface 
|		may have its X coordinates oriented down width, height, depth, 
|		time, or elements.
| Type:         int
| Default:      .symbol index KWIDTH
| Legal Values: .symbol index KWIDTH
|               .symbol index KHEIGHT
|               .symbol index KDEPTH
|               .symbol index KTIME
|               .symbol index KELEMENTS
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT3D_X_ORIENTATION            "plot3DXOrientation"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_Y_ORIENTATION
|
| Description:  This attribute is only used when the plot data is provided
|               via a kobject specified using the
|               .symbol index XVW_PLOT3D_PLOTOBJ
|               or
|               .symbol index XVW_PLOT3D_PLOTFILE
|               attributes;  furthermore, it only applies when the plot data
|               is stored in the value segment of the data object.
|
|               It dictates how the Y coordinates of the 3D plot data is to be 
|               extracted from the value segment of the data object.  The 3D 
|               plot data is always extracted as a surface, but that surface 
|               may have its Y coordinates oriented down width, height, depth, 
|               time, or elements.
| Type:         int
| Default:      .symbol index KHEIGHT
| Legal Values: .symbol index KWIDTH
|               .symbol index KHEIGHT
|               .symbol index KDEPTH
|               .symbol index KTIME
|               .symbol index KELEMENTS
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT3D_Y_ORIENTATION            "plot3DYOrientation"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_WIDTH_OFFSET
|
| Description:  This attribute is used in conjunction with the
|               .symbol index XVW_PLOT3D_X_ORIENTATION 
|		and
|		.symbol index XVW_PLOT3D_Y_ORIENTATION
|		attributes.  When \fIneither\fP of these attributes is set to
|               .symbol index KWIDTH ,
|               it is used to specify the width offset at which the
|               X and Y coordinates for the plot data are to be extracted.
| Type:         int
| Default:      0
| Legal Values: 0 < value < width of data object value segment
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT3D_WIDTH_OFFSET            "plot3DWidthOffset"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_HEIGHT_OFFSET
|
| Description:  This attribute is used in conjunction with the
|               .symbol index XVW_PLOT3D_X_ORIENTATION 
|               and
|               .symbol index XVW_PLOT3D_Y_ORIENTATION
|               attributes.  When \fIneither\fP of these attributes is set to
|               .symbol index KHEIGHT ,
|               it is used to specify the height offset at which the
|               X and Y coordinates for the plot data are to be extracted.
| Type:         int
| Default:      0
| Legal Values: 0 < value < height of data object value segment
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT3D_HEIGHT_OFFSET            "plot3DHeightOffset"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_DEPTH_OFFSET
|
| Description:  This attribute is used in conjunction with the
|               .symbol index XVW_PLOT3D_X_ORIENTATION 
|               and
|               .symbol index XVW_PLOT3D_Y_ORIENTATION
|               attributes.  When \fIneither\fP of these attributes is set to
|               .symbol index KDEPTH ,
|               it is used to specify the depth offset at which the
|               X and Y coordinates for the plot data are to be extracted.
| Type:         int
| Default:      0
| Legal Values: 0 < value < depth of data object value segment
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT3D_DEPTH_OFFSET            "plot3DDepthOffset"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_TIME_OFFSET
|
| Description:  This attribute is used in conjunction with the
|               .symbol index XVW_PLOT3D_X_ORIENTATION 
|               and
|               .symbol index XVW_PLOT3D_Y_ORIENTATION
|               attributes.  When \fIneither\fP of these attributes is set to
|               .symbol index KTIME ,
|               it is used to specify the time offset at which the
|               X and Y coordinates for the plot data are to be extracted.
| Type:         int
| Default:      0
| Legal Values: 0 < value < time of data object value segment
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT3D_TIME_OFFSET            "plot3DTimeOffset"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT3D_ELEMENTS_OFFSET
|
| Description:  This attribute is used in conjunction with the
|               .symbol index XVW_PLOT3D_X_ORIENTATION 
|               and
|               .symbol index XVW_PLOT3D_Y_ORIENTATION
|               attributes.  When \fIneither\fP of these attributes is set to
|               .symbol index KELEMENTS ,
|               it is used to specify the elements offset at which the
|               X and Y coordinates for the plot data are to be extracted.
| Type:         int
| Default:      0
| Legal Values: 0 < value < elements of data object value segment
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT3D_ELEMENTS_OFFSET            "plot3DElementsOffset"


/*
 *  Value definitions for Plot 3D Visual Object
 */

/* Values allowed for XVW_PLOT3D_COLOR_ORIGINATION */
#define KPLOT3D_COLOR_FROM_FOREGROUND 1
#define KPLOT3D_COLOR_FROM_DATA       2

/* 3D Plot Types */

#define KPLOT3D_UNDEFINED          0
#define KPLOT3D_LINEPLOT           20
#define KPLOT3D_WIREFRAME          21
#define KPLOT3D_MESH               22
#define KPLOT3D_HORIZON            23
#define KPLOT3D_SCATTER            24
#define KPLOT3D_IMPULSE            25
#define KPLOT3D_CONTOUR_2D         26
#define KPLOT3D_CONTOUR_3D         27
#define KPLOT3D_CONSTANT_SHADING   28
#define KPLOT3D_PHONG_SHADING      29
#define KPLOT3D_GHOURAUD_SHADING   30

/* 3D Shading Types */

#define KPLOT3D_SHADE_IMAGERY       1
#define KPLOT3D_SHADE_ELEVATION     2
#define KPLOT3D_SHADE_NORMAL        3

/*
 *  Define the Class name, by which people should be externally referencing
 *  the Plot3D Object by.
 */
#define Plot3DGadgetClass	xvwPlot3DGadgetClass

    /*-------------------------------------*
    |       global variable declarations
    ---------------------------------------*/

/*
 *  XToolkit Widget Declaration for Plot3DGadgetClass
 */
extern   WidgetClass     Plot3DGadgetClass;


    /*-------------------------------------*
    |       routine definitions
    ---------------------------------------*/

/*
 *  Function definitions for Plot 3D Visual Object
 */
xvobject xvw_create_plot3d  PROTO((xvobject, char *));


#endif /* _xvisual_Plot3D_h_ */
/* Don't add after this point */
