 /*
  * Khoros: $Id: ToolboxList.h,v 1.6 1996/04/19 09:34:28 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Definitions for Khoros ToolboxList Object
   >>>>
   >>>>    Written By: Mark Young
   >>>>
   >>>>          Date: Jul 02, 1994
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvlang_ToolboxList_h_
#define _xvlang_ToolboxList_h_

        /*-------------------------------------*
        |       #defines
        ---------------------------------------*/


/*------------------------------------------------------------------------
| Attribute:    XVW_TOOLBOXLIST_SELECT
|
| Description:  If desired,  \fIxvw_add_callback()\fP may be used to install
|               a callback on the toolboxlist object which will be fired each
|               time the user selects a category, subcategory, or icon name 
|               from the toolboxlist.
|
|               When calling \fIxvw_add_callback()\fP, pass this attribute
|               directly, as in
|               .begin code
|               xvw_add_callback(toolboxlist, XVW_TOOLBOXLIST_SELECT,
|                                toolboxlist_cb, client_data);
|               .end code
|               An \fIxvw_program_info\fP structure will be passed into
|               the callback as the \fIcall_data\fP; this structure consists
|               of the information necessary to create or print information
|               about the operator specified by the category, subcategory,
|               and icon name currently selected in the toolboxlist.
|
|               The xvw_program_info() structure is defined as follows:
|               .begin code
|               typedef struct
|               {
|                   int  type;         { type of glyph; one of:
|                                        XVL_OPERATOR    (normal operator),
|                                        XVL_PROCEDURE   (procedure),
|                                        XVL_LOOP        (loop), or
|                                        XVL_CONDITIONAL (conditional) }
|                   char *tbname;      {  toolbox name       }
|                   char *oname;       {  object name        }
|                   char *category;    {  category           }
|                   char *subcategory; {  subcategory        }
|                   char *icon_name;   {  icon name          }
|                   char *panefile;    {  filename of pane file  }
|                   char *short_desc;  {  short description  }
|                   char *workspace;   {  saved workspace file from which
|                                         glyph originated (if applicable)  }
|               } xvw_program_info;
|
|               Inside the callback, the call_data must be cast to the
|               xvw_program_info typedef before use, as follows:
|               .begin code
|               xvw_program_info *program_info = (xvw_program_info *) call_data;
|               .end code
|
| Type:         kfunc_void
| Default:      NULL
| Legal Values: N/A
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_TOOLBOXLIST_SELECT	   "toolboxlistSelect"

/*------------------------------------------------------------------------
| Attribute:    XVW_TOOLBOXLIST_CREATE
|
| Description:  If desired, a callback may be installed on the toolboxlist
|               object that will be fired each time the user indicates the
|		desire to create a glyph by doubleclicking on an icon name
|		in the toolboxlist.  It may also be fired by the application, 
|		using the
|               .symbol index XVW_TOOLBOXLIST_OPEN
|               \fIaction attribute\fP.
|               When calling \fIxvw_add_callback()\fP, pass this attribute
|               directly, as in
|               .begin code
|               xvw_add_callback(toolboxlist, XVW_TOOLBOXLIST_CREATE,
|                                toolboxlist_cb, client_data);
|               .end code
|               An \fIxvw_program_info\fP structure will be passed into
|               the callback as the \fIcall_data\fP; this structure consists
|               of the information necessary to create or print information
|               about the operator that was selected from the toolboxlist.
|               See
|               .symbol index XVW_TOOLBOXLIST_SELECT
|               for an explanation of the xvw_program_info structure.
|
| Type:         kfunc_void
| Default:      NULL
| Legal Values: N/A
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_TOOLBOXLIST_CREATE	   "toolboxlistCreate"

/*------------------------------------------------------------------------
| Attribute:    XVW_TOOLBOXLIST_OPEN
| Description:  This \fIaction attribute\fP causes the
|		.symbol index XVW_TOOLBOXLIST_CREATE
|		to be fired with the operator represented by the currently 
|		selected category, subcategory, and icon name in the toolbox 
|               list. See the explanation of
|               .symbol index XVW_TOOLBOXLIST_CREATE for more details .
|
| Type:         int
| Default:      N/A
| Legal Values: TRUE
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_TOOLBOXLIST_OPEN	   "toolboxlistOpen"

/*
 *  Define the Class name, by which people should be externally referencing
 *  the ToolboxList Object by.
 */
#define ToolboxListWidgetClass	xvwToolboxListWidgetClass

        /*-------------------------------------*
        |       global variable declarations
        ---------------------------------------*/

/*
 *  XToolkit WidgetClass Declaration ToolboxListWidgetClass
 */
extern   WidgetClass     ToolboxListWidgetClass;


        /*-------------------------------------*
        |       routine definitions
        ---------------------------------------*/

/*
 *  Khoros Function Definition for ToolboxList Object
 */
xvobject xvw_create_toolboxlist PROTO((xvobject, char *));


#endif /* _xvlang_ToolboxList_h_ */
/* Do not add after this line */
