/*
  * Khoros: $Id: Athena.c,v 1.9 1996/06/21 06:57:45 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Athena.c,v 1.9 1996/06/21 06:57:45 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		<<<<
   >>>>		Athena xvobject Creation                  <<<<
   >>>>		<<<<
   >>>>  Private:
   >>>>                 xvw_create_text()
   >>>>                 xvw_create_scrollbar()
   >>>>                 xvw_create_list()
   >>>>   Static:
   >>>>                 text_focus()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

#ifdef KATHENA_DEF

klist *xvw_listwidget_list = NULL;


static void text_focus PROTO((xvobject, kaddr, XEvent *));

/*-----------------------------------------------------------
|
|  Routine Name: xvw_create_text
|
|       Purpose: Creates a text object using the Athena Widget set.
|
|         Input: parent - the parent xvobject
|                name   - name with which to reference text object
|        Output: none
|       Returns: none
|    Written By: Danielle Argiro 
|          Date: April 10, 1992
| Modifications:
|
------------------------------------------------------------*/

static xvattribute text_attributes[] = {
{XVW_TEXT_INSERT_POS,   XtNinsertPosition,   XtRInt,              NULL},
{XVW_TEXT_LENGTH,       XtNlength,           XtRInt,              NULL},
{XVW_TEXT_LEFT_MARGIN,  XtNleftMargin,       XtRDimension,        NULL},
{XVW_TEXT_STRING,       XtNstring,           XtRString,           NULL},
};

xvobject xvw_create_text(
   xvobject parent,
   char     *name)
{
	int     i;
	Arg     args[MAX_ARGS];
	Widget  text;

	static int initialized = FALSE;

        i = 0;
      	XtSetArg(args[i], XtNeditType,          XawtextEdit);		i++;
        XtSetArg(args[i], XtNmappedWhenManaged, TRUE);		        i++;
        XtSetArg(args[i], XtNwidth,             10);			i++;
        XtSetArg(args[i], XtNwrap,              XawtextWrapWord);	i++;
        XtSetArg(args[i], XtNpieceSize,         50);			i++;
        XtSetArg(args[i], XtNdisplayCaret,      FALSE);			i++;
	text = XtCreateManagedWidget(name, asciiTextWidgetClass,
		       xvw_widget(parent), args,  (unsigned) i);

	if (!initialized)
	{
	    initialized = TRUE;

	    xvw_init_attributes(XtClass(text), text_attributes,
	                        (int) knumber(text_attributes), NULL, 0, NULL);
	    xvw_define_attributes(XtClass(text),
	     XVW_TEXT_EDIT_TYPE,    XtRInt, xvw_set_misc,    xvw_get_misc,
	     XVW_TEXT_TYPE,         XtRInt, xvw_set_misc,    xvw_get_misc,
	     XVW_TEXT_WRAP,         XtRInt, xvw_set_misc,    xvw_get_misc,
             XVW_TEXT_SCROLL_VERT,  XtRInt, xvw_set_misc,    xvw_get_misc,
             XVW_TEXT_SCROLL_HORIZ, XtRInt, xvw_set_misc,    xvw_get_misc,
             XVW_TEXT_STRING,       XtRString, xvw_set_misc, NULL,
	     XVW_TEXT_MULTILINE,    XtRInt, xvw_set_textmulti,xvw_get_textmulti,
	     NULL);
	}

	xvw_set_attribute(xvw_object(text), XVW_TEXT_MULTILINE, FALSE);
	xvw_add_event(xvw_object(text), EnterWindowMask | LeaveWindowMask, 
		      text_focus, NULL);
	return(xvw_object(text));
}
/*-----------------------------------------------------------
|
|  Routine Name: text_focus
|
|       Purpose: Focuses into the text window when the pointer is in it.
|
|         Input: object      - the text xvobject
|                client_data - client data (unused)
|                event       - event responsible for firing event handler 
|                              (EnterWindowMask | LeaveWindowMask)
|        Output: none
|       Returns: none
|    Written By: Danielle Argiro
|          Date: December 9, 1993
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void text_focus(
   xvobject object,
   kaddr    client_data,
   XEvent   *event)
{
	int     i;
        Arg     args[MAX_ARGS];

	i = 0;
	if (event->type == EnterNotify)
	{
	    XtSetArg(args[i], XtNdisplayCaret, TRUE);	i++;
	}
	else 
	{
	    XtSetArg(args[i], XtNdisplayCaret, FALSE);	i++;
	}
	XtSetValues(xvw_widget(object), args, (unsigned) i); 
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_create_scrollbar
|
|       Purpose: Creates a scrollbar using the Athena Widget set.
|
|         Input: parent - the parent xvobject
|                name   - name with which to reference scrollbar
|        Output: none
|       Returns: none
|    Written By: Danielle Argiro 
|          Date: April 9, 1992
| Modifications:
|
------------------------------------------------------------*/

xvobject xvw_create_scrollbar(
   xvobject parent,
   char     *name)
{
	Widget  scroll;
	Arg     args[MAX_ARGS];
	int     i;

	static int initialized = FALSE;

	i = 0;
        XtSetArg(args[i], XtNmappedWhenManaged, TRUE);	        i++;
        XtSetArg(args[i], XtNorientation, XtorientHorizontal);	i++;
	scroll = XtCreateManagedWidget(name, xvwScrollbarWidgetClass,
			 xvw_widget(parent), args, (unsigned) i);

	if (!initialized)
	{
	   initialized = TRUE;

	   xvw_init_attributes(XtClass(scroll), NULL, 0, NULL, 0, NULL);
	   xvw_define_callback(XVW_SCROLL_CONT_MOTION, XtNjumpProc,
	                       xvw_update_scroll_value);
	   xvw_define_callback(XVW_SCROLL_INCR_MOTION, XtNscrollProc,
	                       xvw_update_scroll_value);
	   xvw_define_attributes(XtClass(scroll),
	     XVW_SCROLL_MIN,   XtRDouble, xvw_set_scrollbar, xvw_get_scrollbar,
	     XVW_SCROLL_MAX,   XtRDouble, xvw_set_scrollbar, xvw_get_scrollbar,
	     XVW_SCROLL_VALUE, XtRDouble, xvw_set_scrollbar, xvw_get_scrollbar,
	     XVW_SCROLL_INCR,  XtRDouble, xvw_set_scrollbar, xvw_get_scrollbar,
	     XVW_ORIENTATION,  XtRInt,    xvw_set_scrollbar, xvw_get_scrollbar,
	     NULL);
	}
	return(xvw_object(scroll));
}


/*-----------------------------------------------------------
|
|  Routine Name: xvw_create_list
|
|       Purpose: Creates a compound list object using the Athena Widget set.
|
|         Input: parent - the parent xvobject
|                name   - name with which to reference list
|        Output: none
|       Returns: none
|    Written By: Danielle Argiro 
|          Date: April 12, 1992
| Modifications:
|
------------------------------------------------------------*/

xvobject xvw_create_list(
   xvobject parent,
   char     *name)
{
	int      i;
	Widget   list;
	Arg      args[MAX_ARGS];
	static   char *default_list[] = { "" };

	static int initialized = FALSE;

	i = 0;
        XtSetArg(args[i], XtNlist, default_list);		i++;
        XtSetArg(args[i], XtNnumberStrings, 1);			i++;
	list = XtCreateManagedWidget(name, xfwfScrolledListWidgetClass,
		       xvw_widget(parent), args, (unsigned) i);


	if (!initialized)
	{
	   initialized = TRUE;

	   xvw_init_attributes(XtClass(list), NULL, 0, NULL, 0, NULL);
	   xvw_define_callback(XVW_LIST_ITEM_SELECT, XtNcallback,
               xvw_update_list_struct);
	   xvw_define_callback(XVW_LIST_ITEM_ACTION, XtNactionCallback,
               xvw_update_list_struct); 
	   xvw_define_attributes(XtClass(list),
	      XVW_LIST_SIZE,          XtRInt, xvw_set_list_num, 
					      xvw_get_list_num,
	      XVW_LIST_HIGHLT_ELEM,   XtRInt, xvw_set_list_highlight,   
				              xvw_get_list_highlight,
	      XVW_LIST_UNHIGHLT_ELEM, XtRInt, xvw_set_list_unhighlight, NULL,
	      NULL);
	}
	return(xvw_object(list));
}

#else
/*
 * These are dummy function calls so that if the widget set is not supported
 * on the machine a message is printed out.
 */

xvobject xvw_create_scrollbar(
   xvobject parent,
   char   *name)
{
   kfprintf(kstderr,"\nThe Athena widget set is either not supported for this \
machine or was not\ncompiled in by the installer of Khoros.  Please contact \
the installer.\n\n");
   return(NULL);
}

xvobject xvw_create_list(
   xvobject parent,
   char   *name)
{
   kfprintf(kstderr,"\nThe Athena widget set is either not supported for this \
machine or was not\ncompiled in by the installer of Khoros.  Please contact \
the installer.\n\n");
   return(NULL);
}

xvobject xvw_create_text(
   xvobject parent,
   char   *name)
{
   kfprintf(kstderr,"\nThe Athena widget set is either not supported for this \
machine or was not\ncompiled in by the installer of Khoros.  Please contact \
the installer.\n\n");
   return(NULL);
}
#endif /* KATHENA_DEF */
/* Don't add after the endif */
