/*
 * Khoros: $Id: ListUtils.c,v 1.5 1996/04/04 23:25:50 fas Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: ListUtils.c,v 1.5 1996/04/04 23:25:50 fas Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		List Utilities for MultiList & ScrolledList Widgets
   >>>>		
   >>>>  Private:
   >>>>                 XfwfDrawString()
   >>>>                 XfwfTextWidth()
   >>>>                 XfwfTablist2Tabs()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#ifdef KATHENA_DEF


/*
 *	Like DrawString, except it takes an additional  "tabs"
 *	argument, used to specify what horizontal pixel position to
 *	move to when tab characters are present in the string.  If
 *	the "tabs" argument is NULL, works exactly like its
 *	counterpart.
 */
void
XfwfDrawString(
     Display *display,
     Drawable drawable,
     GC gc,
     int x,
     int y,
     String string,
     int length,
     int *tabs)
{
	register char	*p, *ep;
	register int	tx, tab;

	tab = tx = 0;
	for (p = string; length; )
	{
		ep = kstrchr(p, '\t'/*, length*/);
		if (ep && tabs)
		{
			XDrawString(display, drawable, gc, x+tx, y,
				     p, (int) (ep - p));
			tx = tabs[tab++];
			length -= ep - p + 1;
			p = ep + 1;
		}
		else
		{
			XDrawString(display, drawable, gc, x+tx, y,
				p, length);
			break;
		}
	}
}

/*
 *	Like TextWidth, except it takes an additional  "tabs"
 *	argument, used to specify what horizontal pixel position to
 *	move to when tab characters are present in the string.  If
 *	the "tabs" argument is NULL, works exactly like its
 *	counterpart.
 */
int
XfwfTextWidth(
     XFontStruct *font,
     String str,
     int length,
     int *tabs)
{
	register char	*p, *ep;
	register int	tx, tab, rc;

	tab = tx = 0;
 	if (length == 0) return 0;
	for (p = str; length; )
	{
		ep = kstrchr(p, '\t'/*, length*/);
		if (ep && tabs)
		{
			tx = tabs[tab++];
			length -= ep - p + 1;
			p = ep + 1;
		}
		else
		{
			rc = XTextWidth(font, p, length);
			if (rc < 0) return rc; else return rc + tx;
		}
	}
 	return(TRUE);
}


/*
 *	Converts a string list of tabs to an array of tabs
 */
int *
XfwfTablist2Tabs(
    char *tablist)
{
	register int *tabs = NULL;
	register int ntabs = 0;
	unsigned int size;

	if (!tablist)
		return NULL;
	for (;;)
	{
		/* Skip leading blanks */
		while (*tablist && *tablist == ' ') ++tablist;
		if (!*tablist) break;

		/* Allocate space for the new tab */
		if (ntabs)
		{
		    size = (unsigned int) ((ntabs+1) * sizeof(int));
		    tabs = (int *) XtRealloc( (char *) tabs, size);
		}
		else
		{
		    size = (unsigned int) ((ntabs + 1) * sizeof(int));
		    tabs = (int *) XtMalloc(size);
		}
		/* Add it to the list */
		tabs[ntabs++] = atoi(tablist);
		/* Skip to the next blank */
		while (*tablist && *tablist != ' ') ++tablist;
	}
	return (tabs);
}

#endif /* KATHENA_DEF */
/* Don't add after the endif */
