 /*
  * Khoros: $Id: Button.c,v 1.14 1996/06/25 16:33:19 mirage Exp $
  */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Button.c,v 1.14 1996/06/25 16:33:19 mirage Exp $";
#endif
 
 
/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>                Button Widget Routines
   >>>>
   >>>>	Private:
   >>>>			ClassInitialize()
   >>>>			Initialize
   >>>>                 Destroy()
   >>>>			SetValues()
   >>>>			Layout()
   >>>>			ButtonSetResource()
   >>>>			ButtonGetResource()
   >>>>			NotifyHandler()
   >>>>	Public:
   >>>>			xvw_create_button()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <kwidgets/ButtonP.h>

static void ClassInitialize PROTO((void));
static void Initialize     PROTO((Widget, Widget, ArgList,Cardinal *));
static void Destroy  	   PROTO((Widget));
static Boolean SetValues   PROTO((Widget, Widget, Widget, ArgList, Cardinal *));

static void Layout	      PROTO((Widget));
static int  ButtonSetResource PROTO((xvobject, char *, kaddr));
static int  ButtonGetResource PROTO((xvobject, char *, kaddr));
static void NotifyHandler     PROTO((Widget, XEvent *, String *, Cardinal));

/*-------------------------------------------------------
|
|   Action Handler
|
--------------------------------------------------------*/

static char defaultTranslations[] =
    "<EnterWindow>:     NotifyHandler()         \n\
     <LeaveWindow>:     NotifyHandler()         \n\
     <BtnUp>:           NotifyHandler()         \n\
     <BtnDown>:         NotifyHandler()		\n\
     <KeyUp>Return:     NotifyHandler()         \n\
     <KeyDown>Return:   NotifyHandler()";

static XtActionsRec actionsList[] =
{
  {"NotifyHandler", (XtActionProc) NotifyHandler},
};

/*-------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{"ButtonLabel",       NULL,  XtRObject,   NULL},
{"ButtonPixmap",      NULL,  XtRObject,   NULL},
{"ButtonAText",      NULL,  XtRObject,   NULL},

{XVW_LABEL_JUSTIFY,   NULL,  XtRInt,      NULL},
{XVW_PIXMAP_JUSTIFY,  NULL,  XtRInt,      NULL},
{XVW_BUTTON_SELECTED, NULL,  XtRInt,      XtRBoolean},
{XVW_BUTTON_ARM,      NULL,  XtRCallback, NULL},
{XVW_BUTTON_SELECT,   NULL,  XtRCallback, NULL},
{XVW_BUTTON_SELECT_TYPE,     NULL, XtRInt, NULL},
{XVW_BUTTON_UNSELECT_TYPE,   NULL, XtRInt, NULL},
{XVW_BUTTON_FORCE_ARMED,     NULL, XtRInt, XtRBoolean},
};

/*-------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwButtonWidgetRec, button.field)

static XtResource resources[] = { 
{"ButtonLabel", NULL, XtRObject, sizeof(XtPointer),
      offset(label), XtRImmediate, (XtPointer) NULL},
{"ButtonPixmap", NULL, XtRObject, sizeof(XtPointer),
      offset(pixmap), XtRImmediate, (XtPointer) NULL},
{"ButtonAText", NULL, XtRObject, sizeof(XtPointer),
      offset(atext), XtRImmediate, (XtPointer) NULL},
{XVW_LABEL_JUSTIFY, NULL, XtRInt, sizeof(int),
      offset(label_justify), XtRImmediate, (XtPointer) KLABEL_JUSTIFY_CENTER},
{XVW_PIXMAP_JUSTIFY, NULL, XtRInt, sizeof(int),
      offset(pixmap_justify), XtRImmediate, (XtPointer) KPIXMAP_JUSTIFY_LEFT},
{XVW_BUTTON_SELECTED, NULL, XtRBoolean, sizeof(Boolean),
      offset(selected), XtRImmediate, (XtPointer) FALSE},
{XVW_BUTTON_SELECT, NULL, XtRCallback, sizeof(XtPointer),
      offset(select_callback), XtRCallback, (XtPointer) NULL},
{XVW_BUTTON_ARM, NULL, XtRCallback, sizeof(XtPointer),
      offset(arm_callback), XtRCallback, (XtPointer) NULL},
{XVW_BUTTON_SELECT_TYPE, NULL, XtRInt, sizeof(int),
      offset(select_type), XtRImmediate, (XtPointer) KSHADOW_IN},
{XVW_BUTTON_UNSELECT_TYPE, NULL, XtRInt, sizeof(int),
      offset(unselect_type), XtRImmediate, (XtPointer) KSHADOW_OUT},
{XVW_BUTTON_FORCE_ARMED, NULL, XtRBoolean, sizeof(Boolean),
      offset(force_armed), XtRImmediate, (XtPointer) FALSE},

{XVW_BORDER_WIDTH, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwButtonWidgetRec, core.border_width),
        XtRImmediate, (XtPointer) 1},
{XVW_BUFFER_DIST, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwButtonWidgetRec, manager.buffer_dist),
	XtRImmediate, (XtPointer) 0},
};
#undef offset


#define SUPERCLASS (&xvwManagerWidgetClassRec)

XvwButtonWidgetClassRec xvwButtonWidgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Button",				/* class_name		  */
    sizeof(XvwButtonWidgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    actionsList,			/* actions		  */
    XtNumber(actionsList),		/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    defaultTranslations,		/* tm_table		  */
    NULL,				/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,				    /* subresources	  */
    0,					    /* subresources_count */
    sizeof(XvwButtonWidgetConstraintsRec),  /* constraint_size    */
    NULL,				    /* initialize         */
    NULL,                                   /* destroy            */
    NULL,				    /* set_values	  */
    NULL,                                   /* extension          */
  },  /* ConstraintClass fields initialization */
  {
    Layout,				    /* pre-layout routine     */
    XtInheritLayout,			    /* post-layout routine    */
    XtInheritChangeSel,			    /* change selected proc   */
    XtInheritEraseSel,			    /* erase selected proc    */
    XtInheritRefreshSel,		    /* refresh selection proc */
    XtInheritResize,			    /* resize		      */
    XtInheritGeometryManager,		    /* geometry_manager	      */
  }, /* XvwManagerWidgetClass fields initialization */
  {
    NULL,                                     /* field not used    */
  },  /* XvwButtonWidgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwButtonWidgetClass for public consumption
 */
WidgetClass xvwButtonWidgetClass = (WidgetClass) &xvwButtonWidgetClassRec;

/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for Class and Constraint Declarations
|
--------------------------------------------------------------------*/
 
#define kwidget(widget)      (XvwButtonWidget) (widget)
#define kwidgetclass(widget) (XvwButtonWidgetClass) (widget)->core.widget_class
#define kconstraint(widget)  (XvwButtonWidgetConstraints) (widget)->core.constraints

/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of a xvwButtonWidgetClass has been created. 
|                It will initialize all the class attributes. 
|
|    Written By: Mark Young
|          Date: Mar 10, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
        xvw_init_attributes(xvwButtonWidgetClass, attributes,
		XtNumber(attributes), NULL, 0,
		"$DESIGN/objects/library/kwidgets/uis/Button.pane");
	xvw_load_resources("$DESIGN/objects/library/kwidgets/app-defaults/Button");

	xvw_define_attributes(ButtonWidgetClass,
	   XVW_LABEL,   XtRString, ButtonSetResource, ButtonGetResource,
	   XVW_ATEXT,   XtRString, ButtonSetResource, ButtonGetResource,
	   XVW_PIXMAP,  XtRPixmap, ButtonSetResource, ButtonGetResource,
	   XVW_PIXMAP_FILENAME, XtRString, ButtonSetResource, ButtonGetResource,
	   NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial values
|                for the fields of a label object instance.       
|                The width, height, ascent, and descent are       
|                all initialized to sane values.
|
|         Input: request - not used     
|                new     - object instance after initialization, with
|    Written By: Mark Young
|          Date: Mar 10, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
        XvwButtonWidget xwid = (XvwButtonWidget) new;
	xvobject object = xvw_object(new);

        /*
         *  Initialize our state and also add event handlers to do the
         *  button thang..
         */
        xwid->button.tile     = None;
        xwid->button.atext    = NULL;
        xwid->button.armed    = FALSE;
        xwid->button.selected = FALSE;
        xwid->button.handler_active = FALSE;
	xvw_set_attribute(object, XVW_SHADOW_TYPE, xwid->button.unselect_type);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: This method will free the tile pixmap.
|
|         Input: widget - the widget being destroyed
|    Written By: Mark Young
|          Date: Apr 17, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
        XvwButtonWidget xwid = (XvwButtonWidget) widget;

	if (xwid->button.tile != None)
	   XmuReleaseStippledPixmap(XtScreen(widget), xwid->button.tile);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: Determine what the widget should do by
|		 comparing the current public settable values 
|		 versus the new settings.
|
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new     - the widget processed through all set values methods
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Mar 10, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwButtonWidget cxwid = (XvwButtonWidget) current;
	XvwButtonWidget nxwid = (XvwButtonWidget) new;

	if (cxwid->button.selected      != nxwid->button.selected    ||
	    cxwid->button.select_type   != nxwid->button.select_type ||
	    cxwid->button.unselect_type != nxwid->button.unselect_type)
	{
	   int      type;

	   if (nxwid->button.selected == TRUE)
	      type = nxwid->button.select_type;
	   else
	      type = nxwid->button.unselect_type;

	   xvw_set_attribute(xvw_object(new), XVW_SHADOW_TYPE, type);
	}

	if (cxwid->button.selected != nxwid->button.selected)
	{
	   if (nxwid->button.selected && nxwid->button.tile == None)
	   {
	      nxwid->button.tile = XmuCreateStippledPixmap(XtScreen(new),
                   WhitePixelOfScreen(XtScreen(new)),
		   nxwid->core.background_pixel,
                   new->core.depth);
	   }
	   xvw_set_attribute(xvw_object(new), XVW_BACKGROUND_PIXMAP,
	     nxwid->button.selected ? nxwid->button.tile : XtUnspecifiedPixmap);
	}

	if (cxwid->button.label_justify  != nxwid->button.label_justify ||
	    cxwid->button.pixmap_justify != nxwid->button.pixmap_justify)
	{
	   ManagerRequestLayout(new, FALSE);
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: Layout
|
|       Purpose: This routine will layout the children of the
|		 buttonWidget.  
|
|         Input: widget - the button widget 
|    Written By: Mark Young
|          Date: Jun 1, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Layout(
   Widget widget)
{
	XvwButtonWidget xwid = (XvwButtonWidget) widget;

	Widget label, pixmap, atext;
	int    ypos, width, height, dist;
	int    label_xpos, pixmap_xpos, atext_xpos,
	       label_w, pixmap_w, atext_w,
	       label_h, pixmap_h, atext_h;

	/* Don't Layout if there are no children */
	if (xwid->composite.num_children == 0)
	   return;

	dist = xwid->manager.buffer_dist;
	if (xwid->manager.shadow_type != KSHADOW_NONE)
	   dist += xwid->manager.shadow_thickness;

	/* Get the relevant and useful information from the widget */ 
	label  = xwid->button.label  ? xvw_widget(xwid->button.label)  : NULL;
	pixmap = xwid->button.pixmap ? xvw_widget(xwid->button.pixmap) : NULL;
	atext  = xwid->button.atext  ? xvw_widget(xwid->button.atext)  : NULL;

	width	 = xwid->core.width;
	label_w  = label  ? label->core.width  : 0;
	pixmap_w = pixmap ? pixmap->core.width : 0;
	atext_w  = atext  ? atext->core.width  : 0;
	label_xpos = pixmap_xpos = atext_xpos = dist;

	/*
	 *  Pixmap and Label are left justified by default.  However, if
	 *  both the pixmap and label are left justified then we want the
	 *  label to appear to the right of pixmap.
	 */ 
	if (label && xwid->button.label_justify == KLABEL_JUSTIFY_LEFT &&
	    pixmap && xwid->button.pixmap_justify == KPIXMAP_JUSTIFY_LEFT)
	{
	   label_xpos = kmax(dist, pixmap_xpos+pixmap_w+dist);
	}

	/*
	 *  Center justify the Pixmap and Label by subtracting our half the
	 *  button width by half the Label's or Pixmap's width.
	 */
	if (label && xwid->button.label_justify == KLABEL_JUSTIFY_CENTER)
	   label_xpos = kmax(dist, width/2 - label_w/2);
	if (pixmap && xwid->button.pixmap_justify == KPIXMAP_JUSTIFY_CENTER)
	   pixmap_xpos = kmax(dist, width/2 - pixmap_w/2);

	if (label && xwid->button.label_justify == KLABEL_JUSTIFY_RIGHT)
	{
	   label_xpos = kmax3(dist, pixmap_xpos+pixmap_w, width-label_w-dist);
	   if (pixmap && xwid->button.pixmap_justify == KPIXMAP_JUSTIFY_RIGHT)
	      pixmap_xpos = kmax(dist, width-pixmap_w-dist);
	}
	else if (pixmap && xwid->button.pixmap_justify == KPIXMAP_JUSTIFY_RIGHT)
	   pixmap_xpos = kmax3(dist, label_xpos+label_w, width-pixmap_w-dist);

	if (atext)
	{
	   atext_xpos = kmax4(dist, label_xpos+label_w+dist,
			pixmap_xpos+pixmap_w+dist, width-atext_w-dist);
	}
	height = kmax(xwid->manager.min_height, xwid->manager.preferred_height);
	label_h  = (label  ? label->core.height  : 0);
	pixmap_h = (pixmap ? pixmap->core.height : 0);
	atext_h  = (atext  ? atext->core.height  : 0);
	height = kmax4(height, label_h+dist, pixmap_h+dist, atext_h+dist);
	if (label != NULL)
	{
	   ypos = kmax(dist, height/2 - label_h/2);
	   XtMoveWidget(label, (Position) label_xpos, ypos);
	}

	if (pixmap != NULL)
	{
	   ypos = kmax(dist, height/2 - pixmap_h/2);
	   XtMoveWidget(pixmap, (Position) pixmap_xpos, ypos);
	}

	if (atext != NULL)
	{
	   ypos = kmax(dist, height/2 - atext_h/2);
	   XtMoveWidget(atext, (Position) atext_xpos, ypos);
	}
}

/*-----------------------------------------------------------
|  
|  Routine Name: ButtonSetResource - button set label/pixmap resource
|  
|       Purpose: Resource attribute for setting the label or pixmap resource.
|                We don't want to create the label or pixmap until we have
|		 to.  So until the label or pixmap is set, we won't create
|		 the subpart.  Also, once created we will manage and unmanage
|		 the subpart as the resource is set and unset.
|  
|         Input: button - the button object in which we are to select the item
|                attribute - the XVW_LABEL, XVW_PIXMAP, XVW_PIXMAP_FILENAME
|                calldata - the item to be selected
|    Written By: Mark Young
|          Date: Mar 21, 1996
|  
------------------------------------------------------------*/
/* ARGSUSED */          
static int ButtonSetResource(
   xvobject button,     
   char     *attribute, 
   kaddr    calldata)   
{
	XvwButtonWidget xwid = (XvwButtonWidget) xvw_widget(button);

	if (kstrcmp(attribute, XVW_LABEL) == 0)
	{
	   String label = *(String *) calldata;

	   if (label && xwid->button.label == NULL)
	      xwid->button.label = xvw_create_label(button, "ButtonLabel");

	   xvw_set_attribute(xwid->button.label, XVW_LABEL, label);
	   if (label != NULL && !xvw_check_managed(xwid->button.label))
	      xvw_manage(xwid->button.label);
	   else if (label == NULL && xvw_check_managed(xwid->button.label))
	      xvw_unmanage(xwid->button.label);
	}
	else if (kstrcmp(attribute, XVW_ATEXT) == 0)
	{
	   String atext = *(String *) calldata;

	   if (atext && xwid->button.atext == NULL)
	   {
	      xwid->button.atext = xvw_create_label(button, "ButtonAText");
	      /*xvw_set_attribute(xwid->button.atext, XVW_LEFT_OF, NULL);*/
	   }
	   xvw_set_attribute(xwid->button.atext, XVW_LABEL, atext);
	   if (atext != NULL && !xvw_check_managed(xwid->button.atext))
	      xvw_manage(xwid->button.atext);
	   else if (atext == NULL && xvw_check_managed(xwid->button.atext))
	      xvw_unmanage(xwid->button.atext);
	}
	else if (kstrcmp(attribute, XVW_PIXMAP) == 0 ||
		 kstrcmp(attribute, XVW_PIXMAP_FILENAME) == 0)
	{
	   Pixmap pixmap   = *(Pixmap *) calldata;
	   String filename = *(String *) calldata;

	   if (pixmap && xwid->button.pixmap == NULL)
	      xwid->button.pixmap = xvw_create_pixmap(button, "ButtonPixmap");

	   if (kstrcmp(attribute, XVW_PIXMAP_FILENAME) == 0)
	      xvw_set_attribute(xwid->button.pixmap, XVW_PIXMAP_FILENAME,
				filename);
	   else
	      xvw_set_attribute(xwid->button.pixmap, XVW_PIXMAP, pixmap);

	   if ((pixmap != 0) && (!xvw_check_managed(xwid->button.pixmap)))
	      xvw_manage(xwid->button.pixmap);
	   else if ((pixmap == 0) && (xvw_check_managed(xwid->button.pixmap)))
	      xvw_unmanage(xwid->button.pixmap);
	}
	return(TRUE);
}

/*-----------------------------------------------------------
|  
|  Routine Name: ButtonGetResource - button get label/pixmap resource
|  
|       Purpose: Resource attribute for getting the label or pixmap resource.
|                We don't want to create the label or pixmap until we have
|		 to.  So until the label or pixmap is set, we won't create
|		 the subpart.  Also, once created we will manage and unmanage
|		 the subpart as the resource is set and unset.
|  
|         Input: button - the button object in which we are to select the item
|                attribute - the XVW_LABEL, XVW_PIXMAP
|                calldata - the item to be selected
|    Written By: Mark Young
|          Date: Mar 21, 1996
|  
------------------------------------------------------------*/
/* ARGSUSED */          
static int ButtonGetResource(
   xvobject button,     
   char     *attribute, 
   kaddr    calldata)   
{
	XvwButtonWidget xwid = (XvwButtonWidget) xvw_widget(button);

	if (kstrcmp(attribute, XVW_LABEL) == 0)
	{
	   String *string = (String *) calldata;

	   if (xwid->button.label != NULL)
	      xvw_get_attribute(xwid->button.label, XVW_LABEL, calldata);
	   else
	      *string = NULL;
	}
	else if (kstrcmp(attribute, XVW_ATEXT) == 0)
	{
	   String *string = (String *) calldata;

	   if (xwid->button.atext != NULL)
	      xvw_get_attribute(xwid->button.atext, XVW_LABEL, calldata);
	   else
	      *string = NULL;
	}
	else if (kstrcmp(attribute, XVW_PIXMAP) == 0)
	{
	   Pixmap *pixmap = (Pixmap *) calldata;

	   if (xwid->button.pixmap != NULL)
	      xvw_get_attribute(xwid->button.pixmap, XVW_PIXMAP, calldata);
	   else *pixmap = 0;
	}
	else if (kstrcmp(attribute, XVW_PIXMAP_FILENAME) == 0)
	{
	   String *filename = (String *) calldata;

	   if (xwid->button.pixmap != NULL)
	      xvw_get_attribute(xwid->button.pixmap, XVW_PIXMAP_FILENAME,
				calldata);
	   else
	      *filename = NULL;
	}
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: NotifyHandler
|
|       Purpose: Notify the user of the joyous selection that have
|                so dedicately selected, or are about to select.
|
|         Input: widget - the wonderous selection in question
|                event  - the event that triggered this mess.
|    Written By: Mark Young
|          Date: Mar 10, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void NotifyHandler(
   Widget   widget,
   XEvent   *event,
   String   *params,
   Cardinal num_params)
{
        XvwButtonWidget xwid = kwidget(widget);

	xvobject button;
	int armed, selected = xwid->button.selected;

	if (event->type == ButtonPress || event->type == KeyPress)
	   xwid->button.handler_active = TRUE;
	else if (!xwid->button.handler_active && !xwid->button.force_armed)
	   return;

	button = xvw_object(widget);
        if (event->type == ButtonRelease || event->type == KeyRelease)
	{
	   xwid->button.handler_active = FALSE;
	   if (xwid->button.armed == TRUE)
	   {
	      xwid->button.selected = !selected;
              XtCallCallbacks((Widget) xwid, XVW_BUTTON_SELECT, NULL);
	      if (xwid->button.selected != selected)
	      {
		 xwid->button.selected = selected;
	         xvw_set_attribute(button, XVW_BUTTON_SELECTED, !selected);
	      }
	   }
	}
	else xvw_set_attribute(button, XVW_BUTTON_SELECTED, !selected);

	if (event->type == ButtonPress || event->type == KeyPress || event->type == EnterNotify)
	   armed = xwid->button.armed = TRUE;
	else if (event->type == ButtonRelease || event->type == KeyRelease || event->type == LeaveNotify)
	   armed = xwid->button.armed = FALSE;

	XtCallCallbacks(widget, XVW_BUTTON_ARM, &armed);
}

/************************************************************
*
*  Routine Name: xvw_create_button - create button GUI object
*
*       Purpose: A button object is a rectangular or oval mechanism 
*		 displaying either a text label or a pixmap. If a callback 
*                is installed on the button, software control will be passed 
*                to the callback routine when the user clicks the mouse on the
*                button.
*
*         Input: parent - parent of the button object
*                name   - a name for this particular instance of the button 
*                         object (for use in app-defaults files, etc)
*
*        Output:
*       Returns: The GUI button object on success, NULL on failure.
*  Restrictions: 
*    Written By: Mark Young
*  	   Date: Mar 10, 1996
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_button(
   xvobject parent,
   char     *name)
{
	xvobject  object;


	/*
	 *  Call the xvw_create() routine to do the actual creation.
	 */
	object = xvw_create(parent, FALSE, TRUE, name, ButtonWidgetClass);
	return(object);
}

/*------------------------------------------------------------
|
|  Routine Name: xvw_reverse_colors - reverse the foreground & background
|                                     colors of an object
|       
|       Purpose: Reverses the foreground and background colors
|                on a button object.  This routine is obsolete,
|                it is only provided for backwards compatibility.
|    
|         Input: object - the button object on which to reverse colors
|                value  - TRUE if button should be changed to look selected,
|                         FALSE if button should be changed to look un-selected.
|  Restrictions: The button object must have been previously
|                created with a call to xvf_create_oval_button or
|                xvf_create_rect_button.
|    Written By: Danielle Argiro
|          Date: Jul 17, 1992
| 
------------------------------------------------------------------*/

void xvw_reverse_colors(
   xvobject object,
   int value)
{
	xvw_set_attribute(object, XVW_BUTTON_SELECTED, value);
}
