/*
  * Khoros: $Id: Label.c,v 1.8 1996/07/05 01:44:41 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Label.c,v 1.8 1996/07/05 01:44:41 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Label Gadget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Redisplay()
   >>>>                 Destroy()
   >>>>                 SetValues()
   >>>>                 RecomputeSize()
   >>>>                 CompileString()
   >>>>   Public:
   >>>>                 xvw_create_label()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
#include <kwidgets/LabelP.h>

static void ClassInitialize	 PROTO((void));
static void Initialize		 PROTO((Widget, Widget, ArgList, Cardinal *));
static void Redisplay		 PROTO((Widget, XEvent *, Region));
static void Destroy		 PROTO((Widget));
static Boolean SetValues	 PROTO((Widget, Widget, Widget, ArgList,
					Cardinal *));
static void CompileString	 PROTO((Widget));
static void RecomputeSize	 PROTO((Widget));


/*--------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_LABEL,		   NULL,                     XtRString,    NULL}, 
{XVW_LABEL_EMPHASIZE,	   NULL,                     XtRInt,       XtRBoolean}, 
{XVW_LABEL_EMPHASIZEPIXEL, NULL,                     XtRPixel,     NULL}, 
{XVW_LABEL_EMPHASIZECOLOR, XVW_LABEL_EMPHASIZEPIXEL, XtRString,    XtRPixel}, 
{XVW_LABEL_JUSTIFY,	   NULL,                     XtRInt,       NULL}, 
};

/*--------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwLabelGadgetRec, label.field)

static XtResource resources[] = { 
{XVW_LABEL, NULL, XtRString, sizeof(String),
      offset(string), XtRImmediate, (XtPointer) NULL},
{XVW_LABEL_EMPHASIZE, NULL, XtRBoolean, sizeof(Boolean),
      offset(emphasize), XtRImmediate, (XtPointer) FALSE},
{XVW_LABEL_EMPHASIZEPIXEL, NULL, XtRPixel, sizeof(Pixel),
      offset(pixel), XtRString, (XtPointer) "#000000"},
{XVW_LABEL_JUSTIFY, NULL, XtRInt, sizeof(int),
      offset(justify), XtRImmediate, (XtPointer) KLABEL_JUSTIFY_LEFT},
};
#undef offset


#define SUPERCLASS (&xvwManagerGadgetClassRec)

XvwLabelGadgetClassRec xvwLabelGadgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Label",				/* class_name		  */
    sizeof(XvwLabelGadgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    NULL,				/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    FALSE,				/* compress_motion	  */
    FALSE,				/* compress_exposure	  */
    FALSE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    XtInheritResize,			/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    NULL,				/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* RectObjClass fields initialization */
  {
    XtInheritPicking,			/* pick object proc       */
    XtInheritEraseSel,			/* erase selected proc    */
    XtInheritRefreshSel,		/* refresh selection proc */
    NULL,				/* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
  {
    NULL,				/* extension - not used   */
  },  /* XvwLabelGadgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwLabelGadgetClass for public consumption
 */
WidgetClass xvwLabelGadgetClass = (WidgetClass) &xvwLabelGadgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of label object class has been created.
|                It will initialize all the class attributes. 
|
|    Written By: Mark Young
|          Date: Apr 10, 1994
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
        xvw_init_attributes(xvwLabelGadgetClass, attributes,
                XtNumber(attributes), NULL, 0,
                "$DESIGN/objects/library/kwidgets/uis/Label.pane");
	xvw_load_resources("$DESIGN/objects/library/xvobject/app-defaults/Label");
}


/*-----------------------------------------------------------
|
|  Routine Name: Initialize 
|
|       Purpose: This method will set up the initial values 
|                for the fields of a label object instance.
|                The width, height, ascent, and descent are
|		 all initialized to sane values. 
|
|         Input: request - not used 
|                new     - object instance after initialization, with
|    Written By: Mark Young
|          Date: Apr 10, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwLabelGadget nobj = (XvwLabelGadget) new;

	/*
	 *  Compute the position of the object in device coordinates
	 */
	nobj->label.lines          = NULL;
	nobj->label.numlines       = 0;
	nobj->label.string_width   = 0;
	nobj->label.string_height  = 0;
	nobj->label.string_ascent  = 0;
	nobj->label.string_descent = 0;
	nobj->label.pixel          = nobj->manager.background;
}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This routine will redraw the label object in response
|                to an expose event. 
|
|         Input: widget - the label object that was exposed 
|                event  - the event that caused the redraw
|                region - the region that was exposed 
|    Written By: Mark Young
|          Date: Apr 10, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	XvwLabelGadget xobj = (XvwLabelGadget) widget;

	unsigned int w, h, string_width;
	int  i, x, y, xpos, length, numlines, thickness;

	String  *lines;
	Window  window   = XtWindow(XtParent(widget));
	Display *display = XtDisplay(widget);


	/*
	 *  Call the ManagerObj to do the widget thang (background,
	 *  border, etc)
	 */
	(*xvwLabelGadgetClass->core_class.superclass->core_class.expose)
                (widget, event, region);

	/*
	 *  Check to see if this if there is a label to draw
	 */
	if (kstrlen(xobj->label.string) == 0)
	   return;

	/*
	 *  Compute position using the label's current position
	 */
	thickness = xobj->manager.shadow_thickness + 1;
	x = xobj->rectobj.x + thickness;
	y = xobj->rectobj.y + thickness;
	w = xobj->rectobj.width  - 2*thickness;
	h = xobj->rectobj.height - 2*thickness;

	lines = xobj->label.lines;
	numlines = xobj->label.numlines;

	/*
	 *  Strings should always be drawn in the middle of gadget
	 */
	if (xobj->label.string_height < h)
	   y += (h - xobj->label.string_height)/2;

	y += xobj->label.string_ascent;
	for (i = 0; i < numlines; i++)
	{
	   length = kstrlen(lines[i]);

	   /*
	    *  If the bounding box is greater than our rectangular extent
	    *  then see about changing our x position in order to draw the
	    *  label according to correct justification.
	    */
	   xpos = x;
	   string_width = XTextWidth(xobj->manager.font, lines[i], length);
	   if (string_width < w)
	   {
	      if (xobj->label.justify == KLABEL_JUSTIFY_CENTER)
	         xpos = x + (w - string_width)/2;
	      else if (xobj->label.justify == KLABEL_JUSTIFY_RIGHT)
	         xpos = x + w - string_width;
	   }
	   if (xobj->label.emphasize)
	   {
	      Pixel save = xobj->manager.foreground;
	      xvw_set_attribute(xvw_object(widget), XVW_FOREGROUND, 
			        xobj->label.pixel);
	      XDrawString(display, window, xobj->manager.foreground_gc, 
                          xpos+1, y+1, lines[i], length);
	      xvw_set_attribute(xvw_object(widget), XVW_FOREGROUND, save);
	   }
	   XDrawString(display, window, xobj->manager.foreground_gc, 
			xpos, y, lines[i], length);
	   y += xobj->label.string_ascent+xobj->label.string_descent;
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: This method will free the label string.
|
|         Input: widget - the widget being destroyed
|    Written By: Mark Young & John Salas
|          Date: Apr 17, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
	XvwLabelGadget xobj = (XvwLabelGadget) widget;

	karray_free(xobj->label.lines, KSTRING, 
		    (size_t) xobj->label.numlines, NULL);
	kfree_and_NULL(xobj->label.string);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: If the the justify or postion of the 
|		 label as changed, recalculate the new position. 
|
|         Input: current - the object containing current settings 
|                request - the object containing requested settings
|                new     - the object processed through all set values methods 
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Apr 10, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwLabelGadget cobj = (XvwLabelGadget) current;
	XvwLabelGadget nobj = (XvwLabelGadget) new;

	Boolean redisplay = FALSE;


	if (cobj->label.justify   != nobj->label.justify   ||
	    cobj->label.emphasize != nobj->label.emphasize ||
	    cobj->manager.font	  != nobj->manager.font)
	{
	   RecomputeSize(new);
	   redisplay = TRUE;
	}

	if (cobj->label.string != nobj->label.string)
	{
	   /*
	    *  Free the previously allocated private string
	    */
	   kfree_and_NULL(cobj->label.string);
	   nobj->label.string = kstring_copy(nobj->label.string, NULL);
	   CompileString(new);
	   RecomputeSize(new);
	   redisplay = TRUE;
	}

	if (redisplay && nobj->manager.force_redisplay)
	   ManagerObjForceRedisplay(current, new, TRUE, &redisplay);

	return(redisplay);
}

/*-----------------------------------------------------------
|
|  Routine Name: RecomputeSize
|
|       Purpose: Given the current position recompute the size.
|
|         Input: widget - the label object 
|    Written By: Mark Young
|          Date: Apr 10, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RecomputeSize(
   Widget widget)
{
	XvwLabelGadget xobj = (XvwLabelGadget) widget;

	String *lines;
	int i, numlines, len, ascent, descent, width, buffer;
        XtWidgetGeometry temp;

	/*
	 *  Get the extent of the string
	 */
	len = kstrlen(xobj->label.string);
	if (len > 0 && xobj->manager.font != NULL)
	{
	   lines = xobj->label.lines;
	   numlines = xobj->label.numlines;
	   ascent  = xobj->manager.font->max_bounds.ascent;
	   descent = xobj->manager.font->max_bounds.descent;
	   xobj->label.string_ascent  = ascent;
	   xobj->label.string_descent = descent;
	   xobj->label.string_height  = (ascent + descent) * numlines;

	   for (i = 0, width = 0; i < numlines; i++)
	   {
	      len = XTextWidth(xobj->manager.font, lines[i], 
				(int) kstrlen(lines[i]));
	      if (len > width) width = len;
	   }
	   xobj->label.string_width = width;
	}
	else
	{
	   xobj->label.string_width   =
	   xobj->label.string_height  =
	   xobj->label.string_ascent  =
	   xobj->label.string_descent = 0;
	}

	/*
	 *  Request the new size....
	 */
	buffer = xobj->manager.shadow_thickness + 1;

        temp.request_mode = CWWidth | CWHeight;
        temp.width  = (Dimension) xobj->label.string_width  + 2*buffer;
        temp.height = (Dimension) xobj->label.string_height + 2*buffer;
        (void) XtMakeGeometryRequest(widget, &temp, &temp);
}

/*-----------------------------------------------------------
|
|  Routine Name: CompileString - this routine compiles the string
|
|       Purpose: This routine actually compiles the string into a
|		 multiline detab'ed string.
|
|         Input: widget - the label object
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Apr 10, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void CompileString(
   Widget widget)
{
	XvwLabelGadget xobj = (XvwLabelGadget) widget;

	int    i, len = 0, numlines = 0;
	size_t size; 
	char   temp[KLENGTH], *string, *tmp, **lines = NULL;

	string = xobj->label.string;
	do
	{
	    tmp = kstrpbrk(string, "\t\r\n");
	    if (!tmp)
	       size = (int) kstrlen(string);
	    else
	       size = tmp-string;

	    kstring_ncopy(string, size, &temp[len]); len += size;
	    if (tmp && *tmp == '\t')
	    {
	       size = 8 - (len % 8);
	       for (i = 0; i < size; i++)
		   temp[len++] = ' ';
	       temp[len] = '\0'; tmp++;
	    }
	    else if (!tmp || *tmp == '\n' || *tmp == '\r')
	    {
	       len = 0;
	       lines = karray_add(lines, KSTRING, kstrdup(temp), 
			          (size_t) numlines);
	       numlines++;
	       if (tmp) tmp++;
	    }
	    string = tmp;
	} while (string != NULL);

	/*
	 * Save the text and number of lines...
	 */
	karray_free(xobj->label.lines, KSTRING, 
		    (size_t) xobj->label.numlines, NULL);
	xobj->label.lines    = lines;
	xobj->label.numlines = numlines;
}

/************************************************************
*
*  Routine Name: xvw_create_label - create a label object
*
*       Purpose: A label object is for the display of non-editable text
*                on Khoros graphical user interfaces.  If desired, a label 
*                object may also display a pixmap, in addition to the text.
*                The label object supports multiline text;
*                simply include "\n" in the text to indicate the
*                end of a line.  Label text and pixmaps may be left,
*                center, or right justified.  
*
*		 There are three major differences between a \fIlabel\fP 
*                object and a \fItext\fP object.  Firstly, a label object is 
*                a gadget; ie, it has no window of its own, but uses the 
*                window of its parent.  A text object, on the other hand, is a 
*                widget; it has a dedicated window of its own.  Secondly, the
*                label object supports the display of non-editable text only; 
*                the text displayed by a text object can be edited by the user 
*                unless it is explicitly specified as non-editable, and even
*                then, a cursor will appear.  Thirdly, the label object is
*                implemented as part of Khoros GUI services; the text object,
*                on the other hand, will be provided by the Athena or Motif 
*                widget set, depending on which widget set is specified.
*		 For most non-editable text display uses, a label object is 
*                preferable, and has the advantage that it is much faster to 
*                refresh than a text object.   
*
*	         The difference between a \fIlabel\fP object and a \fIstring\fP
*                object is that the label object is meant for display of text
*                on graphical user interfaces, while the string object is an
*                annotation, meant for display of text on images, area objects,
*                and so on.  
*
*         Input: parent - parent of the label object; NULL will 
*			  cause a default toplevel to be created automatically
*                name   - a name for this particular instance of the object
*                         (for use in app-defaults files, etc)
*        Output:
*	Returns: The label object on success, NULL on failure
*
*  Restrictions: 
*    Written By: Mark Young
*  	   Date: Apr 30, 1996
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_label(
   xvobject parent,
   char     *name)
{
	xvobject  object;


	/*
	 *  Call the xvw_create() routine to do the actual creation.
	 */
	object = xvw_create(parent, FALSE, TRUE, name, LabelGadgetClass);
	return(object);
}
