 /*
  * Khoros: $Id: MenuButton.c,v 1.17 1996/08/10 03:24:37 mirage Exp $
  */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: MenuButton.c,v 1.17 1996/08/10 03:24:37 mirage Exp $";
#endif
 
 
/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>                MenuButton Widget Routines
   >>>>
   >>>>	Private:
   >>>>			ClassInitialize()
   >>>>			Initialize
   >>>>			NotifyHandler()
   >>>>	Public:
   >>>>			xvw_create_menubutton()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <kwidgets/MenuButtonP.h>
#include <kwidgets/MenuP.h>

static void ClassInitialize PROTO((void));
static void Initialize      PROTO((Widget, Widget, ArgList,Cardinal *));
static void Destroy         PROTO((Widget));

static void PopupMenu     PROTO((xvobject));
static void PopdownMenu   PROTO((xvobject));
static void NotifyHandler PROTO((Widget, XEvent *, String *, Cardinal));
static void WindowHandler PROTO((Widget, XEvent *, String *, Cardinal));


/*-------------------------------------------------------
|
|   Action Handler
|
--------------------------------------------------------*/

static char defaultTranslations[] =
    "<EnterWindow>:     WindowHandler()         \n\
     <LeaveWindow>:     WindowHandler()         \n\
     <BtnUp>:           NotifyHandler()         \n\
     <BtnDown>:         NotifyHandler()		\n\
     <KeyUp>Return:     NotifyHandler()         \n\
     <KeyDown>Return:   NotifyHandler()";

static XtActionsRec actionsList[] =
{
  {"NotifyHandler", (XtActionProc) NotifyHandler},
  {"WindowHandler", (XtActionProc) WindowHandler}
};

/*-------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_MENUBUTTON_MENU,  NULL,  XtRObject, NULL},
};

/*-------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwMenuButtonWidgetRec, menubutton.field)

static XtResource resources[] = { 
{XVW_MENUBUTTON_MENU, NULL, XtRObject, sizeof(xvobject),
      offset(menu), XtRImmediate, (XtPointer) NULL},

{XVW_BUTTON_SELECT_TYPE, NULL, XtRInt, sizeof(int),
        XtOffsetOf(XvwButtonWidgetRec, button.select_type),
        XtRImmediate, (XtPointer) KSHADOW_OUT},
{XVW_BUTTON_UNSELECT_TYPE, NULL, XtRInt, sizeof(int),
        XtOffsetOf(XvwButtonWidgetRec, button.unselect_type),
        XtRImmediate, (XtPointer) KSHADOW_OUT},
{XVW_LABEL_JUSTIFY, NULL, XtRInt, sizeof(int),
	XtOffsetOf(XvwMenuButtonWidgetRec, button.label_justify),
	XtRImmediate, (XtPointer) KLABEL_JUSTIFY_LEFT},
{XVW_PIXMAP_JUSTIFY, NULL, XtRInt, sizeof(int),
	XtOffsetOf(XvwMenuButtonWidgetRec, button.pixmap_justify),
	XtRImmediate, (XtPointer) KPIXMAP_JUSTIFY_RIGHT},
};
#undef offset


#define SUPERCLASS (&xvwButtonWidgetClassRec)

XvwMenuButtonWidgetClassRec xvwMenuButtonWidgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "MenuButton",			/* class_name		  */
    sizeof(XvwMenuButtonWidgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    actionsList,			/* actions		  */
    XtNumber(actionsList),		/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    NULL,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    defaultTranslations,		/* tm_table		  */
    NULL,				/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,				    /* subresources	  */
    0,					    /* subresources_count */
    sizeof(XvwMenuButtonWidgetConstraintsRec),  /* constraint_size    */
    NULL,				    /* initialize         */
    NULL,                                   /* destroy            */
    NULL,				    /* set_values	  */
    NULL,                                   /* extension          */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			    /* pre-layout routine     */
    XtInheritLayout,			    /* post-layout routine    */
    XtInheritChangeSel,			    /* change selected proc   */
    XtInheritEraseSel,			    /* erase selected proc    */
    XtInheritRefreshSel,		    /* refresh selection proc */
    XtInheritResize,			    /* resize		      */
    XtInheritGeometryManager,		    /* geometry_manager	      */
  }, /* XvwManagerWidgetClass fields initialization */
  {
    NULL,                                   /* field not used       */
  },  /* XvwButtonWidgetClass fields initialization */
  {
    NULL,                                     /* field not used     */
  },  /* XvwMenuButtonWidgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwMenuButtonWidgetClass for public consumption
 */
WidgetClass xvwMenuButtonWidgetClass = (WidgetClass) &xvwMenuButtonWidgetClassRec;

/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for Class and Constraint Declarations
|
--------------------------------------------------------------------*/
 
#define kwidget(widget)      (XvwMenuButtonWidget) (widget)
#define kwidgetclass(widget) (XvwMenuButtonWidgetClass) (widget)->core.widget_class
#define kconstraint(widget)  (XvwMenuButtonWidgetConstraints) (widget)->core.constraints

static Widget active = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of a xvwMenuButtonWidgetClass has been created. 
|                It will initialize all the class attributes. 
|
|    Written By: Mark Young
|          Date: Mar 10, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
        xvw_init_attributes(xvwMenuButtonWidgetClass, attributes,
		XtNumber(attributes), NULL, 0,
		"$DESIGN/objects/library/kwidgets/uis/MenuButton.pane");
	xvw_load_resources("$DESIGN/objects/library/kwidgets/app-defaults/MenuButton");
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial values
|                for the fields of a label object instance.       
|                The width, height, ascent, and descent are       
|                all initialized to sane values.
|
|         Input: request - not used     
|                new     - object instance after initialization, with
|    Written By: Mark Young
|          Date: Mar 10, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
        XvwMenuButtonWidget xwid = (XvwMenuButtonWidget) new;

	xvobject object = xvw_object(new);

        /*
         *  Initialize our state and also add event handlers to do the
         *  menubutton thang..
         */
	xwid->menubutton.id    = 0;
        xwid->menubutton.menu  = xvw_create_menu(object, "Menu");
        xwid->menubutton.popup = xvw_parent(xwid->menubutton.menu);
	xvw_unmanage(xwid->menubutton.popup);
	xvw_unmap(xwid->menubutton.popup);
	if (!XtIsSubclass(XtParent(new), RowColWidgetClass))
           xwid->button.pixmap = xvw_create_pixmap(object, "MenuButtonPixmap");

	XtRegisterGrabAction((XtActionProc) NotifyHandler, TRUE,
		(int) (ButtonPressMask|ButtonReleaseMask), 
		GrabModeAsync,GrabModeAsync);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: This method will 
|
|         Input: widget - the Menu widget being destroyed
|    Written By: Mark Young
|          Date: May 8, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
	if (widget == active)
	   active = NULL;
}

/*-----------------------------------------------------------
|
|  Routine Name: PopupMenuHandler
|
|       Purpose: This routine will popup the associated menu for
|		 the given menubutton.
|
|         Input: widget - the menubutton widget to popup the menu
|    Written By: Mark Young
|          Date: May 02, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void PopupMenuHandler(
   kaddr data,
   XtIntervalId *id)
{
	xvobject button = (xvobject) data;
        XvwMenuButtonWidget xwid = (XvwMenuButtonWidget) xvw_widget(button);

	unsigned int w, h, bw, dummy;
	int x, y, dispwidth, dispheight;

	Widget   parent, widget = xvw_widget(button);
	xvobject popup = xwid->menubutton.popup;

	/*
	 *  Reset the timer id to zero.
	 */
	xwid->menubutton.id = 0;

	/*
	 *  If the parent of the menubutton is a menu...
	 */
	parent = XtParent(widget);
	xvw_geometry(button, NULL, NULL, &dummy, &h, NULL);
	dispheight = XDisplayHeight(xvw_display(NULL), xvw_screennum(NULL));
	dispwidth  = XDisplayWidth(xvw_display(NULL), xvw_screennum(NULL));
	if (XtIsSubclass(parent, MenuWidgetClass) == TRUE)
	{
	   xvw_manage(popup);
	   xvw_translate_coords(button, 0, 0, NULL, &y);
	   xvw_geometry(xvw_object(parent), NULL, NULL, &w, &dummy, NULL);
	   xvw_translate_coords(xvw_object(parent), (int) w, 0, &x, NULL);

	   /*
	    * move menu position up and back if necessary
	    */
	   xvw_manage(popup);
	   xvw_geometry(popup, NULL, NULL, &w, &h, &bw);
           if (y+h > dispheight)
	      y = dispheight - h - 3*bw;
           if (x+w > dispwidth)
	      x = dispwidth - w - 3*bw;

	   /*
	    *  position menu
	    */
	   xvw_set_attributes(popup, XVW_SHELL_X, x, XVW_SHELL_Y, y, NULL);
	}

	/*
	 * If the parent of the menubutton is not a menu, we need to make
	 * sure that the bottom of the menu won't go off the bottom of the
	 * screen, and that the right of the menu won't go off the right side
	 * of the screen.  We get the location of the menubutton in order to
	 * position the menu, but modify that by moving up and back if the
	 * menu would otherwise go off the screen.
	 */
	else
	{
	   /*
	    * default location for menu position: same as menubutton position
	    */
	   xvw_translate_coords(button, 0, 0, &x, &y);
	   xvw_geometry(button, NULL, NULL, NULL, &h, &bw);
	   x += bw; y += h + bw;

	   /*
	    * move menu position up and back if necessary
	    */
	   xvw_manage(popup);
	   xvw_geometry(popup, NULL, NULL, &w, &h, &bw);
           if (y+h > dispheight)
	      y = dispheight - h - 3*bw;
           if (x+w > dispwidth)
	      x = dispwidth - w - 3*bw;

	   /*
	    *  position menu
	    */
	   xvw_set_attributes(popup, XVW_SHELL_X, x, XVW_SHELL_Y, y, NULL);
	}
	xvw_map(popup);
	xvw_raise(popup);
}

/*-----------------------------------------------------------
|
|  Routine Name: PopupMenu
|
|       Purpose: This routine will popup the associated menu for
|		 the given menubutton.
|
|         Input: widget - the menubutton widget to popup the menu
|    Written By: Mark Young
|          Date: May 02, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void PopupMenu(
   xvobject button)
{
        XvwMenuButtonWidget xwid = (XvwMenuButtonWidget) xvw_widget(button);

	XvwMenuWidget xmenu;
	Widget   check, parent, widget = xvw_widget(button);
	xvobject popup = xwid->menubutton.popup;

	if (xvw_check_mapped(popup))
	   return;

	if (button == NULL)
	   return;

	parent = XtParent(widget); xmenu = (XvwMenuWidget) parent;
	if (XtIsSubclass(parent, MenuWidgetClass) == TRUE)
	   check = xmenu->menu.active;
	else
	   check = active;

	if (check != NULL)
	   PopdownMenu(xvw_object(check));

	if (XtIsSubclass(parent, MenuWidgetClass) == TRUE)
	   xmenu->menu.active = widget;
	else active = widget;

	if (XtIsSubclass(parent, MenuWidgetClass) == TRUE)
	{        
	   xwid->menubutton.id = XtAppAddTimeOut(xvw_appcontext(button),
                   (unsigned long) XtGetMultiClickTime(xvw_display(button)),
		   PopupMenuHandler, button);
	}
	else
	   PopupMenuHandler(button, NULL);

	xvw_set_attribute(button, XVW_BUTTON_SELECTED, TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: PopdownMenu
|
|       Purpose: This routine will popdown the associated menu for
|		 the given menubutton.
|
|         Input: widget - the menubutton widget to popdown the menu
|    Written By: Mark Young
|          Date: May 02, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void PopdownMenu(
   xvobject button)
{
        XvwMenuButtonWidget xwid = (XvwMenuButtonWidget) xvw_widget(button);

	XvwMenuWidget xmenu;
	Widget   parent, widget = (Widget) xwid;

	if (button == NULL)
	   return;

	xmenu = (XvwMenuWidget) xvw_widget(xwid->menubutton.menu);
	if (xmenu->menu.active != NULL)
	   PopdownMenu(xvw_object(xmenu->menu.active));

	parent = XtParent(widget); xmenu = (XvwMenuWidget) parent;
	if (XtIsSubclass(parent, MenuWidgetClass))
	{
	   if (widget == xmenu->menu.active)
              xmenu->menu.active = NULL;
	}
        else if (widget == active)
           active = NULL;

	if (xwid->menubutton.id > 0)
	{
	   XtRemoveTimeOut(xwid->menubutton.id);
	   xwid->menubutton.id = 0;
	}
	xvw_unmap(xwid->menubutton.popup);
	xvw_set_attribute(button, XVW_BUTTON_SELECTED, FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: NotifyHandler
|
|       Purpose: Notify the user of the joyous selection that have
|                so dedicately selected, or are about to select.
|
|         Input: object - the wonderous selection in question
|                client_data - my word, that's me!
|                event  - the event that triggered this mess.
|    Written By: Mark Young
|          Date: Mar 10, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void NotifyHandler(
   Widget   widget,
   XEvent   *event,
   String   *params,
   Cardinal num_params)
{
	xvobject button = xvw_object(widget);

        if (event->type == EnterNotify && active)
	   PopupMenu(button);
	else if (event->type == ButtonPress || event->type == KeyPress)
	   PopupMenu(button);
	else if (event->type == ButtonRelease || event->type == KeyRelease)
	   PopdownMenu(button);
	else if (event->type == LeaveNotify && event->xcrossing.state == 0)
	   PopdownMenu(xvw_object(active));
}

/*-----------------------------------------------------------
|
|  Routine Name: WindowHandler
|
|       Purpose: Notify the user of the joyous selection that have
|                so dedicately selected, or are about to select.
|
|         Input: widget - the wonderous selection in question
|                event  - the event that triggered this mess.
|    Written By: Mark Young
|          Date: Mar 10, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void WindowHandler(
   Widget   widget,
   XEvent   *event,
   String   *params,
   Cardinal num_params)
{
	/*
	 *  It's embarassing, but for some strange reason Xt won't allow
	 *  registration grab of LeaveWindow events.  Hence, we set up
	 *  this dummy procedure for Enter and Leave Window events that
	 *  forwards them to our original desired action routine.
	 */
	NotifyHandler(widget, event, params, num_params);
}

/************************************************************
*
*  Routine Name: xvw_create_menubutton - create menubutton GUI object
*
*       Purpose: A menubutton object is a rectangular or oval mechanism 
*		 displaying either a text label or a pixmap. If a callback 
*                is installed on the menubutton, software control will be passed
*                to the callback routine when the user clicks the mouse on the
*                menubutton.
*
*         Input: parent - parent of the menubutton object
*                name   - a name for this particular instance of the menubutton 
*                         object (for use in app-defaults files, etc)
*
*        Output:
*       Returns: The GUI menubutton object on success, NULL on failure.
*  Restrictions: 
*    Written By: Mark Young
*  	   Date: Mar 10, 1996
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_menubutton(
   xvobject parent,
   char     *name)
{
	xvobject  object;


	/*
	 *  Call the xvw_create() routine to do the actual creation.
	 */
	object = xvw_create(parent, FALSE, TRUE, name, MenuButtonWidgetClass);
	return(object);
}

/*-------------------------------------------------------------
|
|  Routine Name: xvw_retrieve_menu - get the menu component of a
|                                    menubutton object
|                
|       Purpose: Given a compound menubutton object, returns the 
|                menu associated with the menubutton.  
|
|                IMPORTANT NOTE: as of Khoros 2.1, this routine is obsolete;
|                it is included solely for backwards compatibility
|                with Khoros 2.0.2p3.  The correct way to obtain
|                the menu associated with the menubutton is now:
|             \f(CWxvw_get_attribute(menubutton, XVW_MENUBUTTON_MENU, &menu);\fP
|		 Please update code as necessary; in the next release
|                of Khoros, \fIxvw_retrieve_menu()\fP will be deleted.
|
|         Input: menubutton - the compound menubutton object
|                             for which the menu is desired 
|        Output: 
|       Returns: The menu object on success, NULL on failure
|    Written By: Danielle Argiro
|          Date: May 18, 1993
| Modifications:
|
------------------------------------------------------------------*/

xvobject xvw_retrieve_menu(
   xvobject menubutton)
{
	xvobject menu = NULL;

	xvw_get_attribute(menubutton, XVW_MENUBUTTON_MENU, &menu);
	return(menu);
}
