/*
  * Khoros: $Id: Pixmap.c,v 1.9 1996/06/25 16:33:25 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Pixmap Gadget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Destroy()
   >>>>                 Realize()
   >>>>                 Redisplay()
   >>>>                 RecomputeSize()
   >>>>                 SetValues()
   >>>>                 PickGadget()
   >>>>   Public:
   >>>>                 xvw_create_pixmap()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <kwidgets/PixmapP.h>

static void ClassInitialize	 PROTO((void));
static void Initialize           PROTO((Widget, Widget, ArgList, Cardinal *));
static void Destroy  	         PROTO((Widget));
static void Realize              PROTO((Widget));
static void Redisplay		 PROTO((Widget, XEvent *, Region));
static Boolean SetValues	 PROTO((Widget, Widget, Widget, ArgList,
					Cardinal *));
static void RecomputeSize	 PROTO((Widget));
static int  PickGadget           PROTO((Widget, int, int));


/*-------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_PIXMAP,          NULL,        XtRPixmap,     NULL},
{XVW_PIXMAP_FILENAME, XVW_PIXMAP,  XtRString,     XtRPixmap},
{XVW_PIXMAP_MASK,     NULL,        XtRPixmapMask, NULL},
{XVW_PIXMAP_MASKNAME, XVW_PIXMAP_MASK,  XtRString, XtRPixmapMask},
};

/*-------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwPixmapGadgetRec, pixmap.field)

static XtResource resources[] = { 
{XVW_PIXMAP, XVW_PIXMAP_FILENAME, XtRPixmap, sizeof(Pixmap), 
      offset(pixmap), XtRImmediate, (XtPointer) None},
{XVW_PIXMAP_MASK, XVW_PIXMAP_MASKNAME, XtRPixmap, sizeof(Pixmap), 
      offset(mask), XtRImmediate, (XtPointer) None},
};
#undef offset


#define SUPERCLASS (&xvwManagerGadgetClassRec)

XvwPixmapGadgetClassRec xvwPixmapGadgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Pixmap",				/* class_name		  */
    sizeof(XvwPixmapGadgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    (XtProc) Realize,			/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    FALSE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    FALSE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    XtInheritResize,			/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    NULL,				/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* RectObjClass fields initialization */
  {
    PickGadget,				/* pick object proc       */
    XtInheritEraseSel,                  /* erase selected proc    */
    XtInheritRefreshSel,	        /* refresh selection proc */
    NULL,			        /* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
  {
    NULL,				/* extension - not used   */
  },  /* XvwPixmapGadgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwPixmapGadgetClass for public consumption
 */
WidgetClass xvwPixmapGadgetClass = (WidgetClass) &xvwPixmapGadgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of pixmap object class has been created.
|                It will initialize all the class attributes. 
|
|    Written By: Mark Young
|          Date: Mar 03, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
	xvw_init_attributes(xvwPixmapGadgetClass, attributes,
		XtNumber(attributes), NULL, 0, 
		"$DESIGN/objects/library/kwidgets/uis/Pixmap.pane");
	xvw_load_resources("$DESIGN/objects/library/kwidgets/app-defaults/Pixmap");
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This will initialize the pixmap object.
|
|         Input: request - not used
|                new     - widget instance after initialization
|    Written By: Mark Young
|          Date: Mar 03, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget request,
   Widget new,
   ArgList args,
   Cardinal *num_args)
{
	XvwPixmapGadget xobj = (XvwPixmapGadget) new;

	xobj->pixmap.xorigin =
	xobj->pixmap.yorigin = -1;
	xobj->pixmap.width   =
	xobj->pixmap.height  = 0;
	xobj->pixmap.gc      = NULL;
	if (xobj->pixmap.pixmap != None)
	{
	   char *name;
	   xvobject object = xvw_object(new);

	   xvw_get_attribute(object, XVW_PIXMAP_FILENAME, &name);
	   xvw_set_attribute(object, XVW_PIXMAP_MASKNAME, name);
	}
	RecomputeSize(new);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: This method will free the label string.
|
|         Input: widget - the widget being destroyed
|    Written By: Mark Young
|          Date: Apr 17, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
        XvwPixmapGadget xobj = (XvwPixmapGadget) widget;

	if (xobj->pixmap.gc)
	   XtReleaseGC(widget, xobj->pixmap.gc);
}

/*-----------------------------------------------------------
|
|  Routine Name: Realize
|
|       Purpose: This method will redisplay the pixmap if force redisplay
|		 is set.
|
|         Input: widget - the pixmap object to be realized
|    Written By: Mark Young
|          Date: Apr 10, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Realize(
   Widget   widget)
{
	XvwPixmapGadget xobj = (XvwPixmapGadget) widget;


	/*
	 *  Refresh the pixmap if force_redisplay is set.
	 */
	if (xobj->manager.force_redisplay)
	   Redisplay(widget, NULL, NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This routine will redraw the pixmap object in response
|                to an expose event. 
|
|         Input: widget - the pixmap object that was exposed 
|                event  - the event that caused the redraw
|                region - the region that was exposed 
|    Written By: Mark Young
|          Date: Mar 03, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	XvwPixmapGadget xobj = (XvwPixmapGadget) widget;

	XGCValues values;
	int    x, y, w, h;
	unsigned long mask;

	/*
	 *  Call the ManagerObj to do the widget thang (make sure we are
	 *  realized, redraw the background, border, etc)
	 */
	(*xvwPixmapGadgetClass->core_class.superclass->core_class.expose)
                (widget, event, region);

	/*
	 *  If there is no pixmap then 
	 */
	if (!xobj->pixmap.pixmap)
	   return;

	x = widget->core.x + widget->core.border_width;
	y = widget->core.y + widget->core.border_width;
	w = xobj->pixmap.width;
	h = xobj->pixmap.height;
	if (xobj->manager.shadow_type != KSHADOW_NONE)
	{
	   x += xobj->manager.shadow_thickness;
	   y += xobj->manager.shadow_thickness;
	}

	/*
	 *  If the pixmap gc has not been initialized, or if a mask exists
	 *  and our origin has changed then reallocate the gc.  The reason
	 *  why we need to reallocate the gc for an origin change is because
	 *  since we are a gadget we use our parent's window in which to
	 *  display the pixmap.  This means that if our position changes, we
	 *  really have changed position of ourselves, relative to our parent.
	 */
	if (xobj->pixmap.gc == NULL || (xobj->pixmap.mask != 0 &&
	    (x != xobj->pixmap.xorigin || y != xobj->pixmap.yorigin)))
	{
	   xobj->pixmap.xorigin = x; xobj->pixmap.yorigin = y;

	   if (xobj->pixmap.gc != NULL)
	      XtReleaseGC(widget, xobj->pixmap.gc);

           /*
            *  Create the gc to be used to refresh the pixmap
            */
           values.clip_x_origin = x;
           values.clip_y_origin = y;
           values.clip_mask  = xobj->pixmap.mask;
           values.foreground = xobj->manager.foreground;
           values.background = xobj->manager.background;
           mask = GCForeground | GCBackground | GCClipXOrigin | GCClipYOrigin |
		  GCClipMask;
	   xobj->pixmap.gc = XtGetGC(widget, mask, &values);
	}
	XCopyArea(XtDisplay(widget), xobj->pixmap.pixmap, XtWindow(widget),
		  xobj->pixmap.gc, 0, 0, (unsigned) w, (unsigned) h, x, y);
}

/*-----------------------------------------------------------
|
|  Routine Name: RecomputeSize
|
|       Purpose: recomputes the size of the pixmap object, based on the
|		 pixmap being displayed.
|
|         Input: widget - the pixmap object in which we will recompute our size
|    Written By: Mark Young
|          Date: Mar 03, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RecomputeSize(
   Widget  widget)
{
	XvwPixmapGadget xobj = (XvwPixmapGadget) widget;

	int	 x, y;
	Window   rootwindow;
	unsigned int width, height, bw, depth, buffer = 0;

	if (xobj->pixmap.pixmap)
	{
	   XGetGeometry(XtDisplay(widget), xobj->pixmap.pixmap, &rootwindow,
			&x, &y, &width, &height, &bw, &depth);
	}
	else
	   width = height = 0;

	xobj->pixmap.width  = width;
	xobj->pixmap.height = height;
	if (xobj->pixmap.gc != NULL)
	{
	   XtReleaseGC(widget, xobj->pixmap.gc);
	   xobj->pixmap.gc = NULL;
	}

	if (xobj->manager.shadow_type != KSHADOW_NONE)
	   buffer = xobj->manager.shadow_thickness;

	widget->core.width  = width  + 2*buffer;
	widget->core.height = height + 2*buffer;
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: If the the size or postion of the pixmap has changed, 
|		 recalculate the size and position of the pixmap.
|
|         Input: current - the object containing current settings 
|                request - the object containing requested settings
|                new     - the object processed through all set values methods 
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Mar 03, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwPixmapGadget cobj = (XvwPixmapGadget) current;
	XvwPixmapGadget nobj = (XvwPixmapGadget) new;

	char *name = NULL;
	Boolean  redisplay = FALSE;

	if (cobj->pixmap.pixmap != nobj->pixmap.pixmap)
	{
	   xvw_get_attribute(xvw_object(new), XVW_PIXMAP_FILENAME, &name);
	   xvw_set_attribute(xvw_object(new), XVW_PIXMAP_MASKNAME, name);

	   RecomputeSize(new);
	   redisplay = TRUE;
	}

	if (redisplay && nobj->manager.force_redisplay)
	   ManagerObjForceRedisplay(current, new, FALSE, &redisplay);

	return(redisplay);
}

/*-----------------------------------------------------------
|
|  Routine Name: PickGadget
|
|       Purpose: Given a connection object and an attempted picking location,
|                see if the pixmap has been picked.
|
|         Input: widget - the connection object we are checking
|                x      - the x device coordinate that the pick attempt was at
|                y      - the y device coordinate that the pick attempt was at
|       Returns: TRUE (1) if object was successfully picked, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Jun 29, 1992 10:35
|
------------------------------------------------------------*/
/* ARGSUSED */
static int PickGadget(
   Widget widget,
   int    x,
   int    y)
{
	XvwPixmapGadget xobj = (XvwPixmapGadget) widget;
	int xpos, ypos, width, height;

	xpos  = xobj->rectobj.x;     ypos = xobj->rectobj.y;
	width = xobj->rectobj.width; height = xobj->rectobj.height;
	if (xpos <= x && x <= xpos+width && ypos <= y && y <= ypos+height)
	   return(TRUE);
	else
	   return(FALSE);
}


/************************************************************
*
*  Routine Name: xvw_create_pixmap - create a pixmap object
*
*       Purpose: A pixmap object simply supports display of a 
*		 bitmap or pixmap, as defined by an xbm or xpm file.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the object
*                         (for use in app-defaults files, etc)
*
*        Output:
*       Returns: The pixmap object on success, NULL on failure
*  Restrictions: 
*    Written By: Mark Young 
*          Date: Mar 03, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_pixmap(
   xvobject parent,
   char   *name)
{
	xvobject  object;


	/*
	 *  Create the line object
	 */
	object = xvw_create(parent, FALSE, TRUE, name, PixmapGadgetClass);
	if (object != NULL)
	{
	   xvw_set_attributes(object,
                XVW_MENUABLE,     TRUE,     /* menuable       */
                XVW_RESIZABLE,    FALSE,    /* resizable      */
                XVW_SELECTABLE,   TRUE,     /* selectable     */
		NULL);
	}
	return(object);
}
