/*
 * Khoros: $Id: destroy.c,v 1.9 1996/05/15 23:21:52 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: destroy.c,v 1.9 1996/05/15 23:21:52 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		<<<<
   >>>>		GUI Destruction Routines                   <<<<
   >>>>		<<<<
   >>>>  Private:
   >>>>                 xvf_destroy_control()
   >>>>                 xvf_destroy_selection()
   >>>>   Static:
   >>>>                 destroy_sel_list()
   >>>>                 free_extra_call()
   >>>>   Public:
   >>>>                 xvf_destroy_allforms()
   >>>>                 xvf_destroy_form()
   >>>>                 xvf_destroy_subform()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"


static void destroy_sel_list  PROTO((kselection *));
static void free_extra_call   PROTO((klist *));



/************************************************************
*
*  Routine Name: xvf_destroy_allforms - destroy all forms
*
*       Purpose: destroys all current forms
*         Input:
*        Output:
*       Returns:
*  Restrictions:
*    Written By: Mark Young
*          Date: Oct 18, 1993
*      Verified:
*  Side Effects:
* Modifications:
*************************************************************/

void xvf_destroy_allforms(void)
{
	int        i;
	size_t	   num;
	kform_list *entry;
	xvobject   *toplevels;


	entry = kvf_get_entry(NULL);
	while (entry != NULL)
	{
	   if (entry->type == KFORM)
	   {
	      xvf_destroy_form((kform *) entry->treeptr);
	      entry = kvf_get_entry(NULL);
	   }
	   else
	      entry = entry->next;
	}

	toplevels = xvw_getall_toplevels(&num);
	for (i = 0; i < num; i++)
	   xvw_destroy(toplevels[i]);
	kfree_and_NULL(toplevels);
}


/************************************************************
*
*  Routine Name: xvf_destroy_form - destroy GUI of xvroutine & free associated memory
*
*       Purpose: Destroys all GUI objects associated with a particular 
*                form tree and the form tree itself. 
*
*         Input: form - pointer to the form tree being destroyed
*        Output:
*       Returns:
*  Restrictions:
*    Written By: Danielle Argiro
*          Date: March 29, 1993
*      Verified:
*  Side Effects:
* Modifications:
*************************************************************/

void xvf_destroy_form(
    kform *form)
{
	ksubform   *subform;
	kselection *quit_sel = NULL;


        if (form == NULL) return;

	/*
	 *  If there is a master form...
	 */
	if (form->master != NULL)
        {
	    /* 
	     * if there was a master w/ quit button, will need that
	     * to send as the client_data when removing protocol handler 
	     */
	    if (form->master->sel_list != NULL)
	        quit_sel = kvf_find_specified_selection(form->master->sel_list,
                                                        KUIS_QUIT, NULL);
	    /*
	     * destroy all subforms on master 
	     */
            subform = form->master->subform_list;
            while (subform != NULL)
	    {
                xvf_destroy_subform(subform);
		subform = subform->next;
	    }

	    /*
	     * de-activate menuform (if it's up) before destroying master
	     */
            if (xvw_activated_menu(form->master->back))
                xvw_inactivate_menu(form->master->back);
            xvf_destroy_control(form->master);
        }
	
	/*
	 * If there is a subforms
	 */
        else if (form->subform != NULL)
	     xvf_destroy_subform(form->subform);

	if (form->toplevel != NULL)
	{
	      if (quit_sel != NULL)
  	          xvw_remove_protocol(form->toplevel, "WM_DELETE_WINDOW",
                                      xvf_quit_master_cb, 
		    	              (kaddr) quit_sel->back_kformstruct);
              xvw_destroy(form->toplevel);
	}

	kvf_destroy_form(form);
}

/*************************************************************
*
*  Routine Name: xvf_destroy_subform - destroy subform & free associated memory
*
*       Purpose: destroys all xvobjects associated with the subform tree 
*                and the subform tree itself.
*
*         Input: subform - the subform pointer
*        Output:
*       Returns:
*    Written By: Danielle Argiro and Mark Young
*          Date: May 6, 1992
*      Verified:
*  Side Effects:
* Modifications:
*
**************************************************************/

void xvf_destroy_subform(
   ksubform *subform)
{
        kguide     *guide;
	kselection *quit_sel = NULL;

        if (subform == NULL) return;

        if (subform->guidepane != NULL)
        {
	    /* 
             * if there was a subform w/ quit button, will need that
             * to send as the client_data when removing protocol handler 
             */
	     if (subform->guidepane->sel_list != NULL)
		quit_sel = 
		    kvf_find_specified_selection(subform->guidepane->sel_list,
                                                        KUIS_QUIT, NULL);
            guide = subform->guidepane->guide_list;
            while (guide != NULL)
	    {
                if (guide->pane != NULL)
                    xvf_destroy_control(guide->pane);
	        klist_free(guide->extra_call_list, free_extra_call);
		guide = guide->next;
	    }
            xvf_destroy_control(subform->guidepane);

        }
	else if ((subform->guide != NULL) &&
                 (subform->guide->pane != NULL))
	{
	    klist_free(subform->guide->extra_call_list, free_extra_call);
            xvf_destroy_control(subform->guide->pane);
	}

        if (xvw_activated_menu(subform->back))
            xvw_inactivate_menu(subform->back);

        if (subform->toplevel != NULL)
	{
	    if (quit_sel != NULL)
	        xvw_remove_protocol(subform->toplevel, "WM_DELETE_WINDOW",
                                xvf_quit_subform_cb, 
			        (kaddr) quit_sel->back_kformstruct);
            xvw_destroy(subform->toplevel);
	}

	klist_free(subform->extra_call_list, free_extra_call);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvf_destroy_control
|
|       Purpose: destroys the structure associated with a master,
|                guidepane, or pane.
|         Input: control - the pointer to control panel structure
|        Output: none
|          Date: May 7, 1992
|    Written By: Danielle Argiro
| Modifications:
|
------------------------------------------------------------*/

void xvf_destroy_control(
   kcontrol *control)
{
	kselection *quit_sel = NULL;
        if (control->sel_list != NULL)
	{
	    quit_sel = kvf_find_specified_selection(control->sel_list,
                                                    KUIS_QUIT, NULL);
	    if (control->type == KPANE)
            {
		if (quit_sel != NULL)
 	            xvw_remove_protocol(xvw_toplevel(quit_sel->back),
	  		        "WM_DELETE_WINDOW", xvf_quit_pane_cb, 
				(kaddr) quit_sel->back_kformstruct);
           }
           destroy_sel_list(control->sel_list);
 	}
}

/*-----------------------------------------------------------
|
|  Routine Name: destroy_sel_list
|
|       Purpose: destroys a selection list
|         Input: selection - the header of the selection list
|        Output: none
|          Date: May 7, 1992
|    Written By: Danielle Argiro
| Modifications:
|
------------------------------------------------------------*/

static void destroy_sel_list(
   kselection *sel_list)
{
	kselection *selection = sel_list;
        while (selection != NULL)
	{
            xvf_destroy_selection(selection);
	    selection = selection->next;
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: xvf_destroy_selection
|
|       Purpose: destroys a selection
|         Input: selection - the selection to be destroyed
|        Output: none
|          Date: May 7, 1992
|    Written By: Danielle Argiro
| Modifications:
|
------------------------------------------------------------*/
void xvf_destroy_selection(
   kselection *selection)
{
        kselection   *toggle, *group;

        toggle = selection->toggle_next;
        while (toggle != NULL)
	{
            xvf_destroy_selection(toggle);
	    toggle = toggle->next;
	}

        group = selection->group_next;
        while (group != NULL)
	{
            xvf_destroy_selection(group);
	    group = group->next;
	}
	
        if (selection->back != NULL)
        {
	    /* submenu selection; destroy menubutton (label object) */ 
	    if (selection->type == KUIS_STARTSUBMENU) 
		xvw_destroy(selection->label_object);

	    /* 
	     * regular selection; just destroy backplane => as long as it's 
             * not _on_ a submenu, in which case it'll be destroyed above
	     */
	    else if (selection->back_submenu == NULL)
	    {
                if (xvw_activated_menu(selection->back))
                    xvw_inactivate_menu(selection->back);
		xvw_destroy(selection->back);
	    }
        }
	selection->back = NULL;

	klist_free(selection->extra_call_list, free_extra_call);
}

/*-----------------------------------------------------------
|
|  Routine Name: free_extra_call - frees callback entry from
|                                  extra call list
|
|       Purpose: This utility frees a single action handler entry from
|                the action list of an object, called by xvw_destroy_actions()
|                to delete individual handlers from the object action list.
|
|         Input: listptr - pointer to action handler entry to be freed
|        Output:
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Danielle Argiro
|          Date: August 18, 1995
| Modifications:
|
------------------------------------------------------------*/

static void free_extra_call(
    klist *listptr)
{
	xvf_extracall_info *call_info;

        call_info = (xvf_extracall_info *) klist_clientdata(listptr);
        kfree_and_NULL(call_info);
}


