/*
  * Khoros: $Id: Area.c,v 1.13 1996/06/20 22:50:07 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Area.c,v 1.13 1996/06/20 22:50:07 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Area Widget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Realize()
   >>>>                 SetValues()
   >>>>   Public:
   >>>>                 xvw_create_area()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/AreaP.h>

static void ClassInitialize PROTO((void));
static void Initialize	    PROTO((Widget, Widget, ArgList, Cardinal *));
static Boolean SetValues    PROTO((Widget, Widget, Widget, ArgList,Cardinal *));
static void Realize         PROTO((Widget, XtValueMask *,
				   XSetWindowAttributes *));

#undef  kwidget
#define kwidget(widget)      (XvwAreaWidget) (widget)

/*--------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_AREA_DISPLAY_DATE,	     NULL,	        XtRInt,		XtRBoolean},
{XVW_AREA_DISPLAY_TITLE,     NULL,	        XtRInt,		XtRBoolean},
{XVW_AREA_DATE,		     NULL,        	XtRObject,	NULL},
{XVW_AREA_TITLE,	     NULL,   	        XtRObject,	NULL},
{XVW_AREA_TITLE_STRING,	    "areaTitle.stringString",	XtRString,	NULL},
};

/*--------------------------------------------------------
|
|   Full class record constant 
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwAreaWidgetRec, area.field)

static XtResource resources[] = { 
{XVW_AREA_DISPLAY_DATE, NULL, XtRBoolean, sizeof(Boolean),
      offset(display_date), XtRImmediate, (XtPointer) FALSE},
{XVW_AREA_DISPLAY_TITLE, NULL, XtRBoolean,sizeof(Boolean),
      offset(display_title), XtRImmediate, (XtPointer) TRUE},
{XVW_AREA_DATE, NULL, XtRObject, sizeof(xvobject),
        offset(date), XtRImmediate, (XtPointer) NULL},
{XVW_AREA_TITLE, NULL, XtRObject, sizeof(xvobject),
        offset(title), XtRImmediate, (XtPointer) NULL},

{XVW_PREFERRED_WIDTH, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwManagerWidgetRec, manager.preferred_width),
        XtRImmediate, (XtPointer) 512 },
{XVW_PREFERRED_HEIGHT, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwManagerWidgetRec, manager.preferred_height),
        XtRImmediate, (XtPointer) 512},
};
#undef offset


/*--------------------------------------------------------
|
|   Constraint SubResources
|
--------------------------------------------------------*/

#define SUPERCLASS (&xvwGraphicsWidgetClassRec)

XvwAreaWidgetClassRec xvwAreaWidgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Area",				/* class_name		  */
    sizeof(XvwAreaWidgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    Realize,				/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    knumber(resources),			/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal |
    XtExposeGraphicsExposeMerged,	/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    NULL,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    XtInheritTranslations,		/* tm_table		  */
    NULL,				/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,	            			/* subresources       */
    0,	   					/* subresources_count */
    sizeof(XvwGraphicsWidgetConstraintsRec),	/* constraint_size    */
    NULL,				    /* initialize         */
    NULL,                                   /* destroy            */
    NULL,				    /* set_values         */
    NULL,                                   /* extension          */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			    /* pre-layout routine     */
    XtInheritLayout,			    /* post-layout routine    */
    XtInheritChangeSel,		            /* change selected proc   */
    XtInheritEraseSel,			    /* erase selected proc    */
    XtInheritRefreshSel,		    /* refresh selection proc */
    XtInheritResize,			    /* resize		      */
    XtInheritGeometryManager,  		    /* geometry_manager	      */
  }, /* XXvwManagerWidgetClass fields initialization */
  {
    NULL,                                     /* field not used    */
  },  /* XvwGraphicsWidgetClass fields initialization */
  {
    NULL,                                     /* field not used    */
  },  /* XvwAreaWidgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwAreaWidgetClass for public consumption
 */
WidgetClass xvwAreaWidgetClass = (WidgetClass) &xvwAreaWidgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of area widget class has been created.
|		 It will initialize all the class attributes. 
|
|         Input: None 
|
|        Output: None
|
|    Written By: Mark Young & John M. Salas
|          Date: Aug 15, 1992
| Modifications:
|
------------------------------------------------------------*/
static void ClassInitialize(void)
{
	xvw_init_attributes(xvwAreaWidgetClass, attributes, 
			(int) knumber(attributes), NULL, 0,
		       "$DESIGN/objects/library/xvisual/uis/Area.pane");
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/Area");
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial values 
|                for all the fields for a XvwAreaWidget instance. 
|
|         Input: request - widget instance with requested public settings
|		 new	 - widget instance after initialization, with 
|                          settings initialized 
|        Output: None
|
|    Written By: Mark Young
|          Date: Jun 29, 1992
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget request,
   Widget new,
   ArgList args,
   Cardinal *num_args)
{
	XvwAreaWidget xwid = kwidget(new);

        xwid->area.date = xvw_create_date(xvw_object(new), "date");
	xvw_set_attributes(xwid->area.date,
			   XVW_STRING_JUSTIFICATION, KSTRING_JUSTIFY_RIGHT,
			   XVW_MAPPED, xwid->area.display_date,
			   XVW_FONT,   xwid->manager.font,
			   XVW_GRAPHICS_UPDATEDC, FALSE,
			   XVW_ABOVE,    NULL,
			   XVW_LEFT_OF,  NULL,
			   NULL);

        xwid->area.title = xvw_create_string(xvw_object(new), "title");
	xvw_set_attributes(xwid->area.title,
			   XVW_STRING_JUSTIFICATION, KSTRING_JUSTIFY_CENTER,
			   XVW_MAPPED, xwid->area.display_title,
			   XVW_FONT,   xwid->manager.font,
			   XVW_STRING_STRING, "Area Title",
			   XVW_GRAPHICS_UPDATEDC, FALSE,
			   XVW_BELOW,  NULL,
			   XVW_LEFT_OF, NULL,
			   XVW_RIGHT_OF, NULL,
			   NULL);
	xwid->area.display_legend = FALSE;
}

/*-----------------------------------------------------------
|
|  Routine Name: Realize
|
|       Purpose: This method will set up the window attributes 
|                and create a window for the area widget. 
|
|         Input: widget	    - the widget to realize 
|		 valuemask  - determines the window attributes being passed in
|		 attributes - the list of window attributes to be set 
|
|        Output: None 
|
|    Written By: Mark Young
|          Date: Jun 29, 1992
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Realize(
   Widget               widget,
   XtValueMask          *valuemask,
   XSetWindowAttributes *winattrib)
{
	int	   mode;

	/*
	 *  Set backing store if applicable
	 */
	if ((mode = XDoesBackingStore(XtScreen(widget))) != NotUseful)
	{
	   *valuemask |= CWBackingStore;
	   winattrib->backing_store = mode;
	} 
	(*xvwAreaWidgetClass->core_class.superclass->core_class.realize)
		(widget, valuemask, winattrib);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: Determine what the widget should do by
|		 comparing the current values verse the
|		 new settings.
|
|         Input: current - the widget containing current settings 
|		 request - the widget containing requested settings
|		 new	 - the widget processed through all set values methods 
|
|        Output: None directly 
|
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|
|    Written By: Mark Young
|          Date: Jun 29, 1992
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget current,
   Widget request,
   Widget new,
   ArgList  args,
   Cardinal *num)
{
	XvwAreaWidget cxwid = kwidget(current);
	XvwAreaWidget nxwid = kwidget(new);

	if (cxwid->area.display_date != nxwid->area.display_date)
	{
	   xvw_set_attribute(nxwid->area.date, XVW_MAPPED,
			nxwid->area.display_date);
	}

	if (cxwid->area.display_title != nxwid->area.display_title)
	{
	   xvw_set_attribute(nxwid->area.title, XVW_MAPPED,
			nxwid->area.display_title);
	}

	if (cxwid->area.display_legend != nxwid->area.display_legend)
	   kinfo(KSTANDARD,"Legend is not supported at this time");

	return(FALSE);
}


/************************************************************
*
*  Routine Name: xvw_create_area - create a graphics area object
*
*       Purpose: An area object provides an area which contains a 
*                coordinated view of other visual objects; it is primarily
*		 designed to provide a backplane for 2D plot objects, 3D plot
*		 objects, 2D axis objects, and annotations.  Any visual objects 
*		 that are children of the area object will have the same world 
*                view as the controlling visual object, where the controlling 
*		 visual object may be specified by the application using the 
*		 XVW_GRAPHICS_ATTACH attribute (if an axis is used, it is usually
*		 specified as the controlling visual object).
*
*         Input: parent - the parent object; NULL will cause a  
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the area object
*                         (for use in app-defaults files, etc)
*
*        Output: 
*	Returns: The area object on success, NULL on failure
*  Restrictions:
*    Written By: Mark Young & John M. Salas
*          Date: Jun 29, 1992
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_area(
   xvobject parent,
   char     *name)
{
	xvobject object;


	/*
	 *  Call the xvw_create() routine to do the actual creation.
	 */
	object = xvw_create(parent, FALSE, TRUE, name, AreaWidgetClass);
	return(object);
}
