/*
 * Khoros: $Id: Plot2DRoi.c,v 1.7 1996/04/04 23:29:48 fas Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Plot2DRoi.c,v 1.7 1996/04/04 23:29:48 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Plot2D Roi Routines
   >>>>		
   >>>>  Private:
   >>>>                 Plot2DGetRoi()
   >>>>                 xvw_plot2d_point_inside_roi()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/Plot2DP.h>

/*-----------------------------------------------------------
|
|  Routine Name: Plot2DGetRoi
|
|       Purpose: Plot2DGetRoi is used to get a region of interest of the 
|		 plot2d object.
|
|         Input: object    - the object in which we will be getting the roi
|		 attribute - the attribute name XVW_PLOT2D_ROI
|        Output: calldata  - pointer to retrieve the roi
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young & Danielle Argiro
|          Date: August 22, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
int Plot2DGetRoi(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
	XvwPlot2DGadget xwid = (XvwPlot2DGadget) xvw_widget(object);
        kobject *roi  = (kobject *) calldata;

	int val      = 1;
	int fill     = TRUE;
	int shape    = xwid->plot2d.roi_shape;
	int erase    = TRUE;
	int multiple = FALSE;
	int policy   = KROI_INSIDE;

	/*
	 *  Get the ROI extent that we need in which to extract the data set
	 */
	*roi = kpds_create_object();
	xvw_extract_roi(object, *roi, val, fill, shape, 
			erase, multiple, policy, 1, 1, 1);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_plot2d_point_inside_roi
|
|       Purpose: Given a plot object and an roi data object, where
|		 the roi data object was returned from a call
|		 to xvw_get_attribute(plot, XVW_PLOT2D_ROI, &roi),
|		 this routine determines whether or not a particular
|		 point is inside the ROI or not.
|
|         Input: plot      - the plot object from which the ROI was produced
|		 roi       - the ROI data object containing the mask that
|		             indicates the ROI shape specified by the user
|                point     - Coord specifying point to examine
|        Output: 
|       Returns: TRUE (1) if point is inside the ROI shape, FALSE (0) otherwise
|    Written By: Mark Young & Danielle Argiro
|          Date: August 23, 1995
|
------------------------------------------------------------*/
int xvw_plot2d_point_inside_roi(
    xvobject plot,
    kobject  roi,
    Coord    point)
{
	XVGraphicsID id;
	Real         tmpx, tmpy;
	int          w, h, indx, dcx, dcy, xoffset, yoffset;
	unsigned     char *mask = NULL;

        kpds_get_attributes(roi,
                 KPDS_MASK_SIZE, &w, &h, NULL, NULL, NULL,
                 KPDS_SUBOBJECT_POSITION, &xoffset, &yoffset, NULL, NULL, NULL,
                 NULL);

        /*
         * Get the mask data from the kobject returned from getting
         * XVW_PLOT2D_ROI. Need to initialize mask position and data type
         * before obtaining mask data.
         */
        kpds_set_attributes(roi,
                            KPDS_MASK_POSITION,  0, 0, 0, 0, 0,
                            KPDS_MASK_DATA_TYPE, KUBYTE,
                            NULL);
        mask = (unsigned char *) kpds_get_data(roi, KPDS_MASK_PLANE, NULL);
        if (mask == NULL) return(FALSE);
	
        /*
         * Need the graphics ID of the plot to send to
         * X2D_convert_point_wc_to_dc(); this is obtained by casting the
         * plot widget to a int.
         */
        id = (XVGraphicsID) (xvw_widget(plot));

	/* convert plot point from world coordinates to device coordinates */
        X2D_convert_point_wc_to_dc(id, point, &tmpx, &tmpy);
           dcx = (int) tmpx; dcy = (int) tmpy;

        /* test to see if plot point is inside bounding box of ROI */
        if ((dcx >= xoffset && dcx < xoffset+w) &&
            (dcy >= yoffset && dcy < yoffset+h))
            indx = (dcy-yoffset)*w + dcx-xoffset;
	else return(FALSE);

        if (mask[indx] == 1)
	    return(TRUE);
	else return(FALSE);
}
