/*
  * Khoros: $Id: Rectangle.c,v 1.16 1996/05/15 22:21:27 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Rectangle.c,v 1.16 1996/05/15 22:21:27 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Rectangle Gadget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Redisplay()
   >>>>                 SetValues()
   >>>>                 RecomputePosition()
   >>>>                 ChangeGeometry()
   >>>>                 PickGadget()
   >>>>                 EraseSelection()
   >>>>                 RefreshSelection()
   >>>>   Public:
   >>>>                 xvw_create_rectangle()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/RectangleP.h>

static void    ClassInitialize	 PROTO((void));
static void    Redisplay	 PROTO((Widget, XEvent *, Region));
static Boolean SetValues	 PROTO((Widget, Widget, Widget, ArgList,
					Cardinal *));
static void RecomputePosition	 PROTO((Widget));
static void ChangeGeometry	 PROTO((Widget));
static int  PickGadget           PROTO((Widget, int, int));
static void EraseSelection 	 PROTO((Widget));
static void RefreshSelection	 PROTO((Widget));


/*--------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_RECTANGLE_X,      NULL,    XtRDouble,    NULL},
{XVW_RECTANGLE_Y,      NULL,    XtRDouble,    NULL},
{XVW_RECTANGLE_WIDTH,  NULL,    XtRDouble,    NULL},
{XVW_RECTANGLE_HEIGHT, NULL,    XtRDouble,    NULL},
};

/*--------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwRectangleGadgetRec, rectangle.field)

static XtResource resources[] = { 
{XVW_RECTANGLE_X, NULL, XtRDouble, sizeof(double), 
      offset(x), XtRString, (XtPointer) "0.0"},
{XVW_RECTANGLE_Y, NULL, XtRDouble, sizeof(double), 
      offset(y), XtRString, (XtPointer) "0.0"},
{XVW_RECTANGLE_WIDTH, NULL, XtRDouble, sizeof(double), 
      offset(width), XtRString, (XtPointer) "0.5"},
{XVW_RECTANGLE_HEIGHT, NULL, XtRDouble, sizeof(double), 
      offset(height), XtRString, (XtPointer) "0.6"},
};
#undef offset


#define SUPERCLASS (&xvwGraphicsGadgetClassRec)

XvwRectangleGadgetClassRec xvwRectangleGadgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Rectangle",			/* class_name		  */
    sizeof(XvwRectangleGadgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    NULL,     				/* initialize		  */
    NULL,				/* initialize_hook	  */
    NULL,				/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    knumber(resources),			/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    FALSE,				/* compress_motion	  */
    FALSE,				/* compress_exposure	  */
    FALSE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    NULL,				/* destroy		  */
    XtInheritResize,			/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    NULL,				/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* RectObjClass fields initialization */
  {
    PickGadget,				/* pick object proc       */
    EraseSelection,			/* erase selected proc    */
    RefreshSelection,			/* refresh selection proc */
    ChangeGeometry,			/* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
  {
    RecomputePosition,			/* recompute position proc    */
  },  /* XvwGraphicsGadgetClass fields initialization */
  {
    NULL,				/* extension - not used   */
  },  /* XvwRectangleGadgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwRectangleGadgetClass for public consumption
 */
WidgetClass xvwRectangleGadgetClass = (WidgetClass) &xvwRectangleGadgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of rectangle object class has been created.
|                It will initialize all the class attributes. 
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
        xvw_init_attributes(xvwRectangleGadgetClass, attributes,
		      (int) knumber(attributes), NULL, 0, 
		      "$DESIGN/objects/library/xvisual/uis/Rectangle.pane");
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/Rectangle");
}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This routine will redraw the rectangle object in response
|                to an expose event. 
|
|         Input: widget - the rectangle object that was exposed 
|                event  - the event that caused the redraw
|                region - the region that was exposed 
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	XvwRectangleGadget xobj = (XvwRectangleGadget) widget;

	XVGraphicsID id;
	Coord        coord;
	XColor       *fg, *bg;

	id	 = xobj->graphics.id;
	fg	 = &xobj->graphics.fg;
	bg	 = &xobj->graphics.bg;

	kmemset((kaddr) &coord, 0, sizeof(Coord));
	coord.x = xobj->rectangle.x;
	coord.y = xobj->rectangle.y; 

	if (xobj->graphics.filled == FALSE)
	   X2D_draw_rectangle(id, coord, xobj->rectangle.width, 
			      xobj->rectangle.height, fg);
	else
	   X2D_fill_rectangle(id, coord, xobj->rectangle.width, 
			      xobj->rectangle.height, fg, bg);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: If the the size or position of the rectangle has changed, 
|                recalculate the size and position of the rectangle.
|
|         Input: current - the object containing current settings 
|                request - the object containing requested settings
|                new     - the object processed through all set values methods 
|
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwRectangleGadget cobj = (XvwRectangleGadget) current;
	XvwRectangleGadget nobj = (XvwRectangleGadget) new;


	if (cobj->rectangle.x      != nobj->rectangle.x     ||
	    cobj->rectangle.y      != nobj->rectangle.y     ||
	    cobj->rectangle.width  != nobj->rectangle.width ||
	    cobj->rectangle.height != nobj->rectangle.height)
	{
	   GraphicsRecomputePosition(new, FALSE);
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: RecomputePosition
|
|       Purpose: Given a new world coordinate rectangle 
|                corner and diagonal, recompute the new 
|                device coordinate size and position of 
|		 the rectangle. 
|
|         Input: widget - the rectangle object 
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RecomputePosition(
   Widget widget)
{
	XvwRectangleGadget xobj = (XvwRectangleGadget) widget;

	XVGraphicsID id = xobj->graphics.id;
	Real xpos1, xpos2, ypos1, ypos2;
	Coord coord;

	kmemset((kaddr) &coord, 0, sizeof(Coord));
	coord.x = xobj->rectangle.x;
	coord.y = xobj->rectangle.y;
	X2D_convert_point_wc_to_dc(id, coord, &xpos1, &ypos1);

	coord.x = xobj->rectangle.x + xobj->rectangle.width;
	coord.y = xobj->rectangle.y + xobj->rectangle.height;
	X2D_convert_point_wc_to_dc(id, coord, &xpos2,&ypos2);

	widget->core.x      = (Position)  kmin(xpos1, xpos2);
	widget->core.y      = (Position)  kmin(ypos1, ypos2);
	widget->core.width  = (Dimension) kabs(xpos1 - xpos2);
	widget->core.height = (Dimension) kabs(ypos1 - ypos2);

	xvw_set_attributes(xvw_object(widget),
                           XVW_PREFERRED_WIDTH,  widget->core.width,
                           XVW_PREFERRED_HEIGHT, widget->core.height,
                           NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: ChangeGeometry
|
|       Purpose: Given a device coordinate description of the 
|		 rectangle, recompute the world coordinate 
|                corner and diagonal point.
|
|         Input: widget - the rectangle object 
|    Written By: Mark Young, Danielle Argiro, Steve Kubica
|          Date: Jan 5, 1996
| Modifications: 
|
------------------------------------------------------------*/
#define UPPER_LEFT   1
#define UPPER_RIGHT  2
#define LOWER_LEFT   3
#define LOWER_RIGHT  4

/* ARGSUSED */
static void ChangeGeometry(
   Widget widget)
{
	XvwRectangleGadget xobj = (XvwRectangleGadget) widget;
	XVGraphicsID id = xobj->graphics.id;
	Coord    coord1, coord2;
	Real     xpos1, ypos1, xpos2, ypos2;

	kmemset((kaddr) &coord1, 0, sizeof(Coord));
	kmemset((kaddr) &coord2, 0, sizeof(Coord));

	xpos1 = (Real) widget->core.x;
        ypos1 = (Real) widget->core.y;
	X2D_convert_point_dc_to_wc(id, xpos1, ypos1, &coord1);

        xpos2 = (Real) widget->core.x + (Real) widget->core.width;
        ypos2 = (Real) widget->core.y + (Real) widget->core.height;
	X2D_convert_point_dc_to_wc(id, xpos2, ypos2, &coord2);

	xobj->rectangle.x      = kmin(coord1.x, coord2.x);
	xobj->rectangle.y      = kmin(coord1.y, coord2.y);
	xobj->rectangle.width  = kabs(coord1.x - coord2.x);
	xobj->rectangle.height = kabs(coord1.y - coord2.y);
}

/*-----------------------------------------------------------
|
|  Routine Name: PickGadget
|
|       Purpose: Given a rectangle object and an attempted picking location,
|                see if the rectangle has been picked.  Note whether the
|                rectangle was picked on a corner marker (implying a resize)
|                or on the rectangle radius (implying a move).
|
|         Input: widget - the rectangle object we are checking
|                x      - the x device coordinate that the pick attempt was at
|                y      - the y device coordinate that the pick attempt was at
|
|       Returns: TRUE (1) if object was successfully picked, FALSE (0) otherwise
|    Written By: Mark Young, Danielle Argiro, Steve Kubica
|          Date: Jan 5, 1995
| Modifications:
|
------------------------------------------------------------*/

static int PickGadget(
   Widget widget,
   int    x,
   int    y)
{
        XvwRectangleGadget xobj = (XvwRectangleGadget) widget;

        /*
         *  rectangle was picked on one of its corners markers
         */
        if (xvw_pick_cornermarkers(widget, x, y))
            xobj->graphics.pick_status = PICK_RESIZE;

        /*
         *  rectangle was picked on one of its side markers
         */
        else if (xvw_pick_sidemarkers(widget, x, y))
            xobj->graphics.pick_status = PICK_RESIZE;

	/*
	 *  rectangle picked for moving
	 */
	else xobj->graphics.pick_status = PICK_MOVE;

        return(TRUE);
}


/*-----------------------------------------------------------
|
|  Routine Name: EraseSelection
|
|       Purpose: Erase the side and corner markers on the rectangle
|		 which had indicated that the rectangle had been selected
|
|         Input: widget - the rectangle whose corner markers we are erasing
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 5, 1996
| Modifications:
|
------------------------------------------------------------*/

/* ARGSUSED */
static void EraseSelection(
    Widget widget)
{
	XvwGraphicsGadgetClassRec *graphics_class;

        /*
         * call utility routine to refresh the side markers around rectangle
         */
        xvw_erase_sidemarkers(widget);

	/* 
	 * call the EraseSelection method of the Graphics Class
	 * to erase the corner markers around the rectangle
	 */
	graphics_class = (XvwGraphicsGadgetClassRec *) 
		         (xvwRectangleGadgetClass->core_class.superclass);
	(*graphics_class->manager_class.erase_selection)(widget);
}

/*-----------------------------------------------------------
|
|  Routine Name: RefreshSelection
|
|       Purpose: Draws little side and corner markers on the rectangle
|                to indicate that the rectangle has been selected
|
|         Input: widget - rectangle whose side & corner markers we are refreshing
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 5, 1996
| Modifications:
|
------------------------------------------------------------*/

/* ARGSUSED */
static void RefreshSelection(
    Widget widget)
{
        XvwRectangleGadget xwid = (XvwRectangleGadget) widget;
	XvwGraphicsGadgetClassRec *graphics_class;
	GC gc = xwid->manager.foreground_gc;

	/*
	 * call utility routine to refresh the side markers around rectangle
	 */
	xvw_draw_sidemarkers(widget, gc);

	/* 
	 * call the RefreshSelection method of the Graphics Class
	 * to refresh the corner markers around the rectangle
	 */
	graphics_class = (XvwGraphicsGadgetClassRec *) 
		         (xvwRectangleGadgetClass->core_class.superclass);
	(*graphics_class->manager_class.refresh_selection)(widget);
}


/************************************************************
*
*  Routine Name: xvw_create_rectangle - creates a rectangle object
*
*       Purpose: A rectangle visual object supports the display of a rectangle.
*
*                The (x, y) location of the upper left hand corner of the
*                rectangle may be specified in world coordinates, where the
*                world coordinate range is dictated by the "controlling" visual
*                object. By default, the parent of the rectangle object is its
*                controlling visual object; this may be changed using the
*                XVW_GRAPHICS_ATTACH attribute.
*
*                Alternatively, the size and location of the rectangle may be
*                specified using device coordinates. As when using world coordinates,
*                specify the (x,y) location of the upper left hand corner of 
*		 the rectangle, as well as the width and height of the rectangle.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the rectangle
*                         object (for use in app-defaults files, etc)
*        Output: 
*	Returns: The rectangle object on success, NULL on failure
*  Restrictions:
*    Written By: Mark Young 
*          Date: Jun 29, 1992 
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_rectangle(
   xvobject parent,
   char     *name)
{
	xvobject  object;


	/*
	 *  Create the rectangle object
	 */
	object = xvw_create(parent, FALSE, TRUE, name, RectangleGadgetClass);
	return(object);
}
