/*
  * Khoros: $Id: Threshold.c,v 1.16 1996/06/25 16:33:40 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Threshold.c,v 1.16 1996/06/25 16:33:40 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Threshold Widget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Redisplay()
   >>>>                 SetValues()
   >>>>                 ValueCallback()
   >>>>                 UpdateValue()
   >>>>                 ColorReloadCallback()
   >>>>                 ThresholdSetUtil()
   >>>>   Public:
   >>>>                 xvw_create_threshold()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/ThresholdP.h>
#include <xvisual/Palette.h>

static int last_range = 0;

static void ClassInitialize PROTO((void));
static void Initialize	    PROTO((Widget, Widget, ArgList, Cardinal *));
static void Redisplay       PROTO((Widget, XEvent *, Region));
static Boolean SetValues    PROTO((Widget, Widget, Widget, ArgList,
				   Cardinal *));

static void ValueCallback       PROTO((xvobject, kaddr, kaddr));
static void UpdateValue         PROTO((Widget, xvobject, double));
static void ColorReloadCallback PROTO((xvobject, kaddr));
static int  ThresholdSetUtil    PROTO((xvobject, char *, kaddr));


#undef kwidget
#undef kwidgetclass
#undef kconstraint

#define kwidget(widget)      (XvwThresholdWidget) (widget)
#define kwidgetclass(widget) (XvwThresholdWidgetClass) (widget)->core.widget_class
#define kconstraint(widget)  (XvwThresholdWidgetConstraints) (widget)->core.constraints


/*-------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_THRESHOLD_SHOW_PALETTE,   NULL, XtRInt,      XtRBoolean},
{XVW_THRESHOLD_POLICY,	       NULL, XtRInt,      NULL},
{XVW_THRESHOLD_LOWERVAL,       NULL, XtRDouble,   NULL},
{XVW_THRESHOLD_UPPERVAL,       NULL, XtRDouble,   NULL},
{XVW_THRESHOLD_PIXELVAL,       NULL, XtRPixel,    NULL},
{XVW_THRESHOLD_CLIP_PIXELVAL,  NULL, XtRPixel,    NULL},
{XVW_THRESHOLD_INVERT,	       NULL, XtRInt,      XtRBoolean},
{XVW_THRESHOLD_CALLBACK,       NULL, XtRCallback, NULL},
{XVW_THRESHOLD_PALETTE_OBJECT, NULL, XtRPointer,  NULL},
};

/*-------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwThresholdWidgetRec, threshold.field)

static XtResource resources[] = { 
{XVW_THRESHOLD_SHOW_PALETTE, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_palette), XtRImmediate, (XtPointer) TRUE},
{XVW_THRESHOLD_POLICY, NULL, XtRInt, sizeof(int),
      offset(policy), XtRImmediate, (XtPointer) KTHRESHOLD_POLICY_CLIP},
{XVW_THRESHOLD_LOWERVAL, NULL, XtRDouble, sizeof(double),
      offset(lowerval), XtRString, (XtPointer) "0.0"},
{XVW_THRESHOLD_UPPERVAL, NULL, XtRDouble, sizeof(double),
      offset(upperval), XtRString, (XtPointer) "0.0"},
{XVW_THRESHOLD_CLIP_PIXELVAL, NULL, XtRPixel, sizeof(Pixel),
      offset(clip_pixelval), XtRString, (XtPointer) "#000000"},
{XVW_THRESHOLD_PIXELVAL, NULL, XtRPixel, sizeof(Pixel),
      offset(thresh_pixelval), XtRString, (XtPointer) "#ffffff"},
{XVW_THRESHOLD_INVERT, NULL, XtRBoolean, sizeof(Boolean),
      offset(invert), XtRImmediate, (XtPointer) FALSE},
{XVW_THRESHOLD_CALLBACK, NULL, XtRCallback, sizeof(XtPointer),
      offset(threshold_callbacks), XtRCallback, (XtPointer) NULL},
{XVW_THRESHOLD_PALETTE_OBJECT, NULL, XtRObject, sizeof(xvobject),
        offset(palette), XtRImmediate, (XtPointer) NULL},

{XVW_SHADOW_TYPE, NULL, XtRInt, sizeof(int),
        XtOffsetOf(XvwThresholdWidgetRec, manager.shadow_type),
        XtRImmediate, (XtPointer) KSHADOW_OUT},
{XVW_PREFERRED_WIDTH, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwManagerWidgetRec, manager.preferred_width),
        XtRImmediate, (XtPointer) 500},
{XVW_PREFERRED_HEIGHT, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwManagerWidgetRec, manager.preferred_height),
        XtRImmediate, (XtPointer) 175},

{NULL, NULL, XtRPixel, sizeof(Pixel),
      offset(black), XtRString, (XtPointer) "#000000"},
{NULL, NULL, XtRBoolean, sizeof(Boolean),
      XtOffsetOf(XvwColorWidgetRec, color.initialize_visual),
      XtRImmediate, (XtPointer) FALSE},
};
#undef offset


#define SUPERCLASS (&xvwColorWidgetClassRec)

XvwThresholdWidgetClassRec xvwThresholdWidgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Threshold",				/* class_name		  */
    sizeof(XvwThresholdWidgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    knumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    NULL,				/* destroy		  */
    NULL,				/* resize		  */
    Redisplay,	  		        /* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    XtInheritTranslations,		/* tm_table		  */
    NULL,				/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,	            		/* subresources       */
    0,					/* subresources_count */
    sizeof(XvwManagerWidgetConstraintsRec),/* constraint_size    */
    NULL,				/* initialize         */
    NULL,                               /* destroy            */
    NULL,				/* set_values         */
    NULL,                               /* extension          */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			/* pre-layout routine     */
    XtInheritLayout,			/* post-layout routine    */
    XtInheritChangeSel,		        /* change selected proc   */
    XtInheritEraseSel,			/* erase selected proc    */
    XtInheritRefreshSel,		/* refresh selection proc */
    XtInheritResize,			/* resize		  */
    XtInheritGeometryManager,  		/* geometry_manager	  */
  }, /* XvwManagerWidgetClass fields initialization */
  {
    NULL,                                     /* field not used    */
  },  /* XvwGraphicsWidgetClass fields initialization */
  {
    ColorReloadCallback,                /* reload color proc    */
  },  /* XvwColorWidgetClass fields initialization */
  {
    NULL,                               /* field not used    */
  },  /* XvwThresholdWidgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwThresholdWidgetClass for public consumption
 */
WidgetClass xvwThresholdWidgetClass = (WidgetClass) &xvwThresholdWidgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of threshold widget class has been created.
|                It will initialize all the class attributes.
|
|    Written By: Mark Young 
|          Date: Nov 17, 1992
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
        xvw_init_attributes(xvwThresholdWidgetClass, attributes,
		      (int) knumber(attributes), NULL, 0, 
		      "$DESIGN/objects/library/xvisual/uis/Threshold.pane");
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/Threshold");

	 xvw_define_attributes(xvwThresholdWidgetClass,
                XVW_THRESHOLD_RESET, XtRInt,  ThresholdSetUtil, NULL,
		NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|	Purpose: This method will set up the initial threshold color
|                widget instance.
|
|	  Input: request - not used
|                new     - widget instance after initialization
|    Written By: Mark Young
|          Date: May 26, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwThresholdWidget xwid = kwidget(new);
	xvobject object = xvw_object(new);
	xvobject range_text_object, range_crlabel_object;

	XColor color;

	/*
	 *  Create the palette and threshold color scrollbars
	 */
	xwid->threshold.palette = xvw_create_palette(object, "palette");
	if (!xwid->threshold.show_palette)
	   xvw_unmanage(xwid->threshold.palette);

	xwid->threshold.lower = xvw_create_double(object, "lower");
	xwid->threshold.range = xvw_create_double(object, "range");
	xwid->threshold.upper = xvw_create_double(object, "upper");
	xwid->threshold.initialized = FALSE;

	/*
	 *  Initialize the palette to extend the width of the palette
	 */
	xvw_set_attributes(xwid->threshold.palette,
		XVW_ABOVE,	xwid->threshold.lower,
		XVW_BELOW,	NULL,
		XVW_TACK_EDGE,	KMANAGER_TACK_ALL,
		NULL);

        /*
         *  Make sure the scrollbars extends the height of the
         *  threshold widget.
         */
        xvw_set_attributes(xwid->threshold.lower,
		XVW_ABOVE,	      xwid->threshold.upper,
                XVW_RIGHT_OF,         NULL,
                XVW_LEFT_OF,          NULL,
		XVW_TACK_EDGE,	      KMANAGER_TACK_HORIZ,
		XVW_CHAR_HEIGHT,      1.0,
                XVW_DOUBLE_LABEL,     "Lower",
                XVW_DOUBLE_MINVALUE,  0.0,
                XVW_DOUBLE_VALUE,     0.0,
                XVW_DOUBLE_MAXVALUE,  100.0,
		NULL);

        xvw_set_attributes(xwid->threshold.upper,
		XVW_ABOVE,	      xwid->threshold.range,
                XVW_RIGHT_OF,         NULL,
                XVW_LEFT_OF,          NULL,
                XVW_TACK_EDGE,        KMANAGER_TACK_HORIZ,
		XVW_CHAR_HEIGHT,      1.0,
                XVW_DOUBLE_LABEL,     "Upper",
                XVW_DOUBLE_MINVALUE,  0.0,
                XVW_DOUBLE_VALUE,     100.0,
                XVW_DOUBLE_MAXVALUE,  100.0,
		NULL);

        xvw_set_attributes(xwid->threshold.range,
		XVW_ABOVE,	      NULL,
                XVW_RIGHT_OF,         NULL,
                XVW_LEFT_OF,          NULL,
                XVW_TACK_EDGE,        KMANAGER_TACK_HORIZ,
                XVW_CHAR_HEIGHT,      1.0,
                XVW_DOUBLE_LABEL,    "Range",
                XVW_DOUBLE_MINVALUE,  0.0,
                XVW_DOUBLE_VALUE,     0.0,
                XVW_DOUBLE_MAXVALUE,  100.0,
		NULL);


	/*
	 *  Add the callbacks for the red, green, and blue scrollbars that
	 *  will be used to change the color palette.
	 */
	xvw_insert_callback(xwid->threshold.lower, XVW_DOUBLE_CALLBACK,
			 FALSE, ValueCallback, (XtPointer) xwid);
	xvw_insert_callback(xwid->threshold.range, XVW_DOUBLE_CALLBACK,
			 FALSE, ValueCallback, (XtPointer) xwid);
	xvw_insert_callback(xwid->threshold.upper, XVW_DOUBLE_CALLBACK,
			 FALSE, ValueCallback, (XtPointer) xwid);

	/*
	 * display makes more sense if double value for range isn't displayed.
	 */
	xvw_get_attributes(xwid->threshold.range,
			   XVW_DOUBLE_TEXT_OBJECT,    &range_text_object,
		           XVW_DOUBLE_CRLABEL_OBJECT, &range_crlabel_object,
			   NULL);
	xvw_unmap(range_text_object);
	xvw_unmap(range_crlabel_object);

	/*
	 *  Initialize the clip and threshold pixel values.
	 */
        color.flags = DoRed | DoGreen | DoBlue;
 
        color.red = color.green = color.blue = 0;
        if (XAllocColor(XtDisplay(new), new->core.colormap, &color) != 0)
           xwid->threshold.clip_pixelval = color.pixel;
 
        color.red = color.green = color.blue = 65535;
        if (XAllocColor(XtDisplay(new), new->core.colormap, &color) != 0)
           xwid->threshold.thresh_pixelval = color.pixel;

	/* allocate black specially */
	color.red = color.green = color.blue = 0;
	if (XAllocColor(XtDisplay(new), new->core.colormap, &color) != 0)
	   xwid->threshold.black = color.pixel;

}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This method is needed to make sure that the
|		 min & max values are set on the scrollbars.
|
|         Input: widget - widget data structure for palette
|                event  - the event that caused the redraw
|                region - the region that was exposed
|    Written By: Danielle Argiro
|          Date: May 26, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	XvwThresholdWidget xwid = kwidget(widget);

	int    type;
	double minval, maxval, increment;

        /*
         *  Check to see if we need to reload and/or resize the threshold
         */
        if (!XtWindow(widget) || !xvw_check_visible(xvw_object(widget)))
           return;

	if (!xwid->threshold.initialized)
	{
	    minval = xwid->color.info->minval;
	    maxval = xwid->color.info->maxval;
	    kpds_get_attribute(xwid->color.object, KPDS_VALUE_DATA_TYPE, &type);

	    increment = (maxval - minval)/10;
	    if (type != KFLOAT && type != KDOUBLE &&
	        type != KCOMPLEX && type != KDCOMPLEX)
	    {
	       increment = (long) increment;
	    }
	    xvw_set_attributes(xwid->threshold.range,
			   XVW_DOUBLE_MINVALUE,  minval,
			   XVW_DOUBLE_VALUE,     minval,
			   XVW_DOUBLE_MAXVALUE,  maxval,
			   XVW_DOUBLE_INCREMENT, increment,
		           NULL);
	    xwid->threshold.lowerval = minval;
	    xvw_set_attributes(xwid->threshold.lower,
			   XVW_DOUBLE_MINVALUE,  minval,
			   XVW_DOUBLE_VALUE,     minval,
			   XVW_DOUBLE_MAXVALUE,  maxval,
			   XVW_DOUBLE_INCREMENT, increment,
		           NULL);
	    xwid->threshold.upperval = maxval;
  	    xvw_set_attributes(xwid->threshold.upper,
			   XVW_DOUBLE_MINVALUE,  minval,
			   XVW_DOUBLE_VALUE,     maxval,
			   XVW_DOUBLE_MAXVALUE,  maxval,
			   XVW_DOUBLE_INCREMENT, increment,
		           NULL);
	    xwid->threshold.initialized = TRUE;
	    UpdateValue(widget, NULL, 0.0);
	}
        (*xvwThresholdWidgetClass->core_class.superclass->core_class.expose)
                (widget, event, region);
}


/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: Determine what the widget should do by
|		 comparing the current values verse the
|		 new settings.
|
|         Input: current - the widget containing current settings 
|		 request - the widget containing requested settings
|		 new	 - the widget processed through all set values methods 
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 17, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	int redisplay = FALSE;
	XvwThresholdWidget cxwid = kwidget(current);
	XvwThresholdWidget nxwid = kwidget(new);

	if (cxwid->color.object != nxwid->color.object)
	{
	   xvw_set_attribute(nxwid->threshold.palette, XVW_COLOR_COLOROBJ,
			nxwid->color.object);
	   nxwid->threshold.initialized = FALSE;
	   redisplay = TRUE;
	}

	if (cxwid->threshold.policy != nxwid->threshold.policy ||
	    cxwid->threshold.invert != nxwid->threshold.invert)
	{
	   UpdateValue(new, NULL, 0.0);
	}

	if (cxwid->threshold.lowerval != nxwid->threshold.lowerval)
	   UpdateValue(new, nxwid->threshold.lower, nxwid->threshold.lowerval);
	if (cxwid->threshold.upperval != nxwid->threshold.upperval)
	   UpdateValue(new, nxwid->threshold.upper, nxwid->threshold.upperval);

	if (cxwid->threshold.show_palette != nxwid->threshold.show_palette)
	{
	   if (!nxwid->threshold.show_palette)
	      xvw_unmanage(nxwid->threshold.palette);
	   else
	      xvw_manage(nxwid->threshold.palette);
	}

	if (cxwid->threshold.thresh_pixelval != nxwid->threshold.thresh_pixelval ||
	    cxwid->threshold.clip_pixelval != nxwid->threshold.clip_pixelval) 
	{
	    UpdateValue(new, nxwid->threshold.upper, nxwid->threshold.upperval);
	    redisplay = TRUE;
	}
	return(redisplay);
}

/*-----------------------------------------------------------
|
|  Routine Name: ValueCallback
|
|       Purpose: This routine will change the (L, U, or R) component
|	         according to the value in the double object.
|
|         Input: object      - the double object 
|		 client_data - the threshold widget
|		 call_data   - double value of double object
|    Written By: Mark Young & Danielle Argiro
|          Date: Jun 06, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ValueCallback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	XvwThresholdWidget xwid = kwidget(client_data);
	double value = *((double *) call_data);

	int type, pos;
	xvw_threshold_struct threshold;
	double upper, lower, range, diff, max;

	static int active = FALSE;


	/*
	 *  Make sure we aren't in the middle of callback...
	 */
	if (active)
	   return;
	else
	   active = TRUE;

	xvw_get_attribute(xwid->threshold.upper, XVW_DOUBLE_VALUE, &upper);
	xvw_get_attribute(xwid->threshold.lower, XVW_DOUBLE_VALUE, &lower);
	xvw_get_attribute(xwid->threshold.range, XVW_DOUBLE_VALUE, &range);

	/*
	 *  The lower bound has been changed.
	 */
	if (object == xwid->threshold.lower)
	{
	   pos   = 0;
	   type  = KTHRESHOLD_LOWER;

	   /* 
	    * can't allow lower bound value to exceed upper bound.
	    */
	   if (lower > upper)
	      xvw_set_attribute(xwid->threshold.upper, XVW_DOUBLE_VALUE, lower);
	   if (lower > range)
	      xvw_set_attribute(xwid->threshold.range, XVW_DOUBLE_VALUE, lower);
	}

	/*
	 *  The upper bound has been changed.
	 */
	else if (object == xwid->threshold.upper)
	{
	   pos   = 1;
	   type  = KTHRESHOLD_UPPER;

	   /* 
	    * can't allow upper bound value to get below lower bound.
	    */
	   if (upper < lower)
	      xvw_set_attribute(xwid->threshold.lower, XVW_DOUBLE_VALUE, upper);
	   if (upper < range)
	      xvw_set_attribute(xwid->threshold.range, XVW_DOUBLE_VALUE, upper);
	}

	/*
	 *  The range has been changed.
	 */
	else if (object == xwid->threshold.range)
	{
	   pos   = 2;
	   type  = KTHRESHOLD_RANGE;

	   range = value;
           xvw_get_attribute(xwid->threshold.upper, XVW_DOUBLE_MAXVALUE, &max);

	   /*
	    * moving range scrollbar to the right past lower scrollbar
	    */
	   if ((range > last_range) && (range > lower)) 
	   {
	      /*
	       * need to scoot both lower & upper scrollbars up but maintain
	       * relative distance between them.
	       */
	      diff = range - last_range;
	      if ((upper + diff) <= max)
	      {
	          xvw_set_attribute(xwid->threshold.lower, XVW_DOUBLE_VALUE, 
				    lower + diff);
	          xvw_set_attribute(xwid->threshold.upper, XVW_DOUBLE_VALUE, 
				    upper + diff);
	      }

	      /*	 
	       * move lower & upper scrollbars up as far as they can go
	       * (but still maintain relative distance)
	       */
	      else 
	      {
	 	  xvw_set_attribute(xwid->threshold.lower, XVW_DOUBLE_VALUE,
                                    max - (upper - lower));
                  xvw_set_attribute(xwid->threshold.upper, XVW_DOUBLE_VALUE,
                                    max);
	      }
	   }

	   /*
	    * moving range scrollbar to left past upper scrollbar
	    */
	   else if ((range < last_range) && (range < upper)) 
           {
              /*
               * need to scoot both lower & upper scrollbars back but maintain
               * relative distance between them.
               */
              diff = last_range - range;
              if ((lower - diff) >= 0)
              {
                  xvw_set_attribute(xwid->threshold.lower, XVW_DOUBLE_VALUE,
                                    lower - diff);
                  xvw_set_attribute(xwid->threshold.upper, XVW_DOUBLE_VALUE,
                                    upper - diff);
              }
              /*         
               * move lower & upper scrollbars back as far as they can go
               * (but still maintain relative distance)
               */
              else
              {
                  xvw_set_attribute(xwid->threshold.lower,XVW_DOUBLE_VALUE,0.0);
                  xvw_set_attribute(xwid->threshold.upper, XVW_DOUBLE_VALUE,
                                    (upper - lower));
              }
           }
	}
	else return;

	if (xwid->threshold.lowerval != lower ||
	    xwid->threshold.upperval != upper) 
	{
	   xwid->threshold.lowerval = lower;
	   xwid->threshold.upperval = upper;
	   UpdateValue((Widget) xwid, NULL, value);
	}
	threshold.type  = type;
	threshold.value = (int) value;
	XtCallCallbackList((Widget) xwid, xwid->threshold.threshold_callbacks,
			(XtPointer) &threshold);

	active = FALSE;
	last_range = range;
}

/*-----------------------------------------------------------
|
|  Routine Name: UpdateValue
|
|       Purpose: This routine will update the value.
|
|         Input: widget      - the threshold widget
|		 object      - the double object 
|		 value       - the double value
|    Written By: Mark Young & Danielle Argiro
|          Date: Jan 04, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void UpdateValue(
  Widget   widget,
  xvobject object,
  double   value)
{
        XvwThresholdWidget xwid = kwidget(widget);

	double factor;
	XColor *xcolors;
	Pixel  *pixels, pixel;
	double minval, maxval, lower, upper, val;
	int    i, indx, ncolors, invert, update = FALSE;

	if (object)
	   xvw_set_attribute(object, XVW_DOUBLE_VALUE, value);
	if (xwid->color.info == NULL)
	   return;

	pixels   = ColorGetPixels(widget);
	xcolors  = ColorGetXColors(widget);
	ncolors  = xwid->color.info->ncolors;
	minval   = xwid->color.info->minval;
	maxval   = xwid->color.info->maxval;

	lower  = xwid->threshold.lowerval;
	upper  = xwid->threshold.upperval;
	invert = xwid->threshold.invert;
	pixel  = xwid->threshold.black;

	factor = (maxval - minval)/ncolors;

	for (i = 0; i < ncolors; i++)
	{
	   if (xcolors[i].flags == 0) 
	      continue; 

	   /* 
	    * outside range and not inverting 
	    */
	   val = i*factor+minval;
	   if (!invert && (val < lower || val > upper))
	   {
	      /* use clip_pixelval for pixel clipping */
	      if (xwid->threshold.policy == KTHRESHOLD_POLICY_CLIP)
	         pixel = xwid->threshold.clip_pixelval;
	
	      /* use pixelval for pixel thresholding */
	      else if (xwid->threshold.policy == KTHRESHOLD_POLICY_THRESH)
	         pixel = xwid->threshold.black;

	      /* use min pixel for < range in windowing */
	      else if (val <= lower)
	      {
                 if (xcolors[0].flags == 0)
		     ColorAllocate(widget, 0);
		 pixel = xcolors[0].pixel;
	      }
	
	      /* use max pixel for > range in windowing */
	      else
	      {
                 if (xcolors[ncolors-1].flags == 0)
		     ColorAllocate(widget, (unsigned) (ncolors-1));
		 pixel = xcolors[ncolors-1].pixel;
	      }
	   }

	   /* 
	    * inside range and inverting 
	    */
	   else if (invert && (val >= lower && val <= upper))
	   {
	      /* pixel clipping */
	      if (xwid->threshold.policy == KTHRESHOLD_POLICY_CLIP)
	         pixel = xwid->threshold.clip_pixelval;
	      else if (xwid->threshold.policy == KTHRESHOLD_POLICY_THRESH)
	         pixel = xwid->threshold.black;
	   }

	   /*
	    *  outside range and inverting, or inside range and not inverting:
	    *  in either case, you color the range.
	    */
	      /* coloring range and doing clipping: take pixel directly */
	   else if (xwid->threshold.policy == KTHRESHOLD_POLICY_CLIP)
	      pixel = xcolors[i].pixel;

	      /* coloring range and doing thresholding: take pixelval */
	   else if (xwid->threshold.policy == KTHRESHOLD_POLICY_THRESH)
	      pixel = xwid->threshold.thresh_pixelval;

	      /* coloring range and doing windowing: shift into range */
	   else if (xwid->threshold.policy == KTHRESHOLD_POLICY_WINDOWED && !invert)
	   {
	      indx = (int) (((val - lower)/(upper-lower)) * (ncolors-1));
#if 0
kfprintf(kstderr, "indx = %d, ncolors = %d\n", indx, ncolors);
#endif
	      if (xcolors[indx].flags == 0)
		 ColorAllocate(widget, (unsigned) indx);
	      pixel = xcolors[indx].pixel;
	   }
	   else continue;

	   if (pixel != pixels[i])
	   {
	      pixels[i] = pixel;
	      update = TRUE;
	   }
	}

	if (update == TRUE)
	   ColorStore(widget, -1, NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: ColorReloadCallback
|
|       Purpose: Update the threshold since our pixels have changed
|
|         Input: object - the object to be refreshed (the Threshold)
|		 call_data   - (not used)
|    Written By: Mark Young
|          Date: May 11, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ColorReloadCallback(
   xvobject object,
   kaddr    call_data)
{
	Widget widget = xvw_widget(object);
	XvwThresholdWidget xwid = kwidget(widget);

	if (xwid->threshold.initialized)
	   UpdateValue(widget, NULL, 0.0);
}

/*-----------------------------------------------------------
|
|  Routine Name: ThresholdSetUtil
|
|       Purpose: ThresholdSetUtil is used to set miscellaneous 
|		 threshold attributes,  specifically :
|                               XVW_THRESHOLD_RESET
|
|         Input: object    - the threshold object 
|                attribute - the attribute
|                calldata  - the data associated with the action
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young & Danielle Argiro
|          Date: Jun 24, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static int ThresholdSetUtil(
        xvobject object,
        char     *attribute,
        kaddr    calldata)
{
	Widget       widget = (Widget) xvw_widget(object);
	XvwThresholdWidget xwid = (XvwThresholdWidget) xvw_widget(object);
	double       minval, maxval;

        if (kstrcmp(attribute, XVW_THRESHOLD_RESET) == 0)
        {
	    minval = xwid->color.info->minval;
            maxval = xwid->color.info->maxval;
	    xvw_set_attribute(xwid->threshold.range,
                              XVW_DOUBLE_VALUE, minval);
	    xvw_set_attribute(xwid->threshold.lower,
                              XVW_DOUBLE_VALUE, minval);
	    xvw_set_attribute(xwid->threshold.upper,
                              XVW_DOUBLE_VALUE, maxval);

	    xwid->threshold.lowerval = minval;
	    xwid->threshold.upperval = maxval;
	    UpdateValue(widget, NULL, 0.0);
        }
	return(TRUE);
}

/************************************************************
*
*  Routine Name: xvw_create_threshold - create a threshold object
*
*       Purpose: Creates an threshold object which will display
*		 the colormap values of an object's colormap.
*
*		 The threshold object is used by the user to mask out certain
*		 parts of the image so that they are only looking at desired
*		 range of pixels.  The threshold objects contains an
*		 upper and lower scrollbar from which we can extract
*		 the pixel range to be threshold.
*
*		 The threshold objects also contains the "pixel" that we
*		 are going to be setting the threshold values to, and
*		 whether we are to accept or reject that range.  The
*		 "accept" value is a boolean which tells us if we are
*		 to threshold everything outside the lower & upper
*		 range or whether we threshold everything inside that
*		 range.  (If accept is true then we threshold
*		 everything outside the range otherwise we threshold
*		 the pixels inside the range).
*
*		 The threshold works by setting the pixels in the
*		 threshold range to the current threshold color
*		 (specified by pixel).  And then storing these values
*		 in the xvdisplay colormap.  But in order to make sure
*		 none of the other displays over-ride this threshold we
*		 set the pixel's active boolean array to zero.  By
*		 setting the histogram to zero, none of the other
*		 displays will update those pixels with new values,
*		 until we restore the active states.
*
*                Note that thresholding should not be used with RGB images when
*                the calling application is displayed on an 8-bit screen.  
*                This is because a mapping transformation is used in order 
*                to convert the RGB image into an 8-bit representation for 
*                visual display on an 8-bit screen.
*
*		 Because of this transformation, thresholding an RGB image on 
*                an 8-bit screen will not make sense;  any changes to the 
*                colormap made via thresholding would alter the 24-bit to 
*                8-bit transformation (a 3-3-2 RGB map is used), rather than 
*                the RGB values themselves, producing an unexpected result.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the threshold
*                         object (for use in app-defaults files, etc)
*        Output:
*	Returns: The threshold object on success, NULL on failure
*  Restrictions:
*    Written By: Mark Young
*          Date: Nov 17, 1992
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_threshold(
   xvobject parent,
   char     *name)
{
	xvobject object;


	/*
	 *  Call the xvw_create() routine to do the actual creation.
	 */
	object = xvw_create(parent, FALSE, TRUE, name, ThresholdWidgetClass);
	return(object);
}
