 /*
  * Khoros: $Id: CommandBar.c,v 1.31 1996/07/02 16:23:36 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: CommandBar.c,v 1.31 1996/07/02 16:23:36 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

#include "copyright.h"          /* Khoros copyright */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>	              Khoros CommandBar Widget
   >>>>  Private:
   >>>>   Static:
   >>>> 		ClassInitialize()
   >>>>			Initialize()
   >>>> 		SetValues()
   >>>> 		CommandbarAnnounce()
   >>>> 		CommandbarCallbacks()
   >>>> 		WorkspaceActiveCallback()
   >>>> 		WorkspaceRunCallback()
   >>>>   Public:
   >>>>			xvw_create_commandbar()
   >>>>	
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
#include <xvlang/CommandBarP.h>

static Boolean	SetValues            PROTO((Widget, Widget, Widget, ArgList,
				            Cardinal *));
static void ClassInitialize          PROTO((void));
static void Initialize		     PROTO((Widget, Widget,ArgList,Cardinal *));
static void CommandbarUpdate         PROTO((Widget));
static void UpdateWorkspaceRunStop   PROTO((Widget));
static void CommandbarAnnounce       PROTO((xvobject, kaddr, XEvent *, int *));
static void CommandbarCallbacks      PROTO((xvobject, kaddr, kaddr));
static void WorkspaceRunCallback     PROTO((xvobject, kaddr, kaddr));
static void WorkspaceActiveCallback  PROTO((xvobject, kaddr, kaddr));

/*-------------------------------------------------------------------*
|
|   Define Default Pixmap Paths
|
--------------------------------------------------------------------*/

#define LARGE_ICONDIR "$DESIGN/objects/library/xvlang/misc/commandbar/large"
#define SMALL_ICONDIR "$DESIGN/objects/library/xvlang/misc/commandbar/small"

typedef struct
{
        xvobject button;
        char *name;
        char *unset_pixmapfile;
        char *set_pixmapfile;
        char *attribute;
        char *show_attribute;
        char *announce;
} CommandBarItem;

static CommandBarItem commandbar_list[] = 
{
{NULL, "Run", "run.xpm", "running.xpm", XVW_WORKSPACE_RUN,
   XVW_COMMANDBAR_SHOW_RUN, "Run Workspace"},
{NULL, "Stop", "stop.xpm", NULL, XVW_WORKSPACE_STOP,
   XVW_COMMANDBAR_SHOW_STOP, "Stop Running Workspace"},
{NULL, "RunMode", "runonce.xpm", "runconti.xpm", XVW_WORKSPACE_RUNMODE,
   XVW_COMMANDBAR_SHOW_RUNMODE, "Run Mode"},
{NULL, "Reset", "reset.xpm", NULL, XVW_WORKSPACE_RESET,
   XVW_COMMANDBAR_SHOW_RESET, "Reset Workspace"},
{NULL, "Clear", "clear.xpm", NULL, XVW_WORKSPACE_CLEAR,
   XVW_COMMANDBAR_SHOW_CLEAR, "Clear Workspace"},
{NULL, "Remote", "distroff.xpm", "distron.xpm", XVW_NODE_REMOTE_ENABLE,
   XVW_COMMANDBAR_SHOW_REMOTE, "Enable Remote Hosts"},
{NULL, "Check", "check.xpm", NULL, XVW_WORKSPACE_CHECK,
   XVW_COMMANDBAR_SHOW_CHECK, "Check Workspace"},
{NULL, "Redraw", "redraw.xpm", NULL, XVW_WORKSPACE_REDRAW,
   XVW_COMMANDBAR_SHOW_REDRAW, "Redraw Workspace"},
{NULL, "Info", "info.xpm", NULL, XVW_WORKSPACE_INFO,
   XVW_COMMANDBAR_SHOW_INFO, "Workspace Information"},
{NULL, "CreateProcedure", "proc.xpm", NULL, XVW_WORKSPACE_CREATE_PROCEDURE,
   XVW_COMMANDBAR_SHOW_PROCEDURE, "Create Procedure"},
{NULL, "CreateCountloop", "count.xpm", NULL, XVW_WORKSPACE_CREATE_COUNTLOOP,
   XVW_COMMANDBAR_SHOW_COUNTLOOP, "Create Count Loop"},
{NULL, "CreateWhileloop", "while.xpm", NULL, XVW_WORKSPACE_CREATE_WHILELOOP,
   XVW_COMMANDBAR_SHOW_WHILELOOP, "Create While Loop"},
{NULL, "ShowClipboard", "showclip.xpm", NULL, XVW_WORKSPACE_SHOW_CLIPBOARD,
   XVW_COMMANDBAR_SHOW_CLIPBOARD, "Show Workspace Clipboard"},
{NULL, "Cut", "cut.xpm", NULL, XVW_CANVAS_CUT,
   XVW_COMMANDBAR_SHOW_CUT, "Cut Selected Glyphs to Clipboard"},
{NULL, "Copy", "copy.xpm", NULL, XVW_CANVAS_COPY,
   XVW_COMMANDBAR_SHOW_COPY, "Copy Selected Glyphs to Clipboard"},
{NULL, "Paste", "paste.xpm", NULL, XVW_CANVAS_PASTE,
   XVW_COMMANDBAR_SHOW_PASTE, "Paste Clipboard to Workspace"},
{NULL, "Duplicate", "dupe.xpm", NULL, XVW_CANVAS_DUPLICATE,
   XVW_COMMANDBAR_SHOW_DUPLICATE, "Duplicate Selected Glyphs"},
{NULL, "Delete", "delete.xpm", NULL, XVW_CANVAS_DELETE,
   XVW_COMMANDBAR_SHOW_DELETE, "Delete Selected Glyphs"},
{NULL, "Undo", "undelete.xpm", NULL, XVW_CANVAS_UNDO,
   XVW_COMMANDBAR_SHOW_UNDO, "Undo Deletion"},
{NULL, "SelectAll", "select.xpm", NULL, XVW_CANVAS_SELECTALL,
   XVW_COMMANDBAR_SHOW_SELECTALL, "Select All Glyphs"},
{NULL, "UnselectAll", "unselect.xpm", NULL, XVW_CANVAS_UNSELECTALL,
   XVW_COMMANDBAR_SHOW_UNSELECTALL, "Unselect All Glyphs"},
{NULL, "OpenWksp", "open.xpm", NULL, XVW_WORKSPACE_RESTORE,
   XVW_COMMANDBAR_SHOW_OPEN, "Open Workspace"},
{NULL, "SaveWksp", "save.xpm", NULL, XVW_WORKSPACE_SAVEALL,
   XVW_COMMANDBAR_SHOW_SAVE, "Save Workspace"},
{NULL, "SaveWkspAs", "saveas.xpm", NULL, XVW_WORKSPACE_SAVEALL,
   XVW_COMMANDBAR_SHOW_SAVEAS, "Save Workspace As New File"},
};


/*-------------------------------------------------------------------*
|
|   Full class attributes
|
--------------------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_COMMANDBAR_WORKSPACE,      NULL,  XtRObject, NULL},
{XVW_COMMANDBAR_LARGEICONS,     NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_RUN,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_RUNMODE,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_STOP,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_RESET,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_CLEAR,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_CHECK,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_REMOTE,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_REDRAW,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_INFO,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_PROCEDURE,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_COUNTLOOP,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_WHILELOOP,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_CLIPBOARD,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_CUT,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_COPY,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_PASTE,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_DUPLICATE,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_DELETE,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_UNDO,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_SELECTALL,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_UNSELECTALL,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_OPEN,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_SAVE,   NULL,  XtRInt,    XtRBoolean},
{XVW_COMMANDBAR_SHOW_SAVEAS,   NULL,  XtRInt,    XtRBoolean},
};


/*-------------------------------------------------------------------*
|
|   Full class record constant
|
--------------------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwCommandBarWidgetRec, commandbar.field)

static XtResource resources[] = { 
{XVW_COMMANDBAR_WORKSPACE, NULL, XtRObject, sizeof(xvobject),
      offset(workspace), XtRImmediate, (XtPointer) NULL},
{XVW_COMMANDBAR_SHOW_RUN, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_run), XtRImmediate, (XtPointer) TRUE},
{XVW_COMMANDBAR_SHOW_RUNMODE, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_runmode), XtRImmediate, (XtPointer) TRUE},
{XVW_COMMANDBAR_SHOW_STOP, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_stop), XtRImmediate, (XtPointer) TRUE},
{XVW_COMMANDBAR_SHOW_RESET, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_reset), XtRImmediate, (XtPointer) TRUE},
{XVW_COMMANDBAR_SHOW_REMOTE, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_remote), XtRImmediate, (XtPointer) FALSE},
{XVW_COMMANDBAR_SHOW_CLEAR, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_clear), XtRImmediate, (XtPointer) TRUE},
{XVW_COMMANDBAR_SHOW_CHECK, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_check), XtRImmediate, (XtPointer) FALSE},
{XVW_COMMANDBAR_SHOW_REDRAW, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_redraw), XtRImmediate, (XtPointer) FALSE},
{XVW_COMMANDBAR_SHOW_INFO, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_info), XtRImmediate, (XtPointer) FALSE},
{XVW_COMMANDBAR_SHOW_PROCEDURE, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_procedure), XtRImmediate, (XtPointer) TRUE},
{XVW_COMMANDBAR_SHOW_COUNTLOOP, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_countloop), XtRImmediate, (XtPointer) FALSE},
{XVW_COMMANDBAR_SHOW_WHILELOOP, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_whileloop), XtRImmediate, (XtPointer) FALSE},
{XVW_COMMANDBAR_SHOW_CLIPBOARD, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_clipboard), XtRImmediate, (XtPointer) FALSE},
{XVW_COMMANDBAR_SHOW_CUT, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_cut), XtRImmediate, (XtPointer) FALSE},
{XVW_COMMANDBAR_SHOW_COPY, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_copy), XtRImmediate, (XtPointer) FALSE},
{XVW_COMMANDBAR_SHOW_PASTE, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_paste), XtRImmediate, (XtPointer) FALSE},
{XVW_COMMANDBAR_SHOW_DUPLICATE, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_duplicate), XtRImmediate, (XtPointer) TRUE},
{XVW_COMMANDBAR_SHOW_DELETE, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_delete), XtRImmediate, (XtPointer) TRUE},
{XVW_COMMANDBAR_SHOW_UNDO, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_undo), XtRImmediate, (XtPointer) TRUE},
{XVW_COMMANDBAR_SHOW_SELECTALL, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_selectall), XtRImmediate, (XtPointer) FALSE},
{XVW_COMMANDBAR_SHOW_UNSELECTALL, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_unselectall), XtRImmediate, (XtPointer) FALSE},
{XVW_COMMANDBAR_SHOW_OPEN, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_open), XtRImmediate, (XtPointer) TRUE},
{XVW_COMMANDBAR_SHOW_SAVE, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_save), XtRImmediate, (XtPointer) TRUE},
{XVW_COMMANDBAR_SHOW_SAVEAS, NULL, XtRBoolean, sizeof(Boolean),
      offset(show_saveas), XtRImmediate, (XtPointer) TRUE},
{XVW_COMMANDBAR_LARGEICONS, NULL, XtRBoolean, sizeof(Boolean),
      offset(largeicons), XtRImmediate, (XtPointer) TRUE},

{XVW_BUFFER_DIST, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwCommandBarWidgetRec, manager.buffer_dist),
        XtRImmediate, (XtPointer) 1},
{XVW_DEF_HORIZ_DIST, NULL, XtRPosition, sizeof(Position),
        XtOffsetOf(XvwCommandBarWidgetRec, manager.horiz_offset),
        XtRImmediate, (XtPointer) 3},
{XVW_DEF_VERT_DIST, NULL, XtRPosition, sizeof(Position),
        XtOffsetOf(XvwCommandBarWidgetRec, manager.vert_offset),
        XtRImmediate, (XtPointer) 3},
};
#undef offset


/*-------------------------------------------------------------------*
|
|   Class Declaration for CommandBar Widget
|
--------------------------------------------------------------------*/

#define superclass (&xvwRowColWidgetClassRec)

XvwCommandBarWidgetClassRec xvwCommandBarWidgetClassRec =
{
  {
    (WidgetClass) superclass,		/* superclass		  */	
    "CommandBar",			/* class_name		  */
    sizeof(XvwCommandBarWidgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    NULL,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    XtInheritQueryGeometry,		/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,				    /* subresources	  */
    0,					    /* subresources_count */
    sizeof(XvwCommandBarWidgetConstraintsRec),  /* constraint_size	  */
    NULL,				    /* initialize	  */
    NULL,				    /* destroy		  */
    NULL,				    /* set_values	  */
    NULL,				    /* extension	  */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			    /* pre-layout routine     */
    XtInheritLayout,			    /* post-layout routine    */
    XtInheritChangeSel,			    /* change selected proc   */
    XtInheritEraseSel,			    /* erase selected proc    */
    XtInheritRefreshSel,		    /* refresh selection proc */
    XtInheritResize,			    /* resize		      */
    XtInheritGeometryManager,  		    /* geometry_manager	      */
  },  /* XvwManagerWidgetClass fields initialization */
  {
    NULL,  		                    /* extension          */
  },  /* XvwRowColWidgetClass fields initialization */
  {
    NULL,  		                    /* extension          */
  },  /* XvwCommandBarWidgetClass fields initialization */
};
#undef superclass

  /* for public consumption */
WidgetClass xvwCommandBarWidgetClass = (WidgetClass) &xvwCommandBarWidgetClassRec;


/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for widget definition
|
--------------------------------------------------------------------*/

#define kwidget(widget)	     (XvwCommandBarWidget) (widget)


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an
|                instance of a XvwCommandBarWidget class has been created.
|                This is where the attributes for the class are
|		 initialized. 
|
|    Written By: Mark Young
|          Date: Nov 05, 1994
|
------------------------------------------------------------*/
/*ARGSUSED*/
static void ClassInitialize(void)
{
	xvw_init_attributes(xvwCommandBarWidgetClass, attributes,
		XtNumber(attributes), NULL, 0,
		"$DESIGN/objects/library/xvlang/uis/CommandBar.pane");
	xvw_load_resources("$DESIGN/objects/library/xvlang/app-defaults/CommandBar");
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|	Purpose: This method will set up the initial commandbar widget
|		 instance.
|
|	  Input: request - not used
|                new     - widget instance after initialization
|    Written By: Mark Young
|          Date: Nov 05, 1994
|
------------------------------------------------------------*/
/*ARGSUSED*/
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwCommandBarWidget xwid = kwidget(new);

	xwid->commandbar.run_indx     = -1;
	xwid->commandbar.stop_indx    = -1;
	xwid->commandbar.runmode_indx = -1;
	CommandbarUpdate(new);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: This method is used to set the public values
|                of a XvwCommandBarWidget instance.  The public values
|                which can be changed are all related to the display
|		 of the error.
|
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new     - the widget processed through all set values methods
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 05, 1994
|
------------------------------------------------------------*/
/*ARGSUSED*/
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num)
{
	XvwCommandBarWidget cxwid = kwidget(current);
	XvwCommandBarWidget nxwid = kwidget(new);


	if (cxwid->commandbar.workspace != nxwid->commandbar.workspace)
	{
	   xvobject object = xvw_object(new);

	   if (cxwid->commandbar.workspace != NULL)
	   {
	      xvw_remove_callback(cxwid->commandbar.workspace,
	        XVW_WORKSPACE_RUN_CALLBACK, WorkspaceRunCallback, object);
	      xvw_remove_callback(cxwid->commandbar.workspace,
	        XVW_WORKSPACE_ACTIVE_CALLBACK, WorkspaceActiveCallback, object);
	   }

	   if (nxwid->commandbar.workspace != NULL)
	   {
	      xvw_insert_callback(nxwid->commandbar.workspace,
			XVW_WORKSPACE_RUN_CALLBACK, FALSE,
			WorkspaceRunCallback, object);
	      xvw_insert_callback(nxwid->commandbar.workspace,
			XVW_WORKSPACE_ACTIVE_CALLBACK, FALSE,
			WorkspaceActiveCallback, object);
	   }
	   UpdateWorkspaceRunStop(new);
	}

	if (cxwid->commandbar.show_run     != nxwid->commandbar.show_run     ||
	    cxwid->commandbar.show_runmode != nxwid->commandbar.show_runmode ||
	    cxwid->commandbar.show_stop    != nxwid->commandbar.show_stop    ||
	    cxwid->commandbar.show_reset   != nxwid->commandbar.show_reset   ||
	    cxwid->commandbar.show_clear   != nxwid->commandbar.show_clear   ||
	    cxwid->commandbar.show_check   != nxwid->commandbar.show_check   ||
	    cxwid->commandbar.show_info    != nxwid->commandbar.show_info    ||
	    cxwid->commandbar.show_redraw  != nxwid->commandbar.show_redraw  ||
	    cxwid->commandbar.show_procedure !=
	           nxwid->commandbar.show_procedure                          ||
	    cxwid->commandbar.show_countloop !=
	           nxwid->commandbar.show_countloop                          ||
	    cxwid->commandbar.show_whileloop !=
	           nxwid->commandbar.show_whileloop                          ||
	    cxwid->commandbar.show_clipboard !=
	           nxwid->commandbar.show_clipboard                          ||
	    cxwid->commandbar.show_cut    != nxwid->commandbar.show_cut      ||
	    cxwid->commandbar.show_copy   != nxwid->commandbar.show_copy     ||
	    cxwid->commandbar.show_paste  != nxwid->commandbar.show_paste    ||
	    cxwid->commandbar.show_duplicate !=
	    nxwid->commandbar.show_duplicate                                 ||
	    cxwid->commandbar.show_delete != nxwid->commandbar.show_delete   ||
	    cxwid->commandbar.show_undo   != nxwid->commandbar.show_undo     ||
	    cxwid->commandbar.show_selectall !=
       	          nxwid->commandbar.show_selectall                           ||
	    cxwid->commandbar.show_unselectall !=
	          nxwid->commandbar.show_unselectall                         ||
	    cxwid->commandbar.show_open   != nxwid->commandbar.show_open     ||
	    cxwid->commandbar.show_save   != nxwid->commandbar.show_save     ||
	    cxwid->commandbar.show_saveas != nxwid->commandbar.show_saveas   ||
	    cxwid->commandbar.show_remote != nxwid->commandbar.show_remote   ||
	    cxwid->commandbar.largeicons  != nxwid->commandbar.largeicons) 
	{
	    if (cxwid->commandbar.largeicons != nxwid->commandbar.largeicons)
	    {
	       int dist = nxwid->commandbar.largeicons ? 1 : 0;
	       xvw_set_attribute(xvw_object(new), XVW_BUFFER_DIST, dist);
	    }
	    CommandbarUpdate(new);
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: CommandbarCallbacks - commandbar button callbacks routine
|
|       Purpose: This routine is the handler for performing certain workspace
|		 button callbacks functions, such as mapping or unmapping the
|		 clipboard.
|
|         Input: widget - commandbar widget
|    Written By: Mark Young
|          Date: Nov 05, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void UpdateWorkspaceRunStop(
   Widget widget)
{
	XvwCommandBarWidget xwid = (XvwCommandBarWidget) kwidget(widget);

	CommandBarItem *entry;
	int run, schedule, runmode;
	char temp[KLENGTH], *icondir;

	if (xwid->commandbar.workspace == NULL)
	   return;

 	icondir = xwid->commandbar.largeicons ? LARGE_ICONDIR : SMALL_ICONDIR;
	xvw_get_attributes(xwid->commandbar.workspace,
			XVW_WORKSPACE_RUN,      &run,
			XVW_WORKSPACE_SCHEDULE, &schedule,
			XVW_WORKSPACE_RUNMODE,  &runmode,
			NULL);
	if (xwid->commandbar.stop_indx != -1)
	{
	   entry = &commandbar_list[xwid->commandbar.stop_indx];
	   xvw_set_attribute(entry->button, XVW_BUTTON_SELECTED, !schedule);
	}

	if (xwid->commandbar.run_indx != -1)
	{
	   entry = &commandbar_list[xwid->commandbar.run_indx];
	   if (run == TRUE)
	      kfullpath(entry->set_pixmapfile, icondir, temp);
	   else
	      kfullpath(entry->unset_pixmapfile, icondir, temp);

	    xvw_set_attributes(entry->button,
			XVW_PIXMAP_FILENAME, temp,
			XVW_BUTTON_SELECTED, schedule,
			NULL);
	}

	if (xwid->commandbar.runmode_indx != -1)
	{
	   entry = &commandbar_list[xwid->commandbar.runmode_indx];
           if (runmode == KWORKSPACE_RUNMODE_CONTINUOUS)
              kfullpath(entry->set_pixmapfile, icondir, temp);
           else
              kfullpath(entry->unset_pixmapfile, icondir, temp);

	    xvw_set_attribute(entry->button, XVW_PIXMAP_FILENAME, temp);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: CommandbarCallbacks - commandbar button callbacks routine
|
|       Purpose: This routine is the handler for performing certain workspace
|		 button callbacks functions, such as mapping or unmapping the
|		 clipboard.
|
|         Input: object - object in which the object was invoked
|		 client_data - the workspace widget
|		 calldata    - not used
|    Written By: Mark Young
|          Date: Nov 05, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void CommandbarCallbacks(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	CommandBarItem *entry = (CommandBarItem *) client_data;

	int runmode, remote_enable;
 	char *icondir, temp[KLENGTH], *filename = NULL;
	xvobject workspace, commandbar = xvw_parent(object);
	XvwCommandBarWidget xwid = (XvwCommandBarWidget) xvw_widget(commandbar);

	/*
	 *  Sanity check
	 */
	icondir = xwid->commandbar.largeicons ? LARGE_ICONDIR : SMALL_ICONDIR;
	if ((workspace = xwid->commandbar.workspace) == NULL)
	   return;

	if (kstrcmp(entry->name, "RunMode") == 0)
	{
	   xvw_get_attribute(workspace, XVW_WORKSPACE_RUNMODE, &runmode);
	   entry = &commandbar_list[xwid->commandbar.runmode_indx];
	   if (runmode == KWORKSPACE_RUNMODE_CONTINUOUS)
	   {
	      runmode = KWORKSPACE_RUNMODE_ONCE;
	      kfullpath(entry->unset_pixmapfile, icondir, temp);
	   }
	   else
	   {
	      runmode = KWORKSPACE_RUNMODE_CONTINUOUS;
	      kfullpath(entry->set_pixmapfile, icondir, temp);
	   }
	   xvw_set_attribute(workspace, XVW_WORKSPACE_RUNMODE, runmode);
	   xvw_set_attribute(entry->button, XVW_PIXMAP_FILENAME, temp);
	}
	else if (kstrcmp(entry->name, "SaveWksp") == 0)
	{
           xvw_get_attribute(workspace, XVW_WORKSPACE_FILENAME, &filename);
	   xvw_set_attribute(workspace, XVW_WORKSPACE_SAVEALL, filename);
	}
	else if (kstrcmp(entry->name, "Run") == 0)
	   xvw_set_attribute(workspace, XVW_WORKSPACE_RUN, TRUE);
	else if (kstrcmp(entry->name, "Stop") == 0)
	   xvw_set_attribute(workspace, XVW_WORKSPACE_STOP, TRUE);
	else if (kstrcmp(entry->name, "OpenWksp") == 0)
	   xvw_set_attribute(workspace, XVW_WORKSPACE_RESTORE, NULL);
	else if (kstrcmp(entry->name, "SaveWkspAs") == 0)
	   xvw_set_attribute(workspace, XVW_WORKSPACE_SAVEALL,"ObtainFilename");
	else if (kstrcmp(entry->name, "Delete") == 0)
           xvw_set_attribute(workspace, XVW_CANVAS_DELETE, FALSE);
	else if (kstrcmp(entry->name, "Remote") == 0)
        {
	   char **allhosts = NULL;
	   size_t allhost_num = 0;
	   int    i;

           xvw_get_attribute(workspace, XVW_NODE_REMOTE_ENABLE, &remote_enable);
	   remote_enable = !remote_enable;
	   xvw_set_attribute(workspace, XVW_NODE_REMOTE_ENABLE, remote_enable);
           if (remote_enable == TRUE)
              kfullpath(entry->set_pixmapfile, icondir, temp);
           else kfullpath(entry->unset_pixmapfile, icondir, temp);

	   xvw_set_attribute(entry->button, XVW_PIXMAP_FILENAME, temp);
	
	   allhosts = khost_list(&allhost_num);
           for (i = 0; i < allhost_num; i++)
	       kremote_start(allhosts[i]);
	}
	else if (entry->attribute != NULL)
	   xvw_set_attribute(workspace, entry->attribute, TRUE);
}
 
/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceRunCallback - workspace run callback
|
|       Purpose: This routine handlers the run callback for a workspace
|
|         Input: object - object in which the object was invoked
|		 client_data - the commandbar object
|		 calldata    - not used
|    Written By: Mark Young
|          Date: Nov 05, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void WorkspaceRunCallback(
   xvobject workspace,
   kaddr    client_data,
   kaddr    call_data)
{
	xvobject commandbar = (xvobject) client_data;
	XvwCommandBarWidget xwid = (XvwCommandBarWidget) xvw_widget(commandbar);

	UpdateWorkspaceRunStop((Widget) xwid);
}
 
/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceActiveCallback - workspace active callback
|
|       Purpose: This routine handlers the changing of the active workspace.
|
|         Input: object - object in which the object was invoked
|		 client_data - the commandbar object
|		 calldata    - not used
|    Written By: Mark Young
|          Date: Nov 05, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void WorkspaceActiveCallback(
   xvobject workspace,
   kaddr    client_data,
   kaddr    call_data)
{
	xvobject active = *((xvobject *) call_data);
	xvobject commandbar = (xvobject) client_data;
	XvwCommandBarWidget xwid = (XvwCommandBarWidget) xvw_widget(commandbar);

	xwid->commandbar.workspace = active;
	UpdateWorkspaceRunStop((Widget) xwid);
}

/*-----------------------------------------------------------
|
|  Routine Name: CommandbarUpdate
|
|       Purpose: This routine will update which buttons should be
|		 shown and which ones are not.
|
|         Input: widget      - the commandbar widget
|    Written By: Mark Young
|          Date: Mar 22, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void CommandbarUpdate(
   Widget widget)
{
	xvobject object = xvw_object(widget);
        XvwCommandBarWidget xwid = kwidget(widget);
 
        int      i, show;
        CommandBarItem *entry;
        char     filename[KLENGTH], *icondir;
        unsigned long mask = EnterWindowMask | LeaveWindowMask;
 
        icondir = xwid->commandbar.largeicons ? LARGE_ICONDIR : SMALL_ICONDIR;
        for (i = 0; i < knumber(commandbar_list); i++)
        {
           entry = &commandbar_list[i];
           xvw_get_attribute(object, entry->show_attribute, &show);
           if (show == FALSE && entry->button == NULL)
              continue;
 
           if (entry->button == NULL)
           {
              entry->button = xvw_create_button(object, entry->name);
              xvw_insert_callback(entry->button, XVW_BUTTON_SELECT, TRUE,
			CommandbarCallbacks, entry);

              if (kstrcmp(entry->name, "Run") == 0)
                 xwid->commandbar.run_indx = i;
              else if (kstrcmp(entry->name, "Stop") == 0)
                 xwid->commandbar.stop_indx = i;
              else if (kstrcmp(entry->name, "RunMode") == 0)
                 xwid->commandbar.runmode_indx = i;
 
              xvw_insert_event(entry->button, mask, CommandbarAnnounce,
			entry, KLIST_HEAD);
           }
           kfullpath(entry->unset_pixmapfile, icondir, filename);
           xvw_set_attributes(entry->button,
                        XVW_ROWCOL_ACTIVE, show,
                        XVW_PIXMAP_FILENAME, filename,
                        NULL);
           if (show == TRUE)
              xvw_manage(entry->button);
           else
              xvw_unmanage(entry->button);
        }
}

/*-----------------------------------------------------------
|
|  Routine Name: CommandbarAnnounce
|
|       Purpose: This routine will perform a variety of workspace announces
|		 for such things as clear, wkspgui, run, etc.
|
|         Input: object      - the button to announce for
|                client_data - the workspace widget
|                event       - event the activated us
|		 dispatch    - whether to continue dispatch the event
|    Written By: Mark Young
|          Date: Nov 23, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void CommandbarAnnounce(
   xvobject object,
   kaddr    client_data,
   XEvent   *event,
   int      *dispatch)
{
	CommandBarItem *entry = (CommandBarItem *) client_data;

	xvobject workspace, commandbar = xvw_parent(object);
	XvwCommandBarWidget xwid = (XvwCommandBarWidget) xvw_widget(commandbar);

	if ((workspace = xwid->commandbar.workspace) == NULL)
	   return;

	if (event->type == LeaveNotify)
	{
	   kannounce(XVLANG, "CommandbarAnnounce", "");
	}
	else if (event->type == EnterNotify)
	{
	   if (entry->announce != NULL)
	      kannounce(XVLANG, "CommandbarAnnounce", "%s", entry->announce);
	}
}


/************************************************************
*
*  Routine Name: xvw_create_commandbar - create a toolbox menu object
*
*       Purpose: Creates a toolbox menu object, that consists of three
*		 menu objects.  The first menu displays the different
*		 categories.  The second menu displays the sub-categories
*		 for the selected category.  And finally the third
*		 displays the operators for the selected sub-category.
*
*		 Two callbacks are provided which can be used to notify
*		 the programmer when the user selects or activates an operator
*		 from the menu.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - the name with which to reference the object
*        Output: None
*       Returns: The commandbar object on success, NULL on failure
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Nov 05, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*******************************************************************/

xvobject xvw_create_commandbar(
   xvobject parent,
   char     *name)
{
	xvobject object;


	object = xvw_create(parent, FALSE, TRUE, name, CommandBarWidgetClass);
	return(object);
}
