 /*
  * Khoros: $Id: WorkUtil.c,v 1.48.2.1 1996/12/06 20:30:05 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: WorkUtil.c,v 1.48.2.1 1996/12/06 20:30:05 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>	              Khoros Workspace Widget Utilities
   >>>>   Static:
   >>>>			WorkspaceGUIInitialize()
   >>>>			WorkspaceGUIActions()
   >>>>			WorkspaceGUICallback()
   >>>>			WorkspaceCheckRunning()
   >>>>  Private:
   >>>>			WorkspaceChildren()
   >>>>			WorkspaceRun()
   >>>>			WorkspaceGlyphModifiedCallback()
   >>>>			WorkspaceActions()
   >>>>			WorkspaceAnnounce()
   >>>>			WorkspaceActive()
   >>>>
   >>>>			WorkspaceClipboardInit()
   >>>>			WorkspaceClipboardCreate()
   >>>>			WorkspaceClipboardClose()
   >>>>			WorkspaceClipboardClear()
   >>>>			WorkspaceClipboardDelete()
   >>>>			ClipboardUndoDelete()
   >>>>   Public:
   >>>>	
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvlang/WorkspaceP.h>
#include <xvlang/GlyphP.h>

static void WorkspaceGUIInitialize  PROTO((xvobject));
static void WorkspaceGUIActions	    PROTO((xvobject, kaddr, kaddr));
static void WorkspaceGUICallback    PROTO((xvobject, kaddr, kaddr));
static int  WorkspaceCheckRunning   PROTO((xvobject));

xvobject clipboard = NULL;

static char  *options[] = {
	"Craftsman",
	"Edit GUI",
	"Delete Selections",
        "Delete All Selections",
	"Attributes" };

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceGUIInitialize
|
|       Purpose: Initialize the options to be used with the object's option
|                menu.
|
|         Input: workspace - the workspace which to initialize the options for.
|    Written By: Mark Young
|          Date: Apr 25, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void WorkspaceGUIInitialize(
   xvobject workspace)
{
        XvwWorkspaceWidget xwid = (XvwWorkspaceWidget) xvw_widget(workspace);
 
        int      i;
        kguide   *guide;
        xvobject button, menu;
        kform_struct **kformstructs;
        kselection *sel_list, *selection;

	/*
	 *  Check to see if the options have already been initialized...
	 */
	if (xwid->workspace.wkspgui_initialized == TRUE)
	   return;

	xwid->workspace.wkspgui->editable = XVF_FULL_EDIT;
	if (!kvf_check_for_single_pane(xwid->workspace.wkspgui,"Initialize"))
	   return;

	sel_list = xwid->workspace.wkspgui->subform->guide->pane->sel_list;
	selection = kvf_gui_named_item_exists(sel_list, KUIS_ROUTINE, NULL);
	if (selection != NULL)
	{
	   xvf_add_extra_call(selection->back_kformstruct, WorkspaceRun,
			(kaddr) workspace, XVF_CALL_SUBSTITUTE);
	}

	/* find the selected pane */
	guide = kvf_search_sel_guide(xwid->workspace.wkspgui->subform);

	/* get the options button on the backplane for the selections */
	if ((kformstructs = kvf_get_selection_kformstructs(guide->pane,
			KUIS_OPTIONS, &i)) != NULL && i == 1)
	{
	   menu = kformstructs[0]->Selptr->back;
	   for (i = 0; i < knumber(options); i++)
	   {
	      button = xvw_create_button(menu, options[i]);
	      xvw_set_attribute(button, XVW_LABEL, options[i]);
	      xvw_insert_callback(button, XVW_BUTTON_SELECT, TRUE,
			WorkspaceGUIActions, workspace);

              if (kstrcmp(options[i], "Delete Selections") == 0)
              {
                 xvw_sensitive(button, FALSE);
                 xwid->workspace.delete_button = button;
              }
	   }
	   kfree(kformstructs);
	}

	/*
	 * get all the selections so that we can raise them when the form is
	 * put into edit mode.
	 */
	xvw_insert_callback(guide->pane->back, XVW_SELECT_CALLBACK, FALSE,
			WorkspaceGUICallback, xwid);
	xwid->workspace.wkspgui_initialized = TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceGUIActions
|
|       Purpose: This routine will be called when a wkspgui action is
|		 activated.
|
|         Input: object      - which button being manipulated
|                client_data - the glyph
|                call_data   - not used
|    Written By: Mark Young
|          Date: Dec 1, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void WorkspaceGUIActions(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	xvobject workspace = (xvobject) client_data;
	XvwWorkspaceWidget xwid = (XvwWorkspaceWidget) xvw_widget(workspace);

	char     *name = NULL;

	xvw_get_attribute(object, XVW_LABEL, &name);

	/*
	 *  Spawn craftsman
	 */
	if (kstrcmp(name, "Craftsman") == 0)
	   kspawn("craftsman");

	/*
	 *  Toggle edit mode on/off on workspace GUI
	 */
	else if (kstrcmp(name, "Edit GUI") == 0)
	{
	   kcontrol *pane;
	   int      editing = TRUE;

	   pane = xwid->workspace.wkspgui->subform->guide->pane;
	   xvw_get_attribute(pane->back, XVW_EDIT_MODE_ON, &editing);
	   if (editing == FALSE)
	   {
	      /* 
	       *  Turn edit mode on for the pane, and also make sure that
	       *  the options button rides on top of the editing window...
	       */
	      xvw_set_attribute(pane->back, XVW_EDIT_MODE_ON, TRUE);
	   }
	   else
	   {
	      /* Turn editing off...  */
	      xvw_set_attribute(pane->back, XVW_EDIT_MODE_ON, FALSE);
	   }
	}

	/*
	 *  Delete all selections on pane except help, quit, options, etc.
	 */
	else if (kstrcmp(name, "Delete All Selections") == 0)
	{
	   kselection *selection, *next_selection;
	   selection = xwid->workspace.wkspgui->subform->guide->pane->sel_list;
	   while (selection != NULL)
	   {
	      next_selection = selection->next;
	      if (!xvl_excluded_selection(selection))
		 xvf_set_attribute(selection->back_kformstruct,XVF_DELETE,TRUE);

	      selection = next_selection;
	   }
	}

	/*
	 *  Delete selected selections off workspace GUI
	 */
	else if (kstrcmp(name, "Delete Selections") == 0)
	   xvl_delete_selections(workspace);

	/*
	 *  Bring up the Attributes menuform
	 */
	else if (kstrcmp(name, "Attributes") == 0)
	   xvw_activate_menu(workspace);
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceGUICallback
|
|       Purpose: This routine will be called when the wkspgui is put into
|                edit mode.
|
|         Input: object      - which button being manipulated
|                client_data - the glyph
|                call_data   - not used
|    Written By: Mark Young
|          Date: Aug 28, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void WorkspaceGUICallback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
        int     editing = *((int *) call_data);
        XvwWorkspaceWidget xwid = (XvwWorkspaceWidget) client_data;
 
	int	 i;
	xvobject button;
 
        if (xwid->workspace.selections == NULL)
           return;
 
        if (editing)
        {
           if (xwid->workspace.delete_button != NULL)
              xvw_sensitive(xwid->workspace.delete_button, TRUE);

           for (i = 0; i < xwid->workspace.num_selections; i++)
	   {
	      if ((button = xvf_get_xvobject(xwid->workspace.selections[i],
                        XVF_BUTTON_OBJ, TRUE)) != NULL)
	      {
		 xvw_raise(button);
	      }
	   }
        }
        else if (xwid->workspace.delete_button != NULL)
           xvw_sensitive(xwid->workspace.delete_button, FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceCheckRunning
|
|       Purpose: This routine will check to see if the workspace is running
|		 or not.
|
|         Input: workspace   - whether the workspace is running or not
|    Written By: Mark Young
|          Date: Jan 28, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static int WorkspaceCheckRunning(
   xvobject workspace)
{
	size_t   num = 0;
	xvobject *selections;
	int      i, running = FALSE;


	selections = WorkspaceChildren(workspace, FALSE, &num);
	for (i = 0; i < num && running == FALSE; i++)
	{
	   if (xvw_check_subclass(selections[i], NodeWidgetClass) == FALSE)
	      continue;

	   xvw_get_attribute(selections[i], XVW_NODE_EXECUTE, &running);
	}
	kfree(selections);
	return(running);
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceCallbacks - workspace button callbacks routine
|
|       Purpose: This routine is the handler for performing certain workspace
|                button callbacks functions, such as mapping or unmapping the
|                clipboard.
|
|         Input: object - object in which the object was invoked
|                client_data - the workspace widget
|                calldata    - not used
|    Written By: Mark Young
|          Date: Jul 23, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
void WorkspaceCallbacks(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
        int status = *((int *) call_data);
        xvobject workspace = (xvobject) client_data;
        XvwWorkspaceWidget xwid = (XvwWorkspaceWidget) xvw_widget(workspace);

        if (object != xwid->workspace.titlebar)
           return;

        if (status == KTITLEBAR_ICONIFIED)
        {
           if (workspace != clipboard)
           {
              xvw_unmap(workspace);
              WorkspaceActive(workspace, FALSE);
           }
           else
              xvw_set_attribute(workspace, XVW_WORKSPACE_SHOW_CLIPBOARD, FALSE);
        }
        else if (status == KTITLEBAR_DETACHED)
        {
           if (workspace != clipboard)
              WorkspaceActive(workspace, FALSE);
           else
              xvw_set_attribute(workspace, XVW_WORKSPACE_SHOW_CLIPBOARD, FALSE);
        }
        else if (status == KTITLEBAR_REATTACHED)
        {
           if (workspace != clipboard)
              WorkspaceActive(workspace, TRUE);
           else
              xvw_set_attribute(workspace, XVW_WORKSPACE_SHOW_CLIPBOARD, FALSE);
        }
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceGlyphRunCallbacks - glyph run callback routine
|
|       Purpose: This routine is the handler for performing certain workspace
|                button callbacks functions, such as mapping or unmapping the
|                clipboard.
|
|         Input: object - object in which the object was invoked
|                client_data - the workspace widget
|                calldata    - not used
|    Written By: Mark Young
|          Date: Jul 23, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
void WorkspaceGlyphRunCallback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
        xvobject workspace = (xvobject) client_data;
        XvwWorkspaceWidget xwid = (XvwWorkspaceWidget) xvw_widget(workspace);
 
	char *name = NULL;
        int  status, running = *((int *) call_data);
 
        if (!running && xwid->workspace.running)
	   running = WorkspaceCheckRunning(workspace);
 
	if (object && !running)
	{
	   xvw_get_attribute(object, XVW_NODE_STATUS, &status);
	   xwid->workspace.status |= status;
	}

        if (object && running == xwid->workspace.running)
           return;

	/*
	 *  Reporting facility call.  This is used for external monitoring and
	 *  also allows automated testing.
	 */
	if (object != NULL)
	{
           xvw_get_attribute(object, XVW_NODE_NAME, &name);
	   xvl_report(TRUE, "WorkspaceRunCallback '%s' '%s' is now %s\n",
		xvw_name(workspace), name, running == TRUE ?
		"running" : "stopped");
	}
        xwid->workspace.running = running;
	if (!running && xwid->workspace.runmode == KWORKSPACE_RUNMODE_ONCE)
	   xwid->workspace.scheduling = FALSE;

        XtCallCallbacks((Widget) xwid, XVW_WORKSPACE_RUN_CALLBACK, &running);
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceChildren - get the workspace children
|
|       Purpose: This routine gets the workspace children.
|         Input: parent   - 
|                selected - whether to get the selected children, or all
|        Output: num      - number of returned children
|       Returns: the children or NULL
|    Written By: Mark Young
|          Date: Aug 21, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
xvobject *WorkspaceChildren(
   xvobject workspace,
   int      selected,
   size_t   *num)
{
	XvwWorkspaceWidget xwid = (XvwWorkspaceWidget) xvw_widget(workspace);

	xvobject *children, parent = xwid->viewport.plane;
        int      size;

	if (selected == TRUE)
	{
	   xvw_get_attributes(parent,
		XVW_SELECTIONS,     &children,
		XVW_NUM_SELECTIONS, &size,
		NULL);
	   children = (xvobject *) kdupalloc(children, 
				           (size_t) size * sizeof(xvobject));
           if (num != NULL) *num = size;
	}
	else children = xvw_children(parent, NULL, num);
	return(children);
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceRun - the run button on the wkspgui executed
|
|       Purpose: This performs a run of the workspace via the run button
|		 on the wkspgui.
|
|         Input: client_data - the workspace widget
|    Written By: Mark Young
|          Date: Jul 23, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
void WorkspaceRun(
   kaddr    client_data)
{
	xvobject workspace = (xvobject) client_data;

	xvw_set_attribute(workspace, XVW_WORKSPACE_RUN, TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceGlyphModifiedCallback - glyph modified callback
|						   routine
|
|       Purpose: This routine is the handler for performing certain workspace
|		 button callbacks functions, such as mapping or unmapping the
|		 clipboard.
|
|         Input: object - object in which the object was invoked
|		 client_data - the workspace widget
|		 calldata    - not used
|    Written By: Mark Young
|          Date: Jul 23, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
void WorkspaceGlyphModifiedCallback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	xvobject workspace = (xvobject) client_data;
	XvwWorkspaceWidget wksp_wid = (XvwWorkspaceWidget)xvw_widget(workspace);
	XvwGlyphWidget glyph_wid = (XvwGlyphWidget) xvw_widget(object);

	int i, modified = *((int *) call_data);

	/*
	 *  if the workspace is scheduling, execute the glyph
	 */
	if (wksp_wid->workspace.scheduling && modified)
	   xvw_set_attribute(object, XVW_NODE_EXECUTE, TRUE);

	/*
	 * 
	 */
	else if (modified)
	{
	    for (i = 0; i < glyph_wid->node.isize; i++)
                  xvw_set_attribute(glyph_wid->node.iports[i], 
				    XVW_PORT_MODIFIED, FALSE);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceActions
|
|       Purpose: WorkspaceActions is used to set the certain actions associated
|		 with the current workspace.
|
|         Input: object    - the object in which we will be setting the
|			     attribute value for
|		 attribute - the desired attribute
|		 calldata  - the value to be set
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 27, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
int WorkspaceActions(
   xvobject workspace,
   char     *attribute,
   kaddr    calldata)
{
	XvwWorkspaceWidget xwid = (XvwWorkspaceWidget) xvw_widget(workspace);

	kfile    *file;
	size_t   num = 0;
	Widget   widget = (Widget) xwid;
	char     *filename, temp[KLENGTH];
	xvobject *selections, object, procedure;
	int      i, selected, running, scheduling, runnable, modified;


	object = xwid->viewport.plane;
	if (kstrcmp(attribute, XVW_WORKSPACE_RUN) == 0)
	{
	   kannounce(XVLANG, "WorkspaceActions", "Running workspace '%s'",
			xvw_name(workspace));

           /*  if we are logging execution, mark run */
           if (xwid->workspace.glyph_echo == KGLYPH_ECHO_TO_LOG ||
               xwid->workspace.glyph_echo == KGLYPH_ECHO_TO_CONSOLE_AND_LOG)
           {
                file = xvl_get_logfile();
                if (file != NULL)
                {
                    kfprintf(file, "\n#----------- Running Workspace '%s' -----------\n", xvw_name(workspace));
                    kfclose(file);
                }
           }

	   xwid->workspace.status = 0;
	   xwid->workspace.scheduling = running = TRUE;
	   WorkspaceGlyphRunCallback(NULL, workspace, &running);
	   selections = WorkspaceChildren(workspace, FALSE, &num);

	   for (i = 0; i < num; i++)
	   {
	      if (xvw_check_subclass(selections[i], NodeWidgetClass) == TRUE)
		 xvw_set_attribute(selections[i], XVW_NODE_EXECUTE, TRUE);
	   }
	   kfree(selections);

	   /*  in case nothing can or is currently running */
	   if ((running = WorkspaceCheckRunning(workspace)) == FALSE)
	      WorkspaceGlyphRunCallback(NULL, workspace, &running);
	}
	else if (kstrcmp(attribute, XVW_WORKSPACE_STOP) == 0)
	{
	   kannounce(XVLANG, "WorkspaceActions", "Stopping workspace '%s'",
			xvw_name(workspace));

           /*  if we are logging execution, mark stop */
           if (xwid->workspace.glyph_echo == KGLYPH_ECHO_TO_LOG ||
               xwid->workspace.glyph_echo == KGLYPH_ECHO_TO_CONSOLE_AND_LOG)
           {
                file = xvl_get_logfile();
                if (file != NULL)
                {
                    kfprintf(file, "\n#----------- Stopping Workspace '%s' -----------\n", xvw_name(workspace));
                    kfclose(file);
                }
           }

	   xwid->workspace.scheduling = running = FALSE;
	   selections = WorkspaceChildren(workspace, FALSE, &num);

	   for (i = 0; i < num; i++)
	   {
	      if (xvw_check_subclass(selections[i], NodeWidgetClass) == TRUE)
		 xvw_set_attribute(selections[i], XVW_NODE_EXECUTE, FALSE);
	   }
	   WorkspaceGlyphRunCallback(NULL, workspace, &running);
	   kfree(selections);
	}
	else if (kstrcmp(attribute, XVW_WORKSPACE_RESET) == 0)
	{
	   int reset_variables = *((int *) calldata);

	   kannounce(XVLANG, "WorkspaceActions", "Resetting workspace '%s'",
			xvw_name(workspace));

	   /*  if we are logging execution, mark reset */
           if (xwid->workspace.glyph_echo == KGLYPH_ECHO_TO_LOG ||
               xwid->workspace.glyph_echo == KGLYPH_ECHO_TO_CONSOLE_AND_LOG)
           {
                file = xvl_get_logfile();
	        if (file != NULL)
		{
                    kfprintf(file, "\n#----------- Workspace Reset -----------\n");
		    kfclose(file);
		}
           }

	   /*
	    *  Disable the scheduler temporarily.  If it turns out that we are
	    *  actively scheduling we will re-run after resetting everyone.
	    */
	   scheduling = xwid->workspace.scheduling;
	   xwid->workspace.scheduling = FALSE;

	   /*
	    *  This little bit of trick allows the loops to reset without
	    *  reverting variables back to their original values.
	    */
	   if (reset_variables == TRUE)
	   {
	      kexpr_variables_reset((long) workspace);
	      xvl_expr_call_callback((long) workspace, XVL_EXPR_UPDATE);
	   }

	   /*  reset all nodes.  */
	   selections = WorkspaceChildren(workspace, FALSE, &num);
	   for (i = 0; i < num; i++)
	   {
	      if (xvw_check_subclass(selections[i], NodeWidgetClass) == TRUE)
		 xvw_set_attribute(selections[i], XVW_NODE_RESET, TRUE);
	   }
	   kfree(selections);

	   /*  if we were scheduling then re-run. */
	   if (scheduling == TRUE)
	      xvw_set_attribute(workspace, XVW_WORKSPACE_RUN, TRUE);

	}
	else if (kstrcmp(attribute, XVW_WORKSPACE_CHECK) == 0)
	{
	   kannounce(XVLANG, "WorkspaceActions", "Testing workspace '%s'",
			xvw_name(workspace));

	   xwid->workspace.scheduling = running = *((int *) calldata);
	   selections = WorkspaceChildren(workspace, FALSE, &num);
	   for (i = 0; i < num; i++)
	   {
	      if (xvw_check_subclass(selections[i], NodeWidgetClass) == TRUE)
		 xvw_get_attribute(selections[i], XVW_NODE_RUNNABLE, &runnable);
	   }
	   kfree(selections);
	}
	else if (kstrcmp(attribute, XVW_WORKSPACE_STEP) == 0)
	{
	   kannounce(XVLANG, "WorkspaceActions", "Single step workspace '%s'",
			xvw_name(workspace));

           /*  if we are logging execution, mark reset */
           if (xwid->workspace.glyph_echo == KGLYPH_ECHO_TO_LOG ||
               xwid->workspace.glyph_echo == KGLYPH_ECHO_TO_CONSOLE_AND_LOG)
           {
                file = xvl_get_logfile();
                if (file != NULL)
                {
                    kfprintf(file, "\n#----------- Single step workspace '%s' -----------\n", xvw_name(workspace));
                    kfclose(file);
                }
           }

	   selections = WorkspaceChildren(workspace, FALSE, &num);
	   for (i = 0; i < num; i++)
	   {
	      if (!xvw_check_subclass(selections[i], NodeWidgetClass))
		 continue;

	      xvw_get_attributes(selections[i],
			XVW_NODE_RUNNABLE, &runnable,
			XVW_NODE_MODIFIED, &modified,
			NULL);
	      if (runnable == TRUE && modified == TRUE)
	      {
		 xvw_set_attribute(selections[i], XVW_NODE_EXECUTE, TRUE);
		 break;
	      }
	   }
	   kfree(selections);
	}
        else if (kstrcmp(attribute, XVW_WORKSPACE_SHOW_CLIPBOARD) == 0)
	{
	   WorkspaceClipboardInit();
	   if (xvw_check_mapped(clipboard) == FALSE)
	   {
	      xvw_manage(clipboard);
	      xvw_map(clipboard);
	   }
	   else xvw_raise(clipboard);
	}
	else if (kstrcmp(attribute, XVW_WORKSPACE_CLEAR) == 0 ||
		 kstrcmp(attribute, XVW_CANVAS_DELETE) == 0)
	{
	   int prompt_user = *((int *) calldata);
	   int confirmed = TRUE;

	   if (prompt_user == TRUE)
	      confirmed = kprompt(KSTANDARD, NULL, NULL, 1, 
			    "Are you sure you want to clear the workspace?");
	   
	   if (confirmed)
	   {
	       kannounce(XVLANG, "WorkspaceActions", "Clearing workspace '%s'",
	 	 	 xvw_name(workspace));
               if (xwid->workspace.glyph_echo == KGLYPH_ECHO_TO_LOG ||
                   xwid->workspace.glyph_echo == KGLYPH_ECHO_TO_CONSOLE_AND_LOG)
               {
                    file = xvl_get_logfile();
                    if (file != NULL)
                    {
                        kfprintf(file, "\n#----------- Clearing workspace '%s' -----------\n", xvw_name(workspace));
                        kfclose(file);
                    }
               }

	       if (kstrcmp(attribute, XVW_WORKSPACE_CLEAR) == 0)
	       {
	          selected = FALSE;
	          kfree_and_NULL(xwid->workspace.filename);
	       }
	       else
	          selected = TRUE;

	       if (object != clipboard)
	       {
	          if (xwid->workspace.saved != NULL)
		     kfclose(xwid->workspace.saved);

	          xwid->workspace.saved = ktmpfile();
	          WorkspaceSave(widget,xwid->workspace.saved,
			        FALSE, FALSE, selected);
	          krewind(xwid->workspace.saved);
	       }
	       selections = WorkspaceChildren(workspace, selected, &num);

	       for (i = 0; i < num; i++)
	       {
		  if (xvw_check_subclass(selections[i], ExtPortWidgetClass))
		  {
		     kform        *wkspgui;
		     kselection   *sel_list, *selptr, *nextsel;
		     kform_struct *extport_sel;

                     xvw_get_attribute(selections[i], XVW_EXTPORT_SELECTION,
                                       &extport_sel);
	             xvw_get_attribute(workspace, XVW_WORKSPACE_WKSPGUI, 
			               &wkspgui);
		     selptr=sel_list = wkspgui->subform->guide->pane->sel_list;
		     while (selptr != NULL)
                     {
		        nextsel = selptr->next;
			if (selptr->back_kformstruct == extport_sel)
                           xvf_set_attribute(selptr->back_kformstruct, 
					     XVF_DELETE, TRUE);
			selptr = nextsel;
                     }
		     xvw_set_attribute(selections[i], 
			               XVW_EXTPORT_SELECTION, NULL);
                  }
	       }
	       for (i = 0; i < num; i++)
	       {
	          if (selections[i] == NULL)
		     continue;

	          if (xvw_check_subclass(selections[i], NodeWidgetClass))
	             xvw_destroy(selections[i]);
	       }
	       kfree(selections);

	       if (kstrcmp(attribute, XVW_WORKSPACE_CLEAR) == 0)
	       {
	          kfree_and_NULL(xwid->workspace.filename);
	          kexpr_variables_clear((long) workspace);
		  xvw_set_attribute(workspace, XVW_WORKSPACE_STOP, TRUE);
	       }
	       xvl_expr_call_callback((long) workspace, XVL_EXPR_DELETE);

	       /*
                *  create I/O ports on procedure glyph corresponding to
                *  the external connections that we just created.
                */
               procedure = ProcedureLocate(xvw_widget(workspace));
               if (procedure != NULL)
                  GlyphUpdateForm(xvw_widget(procedure));
	    }
	}
	else if (kstrcmp(attribute, XVW_WORKSPACE_CREATE_PROCEDURE) == 0 ||
		 kstrcmp(attribute, XVW_WORKSPACE_CREATE_COUNTLOOP) == 0 ||
		 kstrcmp(attribute, XVW_WORKSPACE_CREATE_FORLOOP)   == 0 ||
		 kstrcmp(attribute, XVW_WORKSPACE_CREATE_WHILELOOP) == 0)
	{
	   /*
	    *  Now save the contents of the current workspace and restore
	    *  it into the clipboard workspace (aka restore).
	    */
	   filename = ktempnam(NULL, "create");
	   file = kfoutput(filename); 
	   if (file == NULL)
	   {
	      kerror(XVLANG, "WorkspaceCreate", "Unable to create temporary "
			"file '%s' for procedure/loop creation.", filename);
	      return(FALSE);
	   }
	   WorkspaceSave(widget, file, FALSE, TRUE, TRUE);
	   xvw_set_attribute(workspace, XVW_CANVAS_DELETE, FALSE);

           /*
            *  Create the procedure/whileloop/countloop
            */
           if (kstrcmp(attribute, XVW_WORKSPACE_CREATE_PROCEDURE) == 0)
           {
              procedure = xvw_create_procedure(workspace, "Procedure");
              xvw_set_attribute(procedure, XVW_NODE_NAME, "Procedure");
           }
           else if (kstrcmp(attribute, XVW_WORKSPACE_CREATE_COUNTLOOP) == 0)
           {
              procedure = xvw_create_loop(workspace, "CountLoop");
              xvw_set_attributes(procedure,
                        XVW_NODE_NAME, "Count Loop",
                        XVW_LOOP_TYPE, KLOOP_TYPE_COUNT,
                        NULL);
           }
           else if (kstrcmp(attribute, XVW_WORKSPACE_CREATE_FORLOOP) == 0)
           {
              procedure = xvw_create_loop(workspace, "ForLoop");
              xvw_set_attributes(procedure,
                        XVW_NODE_NAME, "For Loop",
                        XVW_LOOP_TYPE, KLOOP_TYPE_FOR,
                        NULL);
           }
           else
           {
              procedure = xvw_create_loop(workspace, "WhileLoop");
              xvw_set_attributes(procedure,
                        XVW_NODE_NAME, "While Loop",
                        XVW_LOOP_TYPE, KLOOP_TYPE_WHILE,
                        NULL);
           }

	   /*
	    *  Restore the workspace into procedure
	    */
	   kfclose(file);
	   xvw_set_attribute(procedure, XVW_PROCEDURE_RESTORE, filename);
	   kunlink(filename);
	   kfree(filename);
	}
	else if (kstrcmp(attribute, XVW_CANVAS_DUPLICATE) == 0)
	{
	   /*
	    *  Now save the contents of the current workspace and restore
	    *  it, thus duplicating the currently selected glyphs. 
	    */
	   file = ktmpfile();
	   WorkspaceSave(widget, file, FALSE, FALSE, TRUE);
	   krewind(file);
	   WorkspaceRestore(widget, file, 10, 10);
	   kfclose(file);
	}
	else if (kstrcmp(attribute, XVW_WORKSPACE_SAVE)    == 0 ||
		 kstrcmp(attribute, XVW_WORKSPACE_SAVEALL) == 0)
	{
	   filename = *((String *) calldata);

	   if ((filename == NULL && xwid->workspace.filename == NULL) ||
	       (kstrcmp(filename, "ObtainFilename") == 0))
	   {
              char *prompt = "Filename:";

              /* prompt for file from user... */
              filename = temp; kstrcpy(temp, xwid->workspace.filename);
              if (!xvu_query_wait("Please enter filename", &prompt, "Ok",
                     &filename, 1, 55) || !kstring_cleanup(filename, filename))
		 return(FALSE);
	   }
	   else if (filename == NULL)
	      filename = xwid->workspace.filename;

	   if (koverwrite(KSTANDARD, filename))
	   {
	       kannounce(XVLANG, "WorkspaceActions", "Saving workspace '%s' to "
			"file '%s'", xvw_name(workspace), filename);

	       if ((file = kfoutput(filename)) == NULL)
	       {
	          kerror(XVLANG, "WorkspaceSave", "Unable to open file '%s' for "
		    	"saving workspace", !filename ? "(NULL)" : filename);
	          return(FALSE);
	       }
	       else if (filename != xwid->workspace.filename &&
		    kstrcmp(attribute, XVW_WORKSPACE_SAVEALL) == 0)
	       {
	          kfree_and_NULL(xwid->workspace.filename);
	          xwid->workspace.filename = kstrdup(filename);
	       }

	       if (kstrcmp(attribute, XVW_WORKSPACE_SAVE) == 0)
	          WorkspaceSave(widget, file, TRUE, FALSE, TRUE);
	       else
	          WorkspaceSave(widget, file, TRUE, FALSE, FALSE);

	       kfclose(file);
	   }
	}
	else if (kstrcmp(attribute, XVW_WORKSPACE_RESTORE) == 0)
	{
	   filename = *((String *) calldata);

	   if (filename == NULL && (filename = xvu_browse_wait(NULL)) == NULL)
	      return(FALSE);

	   kannounce(XVLANG, "WorkspaceActions", "Restoring workspace '%s' "
			"from file '%s'", xvw_name(workspace), filename);

           if (xwid->workspace.glyph_echo == KGLYPH_ECHO_TO_LOG ||
               xwid->workspace.glyph_echo == KGLYPH_ECHO_TO_CONSOLE_AND_LOG)
           {
                file = xvl_get_logfile();
                if (file != NULL)
                {
                    kfprintf(file, "\n#----------- Restoring workspace '%s' from file '%s' -----------\n", xvw_name(workspace), filename);
                    kfclose(file);
                }
           }

	   /*
	    *  Check to see if we should clear the workspace before restoring.
	    */
	   kfree(xvw_children(object, NodeWidgetClass, &num));
	   if (num > 0 && kprompt(KSTANDARD, "Yes (Clear)", "No", 1,
                "Do you wish to clear the current workspace before restoring "
		"the workspace '%s'", filename) == TRUE)
           {
              xvw_set_attribute(workspace, XVW_WORKSPACE_CLEAR, FALSE);
	   }
	   scheduling = xwid->workspace.scheduling;
	   xwid->workspace.scheduling = FALSE;

	   if ((file = kfinput(filename)) == NULL)
	   {
	      kerror(XVLANG, "WorkspaceAction", "Unable to open file '%s' for "
			 "restoring workspace", !filename ? "(NULL)" :filename);
	      return(FALSE);
	   }
	   else if (filename != xwid->workspace.filename)
	   {
	      kfree_and_NULL(xwid->workspace.filename);
	      xwid->workspace.filename = kstrdup(filename);
	   }
	   WorkspaceRestore(widget, file, 0, 0);
	   xwid->workspace.scheduling = scheduling;
	   kfclose(file);

	   if (*((String *) calldata) != filename) kfree_and_NULL(filename);
	}
	else if (kstrcmp(attribute, XVW_WORKSPACE_SAVE_WKSPGUI) == 0)
	{
	   filename = *((String *) calldata);

	   kvf_print_UIS_file(xwid->workspace.wkspgui, filename, TRUE, TRUE);
	}
	else if (kstrcmp(attribute, XVW_WORKSPACE_SHOW_WKSPGUI) == 0)
	{
	   /*
	    *  If the workspace GUI is not mapped then map it, otherwise unmap
	    *  it.
	    */
	   if (!xvw_check_mapped(xwid->workspace.wkspgui->subform->toplevel))
              xvf_map_subform(xwid->workspace.wkspgui->subform);
           else
              xvf_unmap_subform(xwid->workspace.wkspgui->subform);

	   WorkspaceGUIInitialize(workspace);
	}
	else if (kstrcmp(attribute, XVW_WORKSPACE_EXPORT_SELECTIONS) == 0)
	{
	   kannounce(XVLANG, "WorkspaceActions", "Export Selections to "
			"workspace GUI '%s'", xvw_name(workspace));

	   selections = WorkspaceChildren(workspace, TRUE, &num);
	   for (i = 0; i < num; i++)
	   {
	      if (xvw_check_subclass(selections[i], GlyphWidgetClass) == TRUE)
		 xvl_copy_pane(workspace, selections[i]);
	   }
	   kfree(selections);
	}
	else if (kstrcmp(attribute, XVW_WORKSPACE_REDRAW) == 0)
	{
	   kannounce(XVLANG, "WorkspaceActions", "Redrawing workspace '%s'",
			xvw_name(workspace));
	   xvw_refresh(object);
	}
        else if (kstrcmp(attribute, XVW_CANVAS_UNDO) == 0)
        {
	   /*
	    *  Check to see if their is a saved workspace to undo.
	    */
	   if (xwid->workspace.saved != NULL)
	   {
	      WorkspaceRestore(widget, xwid->workspace.saved, 0, 0);
	      kfclose(xwid->workspace.saved);
	      xwid->workspace.saved = NULL;
	   }
	   else
	   {
	      kerror(XVLANG, "WorkspaceActions", "No deleted workspace to undo"
			" deletion");
	   }
        }
        else if (kstrcmp(attribute, XVW_CANVAS_COPY) == 0)
        {
	   WorkspaceClipboardInit();

	   /*
	    *  Go ahead and clear the clipboard, before restoring..
	    */
	   xvw_set_attribute(clipboard, XVW_CANVAS_DELETE, FALSE);

	   /*
	    *  Now save the contents of the current workspace and restore
	    *  it into the clipboard workspace (aka restore).
	    */
	   file = ktmpfile();
	   WorkspaceSave(widget, file, FALSE, FALSE, TRUE);
	   krewind(file);
	   WorkspaceRestore(xvw_widget(clipboard), file, 0, 0);
	   kfclose(file);
        }
        else if (kstrcmp(attribute, XVW_CANVAS_PASTE) == 0)
        {
	   WorkspaceClipboardInit();

	   file = ktmpfile();
	   WorkspaceSave(xvw_widget(clipboard), file, FALSE, FALSE, TRUE);
	   krewind(file);
	   WorkspaceRestore(widget, file, 0, 0);
	   kfclose(file);
        }
        else if (kstrcmp(attribute, XVW_CANVAS_CUT) == 0)
        {
	   WorkspaceClipboardInit();

	   xvw_set_attributes(workspace,
		XVW_CANVAS_COPY, TRUE,
		XVW_CANVAS_DELETE, FALSE,
		NULL);
        }
	else if (kstrcmp(attribute, XVW_WORKSPACE_INFO) == 0)
	{
	   int loops, conditionals, procedures, glyphs, connections, nodes;

	   selections = WorkspaceChildren(workspace, FALSE, &num);
	   loops = conditionals = procedures = glyphs = connections = nodes = 0;
	   for (i = 0; i < num; i++)
	   {
	      if (xvw_class(selections[i]) == LoopWidgetClass)
		 loops++;
	      else if (xvw_class(selections[i]) == ConditionalWidgetClass)
		 conditionals++;
	      else if (xvw_class(selections[i]) == ProcedureWidgetClass)
		 procedures++;
	      else if (xvw_class(selections[i]) == GlyphWidgetClass)
		 glyphs++;
	      else if (xvw_class(selections[i]) == ConnectionGadgetClass)
		 connections++;
	      else if (xvw_class(selections[i]) == NodeWidgetClass)
		 nodes++;
	   }
	   kfree(selections);
	   kinfo(KFORCE, "For Workspace '%s':\n\n\
  Glyphs       %d\n  Procedures   %d\n  Conditionals %d\n  Loops        %d\n\
  Connections  %d\n  Nodes        %d", xvw_name(workspace), glyphs,
		procedures, conditionals, loops, connections, nodes);
	}
	else
	{
	   kerror(XVLANG, "WorkspaceActions", "invalid attribute %s",
			attribute);
	   return(FALSE);
	}
	kannounce(XVLANG, "WorkspaceActions", "");
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceInfo
|
|       Purpose: WorkspaceInfo is used to get the certain attributes/
|		 information associated with the current workspace.
|
|         Input: object    - the object for which we will be getting the
|			     attribute
|		 attribute - the desired attribute
|        Output: calldata  - the value to be gotten
|       Returns: TRUE (1) if successful, FALSE (0) otherwise
|    Written By: Mark Young & Danielle Argiro
|          Date: Jul 25, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
int WorkspaceInfo(
   xvobject workspace,
   char     *attribute,
   kaddr    calldata)
{
	if (kstrcmp(attribute, XVW_WORKSPACE_NAME) == 0)
	{
	   char **name = (char **) calldata;
	   *name = xvw_name(workspace);
	}
	else
        {
           kerror(XVLANG, "WorkspaceInfo", "invalid attribute %s", attribute);
           return(FALSE);
        }
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceAnnounce
|
|       Purpose: This routine will perform a variety of workspace announces
|		 for such things as clear, wkspgui, run, etc.
|         Input: object      - the button to announce for
|                client_data - the workspace widget
|                event       - event the activated us
|		 dispatch    - whether to continue dispatch the event
|    Written By: Mark Young
|          Date: Nov 23, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */ 
void WorkspaceAnnounce(
   xvobject object,
   kaddr    client_data,
   XEvent   *event,
   int      *dispatch)
{
	xvobject workspace = (xvobject) client_data;

	char *tmp = NULL;

	if (event->type == EnterNotify)
	{
	   kannounce(XVLANG, "WorkspaceAnnounce", "%s:  %s",
			xvw_name(workspace), tmp);
	}
	else if (event->type == LeaveNotify)
	   kannounce(XVLANG, "WorkspaceAnnounce", "");
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceActive
|
|       Purpose: This routine will update which workspace is active.  It is
|		 used to issue a callback on the top most level of the
|		 workspace callback.
|         Input: workspace - the workspace to update the active status for.
|		 whether this workspace is active or not.
|    Written By: Mark Young
|          Date: Dec 08, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
void WorkspaceActive(
   xvobject workspace,
   int  active)
{
	Widget   widget;
	xvobject object = workspace;

	while (xvw_check_subclass(xvw_parent(object), WorkspaceWidgetClass))
	    object = xvw_parent(object);

	widget = xvw_widget(object);
	if (active == FALSE) workspace = xvw_parent(workspace);
        XtCallCallbacks(widget, XVW_WORKSPACE_ACTIVE_CALLBACK, &workspace);
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceClipboardInit - routine for initializing the
|                               workspace clipboard
|
|       Purpose: This routine initializes the workspace clipboard
|
|    Written By: Mark Young
|          Date: Jul 04, 1994
|
------------------------------------------------------------*/

void WorkspaceClipboardInit(void)
{
        if (clipboard != NULL)
           return;

        clipboard = xvw_create_workspace(NULL, "Clipboard");
        xvw_set_attributes(clipboard,
                XVW_WIDTH, 750,
                XVW_HEIGHT, 512,
                NULL);
        xvw_unmanage(clipboard);
        xvw_unmap(clipboard);
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceProcedureCreate
|
|       Purpose: This routine actually creates contents of
|                the clip board.
|
|    Written By: Danielle Argiro & Mark Young
|          Date: June 21, 1996
|
------------------------------------------------------------*/

void WorkspaceProcedureCreate(
    Widget new)
{
	XvwWorkspaceWidget xwid = (XvwWorkspaceWidget) new;
        xvobject object = xvw_object(new);

        xwid->workspace.titlebar = xvw_create_titlebar(object, "TitleBar");
        xvw_set_attributes(xwid->workspace.titlebar,
                XVW_BORDER_WIDTH, 0,
                XVW_VERT_DIST,    0,
                XVW_HORIZ_DIST,   0,
                XVW_TACK_EDGE,    KMANAGER_TACK_HORIZ,
                XVW_BELOW,        NULL,
                NULL);
        xvw_raise(xwid->workspace.titlebar);
        xwid->viewport.top = xwid->workspace.titlebar;
        xvw_insert_callback(xwid->workspace.titlebar, XVW_TITLEBAR_CALLBACK,
                TRUE, WorkspaceCallbacks, object);
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceClipboardCreate
|
|       Purpose: This routine actually creates contents of
|                the clip board.
|
|    Written By: Danielle Argiro & Mark Young
|          Date: June 21, 1996
|
------------------------------------------------------------*/

void WorkspaceClipboardCreate(
    Widget new)
{
        xvobject object = xvw_object(new);

        xvobject clear, delete, undo, title, label, quit;

        title = xvw_create_manager(object, "clipboard_labelback");
	xvw_set_attributes(title, 
			   XVW_SHADOW_TYPE, KSHADOW_OUT,
			   XVW_TACK_EDGE,   KMANAGER_TACK_HORIZ,
			   NULL);

        label = xvw_create_label(title, "clipboard_label");
        xvw_set_attributes(label,
                XVW_LEFT_OF,  NULL,
                XVW_RIGHT_OF, NULL,
                XVW_ABOVE,    NULL,
                XVW_BELOW,    NULL,
                XVW_LABEL,   "Cantata  Clipboard",
                NULL);

        quit = xvw_create_button(title, "close");
        xvw_set_attributes(quit,
                XVW_LEFT_OF,    NULL,
                XVW_ABOVE,      NULL,
                XVW_BELOW,      NULL,
                XVW_HORIZ_DIST, 25,
                XVW_LABEL,     "Close",
                NULL);
        xvw_add_callback(quit, XVW_BUTTON_SELECT, 
                         WorkspaceClipboardClose, object);

        /* add button to clear clipboard */
        clear = xvw_create_button(title, "clear");
        xvw_set_attributes(clear, 
			  XVW_RIGHT_OF,  NULL,
			  XVW_ABOVE,     NULL,
                          XVW_BELOW,     NULL,
                          XVW_PIXMAP_FILENAME, "$DESIGN/objects/library/xvlang/misc/commandbar/large/clear.xpm",
			  NULL);

        xvw_add_callback(clear, XVW_BUTTON_SELECT, 
			 WorkspaceClipboardClear, NULL);

        /* add button to delete selected glyphs from clipboard */
        delete = xvw_create_button(title, "delete_select");
        xvw_set_attributes(delete,
                XVW_RIGHT_OF,  clear,
                XVW_PIXMAP_FILENAME, "$DESIGN/objects/library/xvlang/misc/commandbar/large/delete.xpm",
                NULL);
        xvw_add_callback(delete, XVW_BUTTON_SELECT, 
			 WorkspaceClipboardDelete, NULL);

        /* add button to undo deletion on clipboard */
        undo = xvw_create_button(title, "undo_delete");
        xvw_set_attributes(undo,
                XVW_RIGHT_OF,  delete,
                XVW_PIXMAP_FILENAME, "$DESIGN/objects/library/xvlang/misc/commandbar/large/undelete.xpm",
                NULL);

        xvw_add_callback(undo, XVW_BUTTON_SELECT, 
			 ClipboardUndoDelete, NULL);

}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceClipboardClose
|
|       Purpose: Closes the clip board.
|    Written By: Danielle Argiro
|          Date: June 21, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
void WorkspaceClipboardClose(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	xvw_unmap(clipboard);
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceClipboardClear
|
|       Purpose: Clears the clip board.
|    Written By: Danielle Argiro
|          Date: June 21, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
void WorkspaceClipboardClear(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{

    xvw_set_attribute(clipboard, XVW_WORKSPACE_CLEAR, TRUE); 
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceClipboardDelete
|
|       Purpose: Deletes selected glyphs from the clip board.
|    Written By: Danielle Argiro
|          Date: June 21, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
void WorkspaceClipboardDelete(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
    xvw_set_attribute(clipboard, XVW_CANVAS_DELETE, FALSE); 
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceClipboardDelete
|
|       Purpose: Undoes clip board deletion.
|    Written By: Danielle Argiro
|          Date: June 21, 1996
|
------------------------------------------------------------*/

/* ARGSUSED */
void ClipboardUndoDelete(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
    xvw_set_attribute(clipboard, XVW_CANVAS_UNDO, TRUE);
}
