/*
 * Khoros: $Id: copydata.c,v 1.7.2.1 1996/12/06 20:30:10 fas Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: copydata.c,v 1.7.2.1 1996/12/06 20:30:10 fas Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	copydata utility routine for xvlang
   >>>> 
   >>>>	Private: 
   >>>>	Static: 
   >>>>		 dispatch()
   >>>>		 nodeipc()
   >>>>		 copydata()
   >>>>	Public: 
   >>>>		 xvl_copydata()
   >>>>		 xvl_stopcopy()
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvlang/NodeP.h>

static void dispatch	PROTO((int, kaddr));
static void nodeipc	PROTO((kfile *, char *, char *, kaddr));
static void copydata	PROTO((void));


typedef struct
{
	char	 *cmd;
	xvobject oport;
} xvw_copyinfo;

static pid_t pid = 0;
static klist *datalist = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: dispatch - the dispatch callback
|
|       Purpose: This routine is the dispatch callback used to "tell" when
|		 the kcp stops.  The kcp really shouldn't stop, but for robust-
|		 ness we probably want call copydata() so that the
|		 "datalist" doesn't get stuck waiting for a kcp that will
|		 never return.
|
|    Written By: Mark Young
|          Date: Jun 29, 1996
|
------------------------------------------------------------*/
static void dispatch(
   int     status,
   kaddr   client_data)
{
	xvobject node = (xvobject) klist_identifier(datalist);
	xvw_copyinfo *info = (xvw_copyinfo *) klist_clientdata(datalist);

	if (node != NULL && xvw_widget(node) == client_data)
	{
	   if (WIFEXITED(status) == TRUE && WEXITSTATUS(status) == 0)
	   {
	      if (info->oport != NULL)
	         xvw_set_attribute(info->oport, XVW_PORT_DAV, TRUE);
	   }

	   /*
	    *  Indicate that the process is not running.
	    */
	   xvw_set_attributes(node,
		XVW_NODE_MODIFIED, FALSE,
		XVW_NODE_EXECUTE, FALSE,
		NULL);

	   /* remove the node from the data list */
	   xvl_stopcopy(node);
	}
	else
	   copydata();
}

/*-----------------------------------------------------------
|
|  Routine Name: nodeipc - routine for handling ipc messages
|
|       Purpose: This routine is called when one of the sub-process wants to
|                inform of some process update
|
|         Input: id  - the callback id
|                fid - the input file descriptor
|                client_data - not used
|    Written By: Mark Young
|          Date: Jul 1, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
void nodeipc(
   kfile *file,
   char  *type,
   char  *request,
   kaddr client_data)
{
	Widget widget = (Widget) client_data;

	char name[KLENGTH];

	if (!widget || widget->core.being_destroyed == TRUE)
	   return;

	if (kstrcmp(type, KIPC_PROCESS) == 0)
	{
	   request = kstring_cleanup(request, name);
	   if (kstrcmp(request, "next contrun") == 0)
	      dispatch(0, widget);

	   xvl_report(FALSE, "%s: (copydata) %d '%s'\n", type, pid, request);
	}
	else if (kstrcmp(type, KIPC_TRANSPORT) == 0)
	{
	   kfputs("yes\n", file);  /* ack a positive response */
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: copydata - dispatch the next entry in the list
|
|       Purpose: This routine dispatches the next entry in the "datalist".
|
|    Written By: Mark Young
|          Date: Jun 29, 1996
|
------------------------------------------------------------*/
static void copydata(void)
{
	xvobject node = (xvobject) klist_identifier(datalist);
	xvw_copyinfo *info = (xvw_copyinfo *) klist_clientdata(datalist);

	char temp[KLENGTH];
	kfile *file = NULL;
	Widget widget = xvw_widget(node);

	if (datalist == NULL)
	   return;

#if 0
	if (pid > 0 && xvw_getinfo_handler(pid, NULL, NULL, NULL, NULL, &file))
	{
	   /*
	    *  Change the clientdata which is the widget used by the Node IPC
	    *  callback routine.
	    */
	   xvw_setinfo_handler(pid, dispatch, NULL, nodeipc, widget, file);
	   ksprintf(temp, "%s\n", info->cmd);
	   kfputs(temp, file);
	}
	else
	   pid = xvw_fork(info->cmd, dispatch, NULL, nodeipc, widget);
#else
	pid = xvw_fork(info->cmd, dispatch, NULL, NULL, widget);
	/*kfprintf(kstderr, "copydata: %s\n", info->cmd);*/
#endif
}

/*-----------------------------------------------------------
|
|  Routine Name: xvl_copydata - copies data for xvlang library
|
|       Purpose: This routine copies data using a continuously running
|                kcp.
|
|         Input: node    - node to copy data for
|                infile  - the input file
|                outfile - the output file
|                oport   - the output port to set data available on
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Jun 29, 1996
|
------------------------------------------------------------*/
int xvl_copydata(
   xvobject node,
   char *infile,
   char *outfile,
   xvobject oport)
{
	Widget widget = xvw_widget(node);
	XvwNodeWidget xwid = (XvwNodeWidget) widget;

	xvw_copyinfo *info;
	char command[KLENGTH];

	if ((kstrlen(infile) == 0) || (kstrlen(outfile) == 0)) 
	    return(FALSE);

	if ((info = kcalloc((size_t) 1, sizeof(xvw_copyinfo))) == NULL)
	   return(FALSE);

	ksprintf(command, "kcp -force -i %s -o %s", infile, outfile);
	info->cmd   = kstrdup(command);

	info->oport = oport;
	if (datalist == NULL)
	{
	   datalist = klist_add(datalist, node, info);
	   copydata();
	}
	else
	   datalist = klist_add(datalist, node, info);

	if (info->oport != NULL)
	   xvw_set_attribute(info->oport, XVW_PORT_DAV, FALSE);

	xvw_set_attribute(xwid->node.icon, XVW_BACKGROUND_COLOR, "red");
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvl_stopcopy - removes an entry from the data list.
|
|       Purpose: This routine removes an entry from the data list.
|
|         Input: node    - node to copy data for
|    Written By: Mark Young
|          Date: Jun 29, 1996
|
------------------------------------------------------------*/
void xvl_stopcopy(
   xvobject node)
{
	klist *entry;
	xvw_copyinfo *info;
	kfile *file = NULL;
	Widget widget = xvw_widget(node);
	XvwNodeWidget xwid = (XvwNodeWidget) widget;

	/*
	 *  Locate the entry on the data list.
	 */
	if ((entry = klist_locate(datalist, node)) == NULL)
	   return;

	/*
	 *  Free associated data from the list.
	 */
	info = (xvw_copyinfo *) klist_clientdata(entry);
	kfree(info->cmd); kfree(info);

	/*
	 *  Remove the node from the data list and set the handler to have
	 */
	if (entry == datalist)
	{
	   xvw_getinfo_handler(pid, NULL, NULL, NULL, NULL, &file);
#if 0
	   xvw_setinfo_handler(pid, dispatch, NULL, nodeipc, NULL, file);
#else
	   xvw_setinfo_handler(pid, dispatch, NULL, NULL, NULL, file);
#endif
	   datalist = klist_delete(datalist, node);
	   copydata();
	}
	else
	   datalist = klist_delete(datalist, node);

	xvw_set_attribute(xwid->node.icon, XVW_BACKGROUND,
			xwid->core.background_pixel);
}
