 /*
  * Khoros: $Id: browser.c,v 1.13 1996/06/11 17:25:13 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: browser.c,v 1.13 1996/06/11 17:25:13 mirage Exp $";
#endif


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		        File / Alias Browser
   >>>>
   >>>>   Static:
   >>>>                 xvu_browser_cb()
   >>>>                 xvu_exit_browser()
   >>>>   Public:
   >>>>                 xvu_browse_wait()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	

static char browser_filename[KLENGTH];
static int  xvu_browser_done  = FALSE;

/*--------------------------------------------------------------
|
|  Routine Name: xvu_browser_cb()
|
|       Purpose: This callback gets called when the user selects something
|		 from the browser list of aliases or files.
|	  Input: object - the browser object
|		 client_data - not used
|		 call_data   - returns the selection from the list
|        Output: none
|    Written By: Danielle Argiro & Mark Young
|          Date: Dec 13, 1994
| Modifications:
|
--------------------------------------------------------------------*/
/* ARGSUSED */
static void xvu_browser_cb(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	char *filename = *((char **) call_data);

	if (filename != NULL)
	   kstrcpy(browser_filename, filename); 
	else browser_filename[0] = '\0';

	xvu_browser_done = TRUE;
}

/*--------------------------------------------------------------
|
|  Routine Name: xvu_exit_browser()
|
|       Purpose: This protocol handler is called when the user
|		 uses the window manager to close the browser.
|         Input: object      - the browser object
|                client_data - not used
|                call_data   - returns the selection from the list
|        Output: none
|    Written By: Danielle Argiro
|          Date: Feb 13, 1996
| Modifications:
|
--------------------------------------------------------------------*/
static void xvu_exit_browser(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
        browser_filename[0] = '\0';
        xvu_browser_done = TRUE;
	xvw_unmap(xvw_toplevel(object));
        xvw_busy(NULL, FALSE);
}


/**************************************************************
*
*  Routine Name:  xvu_browse_wait - pop up the Khoros file/alias browser;
*                                   wait for response
*
*       Purpose:  Causes the Khoros file / alias browser to pop up, and
*                 waits until the user selects a file or an alias before
*                 returning their selection.
*
*                 The first element in the list is the "-- aliases --" 
*                 entry with which the user may access the aliases capability 
*                 of the browser.  For more information on the Aliases 
*                 capability, see Chapter 1 of the Khoros User's Manual.  
*
*                 The second element of the list is "../", which is used to 
*                 move to an upper level directory.  The remaining elements of 
*                 the browser list are the contents of the specified directory.
*                 The browser remains displayed until the user selects a
*                 file from the browser, or clicks on "Cancel".
*
*                 The user may select a file from the browser list;  when
*                 this happens, the full path to the file is returned to
*                 the calling routine.
*                 
*                 Alternatively, the user may select a sub-directory to change
*                 to that directory, or "../" to go up in the directory 
*                 structure.
*
*                 A text object at the bottom of the browser allows the
*                 user to type their own filename, or change directories.
*		  If a directory path is typed into the bottom text object
*                 followed by a <cr>, files in the browser list will be 
*                 updated with the contents of the new directory.
*
*	  Input: directory - path to initial directory 
*
*        Output:
*	Returns: The string chosen from the browser by the user, or
*                NULL if the user clicked on "Cancel".
*  Restrictions: 
*    Written By: Mark Young and Danielle Argiro
*          Date: Dec 13, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
**************************************************************/

char *xvu_browse_wait(
   char *directory)
{
	char   name[KLENGTH];
	static xvobject browser = NULL;
 
	
	xvu_initialize();

	/*
	 *  xvw_create_browser creates the browser object dialog
	 */
	if (browser == NULL)
	{
	   if ((browser = xvw_create_browser(NULL, "browser")) == NULL)
	      return(NULL);

	   (void) ksprintf(name, "Browser for %s", kprog_get_program());
	   xvw_set_attribute(xvw_toplevel(browser), XVW_SHELL_TITLE, name);

	   xvw_add_callback(browser, XVW_BROWSER_CALLBACK, xvu_browser_cb,NULL);
           xvw_set_attribute(browser, XVW_BROWSER_DESTROY_ON_QUIT, FALSE);
           xvw_add_protocol(browser, "WM_DELETE_WINDOW", xvu_exit_browser,NULL);
	}
 
	if (directory != NULL)
	   xvw_set_attribute(browser, XVW_BROWSER_DIRECTORY, directory);

	xvw_busy(NULL, TRUE);
	xvw_busy(browser, FALSE);
	xvw_place(browser, NULL, 0, 0);

	/*
	 *  Do the spin until a valid filename is specified... or cancel occurs
	 */
	xvu_browser_done = FALSE;
	while (!xvu_browser_done)
	    xvw_process_event();  
 
	xvw_busy(NULL, FALSE);
	return(kstring_cleanup(browser_filename, NULL));
}
