 /*
  * Khoros: $Id: choose.c,v 1.11 1996/08/10 03:25:34 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: choose.c,v 1.11 1996/08/10 03:25:34 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		        Choose Dialog Box
   >>>>
   >>>>  Private:
   >>>>                 xvu_choose()
   >>>>                 xvu_create_choose()
   >>>>   Static:
   >>>>                 xvu_choose_callback()
   >>>>                 xvu_cancel_choose()
   >>>>                 destroy_choose_dialog()
   >>>>   Public:
   >>>>                 xvu_choose_wait()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	

static int  xvu_choose_done  = FALSE;
static int  xvu_choose_val;
static char *xvu_choose_string;

static xvobject    list;
static xvobject    text;
static xvobject    ok;
static xvobject    cancel; 

static void ChooseCallback  	     PROTO((xvobject, kaddr, kaddr));
static void destroy_choose_dialog    PROTO((xvobject));

#define CHOOSE_PIXMAP "$DESIGN/objects/library/xvutils/misc/pixmaps/choose.xpm"
#define CR_PIXMAP     "$DESIGN/objects/library/xvutils/misc/pixmaps/lightning.xpm"

/*-----------------------------------------------------------
|  Routine Name: xvu_choose
|
|       Purpose: xvu_choose produces standardized list selections for
|                X-based Khoros library routines and X based Khoros
|                applications.  It should be set up as the choose
|                handler for all X-based list items.
|
|         Input: choices       - an array of strings containing the items
|                                to select from.
|                nchoices      - The number of items in the choices
|                default_val   - The number of the default item, starting
|                                at 1. Pass in -1 for no default (0 will
|                                be returned if the user hits cancel).
|                message       - grammatically correct, clear explanation of
|                                the error that occurred. This can be formatted
|                                like a (void) printf statement.
|
|        Output: return_string  - string that holds the selected item.
|                                If it's NULL, it kmallocs the space necessary,
|                                and returns the string.
|
|                return_val     - 1 if user selected item1
|                                 2 if user selected item2
|                                 3 if user selected item3
|                                 N if user selected itemN.
|                                 0 if there was an error creating selection 
|                                   object, or the user clicked on CANCEL and 
|                                   no default value (ie, a default value of 
|                                   -1) was provided.
|                                -1 if the user_defined flag was passed in as 
|                                   TRUE, and the user entered their own item 
|                                   to use.
|
|    Written By: Danielle Argiro & Tom Sauer
|          Date: April 24, 1996 
|
------------------------------------------------------------*/
int xvu_choose(
    char **choices,
    int  nchoices,
    int  default_val,
    int  *return_val,
    char **return_string,
    char *message)
{
	*return_val = xvu_choose_wait(message, choices, nchoices,
                                        default_val, FALSE, return_string);

	return(TRUE);
}

/************************************************************
*
*  Routine Name: xvu_choose_wait - pop up a choose dialog box; wait for response
*
*       Purpose: Creates and maps a pop-up choose dialog box that consists
*                of a list with N items, and looks like:
*
*                !
*                !        -------------------------------
*                !        |                             |
*                !        |        Choose One:          |
*                !        |                             |
*                !        - - - - - - - - - - - - - - - -
*                !        |  item 1                     |
*                !        |  item 2                     |
*                !        |  item 3                     |
*                !        |  item 4                     |
*                !        |       :                     |
*                !        |  item 5                     |
*                !        - - - - - - - - - - - - - - - -
*                !        |        ----------           |
*                !        |        | Cancel |           |
*                !        |        ----------           |
*                !        -------------------------------
*                !
*
*                You may specify a prompt to replace "Choose 1".
*                You may specify a label to replace "Cancel" on the button.
*
*                If desired, you may specify the "user defined" option to
*                be TRUE, which will cause a text object to be created 
*                underneath the list.  The user may then enter their own
*                selection into the text object and hit <Enter>, which will
*                cause the new selection to appear in the list, where it
*                may be chosen.
*                
*
*                IMPORTANT NOTE: to be consistent with the standards
*                of the Khoros Software development system, you should really
*                be calling kchoose(), not xvu_choose_wait().
*
*         Input: prompt        - prompt to display at top of list object
*                item_labels   - array of strings to be displayed as items
*                                in the list
*                item_num      - size of the 'list' array
*                default_val   - number of the item in the list to return 
*                                (starting at 1 for the first item) if the 
*                                user clicks on "Cancel". Specify default of -
*                                1 if the default is to be NULL. 
*                user_defined  - flag indicating if a string object should be
*                                created at the bottom of the list object to
*                                allow the user to enter a string that is not
*                                part of the list array.  If user defined items
*                                are allowed, and the user specifies their own
*                                list item, the number returned will be -1,
*                                indicating that the user's choice was not a 
*                                member of the original list.
*                return_string - be sure to set this to NULL if you want the
*                                string allocated for you;  be sure to allocate
*                                sufficient space for the string otherwise!
*        Output: return_string - the string chosen from the list by the user
*
*	Returns: Value returned to caller will be:
*		       1 if user selected item1 
*		       2 if user selected item2 
*		       3 if user selected item3 
*		       N if user selected itemN.
*		       0 if there was an error creating selection object,
*			 or the user clicked on CANCEL and no default value
*                        (ie, a default value of -1) was provided.
*                     -1 if the user_defined flag was passed in as TRUE,
*                        and the user entered their own item to use.
*
*		  This routine will NOT return to the calling program until 
*		  one of the items is chosen from the list.  
*  Restrictions:
*    Written By: Danielle Argiro
*          Date: April 24, 1996
*      Verified:
*  Side Effects:
* Modifications: 
*
*************************************************************/

int xvu_choose_wait(
    char  *prompt,
    char  **item_labels,
    int   item_num,
    int   default_val,
    int   user_defined,
    char  **return_string)
{
	knotify	handler;

	xvu_choose_done = FALSE;

	if (xvw_display(NULL) == NULL)
	{
		kinfo(KSTANDARD, "%s\n", prompt);
		return TRUE;
	}

	handler = kset_choosehandler(NULL);
	if (!(xvu_create_choose(prompt, item_labels, item_num,
                                default_val, user_defined))) 
		return(FALSE);

	(void) kset_choosehandler(handler);

	/*
	 * errorcheck default val
	 */
	if (default_val < -1 || default_val == 0 || default_val > item_num)
	{
	    kerror("xvutils", "xvu_choose_wait", 
		   "Invalid default value provided; default value must be "
		   "between 1 and %d, or -1 for no default", item_num);

	    if (*return_string != NULL)
                    **return_string = '\0';

	    return(0);
	}

	/*
	 * set default string if provided
	 */
	if (default_val > 0)
           xvu_choose_string = kstrdup(item_labels[default_val-1]);

	while(!xvu_choose_done)
	   xvw_process_event(); 

	/* 
	 * user clicked on 'Cancel' 
	 */
	if (xvu_choose_val == 0)
	{
	    /* no default */
	    if (default_val == -1)
	    {
                if (*return_string != NULL)
                    **return_string = '\0';
	    }
	
	    /* default provided */
	    else
	    {
		if (*return_string != NULL)
		    kstrcpy(*return_string, item_labels[default_val-1]);
	        else *return_string = kstrdup(item_labels[default_val-1]);
	        xvu_choose_val = default_val;
	    }
	}

	/*
	 *  user chose something from the list or provided their own choice
	 */
        else 
	{
	   if (*return_string == NULL)
               *return_string = kstrdup(xvu_choose_string);
           else kstrcpy(*return_string, xvu_choose_string);
	}
        kfree_and_NULL(xvu_choose_string);

	return(xvu_choose_val);
}


/*------------------------------------------------------------
|
|  Routine Name: xvu_create_choose
|
|       Purpose: This routine actually creates the choose dialog box.
|
|                You may specify a prompt to replace "Choose 1".
|                You may specify a label to replace "Cancel" on the button.
|
|         Input: prompt        - prompt to display at top of list object
|                item_labels   - array of strings to be displayed as items
|                                in the list
|                item_num      - size of the 'list' array
|                default_val   - index of the item in the list to return (in
|                                xvw_list_struct form) if the user clicks on
|                                "Cancel". Specify default of -1 if the default
|                                is to be NULL.  Be aware that indices begin at
|                                zero (0), and run to (size - 1).
|                user_defined  - flag indicating if a string object should be
|                                created at the bottom of the list object to
|                                allow the user to enter a string that is not
|                                part of the list array.  If user defined items
|                                are allowed, and the user specifies their own
|                                list item, the xvw_list_struct returned will
|                                have a list_index of -1, indicating that the
|                                user's choice was not a member of the original
|                                list.
|       Returns: TRUE on success, FALSE on failure
|          Date: April 23, 1996
|    Written By: Danielle Argiro
|
------------------------------------------------------------*/
int xvu_create_choose(
    char *choose_prompt,
    char **item_labels,
    int  item_num,
    int  default_val,
    int  user_defined)
{
        xvobject    back; 
        xvobject    toplevel; 
        xvobject    prompt; 
	xvobject    separator; 
	xvobject    offset;
	xvobject    pixmap;
	xvobject    cr_pixmap;
	int         i;
	char        name[KLENGTH];
	char        prompt_string[KLENGTH];

	xvu_initialize();

        /*
         *  error checks
         */
        if (item_num < 1)
        {
            kerror("xvutils", "xvu_create_choose", "item_num must be > 0\n");
            return(FALSE);
        }
        else if (item_labels == NULL)
        {
            kerror("xvutils", "xvu_create_choose", "item_labels array is NULL");
            return(FALSE);
        }
        else
        {
            for (i = 0; i < item_num; i++)
            {
                if (item_labels[i] == NULL)
                {
                    kerror("xvutils", "xvu_create_choose",
                           "One or more NULL item_labels");
                    return(FALSE);
                }
            }
        }

	back = xvw_create(NULL, TRUE, TRUE, "choose_dialog",ManagerWidgetClass);
        toplevel = xvw_toplevel(back);

	(void) ksprintf(name, "Choose Dialog for %s", kprog_get_program());
	xvw_set_attribute(toplevel, XVW_SHELL_TITLE, name);

        xvw_set_attributes(back,
                           XVW_CHAR_WIDTH,  35.0,
                           XVW_CHAR_HEIGHT, 14.0,
                           NULL);

	ok = xvw_create_button(back, "ok");
        cancel = xvw_create_button(back, "cancel");

	xvw_set_attributes(ok,
                XVW_LABEL,         "OK",
                XVW_RIGHT_OF,       NULL,
                XVW_LEFT_OF,      cancel,
                XVW_CHAR_WIDTH,    7.0,
                XVW_CHAR_HEIGHT,   1.25,
                XVW_ABOVE,         NULL,
                XVW_VERT_DIST,     8,
                NULL);

        xvw_set_attributes(cancel,
                XVW_LABEL,         "Cancel",
                XVW_RIGHT_OF,      ok,
                XVW_LEFT_OF,       NULL,
                XVW_CHAR_WIDTH,    7.0,
                XVW_CHAR_HEIGHT,   1.25,
                XVW_ABOVE,         NULL,
                XVW_VERT_DIST,     8,
                NULL);

        xvw_add_callback(cancel, XVW_BUTTON_SELECT, ChooseCallback, toplevel);
        xvw_add_callback(ok, XVW_BUTTON_SELECT, ChooseCallback, toplevel);

	/*
         *  separator divides cancel button from the rest of the stuff
         */
        separator = xvw_create_separator(back, "separator");
        xvw_set_attributes(separator,
                           XVW_ABOVE,         cancel,
                           XVW_VERT_DIST,     8,
                           NULL);

        /*
         *  if specified, create text object in which user
         *  may specify their own string response
         */
        offset = separator;
        if (user_defined)
        {
            text = xvw_create_text(back, "user_defined");
	    cr_pixmap = xvw_create_pixmap(back, "cr_pixmap");

	    xvw_set_attributes(cr_pixmap,
	        XVW_PIXMAP_FILENAME, CR_PIXMAP,
                XVW_ABOVE,           separator,
                XVW_LEFT_OF,         NULL,
                XVW_VERT_DIST,       8,
                XVW_HORIZ_DIST,      8,
                NULL);

            xvw_set_attributes(text,
                               XVW_TACK_EDGE,    KMANAGER_TACK_HORIZ,
                               XVW_ABOVE,        separator,
                               XVW_LEFT_OF,      cr_pixmap,
                               XVW_RIGHT_OF,     NULL,
                               XVW_VERT_DIST,    8,
                               XVW_HORIZ_DIST,   8,
                               NULL);
            offset = text;
            xvw_add_action(text, "<Key>Return", ChooseCallback, toplevel, TRUE);
        }

        list = xvw_create_list(back, "list");
        prompt = xvw_create_label(back, "prompt");
	pixmap = xvw_create_pixmap(back, "pixmap");

	xvw_set_attributes(pixmap,
	        XVW_PIXMAP_FILENAME, CHOOSE_PIXMAP,
		XVW_BELOW,           NULL,
                XVW_RIGHT_OF,        NULL,
                XVW_VERT_DIST,       8,
		XVW_HORIZ_DIST,      8,
		NULL);

        /*
         *  position prompt object
         */
        if (choose_prompt != NULL)
            ksprintf(prompt_string, "%s", choose_prompt);
        else ksprintf(prompt_string, "Choose One");

        xvw_set_attributes(prompt,
                XVW_LABEL,        prompt_string,
                XVW_BELOW,        pixmap,
                XVW_ABOVE,        pixmap,
                XVW_LEFT_OF,      NULL,
                XVW_RIGHT_OF,     pixmap,
                XVW_VERT_DIST,    8,
                NULL);

        /*
         *  position list object
         */
        xvw_set_attributes(list,
                   XVW_BELOW,        pixmap,
                   XVW_TACK_EDGE,    KMANAGER_TACK_ALL,
                   XVW_ABOVE,        offset,
                   XVW_VERT_DIST,    8,
                   XVW_HORIZ_DIST,    8,
                   NULL);


        /* change the list according to specifications passed in */
        xvw_change_list(xvw_retrieve_list(list),
                        item_labels, (size_t) item_num, FALSE);
	if (default_val > 0)
	xvw_set_attribute(xvw_retrieve_list(list),
			  XVW_LIST_HIGHLT_ELEM, default_val-1);


        /* add callback to be fired when they choose an item from the list */
        xvw_add_callback(xvw_retrieve_list(list), XVW_LIST_ITEM_ACTION,
                         ChooseCallback, toplevel);
        xvw_add_callback(xvw_retrieve_list(list), XVW_LIST_ITEM_SELECT,
                         ChooseCallback, toplevel);

        xvw_add_protocol(toplevel, "WM_DELETE_WINDOW",
                         ChooseCallback, toplevel);

        /* pop up the choose object */
        xvw_place(back, cancel, 0, 0);

        return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ChooseCallback
|
|       Purpose: Callback used for all choose dialog possibilities:
|                1 - the user selects an item from the list with
|                    a single click  (list select)
|                2 - the user selects an item from the list with 
|                    a double click (list action)
|                3 - the user hits the "Ok" button
|                4 - the user hits the "Cancel" button
|
|         Input: object      - the widget that fired the callback
|                client_data - passes in the toplevel object
|                call_data   - for the list object, passes in list choice
|
|    Written By: Danielle Argiro
|          Date: May 3, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ChooseCallback(
    xvobject object,
    kaddr    client_data,
    kaddr    call_data)
{
        xvobject toplevel = (xvobject) client_data;

	/*
	 *  single- or double- click on the list
	 */
	if (object == xvw_retrieve_list(list))
	{
            xvw_list_struct *list_struct = (xvw_list_struct *) call_data;

	    /*
             *  user single-clicked on list item to highlight it only
	     *  (callback fired by XVW_LIST_ITEM_SELECT)
             */
	    if (list_struct->action == FALSE)
	    {
                xvu_choose_val     = list_struct->list_index + 1;
	        kfree_and_NULL(xvu_choose_string);
                xvu_choose_string  = kstrdup(list_struct->string);
	    }

            /*
             *  user double-clicked on list item to select it and be done
	     *  (callback fired by XVW_LIST_ITEM_ACTION)
             */
            else
            {
                destroy_choose_dialog(toplevel);
                xvu_choose_val   = list_struct->list_index + 1;
                xvu_choose_string  = kstrdup(list_struct->string);
                xvu_choose_done    = TRUE;
	    }
        }

	/*
         *  user clicked on "Cancel" button or used the window 
         *  manager to delete window, which kicks in the protocol handler
         */
	else if ((object == cancel) || (object == toplevel))
	{
            destroy_choose_dialog(toplevel);
            xvu_choose_val   = 0;
	    xvu_choose_string  = NULL;
            xvu_choose_done    = TRUE;
	
	}
	
	/*
	 *  user clicked on "Ok" button
	 */
	else if (object == ok)
	{
            destroy_choose_dialog(toplevel);
            xvu_choose_done    = TRUE;
	}

	/*
	 *  user entered value in text object
	 */
	else if (object == text)
	{
	    char *string_return;

            xvw_get_attribute(object, XVW_TEXT_STRING, &string_return);

            destroy_choose_dialog(toplevel);
	    xvu_choose_val   = -1;
            xvu_choose_string  = kstrdup(string_return);
            xvu_choose_done    = TRUE;
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: destroy_choose_dialog()
|
|       Purpose: Utility to destroy the choose dialog box.
|
|         Input: toplevel - the toplevel object
|
|    Written By: Danielle
|          Date: April 24, 1996
|
------------------------------------------------------------*/

/* ARGSUSED */
static void destroy_choose_dialog(
     xvobject toplevel)
{
        xvw_remove_protocol(toplevel, "WM_DELETE_WINDOW",
                            ChooseCallback, toplevel);
        xvw_unmap(toplevel);
        xvw_destroy(toplevel);
}


