 /*
  * Khoros: $Id: multlist.c,v 1.15 1996/06/11 17:25:20 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: multlist.c,v 1.15 1996/06/11 17:25:20 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		        MultiList Dialog Box
   >>>>
   >>>>   Static:
   >>>>                 create_multsel_list_object()
   >>>>                 list_multi_select()
   >>>>                 add_user_defined_item()
   >>>>                 ok_list()
   >>>>                 cancel_list()
   >>>>   Public:
   >>>>                 xvu_run_list_multsel_wait()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	

#define MaxNumLinesInViewport 35

static size_t xvu_list_size  = 0; 
static int xvu_list_done     = FALSE;
static int xvu_use_list      = FALSE;
static int xvu_duplicates_ok = FALSE;

static void list_multi_select        PROTO((xvobject, kaddr, kaddr));
static void add_user_defined_item    PROTO((xvobject, kaddr, kaddr));
static void ok_list                  PROTO((xvobject, kaddr, kaddr));
static void cancel_list              PROTO((xvobject, kaddr, kaddr));

static xvobject create_multsel_list_object   PROTO((char **, size_t, char *, 
					  	    char *, int, int));

xvu_list_entry  *list_head, *list_tail;
static char    **user_list, **display_list;

#define MULTILIST_PIXMAP "$DESIGN/objects/library/xvutils/misc/pixmaps/multi.xpm"

/************************************************************
*
*  Routine Name: xvu_run_list_multsel_wait - display list, wait for multiple 
*                                            choices and acknowledgement
*
*       Purpose: Takes an array of strings, and uses them to create a pop-up
*                list object.  The user is allowed to select as many different
*                strings as desired from the list; those strings are then 
*                returned to the caller with their indices in an array of 
*                xvu_list_struct's.  If the user clicks on the "Cancel" button,
*                NULL is returned.
*
*                Control is not returned to the application program until
*                the user chooses an item from the list, or clicks on "cancel".
*
*                If the "duplicates_ok" flag is passed as FALSE, the user
*                will only be allowed to select any given item from the list
*                once.  Selected items will be marked with a star.  For example,
*                the item "depth" would be displayed as "depth" if it had never
*                been selected, but would be displayed as "* depth" after it
*                was selected.  A second click on the entry would cause it to 
*                be de-selected, and again it would be displayed as "depth".  
*                The array of xvu_list_struct's returned will have all unique
*                elements;  elements will appear according to the order in
*                which selections were made by the user.
*
*                If the "duplicates_ok" flag is passed as TRUE, the user will
*                be allowed to select any item from the list as many times as
*                desired.  Each selection from the list will cause that item 
*                to have the number in front of it incremented.  For example, 
*                the item "depth" would be displayed as "depth" if it had never
*                been selected, but "(1) depth" after the first time it was 
*                selected, and "(2) depth" after the second time it was 
*                selected.  The array of xvu_list_struct's returned will have 
*                identical elements for each item that is selected multiple 
*                times.  The elements of the xvu_list_struct array will appear
*                in the order in which the items were selected by the user.
*                For example, if the user chose "depth" twice, and then "width",
*                and then "depth" again before clicking on "Ok", the array
*                of xvu_list_structs returned would appear in the following
*                order (identified by list_struct->string): depth, depth,
*                width, depth.
*
*                If the "user_defined" flag is passed as TRUE, a string object
*                will be created at the bottom of the list object where the
*                user will be allowed to enter their own new value for the list.
*                When the user hits <cr> following their newly defined list
*                item, the new value will be added to the list, and may 
*                subsequently be selected, either once (if duplicates are
*                not allowed), or many times (if duplicates are allowed).
*
*         Input: list         - array of strings to be displayed in the list
*		 size         - size of the 'list' array
*		 prompt       - prompt to display the list object
*		 label        - label to display at top of list object
*		 user_defined - flag indicating if a string object should be
*			        included to allow the user to enter a string
*				that is not part of the list array.
*		 duplicates_ok - pass TRUE if the user is allowed to
*				 select the same item twice, FALSE otherwise
*
*        Output: num - returns the number of items selected from the list.
*
*	Returns: An array of xvw_list_structs containing the strings and the 
*                indices of those strings describing the selections made from 
*                the list by the user.  Returns NULL if the user clicks on 
*                "Cancel".
*
*
*  Restrictions:
*    Written By: Danielle Argiro & Mark Young
*          Date: May 19, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvw_list_struct
**xvu_run_list_multsel_wait(
   char   *list[],
   size_t  size,
   char   *prompt,
   char   *label,
   int    user_defined,
   int    duplicates_ok,
   int    *num)
{
	char   buffer[KLENGTH];
	int    i, num_selections;
	xvobject toplevel;
	xvu_list_entry  *selection, *temp;
	xvw_list_struct **selections;

	xvu_initialize();

	xvu_list_done     = FALSE;
	xvu_use_list      = FALSE;
	xvu_duplicates_ok = duplicates_ok;

	xvu_list_size  = size;
	user_list      = (char **) kmalloc(sizeof(char *) * size);
	display_list = (char **) kmalloc(sizeof(char *) * size);

	for (i = 0; i < size; i++)
	{
	    if (xvu_duplicates_ok)
	       ksprintf(buffer,"      %s", list[i]);
	    else ksprintf(buffer,"   %s", list[i]);

	    display_list[i] = kstrdup(buffer);
	    user_list[i] = kstrdup(list[i]);
	}

	list_head = list_tail = NULL;
	toplevel = create_multsel_list_object(display_list, size, 
				prompt, label, user_defined, duplicates_ok);

	if (toplevel == NULL) return(NULL);

	while (!xvu_list_done)
	   xvw_process_event();      

       /*
        * delete the list object's toplevel from the list 
        * used by journal playback before destroying it
        */
	xvw_unmap(toplevel);
	xvw_destroy(toplevel); 

	if ((list_head == NULL) && (xvu_use_list == TRUE))
	{
	   *num = 0;
	   return(NULL);
	}
	else if (list_head == NULL)
	{
	   *num = -1;
	   return(NULL);
        }

	/*
	 *  Get the number of entries in the selection list.  This will be used
	 *  in malloc'ing the proper amount of memory for returning the list
	 *  to the user.
	 */
	selection = list_head;
	num_selections = 0;
	while (selection != NULL)
	{
	   num_selections++;
	   selection = selection->next;
	}

	/*
	 *  malloc the selections to be returned and then copy the entries.
	 *  While copying the entries we free the xvu_list_entry structures.
	 */
	selections = (xvw_list_struct **) kmalloc(num_selections *
		      sizeof(xvw_list_struct *));

	selection = list_head;
	for (i = 0; i < num_selections; i++)
	{
	   selections[i] = (xvw_list_struct *) selection->entry;
	   temp = selection;
	   selection = selection->next;
	   if (temp != NULL)
	      kfree_and_NULL(temp);
	}

	/*
	 *  return the number of elements and the selections
	 */
	karray_free(display_list, KSTRING, size, NULL);
	karray_free(user_list, KSTRING, size, NULL);
	*num = num_selections;
	return(selections);
}

/*------------------------------------------------------------
|
|   Routine Name: create_multsel_list_object()
|
|        Purpose: Creates the list object.  It takes an array of strings, 
|                 and uses each string as an item in the list. The prompt 
|                 and label strings are used to complete the list object.
|
|          Input: list         - the array of strings to be displayed as 
|                                elements of the list.
|		  size         - the size of the 'list' array
|		  prompt       - prompt to display at top of the list object
|		  label        - the label of the list object
|		  user_defined - flag indicating if a string object should be
|			        included to allow the user to enter a string
|				that is not part of the list array.
|
|        Output: Returns the toplevel of the list object
|
|    Written By: Danielle Argiro & Mark Young
|          Date: May 19, 1994
| Modifications: 
|
|
--------------------------------------------------------------------*/

static xvobject create_multsel_list_object(
   char   **list,
   size_t size,
   char   *prompt,
   char   *label,
   int    user_defined,
   int    duplicates_ok)
{
	xvobject back;		  /* backplane for list object */
	xvobject labelobj;	  /* label object */
	xvobject promptobj;	  /* prompt object */
	xvobject listobj;	  /* compound list object */
	xvobject actual_list;	  /* list component of compound list object */
	xvobject separator;       /* separator to create visual seperation  */
	xvobject cancel;	  /* button to cancel list object*/
	xvobject string_wid;      /* button to enter user-specified string */
	xvobject pixmap;
	xvobject ok;	  	  /* button to register choices */
	char	 *spec_buffer;
	char     name[KLENGTH];

	xvu_initialize();

	if (list == NULL)
	{  
	    errno = KNULL_PARAMETER;
 	    kerror("xvutils", "create_multsel_list",
			"'list' cannot be null");
	    return(NULL);
	}

	spec_buffer = (char *) kcalloc((size_t) 1, sizeof(char)*KLENGTH);
	kmemset(spec_buffer, 0, (size_t) KLENGTH);

	if (label == NULL)
	    label = kstring_copy(" ", NULL);

	if (prompt == NULL)
	    prompt = kstring_copy("Choose from:", NULL);

	/* 
	 * create backplane object 
	 */ 
	back = xvw_create(NULL, TRUE, TRUE, "multilist_dialog", 
			  ManagerWidgetClass);
	xvw_set_attributes(back,
		XVW_CHAR_WIDTH,  35.0,
		XVW_CHAR_HEIGHT, 13.0,
		NULL);

	(void) ksprintf(name, "Multilist Dialog for %s", kprog_get_program());
	xvw_set_attribute(xvw_toplevel(back), XVW_SHELL_TITLE, name);

	/* 
	 * create the pixmap object 
	 */
	pixmap = xvw_create_pixmap(back, "pixmap");
	xvw_set_attributes(pixmap,
	        XVW_PIXMAP_FILENAME, MULTILIST_PIXMAP,
		XVW_BELOW,           NULL,
                XVW_RIGHT_OF,        NULL,
                XVW_VERT_DIST,       8,
		XVW_HORIZ_DIST,      8,
		NULL);

	/* 
	 * create the label object 
	 */
        labelobj = xvw_create_label(back, "label");
        xvw_set_attributes(labelobj,
		XVW_LABEL,        label,
		XVW_BELOW,        NULL,  
		XVW_LEFT_OF,      NULL,   
		XVW_RIGHT_OF,     NULL,    
	        XVW_VERT_DIST,       8,
		NULL);

	/*
	 * create the prompt object
	 */
	promptobj = xvw_create_label(back, "prompt");
        xvw_set_attributes(promptobj,
		XVW_LABEL,        prompt,   
		XVW_BELOW,        labelobj,   
	        XVW_LEFT_OF,      NULL,    
	        XVW_RIGHT_OF,     NULL,   
		XVW_HORIZ_DIST,   8,
		NULL);

	/* 
	 * need a "Ok" button
	 */
	ok = xvw_create_button(back, "ok");
	cancel = xvw_create_button(back, "cancel");

        xvw_set_attributes(ok,
		XVW_LABEL,	 "Ok",
		XVW_RIGHT_OF,     NULL,
		XVW_LEFT_OF,      cancel,
		XVW_CHAR_WIDTH,   7.0,
		XVW_CHAR_HEIGHT,  1.25,
		XVW_VERT_DIST,    8,
		XVW_ABOVE,        NULL,
		NULL);
	xvw_add_callback(ok, XVW_BUTTON_SELECT, ok_list, NULL);

	/* 
	 * create cancel button 
	 */
        xvw_set_attributes(cancel,
		XVW_LABEL,         "Cancel",
		XVW_LEFT_OF,       NULL, 
		XVW_RIGHT_OF,      ok,
		XVW_CHAR_WIDTH,    7.0,
		XVW_CHAR_HEIGHT,   1.25,
		XVW_VERT_DIST,     8,
		XVW_ABOVE,         NULL,
		NULL);
	xvw_add_callback(cancel, XVW_BUTTON_SELECT, cancel_list, back);

	separator = xvw_create_separator(back, "separator");
	xvw_set_attributes(separator, 
			   XVW_ABOVE,     cancel,
			   XVW_VERT_DIST, 8,
                           NULL);

	/*
	 *  create the compound list object.
	 */
	listobj = xvw_create_list(back, "list");
	xvw_set_attributes(listobj, 
		XVW_BELOW,        promptobj,  
		XVW_TACK_EDGE,    KMANAGER_TACK_ALL,
		XVW_VERT_DIST,    12,
		XVW_HORIZ_DIST,   8,
		NULL);

	/*
	 *  need the actual list from the list object in order to
         *  set list contents and install the callback for list selection
	 */
	actual_list = xvw_retrieve_list(listobj);
	xvw_change_list(actual_list, list, size, FALSE);
	xvw_add_callback(actual_list, XVW_LIST_ITEM_SELECT, 
			 list_multi_select, (kaddr) &duplicates_ok);

	/* 
	 *  if specified, create text object in which user 
	 *  may specify their own string response 
	 */
	if (user_defined)
	{
	    string_wid = xvw_create_text(back, "user_defined");
	    xvw_set_attributes(string_wid,
		XVW_TACK_EDGE,	 KMANAGER_TACK_HORIZ,/* tack horizontal   */
		XVW_ABOVE,	 separator,
		NULL);
	    xvw_set_attribute(listobj, XVW_ABOVE, string_wid);
	    xvw_add_action(string_wid, "<Key>Return", 
			   add_user_defined_item, listobj, TRUE);
	}
	else xvw_set_attribute(listobj, XVW_ABOVE, separator);

	xvw_add_protocol(back, "WM_DELETE_WINDOW", cancel_list, back);
	xvw_place(back, cancel, 0, 0);
	kfree_and_NULL(spec_buffer);
	return(back);
}

/*------------------------------------------------------------
|
|  Routine Name: list_multi_select()
|
|       Purpose: Adds the selected item to the list of 
|		 selections from the list object;  marks the
|                selection in the list object with a '*' (if
|                duplicated entries are not allowed) or with
|                a number (if duplicated entries are allowed).
|
|         Input: object      - the actual list component of compound list object
|                client_data - contents of the text object
|                call_data   - returns the item that was chosen
|
|        Output: none
|    Written By: Danielle Argiro & Mark Young
|          Date: May 19, 1994
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static void list_multi_select(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	int	count = 1;
	char	buffer[10];
	xvu_list_entry *list, *new;

	/* cast the pointer to the Callback data */
	xvw_list_struct *latest = (xvw_list_struct *) call_data;
	int *duplicates_ok = (int *) client_data;

	list = list_head;
	while (list != NULL)
	{
	   if (list->entry->list_index == latest->list_index)
	   {
	      if (*duplicates_ok == FALSE)
	      {
		 display_list[latest->list_index][1] = ' ';
	         xvw_change_list(object, (String *) display_list, 
				 xvu_list_size, FALSE);

		 xvu_delete_list_entry(list);
	         return;
	      }
	      else
	      {
		 list->count++;
		 count = list->count;
		 break;
	      }
	   }
	   else list = list->next;
	}

	new = xvu_add_list_entry();
	new->entry->list_index = latest->list_index;
	new->entry->string = kstring_copy(user_list[latest->list_index], NULL);

	if (*duplicates_ok == FALSE)
	   display_list[latest->list_index][1] = '*';
	else
	{
	   (void) ksprintf(buffer,"(%1d)", count);
	   (void) kstrncpy(display_list[latest->list_index],
			   buffer, kstrlen(buffer));
	}

	xvw_change_list(object, (String *) display_list, 
			xvu_list_size, FALSE);
}

/*------------------------------------------------------------
|
|  Routine Name: add_user_defined_item()
|
|       Purpose: Adds the user-defined item to the list of 
|		 selections from the list object.
|
|         Input: object      - the ascii text object
|                client_data - not used
|                call_data   - unused
|
|        Output: none
|    Written By: Danielle Argiro & Mark Young
|          Date: May 19, 1994
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static void add_user_defined_item(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	char           buffer[KLENGTH];
	char           *string_return;
	xvobject       list_object = (xvobject) client_data;
	xvobject       actual_list;

	xvw_get_attribute(object, XVW_TEXT_STRING, &string_return);

	if (xvu_duplicates_ok)
	    ksprintf(buffer,"      %s", string_return);
	else ksprintf(buffer,"   %s", string_return);

	xvu_list_size++;
	display_list = (char **) krealloc(display_list,
					    sizeof(char *) * xvu_list_size);
	display_list[xvu_list_size-1] = kstrdup(buffer);

	user_list = (char **) krealloc(user_list,
				       sizeof(char *) * xvu_list_size);
	user_list[xvu_list_size-1] = kstrdup(string_return);

	actual_list = xvw_retrieve_list(list_object);
	xvw_change_list(actual_list, (String *) display_list, 
			xvu_list_size, FALSE);
}

/*------------------------------------------------------------
|
|  Routine Name: ok_list()
|
|       Purpose: Registers user's selection from the list object
|
|         Input: object      - the "Ok" button
|                client_data - unused
|                call_data   - unused
|
|        Output: none
|    Written By: Danielle Argiro & Mark Young
|          Date: May 19, 1994
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ok_list(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	xvu_list_done = TRUE;
	xvu_use_list = TRUE;
}

/*------------------------------------------------------------
|
|  Routine Name: cancel_list()
|
|       Purpose: Cancels list object
|
|         Input: object      - the "Cancel" button
|                client_data - unused
|                call_data   - unused
|
|        Output: none
|    Written By: Danielle Argiro & Mark Young
|          Date: May 19, 1994
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static void cancel_list(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	if (list_head != NULL)
	   xvu_delete_list_entry(list_head);
	 
	list_head = NULL;
	xvu_list_done = TRUE;
}
