 /*
  * Khoros: $Id: utilities.c,v 1.12 1996/06/11 17:25:27 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: utilities.c,v 1.12 1996/06/11 17:25:27 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		        Various Utility Routines
   >>>>
   >>>>  Private:
   >>>>                 xvu_figure_width()
   >>>>                 xvu_figure_height()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	


/*------------------------------------------------------------
|
|  Routine Name: xvu_initialize()
|
|       Purpose: Initialization routine for xvutils library.
|
|         Input: 
|        Output: 
|    Written By: Danielle Argiro
|          Date: May 3, 1996
| Modifications:
|
------------------------------------------------------------*/

void xvu_initialize(void)
{
	static int xvutils_initialized = FALSE;

	if (xvutils_initialized == FALSE)
	   return;

	/* load resources for xvutils lib */
	xvutils_initialized = TRUE;
	xvw_load_resources("$DESIGN/objects/library/xvutils/app-defaults/xvutils");
}

/*------------------------------------------------------------
|
|  Routine Name: xvu_figure_width()
|
|       Purpose: Figures the minimum width (in characters)
|		 of a text object needed to display the given 
|		 string, based on the string passed in.
|
|         Input: message - the string to be displayed in the text object
|
|        Output: Returns necessary width (in characters) for the text object.
|
|    Written By: Mark Young
|          Date: Oct 23, 1993
| Modifications: 
|
------------------------------------------------------------*/

int xvu_figure_width(
   char *message)
{
	int i, width = 0;

	if (!message)
	   return(width);

	for (i = 0; *message != '\0'; message++)
	{
	   if (*message == '\n')
	   {
	      if (i > width)
	         width = i;

	      i = 0;
	   }
	   else i++;
	}
	return(width);
}

/*------------------------------------------------------------
|
|  Routine Name: xvu_figure_height()
|
|       Purpose: Figures the minimum height (in characters)
|		 of a text object needed to display the given 
|		 string, based on the width passed in (in characters).
|
|         Input: message - the string to be displayed in the text object
|		 width   - width of the text object to be used (in characters)
|
|        Output: Returns necessary height (in characters) for the text object.
|
|    Written By: Danielle Argiro
|          Date: Jul 15, 1992
| Modifications: Converted from Khoros 1.0 (DA)
|
------------------------------------------------------------*/

int xvu_figure_height(
   char *message,
   int  width)
{
	int i, j, length, row_size, height = 1;

	if (message == NULL) return (0);

	row_size = 0;
	j = 0;
	length = kstrlen(message);
	if (length == 0) return (0);

	i = 0;
	while (i < length)
	{
	     if (message[i] != '\n') 
	     {
		j = i;
		while ((message[j] != ' ') && (message[j] != '\0'))
		{
	            row_size++;
		    j++;
		}
	     }
	     else row_size = width;

	     if (row_size >= width) 
	     {
		row_size = 0;
	        height++;
	     }
	     i += kmax(1, (j-i));
	}
	return(height);
}
