 /*
  * Khoros: $Id: signal.c,v 1.22.2.1 1996/12/06 20:30:32 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: signal.c,v 1.22.2.1 1996/12/06 20:30:32 fas Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>  Private:
   >>>>                 xvw_add_handler()
   >>>>                 xvw_remove_handler()
   >>>>                 xvw_setinfo_handler()
   >>>>                 xvw_getinfo_handler()
   >>>>                 xvw_init_handlers()
   >>>>   Static:
   >>>>                 message_callback()
   >>>>                 process_callback()
   >>>>                 contrun_callback()
   >>>>                 ipc_callback()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/*
 *  definition used by the xvw_fork() & xvw_spawn() to execute
 *  processes.
 */
typedef struct
{
	pid_t  pid;
	kaddr  client_data;
	void   (*dispatch)();
	void   (*message)();
	void   (*process)();

	kfile	  *file;
	XtInputId inputid;
} xvw_process;

static klist *process_list = NULL;


/*-----------------------------------------------------------
|
|  Routine Name: message_callback - 
|
|       Purpose: This should be a complete description that anyone
|                could understand;  it should have acceptable grammar
|                and correct spelling.
|
|         Input: client_data -
|                fid  -
|                id   -
|    Written By: Mark Young
|          Date: Jan 21, 1995
|
------------------------------------------------------------*/
/*ARGSUSED*/
static void message_callback(
   XtPointer client_data,
   int       *fid,
   XtInputId *id)
{
	xvw_process *entry = (xvw_process *) client_data;

	int   status;
	char  buffer[KLENGTH];
	void  (*dispatch)(int, kaddr);
	void  (*message)(kfile *, kaddr, char *);

	if (kfgets(buffer, KLENGTH, entry->file) != NULL)
	{
	   message = entry->message;
	   if (message != NULL)
	      message(entry->file, entry->client_data, buffer);
	   else
	      kfprintf(kstderr, "%d: %s", entry->pid, buffer);
	}
	else
	{
	   /*
	    *  Need to call the callback routine notifying that the
	    *  process is done.   First we copy the information into
	    *  local variables and remove the entry.  We do this just
	    *  in case the user does something stupid like delete the handler
	    *  while in the callback.
	    */
	   dispatch = entry->dispatch;
	   client_data = entry->client_data;
	   status = xvw_remove_handler(entry->pid);

	   if (dispatch != NULL)
	      dispatch(status, (kaddr) client_data);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: process_callback - 
|
|       Purpose: Process IPC callbacks from running routines.  Children
|		 will send IPC messages of the form:
|
|			<Type>:<pid>:message
|
|		 This routine will call the appropriate callback routine
|		 to process the IPC message.
|
|         Input: client_data -
|                fid  -
|                id   -
|    Written By: Mark Young
|          Date: Jan 21, 1995
|
------------------------------------------------------------*/
/*ARGSUSED*/
static void process_callback(
   XtPointer client_data,
   int       *fid,
   XtInputId *id)
{
	kfile *file = (kfile *) client_data;

	pid_t pid;
	klist *list;
	void  (*process)(kfile *, char *, char *, kaddr);
	char  buffer[KLENGTH], type[KLENGTH], request[KLENGTH];

	do
	{
	   /*
	    *  Read the message.  If we fail then simply return.
	    */
	   if (kfgets(buffer, KLENGTH, file) == NULL)
	      return;

	   /*
	    *  Try and parse the IPC message.  All messages should be of the
	    *  form:
	    *		<Type>:<pid>:message
	    */
	   if (ksscanf(buffer, "%[^:]:%d:%[^\n]", type, &pid, request) != 3)
	   {
	      kfprintf(kstderr, "MUNGED IPC: %s", buffer);
	      return;
	   }

	   /*
	    *  Locate the process, otherwise print error status to stderr.
	    */
	   if ((list = klist_locate(process_list, (kaddr) pid)) != NULL)
	   {
	      xvw_process *entry = klist_clientdata(list);

	      process = entry->process;
	      if (process != NULL)
	         process(entry->file, type, request, entry->client_data);
	      else
	         kfprintf(kstderr, "%d: %s '%s'\n", pid, type, request);
	   }
	} while (kfile_checkbuf(file) == TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: contrun_callback - 
|
|       Purpose: 
|
|         Input: 
|    Written By: Mark Young
|          Date: June 10, 1996
|
------------------------------------------------------------*/
/*ARGSUSED*/
static void contrun_callback(
   XtPointer     client_data,
   XtIntervalId  *id)
{
	kfile *ipc;
	char buffer[KLENGTH];
	
	/*
	 *  This needs to be done as a time out rather than right away.
	 */
	if ((ipc = kipc_getipc()) != NULL)
	{
	   ksprintf(buffer, "%s:%d: next contrun\n", KIPC_PROCESS, getpid());
	   kfputs(buffer, ipc);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: ipc_callback - 
|
|       Purpose: 
|
|         Input: 
|    Written By: Mark Young
|          Date: June 8, 1996
|
------------------------------------------------------------*/
/*ARGSUSED*/
static void ipc_callback(
   xvobject object,
   int      fid,
   kaddr    client_data)
{
	char  buffer[KLENGTH];

	if (kfgets(buffer, KLENGTH, kstdin) != NULL)
	{
	   xvw_call_detectfile(NULL, NULL);
	   contrun_callback(NULL, NULL);
	}
	else
	   xvw_remove_detectfid(object, fid, ipc_callback, client_data);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_add_handler - 
|
|       Purpose: This should be a complete description that anyone
|                could understand;  it should have acceptable grammar
|                and correct spelling.
|
|    Written By: Mark Young
|          Date: Jan 21, 1995
|
------------------------------------------------------------*/
/*ARGSUSED*/
pid_t xvw_add_handler(
   void (*dispatch) PROTO((int, kaddr)),
   void (*message) PROTO((kfile *, kaddr, char *)),
   void (*process) PROTO((kfile *, char *, char *, kaddr)),
   kaddr client_data,
   kfile *file)
{
	int   fid;
	kfile *ipc;
	xvw_process *entry;
	static int ipc_initialized = FALSE;

	/*
	 *  create a new entry in which to record the signal information.
	 *  This is where a process records it's pid and status so that when
	 *  the process is done we can call the dispatch back or display any
	 *  errors that may occur.
	 */
	if ((entry = (xvw_process *) kcalloc((size_t) 1,
			sizeof(xvw_process))) == NULL)
	{
	    kerror(NULL, "xvw_add_handler","Unable to allocate process entry"); 
	    return(-1);
	}
	entry->file = file;
	entry->pid  = kpinfo(file);
	entry->client_data = client_data;
	entry->message = message;
	entry->process = process;
	entry->dispatch = dispatch;
	process_list = klist_add(process_list, (kaddr) entry->pid, entry);

	kgetdescriptors(file, &fid, NULL);
	if (fid != -1)
	{
	   entry->inputid = XtAppAddInput(xvw_appcontext(NULL), fid, 
		(kaddr) XtInputReadMask, message_callback, (kaddr) entry);
	}

	if (process != NULL && ipc_initialized == FALSE)
	{
	   ipc_initialized = TRUE;
	   if ((ipc = kipc_start()) != NULL)
	   {
	      kgetdescriptors(ipc, &fid, NULL);
	      if (fid != -1)
	      {
	         XtAppAddInput(xvw_appcontext(NULL), fid, (kaddr)
			XtInputReadMask, process_callback, (kaddr) ipc);
	      }
	   }
	}
	return(entry->pid);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_remove_handler - library routine remove handler
|
|       Purpose: This routine is removes an existing handler.
|
|         Input: pid - the process id of the handler
|    Written By: Mark Young
|          Date: Jan 21, 1995
|
------------------------------------------------------------*/
/*ARGSUSED*/
int xvw_remove_handler(
   pid_t pid)
{
	klist *list;
	int   status;
	xvw_process *entry;

	if ((list = klist_locate(process_list, (kaddr) pid)) == NULL)
	   return(0);
	else
	   entry = klist_clientdata(list);

	/*
	 *  If we found the process then delete it from the linked list,
	 *  and free memory.
	 */
	if (kreminfo(entry->file) != pid)
	   kill(pid, SIGTERM), kill(pid, SIGKILL);

	status = kpclose(entry->file);
	if (entry->inputid != KNONE)
	   XtRemoveInput(entry->inputid);

	process_list = klist_delete(process_list, (kaddr) pid);
	kfree(entry);
	return(status);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_setinfo_handler - library routine setting process handler
|				       info
|
|       Purpose: This routine sets process handler info
|
|         Input: pid - the process id of the handler
|    Written By: Mark Young
|          Date: Jan 21, 1995
|
------------------------------------------------------------*/
/*ARGSUSED*/
int xvw_setinfo_handler(
   pid_t pid,
   void (*dispatch) PROTO((int, kaddr)),
   void (*message) PROTO((kfile *, kaddr, char *)),
   void (*process) PROTO((kfile *, char *, char *, kaddr)),
   kaddr client_data,
   kfile *file)
{
	klist *list;
	xvw_process *entry;

	if ((list = klist_locate(process_list, (kaddr) pid)) == NULL)
	   return(FALSE);
	else
	   entry = klist_clientdata(list);

	entry->file = file;
	entry->client_data = client_data;
	entry->message  = message;
	entry->process  = process;
	entry->dispatch = dispatch;
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_getinfo_handler - library routine getting process handler
|				       info
|
|       Purpose: This routine gets process handler info
|
|         Input: pid - the process id of the handler
|    Written By: Mark Young
|          Date: Jan 21, 1995
|
------------------------------------------------------------*/
/*ARGSUSED*/
int xvw_getinfo_handler(
   pid_t pid,
   void (**dispatch) PROTO((int, kaddr)),
   void (**message) PROTO((kfile *, kaddr, char *)),
   void (**process) PROTO((kfile *, char *, char *, kaddr)),
   kaddr *client_data,
   kfile **file)
{
	klist *list;
	xvw_process *entry;

	if ((list = klist_locate(process_list, (kaddr) pid)) == NULL)
	   return(FALSE);
	else
	   entry = klist_clientdata(list);

	if (file) *file = entry->file;
	if (client_data) *client_data = entry->client_data;
	if (message) *message = entry->message;
	if (process) *process = entry->process;
	if (dispatch) *dispatch = entry->dispatch;
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_init_handlers - library routine for vadd
|
|       Purpose: This should be a complete description that anyone
|                could understand;  it should have acceptable grammar
|                and correct spelling.
|
|         Input: pid - 
|    Written By: Mark Young
|          Date: Jan 21, 1995
|
------------------------------------------------------------*/
/*ARGSUSED*/
void xvw_init_handlers(void)
{
	kfile *ipc;

	/*
	 *  Currently the only handler to be installed is the IPC handler.
	 *  If the ipc has been initialized then we want to install a handler
	 *  on STDIN_FILENO in order to catch IPC requests.
	 */
	if ((ipc = kipc_getipc()) != NULL)
	{
	   knegotiate_continuous_run();
	   xvw_add_detectfid(NULL, STDIN_FILENO, ipc_callback, NULL);
	   XtAppAddTimeOut(xvw_appcontext(NULL), (unsigned long) 0, 
			   contrun_callback, NULL);
	}
}
