 /*
  * Khoros: $Id: utilities.c,v 1.15 1996/06/11 17:20:47 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: utilities.c,v 1.15 1996/06/11 17:20:47 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		General Utility Routines
   >>>>		
   >>>>  Private:
   >>>>                 xvw_widget_color()
   >>>>   Public:
   >>>>                 xvw_allocate_color()
   >>>>                 xvw_copy_colormap()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"


extern klist *colorlist;

/*-----------------------------------------------------------
|
|  Routine Name: xvw_set_mapped - action attribute to map/unmap a widget
|
|       Purpose: This is the general widget routine used to map/unmap a widget.
|
|         Input: object    - the object to have contents set
|                attribute - XVW_MAPPED
|                calldata  -
|    Written By: Mark Young
|          Date: May 11, 1996
|
------------------------------------------------------------*/
int xvw_set_mapped(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
	Widget widget;

	if (kstrcmp(attribute, XVW_MAPPED) == 0)
	{
	   int mapped = *((int *) calldata);

	   widget = xvw_widget(object);
	   XtVaSetValues(widget, XtNmappedWhenManaged, mapped, NULL);
	   if (xvw_check_realized(object))
	   {
	      if (mapped)
		 XtMapWidget(widget);
	      else
		 XtUnmapWidget(widget);
	   }

	   /*
            * if this is a toplevel, its parent will be NULL.
	    * could check this with XtIsShell, except that would
	    * miss the case of menus, which *are* a toplevel but
	    * have a menubutton parent.
            */
	   if (XtParent(widget) == NULL)
	   {
	      if (mapped)
	         xvw_add_toplevel(object);
	      else
	         xvw_remove_toplevel(object);
	   }
	}
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_get_mapped - action attribute to map/unmap a widget
|
|       Purpose: This is the general widget routine used to map/unmap a widget.
|
|         Input: object    - the object to inquire whether mapped or not
|                attribute - XVW_MAPPED
|                calldata  -
|    Written By: Mark Young
|          Date: May 11, 1996
|
------------------------------------------------------------*/
int xvw_get_mapped(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
	Widget widget;

	if (kstrcmp(attribute, XVW_MAPPED) == 0)
	{
	   int *mapped = (int *) calldata;
	   Boolean map_when = FALSE;

	   if ((widget = xvw_widget(object)) == NULL)
	      *mapped = FALSE;
           else if (!XtIsManaged(widget) && !XtIsShell(widget))
	      *mapped = FALSE;
	   else if (!XtIsWidget(widget))
	      *mapped = TRUE;
	   else if (!XtIsRealized(widget))
	      *mapped = FALSE;
           else
	   {
	      XtVaGetValues(widget, XtNmappedWhenManaged, &map_when, NULL);
	      *mapped  = map_when;
	   }
        }
	return(TRUE);
}


/*-----------------------------------------------------------
|
|  Routine Name: xvw_widget_color - private routine to actually allocate a color
|
|       Purpose: This routine will return a pixel that matches the closest
|                color, specified by the name.
|
|         Input: widget - the widget in which to allocate the color
|                colorname - the color name
|       Returns: the pixel that matches the closest
|    Written By: Mark Young
|          Date: May 04, 1995
|
------------------------------------------------------------*/
Pixel xvw_widget_color(
   Widget widget,
   char   *colorname)
{
	Display *display  = XtDisplay(widget);
	Screen   *screen  = widget->core.screen;
	Colormap colormap = widget->core.colormap;
	static int warning_issued = FALSE;

	XColor xcolor;

	if (kstrcmp(colorname, XtDefaultBackground) == 0)
	{
	   xcolor.pixel = WhitePixelOfScreen(screen);
	   return(xcolor.pixel);
	}
	else if (kstrcmp(colorname, XtDefaultForeground) == 0)
	{
	   xcolor.pixel = BlackPixelOfScreen(screen);
	   return(xcolor.pixel);
	}

	xcolor.flags = DoRed | DoGreen | DoBlue;
        if (!XParseColor(display, colormap, colorname, &xcolor))
	{
	   kinfo(KSTANDARD, "RGB color specification \"%s\" has invalid format",
			    colorname);
           return(FALSE);
	}
        else if (!XAllocColor(display, colormap, &xcolor))
	{
	   if (!warning_issued)
	   {
               kinfo(KSTANDARD, "\nWARNING:\n"
		                "Out of Colors for current colormap.\nUnable "
				"to allocate one or more requested colors.\n"
			        "You may need to use a private colormap.");
	       warning_issued = TRUE;
	   }
           return((Pixel) ~0L);
	}
	kinfo(KDEBUG, "Allocated '%s'. Number of pre-allocated colors now "
		      "'%d'\n" , colorname, klist_size(colorlist));
	return(xcolor.pixel);
}

/************************************************************
*
*  Routine Name: xvw_allocate_color - returns the pixel given the color name
*
*       Purpose: This routine will return a pixel that matches the closest
*		 color, specified by the name.
*
*         Input: object - the object in which to allocate the color
*		 colorname - the color name
*        Output:
*       Returns: The pixel that matches closest
*
*  Restrictions:
*    Written By: Mark Young
*          Date: May 04, 1995
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
Pixel xvw_allocate_color(
   xvobject object,
   char     *colorname)
{
	return(xvw_widget_color(xvw_widget(object), colorname));
}

/************************************************************
*
*  Routine Name: xvw_copy_colormap - copies existing colormap into a new one
*
*       Purpose: This routine will copy the existing color into the a new
*		 colormap.
*
*         Input: object - the object in which to copy the colormap from
*        Output:
*       Returns: the newly created colormap or None on failure
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Jun 27, 1995
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

Colormap xvw_copy_colormap(
   xvobject object)
{
	XColor   xcolor;
	Window   window;
	Visual   *visual;
	Display  *display;
	Colormap colormap, default_colormap;

	int	 i;
	Pixel	 pixel, pixels[256];
	klist    *list = colorlist;


	display = xvw_display(object);
	if ((window = xvw_window(object)) == None)
	   window = xvw_rootwindow(object);

	visual = xvw_visual(object);
	colormap = XCreateColormap(display, window, visual, AllocNone); 
	default_colormap = DefaultColormap(display, xvw_screennum(object));
	XAllocColorCells(display, colormap, TRUE, &pixel, 0, pixels, 256);

	xcolor.flags = DoRed|DoGreen|DoBlue;
	while (list != NULL)
	{
	   xcolor.pixel = (Pixel)  klist_identifier(list);
	   XQueryColor(display, default_colormap, &xcolor);
	   if (xcolor.flags != 0)
	   {
	      XFreeColors(display, colormap, &xcolor.pixel, 1, 0L);
	      XAllocColor(display, colormap, &xcolor);
	   }
	   list = klist_next(list);
	}

	for (i = 0; i < 256; i++)
	{
	   pixel = i;
	   if (klist_locate(colorlist, (kaddr) pixel) == NULL)
	      XFreeColors(display, colormap, &pixel, 1, 0L);
	}

kfprintf(kstderr,"number of colors pre-allocated '%d'\n",klist_size(colorlist));
list = colorlist;
while (list != NULL)
{
   kfprintf(kstderr,"pre-allocating '%s'\n", (char *) klist_clientdata(list));
   list = klist_next(list);
}
	return(colormap);
}
