 /*
  * Khoros: $Id: widgets.c,v 1.10 1996/05/18 14:55:20 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: widgets.c,v 1.10 1996/05/18 14:55:20 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Abstract Widget Set
   >>>>		
   >>>>   Public:
   >>>>                 xvw_initialize()
   >>>>                 xvw_create_text()
   >>>>                 xvw_create_scrollbar()
   >>>>                 xvw_create_list()
   >>>>                 xvw_change_list()
   >>>>                 xvw_retrieve_list()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/************************************************************
*
*  Routine Name: xvw_initialize - initialize the xvwidgets library
*
*       Purpose: Opens the display, sets the widget set of choice,
*                and initializes the xvwidgets library prior to use.
*		 A call to this routine \fImust\fP be made before any other
*		 calls to routines in GUI & Visualization services
*		 are made.
*
*         Input: menu_handler - specifies the menu handling routines. 
*                               May be one of:
*                ! XVW_MENUS_XVFORMS -  Enable use of xvforms as internal menus
*                ! XVW_MENUS_NONE    -  Disable internal menus
*        Output:
*       Returns: The newly opened display on success, NULL on failure
*
*  Restrictions: 
*    Written By: Danielle Argiro & Mark Young
*          Date: Jun 17, 1993
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: 
*                Display *xvw_initialize(
*                ! kfunc_void menu_handler)
*
*******************************************************************/

/*
 {
 *   This routine actually exists in ../Athena/initialize.c,
 *   ../Olit/initialize.c, and ../Motif/initialize.c.  The appropriate 
 *   version of xvw_initialize is linked into the application, depending 
 *   on the widget set being used. 
 }
 */


/************************************************************
*
*  Routine Name: xvw_create_scrollbar - create scrollbar GUI object
*
*       Purpose: The \fIscrollbar\fP GUI object is a slider-like mechanism 
*		 called a \fIthumb\fP which the user may move back and forth 
*		 using the mouse to set a value.  
*
*		 A callback can be installed on the scrollbar for either 
*		 incremental movement or continuous movement (or both),
*		 which will be activated when the user moves the thumb of 
*		 the scrollbar.
*
*		 Creation of the scrollbar GUI object results in an 
*		 instantiation of a \fIscrollbar\fP widget in the Athena,
*                Olit, and Motif widget sets.
*
*         Input: parent - parent of the scrollbar object
*                name   - a name for this particular instance of the scrollbar
*                         object (for use in app-defaults files, etc)
*        Output:
*       Returns: The scrollbar GUI object on success, NULL on failure.
*
*  Restrictions: Must be created as the child of a manager object,
*                viewport object, or rowcol object.
*    Written By: Danielle Argiro & Mark Young
*          Date: Apr 9, 1993
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: xvobject xvw_create_scrollbar(
*                !   xvobject parent,
*                !   char   *name)
*
************************************************************/

/*
 {
 *   This routine actually exists in ../Athena/Athena.c, ../Olit/Olit.c,
 *   and ../Motif/Motif.c.  The appropriate version of xvw_create_scrollbar
 *   is linked into the application, depending on the widget set being used. 
 }
 */


/************************************************************
*
*  Routine Name: xvw_create_list - create list GUI object
*
*       Purpose: The \fIlist\fP GUI object presents the user with a set of 
*		 strings inside a scrolled viewport. A callback can be 
*		 installed on the list object which will be fired when 
*		 the user selects an item from the list.
*
*		 A list GUI object is really a \fIcompound\fP object;
*                that is, it is made up of two objects: the viewport and the
*                "actual list", where the viewport contains the list and
*                provides scrolling capabilities, and the "actual list" is
*                inside the viewport and contains the list contents.
*
*                When you create a list, the GUI object returned is the
*                viewport object; this object should be referenced when
*                setting geometry, relative offset, and so on (in other words,
*                when setting or getting any attribute not having to do
*                \fIdirectly\fP with the contents of the list, where those
*		 particular attributes are called out below).
*
*                The "actual list" part of the object is used when changing
*                the contents of the list, setting the size of the list, and
*                highlighting or unhighlighting elements of the list.  The
*                "actual list" part of the compound list object can be obtained
*                with a call to xvw_retrieve_list().
*
*		 When the Athena widget set is used, the list object is
*		 instantiated as a xfwfScrolledListWidget.  When the Olit
*		 widget set is used, the list object is instantiated as a
*		 scrollingListWidget.  When the Motif widget set is used,
*		 the list object is instantiated as a ScrolledList.
*
*         Input: parent - parent of the list object
*                name   - a name for this particular instance of the list object
*                         (for use in app-defaults files, etc)
*        Output:
*       Returns: The compound list GUI object on success, NULL on failure.
*
*  Restrictions: Must be created as the child of a manager object,
*                viewport object, or rowcol object.
*    Written By: Danielle Argiro & Mark Young
*          Date: Apr 9, 1993
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: xvobject xvw_create_list(
*                !   xvobject parent,
*                !   char   *name)
*
*******************************************************************/

/*
 {
 *   This routine actually exists in ../Athena/Athena.c, ../Olit/Olit.c,
 *   and ../Motif/Motif.c.  The appropriate version of xvw_create_list
 *   is linked into the application, depending on the widget set being used. 
 }
 */

/************************************************************
*
*  Routine Name: xvw_change_list - change the contents of a list object
*
*       Purpose: Changes the contents of a list object while it is displayed
*
*         Input: object - the list object to have contents changed
*                list   - new contents of list object
*                size   - size of the new list (# entries)
*                resize - TRUE if list object should try to resize
*                         itself after the change, FALSE otherwise.
*  Restrictions: The list object must have been previously
*                 created with a call to xvw_create_list()
*    Written By: Danielle Argiro
*          Date: Apr 9, 1993
*      Verified:
*  Side Effects:
* Modifications:
*  Declaration: xvobject xvw_change_list(
*               !     xvobject object,
*               !     char   **list,
*               !     int    size,
*               !     int    resize)
*
************************************************************/
/*
 {
 *   This routine actually exists in ../Athena/utilities.c, ../Olit/utilities.c,
 *   and ../Motif/utilities.c.  The appropriate version of xvw_change_list
 *   is linked into the application, depending on the widget set being used.
 }
*/

/************************************************************
*
*  Routine Name: xvw_retrieve_list - get the actual list component of a
*                                    compound list object
*
*       Purpose: Returns the actual list object component of
*                the compound list object returned by xvw_create_list().
*         Input: list - the compound list object for which the 
*                       actual list component is desired
*        Output: 
*       Returns: The actual list object on success, NULL on failure
*  Restrictions:
*    Written By: Danielle Argiro
*          Date: April 21, 1993
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: void xvw_retrieve_list(
*                !   xvobject list_object)
*
*******************************************************************/

/*
 {
 *   This routine actually exists in ../Athena/utilities.c, ../Olit/utilities.c,
 *   and ../Motif/utilities.c.  The appropriate version of xvw_retrieve_list
 *   is linked into the application, depending on the widget set being used.
 }
 */
