/*
 * Khoros: $Id: files.c,v 1.16.2.1 1996/12/06 20:30:51 fas Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: files.c,v 1.16.2.1 1996/12/06 20:30:51 fas Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            Initialization Routines
   >>>>
   >>>>  Private:
   >>>>   Static:
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "composer.h"
#include <kcms/private.h>

static int copy_template PROTO((kobject, char *, char *, int));

/*-----------------------------------------------------------
| Routine Name:	print_file_object - send a file object to a printer
|
| Purpose:	This routine will print the given file object.
|		It checks to see whether a PRINTER is specified in
|		the Printer selection on the `print' pane (which is
|		part of the Utilities subform), and uses that if so.
|
|		At the moment this routine just does a dumb lpr,
|		in future versions it will support printing commands,
|		and provide default ones, like lpr and enscript
|
| Input:	fileobj	-	the file object for the file to be printed.
|
| Returns:	TRUE (1) on success, FALSE (0) otherwise
|
| Written By:	Neil Bowers
| Date:		28-oct-93
------------------------------------------------------------*/
int
print_file_object(
   kobject  fileobj)
{
   kstring  filename;
   kstring  fullpath;
   kstring  printer;
   char     command[KLENGTH];


   kcms_get_attribute(fileobj, KCMS_FOBJ_FULLNAME, &filename);
   printer = gui_info->printfile->printpane->printer;
   fullpath = kfullpath(filename, NULL, NULL);

   if (printer)
      (void)ksprintf(command, "lpr -P%s %s", printer, fullpath);
   else
      (void)ksprintf(command, "lpr %s", fullpath);

   if (xvw_system(command) != 0)
   {
      kinfo(KSTANDARD, "Attempt to print file failed.\n%s.",
	    "You may want to check the print queue.");
      return FALSE;
   }
   kinfo(KSTANDARD, "File printed successfully.");
   return TRUE;
}

/*-----------------------------------------------------------
| Routine Name:	select_file_object - select (make current) a file object
|
| Purpose:	This routine will takes a file object, sets the global
|		which holds the currently selected file object, and updates
|		the user interface to reflect the select file.
|
| Input:	fileobj	-	the file object for the file to be printed.
|
| Returns:	TRUE (1) on success, FALSE (0) otherwise
|
| Written By:	Neil Bowers
| Date:		28-oct-93
------------------------------------------------------------*/
int
select_file_object(
   kobject fileobj)
{
   if (fileobj == NULL)
   {
      kerror(NULL, "Select File Object",
	     "Attempt to select a NULL file object.\n%s.",
	     "This shouldn't happen --- please report as a bug.");
      return FALSE;
   }

   if (fileobj == cur_fileobj)
      return TRUE;

   cur_fileobj = fileobj;
   update_file_info(cur_fileobj);

   return TRUE;
}

/*-----------------------------------------------------------
| Routine Name:	add_file_object - add a new file to software object
|
| Purpose:	This function is used to add a file to the software
|		object being edited.
|
| Input:	filename	-	name of the file to add.
|		should be just the filename,
|		with no path --- we assume path
|		from context.
|
| Returns:	TRUE (1) on success, FALSE (0) otherwise
|
| Written By:	Neil Bowers
| Date:		19-dec-93
| Modifications: George Ruban 10-10-95: "fixed" disallowing adds
|                when COMP_ALL_LIST selected.
------------------------------------------------------------*/
void
add_file_object(
   char *filename)
{
   kstring   routine            = "add_file_object()";
   long      ftype              = KCMS_FOBJ_TYPE_MISC;
   long      fsubtype           = KCMS_FOBJ_SUBTYPE_NONE;
   kstring   directory          = "misc";
   kstring   template_path      = NULL;
   int       template           = KCMS_TEMPLATE_MISC;
   char      fullpath[KLENGTH];
   char      temp[KLENGTH];
   kobject   file_obj;
   int       langtype           = KCMS_LANG_NONE;

   /*UPDATE: memory leak with kstring_cleanup() -----------------------*/
   if (filename == NULL || kstring_cleanup(filename, NULL) == NULL)
   {
      kerror(NULL, routine, "\nPlease give a filename.");
      return;
   }

   if (kstrchr(filename, '/') != NULL)
   {
      kerror(NULL, routine, "\nRestriction for 2.1 release:\n%s.",
	     "Please give a file name only, no path");
      return;
   }

   xvw_format_attribute(xvoNotifier, XVW_NOTIFYWINDOW_MESSAGE,
			"Adding file %s to object.", filename);
   xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, TRUE);

   if(!guess_type(filename, &ftype, &fsubtype, &directory, &template,
		   &langtype)){
     xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, FALSE);
     return;
   }

   ksprintf(fullpath, "%s/%s/%s", cmobj_opath, directory, filename);
   template_path = kcms_query_template(cmsObject, template);

   /*-- Does a file exist with the same path? -------------------------*/
   if (kaccess(fullpath, R_OK) == 0)
   {
      if (!kprompt(KFORCE, "Continue", "Cancel", TRUE,
		   "Operation: add file to software object\n\n"
		   "\tToolbox:  %s\n"
		   "\tObject:   %s\n"
		   "\tFile:     %s\n\n"
		   "A file %s already exists.\n"
		   "Do you want to continue? "
		   "(The old file will be renamed with .bak added).",
		   strToolbox, strObject, filename, fullpath))
      {
         xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, FALSE);
	 return;
      }
      ksprintf(temp, "%s.bak", fullpath);
      if (krename(fullpath, temp) != 0)
      {
	 kerror(NULL, routine, "Rename failed - aborting.");
	 xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, FALSE);
	 return;
      }
   }

   if (copy_template(cmsObject, template_path, fullpath, langtype))
   {
      file_obj = kcms_create_fileobj(cmsObject, fullpath,
				     NULL, ftype, fsubtype,
				     KCMS_FOBJ_GEN_NONE,
				     KCMS_FOBJ_ACCESS_RDWR);
      if (file_obj == NULL)
	 kerror(NULL, routine, "Unable to add the file");
      else
	 reload_object();
   }

   xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, FALSE);
}



/*-----------------------------------------------------------
| Routine Name:       add_existing_file_object - add an existing file to software object
|
| Purpose:    This function is used to add a file to the software
|             object being edited, without overwriting the file if it
|               exists.
|
| Input:      filename        -  name and/or path of the file to add.
|
| Written By: George Ruban
| Date:               02-Oct-1995
| Modifications: 08-Nov-95 disabled existence check in GUI, added here.
|                The one in the GUI was causing problems when a filename
|                of a file that did not exist remained in the input field
|                after deletion.
------------------------------------------------------------*/
void
add_existing_file_object(
   char *filename)
{
   kstring   routine            = "add_existing_file_object()";
   long      ftype;
   long      fsubtype;
   kstring   directory;
   char      fullpath[KLENGTH]; /* full path to where to install the file */
   char      basename[KLENGTH]; /* the file name itself */
   kstring   temp1, temp2;      /* for path comparison */
   char      temp[KLENGTH];
   kobject   file_obj;
   int       mustCopy = FALSE;

   /*UPDATE: memory leak with kstring_cleanup() -----------------------*/
   if (filename == NULL || kstring_cleanup(filename, NULL) == NULL)
   {
      kerror(NULL, routine, "\nPlease give a filename.");
      return;
   }

   kbasename(filename, basename);

   xvw_format_attribute(xvoNotifier, XVW_NOTIFYWINDOW_MESSAGE,
                      "Adding existing file %s to object.", basename);
   xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, TRUE);

   if(!guess_type(filename, &ftype, &fsubtype, &directory, NULL, NULL)){
     xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, FALSE);
     return;
   }

   ksprintf(fullpath, "%s/%s/%s", cmobj_opath, directory, basename);
   /* Does the file exist? */
   if (kaccess(filename, R_OK) != 0){
     kerror(NULL, routine, "The file '%s' does not exist.\n"
          "Use the add file button to create a new file.\n",
          filename);
     xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, FALSE);
     return;
   }
   /* is the specified file in the right directory? */
   temp1 = kexpandpath(filename, NULL, NULL);
   if(temp1== NULL){
     kerror(NULL, routine, "Could not kexpandpath(%s)", filename);
     xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, FALSE);
     return;
   }
   temp2 = kexpandpath(fullpath, NULL, NULL);
   if(temp2 == NULL){
     /* this often happens when the directory does not exist yet -
      ignore this error, treat as TRUE */
     mustCopy = TRUE;
   } else {
     mustCopy = kstrcmp(temp1, temp2);
     kfree(temp2);
   }
   kfree(temp1);
   if(mustCopy){


     /* no, it is not - copy it there */
     /*-- Does a file exist with the same path? -------------------------*/
     if (kaccess(fullpath, R_OK) == 0)
       {
       if (!kprompt(KFORCE, "Continue", "Cancel", TRUE,
                    "Operation: add file to software object\n\n"
                    "\tToolbox:  %s\n"
                    "\tObject:   %s\n"
                    "\tFile:     %s\n\n"
                    "A file %s already exists.\n"
                    "Do you want to continue? "
                    "(The old file will be renamed with .bak added).",
                    strToolbox, strObject, filename, fullpath))
         {
           xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, FALSE);
           return;
         }
       ksprintf(temp, "%s.bak", fullpath);
       if (krename(fullpath, temp) != 0)
         {
           kerror(NULL, routine, "Rename failed - aborting.");
           xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, FALSE);
           return;
         }
       }
     if(kcopyfile(filename, fullpath)<0){
       kerror(NULL, routine, "Copying file from %s to %s failed - aborting.",
            filename, fullpath);
       xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, FALSE);
       return;
     }
   }
   file_obj = kcms_create_fileobj(cmsObject, fullpath,
                                NULL, ftype, fsubtype,
                                KCMS_FOBJ_GEN_NONE,
                                KCMS_FOBJ_ACCESS_RDWR);
   if (file_obj == NULL)
     kerror(NULL, routine, "Unable to add the file \"%s\"", fullpath);
   else
     reload_object();
   xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, FALSE);
}


/*-----------------------------------------------------------
| Routine Name:       guess_type - figures out file type from extension
|
| Purpose:    Massive kludge to figure out file type from its name.
|

| Inputs:     filename - the file name
|
| Outputs:      pftype - the file type
|               pfsubtype - the file subtype
|               ptemplate - the template, if not originally NULL
|               pdirectory - the sub-directory name for the file
|
| Returns:    TRUE (1) on success, FALSE (0) otherwise
|
| Written By: George Ruban
| Date:               10-Oct-95
| Modifications: Nov-21-1995 GR: Added .1 and .3 type man files.
|                Dec-29-1995 GR: synchronized with 
|                 $BOOTSTRAP/include/kcms/fileobj.h
------------------------------------------------------------*/
int guess_type(const char *filename, long *pftype, long *pfsubtype,
		char **pdirectory, int *ptemplate, int *plangtype)
{
  int file_type = cur_file_type;
  long ftype     = KCMS_FOBJ_TYPE_MISC;
  long fsubtype  = KCMS_FOBJ_SUBTYPE_NONE;
  int template  = KCMS_TEMPLATE_MISC;
  int langtype  = KCMS_LANG_NONE;
  kstring directory = NULL;
  kstring routine = "guess_type()";
  char      panename[KLENGTH];
  char      formname[KLENGTH];
  const char *extension; /* .c, .h, .x, etc */
  const char *basename;
  
  basename = kstrrchr(filename, '/');
  if(basename == NULL) basename = filename;
  extension = kstrchr(basename, '.');
  
  if(extension == NULL){
    /* only miscellaneous files don't use an extension */
    file_type = COMP_MISC_LIST;
  } else if(cur_file_type == COMP_ALL_LIST){
    /* massive kludge to set up next kludge, but still better
       than just disallowing all new file adds when ALL is selected! */
    if(!kstrcmp(extension, ".pane") ||
       !kstrcmp(extension, ".form") ||
       !kstrcmp(extension, ".subform"))
      file_type = COMP_UIS_LIST;
    else if(!kstrcasecmp(extension, ".c") ||
          !kstrcmp(extension, ".h") ||
          !kstrcmp(extension, ".x") ||
          !kstrcmp(extension, ".y") ||
          !kstrcmp(extension, ".l") ||
          !kstrcmp(extension, ".lex") ||
          !kstrcasecmp(extension, ".cpp") ||
          !kstrcasecmp(extension, ".c++") ||
          !kstrcmp(extension, ".m") ||
          !kstrcmp(extension, ".cc") ||
          !kstrcasecmp(extension, ".f"))
      file_type = COMP_SRC_LIST;
    else if(!kstrcmp(extension, ".todo") ||
          !kstrcmp(extension, ".bugs") ||
          !kstrcmp(extension, ".done") ||
          !kstrcmp(extension, ".changelog"))
      file_type = COMP_INFO_LIST;
    else if(!kstrcmp(extension, ".doc") ||
          !kstrcmp(extension, ".sec") ||
          !kstrcmp(extension, ".1") ||
          !kstrcmp(extension, ".3") ||
          !kstrcmp(extension, ".hlp"))
      file_type = COMP_DOC_LIST;
    else file_type = COMP_MISC_LIST;
  } else
    file_type = cur_file_type;

  /*UPDATE--------------------------------------------------------------
    |    This next block of code is a kludge for 2.1.
    |    This information will be in a table, with mappings from
    |    kcms file object types to file extensions.  *sigh*
    +-------------------------------------------------------------------*/
  switch (file_type)
    {
    case COMP_UIS_LIST:
      directory = "uis";
      ftype = KCMS_FOBJ_TYPE_UIS;
      langtype = KCMS_LANG_KHOROS_UIS;
      ksprintf(panename, "%s.pane", strObject);
      ksprintf(formname, "%s.form", strObject);
      if (!kstrcmp(basename, panename))
      {
        fsubtype = KCMS_FOBJ_SUBTYPE_PANE;
        template = KCMS_TEMPLATE_UIS_PANE;
      }
      else if (!kstrcmp(basename, formname))
      {
        fsubtype = KCMS_FOBJ_SUBTYPE_FORM;
        template = KCMS_TEMPLATE_UIS_FORM;
      }
      else
      {
        /*-- add the file as a miscellaneous UIS file -------------*/
        fsubtype = KCMS_FOBJ_SUBTYPE_MISC;
        template = KCMS_TEMPLATE_UIS_PANE;
      }
      break;

    case COMP_SRC_LIST:
      if (obj_type == KCMS_PANE)
	kerror(NULL, routine, "Pane objects can not have source.");
      else
      {        
	directory = "src";
        ftype = KCMS_FOBJ_TYPE_SRC;
        langtype = KCMS_LANG_C;
        if (!kstrcmp(extension, ".c"))
          {
            template = (obj_type == KCMS_LIBRARY)
              ? KCMS_TEMPLATE_SRC_LIBRARY
                : KCMS_TEMPLATE_SRC_PROGRAM;
            fsubtype = KCMS_FOBJ_SUBTYPE_C;
          }
	else if (!kstrcmp(extension, ".h"))
          {
            fsubtype = KCMS_FOBJ_SUBTYPE_INCL;
            template = KCMS_TEMPLATE_SRC_INCLUDE;
   
            if (obj_type == KCMS_LIBRARY)
               if (kprompt(KFORCE, "Public", "Private", TRUE,
		   "Operation: add file to software object\n\n"
		   "\tToolbox:  %s\n"
		   "\tObject:   %s\n"
		   "\tFile:     %s\n\n"
		   "Is the file %s a public or private include file?\n",
		   strToolbox, strObject, filename, filename))
               {
                  /* public include file goes in toolbox include dir */
                  char directorybuf[KLENGTH];
	          ksprintf(directorybuf, "../../../include/%s", strObject);
                  directory = (kstring)directorybuf;
               }

          }
        else if (!kstrcasecmp(extension, ".f"))
          {
            langtype = KCMS_LANG_FORTRAN;
            template = (obj_type == KCMS_LIBRARY)
              ? KCMS_TEMPLATE_FORTRAN_LIBRARY
                : KCMS_TEMPLATE_FORTRAN_PROGRAM;
            fsubtype = KCMS_FOBJ_SUBTYPE_FORTRAN;
          }
	else if(!kstrcmp(extension, ".cc") || !kstrcmp(extension, ".C") ||
		!kstrcasecmp(extension, ".cpp") || !kstrcasecmp(extension, ".c++"))
           {
            langtype = KCMS_LANG_CPLUSPLUS;
            template = (obj_type == KCMS_LIBRARY)
              ? KCMS_TEMPLATE_SRC_LIBRARY
                : KCMS_TEMPLATE_SRC_PROGRAM;
            fsubtype = KCMS_FOBJ_SUBTYPE_CPLUSPLUS;
          }
	else if (!kstrcmp(extension, ".m"))
          {
            template = (obj_type == KCMS_LIBRARY)
              ? KCMS_TEMPLATE_SRC_LIBRARY
                : KCMS_TEMPLATE_SRC_PROGRAM;
            fsubtype = KCMS_FOBJ_SUBTYPE_OBJC;
          }
	else if (!kstrcmp(extension, ".y") )
          {
            template = (obj_type == KCMS_LIBRARY)
              ? KCMS_TEMPLATE_SRC_LIBRARY
                : KCMS_TEMPLATE_SRC_PROGRAM;
            fsubtype = KCMS_FOBJ_SUBTYPE_LEX;
          }
	else if (!kstrcmp(extension, ".l") || !kstrcmp(extension, ".lex") )
          {
            template = (obj_type == KCMS_LIBRARY)
              ? KCMS_TEMPLATE_SRC_LIBRARY
                : KCMS_TEMPLATE_SRC_PROGRAM;
            fsubtype = KCMS_FOBJ_SUBTYPE_LEX;
          }
	else if (!kstrcmp(extension, ".x") )
          {
	    fsubtype = KCMS_FOBJ_SUBTYPE_STRUCT;
	    template = KCMS_TEMPLATE_SRC_STRUCT;
          }
        else
          {
            kerror(NULL, routine, "SRC files must have an "
                   "extension of .c, .cc, .C, or .h");
            return FALSE;
          }
      }
      break;

    case COMP_MISC_LIST:
      directory = "misc";
      ftype     = KCMS_FOBJ_TYPE_MISC;
      fsubtype  = KCMS_FOBJ_SUBTYPE_NONE;
      template  = KCMS_TEMPLATE_MISC;
      break;

    case COMP_INFO_LIST:
      directory = "info";
      ftype     = KCMS_FOBJ_TYPE_INFO;
      if (!kstrcmp(extension, ".todo"))
      {
        fsubtype = KCMS_FOBJ_SUBTYPE_TODO;
        template = KCMS_TEMPLATE_TODO;
      }
      else if (!kstrcmp(extension, ".bugs"))
      {
        fsubtype = KCMS_FOBJ_SUBTYPE_BUGS;
        template = KCMS_TEMPLATE_BUGS;
      }
      else if (!kstrcmp(extension, ".done"))
      {
        fsubtype = KCMS_FOBJ_SUBTYPE_DONE;
        template = KCMS_TEMPLATE_DONE;
      }
      else if (!kstrcmp(extension, ".changelog"))
      {
        fsubtype = KCMS_FOBJ_SUBTYPE_CHANGELOG;
        template = KCMS_TEMPLATE_CHANGELOG;
      }
      else{
      kerror(NULL, routine, "INFO files must have en extension "
             "of .todo, .bugs, .done, or .changelog");
      return FALSE;
      }
      break;

    case COMP_DOC_LIST:
      langtype  = KCMS_LANG_ROFF;
      if (!kstrcmp(extension, ".hlp"))
      {
	directory = "help";
	ftype     = KCMS_FOBJ_TYPE_HELP;
        fsubtype = KCMS_FOBJ_SUBTYPE_HELP;
        template = KCMS_TEMPLATE_HLP;
      }
      else if (!kstrcmp(extension, ".doc") ||
             !kstrcmp(extension, ".sec"))
      {
	directory = "help";
	ftype     = KCMS_FOBJ_TYPE_HELP;
        fsubtype = KCMS_FOBJ_SUBTYPE_DOC;
        template = KCMS_TEMPLATE_DOC;
      }
      else if (!kstrcmp(extension, ".1"))
      {
        ftype = KCMS_FOBJ_TYPE_MAN;
        fsubtype = KCMS_FOBJ_SUBTYPE_MAN1;
        template = KCMS_TEMPLATE_MANUAL;
        directory = "man";
      }
      else if (!kstrcmp(extension, ".3"))
      {
        ftype = KCMS_FOBJ_TYPE_MAN;
        fsubtype = KCMS_FOBJ_SUBTYPE_MAN3;
        template = KCMS_TEMPLATE_MANUAL;
        directory = "man";
      }
      else{
	kerror(NULL, routine, "DOC files must have en extension "
	       "of .1, .3, .hlp, .sec, or .doc");
	return FALSE;
      }
      break;

    default:
      kerror(NULL, routine, "\"%s\":Can't figure out this type of file.",
           extension);
      return FALSE;
    }

  /* give whatever answers were requested */
  if(pftype) *pftype = ftype;
  if(pfsubtype) *pfsubtype = fsubtype;
  if(ptemplate) *ptemplate = template;
  if(pdirectory) *pdirectory = directory;
  if(plangtype) *plangtype = langtype;

  return TRUE;
}

/*-----------------------------------------------------------
| Routine Name:	_kcms_copy_template - copy template file into object
|
| Purpose:	This function copies a template file into the specified
|		object, performing substitutions for standard KCMS
|		meta-strings, object name, object type, toolbox name etc
|
| Input:	object   - the software object we're copying into.
|		from     - path of template file.
|		to       - path to copy the template file to.
|		langtype - language used for file.
|
| Returns:	TRUE (1) on success, FALSE (0) otherwise
|
| Written By:	Neil Bowers
| Date:		1-dec-93
------------------------------------------------------------*/
static int
copy_template(
   kobject  object,
   kstring  from,
   kstring  to,
   int      langtype)
{
   kstring  routine = "Copy Template";
   kobject  toolbox;
   kstring  oname;
   kstring  tbname;
   kstring  opath;
   kstring  typestring;
   kstring  short_copyright;
   kstring  long_copyright;
   char     file_base[KLENGTH];
   kstring  tmp;
   char     file_root[KLENGTH];
   int      otype;


   if (kaccess(from, R_OK) == -1)
      kerror(NULL, routine,
             "Could not access template for new file.\n"
	     "The template expected was:\n"
	     "\t%s\n"
	     "The new file was not created.\n", from);
   else if (!kcms_get_attribute(object, KCMS_CMOBJ_ONAME, &oname)
            || !kcms_get_attribute(object, KCMS_CMOBJ_OPATH, &opath)
	    || !kcms_get_attribute(object, KCMS_CMOBJ_PROGTYPE, &otype)
	    || !kcms_get_attribute(object, KCMS_CMOBJ_PARENTOBJ, &toolbox)
	    || !kcms_get_attribute(toolbox, KCMS_TB_NAME, &tbname))
      return FALSE;

   short_copyright = kcms_get_copyright(toolbox, KCMS_TB_COPYRIGHT_SHORT,
					(unsigned long) langtype);
   long_copyright = kcms_get_copyright(toolbox, KCMS_TB_COPYRIGHT_LONG,
				       (unsigned long) langtype);
   if (kbasename(to, file_base) == NULL)
   {
      kerror(NULL, routine, "Could not determine basename of destination.\n");
      return FALSE;
   }
   if ((tmp = kstrchr(file_base, '.')) == NULL)
   {
      kstrcpy(file_root, file_base);
   }
   else
   {
      kstrncpy(file_root, file_base, (size_t) (tmp - &file_base[0]));
      file_root[tmp - &file_base[0]] = '\0';
   }

   if ((typestring = kcms_attr_int2string(KCMS_CMOBJ_TYPE, (unsigned long) otype)) == NULL)
      return FALSE;
   return ksedfile(from, to, KFILE_UPDATE, NULL,
		      "#object-name#",     oname,           FALSE, 
		      "#object-type#",     typestring,      FALSE, 
		      "#object-path#",     opath,           FALSE, 
		      "#toolbox-name#",    tbname,          FALSE, 
		      "#short-copyright#", short_copyright, FALSE, 
		      "#long-copyright#",  long_copyright,  FALSE, 
		      "#file-basename#",   file_root,       FALSE, 
		      "#RCSID#",	   "$I""d$",	    FALSE,
		      "#RCSLOG#",	   "$Lo""g$",	    FALSE,
		      NULL);

}
