#include <xvinclude.h>

void  quit_program PROTO((xvobject, kaddr, XEvent *));
void  scramble_img PROTO((xvobject, kaddr, kaddr));

/*
 * This program operates very similarly to the previous scrambling image 
 * example. However, this version normalizes each region before
 * it restores the region;  this produces a patchwork effect.
 */
void main(
   int  argc,
   char *argv[])
{
	kobject  data_obj;         /* image data object             */
	xvobject manager;          /* backplane for image & buttons */
        xvobject button;           /* button to scramble/unscramble */
        xvobject img_visual_obj;   /* image visual object           */
	char     *filename = "image:moon";
	
        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	if (argc > 1)
	   filename = argv[1];

	/* initialize xvwidgets lib */
        if (!xvw_initialize(XVW_MENUS_XVFORMS))
        {
           kerror(NULL, "main", "unable to open display");
           kexit(KEXIT_FAILURE);
        }


	data_obj  = kpds_open_object(filename, KOBJ_READ);

	/* create a parent for the image & buttons */
	manager = xvw_create_manager(NULL, "parent");

	/* create the image object & associate it with the file specified */
        img_visual_obj = xvw_create_image(manager, "image");
        xvw_set_attributes(img_visual_obj,
		XVW_IMAGE_IMAGEOBJ, data_obj,
		XVW_IMAGE_BACKING, FALSE,
		NULL);

	/* create button to scramble image, centered under image */
	button = xvw_create_button(manager, "button");
	xvw_set_attributes(button, 
			   XVW_LABEL,      "Scramble",
			   XVW_BELOW,       img_visual_obj,
			   XVW_RIGHT_OF,    NULL,
			   XVW_LEFT_OF,    NULL,
			   NULL);

	/* add callback to scramble image */
	xvw_add_callback(button, XVW_BUTTON_SELECT, scramble_img, data_obj);
			
	/* add the action handler to quit on KeyPress 'q' */
	xvw_add_action(img_visual_obj, "<Key>q", quit_program,
	               data_obj, TRUE);

	/* add the event handler to scramble image on KeyPress '<' or '>'*/
	xvw_add_event(img_visual_obj, KeyPressMask, scramble_img,
	              data_obj);

	/* display & run */
        xvf_run_form();
}

/*
 *  event handler to quit program on Button Press
 */
void  quit_program(
   xvobject object,
   kaddr    client_data,
   XEvent   *event)
{
	kobject data_obj = (kobject) client_data;

	xvw_destroy(object);
	kpds_close_object(data_obj);
        kexit(KEXIT_SUCCESS);
}

/*
 * callback to scramble / normalize / and descramble image
 */
void  scramble_img(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	kobject  data_obj;
/*
	char     ch = '\0';
*/
	int      i, j, w, h, d, t, e;
 	static int num = 8;
	unsigned char *data1 = NULL, *data2 = NULL;

	data_obj = (kobject) client_data;

	/* 
	 *  Here we set scaling to normalize between 0 and 255.
	 */
	kpds_set_attributes(data_obj,
		KPDS_VALUE_SCALING,  KNORMALIZE,
                KPDS_VALUE_NORM_MIN, 0.0,
                KPDS_VALUE_NORM_MAX, 255.0,
                NULL);

	/* get the size of the segment for calculation */
	kpds_get_attribute(data_obj, KPDS_VALUE_SIZE, &w, &h, &d, &t, &e);

	/* set size of each region, given image size & region num */
	kpds_set_attribute(data_obj,
                           KPDS_VALUE_REGION_SIZE, w/num, h/num, d, t, e);

	/* circulate through the image, doing 'num' regions */
	for (i = 0; i < num; i++)
	{
	    for (j = 0; j < num; j++)
	    {
		/* swap and then restore regions */
	        kpds_set_attribute(data_obj,
				   KPDS_VALUE_POSITION, (w-w/num) - i*(w/num),
				   (h-h/num)-j*(h/num), 0, 0, 0);
	        data1 = kpds_get_data(data_obj, KPDS_VALUE_REGION, data1);

	        kpds_set_attribute(data_obj,
			           KPDS_VALUE_POSITION, i*(w/num), 
				   j*(h/num), 0, 0, 0);
	        data2 = kpds_get_data(data_obj, KPDS_VALUE_REGION, data2);

	        kpds_put_data(data_obj, KPDS_VALUE_REGION, data1);
	        kpds_set_attribute(data_obj,
				   KPDS_VALUE_POSITION, (w-w/num) - i*(w/num),
                                   (h-h/num)-j*(h/num), 0, 0, 0);
	        kpds_put_data(data_obj, KPDS_VALUE_REGION, data2);
	    }
	}
	kfree(data1);
	kfree(data2);

	/* number used to scramble will be 8, 4, 2, 8, 4, 2 ... */
	if      (num == 8) num = 4;
	else if (num == 4) num = 2;
	else if (num == 2) num = 8;
	
}
