#include <xvinclude.h>

/* 
 * This example shows how you can combine calls to XLib with calls
 * to the xvisual library if necessary.  This example simply draws red dots
 * on top of the image, that appear when the image is exposed.  They
 * are very small but you can see them if you look closely.
 */
static void add_dots PROTO((xvobject, XEvent *, kaddr, int *));

void main(
   int  argc,
   char *argv[])
{
	xvobject image;
	char *filename = "image:moon";

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	/* initialize the xvwidgets library */
	if (!xvw_initialize(XVW_MENUS_NONE)) 
	{
	   kerror("test", "main", "unable to open display");
	   kexit(KEXIT_FAILURE);
	}

	/* create the image display */
	image = xvw_create_image(NULL, "image");

	/* associate image with data object */
	xvw_set_attribute(image, XVW_IMAGE_IMAGEFILE, filename);

	/* install event handler */
	xvw_add_event(image, ExposureMask, add_dots, NULL);

	/* display & run; there is no way to exit example but ^C */
	xvf_run_form();
}

/* 
 * event hander to add dots
 */
static void add_dots(
   xvobject object,
   XEvent   *event,
   kaddr    client_data,
   int      *dispatch)
{
	Display  *display;
	Window    window;
	int       i, j;
	GC        gc;
        XGCValues values;
	unsigned  long mask;
	Pixel     red;

	display = xvw_display(object);
	window  = xvw_window(object);

	red = xvw_allocate_color(object, "red");

	mask = GCFunction | GCForeground;
	values.function = GXcopy;
	values.foreground = red;
	gc = XCreateGC(display, window, mask, &values);

	for (i = 10; i < 500; i+= 50)
	    for (j = 10; j < 500; j+=50)
		XDrawPoint(display, window, gc, i, j);
	
	XFreeGC(display, gc);
}

