#include <xvinclude.h>

/* 
 * This example displays a big image which needs a pan icon; the image and
 * the pan icon are created in separate windows.  The panicon callback is
 * used to report our new position.
 */
void panicon_callback PROTO((xvobject, kaddr, kaddr));

void main(
   int  argc,
   char *argv[])
{
	kobject  data_object;
	xvobject image, panicon;
 	char *filename = "clust:tmi";
	int  w, h, d, t, e;
	int  desired_width  = 2000;
	int  desired_height = 2000;

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	/* initialize the xvwidgets lib */
	if (!xvw_initialize(XVW_MENUS_XVFORMS))
	{
	   kerror(NULL, "main", "unable to open display");
	   kexit(KEXIT_FAILURE);
	}

	/*
	 * If we input an image that is smaller than 2000 x 2000, scale it UP 
         * to that size, so that we will actually need a pan icon.  The pan 
	 * icon will not display unless it is needed; it is considered needed 
         * only when the image object is able to to display only a portion of 
         * it.  In general, you *don't* want to do this!
	 */
	data_object = kpds_open_object(filename, KOBJ_READ);
        kpds_get_attribute(data_object, KPDS_VALUE_SIZE, &w, &h, &d, &t, &e);
        w = kmax(w, desired_width);
        h = kmax(h, desired_height);
        kpds_set_attributes(data_object,
			    KPDS_VALUE_SIZE, w, h, d, t, e,
			    KPDS_VALUE_INTERPOLATE, KZERO_ORDER,
			    NULL);

	image = xvw_create_image(NULL, "image");
	xvw_set_attribute(image, XVW_IMAGE_IMAGEOBJ, data_object);

	/*
	 * create the pan icon;  it will only be displayed when it is necessary;
	 * although in this program we have forced it to be necessary.
         * note that setting the parent to NULL forces the pan icon to have
         * a window that is independent from the window in which the image is
         * displayed.  more often, it is useful to have a single backplane
         * hold both the image and the pan icon.
	 */
	panicon = xvw_create_panicon(NULL, "panicon");
	xvw_set_attribute(panicon, XVW_IMAGE_IMAGEOBJ, data_object);
	xvw_add_callback(panicon, XVW_PANICON_CALLBACK, panicon_callback, NULL);


	/* display & run; there is no way to exit the program but ^C */
	xvf_run_form();
}

/*
 * here's the callback for the panicon
 */
void panicon_callback(
    xvobject panicon,     /* the panicon */
    kaddr    client_data, /* not used   */
    kaddr    call_data)   /* not used   */
{
	int x, y;

	xvw_get_attributes(panicon, 
		XVW_PANICON_XPOS, &x,
		XVW_PANICON_YPOS, &y,
		NULL);

	kfprintf(kstderr, "Updating pan icon to new (%d, %d) position\n", x, y);
}
