#include <xvinclude.h>

/* 
 * This program creates a blue 3D mesh plot from info stored in an 
 * array of data points.
 */

#define PLOTSIZE 9
static Coord *set_plot_points PROTO((void));
void main(
   int  argc,
   char *argv[])
{
	xvobject parent, area, plot;
	Coord *plot_points;

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	/* initialize the xvwidgets lib */
	if (!xvw_initialize(XVW_MENUS_XVFORMS))
	{
	   kerror(NULL, "main", "unable to open display");
	   kexit(KEXIT_FAILURE);
	}

	plot_points = set_plot_points();

        /* create parent */
	parent = xvw_create_manager(NULL, "parent");
	xvw_set_attributes(parent,
			   XVW_WIDTH,  500,
			   XVW_HEIGHT, 500,
			   NULL);

        /* create area */
        area = xvw_create_area(parent, "area");
        xvw_set_attributes(area,
                           XVW_AREA_TITLE_STRING,     "3D Plot",
                           XVW_TACK_EDGE,              KMANAGER_TACK_ALL,
                           XVW_BACKGROUND_COLOR,       "black",
                           XVW_GRAPHICS_VIEWPORT_MIN_X, 0.1,
                           XVW_GRAPHICS_VIEWPORT_MIN_Y, 0.1,
                           XVW_GRAPHICS_VIEWPORT_MIN_Z, 0.1,
                           XVW_GRAPHICS_VIEWPORT_MAX_X, 0.9,
                           XVW_GRAPHICS_VIEWPORT_MAX_Y, 0.9,
                           XVW_GRAPHICS_VIEWPORT_MAX_Z, 0.9,
                           NULL );

	/* create a 3D color mesh plot from the VIFF file */
	plot = xvw_create_plot3d(area, "Plot3D");
	xvw_set_attributes(plot, 
                           XVW_PLOT3D_POINTS,           plot_points,
                           XVW_PLOT3D_PLOTSIZE,         PLOTSIZE,
                           XVW_PLOT3D_PLOTTYPE,         KPLOT3D_MESH,
                           XVW_BACKGROUND_COLOR,        "blue",
                           XVW_FOREGROUND_COLOR,        "green",
                           XVW_PLOT3D_PLOTWIDTH,        3,
			   NULL);

	/* display & run; there is no way to exit the program but ^C */
	xvf_run_form();
}

static Coord *set_plot_points(void)
{
        Coord *plot_points;

	plot_points = (Coord *) malloc(PLOTSIZE * sizeof(Coord));

        plot_points[0].x = 0.0;
        plot_points[0].y = 0.0;
        plot_points[0].z = 0.0;
        plot_points[0].d = KGRAPHICS_UNINITIALIZED;

        plot_points[1].x = 1.0;
        plot_points[1].y = 0.0;
        plot_points[1].z = 0.25;
        plot_points[1].d = KGRAPHICS_UNINITIALIZED;

        plot_points[2].x = 2.0;
        plot_points[2].y = 0.0;
        plot_points[2].z = 0.5;
        plot_points[2].d = KGRAPHICS_UNINITIALIZED;

        plot_points[3].x = 0.0;
        plot_points[3].y = 1.0;
        plot_points[3].z = 0.25;
        plot_points[3].d = KGRAPHICS_UNINITIALIZED;

        plot_points[4].x = 1.0;
        plot_points[4].y = 1.0;
        plot_points[4].z = 0.5;
        plot_points[4].d = KGRAPHICS_UNINITIALIZED;

        plot_points[5].x = 2.0;
        plot_points[5].y = 1.0;
        plot_points[5].z = 0.75;
        plot_points[5].d = KGRAPHICS_UNINITIALIZED;

        plot_points[6].x = 0.0;
        plot_points[6].y = 2.0;
        plot_points[6].z = 0.5;
        plot_points[6].d = KGRAPHICS_UNINITIALIZED;

        plot_points[7].x = 1.0;
        plot_points[7].y = 2.0;
        plot_points[7].z = 0.75;
        plot_points[7].d = KGRAPHICS_UNINITIALIZED;

        plot_points[8].x = 2.0;
        plot_points[8].y = 2.0;
        plot_points[8].z = 1.0;
        plot_points[8].d = KGRAPHICS_UNINITIALIZED;

	return(plot_points);
}
