#include <xvinclude.h>

/*
 * This example displays the plot of calculated data using
 * XVW_PLOT2D_POINTS, and sets the color of the entire plot to
 * red using XVW_FOREGROUND.
 */

#define POINTNUM 50
void main(
   int  argc,
   char *argv[])
{
	xvobject area, axis, plot;
	Coord    coords[POINTNUM];
	int      i, degrees;

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	/* initialize the xvwidgets lib */
        if (!xvw_initialize(XVW_MENUS_XVFORMS))
        {
	   kerror(NULL, "main", "Cannot open display");
	   kexit(KEXIT_FAILURE);
        }

	/* 
	 * generate data to plot 
	 * NOTE: we MUST set coord.d values to KGRAPHICS_UNINITIALIZED so we 
	 *       can later use XVW_FOREGROUND_COLOR to set the color of the plot
	 */
	for (i = 0, degrees = 720; i < POINTNUM; i++, degrees+=20)
	{
	   coords[i].x = i;
	   coords[i].y = sin((double) kdegrees_radians(degrees));
	   coords[i].d = KGRAPHICS_UNINITIALIZED;
	}

	/* create a black 500x300 area */
        area = xvw_create_area(NULL, "area");
        xvw_set_attributes(area,
                           XVW_WIDTH,             500,
                           XVW_HEIGHT,            300,
                           XVW_BACKGROUND_COLOR, "black",
                           NULL);

	/* create axis, which will dictate world view; set viewport */
        axis = xvw_create_axis2d(area, "axis");
        xvw_set_attributes(axis, 
                           XVW_GRAPHICS_ATTACH,         axis,
                           XVW_GRAPHICS_VIEWPORT_MIN_X, 0.2,
                           XVW_GRAPHICS_VIEWPORT_MIN_Y, 0.2,
                           XVW_GRAPHICS_VIEWPORT_MAX_X, 0.9,
                           XVW_GRAPHICS_VIEWPORT_MAX_Y, 0.9,
                           NULL);

	/* 
	 * create plot, and:
	 * 1 - attach plot to axis so that it gets its world view from the axis
	 * 2 - set plot size; set plot points to generated data in coord array
	 * 3 - set plot type
	 * 4 - set foreground color to red.
	 */
	plot = xvw_create_plot2d(area, "plot");
	xvw_set_attributes(plot,
                           XVW_GRAPHICS_ATTACH,   axis,
		           XVW_PLOT2D_PLOTSIZE,   POINTNUM,
		           XVW_PLOT2D_POINTS,     coords,
		           XVW_PLOT2D_PLOTTYPE,   KPLOT2D_POLYMARKER,
		           XVW_FOREGROUND_COLOR, "red",
		           NULL);

        /* bring up menuform automatically */
        xvw_activate_menu(plot);

	/* display & run */
	xvf_run_form();
}
