#include <xvinclude.h>

/*
 * This example displays the plot of the data in a KDF file by getting 
 * the data from the file and storing it in a coord array using 
 * XVW_PLOT2D_POINTS; it sets the color of each point in the plot
 * individually by setting the "d" value of the coord structure to
 * the values specified by the data.
 */

void main(
   int  argc,
   char *argv[])
{
	xvobject plot, area, axis;
	kobject  plotobj;
	int      i, w, h;
	char     *filename = "plot2d:kitten";
	double   *plotdata = NULL;
	Coord    *coord;

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	/* initialize the xvwidgets lib */
        if (!xvw_initialize(XVW_MENUS_XVFORMS))
        {
	   kerror(NULL, "main", "Cannot open display");
	   kexit(KEXIT_FAILURE);
        }
	
	/* open the KDF file, get the width & height, and the plot data */
	plotobj = kpds_open_input_object(filename);
	kpds_get_attribute(plotobj, KPDS_VALUE_SIZE, &w, &h, NULL, NULL, NULL);
	kpds_set_attribute(plotobj, KPDS_VALUE_DATA_TYPE, KDOUBLE);
	plotdata = kpds_get_data(plotobj, KPDS_VALUE_ALL, NULL);

	/* 
	 * set contents of the Coord array according to the data in the file 
	 * NOTE: each coord.d value is set, so that the plot data will specify
	 *       the color of each point in the plot as well as the Y value.
	 */
	coord = (Coord *) kmalloc((size_t)(w*h) * sizeof(Coord));
	for (i = 0; i < w*h; i++)
	{
	   coord[i].x = i;
	   coord[i].y = plotdata[i];
	   coord[i].d = (Index) plotdata[i];
	}

	/* create a black 500x300 area */
        area = xvw_create_area(NULL, "area");
	xvw_set_attributes(area,
			   XVW_WIDTH,             500,
			   XVW_HEIGHT,            300,
			   XVW_BACKGROUND_COLOR, "black", 
			   NULL);

	/* create axis, which will dictate world view; set viewport */
        axis = xvw_create_axis2d(area, "axis");
        xvw_set_attributes(axis, 
			   XVW_GRAPHICS_ATTACH, axis,
			   XVW_GRAPHICS_VIEWPORT_MIN_X, 0.2,
                           XVW_GRAPHICS_VIEWPORT_MIN_Y, 0.2,
                           XVW_GRAPHICS_VIEWPORT_MAX_X, 0.9,
                           XVW_GRAPHICS_VIEWPORT_MAX_Y, 0.9,
			   NULL);

	/* 
	 * create plot, and:
	 * 1 - attach plot to axis so that it gets its world view from the axis
	 * 2 - set plot size and and plot coordinate array
	 * 3 - set color object so that the KDF file specifies the colormap;
	 *     note that the colormap will be incorrect unless this is done.
	 */
	plot = xvw_create_plot2d(area, "plot");
	xvw_set_attributes(plot,
 			   XVW_GRAPHICS_ATTACH,  axis,
		           XVW_PLOT2D_PLOTSIZE,  w*h,
		           XVW_PLOT2D_POINTS,    coord,
		           XVW_COLOR_COLOROBJ,   plotobj,
		           NULL);

        /* bring up menuform automatically */
        xvw_activate_menu(plot);

	/* display & run */
	xvf_run_form();
}
