/*
  * Khoros: $Id: do_capture.c,v 1.7 1996/08/10 03:55:28 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_capture.c,v 1.7 1996/08/10 03:55:28 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Functionality routines for pane capture
   >>>> 
   >>>>  Private: 
   >>>> 	capture_capture
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "editimage.h"
static int capture_image PROTO((kobject, options_capture *));


/*-----------------------------------------------------------
| 
|  Routine Name: capture_capture
| 
|       Purpose: Do routine which is called when
|                  pane action button capture is used
| 
|         Input: capture_info - ptr to PaneInfo struct for capture pane
| 
|        Output: None
|    Written By: 
|          Date: Apr 04, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void capture_capture(
     options_capture *capture_info)
{
        kobject  tmp, out;

	if (capture_info->o == NULL)
	{
	    kwarn(NULL, "capture_capture", "No output file specified - "
			"captured image will only be displayed, not written "
			"out.");
	}

        /*
         * open output object
         */
	tmp = kpds_create_object();
        if (tmp == NULL)
        {
           kerror(NULL, "capture_capture",
                  "Failed to create output object");
           return;
        }

	if (!(capture_image(tmp, capture_info)))
	    return;

	current_image = tmp;
	edimg_input_newimage();

	if (capture_info->o != NULL)
	{
           kinfo(KSTANDARD, "Done writing %s output file to %s",
                  capture_info->format_label, capture_info->o);
	   out = kpds_open_output_object(capture_info->o);
           if (!kpds_set_attribute(out, KPDS_FORMAT, 
			        capture_info->format_label))
           {
              kerror(NULL, "capture_capture",
                  "Unable to set format (%s) on destination object", 
		  capture_info->format_label);
           }
	   else
	      kpds_copy_object(tmp, out);
	   kpds_close_object(out);
	}

}

static int capture_image(
    kobject         out_image,
    options_capture *capture_info)
{
        xvobject object = NULL;
        int      rubberband = FALSE;
 
	if (capture_info->win)
        {
           /*
            * Get the entire window rather than a portion or subimage of one.
            */
           kinfo(KVERBOSE, "Please click on the window to be captured.");
        }

        /*
         *  Get the object depending on whether they specified the name of
         *  window or the entire screen.  This will be passed to xvw_getimage,
         *  which will do the actual work of capturing the window's image.
         */
        if (capture_info->screen == TRUE)
           object = xvw_getobject(NULL, xvw_rootwindow(NULL), NULL);
        else
           object = xvw_getobject(NULL, (Window) KNONE, NULL);

	/*
         *  rubberbanding a portion of the screen or the specified window
         */
        if (!capture_info->all)
        {
           /*
            * Get a portion or subimage of the window/screen rather than
            * entire one.
            */
           kinfo(KVERBOSE,"Please rubberband the portion of the screen to be captured\n\n");
           rubberband = TRUE;
        }


        /*
         *  Now that we have opened the output data object, call xvw_getimage
         *  to do the actual work of getting the screen dump.  We pass in
         *  object for the xvobject (the first parameter), since this will
         *  be the object in which we will be getting the image for.
         */
        if (!xvw_getimage(object, rubberband, capture_info->decor, out_image))
        {
           kpds_close_object(out_image);
           return(FALSE);
        }

	/*
	 *  destroy the GenericWindow object that was created using
	 *  xvw_getobject()
	 */
	if (object != NULL)
	    xvw_destroy(xvw_toplevel(object));

	return(TRUE);
}


