/*
  * Khoros: $Id: create.c,v 1.16 1996/06/12 23:55:30 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: create.c,v 1.16 1996/06/12 23:55:30 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>         Functionality routines for putdata
   >>>>
   >>>>  Private:
   >>>>          create_putdata_workspace
   >>>>
   >>>>   Static:
   >>>>          create_imageicon_display
   >>>>          create_zoom_display
   >>>>          create_animate_display
   >>>>          create_plot2d_display
   >>>>          create_plot3d_display
   >>>>          create_palette_display
   >>>>          create_pixel_display
   >>>>          create_mapval_display
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "putdata.h"

xvobject position = NULL;
xvobject panicon  = NULL;

static xvobject create_image_display     PROTO((xvobject, xvobject));
static xvobject create_imageicon_display PROTO((xvobject, xvobject));
static xvobject create_zoom_display      PROTO((xvobject, xvobject));
static xvobject create_animate_display   PROTO((xvobject, xvobject));
static xvobject create_plot2d_display    PROTO((xvobject, xvobject));
static xvobject create_plot3d_display    PROTO((xvobject, xvobject));
static xvobject create_palette_display   PROTO((xvobject, xvobject));
static xvobject create_pixel_display     PROTO((xvobject, xvobject));
static xvobject create_mapval_display    PROTO((xvobject, xvobject));

/*-----------------------------------------------------------
|
|  Routine Name: create_putdata_workspace()
|
|       Purpose: Creates the display workspace for putdata 
|
|         Input: 
|        Output: 
|       Returns: 
|    Written By: Danielle Argiro & Mark Young
|          Date: April 12, 1996
| Modifications:
|
------------------------------------------------------------*/

void create_putdata_workspace(void)
{
	char	 temp[KLENGTH];
	int	 width, height;
	xvobject parent, quit, help, menu, object = NULL;

	/*
         * initialize xvwidgets
         */
	if (!xvw_initialize(XVW_MENUS_XVFORMS))
        {
           kerror(NULL, "create_putdata_workspace", 
		  "Unable to open display");
           kexit(KEXIT_FAILURE);
        }

	/*
	 *  create parent object
	 */
	ksprintf(temp, "Putdata: %s", !clui_info->i_file ? "(Unknown)" :
			clui_info->i_file);
	parent = xvw_create_manager(NULL, temp);


	/*
	 *  create quit button
	 */
	if (!clui_info->imageicon_flag)
	{
	    quit = xvw_create_button(parent, "quit");
	    xvw_set_attributes(quit,
		    XVW_LEFT_OF,     NULL,
		    XVW_LABEL,      "Quit",
		    XVW_CHAR_WIDTH,  5.0,
		    NULL);
	    xvw_add_callback(quit, XVW_BUTTON_SELECT, quit_callback, NULL);

	    /*
	     *  create help button
	     */
	    help = xvw_create_button(parent, "help");
	    xvw_set_attributes(help,
		    XVW_LEFT_OF,     quit,
		    XVW_LABEL,      "Help",
		    XVW_CHAR_WIDTH,  5.0,
		    NULL);
	    xvw_add_callback(help, XVW_BUTTON_SELECT, help_callback, NULL);
	}

	/*
	 * create button which will bring up internal menuform
	 */
	menu = xvw_create_button(parent, "Internal Menu");
	xvw_set_attributes(menu,
		XVW_BELOW,	  NULL,
		XVW_RIGHT_OF,	  NULL,
		XVW_LABEL,	  "Options",
		NULL);

	/*
	 * if (x,y) (w,h) device coordinates are provided, 
	 * have the toplevel pop up at that location
	 */
	if (clui_info->x_flag     || clui_info->y_flag ||
	    clui_info->width_flag || clui_info->height_flag)
	{
	   xvobject toplevel = xvw_toplevel(parent);

	   if (clui_info->x_flag)
	      xvw_set_attribute(toplevel, XVW_SHELL_X, clui_info->x_int);
	   if (clui_info->y_flag)
	      xvw_set_attribute(toplevel, XVW_SHELL_Y, clui_info->y_int);
	   if (clui_info->width_flag)
	       xvw_set_attribute(parent, XVW_WIDTH, clui_info->width_int);
	   if (clui_info->height_flag)
	       xvw_set_attribute(parent, XVW_HEIGHT, clui_info->height_int);
	}
	width = clui_info->width_int; height = clui_info->height_int;

	/*
	 * For '% putimage' or '% putdata -image'
	 * Create image object with position & panicon
	 */
	if (clui_info->image_flag)
	    object = create_image_display(parent, menu);

	/*
         * For '% puticon' or '% putdata -imageicon'
         * Create imageicon object
	 */
	else if (clui_info->imageicon_flag)
	    object = create_imageicon_display(parent, menu);

	/*
         * For '% putzoom' or '% putdata -zoom'
         * Create zoom object 
	 */
	else if (clui_info->zoom_flag)
	    object = create_zoom_display(parent, menu);

	/*
         * For '% putanimate' or '% putdata -animate'
         * Create animate object
	 */
	else if (clui_info->animate_flag)
	    object = create_animate_display(parent, menu);

	/*
         * For '% putplot2' or '% putdata -plot2d'
         * Create area object w/ 2D plot object and 2D axis 
	 */
	else if (clui_info->plot2d_flag)
	{
	    xvw_set_attributes(parent,
		               XVW_WIDTH,  width,
		               XVW_HEIGHT, height,
		               NULL);
	    object = create_plot2d_display(parent, menu);
	}

	/*
         * For '% putplot3' or '% putdata -plot23'
         * Create area object w/ 3D plot object 
	 */
	else if (clui_info->plot3d_flag)
	{
	    xvw_set_attributes(parent,
		               XVW_WIDTH,  width,
		               XVW_HEIGHT, height,
		               NULL);
	    object = create_plot3d_display(parent, menu);
	}

	/*
         * For '% putpalette' or '% putdata -ppalette'
         * Create palette object
	 */
	else if (clui_info->palette_flag)
	{
	    object = create_palette_display(parent, menu);
	}

	/*
         * For '% putpixel' or '% putdata -pixel'
         * Create printpixel object
	 */
	else if (clui_info->pixel_flag)
	    object = create_pixel_display(parent, menu);

	/*
         * For '% putmapval' or '% putdata -mapval'
         * Create printmapval object
	 */
	else if (clui_info->mapval_flag)
	    object = create_mapval_display(parent, menu);
 
	/*
	 *  See if the user wishes to modify the default update time
	 */
	if (clui_info->update_flag || clui_info->update_float > 0.0)
	{
	   xvw_add_detectfile(NULL, clui_info->i_file, clui_info->update_float,
			detectfile_handler, object);
	   xvw_call_detectfile(NULL, NULL);
	}
	xvw_add_callback(menu, XVW_BUTTON_SELECT, menu_callback, object);
	xvw_add_action(object, "<Key>q",    quit_handler, NULL, TRUE);
	xvw_add_action(object, "<BtnDown>(2)", quit_handler, NULL, TRUE);
	if (clui_info->priv_flag)
            xvw_set_attribute(object, XVW_COLOR_PRIVATE_CMAP, 
			      clui_info->priv_logic);
}


/*-----------------------------------------------------------
|
|  Routine Name: create_image_display
|
|       Purpose: This utility routine is used when the user executes
|		 % putdata -image 
|		   -or-
|	         % putimage.
|
|	         It creates the putdata display with an image
|		 object to display the image, a position object
|		 to track the mouse, and a pan icon for when
|		 a large image is displayed.
|
|         Input: parent  - the manager backplane
|		 menu    - menubutton for Options menu
|        Output:
|    Written By: Mark Young & Danielle Argiro
|          Date: April 12, 1996
| Modifications:
|
------------------------------------------------------------*/

static xvobject create_image_display(
    xvobject parent,
    xvobject menu)
{
	xvobject image;
	kobject  data_object, tmp_data_object;
        int	 map_width; 

	/* open input object */
	data_object = kpds_open_input_object(clui_info->i_file);
	
	/* create image object */
        image = xvw_create_image(parent, "image");
        xvw_set_attributes(image,
		XVW_IMAGE_IMAGEOBJ,         data_object,
		XVW_IMAGE_XOFFSET,	    clui_info->xoffset_int,
		XVW_IMAGE_YOFFSET,	    clui_info->yoffset_int,
		XVW_IMAGE_COMPLEX_CONVERT,  clui_info->complex_list,
	        XVW_COLOR_NORM_TYPE,	    clui_info->normaltype_cycle,
		XVW_COLOR_NORM_METHOD,	    clui_info->normalmethod_list,
	        XVW_BELOW,                  menu,
	        NULL);

	if (clui_info->cmap_flag)
 	    xvw_set_attribute(image, XVW_COLOR_COLORFILE, clui_info->cmap_file);

	if (clui_info->clip_flag)
 	    xvw_set_attribute(image, XVW_IMAGE_CLIPFILE, clui_info->clip_file);

	/* 
	 * note: we need to get the map width of the original input object,
	 *     (or of the data object providing the colormap specified w/ -cmap)
	 *     NOT of the data object obtained with XVW_IMAGE_IMAGEOBJ -- this
	 *     is because we need the size of the map of the *original*
	 *     file - the image object will create a 3 col map for
	 *     images with no map segment
	 */
	if (clui_info->cmap_file != NULL)
	    tmp_data_object = kpds_open_input_object(clui_info->cmap_file);
	else tmp_data_object = kpds_open_input_object(clui_info->i_file);

	if (kpds_query_map(tmp_data_object))
  	    kpds_get_attribute(tmp_data_object, KPDS_MAP_SIZE, &map_width, 
			       NULL, NULL, NULL, NULL);
	else map_width = 0;

	if (map_width > 0)
	    xvw_set_attributes(image,
	    	    XVW_COLOR_RED_MAPCOL,    clui_info->redcol_list-1,
		    XVW_COLOR_RED_FUNCTION,  clui_info->redfn_string,
		    NULL);

	if (map_width > 1)
	    xvw_set_attributes(image,
		    XVW_COLOR_GREEN_MAPCOL,   clui_info->greencol_list-1,
		    XVW_COLOR_GREEN_FUNCTION, clui_info->greenfn_string,
		    NULL);

	if (map_width > 2)
	   xvw_set_attributes(image,
		    XVW_COLOR_BLUE_MAPCOL,    clui_info->bluecol_list-1,
		    XVW_COLOR_BLUE_FUNCTION,  clui_info->bluefn_string,
		    NULL);

	/* create position object */
        position = xvw_create_position(parent, "position");
        xvw_set_attributes(position,
		XVW_POSITION_OBJECT,      data_object,
		XVW_STRING_JUSTIFICATION, KSTRING_JUSTIFY_CENTER,
		XVW_BELOW,                image,
		XVW_LEFT_OF,	          NULL,
		XVW_RIGHT_OF,	          NULL,
		XVW_TACK_EDGE,            KMANAGER_TACK_HORIZ, 
		NULL);

	/* 
	 * create panicon object -- note the reason why we get the data object
         * from the image object is that if there was an input image w/ no
	 * value data, the image object will have substituted the Khoros image.
	 */
	panicon = xvw_create_panicon(parent, "panicon");
	xvw_get_attribute(image, XVW_IMAGE_IMAGEOBJ, &data_object);
	xvw_set_attributes(panicon,
		XVW_BELOW,            menu,
		XVW_IMAGE_IMAGEOBJ,   data_object,
		NULL);
	if (clui_info->cmap_flag)
	    xvw_set_attribute(panicon, XVW_COLOR_COLORFILE, clui_info->cmap_file);

	xvw_add_callback(image, XVW_IMAGE_UPDATE_CALLBACK,  
		         image_update_callback, NULL);

	return(image);
}

/*-----------------------------------------------------------
|
|  Routine Name: create_imageicon_display
|
|       Purpose: This utility routine is used when the user executes
|                % putdata -imageicon
|                  -or-
|                % puticon
|
|                It creates the putdata display with an imageicon object.
|
|         Input: parent  - the manager backplane
|                menu    - menubutton for Options menu
|        Output:
|    Written By: Mark Young & Danielle Argiro
|          Date: April 12, 1996
| Modifications:
|
------------------------------------------------------------*/

static xvobject create_imageicon_display(
    xvobject parent,
    xvobject menu)
{
	xvobject imageicon;

	xvw_set_attributes(parent,
			  XVW_WIDTH,  clui_info->iconsize_int,
			  XVW_HEIGHT, clui_info->iconsize_int,
			  NULL);

	imageicon = xvw_create_imageicon(parent, "icon");
        xvw_set_attributes(imageicon,
		XVW_IMAGE_IMAGEFILE, clui_info->i_file,
		XVW_IMAGE_CLIPFILE,  clui_info->clip_file,
		XVW_COLOR_COLORFILE, clui_info->cmap_file,
	        XVW_IMAGEICON_SIZE,  clui_info->iconsize_int,
		XVW_BELOW,           menu,
		NULL);

	return(imageicon);
}


/*-----------------------------------------------------------
|
|  Routine Name: create_zoom_display
|
|       Purpose: This utility routine is used when the user executes
|                % putdata -zoom
|                  -or-
|                % putzoom
|
|                It creates the putdata display with a zoom object,
|		 and a position object.
|
|         Input: parent  - the manager backplane
|                menu    - menubutton for Options menu
|        Output:
|    Written By: Mark Young & Danielle Argiro
|          Date: April 12, 1996
| Modifications:
|
------------------------------------------------------------*/


static xvobject create_zoom_display(
    xvobject parent,
    xvobject menu)
{
	xvobject zoom;
	kobject  data_object;

	xvw_set_attributes(parent,
		   XVW_WIDTH,   250,
		   XVW_HEIGHT,  250,
	           NULL);

        zoom = xvw_create_zoom(parent, "zoom");
        position = xvw_create_position(parent, "position");

	data_object = kpds_open_input_object(clui_info->i_file);

        xvw_set_attributes(zoom,
		XVW_IMAGE_IMAGEOBJ,  data_object,
		XVW_IMAGE_CLIPFILE,  clui_info->clip_file,
		XVW_COLOR_COLORFILE, clui_info->cmap_file,
	        XVW_ZOOM_XPOSITION,  clui_info->zoomx_int,
	 	XVW_ZOOM_YPOSITION,  clui_info->zoomy_int,
		XVW_ZOOM_FACTOR,     clui_info->zoomfactor_double,
		XVW_BELOW,           menu,
		XVW_ABOVE,           position,
		NULL);

	xvw_set_attributes(position,
		XVW_POSITION_OBJECT,      data_object,
		XVW_STRING_JUSTIFICATION, KSTRING_JUSTIFY_CENTER,
		XVW_ABOVE,                NULL,
		XVW_LEFT_OF,              NULL,
		XVW_RIGHT_OF,             NULL,
		XVW_TACK_EDGE,            KMANAGER_TACK_HORIZ, 
		NULL);

	xvw_set_attribute(zoom, XVW_TACK_EDGE,  KMANAGER_TACK_ALL);
	return(zoom);
}

/*-----------------------------------------------------------
|
|  Routine Name: create_animate_display
|
|       Purpose: This utility routine is used when the user executes
|                % putdata -animate
|                  -or-
|                % putanimate
|
|                It creates the putdata display with an animate object.
|
|         Input: parent  - the manager backplane
|                menu    - menubutton for Options menu
|        Output:
|    Written By: Mark Young & Danielle Argiro
|          Date: April 12, 1996
| Modifications:
|
------------------------------------------------------------*/

static xvobject create_animate_display(
    xvobject parent,
    xvobject menu)
{
	xvobject animate, position_object;
	kobject  data_object;

	animate = xvw_create_animate(parent, "animate");
	position_object = xvw_create_position(parent, "position");

        xvw_set_attributes(animate,
		XVW_IMAGE_IMAGEFILE,    clui_info->i_file,
		XVW_IMAGE_CLIPFILE,     clui_info->clip_file,
		XVW_COLOR_COLORFILE,    clui_info->cmap_file,
		XVW_ANIMATE_DIRECTION,  clui_info->dir_list,
		XVW_ANIMATE_CONTROL,    clui_info->control_list,
		XVW_ANIMATE_UPDATETIME, clui_info->speed_double,
                XVW_BELOW,              menu,
		NULL);

	xvw_get_attribute(animate, XVW_IMAGE_IMAGEOBJ, &data_object);
        xvw_set_attributes(position_object,
		XVW_POSITION_OBJECT,      data_object,
		XVW_STRING_JUSTIFICATION, KSTRING_JUSTIFY_CENTER,
		XVW_BELOW,                animate,
		XVW_LEFT_OF,	          NULL,
		XVW_RIGHT_OF,	          NULL,
		XVW_TACK_EDGE,            KMANAGER_TACK_HORIZ, 
		NULL);

	return(animate);
}


/*-----------------------------------------------------------
|
|  Routine Name: create_plot2d_display
|
|       Purpose: This utility routine is used when the user executes
|                % putdata -plot2d
|                  -or-
|                % putplot2
|
|                It creates the putdata display with an area object
|		 containing a 2D axis object and a 2D plot object.
|
|         Input: parent  - the manager backplane
|                menu    - menubutton for Options menu
|        Output:
|    Written By: Mark Young & Danielle Argiro
|          Date: April 12, 1996
| Modifications:
|
------------------------------------------------------------*/

static xvobject create_plot2d_display(
    xvobject parent,
    xvobject menu)
{
	xvobject plot2d, area, axis;
	
	area = xvw_create_area(parent, "Area");
	xvw_set_attributes(area,
		XVW_TACK_EDGE,               KMANAGER_TACK_ALL,
		XVW_AREA_DISPLAY_DATE,       FALSE,
                XVW_AREA_DISPLAY_TITLE,      FALSE,
		NULL);

	if (clui_info->axis2d_flag && clui_info->axis2d_logic == TRUE)
	{
            axis = xvw_create_axis2d(area, "Axis2d");
	    xvw_set_attributes(axis, 
                               XVW_GRAPHICS_VIEWPORT_MIN_X, 0.1,
                               XVW_GRAPHICS_VIEWPORT_MIN_Y, 0.1,
                               XVW_GRAPHICS_VIEWPORT_MAX_X, 0.9,
                               XVW_GRAPHICS_VIEWPORT_MAX_Y, 0.9,
			       XVW_GRAPHICS_ATTACH, axis,
			       NULL);
	}
	else axis = area;

	plot2d = xvw_create_plot2d(area, "Plot2d");

	xvw_set_attributes(plot2d,
		XVW_GRAPHICS_ATTACH,     axis,
		XVW_PLOT2D_PLOTFILE,     clui_info->i_file,
		XVW_PLOT2D_PLOTTYPE,     clui_info->ptype2d_list,
		XVW_GRAPHICS_LINETYPE,   clui_info->ltype2d_list,
		XVW_GRAPHICS_MARKERTYPE, clui_info->mtype2d_list,
		XVW_FOREGROUND_COLOR,    clui_info->pcolor2d_string,
		NULL);

	xvw_set_attribute(area, XVW_BELOW, menu);

	/* 
	 * need to install action handlers for quit on area object as
	 * well as on the plot object itself (which gets done later)
	 */
	xvw_add_action(area, "<Key>q", quit_handler, NULL, TRUE);
        xvw_add_action(area, "<BtnDown>(2)", quit_handler, NULL, TRUE);

	return(plot2d);
}

/*-----------------------------------------------------------
|
|  Routine Name: create_plot3d_display
|
|       Purpose: This utility routine is used when the user executes
|                % putdata -plot3d
|                  -or-
|                % putplot3
|
|                It creates the putdata display with an area object
|                containing a 3D plot object.
|
|         Input: parent  - the manager backplane
|                menu    - menubutton for Options menu
|        Output:
|    Written By: Mark Young & Danielle Argiro
|          Date: April 12, 1996
| Modifications:
|
------------------------------------------------------------*/

static xvobject create_plot3d_display(
    xvobject parent,
    xvobject menu)
{
	xvobject plot3d, area;

	area = xvw_create_area(parent, "Area");
	xvw_set_attributes(area,
		XVW_TACK_EDGE,               KMANAGER_TACK_ALL,
		XVW_AREA_DISPLAY_DATE,       FALSE,
                XVW_AREA_DISPLAY_TITLE,      FALSE,
		XVW_GRAPHICS_VIEWPORT_MIN_X, 0.1,
		XVW_GRAPHICS_VIEWPORT_MIN_Y, 0.1,
		XVW_GRAPHICS_VIEWPORT_MIN_Z, 0.1,
		XVW_GRAPHICS_VIEWPORT_MAX_X, 0.9,
		XVW_GRAPHICS_VIEWPORT_MAX_Y, 0.9,
		XVW_GRAPHICS_VIEWPORT_MAX_Z, 0.9,
		NULL);

	plot3d = xvw_create_plot3d(area, "Plot3d");

	xvw_set_attributes(plot3d,
		XVW_GRAPHICS_ATTACH,     area,
		XVW_PLOT3D_PLOTFILE,     clui_info->i_file,
		XVW_PLOT3D_PLOTTYPE,     clui_info->ptype3d_list,
		XVW_GRAPHICS_LINETYPE,   clui_info->ltype3d_list,
		XVW_GRAPHICS_MARKERTYPE, clui_info->mtype3d_list,
		XVW_PLOT3D_SHADETYPE,    clui_info->stype3d_list,
		XVW_FOREGROUND_COLOR,    clui_info->pcolor3d_string,
		XVW_GRAPHICS_ALPHA,      clui_info->xrot3d_float,
		XVW_GRAPHICS_THETA,      clui_info->zrot3d_float,
		XVW_GRAPHICS_EYE,        clui_info->edist3d_float,
		NULL);

	xvw_set_attribute(parent, XVW_BACKGROUND_COLOR, 
			  clui_info->fcolor3d_string);

	xvw_add_action(area, "<Key>q", quit_handler, NULL, TRUE);

	/* 
	 * need to install action handlers for quit on area object as
	 * well as on the plot object itself (which gets done later)
	 */
	xvw_add_action(area, "<BtnDown>(2)", quit_handler, NULL, TRUE);
	               xvw_set_attribute(area, XVW_BELOW, menu);

	return(plot3d);
}

/*-----------------------------------------------------------
|
|  Routine Name: create_palette_display
|
|       Purpose: This utility routine is used when the user executes
|                % putdata -palette
|                  -or-
|                % putpalette
|
|                It creates the putdata display with a palette object.
|
|         Input: parent  - the manager backplane
|                menu    - menubutton for Options menu
|        Output:
|    Written By: Mark Young & Danielle Argiro
|          Date: April 12, 1996
| Modifications:
|
------------------------------------------------------------*/

static xvobject create_palette_display(
    xvobject parent,
    xvobject menu)
{
	xvobject palette;

	if (clui_info->palettetype_list == KPALETTE_TYPE_COLORBAR)
	{
            xvw_set_attributes(parent,
                               XVW_WIDTH,  275,
                               XVW_HEIGHT, 100,
                               NULL);
	}
	else if (clui_info->palettetype_list == KPALETTE_TYPE_COLORCELL)
	{
            xvw_set_attributes(parent,
                               XVW_WIDTH,  300,
                               XVW_HEIGHT, 175,
                               NULL);
	}
	else if (clui_info->palettetype_list == KPALETTE_TYPE_COLORWHEEL)
	{
            xvw_set_attributes(parent,
                               XVW_WIDTH,  275,
                               XVW_HEIGHT, 145,
                               NULL);
	}

	palette = xvw_create_palette(parent, "palette");
	xvw_set_attribute(palette, XVW_COLOR_COLORFILE, clui_info->i_file);

	xvw_set_attributes(palette, 
	        XVW_BELOW,        menu,
		XVW_PALETTE_TYPE, clui_info->palettetype_list,
	        XVW_TACK_EDGE,    KMANAGER_TACK_ALL,
		NULL);

	return(palette);
}

/*-----------------------------------------------------------
|
|  Routine Name: create_pixel_display
|
|       Purpose: This utility routine is used when the user executes
|                % putdata -pixel
|                  -or-
|                % putpixel
|
|                It creates the putdata display with a printpixel object.
|
|         Input: parent  - the manager backplane
|                menu    - menubutton for Options menu
|        Output:
|    Written By: Mark Young & Danielle Argiro
|          Date: April 12, 1996
| Modifications:
|
------------------------------------------------------------*/

static xvobject create_pixel_display(
    xvobject parent,
    xvobject menu)
{
	xvobject printpixel;

	printpixel = xvw_create_printpixel(parent, "pixel");
	xvw_set_attributes(printpixel, 
		XVW_PRINTPIXEL_FILENAME,  clui_info->i_file,
		XVW_PRINTPIXEL_WIDTH,     clui_info->valuewidth_int,
		XVW_PRINTPIXEL_HEIGHT,    clui_info->valueheight_int,
		XVW_PRINTPIXEL_SHOWCOLOR, clui_info->showcolorvalue_logic,
		XVW_BELOW,		  menu,
		NULL);

	return(printpixel);
}

/*-----------------------------------------------------------
|
|  Routine Name: create_mapval_display
|
|       Purpose: This utility routine is used when the user executes
|                % putdata -mapval
|                  -or-
|                % putmapval
|
|                It creates the putdata display with a printmapval object.
|
|         Input: parent  - the manager backplane
|                menu    - menubutton for Options menu
|        Output:
|    Written By: Mark Young & Danielle Argiro
|          Date: April 12, 1996
| Modifications:
|
------------------------------------------------------------*/

static xvobject create_mapval_display(
    xvobject parent,
    xvobject menu)
{
	xvobject mapval;

        mapval = xvw_create_printmapval(parent, "mapval");
        xvw_set_attributes(mapval,
                   XVW_PRINTMAPVAL_FILENAME,  clui_info->i_file,
                   XVW_PRINTMAPVAL_WIDTH,     clui_info->mapwidth_int,
                   XVW_PRINTMAPVAL_HEIGHT,    clui_info->mapheight_int,
                   XVW_PRINTMAPVAL_SHOWCOLOR, clui_info->showcolormap_logic,
                   XVW_PRINTMAPVAL_POLICY,    clui_info->displaypolicy_cycle,
                   XVW_BELOW,                 menu,
                   NULL);

	return(mapval);
}

