/*
  * Khoros: $Id: preclassed.c,v 1.6 1996/04/05 02:11:11 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: preclassed.c,v 1.6 1996/04/05 02:11:11 fas Exp $";
#endif




/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>  	Predetermined Classification File Routines
   >>>>
   >>>>  Private:
   >>>>			spc_input_preclassed_file()
   >>>>   Static:
   >>>>			read_preclassed_file()
   >>>>			get_preclassed_entry()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/

#include "spectrum.h"

/*-----------------------------------------------------------
|
|  Routine Name: spc_input_preclassed_file
|
|       Purpose: Reads a predetermined classification file
|
|         Input: filename - name of predetermined classification file to read in
|                ofilename - name of conflict file to write out
|        Output: None
|       Returns: TRUE on success, FALSE on failure
|    Written By: Danielle Argiro
|          Date: Sept 23, 1995
| Modifications: 
|
------------------------------------------------------------*/

#define PROVIDE_NEW_FILENAME   1
#define WRITE_CONFLICTS_TO_TTY 2

static char *line_scan    = "%lg %lg %d";

static char *get_preclassed_entry  PROTO((kfile *));
static int   read_preclassed_file  PROTO((kfile *, kfile *));

int spc_input_preclassed_file(
    char *infilename)
{
	char  *date;
	int   choice = 0;
	kfile *infile = NULL, *outfile = NULL;
	char  temp[KLENGTH];
	char  outfilename[KLENGTH];
	char  **prompts, **answers, **choices;

	/*
	 *  error check: have to have display environment file
 	 */
	if (spc_display->pixel_dim_x == -1 ||
	    spc_display->pixel_dim_y == -1)
	{
	    kerror(NULL, "spc_read_preclassed_file", "Before inputting a Predetermined Classification file, you must first input a Display Environment file that specifies the UTM coordinates of the upper left hand corner of the image, the pixel dimensions, and the UTM zone.  Please input a Display Environment file, and try again."); 
	    return(FALSE);
	}

	/*
	 *  ensure that we are in "Add Cluster" mode
	 */
	ksprintf(temp, "%d", SPC_ADD);
	xvf_set_attribute(gui_info->cluster_operation_struct,
			  XVF_TOGGLE_VAL, temp);
	gui_info->cluster_operation_val = SPC_ADD;


	/* open predetermined-classification file */
	if ((infile = kfopen(infilename, "r")) == NULL)
        {
	   errno = SPC_FILE_ACCESS;
	   kerror(NULL, "spc_read_preclassed_file", 
	          "Unable to open '%s' to read predetermined classification file.", infilename);
	   kinfo(KHOSTILE, "Get it right!"); return(FALSE);
        }

	ksprintf(outfilename, "%s.conflicts", infilename);

	/* check to see if conflict file already exists */
	if (!(koverwrite(KSTANDARD, outfilename)))
	{
	    choices = (char **) kcalloc((size_t) 3, sizeof(char *));
	    choices[0] = kstrdup("I will provide a new filename");
	    choices[1] = kstrdup("Write conflict notification to tty");
	    ksprintf(temp, "How to procede?");
	    (void) kchoose(KSTANDARD, choices, 2, 2, NULL, &choice, temp);
	    karray_free(choices, KSTRING, (size_t) 2, NULL);
	}
	if (choice == WRITE_CONFLICTS_TO_TTY)
	    outfile = kstderr;
	else
	{
	    if (choice == PROVIDE_NEW_FILENAME)
	    {
	 	prompts = (char **) kcalloc((size_t) 1, sizeof(char *));
                answers = (char **) kcalloc((size_t) 1, sizeof(char *));
	        prompts[0] = kstrdup("Enter new filename");
	        answers[0] = (char *) kcalloc((size_t) 1, KLENGTH * sizeof(char));
	        ksprintf(answers[0], "%s", outfilename); 
	    	(void) xvu_query_wait("New filename for output of conflicts",
                                      prompts, "ok", answers, 1, 25);
	        ksprintf(outfilename, "%s", answers[0]);
	        karray_free(prompts, KSTRING, (size_t) 1, NULL);
	        karray_free(answers, KSTRING, (size_t) 1, NULL);
	    }
	    if ((outfile = kfopen(outfilename, "w")) == NULL)
            {
               errno = SPC_FILE_ACCESS;
               kerror(NULL, "spc_read_preclassed_file",
                      "Unable to open '%s' to write conflicts from predetermined classification file.", outfilename);
               kinfo(KHOSTILE, "Get it right!"); return(FALSE);
            }
	    date = spc_get_date();
	    kfprintf(outfile, "\n---- Predetermined Classification Conflict File ----\n");
	    kfprintf(outfile, "     %s, created %s     \n\n", outfilename, date);
	    kfree_and_NULL(date);

	    if (gui_info->Files->files->in_image != NULL)
	        ksprintf(temp, "%s", gui_info->Files->files->in_image);
	    else  ksprintf(temp, "none");
	    kfprintf(outfile, "Image  File:                       %s\n", temp);

	    if (gui_info->Files->files->in_legend != NULL)
                ksprintf(temp, "%s", gui_info->Files->files->in_legend);
            else  ksprintf(temp, "none");
	    kfprintf(outfile, "Legend File:                       %s\n", temp);

	    if (gui_info->Files->files->in_environ != NULL)
                ksprintf(temp, "%s", gui_info->Files->files->in_environ);
            else  ksprintf(temp, "none");
            kfprintf(outfile, "Display Environment File:          %s\n", temp);

	    if (gui_info->Files->files->in_classes != NULL)
                ksprintf(temp, "%s", gui_info->Files->files->in_classes);
            else  ksprintf(temp, "none");
            kfprintf(outfile, "Predetermined Classification File: %s\n\n", temp);
	}

	/* read in the predetermined classification file */
	if (!(read_preclassed_file(infile, outfile)))
	    return(FALSE);

	kfclose(infile);
	if (choice != WRITE_CONFLICTS_TO_TTY)
	    kfclose(outfile);

	/* update class contents labels */
	spc_update_all_classcontents_labels();

	/* inform user */
	if (choice != WRITE_CONFLICTS_TO_TTY)
	    kinfo(KSTANDARD, "Predetermined classifications input from file '%s'; conflict information written to '%s'\n", infilename, outfilename);
	else  kinfo(KSTANDARD, "Predetermined classifications input from file '%s'; conflict information written to tty\n", infilename);



	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: read_preclassed_file
|
|       Purpose: Reads the predetermined classification file
|
|         Input: file  - open stream to input predetermined classification file
|		 ofile - open stream to output conflict file
|        Output: None
|       Returns: TRUE on success, FALSE on failure
|    Written By: Danielle Argiro
|          Date: Sept 23, 1995
| Modifications:
|
------------------------------------------------------------*/

static int read_preclassed_file(
    kfile *infile,
    kfile *outfile)
{
	char        *line = NULL;
        LegendEntry *legend_ptr;
        char        temp[KLENGTH];
        double      northing, easting, tmp_double;
        int         xposition, yposition, cluster;
        int         w, h, i, entrynum, class_id, status; 
	int         linenum = 0, conflicts = 0, oob = 0;

	/* premature end of file */
        if (kfeof(infile))
           return(FALSE);

	/*
	 *  get size of file from first line
	 */
        line = get_preclassed_entry(infile);
	if (line == NULL)
	{
	    kerror(NULL, "read_preclassed_file", "Predetermined classification file is empty!");
	    return(FALSE);
	}
	sscanf(line, "%d", &entrynum);

	kpds_get_attribute(spc_image, KPDS_VALUE_SIZE, 
			   &w, &h, NULL, NULL, NULL);

	for (i = 0; i < entrynum; i++)
	{
	    linenum++;
	    /* get a line from the file */
	    line = get_preclassed_entry(infile);
	    if (line == NULL)
	    {
	        errno = SPC_FILE_CONTENTS;
		kerror(NULL, "spc_read_preclassed_file",
		       "Incorrect number of lines in file");
	        return(FALSE);
	    }

	    /* read in UTM coordinate and class ID */
	    status = ksscanf(line, line_scan, &northing, &easting, &class_id);
	    if (status < 3)
	    {
		kerror(NULL, "read_preclassed_file", "Incomplete data on line %d of predetermined classification file; only able to read %d of 3 required fields.  Aborting predetermined classification input.", i+2, status);
		return(FALSE);
	    }

	    /* get the cluster which is at that UTM coordinate location */
	    yposition = (int) -1.0*(northing - spc_display->utm_00_N)/
				    spc_display->pixel_dim_y;
            xposition = (int) 1.0*(easting - spc_display->utm_00_E)/
				    spc_display->pixel_dim_x;

	    if ((xposition < 0) || (xposition > w) ||
		(yposition < 0) || (yposition > h))
	    {
	        kfprintf(outfile, "\nOut of Bounds:\n");
		kfprintf(outfile, "    UTM coordinate %.1fN, %.1fE is not within the \n", northing, easting);
	        kfprintf(outfile, "    bounds of the image; ignoring line and continuing.\n");
		oob++;
		continue;
	    }
	    xvw_set_attributes(spc_display->workspace,
                           XVW_IMAGE_XPOSITION, xposition,
                           XVW_IMAGE_YPOSITION, yposition,
                           NULL);
            xvw_get_attribute(spc_display->workspace, 
			      XVW_IMAGE_VALUE, &tmp_double);
            cluster = (int) tmp_double;

	    /* get established class from ID number */
	    legend_ptr = spc_find_legend_from_class_id(class_id);

	    /* no such class yet: create new class with that ID number */
	    if (legend_ptr == NULL)
	    {
	        ksprintf(temp, "Class %d", class_id);
	 	legend_ptr = spc_add_legend_entry(class_id, 0, 0, 0,
				   	      temp, NULL, 0, SPC_USER_CREATED);
	    }

	    if ((spc_classes != NULL) && (spc_classes[cluster] == -1) &&
		spc_add_cluster_to_class(legend_ptr, cluster))
	    {
                kfprintf(kstderr,
                         "Adding cluster %d to class '%s' \n",
                         cluster, legend_ptr->classname);
	        xvw_set_attributes(legend_ptr->colorbox,
                               XVW_COLORCELL_REDVAL,   legend_ptr->redval,
                               XVW_COLORCELL_GREENVAL, legend_ptr->greenval,
                               XVW_COLORCELL_BLUEVAL,  legend_ptr->blueval,
                               XVW_COLORCELL_UPDATE,   TRUE,
                               NULL);
	    }
	    else if ((spc_classes != NULL) && 
		     (spc_classes[cluster] != class_id))
	    {
	        kfprintf(outfile, "\nAssignment Conflict:\n");
	        kfprintf(outfile, "    At UTM coordinate %.1fN, %.1fE (image location (%d x %d)\n",
		    northing, easting, xposition, yposition);
		kfprintf(outfile, "    attempt failed to add cluster %d to class '%s'\n",
		    cluster, legend_ptr->classname );
		kfprintf(outfile, "    since cluster %d was already assigned to class '%s'\n",
		   cluster, spc_legend_lookup[cluster]->classname);
		conflicts++;
 	    }
	    kfree_and_NULL(line);
	}

	kfprintf(outfile, "\nSUMMARY:\n");
	kfprintf(outfile, 
                 "\nOf the %d lines in the Predetermined Classification File,\n", linenum);
	kfprintf(outfile,
                 "there were %d conflicts, %d UTM coordinates out of bounds.\n\n\n",
		 conflicts, oob);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: get_preclassed_entry
|
|       Purpose: Reads a single entry from the predetermined classification file
|
|         Input: file - open stream to display environment file
|        Output: None
|       Returns: a single entry from the display environment file
|    Written By: Danielle Argiro
|          Date: Aug 8, 1994
| Modifications:
|
------------------------------------------------------------*/

static char *get_preclassed_entry(
       kfile *infile)
{
	int  j, ready;
	char temp[KLENGTH];

	ready = FALSE;
        while (!ready)
        {
           j = 0;
           if (kfgets(temp, KLENGTH, infile) == NULL)
                return(NULL);

           /* skip over any leading whitespace */
           while(temp[j] == ' ' || temp[j] == '\t') j++;

           /* skip over any blank lines;  comments have # symbol */
            if ((temp[j] != '\n') && (temp[j] != '#'))
                ready = TRUE;
        }

	if (ready)
	    return(kstrdup(&temp[j]));
	else return(NULL);
}

