
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 

#include "design.h"

void
bivar_work(float *in_x,
	   float *in_y,
	   float *in_z,
	   int inpoints,
	   float *out_x,
	   float *out_y,
	   float *out_z,
	   int xres,
	   int yres,
	   float *undef_value,
	   float *exponent,
	   float *radius)
{
    int i,j,k,l;
    int offset=0;
    double xdist,t,xnumer,xdenom,wf,sr,t2;
    int direct_hit;

    wf = *exponent;
    sr = *radius;
    
    for (k=0;k<yres;k++)  /* loop over grid J dimension */
    {
	for (j=0;j<xres;j++)  /* loop over grid I dimension */
	{
	    xnumer = 0.;
	    xdenom = 0.;
	    direct_hit = 0;
	    l=0;
	    for (i=0;i<inpoints;i++)  /* loop on all input scattter points */
	    {
		
/*	old rectilinear version:
	t = (xg[j]-x[i])*(xg[j]-x[i]) + (yg[k]-y[i])*(yg[k]-y[i]); */
		
		t = (out_x[offset]-in_x[i])*(out_x[offset]-in_x[i]) +
		    (out_y[offset]-in_y[i])*(out_y[offset]-in_y[i]); 
	    
		xdist = ksqrt(t);
		
		if (xdist < 1.0e-5)  /* if distance == 0 */
		{
		    direct_hit = 1;
		    out_z[offset] = in_z[i];
		    break;
		}

		if (xdist < sr)  /* if distance < search radius */
		{
		    l++;
		    t2 = 1./(kpow(xdist,wf));
		    xnumer = xnumer+ t2 * in_z[i];
		    xdenom = xdenom+t2;
		}
	    }
	    
	    if (direct_hit != 1)
	    {
		if (l == 0)
/*		    zg[d] = *undef_value; */
		    out_z[offset] = *undef_value;
		else
/*		    zg[d] = xnumer/xdenom; */
		    out_z[offset] = xnumer/xdenom;
	    }
	    offset++;
	}
    }
}
