
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: gprgeom_util.c,v 1.5 1996/06/24 22:15:26 mirage Exp $";
#endif

 /*
  * $Log: gprgeom_util.c,v $
  * Revision 1.5  1996/06/24 22:15:26  mirage
  * mark is on a plane and wes is in Norway... committing to get the latest version
  *
  * Revision 1.4  1996/05/30 15:51:19  mirage
  * wes visit to kri may 96
  *
  * Revision 1.3  1996/05/06 18:02:33  mirage
  * latest copy from lbl
  *
  */



#include <geometry.h>
#include "gprgeom.h"

#define NUM_PRIMS 12

static void
    print_polyline_dis PROTO((kobject, int, int, int,int,int,int,int,kfile *)),
    print_polyline_con PROTO((kobject, int, int,int,int,int,int,int,kfile *)),
    print_triangles_dis PROTO((kobject, int, int,int,int,int,int,int,kfile *)),
    print_triangles_con PROTO((kobject, int, int,int,int,int,int,int,kfile *)),
    print_spheres PROTO((kobject, int, int,int,int,int,int,int,kfile *)),
    print_qmesh PROTO((kobject, int, int,int,int,int,int,int,kfile *)),
    print_omesh PROTO((kobject, int, int,int,int,int,int,int,kfile *)),
    print_texture2d PROTO((kobject, int, int,int,int,int,int,int,kfile *)),
    print_text PROTO((kobject, int, int,int,int,int,int,int,kfile *)),
    print_obj_xform PROTO((kobject, int, int,int,int,int,int,int,kfile *)),
    print_view_xform PROTO((kobject, int, int,int,int,int,int,int,kfile *)),
    print_camera_stuff PROTO((kobject, int, int,int,int,int,int,int,kfile *)),
    print_unknown PROTO((kobject, int, int,int,int,int,int,int,kfile *));

static void (*prtfuncs[NUM_PRIMS+1])()=
{
    print_polyline_dis,
    print_polyline_con,
    print_triangles_dis,
    print_triangles_con,
    print_spheres,
    print_qmesh,
    print_omesh,
    print_texture2d,
    print_text,
    print_obj_xform,
    print_view_xform,
    print_camera_stuff,
    print_unknown
};

static void print_vectors PROTO((float *, int, int, char *, char *, kfile *));
static void print_att PROTO((kobject, kfile *));

/*-----------------------------------------------------------
| 
|  Routine Name: kprgeom_util()
| 
|       Purpose: routine to print out information contained
|                in an object containing geometry information.
| 
|         Input: kobject "k" - the object in question
| 
|        Output: bunch of text 
|    Written By: wes
|          Date: May 20, 1993
| Modifications:
|                Wed Jan 10 14:14:20 PST 1996  add support for text
| 
------------------------------------------------------------*/
void
kprgeom_util(kobject k,
	     kfile *output,
	     int do_att,
	     int do_loc,
	     int do_color,
	     int do_norm_radii,
	     int do_texture)
{
    int status;
    int num_prims,i,prim_type;
    int func_index;
    int *prim_list=NULL;
    int has_alpha;


    if (do_att)
	print_att(k,output);
    
    kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_HAS_ALPHA,&has_alpha);
	
    /* ask for floating point presentation of location data */
    kgeom_set_attribute(k,KGEOM_OBJECT,KGEOM_LOCATION_DATA_TYPE,KFLOAT);

    /* check to see how many primitives are in the object */
    status = kgeom_query_primitive_list(k);
    if (status)
        status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_NUMBER_PRIMITIVES,&num_prims);

    if (status == FALSE)
	kfprintf(output,"\tCan't get PRIM_NUM attribute.  Maybe not a GEOM file?\n");
    else
	kfprintf(output,"\tNumber of primitives in this object is %d.\n",num_prims);

    /* grab the list of primitives */
    status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_PRIMITIVE_LIST,&prim_list);
    if (status == FALSE)
    {
	kfprintf(output,"\tCan't get PRIM_LIST attribute.  Maybe not a GEOM file?\n");
	return;
    }

    for (i=0;i<num_prims;i++)
    {
        /* TEMPORARY KLUDGE... */
        kgeom_set_attribute(k,KGEOM_OBJECT,KGEOM_PRIMITIVE_POSITION,i);
	switch (prim_list[i])
	{
	case KGEOM_POLYLINE_DISJOINT:
	    func_index = 0;
	    break;
	case KGEOM_POLYLINE_CONNECTED:
	    func_index = 1;
	    break;
	case KGEOM_TRIANGLES_DISJOINT:
	    func_index = 2;
	    break;
	case KGEOM_TRIANGLES_CONNECTED:
	    func_index = 3;
	    break;
        case KGEOM_SPHERES:
            func_index = 4;
	    break;
	case KGEOM_QUADMESH:
	    func_index = 5;
	    break;
	case KGEOM_OCTMESH:
	    func_index = 6;
	    break;
	case KGEOM_TEXTURE2D:
	    func_index = 7;
	    break;
	case KGEOM_TEXT:
	    func_index = 8;
	    break;
	case KGEOM_OBJECT_XFORM:
	    func_index = 9;
	    break;
	case KGEOM_VIEW_XFORM:
	    func_index = 10;
	    break;
	case KGEOM_CAMERA_ORIENTATION:
	    func_index = 11;
	    break;
	default:
	    func_index = NUM_PRIMS;
	    break;
	}
	(*prtfuncs[func_index])(k,i,prim_list[i],do_loc,do_color,do_norm_radii,do_texture,has_alpha,output);
    }
}

static void
print_text(kobject k,
	   int n,
	   int prim_type,
	   int do_loc,
	   int do_color,
	   int do_norm_radii,
	   int do_texture,
	   int has_alpha,
	   kfile *o)
{
    int status,nstrings,layout;
    char **strings=NULL;
    char **fonts=NULL;
    float *locs=NULL,*offset=NULL,*ups=NULL,*flows=NULL;
    float *colors=NULL, *back_colors=NULL;

    
    status = kgeom_get_attribute(k,KGEOM_TEXT,KGEOM_NUMBER_VERTICES,&nstrings);
    if (status == FALSE)
    {
	kfprintf(o,"\t\tError obtaining KGEOM_NUMBER from the KGEOM_TEXT primitive. Skipping it an continuing...\n");
	return;
    }
    
   
    status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_LAYOUT,&layout);
    
    kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_TEXT <%d>.\n",n,prim_type);
    kfprintf(o,"\tand has %d strings\n",nstrings);

    status = kgeom_get_data(k,KGEOM_TEXT,&strings,&fonts,&locs,&offset,&ups,&flows,&colors,&back_colors);

    if (strings == NULL)
	    kfprintf(o,"\tString data is NOT present.\n");
    else
    {
	int i;
	for (i=0;i<nstrings;i++)
	    kfprintf(o,"String %d:\t>%s<\tFont: <%s>\n",i,strings[i],
		     (((fonts == NULL)||(fonts[i] == NULL))?"no font specified":fonts[i]));
    }

    if (locs == NULL)
	kfprintf(o,"\tLocation data is NOT present.\n");
    else if (do_loc)
       print_vectors(locs,nstrings,3,"Vertices","index\tX\tY\tZ",o);

    if (colors == NULL)
	kfprintf(o,"\tColor data is NOT present.\n");
	
    if (do_color && colors)
       print_vectors(colors,nstrings,(has_alpha == TRUE ? 4 : 3),"Colors",(has_alpha == TRUE ? "index\tR\tG\tB\tA" : "index\tR\tG\tB"),o);
}


static void
print_polyline_dis(kobject k,
		   int n,
		   int prim_type,
		   int do_loc,
		   int do_color,
		   int do_norm_radii,
		   int do_texture,
		   int has_alpha,
		   kfile *o)
{
    int nverts,status,i,layout;
    float *verts=NULL,*colors=NULL;

    status = kgeom_get_attribute(k,KGEOM_POLYLINE_DISJOINT,KGEOM_NUMBER_VERTICES,&nverts);
    if (status == FALSE)
    {
	kfprintf(o,"\t\tError obtaining KGEOM_NUMBER from this primitive. Skipping it an continuing...\n");
	return;
    }
    
    kfprintf(o,"\n------- PRIMITIVE %d is KPOLYLINE_DIS <%d>.\n",n,prim_type);
    kfprintf(o,"\tand has %d vertices (%d lines).\n",nverts, nverts/2);

    status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_LAYOUT,&layout);
    if (status == FALSE)
    {
       kfprintf(o,"\t\tError obtaining the KGEOM_LAYOUT information.\n");
       layout = -1;
    }
    
    /* assert that we want floating point, RGB data */
    kgeom_set_attribute(k,KGEOM_OBJECT,KGEOM_COLOR_DATA_TYPE,KFLOAT);
    
    kgeom_get_data(k,KGEOM_POLYLINE_DISJOINT,&verts,&colors);
    
    if (colors == NULL)
       kfprintf(o,"\t\tcolor data is NOT present.\n");
    else
       kfprintf(o,"\t\tcolor data is present on a %s basis.\n",(layout == KPER_VERTEX ? "PER VERTEX" : "PER LINE"));
    
    if (do_loc)
       print_vectors(verts,nverts,3,"Vertices","index\tX\tY\tZ",o);
    
    if (do_color && colors)
       print_vectors(colors,(layout == KPER_VERTEX ? nverts : nverts/2),(has_alpha == TRUE ? 4 : 3),"Colors",(has_alpha == TRUE ? "index\tR\tG\tB\tA" : "index\tR\tG\tB"),o);
}

static void
print_spheres(kobject k,
	      int n,
	      int prim_type,
	      int do_loc,
	      int do_color,
	      int do_norm_radii,
	      int do_texture,
	      int has_alpha,
	      kfile *o)
{
    int nspheres,i;
    float *v=NULL,*r=NULL,*c=NULL;

    kgeom_get_attribute(k,KGEOM_SPHERES,KGEOM_NUMBER_VERTICES,&nspheres);

    kfprintf(o,"\n------- PRIMITIVE %d is KSPHERES <%d>\n",n,prim_type);
    kfprintf(o,"\tand has %d spheres.\n",nspheres);

    /* ask for floating point presentation of the radius data */
    kgeom_set_attribute(k,KGEOM_SPHERES,KGEOM_RADIUS_DATA_TYPE,KFLOAT);
    kgeom_set_attribute(k,KGEOM_SPHERES,KGEOM_COLOR_DATA_TYPE,KFLOAT);

    kgeom_get_data(k,KGEOM_SPHERES,&v,&c,&r);

    if (c == NULL)
	kfprintf(o,"\t\tcolor data is NOT present.\n");
    else
    {
	kfprintf(o,"\t\tcolor data is present.\n");
    }

    if (r == NULL)
	kfprintf(o,"\t\tradius data is NOT present.\n");
    else
    {
	kfprintf(o,"\t\tradius data is present.\n");
    }

    if (do_loc)
       print_vectors(v,nspheres,3,"Centers","index\tX\tY\tZ",o);

    if (do_color && c)
	print_vectors(c,nspheres,(has_alpha == TRUE ? 4 : 3),"Colors",
		      (has_alpha == TRUE ? "index\tR\tG\tB\tA" : "index\tR\tG\tB"),o);
    
    if (do_norm_radii && r)
	print_vectors(r,nspheres,1,"Radii","index\tRadius",o);
}

static void
print_polyline_con(kobject k, 
		   int n,
		   int prim_type,
		   int do_loc,
		   int do_color,
		   int do_norm_radii,
		   int do_texture,
		   int has_alpha,
		   kfile *o)
{
    int nverts,status,i,layout;
    float *verts=NULL,*colors=NULL;

    status = kgeom_get_attribute(k,KGEOM_POLYLINE_CONNECTED,KGEOM_NUMBER_VERTICES,&nverts);
    if (status == FALSE)
    {
	kfprintf(o,"\t\tError obtaining KGEOM_NUMBER_VERTICES from this primitive. Skipping it and continuing...\n");
	return;
    }

    kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_POLYLINE_CONNECTED <%d>\n",n,prim_type);
    kfprintf(o,"\tand has %d vertices (%d lines).\n",nverts, nverts-1);

    status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_LAYOUT,&layout);
    if (status == FALSE)
    {
       kfprintf(o,"\t\tError obtaining the KGEOM_LAYOUT information.\n");
       layout = -1;
    }
    
    /* assert that we want floating point, RGB data */
    kgeom_set_attribute(k,KGEOM_OBJECT,KGEOM_COLOR_DATA_TYPE,KFLOAT);
    
    kgeom_get_data(k,KGEOM_POLYLINE_CONNECTED,&verts,&colors);
    if (colors == NULL)
       kfprintf(o,"\t\tcolor data is NOT present.\n");
    else
       kfprintf(o,"\t\tcolor data is present on a %s basis.\n",(layout == KPER_VERTEX ? "PER VERTEX" : "PER LINE"));
    
    if (do_loc)
       print_vectors(verts,nverts,3,"Vertices","index\tX\tY\tZ",o);
    
    if (do_color && colors)
       print_vectors(colors,(layout == KPER_VERTEX ? nverts : nverts-1),(has_alpha == TRUE ? 4 : 3),"Colors",(has_alpha == TRUE ? "index\tR\tG\tB\tA" : "index\tR\tG\tB"),o);
}

static void
print_triangles_dis(kobject k,
		    int n,
		    int prim_type,
		    int do_loc,
		    int do_color,
		    int do_norm_radii,
		    int do_texture,
		    int has_alpha,
		    kfile *o)
{
    int nverts;
    int status;
    int layout;
    int i,j;
    float *v=NULL,*norms=NULL,*c=NULL,*tc=NULL;

    kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_TRIANGLES_DISJOINT <%d>\n",n,prim_type);
    kgeom_get_attribute(k,KGEOM_TRIANGLES_DISJOINT,KGEOM_NUMBER_VERTICES,&nverts);
    kfprintf(o,"\tand has %d vertices (%d triangles).\n",nverts, nverts/3);

    kgeom_get_data(k,KGEOM_TRIANGLES_DISJOINT,&v,&c,&norms,&tc);
    status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_LAYOUT,&layout);

    if (norms == NULL)
	kfprintf(o,"\t\tnormal data is NOT present.\n");
    else
	kfprintf(o,"\t\tnormal data is present on a %s basis.\n",(layout == KPER_VERTEX ? "PER VERTEX" : "PER FACE"));

    if (c == NULL)
	kfprintf(o,"\t\tcolor data is NOT present.\n");
    else
	kfprintf(o,"\t\tcolor data is present on a %s basis.\n",(layout == KPER_VERTEX ? "PER VERTEX" : "PER FACE"));

    if (tc == NULL)
	kfprintf(o,"\t\ttexture coordinate data is NOT present.\n");
    else
	kfprintf(o,"\ttexture coordinate data is present.\n");

    if ((do_loc) && (v != NULL))
	print_vectors(v,nverts,3,"Vertices","index\tX\tY\tZ",o);

    if ((do_norm_radii) && (norms != NULL))
	print_vectors(norms,(layout == KPER_VERTEX ? nverts : nverts/3),3,"Normals","index\tX\tY\tZ",o);

    if ((do_color) && (c != NULL))
	print_vectors(c,(layout == KPER_VERTEX ? nverts : nverts/3),(has_alpha == TRUE ? 4 : 3),"Colors",(has_alpha == TRUE ? "index\tR\tG\tB\tA" : "index\tR\tG\tB"),o);

    if (do_texture)
	kfprintf(o,"\tTexture Coordinates are not yet supported.\n");
	
}


static void
print_triangles_con(kobject k,
		    int n,
		    int prim_type,
		    int do_loc,
		    int do_color,
		    int do_norm_radii,
		    int do_texture,
		    int has_alpha,
		    kfile *o)
{ 
    int nverts,i,j;
    int layout;
    int offset;
    char *texture_trans_name = NULL;
    float *v=NULL,*c=NULL,*norm=NULL,*tc=NULL;
    int has_tc,tc_size;

    kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_TRIANGLES_CONNECTED <%d>\n",n,prim_type);
    kgeom_get_attribute(k,KGEOM_TRIANGLES_CONNECTED,KGEOM_NUMBER_VERTICES,&nverts);
    kfprintf(o,"\tand has %d vertices (%d triangles).\n",nverts,nverts-2);

    /*-- test getting this before the get_data --*/
    kgeom_get_attribute(k,KGEOM_TRIANGLES_CONNECTED,KGEOM_TEXTURE,&texture_trans_name);

    kgeom_get_data(k,KGEOM_TRIANGLES_CONNECTED,&v,&c,&norm,&tc);
    kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_LAYOUT,&layout);
   
    has_tc = (tc == NULL) ? 0 : 1;
    kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_TEXTURE_COORD_SIZE,&tc_size);

    if (c==NULL)
        kfprintf(o,"\t\tcolor data is NOT present.\n");
    else
	kfprintf(o,"\t\tcolor data is present on a %s basis.\n",(layout == KPER_VERTEX ? "PER VERTEX" : "PER FACE"));

    if (norm==NULL)
	kfprintf(o,"\t\tnormal data is NOT present.\n");
    else
	kfprintf(o,"\t\tnormal data is present.\n");

    if (tc==NULL)
	kfprintf(o,"\t\ttexture coordinate data is NOT present.\n");
    else
	kfprintf(o,"\t\ttexture coordinate data is present.\n");

#if 0
    kgeom_get_attribute(k,KGEOM_TRIANGLES_CONNECTED,KGEOM_TEXTURE,&texture_trans_name);
#endif
    if (texture_trans_name != NULL)
    {
	kfprintf(o,"\t\ttexture transport is present: <%s>\n",texture_trans_name);
    }
    else
       kfprintf(o,"\t\ttexture transport is NOT present.\n");
    /* print the vertices */
    if (do_loc)
	print_vectors(v,nverts,3,"Vertices","index\tX\tY\tZ",o);

   if (do_norm_radii)
       print_vectors(norm,nverts,3,"Normals","index\tX\tY\tZ",o);

    if (do_color)
    {
	if (c != NULL)	/* print the colors */
	    print_vectors(c,(layout == KPER_FACE ? nverts-2 : nverts),(has_alpha == TRUE ? 4 : 3),"Colors",(has_alpha == TRUE ? "index\tR\tG\tB\tA" : "index\tR\tG\tB"),o);
    }
   
    if (has_tc)
    {
	if (do_texture)
	{
	    /* this assumes per-vertex layout of tcoords */
	    if (tc_size == 2)
		print_vectors(tc,nverts,2,"Texture Coords","index\tU\tV",o);
	    else /* assume tc == 3 */
		print_vectors(tc,nverts,3,"Texture Coords","index\tU\tV\tW",o);
	}
    }
}

static void
print_qmesh(kobject k, 
	    int n,
	    int prim_type,
	    int do_loc,
	    int do_color,
	    int do_norm_radii,
	    int do_texture,
	    int has_alpha,
	    kfile *o)
{
    int nverts,i,j;
    int usize,vsize;
    int layout;
    int offset;
    int status;
    
    float *v=NULL,*c=NULL,*norm=NULL,*tc=NULL;
    int has_loc = FALSE,has_color=FALSE,has_norms=FALSE,has_tc=FALSE,tc_size,
        has_texture=FALSE;
    char *texture_trans_name;
    
    kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_QUADMESH <%d>\n",n,prim_type);
    status = kgeom_get_attribute(k,KGEOM_QUADMESH,KGEOM_QUADMESH_SIZE,&usize,&vsize);
    if (status)
	kfprintf(o,"\twhich has %d by %d vertices in U and V.\n",usize,vsize);
    else
	kfprintf(o,"\t\tError: can't get quadmesh size.\n");

    has_loc =  kgeom_get_data(k,KGEOM_QUADMESH_LOCATION_ALL,&v);
    has_color = kgeom_get_data(k,KGEOM_QUADMESH_COLOR_ALL,&c);
    has_norms = kgeom_get_data(k,KGEOM_QUADMESH_NORMAL_ALL,&norm);
    has_tc = kgeom_get_data(k,KGEOM_QUADMESH_TEXTURE_COORD_ALL,&tc);

    kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_TEXTURE_COORD_SIZE,&tc_size);
    
    kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_LAYOUT,&layout);

    kfprintf(o,"\n");

    if (!has_loc)
       kfprintf(o,"\t\tlocaton data is NOT present.\n");
    else
       kfprintf(o,"\t\tlocaton data is present.\n");

    if (!has_color)
       kfprintf(o,"\t\tcolor data is NOT present.\n");
    else
       kfprintf(o,"\t\tcolor data is present.\n");

    if (!has_norms)
       kfprintf(o,"\t\tnormal data is NOT present.\n");
    else
       kfprintf(o,"\t\tnormal data is present.\n");

    if (!has_tc)
       kfprintf(o,"\t\ttexture coordinate data is NOT present.\n");
    else
       kfprintf(o,"\t\ttexture coordinate data is present.\n");
       
    if (has_tc == 1)
	kfprintf(o,"\t\ttexture coord size is %d.\n",tc_size);

    kgeom_get_attribute(k,KGEOM_QUADMESH,KGEOM_TEXTURE,&texture_trans_name);
    if (texture_trans_name != NULL)
    {
	kfprintf(o,"\t\ttexture transport is present: <%s>\n",texture_trans_name);
    }
    else
       kfprintf(o,"\t\ttexture transport is NOT present.\n");
    
    
    if (has_loc)
    {
	if (do_loc)
	    print_vectors(v,usize*vsize,3,"Vertices","index\tX\tY\tZ",o);
    }
    if (has_color)
    {
	if (do_color)  /* assume for now per-vertex coloring */
	{
	    if (layout == KPER_VERTEX)
		print_vectors(c,usize*vsize,((has_alpha == 1) ? 4 : 3),"Colors",
			      ((has_alpha == 1) ? ("index\tR\tG\tB\tA") :
				("Colors",("index\tR\tG\tB"))),o);
	}
    }
    if (has_norms)
	if (do_norm_radii)
	    print_vectors(norm,usize*vsize,3,"Normals","index\tX\tY\tZ",o);
    
    if (has_tc)
    {
	if (do_texture)
	{
	    if (tc_size == 2)
		print_vectors(tc,usize*vsize,2,"Texture Coords","index\tU\tV",o);
	    else /* assume tc == 3 */
		print_vectors(tc,usize*vsize,3,"Texture Coords","index\tU\tV\tW",o);
	}
    }
}

static void
print_texture2d(kobject k, 
		int n,
		int prim_type,
		int do_loc,
		int do_color,
		int do_norm_radii,
		int do_texture,
		int has_alpha,
		kfile *o)
{
    int nverts,i,j;
    int usize,vsize;
    int layout;
    int offset;
    int status;
    
    float *v=NULL,*c=NULL,*norm=NULL,*tc=NULL;
    int has_loc = FALSE,has_color=FALSE,has_norms=FALSE,has_tc=FALSE,tc_size;
    
    kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_TEXTURE2D <%d>\n",n,prim_type);
    status = kgeom_get_attribute(k,KGEOM_TEXTURE2D,KGEOM_TEXTURE2D_SIZE,&usize,&vsize);
    if (status)
	kfprintf(o,"\twhich has %d by %d vertices in U and V.\n",usize,vsize);
    else
	kfprintf(o,"\t\tError: can't get texture2d size.\n");


#if 0
    has_loc =  kgeom_get_data(k,KGEOM_QUADMESH_LOCATION_ALL,&v);
    has_color = kgeom_get_data(k,KGEOM_QUADMESH_COLOR_ALL,&c);
    has_norms = kgeom_get_data(k,KGEOM_QUADMESH_NORMAL_ALL,&norm);
    has_tc = kgeom_get_data(k,KGEOM_QUADMESH_TEXTURE_COORD_ALL,&tc);
    kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_TEXTURE_COORD_SIZE,&tc_size);
    
    kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_LAYOUT,&layout);

    kfprintf(o,"\n");

    if (!has_loc)
       kfprintf(o,"\t\tlocaton data is NOT present.\n");
    else
       kfprintf(o,"\t\tlocaton data is present.\n");

    if (!has_color)
       kfprintf(o,"\t\tcolor data is NOT present.\n");
    else
       kfprintf(o,"\t\tcolor data is present.\n");

    if (!has_norms)
       kfprintf(o,"\t\tnormal data is NOT present.\n");
    else
       kfprintf(o,"\t\tnormal data is present.\n");

    if (!has_tc)
       kfprintf(o,"\t\ttexture coordinate data is NOT present.\n");
    else
       kfprintf(o,"\t\ttexture coordinate data is present.\n");
       
    if (has_tc == 1)
	kfprintf(o,"\t\ttexture coord size is %d.\n",tc_size);
    
    if (has_loc)
    {
	if (do_loc)
	    print_vectors(v,usize*vsize,3,"Vertices","index\tX\tY\tZ",o);
    }
    if (has_color)
    {
	if (do_color)  /* assume for now per-vertex coloring */
	{
	    if (layout == KPER_VERTEX)
		print_vectors(c,usize*vsize,((has_alpha == 1) ? 4 : 3),"Colors",
			      ((has_alpha == 1) ? ("index\tR\tG\tB\tA") :
				("Colors",("index\tR\tG\tB"))),o);
	}
    }
    if (has_norms)
	if (do_norm_radii)
	    print_vectors(norm,usize*vsize,3,"Normals","index\tX\tY\tZ",o);
    
    if (has_tc)
    {
	if (do_texture)
	{
	    if (tc_size == 2)
		print_vectors(tc,usize*vsize,2,"Texture Coords","index\tU\tV",o);
	    else /* assume tc == 3 */
		print_vectors(tc,usize*vsize,3,"Texture Coords","index\tU\tV\tW",o);
	}
    }
#endif
}

static void
print_omesh(kobject k, 
	    int n,
	    int prim_type,
	    int do_loc,
	    int do_color,
	    int do_norm_radii,
	    int do_texture,
	    int has_alpha,
	    kfile *o)
{
    int nverts,i,j;
    int usize,vsize,wsize;
    int layout;
    int offset;
    int status;
    
    float *v=NULL,*c=NULL,*norm=NULL,*tc=NULL;
    int has_loc = FALSE,has_color=FALSE,has_norms=FALSE;
    
    kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_OCTMESH <%d>\n",n,prim_type);
    status = kgeom_get_attribute(k,KGEOM_OCTMESH,KGEOM_OCTMESH_SIZE,&usize,&vsize,&wsize);
    if (status)
	kfprintf(o,"\twhich has %d, %d, and %d vertices in U,V and W.\n",usize,vsize,wsize);
    else
	kfprintf(o,"\t\tError: can't get octmesh size.\n");

    has_loc =  kgeom_get_data(k,KGEOM_OCTMESH_LOCATION_ALL,&v);
    has_color = kgeom_get_data(k,KGEOM_OCTMESH_COLOR_ALL,&c);
    has_norms = kgeom_get_data(k,KGEOM_OCTMESH_NORMAL_ALL,&norm);
    
    kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_LAYOUT,&layout);

    kfprintf(o,"\n");

    if (!has_loc)
       kfprintf(o,"\t\tlocaton data is NOT present.\n");
    else
       kfprintf(o,"\t\tlocaton data is present.\n");

    if (!has_color)
       kfprintf(o,"\t\tcolor data is NOT present.\n");
    else
       kfprintf(o,"\t\tcolor data is present.\n");

    if (!has_norms)
       kfprintf(o,"\t\tnormal data is NOT present.\n");
    else
       kfprintf(o,"\t\tnormal data is present.\n");
    
    if (has_loc)
    {
	if (do_loc)
	    print_vectors(v,usize*vsize*wsize,3,"Vertices","index\tX\tY\tZ",o);
    }
    if (has_color)
    {
	if (do_color)  /* assume for now per-vertex coloring */
	{
	    if (layout == KPER_VERTEX)
		print_vectors(c,usize*vsize*wsize,((has_alpha == 1) ? 4 : 3),"Colors",
			      ((has_alpha == 1) ? ("index\tR\tG\tB\tA") :
				("Colors",("index\tR\tG\tB"))),o);
	}
    }
    if (has_norms)
	if (do_norm_radii)
	    print_vectors(norm,usize*vsize*wsize,3,"Normals","index\tX\tY\tZ",o);
}

static void
print_unknown(kobject k, 
	      int n,
	      int prim_type,
	      int do_loc,
	      int do_color,
	      int do_norm_radii,
	      int do_texture,
	      int has_alpha,
	      kfile *o)
{
    kfprintf(o,"\tUnrecognized primitive in position #%d is of type %d.\n",n,prim_type);
}

static void
print_obj_xform(kobject k, 
		int n,
		int prim_type,
		int do_loc,
		int do_color,
		int do_norm_radii,
		int do_texture,
		int has_alpha,
		kfile *o)
{
    float *matrix=NULL;
    int i;
    int relative;
    char *name;
   
    kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_OBJECT_XFORM <%d>\n",n,prim_type);
    
    kgeom_get_attributes(k, KGEOM_OBJECT, 
			 KGEOM_OBJECT_XFORM_MATRIX,   	 &matrix,
			 KGEOM_OBJECT_XFORM_RELATIVE, 	 &relative,
			 KGEOM_OBJECT_XFORM_OBJECT_NAME, &name, 
			 NULL);

    kfprintf(o,"\tObject Name: <%s>\n",name);
    kfprintf(o,"\tRelative value is <%s> \n",(relative == KABSOLUTE) ? "KABSOLUTE" : "KRELATIVE");
    kfprintf(o,"\tMatrix:\n");
    for (i=0;i<16;i++)
    {
        kfprintf(o,"%f ",matrix[i]);
	if (((i+1) % 4) == 0)
	    kfprintf(o,"\n");
    }
}

static void
print_view_xform(kobject k, 
		 int n,
		 int prim_type,
		 int do_loc,
		 int do_color,
		 int do_norm_radii,
		 int do_texture,
		 int has_alpha,
		 kfile *o)
{
    float *matrix=NULL;
    int i;
    int relative;
    char *name;
   
    kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_VIEW_XFORM <%d>\n",n,prim_type);
    
    kgeom_get_attributes(k, KGEOM_OBJECT,
			 KGEOM_VIEW_XFORM_MATRIX, &matrix,
			 NULL);
    
    kfprintf(o,"\tMatrix:\n");
    for (i=0;i<16;i++)
    {
        kfprintf(o,"%f ",matrix[i]);
	if (((i+1) % 4) == 0)
	    kfprintf(o,"\n");
    }
}

static void
print_camera_stuff(kobject k, 
		   int n,
		   int prim_type,
		   int do_loc,
		   int do_color,
		   int do_norm_radii,
		   int do_texture,
		   int has_alpha,
		   kfile *o)
{
    float fov,hither,yon;
    int projection;
    int is_stereo;
    float eye_sep,foc_len;
    int i;
    int relative;
    char *name;
    float *eye=NULL,*at=NULL,*up=NULL;
   
    kfprintf(o,"\n------- PRIMITIVE %d is KGEOM_CAMERA_ORIENTATAION <%d>\n",n,prim_type);
    
    kgeom_get_attributes(k, KGEOM_OBJECT,
			 KGEOM_CAMERA_POSITION_AND_ORIENTATION, &eye, &at, &up,
			 KGEOM_CAMERA_FIELD_OF_VIEW,            &fov, 
			 KGEOM_CAMERA_HITHER_YON,               &hither, &yon,
			 KGEOM_CAMERA_PROJECTION,               &projection, 
			 KGEOM_CAMERA_IS_STEREO,                &is_stereo,
			 KGEOM_CAMERA_STEREO_VIEWING_PARAMETERS, &eye_sep, &foc_len,
			 NULL);

    
    kfprintf(o,"\tCamera Info\n");
    kfprintf(o,"\t\tEye:\t%g %g %g \n",eye[0],eye[1],eye[2]);
    kfprintf(o,"\t\tLookAt:\t%g %g %g \n",at[0],at[1],at[2]);
    kfprintf(o,"\t\tUp:\t%g %g %g \n",up[0],up[1],up[2]);
    kfprintf(o,"\t\tHither/Yon\t %g %g\n",hither,yon);
    kfprintf(o,"\t\tFOV\t%g \n",fov);
    kfprintf(o,"\t\tProjection: %s \n",(projection == KPROJECTION_PERSPECTIVE)?"KPROJECTION_PERSPECTIVE":"KPROJECTION_PARALLEL");
}

static void
print_vectors(float *v,
	      int n,
	      int length,
	      char *string,
	      char *hdr_string,
	      kfile *f)
{
    int i,j,offset;
    
    kfprintf(f,"\n\t%s\n",string);
    kfprintf(f,"%s\n",hdr_string);
    offset = 0;
    for (i=0;i<n;i++)
    {
	kfprintf(f,"%d",i);
	for (j=0;j<length;j++)
	    kfprintf(f,"\t%g ",v[offset++]);
	kfprintf(f,"\n");
    }

}

static void
print_att(kobject k,
	  kfile *f)
{
    /**
      * this routine prints all the object-level attributes.
    **/
    
    int status,layout;
    int i,j;
    char *name=NULL;
/*    float *min=NULL,*max=NULL,*center=NULL,*rotvec=NULL,*scalevec=NULL; */
    float *min=NULL,*max=NULL,*center=NULL,*mx=NULL,*mx2;
    
    double ambient,diffuse,spec,spec_exp,metal;

    lset_common_geom_attributes(k);
    
    status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_NAME,&name);
    if (status == FALSE)
	kfprintf(f,"\tCan't get NAME attribute.  Maybe not a GEOM file?\n");
    else
       if (name == NULL)
	  kfprintf(f,"\tObject name is : <NULL>\n",name);
       else
	  kfprintf(f,"\tObject name is : <%s>\n",name);

    status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_BOUNDING_BOX,&min,&max);
    if (status == TRUE)
        kfprintf(f,"\tBounding box: %g,%g,%g <-> %g,%g,%g \n",min[0],min[1],min[2],max[0],max[1],max[2]);
    else
        kfprintf(f,"\tThis object has NO bounding box.\n");

    status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_CENTER,&center);
    if (status == TRUE)
	kfprintf(f,"\tCenter point: %g,%g,%g \n",center[0],center[1],center[2]);
    else
	kfprintf(f,"\tThis object has NO center point.\n");
#if 0
    /* obsolete, 1/27/95. wes */
    status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_ROTATE,&rotvec);
    if (status == TRUE)
	kfprintf(f,"\tRotate angles:\n\t\tX Axis: %g\n\t\tY Axis: %g\n\t\tZ Axis: %g \n",rotvec[0],rotvec[1],rotvec[2]);
    else
	kfprintf(f,"\tNo rotate attribute is present for this object.\n");

    status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_SCALE,&scalevec);
    if (status == TRUE)
	kfprintf(f,"\tScaling factors:\n\t\tX Axis: %g\n\t\tY Axis: %g\n\t\tZ Axis: %g \n",scalevec[0],scalevec[1],scalevec[2]);
    else
	kfprintf(f,"\tNo scale attribute is present for this object.\n");
#endif
    status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_MATRIX,&mx);
    if (status == TRUE)
    {
	kfprintf(f,"\tObject Transformation Matrix:\n");
	mx2 = mx;
	for (j=0;j<4;j++)
	{
	    kfprintf(f,"\t");
	    for (i=0;i<4;i++)
		kfprintf(f,"\t%g",*mx2++);
	    kfprintf(f,"\n");
	}
    }
    else
	kfprintf(f,"\tThis object has NO transformation matrix.\n");

    /* deal with colors, etc.*/
    status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_AMBIENT,&ambient);
    if (!status)
       kfprintf(f,"\tAmbient attribute is not defined.\n");
    else
       kfprintf(f,"\tAmbient:\t%g\n",ambient);

    status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_DIFFUSE,&diffuse);
    if (!status)
       kfprintf(f,"\tDiffuse attribute is not defined.\n");
    else
       kfprintf(f,"\tDiffuse:\t%g\n",diffuse);


    status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_SPECULAR_COEFFICIENT,&spec);
    if (!status)
       kfprintf(f,"\tSpecular Coefficient attribute is not defined.\n");
    else
       kfprintf(f,"\tSpecularC:\t%g\n",spec);

    status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_SPECULAR_EXPONENT,&spec_exp);
    if (!status)
       kfprintf(f,"\tSpecular Exponent attribute is not defined.\n");
    else
       kfprintf(f,"\tSpecularE:\t%g\n",spec_exp);

    status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_METAL,&metal);
    if (!status)
       kfprintf(f,"\tMetalness attribute is not defined.\n");
    else
      kfprintf(f,"\tMetalness:\t%g\n",metal);


    status = kgeom_get_attribute(k,KGEOM_OBJECT,KGEOM_LAYOUT,&layout);
    if (status)
	kfprintf(f,"\tLayout is <%s>\n",layout == KPER_VERTEX ? "PER_VERTEX" : "PER_FACE, or PER_SPHERE, etc.");
    else
	kfprintf(f,"\tLayout attribute is not defined.\n");
}
