 /*
  * Khoros: $Id: gtransform.c,v 1.4 1996/07/05 02:47:47 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: gtransform.c,v 1.4 1996/07/05 02:47:47 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for gtransform
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "gtransform.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - geometric transformation of geometry objects.
|
|       Purpose: main program for gtransform
|
|         Input:
|		char *clui_info->i_file; {Input Geometry Object }
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {Output Geometry Object }
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		float clui_info->xscale_float; {Scale to be applied to X coordinates of vertices (and normals, if present).}
|		int   clui_info->xscale_flag; {TRUE if -xscale specified}
|
|		float clui_info->yscale_float; {Scale to be applied to Y coordinates of vertices (and normals, if present).}
|		int   clui_info->yscale_flag; {TRUE if -yscale specified}
|
|		float clui_info->zscale_float; {Scale to be applied to Z coordinates of vertices (and normals, if present).}
|		int   clui_info->zscale_flag; {TRUE if -zscale specified}
|
|		int clui_info->do_normals_logic; {Toggle to enable or prohibit transformation of the normals, if present.}
|		int clui_info->do_normals_flag; {TRUE if -do_normals specified}
|
|		int clui_info->parallel_int; {indicates how many nodes will be used in a threaded operation.}
|		int clui_info->parallel_flag; {TRUE if -parallel specified}
|
|        Output:
|       Returns:
|
|    Written By: wes
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
	kobject input,output;
	int status;
	char *name=NULL;
	float scale[3];
	int do_normals;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "GEOMETRY");
	kexit_handler(gtransform_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "GEOMETRY", "gtransform",
		gtransform_usage_additions, gtransform_get_args,
		gtransform_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	input = kgeom_open_object(clui_info->i_file,KOBJ_READ);
	status = kgeom_get_attribute(input,KGEOM_OBJECT,KGEOM_NAME,&name);
	if (status == FALSE)
	{
	    kerror(NULL,"gtubeness","Can't get the NAME attribute from the input object <%s>.  Maybe it's not a GEOM file?\n",clui_info->i_file);
	    kpds_close_object(input);
	    kexit(KEXIT_FAILURE);
	}

	output = kgeom_open_output_object(clui_info->o_file);

	scale[0] = clui_info->xscale_float;
	scale[1] = clui_info->yscale_float;
	scale[2] = clui_info->zscale_float;

	do_normals = clui_info->do_normals_logic;
	
	/* initialize the threads mechanism... */
#ifdef _USE_THREADS
	init_gthreads(clui_info->parallel_int);
#endif
/* -main_before_lib_call_end */

/* -main_library_call */
	lgtransform(input,output,scale,do_normals);
/* -main_library_call_end */

/* -main_after_lib_call */
	kpds_close_object(input);
	kpds_close_object(output);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: gtransform_usage_additions
| 
|       Purpose: Prints usage additions in gtransform_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gtransform
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void gtransform_usage_additions(void)
{
	kfprintf(kstderr, "\tgeometric transformation of geometry objects.\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: gtransform_free_args
| 
|       Purpose: Frees CLUI struct allocated in gtransform_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname gtransform
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
gtransform_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
