
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 *
 * $Log: get_coords.c,v $
 * Revision 1.3  1996/05/03 00:08:44  mirage
 * updated from lbl
 *
 * Fri Apr 26 21:06:34 PDT 1996 - copyright update.
 */
 
/*####################################################################*/

 /*
  * Khoros: $Id: get_coords.c,v 1.3 1996/05/03 00:08:44 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: get_coords.c,v 1.3 1996/05/03 00:08:44 mirage Exp $";
#endif


#include "internals.h"


/****************************************************************
* 
*  Routine Name: lget_coords - obtain coordinate (location) data
* 
*       Purpose: This function allows access to location data in a
*                consistent way.  The idea is that all geometry
*                visualization tools need coordinate information
*                from the input data file.  If location data is
*                available, it is used.  If not, the convention
*                taken is that the u,v,w indeces are used as
*                coordinate values.  This convention may change over
*                time, thus the API rather than having lots of
*                duplicated code for dealing with this problem.
*
*         Input: kobj - data object to scan
* 		 i - width index 
* 	         j - height index
*                k - depth index 
*
*        Output: v - 3d vertex
*
*       Returns: CHILL (1) on success, WHACKED (0) on failure
*  Restrictions: 
*    Written By: Wes Bethel
*          Date: Jun 14, 1993
*      Verified: 
*  Side Effects: 
* Modifications:
*
*   Declaration: lget_coords (
*		 !    kobject kobj,
*		 !    int i,
*		 !    int j,
*		 !    int k,
*		 !    vertex_3d *v)
****************************************************************/
/* -library_def */
int lget_coords (
		 kobject kobj,
		 int i,
		 int j,
		 int k,
		 vertex_3d *v)

/* -library_def_end */
/* -library_code */
{
    int loc_avail,lw,lh,ld,ldim;
    int vw,vh,vd,vt,ve;
    float coords[3];

    /* first, check to see if there is _any_ location data. */
    loc_avail = kpds_query_location(kobj);

    /* if so, then use it. */
    if (loc_avail == TRUE)
    {
	kpds_get_attribute(kobj,KPDS_LOCATION_SIZE,&lw,&lh,&ld,&ldim);
	/* and insist on floating point presentation. */
	kpds_set_attribute(kobj,KPDS_LOCATION_DATA_TYPE,KFLOAT);
	
	if (ldim == 3) /* the usual case. */
        {
	    kpds_set_attribute(kobj,KPDS_LOCATION_POSITION,i,j,k,0);
	    kpds_get_data(kobj,KPDS_LOCATION_VECTOR,coords);
	    v->v[0] = coords[0];
	    v->v[1] = coords[1];
	    v->v[2] = coords[2];
	}
	else  /* for now, an error condition. */
        {
/* KERROR CAT: "library" */
	    kerror("GEOMETRY","lget_coords","the Input data doesn't have 3 dimensions of location data.  ");
	    return(WHACKED);
	}
    }
    else
    {
        kpds_get_attribute(kobj,KPDS_VALUE_SIZE,&vw,&vh,&vd,&ve,&vt);
        if ((i < 0) || (i > vw-1))
	    return(WHACKED);
	if ((j<0) || (j > vh-1))
	    return(WHACKED);
	if ((k<0) || (k > vd-1))
	    return(WHACKED);

	/* no coordinate information - return the indeces */
	v->v[0] = i;
	v->v[1] = j;
	v->v[2] = k;
    }
    
    return(CHILL);
}
