 /*
  * Khoros: $Id: lg3daxis.c,v 1.4 1996/07/05 02:49:22 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lg3daxis.c,v 1.4 1996/07/05 02:49:22 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for g3daxis
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lg3daxis
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
static int lcreate_base_axes PROTO((kobject,float *,int *));
static int lcreate_axis_labels PROTO((kobject,float *,int *,int,char *));
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lg3daxis - short description of lg3daxis()
* 
*       Purpose: This should be a complete description that anyone
*                could understand;  it should have acceptable grammar
*                and correct spelling.
*
*         Input: argument1 - explanation
*                argument2 - explanation
*                argument3 - explanation
*
*        Output: argument4 - explanation
*                argument5 - explanation
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: Restrictions on data or input as applicable
*    Written By: 
*          Date: Jul 04, 1996
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int
lg3daxis(kobject input,
	 kobject output,
	 int do_ticks,
	 int do_labels,
	 int do_autoextents,
	 int tick_density,
	 char *name,
	 float *extents,
	 char *format_string)
/* -library_def_end */

/* -library_code */
{
    float loc_extents[6];
    int w,h,d,dim;
    int nprims=1;

    /* get the coordinate info used in building the axes */
    if (do_autoextents)
    {
	kpds_get_attribute(input,KPDS_LOCATION_SIZE,&w,&h,&d,&dim);
	if (dim != 3)
	{
	    kerror("GEOMETRY","lglocstats","Being just a lowly software cretin, I can only deal with 3space location data.  You have given me %d-space location data.  Aborting.",dim);
	    return(WHACKED);
	}
	find_location_minmax(input,w,h,d,dim,(vertex *)loc_extents,
		  	     (vertex *)(loc_extents+3));
    }
    else
	kmemcpy(loc_extents,extents,sizeof(float)*6);

    /* set up the output object */
    kgeom_create_primitive_list(output);
    lset_common_geom_attributes(output);
    kgeom_set_attribute(output,KGEOM_OBJECT,KGEOM_NAME,name);

    /* create the axes themselves */
    lcreate_base_axes(output,loc_extents,&nprims);

    /* create the text that goes with it */
    if (do_labels && do_ticks)
    {
	lcreate_axis_labels(output,loc_extents,&nprims,tick_density,format_string);
    }

    return TRUE;
}

#define MAXSTRINGLENGTH 32

static int
lcreate_axis_labels(kobject output,
		    float *loc_extents,
		    int *nprims,
		    int nticks,
		    char *format_string)
{
    char **strings,*work;
    vertex *locs;

    /* total number of strings is:
        3 axes * (nticks-1 strings per axis) + 1 (the origin label)     */
    int nstrings,i,index=0;
    float axis,daxis;
    char fstring[64];

    nstrings = 3 * (nticks-1) + 1;
    
    strings = (char **)kmalloc(sizeof(char *)*nstrings);
    work = (char *)kmalloc(sizeof(char)*nstrings*MAXSTRINGLENGTH);
    kmemset(work,0,(size_t)nstrings*MAXSTRINGLENGTH);
    
    for (i=0;i<nstrings;i++)
	strings[i] = work+MAXSTRINGLENGTH*i;
    locs = (vertex *)kmalloc(sizeof(vertex)*nstrings);

    /* the first string is at the origin. */
    kmemset(fstring,0,64);
    sprintf(fstring,"(%s,%s,%s)",format_string,format_string,format_string);
    sprintf(strings[index],fstring,loc_extents[0],loc_extents[1],loc_extents[2]);
    locs[index].x = loc_extents[0];
    locs[index].y = loc_extents[1];
    locs[index].z = loc_extents[2];
    index++;

    /* do the X axis */
    kmemset(fstring,0,64);
    sprintf(fstring,"%s",format_string);
    daxis = (loc_extents[3]-loc_extents[0])/(nticks - 1);
    axis = loc_extents[0] + daxis;
    for (i=0;i<nticks-1;i++)
    {
	sprintf(strings[index],format_string,axis);
	locs[index].x = axis;
	locs[index].y = loc_extents[1];
	locs[index].z = loc_extents[2];
	index++;
	axis += daxis;
    }

    /* do the Y axis */
    daxis = (loc_extents[4]-loc_extents[1])/(nticks - 1);
    axis = loc_extents[1] + daxis;
    for (i=0;i<nticks-1;i++)
    {
	sprintf(strings[index],format_string,axis);
	locs[index].x = loc_extents[0];
	locs[index].y = axis;
	locs[index].z = loc_extents[2];
	index++;
	axis += daxis;
    }

    /* do the Z axis */
    daxis = (loc_extents[5]-loc_extents[2])/(nticks - 1);
    axis = loc_extents[2] + daxis;
    for (i=0;i<nticks-1;i++)
    {
	sprintf(strings[index],format_string,axis);
	locs[index].x = loc_extents[0];
	locs[index].y = loc_extents[1];
	locs[index].z = axis;
	index++;
	axis += daxis;
    }

    /* increment the primitive position */
    kgeom_set_attribute(output,KGEOM_OBJECT,KGEOM_NUMBER_PRIMITIVES,*nprims);
    kgeom_set_attribute(output,KGEOM_OBJECT,KGEOM_PRIMITIVE_POSITION,(*nprims)-1);
    *nprims += 1;
    
    /* stuff the data */
    kgeom_set_attribute(output,KGEOM_TEXT,KGEOM_NUMBER_VERTICES,nstrings);
    kgeom_set_attribute(output,KGEOM_TEXT,KGEOM_LOCATION_DATA_TYPE,KFLOAT);
    kgeom_put_data(output,KGEOM_TEXT,strings,NULL,locs,NULL,NULL,NULL,NULL,NULL);
    return TRUE; 
}


surface_colors_rgb red = {1.,0.,0.};
surface_colors_rgb green = {0.,1.,0.};
surface_colors_rgb cyan = {0.,1.,1.};


static int
lcreate_base_axes(kobject output,
		  float *loc_extents,
		  int *nprims)
{
    /* in this routine, we create a primitive which has 3 disjoint line
       segments. the convention we use here is that the X axis will be colored
       red, the Y axis will be colored green, the Z axis will be colored cyan*/
    
    vertex lverts[6];
    surface_colors_rgb lcolors[6];
    int i;

    /* copy the origin over 3 times */
    for (i=0;i<3;i++)
	VCOPY(lverts+(i*2),loc_extents);

    /* end of 1st line is xmax,ymin,zmin */
    VCOPY(lverts+1,loc_extents);
    lverts[1].x = loc_extents[3];

    /* end of 2nd line segment is xmin,ymax,zmin */
    VCOPY(lverts+3,loc_extents);
    lverts[3].y = loc_extents[4];

    /* end of 3rd line segment is xmin,ymin,zmax */
    VCOPY(lverts+5,loc_extents);
    lverts[5].z = loc_extents[5];

    /* now, set the colors.. */

    for (i=0;i<2;i++)
	VCOPY(lcolors+i,&red);
    
    for (i=2;i<4;i++)
	VCOPY(lcolors+i,&green);
    
    for (i=4;i<6;i++)
	VCOPY(lcolors+i,&cyan);

    /* take care of bookkeeping before we do any put_data()'s */
    kgeom_set_attribute(output,KGEOM_OBJECT,KGEOM_NUMBER_PRIMITIVES,*nprims);
    kgeom_set_attribute(output,KGEOM_OBJECT,KGEOM_PRIMITIVE_POSITION,(*nprims)-1);
    *nprims += 1;

    kgeom_set_attribute(output,KGEOM_POLYLINE_DISJOINT,KGEOM_NUMBER_VERTICES,6);
    kgeom_put_data(output,KGEOM_POLYLINE_DISJOINT,(float *)lverts,(float *)lcolors);
    return TRUE;
}
/* -library_code_end */
