 /*
  * Khoros: $Id: lgcreate2dtex.c,v 1.4 1996/07/05 02:49:28 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lgcreate2dtex.c,v 1.4 1996/07/05 02:49:28 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for gcreate2dtex
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lgcreate2dtex
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lgcreate2dtex - take an "image" and create a 2d texture for use by renderers,etc. in the GEOMETRY toolbox.
* 
*       Purpose: provide a routine which will convert an arbitrary
*        image into a 2d "texture"
*
*         Input: kobject inobj - contains a source image which will be
*                  used as the texture.
*
*        Output: kobject outobj - will contain a geometry services texture
*                  which can be bound to an object with gbindtexture.
*
*        Parms:  float tc[4] - contains texture coordinates that will
*                  be associated with the min/max corners of the texture.
*                  these coordinates, along with multiple textures, can
*                  be used to implement tiled textures.
*                int method - indicates what kind, if any, of filtering
*                  will be applied to the source image.
*
*   september 1995 NOTES:
*   1. at present, we only do a straight copy of the source texture data.
*      in the future, we will implement MIPmapping of textures.  some
*      work is needed on the data services infrastructure before this
*      will funciton properly.
*   2. it seems to make sense to put tiled textures along the depth axis.
*      this is not implemented.  furthermore, if it were, then the tc[]
*      array would need to be two-dimensional as we need a set of coordinates
*      for each tile in the set.
*   3. the stuff in the tc[] array is not used at all right now.  this is
*      because there is no corresponding attribute in the geom services
*      infrastructure.  the default behavior in rmonster will be to use
*      a coordinate range of 0..1 for the texture (effectively disabling
*      the use of tiled textures).
*
*       Returns: CHILL on success, WHACKED otherwise
*
*  Restrictions: Restrictions on data or input as applicable
*    Written By: wes
*          Date: Jul 04, 1996
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int lgcreate2dtex(kobject inobj,
		  kobject outobj,
		  float *tc,
		  int method)
/* -library_def_end */

/* -library_code */
{
    int has_alpha,size;
    int vw,vh,vd,vt,ve;
    unsigned char *ival;
    
    /* configuration check on input object */
    kpds_get_attribute(inobj,KPDS_VALUE_SIZE,&vw,&vh,&vd,&vt,&ve);

    /* check for d==1, t==1 */
    if ((vd != 1) || (vt != 1))
    {
	kerror("GEOMETRY","lcreate2dtex","The input image has d!=1 or t!=1. Please supply a 2d image as input.");
	return(WHACKED);
    }

    /* check for "has alpha" */
    /* index order.. */
#if 0    
    index_order[0] = KELEMENTS;
    index_order[1] = KWIDTH;
    index_order[2] = KHEIGHT;
    index_order[3] = KDEPTH;
    index_order[4] = KTIME;
    kdms_set_attribute(inobj,KDMS_SEGMENT_VALUE,KDMS_INDEX_ORDER,index_order);
#endif
    
    kcolor_set_attribute(inobj,KCOLOR_COLORSPACE,KRGB);
    kpds_set_attribute(inobj,KPDS_MAPPING_MODE,KMAPPED); 
    
    kcolor_get_attribute(inobj,KCOLOR_HAS_ALPHA,&has_alpha);

    size = (has_alpha) ? 4 : 3;
    kpds_set_attribute(inobj,KPDS_VALUE_SIZE,-1,-1,-1,-1,size);

    /* set the colorspace attributes so that we get RGB(a) */

    kpds_set_attribute(inobj,KPDS_VALUE_DATA_TYPE,KUBYTE);

    kpds_set_attribute(inobj,KPDS_ELEMENTS_FIRST,TRUE);
    
    ival=NULL;
    ival = kpds_get_data(inobj,KPDS_VALUE_ALL,ival);

    /* configure output object */
    
    kgeom_create_primitive_list(outobj);
    kgeom_set_attribute(outobj,KGEOM_OBJECT,KGEOM_NUMBER_PRIMITIVES,1);
    kgeom_set_attribute(outobj,KGEOM_OBJECT,KGEOM_COLORSPACE,KRGB);
    kgeom_set_attribute(outobj,KGEOM_TEXTURE2D,KGEOM_TEXTURE2D_SIZE,vw,vh);
    kgeom_set_attribute(outobj,KGEOM_OBJECT,KGEOM_HAS_ALPHA,has_alpha);
    
    kgeom_set_attribute(outobj,KGEOM_OBJECT,KGEOM_COLOR_DATA_TYPE,KUBYTE);

    /* copy over image data to output object */
    kgeom_put_data(outobj,KGEOM_TEXTURE2D_ALL,ival);  
    
    return(CHILL);
}

/****************************************************************
* 
*  Routine Name: lgisa2dtex - determine if the object is a 2d texture
* 
*       Purpose: a short function for checking to see if the given object
*                is a "2d texture"
*
*         Input: kobject inobj - input object, will be sanity-checked
*                to determine if it's contents are a 2d texture.
*
*        Output: none
*
*       Returns: TRUE if the object is a 2d texture, FALSE otherwise
*
*  Restrictions: 
*    Written By: wes
*          Date: Sep 07, 1995
*      Verified: 
*  Side Effects: 
* Modifications: 
****************************************************************/

int
lgisa2dtex(kobject inobj)
{
    int status, num_prims, *prim_list=NULL;
    
    status = kgeom_query_primitive_list(inobj);
    if (status)
        status = kgeom_get_attribute(inobj,KGEOM_OBJECT,KGEOM_NUMBER_PRIMITIVES,&num_prims);

    if (status == FALSE)
    {
	kerror("GEOMETRY","lgisa2dtex","Can't get PRIM_NUM attribute.  Maybe not a GEOM file?\n");
	return(FALSE);
    }
    
    status = kgeom_get_attribute(inobj,KGEOM_OBJECT,KGEOM_PRIMITIVE_LIST,&prim_list);
    if (status == FALSE)
    {
	kerror("GEOMETRY","lgisa2dtex","Can't get PRIM_LIST attribute.  Maybe not a GEOM file?\n");
	return(FALSE);
    }
    
    kgeom_set_attribute(inobj,KGEOM_OBJECT,KGEOM_PRIMITIVE_POSITION,0);

    if (prim_list[0] == KGEOM_TEXTURE2D)
	return(TRUE);

    return(FALSE);
}
/* -library_code_end */
