 /*
  * Khoros: $Id: lglocstats.c,v 1.4 1996/07/05 02:49:38 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lglocstats.c,v 1.4 1996/07/05 02:49:38 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for glocstats
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lglocstats
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lglocstats - short description of lglocstats()
* 
*       Purpose: This should be a complete description that anyone
*                could understand;  it should have acceptable grammar
*                and correct spelling.
*
*         Input: argument1 - explanation
*                argument2 - explanation
*                argument3 - explanation
*
*        Output: argument4 - explanation
*                argument5 - explanation
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: Restrictions on data or input as applicable
*    Written By: 
*          Date: Jul 04, 1996
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int lglocstats(kobject kobj,
	       int do_minmax)
/* -library_def_end */

/* -library_code */
{
    int loc_avail;
    int w,h,d,dim;
    vertex lmin,lmax;
    
    /* get dimesnions of value data */
    loc_avail = kpds_query_location(kobj);
    if (loc_avail)
    {
	kpds_get_attribute(kobj,KPDS_LOCATION_SIZE,&w,&h,&d,&dim);
	if (dim != 3)
	{
	    kerror("GEOMETRY","lglocstats","Being just a lowly software cretin, I can only deal with 3space location data.  You have given me %d-space location data.  Aborting.",dim);
	    return(WHACKED);
	}
	
	find_location_minmax(kobj,w,h,d,dim,&lmin,&lmax);

	fprintf(stderr," Min (x,y,z):\t%g\t%g\t%g\n",lmin.x,lmin.y,lmin.z);
	fprintf(stderr," Max (x,y,z):\t%g\t%g\t%g\n",lmax.x,lmax.y,lmax.z);
    }
    else
    {
	/* fudge it... */
	fprintf(stderr," -- lglocstats() cannot yet deal with the case in which there is no location data. \n");
    }
    return (CHILL);
}

int
find_location_minmax(kobject kobj,
		     int w,
		     int h,
		     int d,
		     int dim,
		     vertex *lmin,
		     vertex *lmax)
{
     vertex rlmin,rlmax;
     float *work;
     int i;
    int iw;
    float *base;

    /* NOTE: we're assuming dim == 3. */
    
    base = work = kmalloc(sizeof(float)*w*h);

    rlmin.x = rlmin.y = rlmin.z = KGEOM_BIGNUM;
    rlmax.x = rlmax.y = rlmax.z = KGEOM_SMALLNUM;

    /*  loop over depth.  */
    for (iw=0;iw<d;iw++)
    {
	/* grab some data */
	work = base;
	kpds_set_attribute(kobj,KPDS_LOCATION_POSITION,0,0,iw,0);
	kpds_get_data(kobj,KPDS_LOCATION_PLANE,base);

	/* do the x coordinate checks */
	if (*work < rlmin.x)
	    rlmin.x = *work;
	if (*work > rlmax.x)
	    rlmax.x = *work;

	work++;
	for (i=1;i<w*h;i++,work++)
	{
	    if (*work < rlmin.x)
		rlmin.x = *work;
	    else if (*work > rlmax.x)
		rlmax.x = *work;
	}

	work = base;
	kpds_set_attribute(kobj,KPDS_LOCATION_POSITION,0,0,iw,1);
	kpds_get_data(kobj,KPDS_LOCATION_PLANE,base);

	/* do the y coordinate checks */
	if (*work < rlmin.y)
	    rlmin.y = *work;
	if (*work > rlmax.y)
	    rlmax.y = *work;

	work++;
	for (i=1;i<w*h;i++,work++)
	{
	    if (*work < rlmin.y)
		rlmin.y = *work;
	    else if (*work > rlmax.y)
		rlmax.y = *work;
	}
	    
	work = base;
	kpds_set_attribute(kobj,KPDS_LOCATION_POSITION,0,0,iw,2);
	kpds_get_data(kobj,KPDS_LOCATION_PLANE,(float *)base);

	/* do the y coordinate checks */
	if (*work < rlmin.z)
	    rlmin.z = *work;
	if (*work > rlmax.z)
	    rlmax.z = *work;

	work++;
	for (i=1;i<w*h;i++,work++)
	{
	    if (*work < rlmin.z)
		rlmin.z = *work;
	    else if (*work > rlmax.z)
		rlmax.z = *work;
	}
    }

    lmin->x = rlmin.x;
    lmin->y = rlmin.y;
    lmin->z = rlmin.z;

    lmax->x = rlmax.x;
    lmax->y = rlmax.y;
    lmax->z = rlmax.z;
    
    kfree(base);

    return TRUE;
}
/* -library_code_end */
