
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 
/*********************************************************************


10 jan 95, wes. 
     changed how xsteps, ysteps are computed to fix lingering weirdness.


*********************************************************************/



#include <design.h>
#include <geometry.h>
#include <math.h>
#include "poly.h"


#define BIGNUM 1e10
    
line_draw(Poly *line,
	  PWindow *win,
	  void (*pixelproc)(int, int, Poly_vert *),
	  surface_colors_rgba *color1,
	  surface_colors_rgba *color2)
{
    int xsteps,ysteps;
    float x,y,m;
    double dz;
    Poly_vert p;
    int ix,iy,idx,idy;
    int start,end;

#ifdef DEBUG
    kfprintf(kstderr,"Drawing line from %g,%g to %g,%g\n",line->vert[0].sx,line->vert[0].sy,line->vert[1].sx,line->vert[1].sy);
#endif

#if 0    
    xsteps = (int)(line->vert[1].sx) - (int)(line->vert[0].sx);
    ysteps = (int)(line->vert[1].sy) - (int)(line->vert[0].sy);
#endif
    xsteps = (int)(line->vert[1].sx - line->vert[0].sx);
    ysteps = (int)(line->vert[1].sy - line->vert[0].sy);

#if 0
    if (kabs(xsteps) < kabs(ysteps))  /* increment along y */
#endif

    if (IABS(xsteps) < IABS(ysteps))  /* increment along y */
    {
	if (line->vert[0].sy < line->vert[1].sy)
	{
	    start = 0;
	    end = 1;
	}
	else
	{
	    start = 1;
	    end = 0;
	}

#if 0	
	xsteps = (int)(line->vert[end].sx) - (int)(line->vert[start].sx);
	ysteps = (int)(line->vert[end].sy) - (int)(line->vert[start].sy);
#endif
	xsteps = (int)(line->vert[end].sx - line->vert[start].sx);
	ysteps = (int)(line->vert[end].sy - line->vert[start].sy);

	if (xsteps == 0.)
	    m = 0.;
	else
	    m = (line->vert[end].sx - line->vert[start].sx)/(line->vert[end].sy - line->vert[start].sy);

	idy = 1.;

	if (ysteps != 0)
	    dz = (line->vert[end].sz - line->vert[start].sz)/(ysteps);
	else
	    dz = 0.;
	p.sz = line->vert[start].sz;
	
	
	for (iy=line->vert[start].sy,
	     x=line->vert[start].sx; ;iy+=idy,x+=m,ysteps--)
	{
	    ix = x;

	    (*pixelproc)(ix, iy, &p);
	     if ((ysteps == 0) || (iy >= (int)(line->vert[end].sy)))
		 break;

	    p.sz += dz;
	}

    }
    else  /* increment along X */
    {
	if (line->vert[0].sx < line->vert[1].sx)
	{
	    start = 0;
	    end = 1;
	}
	else
	{
	    start = 1;
	    end = 0;
	}

	xsteps = (int)(line->vert[end].sx - line->vert[start].sx);
	ysteps = (int)(line->vert[end].sy - line->vert[start].sy);

	if (xsteps == 0.)
	    m = BIGNUM;
	else
	    m = (line->vert[end].sy - line->vert[start].sy)/(line->vert[end].sx - line->vert[start].sx);

	idx = 1.;
	if (xsteps != 0)
	    dz = (line->vert[end].sz - line->vert[start].sz)/(xsteps);
	else
	    dz = 0.;
	p.sz = line->vert[start].sz;

	for (y=line->vert[start].sy,
	     ix=line->vert[start].sx; ;ix+=idx,y+=m,xsteps--)
	{
	    iy = y;
	    
	    (*pixelproc)(ix, iy, &p);
	     if ((xsteps==0) || (ix >= (int)(line->vert[end].sx)))
		 break;

	    p.sz += dz;
	}
    }	
    return(CHILL);
}


