
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 

/*
 * Khoros: $Id: rop.c,v 1.2 1996/05/30 15:48:30 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: rop.c,v 1.2 1996/05/30 15:48:30 mirage Exp $";
#endif

/*
 * $Log: rop.c,v $
 * Revision 1.2  1996/05/30 15:48:30  mirage
 * wes visit to kri may 96
 *
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  Static:
   >>>>             _static_routines()
   >>>>  Public:
   >>>>             public_routines()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "rmonster.h"
#include "zbuffer.h"
#include "rop.h"

int
rop_stencil(int xs,
	    int ys,
	    float *z,
	    rmonster_text_struct *rmt,
	    surface_colors_rgba *body_color,
	    int width,
	    int height)
{
    /* take an image mask, pointed to by rmt->mask, and for each non-zero
     entry in the mask, write a pixel iff the local z value is less than
     the value in the zbuffer.  this is a mask & zbuffered raster operation. */
    
    int w,h;
    int xscreen,yscreen,yflip;
    float dest_z,loc_z;
    surface_colors_rgb stencil_color;
    int status;

    /* this version ignores alpha */

    if (rmt->fg_color == NULL)
    {
	stencil_color.r = body_color->r;
	stencil_color.g = body_color->g;
	stencil_color.b = body_color->b;
    }
    else
    {
	stencil_color.r = rmt->fg_color->r;
	stencil_color.g = rmt->fg_color->g;
	stencil_color.b = rmt->fg_color->b;
    }

    loc_z = *z;
    yscreen = ys;
    yflip = rmt->r_height - 1;
    
    for (h=0;h<rmt->r_height;h++,yscreen++)
    {
	xscreen = xs;
	for (w=0;w<rmt->r_width;w++,xscreen++)
	{
	    if (rmt->mask[yflip - h][w] != 0)
	    {
		status = ((xscreen >=0) && (xscreen < width)) &&
		    ((yscreen >= 0) && (yscreen < height));
		if (status)
		{
		    zbuf_read(xscreen,yscreen,&dest_z);
		    if (loc_z < dest_z)
		    {
			zbuf_write(xscreen,yscreen,loc_z);
			write_dot(xscreen,yscreen,stencil_color.r,
				  stencil_color.g,stencil_color.b);
		    } /* if "in front" */
		} /* if on-screen */
	    } /* if text mask set */
	} /* loop over columns of text stencil */
    } /* loop over rows of text stencil */
}




