
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 

/*
 * Khoros: $Id: vrxfrm.c,v 1.4 1996/07/05 02:50:19 mirage Exp $
 */

/*
 * Log
 * Wed May 22 12:17:48 MDT 1996
 * support for xformation output from rmonster.
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vrxfrm.c,v 1.4 1996/07/05 02:50:19 mirage Exp $";
#endif


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            File Title
   >>>>
   >>>>  Static:
   >>>>             _static_routines()
   >>>>  Public:
   >>>>             public_routines()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "rmonster.h"
#include "rmonster_objs.h"
#include "vrxfrm.h"


static char vr_output_transport_name[256];
static int initted=0;

int
init_xform_output(char *transport_name)
{
  if ((transport_name == NULL) || (transport_name[0] == '\0'))
    initted = 0;
  else
  {
    kstrcpy(vr_output_transport_name,transport_name);
    initted = 1;
  }
  return(CHILL);
}

int
dump_xform_output(void)
{
  kobject o;
  int num_prims=3;
  matrix4x4 view,model;
  rmonsterobj *current_obj;

  /* the three prims we're going to write out are:
   * 1. the transformation of the currently selected object
   * 2. the rotation matrix associated with the current camera settings.
   *    NOTE: no translational or scaling values are present in the
   *          camera transformation matrix.
   * 3. camera position & orientation parameters.
   */

  if (initted == 0)
    return(1);

  /* open the output object... */
  o = kgeom_open_output_object(vr_output_transport_name);

  /* create the prim list.. */
  kgeom_create_primitive_list(o);

  kgeom_set_attribute(o,KGEOM_OBJECT,KGEOM_NUMBER_PRIMITIVES,num_prims);


  /* first, do the transformation matrix associated with the currently
   selected object.*/
  current_obj = get_current_object();
  kgeom_put_data(o, KGEOM_OBJECT_XFORM);
  kgeom_set_attributes(o, KGEOM_OBJECT, 
		       KGEOM_OBJECT_XFORM_MATRIX, &(current_obj->xfrm.m[0][0]),
		       KGEOM_OBJECT_XFORM_RELATIVE, 	KRELATIVE,
		       KGEOM_OBJECT_XFORM_OBJECT_NAME, current_obj->name,
		       NULL);

  
  /* next, do the transformation matrix associated with the camera */
/*  get_view_xform(&view); */
  get_view_rotation_matrix(&view);
  kgeom_put_data(o,KGEOM_VIEW_XFORM);
  kgeom_set_attributes(o, KGEOM_OBJECT,
		       KGEOM_VIEW_XFORM_MATRIX, &(view.m[0][0]),
		       NULL);

  {
      /* do the camera stuff.. */
      vertex_3d leye,lat,lup;
      float fov;
      int projection;
      float hither,yon;
      float focal_len;
      int i;
      
      get_eye(&leye);
      get_at(&lat);
      get_up(&lup);
      get_fov(&fov);
      get_projection(&projection);
      get_user_yon(&yon);
      get_user_hither(&hither);

      focal_len = 0;
      for (i=0;i<3;i++)
	  focal_len += ((leye.v[i]-lat.v[i])*(leye.v[i]-lat.v[i]));
      focal_len = (float)ksqrt((double)focal_len);
      
      
      kgeom_put_data(o,KGEOM_CAMERA_ORIENTATION);
      kgeom_set_attributes(o,KGEOM_OBJECT,
			   KGEOM_CAMERA_POSITION_AND_ORIENTATION,
			   &(leye.v[0]), &(lat.v[0]), &(lup.v[0]),
			   KGEOM_CAMERA_FIELD_OF_VIEW,fov,
			   KGEOM_CAMERA_HITHER_YON,hither,yon,
			   KGEOM_CAMERA_PROJECTION,projection,
			   KGEOM_CAMERA_IS_STEREO,FALSE, /* someday.. */
			   KGEOM_CAMERA_STEREO_VIEWING_PARAMETERS, 10.0, focal_len,
			   NULL);
  }

  /*fprintf(stderr,"stuffing vroutput...\n");*/
  kgeom_close_object(o);
}

int
vr_process_object(char *obj_file)
{
    kobject o;
    int i,num,*list;
    int rstat = WHACKED;

    /* this routine will return WHACKED if no net changes occured.  a
     value of CHILL corresponds roughly to a need to rerender. */

    o = kgeom_open_input_object(obj_file);
    kgeom_get_attributes(o, KGEOM_OBJECT, 
			 KGEOM_NUMBER_PRIMITIVES, &num,
			 KGEOM_PRIMITIVE_LIST,    &list,
			 NULL);
    for (i=0;i<num;i++)
    {
        if (list[i] == KGEOM_OBJECT_XFORM)
	{
	    float *matrix=NULL;
	    int relative;
	    char *name=NULL;
	    rmonsterobj *rmobj;
	    
	    kgeom_get_attributes(o, KGEOM_OBJECT, 
				 KGEOM_OBJECT_XFORM_MATRIX,&matrix,
				 KGEOM_OBJECT_XFORM_RELATIVE,&relative,
				 KGEOM_OBJECT_XFORM_OBJECT_NAME,&name, 
				 NULL);
	    {
		for (i=0;i<16;i++)
		    if (isnan(matrix[i]))
			break;
		if (i != 16)
	        {
		    kerror("GEOMETRY","rmonster","rmonster: NaN detected in input vr event. Skipping this event and continuing.\n");
		    return(WHACKED);
		}
		
	    }

	    if ((name == NULL) || (name[0] == '\0'))
	        rmobj = get_root_object();
	    else
	        find_rmonsterobj(name, &rmobj);
	    
	    if (rmobj == NULL)
	    {
	        kerror("GEOMETRY","rmonster", "i can't find the object named <%s> to which i would have applied a KGEOM_OBJECT_XFORM operation.  skipping the request and continuing. ",name);
	    }
	    else
	    {
	        if (relative == KABSOLUTE)
		    kmemcpy(&(rmobj->xfrm),matrix,sizeof(matrix4x4));
		else
		{
		    mmul_4x4(&(rmobj->xfrm),(matrix4x4 *)matrix,
			     &(rmobj->xfrm));
#if 0
		    {
			int i,j,offset=0;
			fprintf(stderr," matrix \n");
			for (j=0;j<4;j++)
			{
			    for (i=0;i<4;i++)
				fprintf(stderr," %g ",matrix[offset++]);
			    fprintf(stderr," \n");
			}
		    }
#endif
		}
		rstat = CHILL;
	    }
	    
	}
	else if (list[i] == KGEOM_CAMERA_ORIENTATION)
	{
	    /* read in new camera parameters. */
	    float fov,hither,yon;
	    int projection;
	    int is_stereo;
	    float eye_sep,foc_len;
	    int relative;
	    float *eye=NULL,*at=NULL,*up=NULL;
   
	    kgeom_get_attributes(o, KGEOM_OBJECT,
				 KGEOM_CAMERA_POSITION_AND_ORIENTATION,&eye, &at, &up,
				 KGEOM_CAMERA_FIELD_OF_VIEW,&fov, 
				 KGEOM_CAMERA_HITHER_YON,&hither, &yon,
				 KGEOM_CAMERA_PROJECTION,&projection, 
				 KGEOM_CAMERA_IS_STEREO, &is_stereo,
				 KGEOM_CAMERA_STEREO_VIEWING_PARAMETERS, &eye_sep, &foc_len,
				 NULL);

	    /* there is no notion of a "relative" camera operation. some
	     other module, namely gvr_xform_cam, deals with all that crap.
	     so, we just copy the values striaght into the guts of rmonster. */
	    
	    set_eye((vertex_3d *)eye);
	    set_at((vertex_3d *)at);
	    set_up((vertex_3d *)up);
	    set_fov(fov);
	    set_projection(projection);
	    set_user_yon(yon);
	    set_user_hither(hither);

	    compute_view_xform();
	    rstat = CHILL;
	}
	else if (list[i] == KGEOM_VIEW_XFORM)
	  /* in anticipation of this working someday */
	  kerror("GEOMETRY","rmonster","vr_process_object can't deal with a KGEOM_VIEW_XFORM operation.");
    }
    kgeom_close_object(o);
    return(rstat);
}
