 /*
  * Khoros: $Id: ifiltdesign.c,v 1.10 1996/07/05 02:54:00 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: ifiltdesign.c,v 1.10 1996/07/05 02:54:00 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for ifiltdesign
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "ifiltdesign.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - General 2-D image filter design
|
|       Purpose: main program for ifiltdesign
|
|         Input:
|		char *clui_info->o_file; {Output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		Required M.E. group:
|		    int clui_info->lp_flag; {TRUE if -lp specified}
|
|		    int clui_info->hp_flag; {TRUE if -hp specified}
|
|		    int clui_info->bp_flag; {TRUE if -bp specified}
|
|		    int clui_info->bs_flag; {TRUE if -bs specified}
|
|		Required M.E. group:
|		    int clui_info->ideal_flag; {TRUE if -ideal specified}
|
|		    int clui_info->but_flag; {TRUE if -but specified}
|
|		    int clui_info->gauss_flag; {TRUE if -gauss specified}
|
|		    int clui_info->ch1_flag; {TRUE if -ch1 specified}
|
|		    int clui_info->ch2_flag; {TRUE if -ch2 specified}
|
|		int clui_info->wsize_int; {Image width size (pixels)}
|		int clui_info->wsize_flag; {TRUE if -wsize specified}
|
|		int clui_info->hsize_int; {Image height (pixels)}
|		int clui_info->hsize_flag; {TRUE if -hsize specified}
|
|		int   clui_info->pos_cycle; {Low frequencies located at image center or corners}
|		char *clui_info->pos_label;
|		int   clui_info->pos_flag; {TRUE if -pos specified}
|
|		int clui_info->order_int; {Filter order (related to steepness of cutoff and amount of attenuation)}
|		int clui_info->order_flag; {TRUE if -order specified}
|
|		double clui_info->lcutoff_double; {Normalized lower cutoff frequency (fraction of Nyquist)}
|		int    clui_info->lcutoff_flag; {TRUE if -lcutoff specified}
|
|		double clui_info->ucutoff_double; {Normalized upper cutoff frequency (fraction of Nyquist)}
|		int    clui_info->ucutoff_flag; {TRUE if -ucutoff specified}
|
|		double clui_info->eps_double; {Ripple parameter (Chebyshev only)}
|		int    clui_info->eps_flag; {TRUE if -eps specified}
|
|		int   clui_info->type_cycle; {Data type for output image}
|		char *clui_info->type_label;
|		int   clui_info->type_flag; {TRUE if -type specified}
|
|        Output:
|       Returns:
|
|    Written By: Steven Jorgensen, Scott Wilson
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
	kobject fimage = NULL;
	int type = KFILT_LP;
	int brand = KFILT_IDEAL;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "IMAGE");
	kexit_handler(ifiltdesign_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "IMAGE", "ifiltdesign",
		ifiltdesign_usage_additions, ifiltdesign_get_args,
		ifiltdesign_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	if ((fimage = kpds_open_output_object(clui_info->o_file)) 
		== KOBJECT_INVALID)
	{
		kerror(NULL, "ifiltdesign", "Cannot open output image.");
		kexit(KEXIT_FAILURE);
	}
	if (!kpds_create_value(fimage))
	{
		kerror(NULL, "ifiltdesign", 
                             "Cannot create value data for file '%s'", 
                             clui_info->o_file);
		kexit(KEXIT_FAILURE);
	}
	if (!kpds_set_attributes(fimage, KPDS_VALUE_DATA_TYPE,
				(clui_info->type_cycle == 10 ?
				 KFLOAT : KDOUBLE),
				 KPDS_VALUE_SIZE, clui_info->wsize_int,
				 clui_info->hsize_int, 1, 1, 1, NULL))
	{
		kerror(NULL,"ifiltdesign", 
                            "Cannot set datatype and/or image size.");
		kexit(KEXIT_FAILURE);
	}

	if (clui_info->lp_flag)
		type = KFILT_LP;
	else if (clui_info->hp_flag)
		type = KFILT_HP;
	else if (clui_info->bp_flag)
		type = KFILT_BP;
	else if (clui_info->bs_flag)
		type = KFILT_BS;
	if (clui_info->ideal_flag)
		brand = KFILT_IDEAL;
	else if (clui_info->but_flag)
		brand = KFILT_BUTTERWORTH;
	else if (clui_info->gauss_flag)
		brand = KFILT_GAUSSIAN;
	else if (clui_info->ch1_flag)
		brand = KFILT_CHEBYSHEV_I;
	else if (clui_info->ch2_flag)
		brand = KFILT_CHEBYSHEV_II;
/* -main_before_lib_call_end */

/* -main_library_call */
	if (!lifiltdesign(type, brand,
			   clui_info->order_int,
			   clui_info->hsize_int, clui_info->wsize_int,
			   clui_info->lcutoff_double,
			   clui_info->ucutoff_double,
			   clui_info->eps_double,
			   clui_info->pos_cycle,
			   fimage))
	{
		kpds_close_object(fimage);
		kexit(KEXIT_FAILURE);
	}
/* -main_library_call_end */

/* -main_after_lib_call */
	/* add history to the output object */
	if (!kpds_set_attribute(fimage, KPDS_HISTORY, kpds_history_string()))
	{
	   kerror("ifiltdesign", "main", 
		  "Unable to set history on the destination object");
	   kexit(KEXIT_FAILURE);
	}

	kpds_close_object(fimage);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: ifiltdesign_usage_additions
| 
|       Purpose: Prints usage additions in ifiltdesign_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname ifiltdesign
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void ifiltdesign_usage_additions(void)
{
	kfprintf(kstderr, "\tGeneral 2-D image filter design\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: ifiltdesign_free_args
| 
|       Purpose: Frees CLUI struct allocated in ifiltdesign_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname ifiltdesign
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
ifiltdesign_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->pos_label);
		kfree_and_NULL(clui_info->type_label);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
