#! /bin/sh

echo "running test.sh for igradient ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-4 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0

# ------------------------------------------------------------------------
#  set path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/images"
BINARY_PATH=${BINARY_PATH:-`kecho -tb IMAGE -echo binpath`}

# ---------- Test 1 --> Roberts on impulse (double)
test_num=1
descr="Roberts on impulse (double)"

kimpulse -o $tmpdir/in.$$ -wsize 5 -hsize 5 -wnum 1 -hnum 1 -woff 2 -hoff 2 \
  -fgreal 1.0 -type 11

${BINARY_PATH}/igradient -roberts -i $tmpdir/in.$$ -o $tmpdir/out.$$

cat << EOF > $tmpdir/result.$$
0 0 0 0 0 
0 1 1 0 0 
0 1 1 0 0 
0 0 0 0 0 
0 0 0 0 0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ---------- Test 2 --> Sobel on impulse (double)
test_num=2
descr="Sobel on impulse (double)"

kimpulse -o $tmpdir/in.$$ -wsize 5 -hsize 5 -wnum 1 -hnum 1 -woff 2 -hoff 2 \
  -fgreal 1.0 -type 11

${BINARY_PATH}/igradient -sobel -i $tmpdir/in.$$ -o $tmpdir/out.$$

cat << EOF > $tmpdir/result.$$
0 0 0 0 0 
0 1.41421 2 1.41421 0 
0 2 0 2 0 
0 1.41421 2 1.41421 0 
0 0 0 0 0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ---------- Test 3 --> Prewitt on impulse (double)
test_num=3
descr="Prewitt on impulse (double)"

kimpulse -o $tmpdir/in.$$ -wsize 5 -hsize 5 -wnum 1 -hnum 1 -woff 2 -hoff 2 \
  -fgreal 1.0 -type 11

${BINARY_PATH}/igradient -prewitt -i $tmpdir/in.$$ -o $tmpdir/out.$$

cat << EOF > $tmpdir/result.$$
0 0 0 0 0 
0 1.41421 1 1.41421 0 
0 1 0 1 0 
0 1.41421 1 1.41421 0 
0 0 0 0 0
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ---------- Test 4 --> Isotropic on impulse (double)
test_num=4
descr="Isotropic on impulse (double)"

kimpulse -o $tmpdir/in.$$ -wsize 5 -hsize 5 -wnum 1 -hnum 1 -woff 2 -hoff 2 \
  -fgreal 1.0 -type 11

${BINARY_PATH}/igradient -isotropic -i $tmpdir/in.$$ -o $tmpdir/out.$$

cat << EOF > $tmpdir/result.$$
0 0 0 0 0 
0 1.41421 1.41421 1.41421 0 
0 1.41421 0 1.41421 0 
0 1.41421 1.41421 1.41421 0 
0 0 0 0 0 
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ---------- Test 5 --> Roberts on impulse (long)
test_num=5
descr="Roberts on impulse (double)"

kimpulse -o $tmpdir/in.$$ -wsize 5 -hsize 5 -wnum 1 -hnum 1 -woff 2 -hoff 2 \
  -fgreal 1.0 -type 8 

${BINARY_PATH}/igradient -roberts -i $tmpdir/in.$$ -o $tmpdir/out.$$

cat << EOF > $tmpdir/result.$$
0 0 0 0 0
0 1 1 0 0
0 1 1 0 0
0 0 0 0 0
0 0 0 0 0
EOF

kasc2val -i1 $tmpdir/result.$$ -o $tmpdir/result.$$ -type 8 -wsize 5 -hsize 5

report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

exit $status
