#! /bin/sh

echo "running test.sh for itexture ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-4 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0

# ------------------------------------------------------------------------
#  set path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/images"
BINARY_PATH=${BINARY_PATH:-`kecho -tb IMAGE -echo binpath`}

# ---------- Test 1 --> L5L5 on impulse (double)
 
test_num=1
descr="L5L5 on impulse (double)"

kimpulse -o $tmpdir/in.$$ -wsize 5 -hsize 5 -wnum 1 -hnum 1 -woff 2 -hoff 2 -fgreal 1.0 -type 11

${BINARY_PATH}/itexture -L5L5 -i $tmpdir/in.$$ -o $tmpdir/out.$$

cat << EOF > $tmpdir/result.$$
         1  4  6  4  1
         4 16 24 16  4
         6 24 36 24  6
         4 16 24 16  4
         1  4  6  4  1
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ---------- Test 2 --> E5R5 on impulse (byte)
 
test_num=2
descr="E5R5 on impulse (byte)"

kimpulse -o $tmpdir/in.$$ -wsize 5 -hsize 5 -wnum 1 -hnum 1 -woff 2 -hoff 2 -fgreal 1.0 -type 2

${BINARY_PATH}/itexture -i $tmpdir/in.$$ -E5R5 -o $tmpdir/out.$$

cat << EOF > $tmpdir/ascii.$$
         -1  4 -6  4 -1
         -2  8 -12 8 -2
          0  0  0  0  0
          2 -8 12 -8  2
          1 -4  6 -4  1
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 5 -hsize 5 -type 8 -o $tmpdir/result.$$

report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ---------- Test 3 --> L5L5 and E5R5 on impulse (byte)
 
test_num=3
descr="L5L5 and E5R5 on impulse (byte)"

kimpulse -o $tmpdir/in.$$ -wsize 5 -hsize 5 -wnum 1 -hnum 1 -woff 2 -hoff 2 -fgreal 1.0 -type 2

${BINARY_PATH}/itexture -i $tmpdir/in.$$ -L5L5 -E5R5 -o $tmpdir/out.$$

cat << EOF > $tmpdir/ascii.$$
          1  4  6  4  1
          4 16 24 16  4
          6 24 36 24  6
          4 16 24 16  4
          1  4  6  4  1
         -1  4 -6  4 -1
         -2  8 -12 8 -2
          0  0  0  0  0
          2 -8 12 -8  2
          1 -4  6 -4  1
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 5 -hsize 5 -esize 2 -type 8 -o $tmpdir/result.$$

report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ---------- Test 4 --> L5L5 and E5R5 on 3-band impulse (byte)
 
test_num=4
descr="L5L5 and E5R5 on 2-band (elements) impulse (byte)"
 
kimpulse -o $tmpdir/in.$$ -wsize 5 -hsize 5 -esize 3 \
                          -wnum 1 -hnum 1 -enum 3 \
                          -woff 2 -hoff 2 -eoff 0 \
                          -esp 1 -fgreal 1.0 -type 2
 
${BINARY_PATH}/itexture -i $tmpdir/in.$$ -L5L5 -E5R5 -o $tmpdir/out.$$
 
cat << EOF > $tmpdir/ascii.$$
          1  4  6  4  1
          4 16 24 16  4
          6 24 36 24  6
          4 16 24 16  4
          1  4  6  4  1
          1  4  6  4  1
          4 16 24 16  4
          6 24 36 24  6
          4 16 24 16  4
          1  4  6  4  1
          1  4  6  4  1
          4 16 24 16  4
          6 24 36 24  6
          4 16 24 16  4
          1  4  6  4  1
         -1  4 -6  4 -1
         -2  8 -12 8 -2
          0  0  0  0  0
          2 -8 12 -8  2
          1 -4  6 -4  1
         -1  4 -6  4 -1
         -2  8 -12 8 -2
          0  0  0  0  0
          2 -8 12 -8  2
          1 -4  6 -4  1
         -1  4 -6  4 -1
         -2  8 -12 8 -2
          0  0  0  0  0
          2 -8 12 -8  2
          1 -4  6 -4  1
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 5 -hsize 5 -esize 6 -type 8 -o $tmpdir/result.$$
 
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

exit $status
