 /*
  * Khoros: $Id: vcast.c,v 1.11 1996/07/05 03:19:45 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vcast.c,v 1.11 1996/07/05 03:19:45 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vcast
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vcast.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Upconvert All Inputs to the Highest Input Data Type  (K1)
|
|       Purpose: main program for vcast
|
|         Input:
|		char *clui_info->i1_file; {input image 1}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->i2_file; {input image 2}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		char *clui_info->o1_file; {output image 1}
|		int   clui_info->o1_flag; {TRUE if -o1 specified}
|
|		char *clui_info->o2_file; {output image 2}
|		int   clui_info->o2_flag; {TRUE if -o2 specified}
|
|		char *clui_info->i3_file; {input image 3}
|		int   clui_info->i3_flag; {TRUE if -i3 specified}
|
|		char *clui_info->i4_file; {input image 4}
|		int   clui_info->i4_flag; {TRUE if -i4 specified}
|
|		char *clui_info->i5_file; {input image 5}
|		int   clui_info->i5_flag; {TRUE if -i5 specified}
|
|		char *clui_info->t_toggle; {Select data type [default = Auto Upcast to Highest Input Data Type]}
|		int   clui_info->t_flag; {TRUE if -t specified}
|
|		char *clui_info->o3_file; {output image 3}
|		int   clui_info->o3_flag; {TRUE if -o3 specified}
|
|		char *clui_info->o4_file; {output image 4}
|		int   clui_info->o4_flag; {TRUE if -o4 specified}
|
|		char *clui_info->o5_file; {output image 5}
|		int   clui_info->o5_flag; {TRUE if -o5 specified}
|
|        Output:
|       Returns:
|
|    Written By: Charlie Gage
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
char *program = NULL;
xvimage *img1, *img2, *img3, *img4, *img5, **images;
int  i, num_images, type;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vcast_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vcast",
		vcast_usage_additions, vcast_get_args,
		vcast_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
  i = 0;

  /*
   * If the optional type selection is used 
   *  then get the appropriate data type.
   */
  if (clui_info->t_flag)
  {
    if (!vget_type(&type, clui_info->t_toggle))
    {
      (void)kfprintf(kstderr, "vcast: unknown data type!\n");
      kexit(KEXIT_FAILURE);
    }
  }
  else    /* otherwise, do auto upconvert to highest input image data type */
  {
    type = -1;
  }

                          /* Find out how many images there are. */
  num_images = 2;         /* must have at least two input images */
  if (clui_info->i3_flag)     /* now count the optional images */
      num_images++;
  if (clui_info->i4_flag)
      num_images++;
  if (clui_info->i5_flag)
      num_images++;
                          /* get space for image structures */
  images = (xvimage **) kmalloc(sizeof(xvimage *) * num_images);
  if (images == NULL)
  {
    (void) kfprintf(kstderr,"vcast: Unable to malloc space for image structures\n");
    kexit(KEXIT_FAILURE);
  }

  img1 = readimage(clui_info->i1_file);
  if (img1 == NULL)
  {
   (void) kfprintf(kstderr,"vcast: Can not read input image #1\n");
    kexit(KEXIT_FAILURE);                    
  }
  images[i++] = img1;

  img2 = readimage(clui_info->i2_file);
  if (img2 == NULL)
  {
   (void) kfprintf(kstderr,"vcast: Can not read input image #2\n");
    kexit(KEXIT_FAILURE);                    
  }
  images[i++] = img2;

  (void) proper_num_images(program, img1, 1, TRUE);

  MATCHINPUT(program, img1, img2);      /* Verify input images */

    /* Now check for optional input images */
if ( clui_info->i3_flag )
{
  img3 = readimage(clui_info->i3_file);
  if (img3 == NULL)
  {
   (void) kfprintf(kstderr,"vcast: Can not read input image #3\n");
    kexit(KEXIT_FAILURE);                    
  }
  images[i++] = img3;
  MATCHINPUT(program, img1, img3);      /* Verify input images */
}

if ( clui_info->i4_flag )
{
  img4 = readimage(clui_info->i4_file);
  if (img4 == NULL)
  {
   (void) kfprintf(kstderr,"vcast: Can not read input image #4\n");
    kexit(KEXIT_FAILURE);                    
  }
  images[i++] = img4;
  MATCHINPUT(program, img1, img4);      /* Verify input images */
}

if ( clui_info->i5_flag )
{
  img5 = readimage(clui_info->i5_file);
  if (img5 == NULL)
  {
   (void) kfprintf(kstderr,"vcast: Can not read input image #5\n");
    kexit(KEXIT_FAILURE);                    
  }
  images[i] = img5;
  MATCHINPUT(program, img1, img5);      /* Verify input images */
}
/* -main_before_lib_call_end */

/* -main_library_call */
if(! lvcast(images, num_images, type) )
  {
   (void) kfprintf(kstderr, "lvcast Failed\n");
   kexit(KEXIT_FAILURE);
  }
/* -main_library_call_end */

/* -main_after_lib_call */
(void) writeimage(clui_info->o1_file, img1);
 (void) writeimage(clui_info->o2_file, img2);

 if ( clui_info->i3_flag)
    (void) writeimage(clui_info->o3_file, img3);

 if ( clui_info->i4_flag)
    (void) writeimage(clui_info->o4_file, img4);

 if ( clui_info->i5_flag)
    (void) writeimage(clui_info->o5_file, img5);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vcast_usage_additions
| 
|       Purpose: Prints usage additions in vcast_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vcast
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vcast_usage_additions(void)
{
	kfprintf(kstderr, "\tUpconvert All Inputs to the Highest Input Data Type  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vcast_free_args
| 
|       Purpose: Frees CLUI struct allocated in vcast_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vcast
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vcast_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->o1_file);
		kfree_and_NULL(clui_info->o2_file);
		kfree_and_NULL(clui_info->i3_file);
		kfree_and_NULL(clui_info->i4_file);
		kfree_and_NULL(clui_info->i5_file);
		kfree_and_NULL(clui_info->t_toggle);
		kfree_and_NULL(clui_info->o3_file);
		kfree_and_NULL(clui_info->o4_file);
		kfree_and_NULL(clui_info->o5_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
