 /*
  * Khoros: $Id: vcmtrans.c,v 1.11 1996/07/05 03:20:12 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vcmtrans.c,v 1.11 1996/07/05 03:20:12 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vcmtrans
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vcmtrans.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Linear Color Space Conversion with Specified 3x3 Matrix  (K1)
|
|       Purpose: main program for vcmtrans
|
|         Input:
|		char *clui_info->i1_file; {input image}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->i2_file; {input transformation matrix}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		char *clui_info->o_file; {output resulting image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		float clui_info->n_float; {specifies normalization factor}
|		int   clui_info->n_flag; {TRUE if -n specified}
|
|        Output:
|       Returns:
|
|    Written By: Tom Sauer
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
char *program = NULL;
xvimage *image, *image1;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vcmtrans_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vcmtrans",
		vcmtrans_usage_additions, vcmtrans_get_args,
		vcmtrans_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
READINPUT (image);
READINPUT1 (image1);
CHECKINPUT (program, image);
CHECKMATRIXINPUT (program, image1);
/* -main_before_lib_call_end */

/* -main_library_call */
if(!lvcltrans(image, 0, 0, clui_info->n_float, clui_info->n_flag, TRUE, image1))
 {
    (void) kerror(NULL, NULL, "lvcltrans Failed");
    kexit(1);
 }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vcmtrans_usage_additions
| 
|       Purpose: Prints usage additions in vcmtrans_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vcmtrans
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vcmtrans_usage_additions(void)
{
	kfprintf(kstderr, "\tLinear Color Space Conversion with Specified 3x3 Matrix  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vcmtrans_free_args
| 
|       Purpose: Frees CLUI struct allocated in vcmtrans_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vcmtrans
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vcmtrans_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
