 /*
  * Khoros: $Id: vcost.c,v 1.11 1996/07/05 03:20:27 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vcost.c,v 1.11 1996/07/05 03:20:27 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vcost
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vcost.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Compute Cost (Surface Arc Length) for Image  (K1)
|
|       Purpose: main program for vcost
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->m_file; {mask image}
|		int   clui_info->m_flag; {TRUE if -m specified}
|
|		int clui_info->x_int; {X coordinate of reference pixel}
|		int clui_info->x_flag; {TRUE if -x specified}
|
|		int clui_info->y_int; {Y coordinate of reference pixel}
|		int clui_info->y_flag; {TRUE if -y specified}
|
|        Output:
|       Returns:
|
|    Written By: Scott Wilson
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
	char    *program = NULL;
	xvimage *img, *mask, *m;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vcost_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vcost",
		vcost_usage_additions, vcost_get_args,
		vcost_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        img = readimage(clui_info->i_file);
        if (img == NULL)
	  {
	    kerror("vipl", "vsurf", "Could not read %s.\n", clui_info->i_file);
	    kexit(KEXIT_FAILURE);                    
	  }

        proper_num_images(program,img,1,1);
        proper_loc_type(program,img,VFF_LOC_IMPLICIT,1);
        proper_num_bands(program,img,1,1);
        if (img->data_storage_type != VFF_TYP_FLOAT)
          {
            if (! lvconvert(img,VFF_TYP_FLOAT,0,1,0.0,1.0,0))
	      {
		kexit(KEXIT_FAILURE);
	      }
          }

        mask = NULL;
        if (clui_info->m_flag != 0)
          {
            m =readimage(clui_info->m_file);
            if (m == NULL)
	      {
		kerror("vipl", "vsurf", "Could not read %s.\n",
		       clui_info->m_file);
		kexit(KEXIT_FAILURE);
	      }
            match_num_images(program,m,img,1);
            match_loc_type(program,m,img,1);
            match_num_bands(program,m,img,1);
            matchsize(program,m,img,1);
            if (m->data_storage_type != VFF_TYP_FLOAT)
              {
                mask = copyimage(m);
                if (! lvconvert(mask,VFF_TYP_FLOAT,0,1,0.0,1.0,0))
	        {
		  kexit(KEXIT_FAILURE);
	        }
              }
            else
              mask = m;
          }
/* -main_before_lib_call_end */

/* -main_library_call */
	if(! lvcost(img,mask,clui_info->x_int,clui_info->y_int))
        {
            kexit(KEXIT_FAILURE);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
	(void) writeimage(clui_info->o_file,img);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vcost_usage_additions
| 
|       Purpose: Prints usage additions in vcost_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vcost
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vcost_usage_additions(void)
{
	kfprintf(kstderr, "\tCompute Cost (Surface Arc Length) for Image  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vcost_free_args
| 
|       Purpose: Frees CLUI struct allocated in vcost_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vcost
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vcost_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->m_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
