 /*
  * Khoros: $Id: vdistance.c,v 1.11 1996/07/05 03:20:46 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vdistance.c,v 1.11 1996/07/05 03:20:46 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vdistance
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vdistance.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Compute Distance from the Boundary in a Two-Valued Image  (K1)
|
|       Purpose: main program for vdistance
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|        Output:
|       Returns:
|
|    Written By: Pascal ADAM
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
char *program = NULL;
xvimage *image;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vdistance_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vdistance",
		vdistance_usage_additions, vdistance_get_args,
		vdistance_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	image = readimage(clui_info->i_file);
	if (image == NULL) {
	   (void) kfprintf(kstderr, "vdistance: Can not read input image \n");
	   kexit(KEXIT_FAILURE);  /* Quit if bad image */
	}
        CHECKINPUT(program, image) ;
/* -main_before_lib_call_end */

/* -main_library_call */
	if(! lvdistance(image))
        {
                (void) kfprintf(kstderr, "vdistance: lvdistance Failed\n");
                kexit(KEXIT_FAILURE);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vdistance_usage_additions
| 
|       Purpose: Prints usage additions in vdistance_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vdistance
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vdistance_usage_additions(void)
{
	kfprintf(kstderr, "\tCompute Distance from the Boundary in a Two-Valued Image\n");
	kfprintf(kstderr, "\t(K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vdistance_free_args
| 
|       Purpose: Frees CLUI struct allocated in vdistance_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vdistance
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vdistance_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
