 /*
  * Khoros: $Id: vfractal.c,v 1.11 1996/07/05 03:21:21 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vfractal.c,v 1.11 1996/07/05 03:21:21 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vfractal
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vfractal.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Fractal Feature Extraction  (K1)
|
|       Purpose: main program for vfractal
|
|         Input:
|		char *clui_info->i_file; {input image filename}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output image filename}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->r_int; {region size argument}
|		int clui_info->r_flag; {TRUE if -r specified}
|
|		int clui_info->w_int; {initial window size argument}
|		int clui_info->w_flag; {TRUE if -w specified}
|
|		int clui_info->s_int; {step size argument}
|		int clui_info->s_flag; {TRUE if -s specified}
|
|        Output:
|       Returns:
|
|    Written By: Charlie Gage
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
char *program = NULL;
xvimage *image;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vfractal_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vfractal",
		vfractal_usage_additions, vfractal_get_args,
		vfractal_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
READINPUT(image) ;
CHECKINPUT(program, image) ;
/* -main_before_lib_call_end */

/* -main_library_call */
/* Call the lib routine for calculation of fractal dimension */

if(! lvfractal(image, clui_info->r_int, clui_info->w_int, clui_info->s_int))
 {
   (void) kfprintf(kstderr, "lvfractal Failed\\n");
   kexit(KEXIT_FAILURE);
 }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vfractal_usage_additions
| 
|       Purpose: Prints usage additions in vfractal_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vfractal
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vfractal_usage_additions(void)
{
	kfprintf(kstderr, "\tFractal Feature Extraction  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vfractal_free_args
| 
|       Purpose: Frees CLUI struct allocated in vfractal_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vfractal
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vfractal_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
