 /*
  * Khoros: $Id: vgamth.c,v 1.11 1996/07/05 03:21:29 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vgamth.c,v 1.11 1996/07/05 03:21:29 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vgamth
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vgamth.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Generate a Binary Image by Thresholding a Color Image  (K1)
|
|       Purpose: main program for vgamth
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {resulting image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->b_int; {high threshold for map column 1}
|		int clui_info->b_flag; {TRUE if -b specified}
|
|		int clui_info->c_int; {low threshold for map column 1}
|		int clui_info->c_flag; {TRUE if -c specified}
|
|		int clui_info->d_int; {high threshold for map column 2}
|		int clui_info->d_flag; {TRUE if -d specified}
|
|		int clui_info->e_int; {low threshold for map column 2}
|		int clui_info->e_flag; {TRUE if -e specified}
|
|		int clui_info->f_int; {high threshold for map column 3}
|		int clui_info->f_flag; {TRUE if -f specified}
|
|		int clui_info->g_int; {low threshold for map column 3}
|		int clui_info->g_flag; {TRUE if -g specified}
|
|		int clui_info->v_int; {non-zero output value}
|		int clui_info->v_flag; {TRUE if -v specified}
|
|        Output:
|       Returns:
|
|    Written By: Tom Sauer
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
char *program = NULL;
xvimage *img1;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vgamth_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vgamth",
		vgamth_usage_additions, vgamth_get_args,
		vgamth_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
img1 = readimage(clui_info->i_file);
    if (img1 == NULL) kexit(KEXIT_FAILURE);

    CHECKINPUT(program,img1);   /* Verify input image */
/* -main_before_lib_call_end */

/* -main_library_call */
if(! lvgamth(img1,clui_info->b_int,clui_info->c_int,clui_info->d_int,clui_info->e_int,
                 clui_info->f_int,clui_info->g_int,clui_info->c_flag,clui_info->b_flag,
                 clui_info->g_flag,clui_info->f_flag,clui_info->e_flag,clui_info->d_flag,
                 clui_info->v_flag, clui_info->v_int))
    {
        (void) kfprintf(kstderr, "lvgamth Failed\n");
        kexit(KEXIT_FAILURE);
    }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,img1);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vgamth_usage_additions
| 
|       Purpose: Prints usage additions in vgamth_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vgamth
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vgamth_usage_additions(void)
{
	kfprintf(kstderr, "\tGenerate a Binary Image by Thresholding a Color Image  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vgamth_free_args
| 
|       Purpose: Frees CLUI struct allocated in vgamth_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vgamth
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vgamth_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
