 /*
  * Khoros: $Id: vgbox.c,v 1.12 1996/07/05 03:21:36 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vgbox.c,v 1.12 1996/07/05 03:21:36 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vgbox
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vgbox.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Create Image of Parallelogram Projected onto CCD Sensor  (K1)
|
|       Purpose: main program for vgbox
|
|         Input:
|		char *clui_info->o_file; {resulting output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->trigger_file; {trigger input}
|		int   clui_info->trigger_flag; {TRUE if -trigger specified}
|
|		int clui_info->r_int; {Number of Rows (Height in pixels) of the CCD sensor}
|		int clui_info->r_flag; {TRUE if -r specified}
|
|		int clui_info->c_int; {Number of Columns (Width in pixels) of the CCD sensor}
|		int clui_info->c_flag; {TRUE if -c specified}
|
|		float clui_info->px_h_float; {Active Pixel Height (vertical)  }
|		int   clui_info->px_h_flag; {TRUE if -px_h specified}
|
|		float clui_info->px_w_float; {Active Pixel Width (horizontal) }
|		int   clui_info->px_w_flag; {TRUE if -px_w specified}
|
|		float clui_info->px_y_off_float; {Active Pixel offset from the top  of the cell}
|		int   clui_info->px_y_off_flag; {TRUE if -px_y_off specified}
|
|		float clui_info->px_x_off_float; {Active Pixel offset from the left of the cell}
|		int   clui_info->px_x_off_flag; {TRUE if -px_x_off specified}
|
|		float clui_info->h_float; {Height of the parallelogram (box) (in pixel unit)}
|		int   clui_info->h_flag; {TRUE if -h specified}
|
|		float clui_info->w_float; {Width  of the parallelogram (box) (in pixel unit)}
|		int   clui_info->w_flag; {TRUE if -w specified}
|
|		float clui_info->im_y_off_float; {Downward offset of box (0 is CCD center = box center)}
|		int   clui_info->im_y_off_flag; {TRUE if -im_y_off specified}
|
|		float clui_info->im_x_off_float; {Right    offset of box (0 is CCD center = box center)}
|		int   clui_info->im_x_off_flag; {TRUE if -im_x_off specified}
|
|		float clui_info->angle_float; {Box rotation (clockwise degrees, center = box center)}
|		int   clui_info->angle_flag; {TRUE if -angle specified}
|
|		float clui_info->corner_angl_float; {the value of the 1st corner angle (degrees)}
|		int   clui_info->corner_angl_flag; {TRUE if -corner_angl specified}
|
|		int clui_info->g_o_int; {Grey level of the background pixels outside of box}
|		int clui_info->g_o_flag; {TRUE if -g_o specified}
|
|		int clui_info->g_i_int; {Grey level of the lighted pixels inside of box}
|		int clui_info->g_i_flag; {TRUE if -g_i specified}
|
|        Output:
|       Returns:
|
|    Written By: Jean-Pierre GUERIN  (CEA)
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
char *program = NULL;

    xvimage *image;

    char           *dummy = '\0';
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vgbox_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vgbox",
		vgbox_usage_additions, vgbox_get_args,
		vgbox_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
    image    = createimage(clui_info->r_int,
                           clui_info->c_int,
                           VFF_TYP_1_BYTE,
                           1,
                           1,
                           dummy,
                           0,
                           0,
                           VFF_MS_NONE,
                           VFF_MAPTYP_NONE,
                           VFF_LOC_IMPLICIT,
                           0);
    if (image == NULL)
    {
       (void) kfprintf(kstderr,"vgbox: Unable to allocate memory!\\n");
       kexit(KEXIT_FAILURE);
    }
/* -main_before_lib_call_end */

/* -main_library_call */
    switch (lvgbox(
                 (unsigned char *) image->imagedata,
                 (unsigned short) clui_info->c_int,
                 (unsigned short) clui_info->r_int,
                 clui_info->px_w_float,
                 clui_info->px_h_float,
                 clui_info->px_x_off_float,
                 clui_info->px_y_off_float,
                 (unsigned char) clui_info->g_o_int,
                 (unsigned char) clui_info->g_i_int,
                 clui_info->w_float,
                 clui_info->h_float,
                 clui_info->im_x_off_float,
                 clui_info->im_y_off_float,
                 (float) clui_info->angle_float * KPI/180.0,
                 (float) clui_info->corner_angl_float * KPI/180.0))
     {
        case 0: break;                    /* ok       */
        case 1:                           /* warnings */
                (void) kfprintf(kstderr,
                               "lvgbox: Internal Warning, pixel area found > 1. \n");
                break;
        case 2:                           /* error */
                (void) kfprintf(kstderr,
                               "lvgbox: Pixel width+pixel offset must be <= 1\n");
                kexit(KEXIT_FAILURE);
                break;
        default:                          /* unknown error */
                (void) kfprintf(kstderr,
                               "lvgbox: Not Registrated Error \n");
                kexit(KEXIT_FAILURE);
                break;
      }
/* -main_library_call_end */

/* -main_after_lib_call */
    writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vgbox_usage_additions
| 
|       Purpose: Prints usage additions in vgbox_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vgbox
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vgbox_usage_additions(void)
{
	kfprintf(kstderr, "\tCreate Image of Parallelogram Projected onto CCD Sensor  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vgbox_free_args
| 
|       Purpose: Frees CLUI struct allocated in vgbox_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vgbox
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vgbox_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->trigger_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
