 /*
  * Khoros: $Id: vgcirc.c,v 1.11 1996/07/05 03:21:44 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vgcirc.c,v 1.11 1996/07/05 03:21:44 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vgcirc
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vgcirc.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Create Image of a Filled Circle  (K1)
|
|       Purpose: main program for vgcirc
|
|         Input:
|		char *clui_info->o_file; {output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->trigger_file; {trigger input}
|		int   clui_info->trigger_flag; {TRUE if -trigger specified}
|
|		int clui_info->r_int; {Number of rows in output image}
|		int clui_info->r_flag; {TRUE if -r specified}
|
|		int clui_info->c_int; {Number of columns in output image}
|		int clui_info->c_flag; {TRUE if -c specified}
|
|		float clui_info->d_float; {Diameter of circle (in pixels)}
|		int   clui_info->d_flag; {TRUE if -d specified}
|
|		float clui_info->x_float; {X coordinate of center of circle}
|		int   clui_info->x_flag; {TRUE if -x specified}
|
|		float clui_info->y_float; {Y coordinate of center of circle}
|		int   clui_info->y_flag; {TRUE if -y specified}
|
|		float clui_info->b_float; {Background level}
|		int   clui_info->b_flag; {TRUE if -b specified}
|
|		float clui_info->f_float; {Foreground level}
|		int   clui_info->f_flag; {TRUE if -f specified}
|
|		char *clui_info->t_toggle; {Selects data type of output image}
|		int   clui_info->t_flag; {TRUE if -t specified}
|
|        Output:
|       Returns:
|
|    Written By: Scott Wilson
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
char *program = NULL;

xvimage *image;
int type;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vgcirc_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vgcirc",
		vgcirc_usage_additions, vgcirc_get_args,
		vgcirc_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
if(!vget_type(&type, clui_info->t_toggle)){
   kfprintf(kstderr,"vgcirc:  Invalid data type specified\n");
   kexit(KEXIT_FAILURE);
}

if(type!=VFF_TYP_1_BYTE && type!=VFF_TYP_FLOAT){
   kfprintf(kstderr,"%s:  Illegal image data type specified.\n",program);
   kexit(KEXIT_FAILURE);
}

if (clui_info->r_int <= 0 || clui_info->c_int <= 0){
  kfprintf(kstderr,"%s: Rows and columns must be positive and non-zero\n",program);
  kexit(KEXIT_FAILURE);
}
/* -main_before_lib_call_end */

/* -main_library_call */
if(! lvgcirc( &image,clui_info->r_int,clui_info->c_int, type, clui_info->b_float, clui_info->f_float,clui_info->x_float,clui_info->y_float, clui_info->d_float))
  {
      (void) kfprintf(kstderr, "lvgcirc Failed\n");
      kexit(KEXIT_FAILURE);
  }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vgcirc_usage_additions
| 
|       Purpose: Prints usage additions in vgcirc_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vgcirc
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vgcirc_usage_additions(void)
{
	kfprintf(kstderr, "\tCreate Image of a Filled Circle  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vgcirc_free_args
| 
|       Purpose: Frees CLUI struct allocated in vgcirc_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vgcirc
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vgcirc_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->trigger_file);
		kfree_and_NULL(clui_info->t_toggle);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
