 /*
  * Khoros: $Id: vgpwl.c,v 1.11 1996/07/05 03:22:16 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vgpwl.c,v 1.11 1996/07/05 03:22:16 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vgpwl
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vgpwl.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Create 2D Piecewise Linear Periodic Function  (K1)
|
|       Purpose: main program for vgpwl
|
|         Input:
|		char *clui_info->o_file; {resulting image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->trigger_file; {trigger input}
|		int   clui_info->trigger_flag; {TRUE if -trigger specified}
|
|		int clui_info->r_int; {number of rows}
|		int clui_info->r_flag; {TRUE if -r specified}
|
|		float clui_info->s_float; {sampling frequency}
|		int   clui_info->s_flag; {TRUE if -s specified}
|
|		int clui_info->c_int; {number of columns}
|		int clui_info->c_flag; {TRUE if -c specified}
|
|		float clui_info->l_float; {signal minimum}
|		int   clui_info->l_flag; {TRUE if -l specified}
|
|		int clui_info->b_int; {number of bands}
|		int clui_info->b_flag; {TRUE if -b specified}
|
|		float clui_info->m_float; {signal maximum}
|		int   clui_info->m_flag; {TRUE if -m specified}
|
|		float clui_info->xp_float; {signal period in X-direction}
|		int   clui_info->xp_flag; {TRUE if -xp specified}
|
|		float clui_info->yp_float; {signal period in Y-direction}
|		int   clui_info->yp_flag; {TRUE if -yp specified}
|
|		float clui_info->xr_float; {rise time in X-direction}
|		int   clui_info->xr_flag; {TRUE if -xr specified}
|
|		float clui_info->yr_float; {rise time in Y-direction }
|		int   clui_info->yr_flag; {TRUE if -yr specified}
|
|		float clui_info->xf_float; {fall time in X-direction }
|		int   clui_info->xf_flag; {TRUE if -xf specified}
|
|		float clui_info->yf_float; {fall time in Y-direction }
|		int   clui_info->yf_flag; {TRUE if -yf specified}
|
|		float clui_info->xw_float; {pulse width in X-direction }
|		int   clui_info->xw_flag; {TRUE if -xw specified}
|
|		float clui_info->yw_float; {pulse width in Y-direction }
|		int   clui_info->yw_flag; {TRUE if -yw specified}
|
|		char *clui_info->t_toggle; {lets user choose data type of image}
|		int   clui_info->t_flag; {TRUE if -t specified}
|
|        Output:
|       Returns:
|
|    Written By: Jeremy Worley
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
char *program = NULL;
xvimage *image;
int type;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vgpwl_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vgpwl",
		vgpwl_usage_additions, vgpwl_get_args,
		vgpwl_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
  if(!vget_type(&type,clui_info->t_toggle)){
     kfprintf(kstderr,"vgpwl:  Invalid data type specified\n");
     kexit(KEXIT_FAILURE);
  }
/* -main_before_lib_call_end */

/* -main_library_call */
  if(!lvgpwl(&image,clui_info->r_int,clui_info->c_int,clui_info->b_int,clui_info->s_float,
             clui_info->l_float,clui_info->m_float,clui_info->xp_float,clui_info->yp_float,
             clui_info->xr_float,clui_info->yr_float,clui_info->xf_float,clui_info->yf_float,
             clui_info->xw_float,clui_info->yw_float,type)){
    (void)kfprintf(kstderr, "lvgpwl Failed\n");
    kexit(KEXIT_FAILURE);
  }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vgpwl_usage_additions
| 
|       Purpose: Prints usage additions in vgpwl_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vgpwl
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vgpwl_usage_additions(void)
{
	kfprintf(kstderr, "\tCreate 2D Piecewise Linear Periodic Function  (K1)\n");

/* -usage_additions */
kfprintf(kstderr,"\nThe -t option will accept input in either of two forms:\n");
kfprintf(kstderr,"  an alphabetic representation, i.e. 'float' or 'complex',\n");
kfprintf(kstderr,"  or a numeric representation, such as 4 for real.\n");
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vgpwl_free_args
| 
|       Purpose: Frees CLUI struct allocated in vgpwl_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vgpwl
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vgpwl_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->trigger_file);
		kfree_and_NULL(clui_info->t_toggle);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
