 /*
  * Khoros: $Id: vmediaxis.c,v 1.11 1996/07/05 03:23:13 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vmediaxis.c,v 1.11 1996/07/05 03:23:13 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vmediaxis
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vmediaxis.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Compute the Medial Axis Transform  (K1)
|
|       Purpose: main program for vmediaxis
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->w_logic; {if flag=1 the medial axis has the value 255, otherwise (default) the value is the distance from the border}
|		int clui_info->w_flag; {TRUE if -w specified}
|
|        Output:
|       Returns:
|
|    Written By: Pascal ADAM
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
	char    *program = NULL;
	xvimage *image;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vmediaxis_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vmediaxis",
		vmediaxis_usage_additions, vmediaxis_get_args,
		vmediaxis_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	image = readimage(clui_info->i_file);
	if (image == NULL) {
	   kerror("vmediaxis", "main", "Can not read input image \n");
	   kexit(KEXIT_FAILURE);  /* Quit if bad image */
	}
        CHECKINPUT(program, image) ;
/* -main_before_lib_call_end */

/* -main_library_call */
	if (! lvmediaxis(image,clui_info->w_logic))
        {
           kexit(KEXIT_FAILURE);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
	writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vmediaxis_usage_additions
| 
|       Purpose: Prints usage additions in vmediaxis_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vmediaxis
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vmediaxis_usage_additions(void)
{
	kfprintf(kstderr, "\tCompute the Medial Axis Transform  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vmediaxis_free_args
| 
|       Purpose: Frees CLUI struct allocated in vmediaxis_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vmediaxis
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vmediaxis_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
