 /*
  * Khoros: $Id: vmos2band.c,v 1.11 1996/07/05 03:23:28 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vmos2band.c,v 1.11 1996/07/05 03:23:28 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vmos2band
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vmos2band.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Convert Mosaic Image to Multiband Image  (K1)
|
|       Purpose: main program for vmos2band
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {resulting output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->r_int; {number of subimage rows (must be equal for all subimages)}
|		int clui_info->r_flag; {TRUE if -r specified}
|
|		int clui_info->c_int; {number of subimage columns (must be equal for all subimages)}
|		int clui_info->c_flag; {TRUE if -c specified}
|
|		int clui_info->bs_int; {Subimage border size in pixels (inner borders)}
|		int clui_info->bs_flag; {TRUE if -bs specified}
|
|		int clui_info->bm_int; {Mosaic image border size in pixels (outer border)}
|		int clui_info->bm_flag; {TRUE if -bm specified}
|
|		int clui_info->x_int; {Horizontal index of first subimage (will be first band)}
|		int clui_info->x_flag; {TRUE if -x specified}
|
|		int clui_info->y_int; {Vertical index of first subimage (will be first band)}
|		int clui_info->y_flag; {TRUE if -y specified}
|
|		int clui_info->s_int; {Number of subimages to skip (periodic) when putting subimages into band format}
|		int clui_info->s_flag; {TRUE if -s specified}
|
|		int clui_info->n_int; {Number of subimages to be put into band format}
|		int clui_info->n_flag; {TRUE if -n specified}
|
|        Output:
|       Returns:
|
|    Written By: Donna Koechner
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
char *program = NULL;

xvimage  *image, *img1;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vmos2band_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vmos2band",
		vmos2band_usage_additions, vmos2band_get_args,
		vmos2band_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
/* -main_before_lib_call_end */

/* -main_library_call */
READINPUT(image) ;
CHECKINPUT(program, image) ;

 /*
  * Call the library.
  */
 if (!(lvmos2band(image, &img1, clui_info->r_int, clui_info->c_int, clui_info->bs_int, clui_info->bm_int, clui_info->x_int, clui_info->y_int, clui_info->s_int, clui_info->n_flag, clui_info->n_int)) )
  {
    (void) kfprintf(kstderr, "lvmos2band Failed\n");
     kexit(KEXIT_FAILURE);
  }
/* -main_library_call_end */

/* -main_after_lib_call */
(void) writeimage(clui_info->o_file,img1);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vmos2band_usage_additions
| 
|       Purpose: Prints usage additions in vmos2band_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vmos2band
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vmos2band_usage_additions(void)
{
	kfprintf(kstderr, "\tConvert Mosaic Image to Multiband Image  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vmos2band_free_args
| 
|       Purpose: Frees CLUI struct allocated in vmos2band_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vmos2band
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vmos2band_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
