 /*
  * Khoros: $Id: vshape.c,v 1.11 1996/07/05 03:24:20 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vshape.c,v 1.11 1996/07/05 03:24:20 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vshape
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vshape.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Perform Shape Analysis on Image  (K1)
|
|       Purpose: main program for vshape
|
|         Input:
|		Required M.E. group:
|		    char *clui_info->i1_file; {Input Cluster/Label Image}
|		    int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		    char *clui_info->i2_file; {Input Grey Level Image}
|		    int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		int clui_info->ms_logic; {if true, compute standard moments}
|		int clui_info->ms_flag; {TRUE if -ms specified}
|
|		int clui_info->mc_logic; {if true, compute central moments}
|		int clui_info->mc_flag; {TRUE if -mc specified}
|
|		int clui_info->mi_logic; {if true, compute invariant moments}
|		int clui_info->mi_flag; {TRUE if -mi specified}
|
|		int clui_info->mr_logic; {if false, computes axis of bounded box}
|		int clui_info->mr_flag; {TRUE if -mr specified}
|
|		int clui_info->col_logic; {if false, color = white, with black background}
|		int clui_info->col_flag; {TRUE if -col specified}
|
|		char *clui_info->o1_file; {output Axis image}
|		int   clui_info->o1_flag; {TRUE if -o1 specified}
|
|		char *clui_info->o2_file; {output outline image}
|		int   clui_info->o2_flag; {TRUE if -o2 specified}
|
|		char *clui_info->o3_file; {output VIFF file containing statistics}
|		int   clui_info->o3_flag; {TRUE if -o3 specified}
|
|		char *clui_info->f_file; {output ASCII file}
|		int   clui_info->f_flag; {TRUE if -f specified}
|
|        Output:
|       Returns:
|
|    Written By: Pascal ADAM, Shirley Lee
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
char *program = NULL;
xvimage *image,
        *outline,
        *axis,
        *img3;

kfile *printdev;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vshape_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vshape",
		vshape_usage_additions, vshape_get_args,
		vshape_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
   if (clui_info->i2_flag == TRUE) {
       image = readimage(clui_info->i2_file);
       if (image == NULL){
            (void) kfprintf(kstderr,"vshape: Image could not be read\n");
            kexit(KEXIT_FAILURE);
          }
       }
   else {
       if (clui_info->i1_flag == TRUE) {
          image = readimage(clui_info->i1_file);
          if (image == NULL){
               (void) kfprintf(kstderr,"vshape: Image could not be read\n");
               kexit(KEXIT_FAILURE);
             }
          }
       else {
            (void) kfprintf(kstderr,"vshape: Input image required\n");
            kexit(KEXIT_FAILURE);
          }
   }

   CHECKINPUT(program, image) ;
   if ( clui_info->f_flag ) 
   {
      if ((printdev = kfoutput(clui_info->f_file)) == NULL) 
      {
         (void) kfprintf(kstderr, "vshape:  Can't open ascii output file: \n\n");
         kexit(KEXIT_FAILURE);
      }
   }

   if (lvconvert(image,VFF_TYP_4_BYTE,0,1,0.0,1.0,FALSE) == 0 ) 
   {
      (void) kfprintf(kstderr,"vshape: lvconvert failure\n\n");
      kexit(KEXIT_FAILURE);
   }

   /* Check that one output at least has been selected */

   if ((clui_info->o1_flag==FALSE)&&(clui_info->o2_flag==FALSE)&&(clui_info->o3_flag==FALSE)&&(clui_info->f_flag==FALSE))
   {
      (void) kfprintf(kstderr,
                     "vshape:  Need at least one output selected !!\n\n");
      kexit(KEXIT_FAILURE);
   }
/* -main_before_lib_call_end */

/* -main_library_call */
lvshape(clui_info->i1_flag,
        image,
        clui_info->mr_logic,
        clui_info->col_logic,
        clui_info->o1_flag,
        &axis,
        clui_info->o2_flag,
        &outline,
        clui_info->o3_flag,
        &img3,
        clui_info->f_flag,
        clui_info->f_file,
        printdev,
        clui_info->ms_logic,
        clui_info->mc_logic,
        clui_info->mi_logic);
/* -main_library_call_end */

/* -main_after_lib_call */
if ( clui_info->f_flag ) (void) kfclose(printdev);
if ( clui_info->o1_flag ) writeimage(clui_info->o1_file,axis);
if ( clui_info->o2_flag ) writeimage(clui_info->o2_file,outline);
if ( clui_info->o3_flag ) writeimage(clui_info->o3_file,img3);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vshape_usage_additions
| 
|       Purpose: Prints usage additions in vshape_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vshape
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vshape_usage_additions(void)
{
	kfprintf(kstderr, "\tPerform Shape Analysis on Image  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vshape_free_args
| 
|       Purpose: Frees CLUI struct allocated in vshape_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vshape
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vshape_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->o1_file);
		kfree_and_NULL(clui_info->o2_file);
		kfree_and_NULL(clui_info->o3_file);
		kfree_and_NULL(clui_info->f_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
