 /*
  * Khoros: $Id: vspeckle.c,v 1.12 1996/07/05 03:24:48 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vspeckle.c,v 1.12 1996/07/05 03:24:48 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vspeckle
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vspeckle.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Reduce Speckle Noise Using the Crimmins Algorithm  (K1)
|
|       Purpose: main program for vspeckle
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->n_int; {Iterations}
|		int clui_info->n_flag; {TRUE if -n specified}
|
|        Output:
|       Returns:
|
|    Written By: Caryl Peterson and R. J. Fogler
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
char *program = NULL;
xvimage *image;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vspeckle_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vspeckle",
		vspeckle_usage_additions, vspeckle_get_args,
		vspeckle_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
    image = readimage(clui_info->i_file);
    CHECKINPUT (program, image);
/* -main_before_lib_call_end */

/* -main_library_call */
if(! lvspeckle(image,clui_info->n_int))
    {
        (void) kerror(NULL, NULL, "lvspeckle Failed\n");
        kexit(1);
    }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vspeckle_usage_additions
| 
|       Purpose: Prints usage additions in vspeckle_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vspeckle
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vspeckle_usage_additions(void)
{
	kfprintf(kstderr, "\tReduce Speckle Noise Using the Crimmins Algorithm  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vspeckle_free_args
| 
|       Purpose: Frees CLUI struct allocated in vspeckle_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vspeckle
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vspeckle_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
