 /*
  * Khoros: $Id: vtilt.c,v 1.11 1996/07/05 03:25:05 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vtilt.c,v 1.11 1996/07/05 03:25:05 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vtilt
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vtilt.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Add a Specified Illumination Gradient Plane to Image  (K1)
|
|       Purpose: main program for vtilt
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {resulting output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->m_file; {tilt operation masking image}
|		int   clui_info->m_flag; {TRUE if -m specified}
|
|		float clui_info->x_float; {specifies the slope in the X direction}
|		int   clui_info->x_flag; {TRUE if -x specified}
|
|		float clui_info->y_float; {specifies the slope in the Y direction}
|		int   clui_info->y_flag; {TRUE if -y specified}
|
|        Output:
|       Returns:
|
|    Written By: Scott Wilson, Donna Koechner
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
	char    *program = NULL;
	xvimage *img1, *img2;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vtilt_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vtilt",
		vtilt_usage_additions, vtilt_get_args,
		vtilt_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	img1 = readimage(clui_info->i_file);
	if (img1 == NULL){
	  kerror("vtilt", "main", "Unable to open %s", clui_info->i_file);
	  kexit(KEXIT_FAILURE);		        /* Quit if bad image */
	}
	if (clui_info->m_flag) img2 = readimage(clui_info->m_file);
	if (clui_info->m_flag && (img2 == NULL)){
	  kerror("vtilt", "main", "Unable to open mask %s", clui_info->m_file);
	  kexit(KEXIT_FAILURE);                 /* Quit if bad image */
	}

	CHECKINPUT(program,img1);               /* Verify input image */

	if(clui_info->m_flag){
	  MATCHINPUT(program,img1,img2);        /* Verify input images */
	}
/* -main_before_lib_call_end */

/* -main_library_call */
	if(! lvtilt(img1,img2,clui_info->m_flag,
		    clui_info->x_float,clui_info->y_float) )
	{
  	  kexit(KEXIT_FAILURE);
	}
/* -main_library_call_end */

/* -main_after_lib_call */
	writeimage(clui_info->o_file,img1);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vtilt_usage_additions
| 
|       Purpose: Prints usage additions in vtilt_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vtilt
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vtilt_usage_additions(void)
{
	kfprintf(kstderr, "\tAdd a Specified Illumination Gradient Plane to Image  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vtilt_free_args
| 
|       Purpose: Frees CLUI struct allocated in vtilt_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vtilt
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vtilt_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->m_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
