/*
  * Khoros: $Id: prope_colr.c,v 1.6 1996/04/05 00:53:46 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: prope_colr.c,v 1.6 1996/04/05 00:53:46 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		File Contents Title
   >>>>		
   >>>>   Public:
   >>>>                 proper_color_model()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/************************************************************
*
*  Routine Name: proper_color_model
*
*       Purpose: Check to see if a viff data structure has the correct
*               color space model.
*
*         Input: prog 	- program name
*		*image	- viff data
*		type	- desired color map model
*		exit_flag
*
*        Output: 
*
*       Returns: If exit_flag is true and match fails, an error 
*          	message is printed.
*      		If exit_flag is false and match fails, return(0).
*
*  Restrictions: 
*    Written By:
*          Date: 
*      Verified:
*  Side Effects:
* Modifications:
*************************************************************/
int proper_color_model(char *prog, xvimage *image, long unsigned int type, int exit_flag)
{
   if (image->color_space_model != type) {
      if (exit_flag == TRUE) {
         (void) kfprintf(kstderr, "\nError in program %s: \n",prog);
         (void) kfprintf(kstderr, 
"The color space model should be converted to make it compatible with\n");
         (void) kfprintf(kstderr, 
"this program.\n");
         switch(type){
            case VFF_CM_NONE :
               (void) kfprintf(kstderr, "Color space model should be VFF_CM_NONE\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_CM_genericRGB :
               (void) kfprintf(kstderr, "Color space model should be VFF_CM_genericRGB\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_CM_ntscRGB :
               (void) kfprintf(kstderr, "Color space model should be VFF_CM_ntscRGB\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_CM_ntscCMY :
               (void) kfprintf(kstderr, "Color space model should be VFF_CM_ntscCMY\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_CM_ntscYIQ :
               (void) kfprintf(kstderr, "Color space model should be VFF_CM_ntscYIQ\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_CM_HSV :
               (void) kfprintf(kstderr, "Color space model should be VFF_CM_HSV\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_CM_HLS :
               (void) kfprintf(kstderr, "Color space model should be VFF_CM_HLS\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_CM_IHS :
               (void) kfprintf(kstderr, "Color space model should be VFF_CM_IHS\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_CM_cieRGB :
               (void) kfprintf(kstderr, "Color space model should be VFF_CM_cieRGB\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_CM_cieXYZ :
               (void) kfprintf(kstderr, "Color space model should be VFF_CM_cieXYZ\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_CM_cieUVW :
               (void) kfprintf(kstderr, "Color space model should be VFF_CM_cieUVW\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_CM_cieucsUVW :
               (void) kfprintf(kstderr, "Color space model should be VFF_CM_cieucsUVW\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_CM_cieucsSOW :
               (void) kfprintf(kstderr, "Color space model should be VFF_CM_cieucsSOW\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_CM_cieucsLab :
               (void) kfprintf(kstderr, "Color space model should be VFF_CM_cieucsLab\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_CM_cieucsLuv :
               (void) kfprintf(kstderr, "Color space model should be VFF_CM_cieucsLuv\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_CM_GENERIC :
               (void) kfprintf(kstderr, "Color space model should be VFF_CM_GENERIC\n");
               kexit(KEXIT_FAILURE);
               break;
            default :
               (void) kfprintf(kstderr, "Color space model is unknown.\n");
               kexit(KEXIT_FAILURE);
               break;
	 }
      }
      else {
         return(0);
      }
   }
return(1);
}
