/*
  * Khoros: $Id: proper_loc.c,v 1.6 1996/04/05 00:53:50 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: proper_loc.c,v 1.6 1996/04/05 00:53:50 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		File Contents Title
   >>>>		
   >>>>   Public:
   >>>>                 proper_loc_type()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/************************************************************
*
*  Routine Name: proper_loc_type
*
*       Purpose: Check to see if a viff file or structure has the proper
*               location type, currently either implied or explicit.
*
*         Input: prog 	- program name
*		*image	- viff data
*		type - desired location type.
*		exit_flag
*
*        Output: 
*
*       Returns: If exit_flag is true and match fails, an error 
*          	message is printed.
*      		If exit_flag is false and match fails, return(0).
*
*  Restrictions: 
*    Written By:
*          Date: 
*      Verified:
*  Side Effects:
* Modifications:
*************************************************************/
int proper_loc_type(char *prog, xvimage *image, long unsigned int type, int exit_flag)
{
    if( image->location_type != type ){
      if (exit_flag == TRUE) {
         (void) kfprintf(kstderr, "\nError in program %s: \n",prog);
         switch(type){
            case VFF_LOC_IMPLICIT :
               (void) kfprintf(kstderr, 
"The viff data location type should be VFF_LOC_IMPLICIT\n");
               (void) kfprintf(kstderr, 
"Implicit data means that the actual locations of the data points are not\n");
               (void) kfprintf(kstderr, 
"specified with the data.  This data file probably contains actual locations.\n");
               kexit(KEXIT_FAILURE);
               break;
            case VFF_LOC_EXPLICIT :
               (void) kfprintf(kstderr, 
"The viff data location type should be VFF_LOC_EXPLICIT\n");
               (void) kfprintf(kstderr, 
"Explicit data means that actual positions are given for the data points.\n");
               (void) kfprintf(kstderr, 
"This data may have implied positions.\n");
               kexit(KEXIT_FAILURE);
               break;
            default :
               (void) kfprintf(kstderr, "viff data location type is unknown.\n");
               kexit(KEXIT_FAILURE);
               break;
	 }
      }
      else return(0);
   }
return(1);
}
