 /*
  * Khoros: $Id: lvgamth.c,v 1.12 1996/07/05 03:27:31 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lvgamth.c,v 1.12 1996/07/05 03:27:31 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vgamth
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvgamth
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvgamth - Generate a binary image (value or 0) by thresholding the input image
* 
*       Purpose: Generate a binary image (value or 0) by thresholding the input
*		 color image.  A color image implies the Image is of data type
*		 BYTE, It contains 3 map columns of data type BYTE with Map
*		 enable set to FORCE.  The resulting image is of data type BYTE.
*
*		 The output pixel is given the value specified by the input
*		 variable value, if the input pixel has a value above the lower
*		 threshold, below the upper threshold, or between the two
*		 threshold levels.
*
*		 If the lower threshold flag is set then any pixel
*		 with value larger than the lower threshold is set
*		 to true, otherwise false. If the upper threshold
*		 flag is set then any pixel with value larger
*		 than the upper threshold level is set to false,
*		 otherwise true. If both flags are set then it is
*		 possible to isolate a band of pixel values. If the
*		 lower and upper threshold flags are set and the lower
*		 and upper threshold values are equal, then the output
*		 is set to true if it is exactly equal to the threshold
*		 level. The region specified by the upper and lower
*		 threshold values must be true for colormap #1 ,
*		 colormap #2 and colormap #3 pixel values for the
*		 above to work.
*         Input: image - input pseudo colored image in byte format
*		 rlower - lower threshold value for map column #1 pixel value
*		 rupper - upper threshold value for map column #1 pixel value
*		 lrflag - flag that says map column #1 lower thresh value is
*			  valid
*		 urflag - flag that says map column #1 upper thresh value is
*			  valid
*		 glower - lower threshold value for map column #2 pixel value
*		 gupper - upper threshold value for map column #2 pixel value
*		 lgflag - flag says map column #2 lower thresh value is valid
*		 ugflag - flag says map column #2 upper thresh value is valid
*		 blower - lower threshold value for map column #3 pixel value
*		 bupper - upper threshold value for map column #3 pixel value
*		 lbflag - flag says map map column #3 lower thresh value is
*			  valid
*		 ubflag - flag says map column #3 upper thresh value is valid
*		 value  - Value for the non-zero pixel.
*        Output: image - thresheld binary output image.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: lvgamth will only operate on 1 band images of data type BYTE,
*		 with the map enable FORCE, and map data type BYTE.
*    Written By: Tom Sauer
*          Date: Jul 04, 1996
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
/* ARGS_USED */
int lvgamth(
	xvimage *image,
	int rupper,
	int rlower,
	int gupper,
	int glower,
	int bupper,
	int blower,
        int lrflag,
	int urflag,
	int lgflag,
	int ugflag,
	int lbflag,
	int ubflag,
	int vflag,
	int value)
/* -library_def_end */

/* -library_code */
{

    unsigned char *c, cvalue, *rptr, *bptr, *gptr; 
    int j;
    char *program = "lvgamth";

        /* by default the lower threshhold value is used, if nothing
                was specified */
     if (!lrflag && !urflag) lrflag = 1;
     if (!lbflag && !ubflag) lbflag = 1;
     if (!lgflag && !ugflag) lgflag = 1;

      /* can only have one image */
     if (! (proper_num_images(program,image,(unsigned long) 1,FALSE))) {
          kerror("vipl", "lvgamth",
		 "ERROR All inputs must contain only one image\n");
          return(FALSE);
      }

      /* can only have one data band */
     if (! (proper_num_bands(program,image,(unsigned long) 1,FALSE))) {
          kerror("vipl", "lvgamth",
		 "ERROR All inputs must contain only one data band\n");
          return(FALSE);
      }

      /* can only have data type of BYTE */
     if (! (propertype(program,image,(unsigned long) VFF_TYP_1_BYTE,FALSE))) {
          kerror("vipl", "lvgamth",
		 "ERROR input must have data type of BYTE\n");
          return(FALSE);
      }

      /* must have map enable set to FORCE */
     if (! (proper_map_enable(program,image,(unsigned long) VFF_MAP_FORCE,FALSE))) {
          kerror("vipl", "lvgamth",
		 "ERROR input must have map enable type FORCE\n");
           return(FALSE);
      }

        /* must have 3 columns in the map */
     if (! (proper_map_rowsize(program,image,(unsigned long) 3,FALSE))) {
          kerror("vipl", "lvgamth",
		 "ERROR input must have map row size of 3\n");
          return(FALSE);
      }

        /* map data type must be BYTE */
     if (! (proper_map_type(program,image,(unsigned long) VFF_MAPTYP_1_BYTE,FALSE))) {
          kerror("vipl", "lvgamth",
		 "ERROR input must have map data type of BYTE\n");
           return(FALSE);
      }

       /* point to each column in the map */

     rptr = (unsigned char *)&image->maps[0];
     gptr = (unsigned char *)&image->maps[image->map_col_size];
     bptr = (unsigned char *)&image->maps[2 * image->map_col_size];
    
     c = (unsigned char *)image->imagedata;

       /* loop through the image and set each pixel to either zero
          or to value depending on the threshhold specifications */

     for (j = 0;j < (image->row_size * image->col_size);j++)
     {
       cvalue = *c;
       *c = 0;

           /* all lower flags must be set */

       if ( lrflag && (*(rptr + cvalue) > (unsigned) rlower) 
            && lbflag && (*(bptr + cvalue) > (unsigned)blower) 
            && lgflag && (*(gptr + cvalue) > (unsigned) glower)) *c = value;

          /* all upper flags must be set */

       if ( urflag && (*(rptr + cvalue) > (unsigned) rupper) 
            && ubflag && (*(bptr + cvalue) > (unsigned) bupper) 
            && ugflag && (*(gptr + cvalue) > (unsigned) gupper)) *c = 0;

            /* map colunm 1 has upper and lower equal, with 
               the other 2 lower flags set */

       if ( lrflag && urflag && rlower == rupper 
            && (*(rptr + cvalue) == rlower) 
            && lbflag && (*(bptr + cvalue) > (unsigned) blower) 
            && lgflag && (*(gptr + cvalue) > (unsigned) glower))  *c = value;

            /* map colunm 1 has upper and lower equal, with 
               the other 2 upper flags set */

       if ( lrflag && urflag && rlower == rupper 
            && (*(rptr + cvalue) == rlower) 
            && ubflag && (*(bptr + cvalue) > (unsigned) bupper) 
            && ugflag && (*(gptr + cvalue) > (unsigned) gupper))  *c = 0;

            /* map colunm 2 has upper and lower equal, with 
               the other 2 lower flags set */

       if ( lgflag && ugflag && glower == gupper 
            && (*(gptr + cvalue) == glower) 
            && lbflag && (*(bptr + cvalue) > (unsigned) blower) 
            && lrflag && (*(rptr + cvalue) > (unsigned) rlower))  *c = value;

            /* map colunm 2 has upper and lower equal, with 
               the other 2 upper flags set */

       if ( lgflag && ugflag && glower == gupper 
            && (*(gptr + cvalue) == glower) 
            && ubflag && (*(bptr + cvalue) > (unsigned) bupper) 
            && urflag && (*(rptr + cvalue) > (unsigned) rupper))  *c = 0;

            /* map colunm 3 has upper and lower equal, with 
               the other 2 lower flags set */

       if ( lbflag && ubflag && blower == bupper 
            && (*(bptr + cvalue) == blower) 
            && lgflag && (*(gptr + cvalue) > (unsigned) glower) 
            && lrflag && (*(rptr + cvalue) > (unsigned) rlower))  *c = value;

            /* map colunm 3 has upper and lower equal, with 
               the other 2 upper flags set */

       if ( lbflag && ubflag && blower == bupper 
            && (*(bptr + cvalue) == blower) 
            && ugflag && (*(gptr + cvalue) > (unsigned) gupper) 
            && urflag && (*(rptr + cvalue) > (unsigned) rupper))  *c = 0;

            /* all 3 map column upper and lower flags set */

       if ( lrflag && urflag && rlower == rupper 
            && (*(rptr + cvalue) == rlower) 
            && lbflag && ubflag && blower == bupper 
            && (*(bptr + cvalue) == blower) 
            && lgflag && ugflag && glower == gupper 
            && (*(gptr + cvalue) == glower))  *c = value;
       c++;
     }

        /* maps are no longer needed since the image is now a 
           binary image */
     kfree_and_NULL(image->maps);
     image->maps = NULL;
     image->map_row_size = 0;
     image->map_col_size = 0;
     image->map_enable = VFF_MAP_OPTIONAL;
     image->map_scheme = VFF_MS_NONE;
     image->color_space_model = VFF_CM_NONE;

     return(TRUE);
}
/* -library_code_end */
