 /*
  * Khoros: $Id: lvgbox.c,v 1.14 1996/07/05 03:27:33 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lvgbox.c,v 1.14 1996/07/05 03:27:33 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vgbox
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvgbox
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvgbox - Fill a buffer with a grey level representation of a parallelogram projected on a CCD sensor.
* 
*       Purpose: This procedure fills a buffer with a grey level represention
*		 of a parallelogram projected on a CCD sensor.  It calculates
*		 the grey level of each pixel from the intersection of the
*		 theorical parallelogram and the rectangular active area of
*		 each sensor-pixel.  The sensor is made of pixels suposed to
*		 be all identical, composed of an active rectangular area in a
*		 rectangular cell.
*         Input: n_cols - number of cols (x)
*		 n_rows - number of rows (y)
*		 px_x_w - active area pixel width in the pixel_cell
*		 px_y_h - active area pixel height in the pixel_cell
*		 px_x_of - active pixel offset from left pixel_cell
*		 px_y_of - active pixel offset from top pixel_cell
*		 grey_out - char   grey level outside of parallelogram
*		 grey_in - char   grey level inside of parallelogram
*		 im_x_w - parallelogram euclidian width (in pixel unit)
*		 im_y_h - parallelogram euclidian height (in pixel unit)
*		 im_x_of - parallelogram: x right offset (origin: parallelogram
*			   center = CCD center)
*		 im_y_of - parallelogram: y downward offset
*			   (origin: parallelogram center = CCD center)
*		 phi - clockwise parallelogram rotation (radians, rot.
*			   center = parallelogram center)
*		 psi - first parallelogram corner angle (radians)
*        Output: pt_image_data - a pointer to the memory allocated by the
*				 calling routine.  holds the result of the
*				 image data generation
*       Returns: 0 on success, n=1 on warning, n=2 on error.
*  Restrictions: Pixel width + pixel offset must be egal or less than 1.000.
*
*		 The CCD cells are supposed square in order to obtain the asked
*		 angles, although the active area can be rectangular.
*
*		 For a parallelepipedic box, the width and heigth sizes of the
*		 box are the euclidian distances between the parallel lines.
*    Written By: Jean-Pierre GUERIN  (CEA)
*          Date: Jul 04, 1996
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: August 1991 : new float psi angle parameter for parallelogram
****************************************************************/
/* -library_def */
int lvgbox(
	unsigned char *pt_image_data,
	unsigned short n_cols,
	unsigned short n_rows,
	float px_x_w,
	float px_y_h,
	float px_x_of,
	float px_y_of,
	unsigned char grey_out,
	unsigned char grey_in,
	float im_x_w,
	float im_y_h,
	float im_x_of,
	float im_y_of,
	float phi,
	float psi)
/* -library_def_end */

/* -library_code */
{
struct str_droite
{
   /*teta is the line's director vector (orthogonal):
   *       x*cos(teta) + y*sin(teta) - ro = 0
   */
   float   ro, teta;
   float   delta_y;
   float   yphh, yph, yphb, ypbh, ypb, ypbb; /* y caracteristical points */
   float   y_intersection, area, tan_teta, cst2, grey_val;
   char    deep_slope, flg_vertical;
};

struct str_droite st_d[4];

/*  usage:
 *       st_d[i].teta      =  teta_init  ;    i th line's teta
 */


   unsigned short no_row,no_col;
   float          cumul_area, fl_tmp, ro;
   int            i, return_flag;
   float          xp;   /* vertical axe of pixel */

   return_flag = 0;

   if ( ((px_x_of + px_x_w) > 1.) || ((px_y_of + px_y_h) > 1.) )
   {
      return_flag=2;
      return(return_flag);       /* error */
   }

   phi = - phi;   /* clockwise ==> trigonometrical direction */
   if ( phi<0 )   /* 0 <= phi < KPI */
   {
      fl_tmp = kfmod(phi, KPI);    /* negative modulo */
      phi   = KPI + fl_tmp;
   }

   else phi = kfmod(phi, KPI);      /* modulo KPI */

   /* 4 lines of the parallelogram are to be proceeded.
    *  rotation is around the parallelogram center, parallelogram offsets are in the sensor plane.
    * st_d[i].teta is the line director vector (orthogonal to the line):
    * x*cos(teta) + y*sin(teta) - ro = 0
    */
   for ( i=0  ; i<4 ; i++)
   {
      switch(i)  /* process for parallogram */
        {
          case 0:
            st_d[i].teta = kfmod(phi,            2*KPI);
            break;
          case 1:
            st_d[i].teta = kfmod(phi + psi,      2*KPI);
            break;
          case 2:
            st_d[i].teta = kfmod(phi + KPI,       2*KPI);
            break;
          case 3:
            st_d[i].teta = kfmod(phi + psi + KPI, 2*KPI);
            break;
         }

      if ((i % 2) == 0) ro = im_x_w/2;

      else ro = im_y_h/2;

      /* axe change: */
      st_d[i].ro = ro +(  (float)(n_cols)/2. + im_x_of)*cos(st_d[i].teta) +
                       (  (float)(n_rows)/2. - im_y_of)*sin(st_d[i].teta);

      st_d[i].delta_y = px_x_w*kfabs(tan(st_d[i].teta+KPI/2))/2; /* abs. value */

      st_d[i].tan_teta = kfabs(tan(st_d[i].teta)); /* absolute value */

/* Modified test instruction: */
      if ( ( (st_d[i].teta >=   KPI-0.00001) && (st_d[i].teta <=   KPI+0.00001) ) ||
           ( (st_d[i].teta >=     -0.00001) && (st_d[i].teta <=      0.00001) ) ||
           ( (st_d[i].teta >= 2*KPI-0.00001) && (st_d[i].teta <= 2*KPI+0.00001) ) )
      {
         st_d[i].flg_vertical =  TRUE;
/* Modified test instruction: */
         if ( (st_d[i].teta >= KPI-0.00001) &&
              (st_d[i].teta <= KPI+0.00001) )
         {
            st_d[i].ro    = -st_d[i].ro;
            st_d[i].teta += KPI;  /* not used */
         }
      }
      else st_d[i].flg_vertical =  FALSE;

      st_d[i].cst2 = st_d[i].tan_teta/2; /* absolute value */
   }

   /* col after col */
   for ( no_col=0 ; no_col<n_cols ; no_col++ )
   {
      /* xp = x center pixel position: */
      xp = ((float) no_col) + px_x_of + px_x_w/2.0;

      /* y caracteristical points at xp center pixel position: */
      for ( i=0; i<4 ;i++)
      {
         if (st_d[i].flg_vertical)
         {
            if (st_d[i].ro < (xp - px_x_w/2.) )
		st_d[i].grey_val=1.;
            else if (st_d[i].ro > (xp + px_x_w/2.))
		st_d[i].grey_val=0.;
            else
		st_d[i].grey_val= 1. -
			      (st_d[i].ro - ((float)no_col) - px_x_of)/(px_x_w);

         }
         else   /* not vertical */
         {
            st_d[i].y_intersection = (st_d[i].ro -
                                     xp*cos(st_d[i].teta))/sin(st_d[i].teta);
            /* top pixel: */
            st_d[i].yph  = (float)n_rows - px_y_of;
            st_d[i].ypb  = st_d[i].yph - px_y_h;
            st_d[i].yphh = st_d[i].yph + st_d[i].delta_y;
            st_d[i].ypbb = st_d[i].ypb - st_d[i].delta_y;

            if (st_d[i].tan_teta >= (px_x_w/px_y_h))   /* slow slope? */
            {
               st_d[i].deep_slope = FALSE;
               st_d[i].yphb = st_d[i].ypbb + px_y_h;
               st_d[i].ypbh = st_d[i].yphh - px_y_h;
            }
            else                                       /* deep slope */
            {
               st_d[i].deep_slope = TRUE;
               st_d[i].yphb = st_d[i].yphh - px_y_h;
               st_d[i].ypbh = st_d[i].ypbb + px_y_h;
            }
         }
      }   /* end i */

      for ( no_row=0 ; no_row<n_rows ; no_row++ )
      {
         for (cumul_area=0, i=0; i<4 ;i++)
         {
            if (st_d[i].flg_vertical)
               st_d[i].area = st_d[i].grey_val;

            else if (st_d[i].y_intersection >= st_d[i].yphh)
            {  /* line above the pixel: empty pixel */
               st_d[i].area = 0.;
            }
            else if  (st_d[i].y_intersection <= st_d[i].ypbb)
            {  /* line under the pixel: full pixel */
               st_d[i].area = 1. ;
            }
            else if (st_d[i].y_intersection >= st_d[i].yphb)
            {  /* intersection is a triangle: */
               st_d[i].area = st_d[i].cst2*
                              (st_d[i].yphh - st_d[i].y_intersection)*
                              (st_d[i].yphh - st_d[i].y_intersection);
            }
            else if (st_d[i].y_intersection >= st_d[i].ypbh)
            {  /* intersection is a trapeze: */
               if (st_d[i].deep_slope==FALSE)
                  st_d[i].area = (st_d[i].yph-st_d[i].y_intersection)/px_y_h;

               else   /* deep slope */
                  st_d[i].area = (st_d[i].yphb - st_d[i].y_intersection +
                                 px_y_h/2) *st_d[i].tan_teta/px_x_w;
            }
            else /* if (st_d[i].y_intersection >= st_d[i].ypbb) */
            {  /* intersection is (full - triangle): */
               st_d[i].area =  1. - st_d[i].cst2*
                               (st_d[i].y_intersection - st_d[i].ypbb)*
                               (st_d[i].y_intersection - st_d[i].ypbb);
            }
         }   /* end i */
         cumul_area = (- st_d[0].area + st_d[2].area) *
                      (st_d[1].area - st_d[3].area);

         if (cumul_area < 0.0)
         {
            cumul_area = -cumul_area;
         }

/* Modified test instruction: */
         if (cumul_area > 1.000)
         {
            if (cumul_area > 1.0001)
                return_flag = 1;
            cumul_area = 1.;
         }

/* Modified cast: */
         pt_image_data[no_row*n_cols + no_col] =
                              (unsigned char) ((float)grey_out+
                              0.5 + cumul_area*
                              ((float)grey_in-(float)grey_out));

         /* next row initialisation: */
         for ( i=0; i<4 ;i++)
         {
            st_d[i].yph-- ;
            st_d[i].yphh-- ;
            st_d[i].yphb-- ;
            st_d[i].ypb-- ;
            st_d[i].ypbh-- ;
            st_d[i].ypbb-- ;
         }   /* end i */
      }   /* end no_row */
   }   /* end no_col */
   return(return_flag);
}   /* end synth_image() */
/* -library_code_end */
