 /*
  * Khoros: $Id: lvmarr.c,v 1.12 1996/07/05 03:27:46 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lvmarr.c,v 1.12 1996/07/05 03:27:46 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vmarr
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvmarr
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvmarr - create the Marr Laplacian of Gaussian convolution kernel
* 
*       Purpose: This routine creates a convolution filter kernel using the
*		 following equation. 
*		 ! pixel = [-1/(PI*var*var)]*[1-r*r/(2*var)]*exp[-r*r/(2*var)],
*		 ! where r is the calculated distance from the center of the
*		   kernel.
*		 A Marr filter is the Laplacian of a Gaussian, and is intended
*		 for high lighting image edges and at the same time smooth out
*		 the noise.  vmarr creates image data of type float.
*         Input: image - pointer to VIFF structure containing image data to be
*			 processed.
*		 rows - number of rows in kernel.
*		 cols - number of columns in kernel.
*		 var - the variance of the Gaussian
*        Output: image - pointer to VIFF structure containing image data after
*		         processing.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: only generates float kernels
*    Written By: Richard Krukar, John Rasure, Robert Estes
*          Date: Jul 04, 1996
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: John Rasure, Marcelo Teran, Ramiro Jordan, Tom Sauer 7/12/89 -
*		 changed the Laplacian of a Gaussian operator.  The new
*		 operator was taken from IEEE Transactions on Pattern Analysis
*		 and Machine Intelligence, Vol 11 No. 4, April 1989.
*
*		 Jeremy Worley 7/02/90 - minor modifications to code so that it
*		 conforms to Khoros conventions.  Added error check on kernel
*		 dimensions.  No functional changes.
*
*		 Robert Estes 4/25/90 - changed the code to implement a 
*                true Log
****************************************************************/
/* -library_def */
int lvmarr (
	xvimage **image,
	int rows,
	int cols,
	float var)
/* -library_def_end */

/* -library_code */
{
        int i,j,x_cent,y_cent;
        double r2;
        float  *fptr;

        xvimage *kern;

/*
** a little error checking here, please...
*/

        if(rows<=0 || cols<=0){
           kerror("vipl","lvmarr", "Kernel dimension must be greater than 0\n");
           return(FALSE);
        }

/*
** thanks..
*/

        kern = createimage((unsigned long) rows, /* number of rows */
                        (unsigned long) cols,    /* number of columns */
                        (unsigned long)
                        VFF_TYP_FLOAT,           /* data_storage_type */
                        (unsigned long) 1,       /* num_of_images */
                        (unsigned long) 1,       /* num_data_bands */
                        "created by vmarr",      /* comment */
                        (unsigned long) 0,       /* map_row_size */
                        (unsigned long) 0,       /* map_col_size */
                        (unsigned long)
                        VFF_MS_NONE,             /* map_scheme */
                        (unsigned long)
                        VFF_MAPTYP_NONE,         /* map_storage_type */
                        (unsigned long)
                        VFF_LOC_IMPLICIT,        /* location_type */
                        (unsigned long) 0);      /* location_dim */

        if (kern == NULL)
        {
           kerror("vipl","lvmarr", "Unable to allocate new image!\n");
           *image = NULL;
           return(FALSE);
        }

        if (var == 0.0)
        {
            kerror("vipl","lvmarr", "variance can not equal zero!\n");
            return(FALSE);
        }

/* get the center point of the mask  */
        x_cent = cols/2;
        y_cent = rows/2;

        fptr = (float *) kern->imagedata;

/* crank out the kernel */
        for (i = 0; i < rows; i++)
        {
           for (j = 0; j < cols; j++)
           {
                 r2 = (double)((i-y_cent)*(i-y_cent)+(j-x_cent)*(j-x_cent));
                 *fptr++ =
                 (float)((0.5*r2/var-1.0)*exp(-r2/(2.0*var))/(KPI*var*var));

            }
        }

        *image = kern;
        return(TRUE);
}
/* -library_code_end */
