 /*
  * Khoros: $Id: lvmindis.c,v 1.12 1996/07/05 03:27:48 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lvmindis.c,v 1.12 1996/07/05 03:27:48 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vmindis
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvmindis
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/******************************************************************

        Compute Distance for k-NN routine

        source filename:                        dist.c

        revision          date                  programmer
        --------          ----                  ----------
           0.0          05/19/88                Don Hush

******************************************************************

        usage :

        float
        dist(x, y, n)

        float *x;
        float *y;
        int n;

        description:

        This routine computes the euclidean distance between
        two n-dimension floating point vectors.

******************************************************************/
float
dist (
   float  *x,
   float  *y,
   int     n)
{
    int     i;
    float   d,
            a;

    d = 0.0;
    for (i = 0; i < n; i++) {
        a = x[i] - y[i];
        d += a * a;
    }
    d = (float) sqrt ((double) d);

    return (d);
}
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvmindis - Minimum Distance Classifier
* 
*       Purpose: vmindis is a simple minimum distance classifier.
*		 The distance metric used to the Euclidean distance.
*		 This routine accpets two images as the input. The
*		 image that corresponds to the variable image is the
*		 image that needs to be classified. This image can
*		 be have as many data bands as necessary. The number
*		 of data bands depicts the dimensionality of the
*		 vectors. Each pixel in the image is a vector with
*		 dimensionality of the number of data bands.
*
*		 The other input image which corresponds to the
*		 variable center is the prototype image. The last
*		 data band of this image is the class data band.
*		 This image must contain the same number of data
*		 bands as the other input image plus an extra data
*		 band that represents the class mapping.  This
*		 image would most likely have been created by
*		 vkmeans or some other routine that will give
*		 cluster centers. This image contains vectors that
*		 correspond to the prototype of each class.
*
*		 As stated above the center image's last data band
*		 is a class data band.  The class data band simply
*		 maps each vector in the center image to the final
*		 class.
*
*		 The border variable allows the user to specify a
*		 border width, in pixels, encompassing the image.
*		 The border region is skipped by vmindis when
*		 classification is performed. This usefull if
*		 neighborhood operators have been used previously
*		 and have not updated edge pixels.
*
*		 All input images must be of data storage type FLOAT. 
*
*         Input: img    - the input image that is to be classified.
*		 center - the image containing the center values (or
*		          prototype vectors) with the last data band
*		          being the class image.
*		 border - the border width on the image. The border
*		          corresponds to a no class assignment.
*        Output: img    - a single band classified image.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: All input images must be of data storage type FLOAT.
*    Written By: Tom Sauer
*          Date: Jul 04, 1996
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int lvmindis (
	xvimage *img,
	xvimage *center,
	int	border)
/* -library_def_end */

/* -library_code */
{
    int     nc,                 /* column size of image */
            nr,                 /* row size of image */
            cc_nc,              /* column size of image */
            cc_nr,              /* row size of image */
            vect,               /* number of vectors in image */
            dimension,          /* size of vector - num_bands */
            n_centers,
            i,                  /* loop indices */
            j, m, k,
            x,
            c,
            z;

float       d, dmin;

    float  *ptr,                /* pointer to the image data char */
          **input_vectors,      /* pointer to image data reorganized */
           *new_class,          /* pointer to result image */
           *class,              /* pointer to class data that corresponds */
                                /* to the cluster centers */
          **c_center;           /* cluster centers */

    nr = img -> col_size;       /* number of rows in image */
    nc = img -> row_size;       /* number of columns in image */
    cc_nr = center->col_size;    /* number of rows in center image */
    cc_nc = center->row_size;    /* number of columns in center image */

    dimension = img -> num_data_bands;
    vect = (nr - (border * 2)) * (nc - (border * 2)); /* number of vectors */
    n_centers = cc_nr * cc_nc;

    /* Allocate space for input_vectors image */
    /* reorganize the data because it is not organized in an efficient */
    /* manner for the computational complexity of the routine */

    input_vectors = (float **) kmalloc ((unsigned int) vect * sizeof (float));
    if (input_vectors == NULL) {
        kerror ("vipl", "lvmindis", "Insufficient memory available\n");
        return (FALSE);
    }

    for (i = 0; i < vect; i++) {
        input_vectors[i] = (float *) kmalloc (dimension * sizeof (float));
        if (input_vectors[i] == NULL) {
            kerror ("vipl", "lvmindis", "Insufficient memory available\n");
            return (FALSE);
        }
    }

/* Allocate space for cluster centers */

    c_center = (float **) kmalloc (sizeof (float *) * cc_nr * cc_nc);
    if (c_center == NULL) {
        kerror ("vipl", "lvmindis", "Insufficient memory available\n");
        return (FALSE);
    }

    for (i = 0; i < n_centers; i++) {
        c_center[i] = (float *) kmalloc (sizeof (float) * dimension);
        if (c_center[i] == NULL) {
            kerror ("vipl", "lvmindis", "Insufficient memory available\n");
            return (FALSE);
        }
    }

    /* Allocate space for output image data */

    new_class = (float *) kmalloc ((unsigned int) nr * nc* sizeof (float));
    if (new_class == NULL) {
        kerror ("vipl", "lvmindis", "Insufficient memory available\n");
        return (FALSE);
    }

    /* Zero out output imagedata */

    for (i = 0 ; i< nr; i++) {
        for (j = 0 ; j< nc; j++) {
            new_class[i * nc + j] = 0.0;
        }
    }

    /* Assign image data address to ptr */

    ptr = (float *) img -> imagedata;

    /* Assign ptr to input vectors minus the border */
    m = 0;
    for (i = border; i < nr - border; i++) {
        for (j = border; j < nc - border; j++) {
            for (k = 0; k < dimension; k++) {
                input_vectors[m][k] = ptr[(i * nc + j) + (k * nc * nr)];
            }
            m++;
        }
    }

 /* Assign cluster center image data to c_center */

   ptr = (float *) center -> imagedata;
       for (i = 0; i < cc_nr * cc_nc; i++) {
          for (j = 0; j < dimension; j++) {
             c_center[i][j] = ptr[i + (cc_nc * cc_nr * j)];
          }
       }


 /* Assign class to the last band of the center image */

    class = (float *) (&ptr[cc_nr * cc_nc * dimension]);

/*  run the classifier */

    i = 0;
    for (x = border; x < nr - border; x++) {
       for (z = border; z < nc - border; z++) {
     
           dmin = KMAXFLOAT;

           for (j = 0; j < n_centers; j++) {
               d = dist(input_vectors[i], c_center[j], dimension);
               if (d < dmin) {
                  dmin = d;
                  c = j;
               }
           }
              /* assign the classified vector */
           new_class[x * nc + z] = class[c];
           i++;
       }
    }

    /* release memory from input image */

    (void) kfree_and_NULL (img -> imagedata);
    (void) kfree_and_NULL (img -> maps);

       /* adjust header in the output image */
    img -> imagedata = (char *) new_class;
    img -> num_data_bands = 1;
    img -> map_scheme = VFF_MS_NONE;
    img -> map_row_size = 0;
    img -> map_col_size = 0;
    img -> map_subrow_size = 0;
    img -> color_space_model = VFF_CM_NONE;

    return (TRUE);
}
/* -library_code_end */
