 /*
  * Khoros: $Id: lvqerr.c,v 1.12 1996/07/05 03:27:52 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lvqerr.c,v 1.12 1996/07/05 03:27:52 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vqerr
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvqerr
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvqerr - Compute the quantization error between a quantized image and the original.
* 
*       Purpose: lvqerr computes the RMS quantization error between
*		 a quantized image and the original image.
*
*		 The first input image is the quantized image and
*		 should have maps attached.  The data type of the
*		 first image should be BYTE or INT.  The map should
*		 be of the same data type as the data in the second
*		 image (the original). The original image should
*		 have no maps attached.
*
*		 The mask image must be the same data storage type
*		 and size as the first input image.
*
*		 The number of columns in the map of the first input
*		 image should match the number of bands in the second
*		 (original) input image.
*
*		 The output is in text format, with the RMS quantization
*		 error printed in using the %e format.
*         Input: img1 - quantized VIFF image structure
*		 img2 - original VIFF image structure
*		 img3 - gating mask image
*		 mflg - a flag set (equal 1) if gating mask available
*		 rms  - pointer to a DOUBLE to receive the RMS error
*        Output: rms  - is modified to contain the RMS quantization error
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: lvqerr will not operate on VFF_TYP_BIT, VFF_TYP_COMPLEX,
*		 or VFF_TYP_DOUBLE data storage types.
*    Written By: 
*          Date: Jul 04, 1996
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int lvqerr(
	xvimage *img1,
	xvimage *img2,
	xvimage *img3,
	int mflg,
	double *rms)
/* -library_def_end */

/* -library_code */
{
    int		  i, j, k, count;
    unsigned char *c1,*c2,*c3,*mpc;
    short         *s2,*s3,*mps;
    int           *i1,*i2,*i3,*mpi;
    float         *f2,*f3,*mpf;
    double	  err,e;
    unsigned int  mapind;

    err = 0;
    count = 0;

    switch (img2->data_storage_type)
      {
        case VFF_TYP_BIT :
          kerror("vipl", "lvqerr", "Don't know how to handle BIT images\n");
          return(FALSE);
          /* break; */

        case VFF_TYP_1_BYTE : 
          k = img1->row_size*img1->col_size;
          mpc = (unsigned char *)(img1->maps);
          if (!mflg)
            {
              c2 = (unsigned char *)(img2->imagedata);
              for (i=0; i<img2->num_data_bands; i++)
                {
                  if (img1->data_storage_type == VFF_TYP_1_BYTE)
                    c1 = (unsigned char *)(img1->imagedata);
                  else
                    i1 = (int *)(img1->imagedata);
                  for (j=0; j<k; j++)
                    {
                      if (img1->data_storage_type == VFF_TYP_1_BYTE)
                        mapind = i*img1->map_col_size + *c1++;
                      else
                        mapind = i*img1->map_col_size + *i1++;
                      e = (double)(mpc[mapind]) - (double)(*c2++);
                      err += e*e;
                      count++;
                    }
                }
            }
          else
            {
              c2 = (unsigned char *)(img2->imagedata);
              for (i=0; i<img2->num_data_bands; i++)
                {
                  if (img1->data_storage_type == VFF_TYP_1_BYTE)
                    c1 = (unsigned char *)(img1->imagedata);
                  else
                    i1 = (int *)(img1->imagedata);
                  c3 = (unsigned char *)(img3->imagedata);
                  for (j=0; j<k; j++)
                    {
                      if (*c3++)
                        {
                          if (img1->data_storage_type == VFF_TYP_1_BYTE)
                            mapind = i*img1->map_col_size + *c1++;
                          else
                            mapind = i*img1->map_col_size + *i1++;
                          e = (double)(mpc[mapind]) - (double)(*c2++);
                          err += e*e;
                          count++;
                        }
                      else
                        {
                          if (img1->data_storage_type == VFF_TYP_1_BYTE) c1++;
                          else i1++;
                          c2++;
                        }
                    }
                }
            }
          break;
 
        case VFF_TYP_2_BYTE :
          k = img1->row_size*img1->col_size;
          mps = (short *)(img1->maps);
          if (!mflg)
            {
              s2 = (short *)(img2->imagedata);
              for (i=0; i<img2->num_data_bands; i++)
                {
                  if (img1->data_storage_type == VFF_TYP_1_BYTE)
                    c1 = (unsigned char *)(img1->imagedata);
                  else
                    i1 = (int *)(img1->imagedata);
                  for (j=0; j<k; j++)
                    {
                      if (img1->data_storage_type == VFF_TYP_1_BYTE)
                        mapind = i*img1->map_col_size + *c1++;
                      else
                        mapind = i*img1->map_col_size + *i1++;
                      e = (double)(mps[mapind]) - (double)(*s2++);
                      err += e*e;
                      count++;
                    }
                }
            }
          else
            {
              s2 = (short *)(img2->imagedata);
              for (i=0; i<img2->num_data_bands; i++)
                {
                  if (img1->data_storage_type == VFF_TYP_1_BYTE)
                    c1 = (unsigned char *)(img1->imagedata);
                  else
                    i1 = (int *)(img1->imagedata);
                  s3 = (short *)(img3->imagedata);
                  for (j=0; j<k; j++)
                    {
                      if (*s3++)
                        {
                          if (img1->data_storage_type == VFF_TYP_1_BYTE)
                            mapind = i*img1->map_col_size + *c1++;
                          else
                            mapind = i*img1->map_col_size + *i1++;
                          e = (double)(mps[mapind]) - (double)(*s2++);
                          err += e*e;
                          count++;
                        }
                      else
                        {
                          if (img1->data_storage_type == VFF_TYP_1_BYTE) c1++;
                          else i1++;
                          s2++;
                        }
                    }
                }
            }
          break;
 

        case VFF_TYP_4_BYTE :
          k = img1->row_size*img1->col_size;
          mpi = (int *)(img1->maps);
          if (!mflg)
            {
              i2 = (int *)(img2->imagedata);
              for (i=0; i<img2->num_data_bands; i++)
                {
                  if (img1->data_storage_type == VFF_TYP_1_BYTE)
                    c1 = (unsigned char *)(img1->imagedata);
                  else
                    i1 = (int *)(img1->imagedata);
                  for (j=0; j<k; j++)
                    {
                      if (img1->data_storage_type == VFF_TYP_1_BYTE)
                        mapind = i*img1->map_col_size + *c1++;
                      else
                        mapind = i*img1->map_col_size + *i1++;
                      e = (double)(mpi[mapind]) - (double)(*i2++);
                      err += e*e;
                      count++;
                    }
                }
            }
          else
            {
              i2 = (int *)(img2->imagedata);
              for (i=0; i<img2->num_data_bands; i++)
                {
                  if (img1->data_storage_type == VFF_TYP_1_BYTE)
                    c1 = (unsigned char *)(img1->imagedata);
                  else
                    i1 = (int *)(img1->imagedata);
                  i3 = (int *)(img3->imagedata);
                  for (j=0; j<k; j++)
                    {
                      if (*i3++)
                        {
                          if (img1->data_storage_type == VFF_TYP_1_BYTE)
                            mapind = i*img1->map_col_size + *c1++;
                          else
                            mapind = i*img1->map_col_size + *i1++;
                          e = (double)(mpi[mapind]) - (double)(*i2++);
                          err += e*e;
                          count++;
                        }
                      else
                        {
                          if (img1->data_storage_type == VFF_TYP_1_BYTE) c1++;
                          else i1++;
                          i2++;
                        }
                    }
                }
            }
          break;
 

        case VFF_TYP_FLOAT :
          k = img1->row_size*img1->col_size;
          mpf = (float *)(img1->maps);
          if (!mflg)
            {
              f2 = (float *)(img2->imagedata);
              for (i=0; i<img2->num_data_bands; i++)
                {
                  if (img1->data_storage_type == VFF_TYP_1_BYTE)
                    c1 = (unsigned char *)(img1->imagedata);
                  else
                    i1 = (int *)(img1->imagedata);
                  for (j=0; j<k; j++)
                    {
                      if (img1->data_storage_type == VFF_TYP_1_BYTE)
                        mapind = i*img1->map_col_size + *c1++;
                      else
                        mapind = i*img1->map_col_size + *i1++;
                      e = (double)(mpf[mapind]) - (double)(*f2++);
                      err += e*e;
                      count++;
                    }
                }
            }
          else
            {
              f2 = (float *)(img2->imagedata);
              for (i=0; i<img2->num_data_bands; i++)
                {
                  if (img1->data_storage_type == VFF_TYP_1_BYTE)
                    c1 = (unsigned char *)(img1->imagedata);
                  else
                    i1 = (int *)(img1->imagedata);
                  f3 = (float *)(img3->imagedata);
                  for (j=0; j<k; j++)
                    {
                      if (*f3++)
                        {
                          if (img1->data_storage_type == VFF_TYP_1_BYTE)
                            mapind = i*img1->map_col_size + *c1++;
                          else
                            mapind = i*img1->map_col_size + *i1++;
                          e = (double)(mpf[mapind]) - (double)(*f2++);
                          err += e*e;
                          count++;
                        }
                      else
                        {
                          if (img1->data_storage_type == VFF_TYP_1_BYTE) c1++;
                          else i1++;
                          f2++;
                        }
                    }
                }
            }
          break;

        case VFF_TYP_COMPLEX :
          kerror("vipl", "lvqerr", "Unsupported data type!\n");
          return(FALSE);
          /* break; */

        default:
          kerror("vipl", "lvqerr",
		 "Unknown data storage type, %d\n",img1->data_storage_type);
          return(FALSE);
          /* break; */
      }
      err = err / count;
      *rms = sqrt(err);
      return(TRUE);
}
/* -library_code_end */
