/*
  * Khoros: $Id: cr_comment.c,v 1.6 1996/04/05 00:56:15 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: cr_comment.c,v 1.6 1996/04/05 00:56:15 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		File Contents Title
   >>>>		
   >>>>   Public:
   >>>>                 create_comment()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */
#include "internals.h"

/************************************************************
*
*  Routine Name: create_comment - create a new comment field for an image file
*
*       Purpose: creates a new comment field from the original comment
*		 field and the filename comment that it came from.  The
*		 "image" field is of size 512, which means we can fit
*		 255 characters and 1 end of string character. The
*		 routine fits as much of the "filename" into "image"
*		 and if there is any left it fills it with the "original"
*		 comment field.
*
*  		 This code was taken from khoros-libvrgbimage 
*		 written by Mark Young, and modified.
*
*         Input: filename - a comment field describing where the new image
*			    came from.
*	         original - the original image's comment field
*
*        Output: image -  the new image comment field
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions:
*    Written By: Tom Sauer
*          Date: 5/27/89
*      Verified:
*  Side Effects:
* Modifications:
*************************************************************/
int create_comment(char *image, char *filename, char *original)
{
	size_t length1, length2;

	/*
	 * Copy the filename comment into the image's comment field.
	 * Make sure the filename comment is not greater than 512,
	 * the size of the comment field.
	 */

	length1 = kstrlen(filename);
	if (length1 >= 512)
	{
	   (void) kstrncpy(image,filename,(size_t)511);
	}
	else
	{
	   (void) kstrcpy(image, filename);

	   length2 = kstrlen(original);
	   if ((length1 + length2) >= 512)
	   {
	      /*
	       * Compute the amount of the original comment field we can fit 
	       * into the new image's comment field.  The result is stored in
	       * length1, which is really one less than the max since we need
	       * to make room for the end of string character '\0'.
	       */
	      length1 = length2 - ((length1 + length2) - 511);
	      (void) kstrncat(image, original, (size_t)length1);
	   }
	   else
	      (void) kstrcat(image, original);
	}
	return(TRUE);
}
