/*
  * Khoros: $Id: signal.c,v 1.5 1996/04/05 00:56:22 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: signal.c,v 1.5 1996/04/05 00:56:22 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		File Contents Title
   >>>>		
   >>>>  Private:
   >>>>                 gpwl()
   >>>>   Public:
   >>>>                 FMODULO()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	


/************************************************************
*
*  Routine Name: FMODULO - floating point mod function.
*
*       Purpose: Implements a "floating point" mod function that can 
*                handle remainders that are non integer.
*
*         Input: x - first argument of mod
*                y - second argument of mod
*        Output:
*       Returns: (float)(x mod y)
*  Restrictions:
*    Written By: Jeremy Worley
*          Date: February 1990
*      Verified:
*  Side Effects:
* Modifications: March 1990 - Casted x and y so that they are "type
*                             independant"
*   Declaration: float FMODULO(
* 		 !   float x,
*                !   float y)
*
*************************************************************/
#define FMODULO(x,y) ((float)((double)x-(double)y*(int)((double)x/(double)y)))

/************************************************************
*
*  Routine Name: gpwl - generate a 2d piecewise linear data set.
*
*       Purpose: Generates a 2d "piecewise linear" data set.  By properly
*                setting arguments to this routine, you can generate
*                impulse data (spikes), triangular waves, sawtooth waves,
*                reverse sawtooth waves, and square waves.
*
*         Input: vect     - where the data set goes
*                rows     - number of rows
*                cols     - number of columns
*                sample   - sampling frequency
*                minimum  - minimum value of data set.
*                maximum  - maximum value of the data set.
*                rperiod  - period of function in the row direction
*                cperiod  - period of function in the column direction
*                rrise    - rise time of function in the row direction
*                crise    - rise time of function in the column direction
*                rwidth   - width (when high) in the row direction
*                cwidth   - width (when high) in the column direction
*
*        Output: argument4 - explanation
*                argument5 - explanation
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*  Restrictions:
*    Written By: Jeremy Worley
*          Date: April 23, 1990
*      Verified:
*  Side Effects:
* Modifications: Thu Oct 18 15:16:34 MDT 1990 Jeremy Worley
*			Fixed a bug that made the fall part of the pulse
*			start at a value less than maximum and drop into
*			negative values:
*				(section-rwidth) --> (section-rwidth-rrise)
*				(section-cwidth) --> (section-cwidth-crise)
*************************************************************/
int gpwl(
   float *vect,
   int rows,
   int cols,
   float sample,
   float minimum,
   float maximum,
   float rperiod,
   float cperiod,
   float rrise,
   float crise,
   float rfall,
   float cfall,
   float rwidth,
   float cwidth)
{
    int r,c;
    float ramp,camp,amp,section,rpart,cpart;

  amp = maximum - minimum;

  if(rows==1){
     ramp=0.0;
  }else
     ramp=amp;

  if(cols==1){
     camp=0.0;
     ramp=amp;
  }else
     camp=amp;


    for(r=0;r<rows;r++){
        for(c=0;c<cols;c++){
/*
** rows
*/
            section = (FMODULO(r,(rperiod*sample))/sample);
            if(section<rrise && rrise!=0.0){
               rpart = section/rrise;
            }else if(section<(rrise + rwidth) && rwidth!=0.0){
               rpart = 1;
            }else if(section<(rrise+rfall+rwidth) && rfall!=0.0){
               rpart = 1.0 - (section-rwidth-rrise)/rfall; 
            }else{
               rpart = 0.0;
            }
/*
** columns
*/
            section = (FMODULO(c,(cperiod*sample))/sample);
            if(section<crise && crise!=0.0){
               cpart = section/crise;
            }else if(section<(crise + cwidth) && cwidth!=0.0){
               cpart = 1;
            }else if(section<(crise+cfall+cwidth) && cfall!=0.0){
               cpart = 1.0 - (section-cwidth-crise)/cfall;
            }else{
               cpart = 0.0;
            }
            
            *vect++ = ramp*rpart + camp*cpart + minimum;

        }/* end c loop */
    } /* end r loop */

    return(1);
}
