#! /bin/sh

echo "running test.sh for vconvert ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-4 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0

# ------------------------------------------------------------------------
#  set path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/images"
BINARY_PATH=${BINARY_PATH:-`kecho -tb RETRO -echo binpath`}

# ---------- Test 1 --> test description
 
test_num=1
descr="vconvert to byte"
${BINARY_PATH}/vconvert -i ./results/test_input.xv -t 'byte' -b 0 -o $tmpdir/out1.$$ -n 255 
report $test_num "$tmpdir/out1.$$" "./results/test1_results.xv" "$descr"
/bin/rm -f $tmpdir/out1.$$ 

test_num=2
descr="vconvert to short"
${BINARY_PATH}/vconvert -i ./results/test_input.xv -t 'short' -b 0 -o $tmpdir/out2.$$ -n 1000 
report $test_num "$tmpdir/out2.$$" "./results/test2_results.xv" "$descr"
/bin/rm -f $tmpdir/out2.$$ 

test_num=3
descr="vconvert to int"
${BINARY_PATH}/vconvert -i ./results/test_input.xv -t 'int' -b 0 -o $tmpdir/out3.$$ -n 1500 
report $test_num "$tmpdir/out3.$$" "./results/test3_results.xv" "$descr"
/bin/rm -f $tmpdir/out3.$$ 

test_num=4
descr="vconvert to float"
${BINARY_PATH}/vconvert -i ./results/test_input.xv -t 'float' -b 0 -o $tmpdir/out4.$$ -s 10 
report $test_num "$tmpdir/out4.$$" "./results/test4_results.xv" "$descr"
/bin/rm -f $tmpdir/out4.$$ 

test_num=5
descr="vconvert to complex"
${BINARY_PATH}/vconvert -i ./results/test_input.xv -t 'complex' -b 0 -o $tmpdir/out5.$$ -n 1 
report $test_num "$tmpdir/out5.$$" "./results/test5_results.xv" "$descr"
/bin/rm -f $tmpdir/out5.$$ 

test_num=6
descr="vconvert to double"
${BINARY_PATH}/vconvert -i ./results/test_input.xv -t 'double' -b 0 -o $tmpdir/out6.$$ -n 1 
report $test_num "$tmpdir/out6.$$" "./results/test6_results.xv" "$descr"
/bin/rm -f $tmpdir/out6.$$ 

# test_num=7
# descr="vconvert float to bit"
# cat <<EOF > $tmpdir/ascii.$$
# 0 1 
# 1 1 
# 0 0 
# 0 1 
# 0 1
# EOF
# kasc2val -i1 $tmpdir/ascii.$$ -wsize 2 -hsize 5 -type float -o $tmpdir/input.$$
# kformats -i $tmpdir/input.$$ -o $tmpdir/input.$$ -viff
# ${BINARY_PATH}/vconvert -i $tmpdir/input.$$ -t 'bit' -o $tmpdir/out.$$ -s 1
# cat <<EOF > $tmpdir/ascii.$$
# 0 1 
# 1 1 
# 0 0 
# 0 1 
# 0 1
# EOF
# kasc2val -i1 $tmpdir/ascii.$$ -wsize 2 -hsize 5 -type bit -o $tmpdir/input.$$
# kformats -i $tmpdir/input.$$ -o $tmpdir/input.$$ -kdf
# report $test_num "$tmpdir/out.$$" "$tmpdir/input.$$" "$descr"
# /bin/rm -f $tmpdir/out.$$  $tmpdir/input.$$ $tmpdir/ascii.$$
# 
# test_num=8
# descr="vconvert bit to byte"
# cat <<EOF > $tmpdir/ascii.$$
# 0 1
# 1 1
# 0 0
# 0 1
# 0 1
# EOF
# kasc2val -i1 $tmpdir/ascii.$$ -wsize 2 -hsize 5 -type 1 -o $tmpdir/input.$$
# kformats -i $tmpdir/input.$$ -o $tmpdir/input.$$ -viff
# ${BINARY_PATH}/vconvert -i $tmpdir/input.$$ -t 'byte' -o $tmpdir/out.$$ -s 1
# cat <<EOF > $tmpdir/ascii.$$
# 0 1
# 1 1
# 0 0
# 0 1
# 0 1
# EOF
# kasc2val -i1 $tmpdir/ascii.$$ -wsize 2 -hsize 5 -type 3 -o $tmpdir/input.$$
# kformats -i $tmpdir/input.$$ -o $tmpdir/input.$$ -kdf
# report $test_num "$tmpdir/out.$$" "$tmpdir/input.$$" "$descr"
# /bin/rm -f $tmpdir/out.$$  $tmpdir/input.$$ $tmpdir/ascii.$$
  
exit $status
