/*
  * Khoros: $Id: utilities.c,v 1.7 1996/04/24 19:45:45 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: utilities.c,v 1.7 1996/04/24 19:45:45 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            Diff Objects
   >>>>
   >>>>   Static:
   >>>>  Private:
   >>>>             generate_diffobj()
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "kdiffobj.h"
#include "kcms/private.h"


/*-----------------------------------------------------------
|
|  Routine Name: generate_files - generate the files to be diffed
|
|       Purpose: This routine does the actual work of creating the
|		 list of files to be diffed.
|
|         Input: program   - the object to generate the list of files for
|        Output: num	   - the number of files in the list
|       Returns: the list on success, NULL otherwise
|
|    Written By: Mark Young
|          Date: Sep 18, 1993
| Modifications:
|
------------------------------------------------------------*/

static char **generate_files(
   kobject  program,
   size_t  *num)
{
	kobject fileobj;
	klist  *list = NULL;
	size_t  numfiles = 0;
	char   *path, **files = NULL;
	long    type;


	kcms_get_attribute(program, KCMS_CMOBJ_ALL_FILES, &list);
	while (list != NULL)
	{
	   fileobj = klist_clientdata(list);
	   kcms_get_attribute(fileobj, KCMS_FOBJ_TYPE, &type);
	   if (type != KCMS_FOBJ_TYPE_DBM &&
	       type != KCMS_FOBJ_TYPE_CONFIG)
	   {
	      path = NULL;
	      kcms_get_attribute(fileobj, KCMS_FOBJ_FULLNAME, &path);
	      if (path != NULL)
	      {
		 files = karray_insert(files, KSTRING, kstrdup(path),
				numfiles++, KLIST_TAIL, TRUE);
	      }
	   }
	   list = klist_next(list);
	}
	files = karray_sort(files, KSTRING, numfiles, TRUE);
	*num = numfiles;
	return(files);
}


/*-----------------------------------------------------------
|
|  Routine Name: generate_imakefile - generate diff object
|
|       Purpose: This routine performs the actually diffing of the objects.
|		 This is done by getting the list of files returned from
|		 generate files.  The list are then 
|
|         Input: tbname - the toolbox name
|                oname  - the object name in which we are to re-gen the
|			  Imakefile
|		 dir_flag - if this is not an object, but a directory imakefile
|        Output: 
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By: Mark Young
|          Date: Sep 10, 1993
| Modifications:
|
------------------------------------------------------------*/

int generate_diffobj(
   char *tbname1,
   char *tbname2,
   char *oname)
{
	kobject tb1 = NULL, tb2 = NULL, obj1 = NULL, obj2 = NULL;
	int	i, status, value;
	ssize_t num = 0;
	size_t  indx1, indx2, num1, num2;
	char    **files1, **files2, **list1 = NULL, **list2 = NULL,
		**list = NULL, *tmp, temp1[KLENGTH], temp2[KLENGTH],
		*tmp1, *tmp2, temp[KLENGTH];


	if ((tb1 = kcms_open_toolbox(tbname1)) == NULL ||
	    (tb2 = kcms_open_toolbox(tbname2)) == NULL)
	{
	   kinfo(KSTANDARD, "Warning: Cannot open toolbox object '%s'\n",
			(!tb1) ? tbname1 : tbname2);
	   kexit(1);
	}

	if ((obj1 = kcms_open_cmobj(tb1, oname)) == NULL ||
	    (obj2 = kcms_open_cmobj(tb2, oname)) == NULL)
	{
	   kinfo(KSTANDARD, "Warning: Cannot open program object '%s' for \
toolbox '%s'\n", oname, (!obj1) ? tbname1 : tbname2);
	   kexit(1);
	}

	files1 = generate_files(obj1, &num1);
	files2 = generate_files(obj2, &num2);
	if (!files1 || !files2)
	{
	   kinfo(KSTANDARD, "\nDiff'ing '%s' for toolboxes '%s' and '%s'\n\n \
\tNo files found for toolbox '%s'", oname, tbname1, tbname2,
		!files1 ? tbname1 : tbname2);
	   return(FALSE);
	}

	indx1 = indx2 = 0;
	while (indx1 < num1 || indx2 < num2)
	{
	    /*
	     *  Set the tmp1 & tmp2 pointers to be empty.  We use these
	     *  pointers to hold the filenames in which we will perform
	     *  the comparisons.
	     */
	    tmp1 = tmp2 = "";

	    /*
	     *  If there are remaining files from the first object, then
	     *  initialize tmp1 & temp1 to point to it.
	     */
	    if (indx1 < num1)
	    {
	       tmp1 = kstrchr(files1[indx1], '/');
	       kbasename(files1[indx1], temp1);
	    }

	    /*
	     *  If there are remaining files from the second object, then
	     *  initialize tmp2 & temp2 to point to it.
	     */
	    if (indx2 < num2)
	    {
	       tmp2 = kstrchr(files2[indx2], '/');
	       kbasename(files2[indx2], temp2);
	    }

	    /*
	     *  See if we are looking at the same file.  If so then we can
	     *  check to see if the files are different.  If they are not
	     *  the same then we want to automatically put the file on the
	     *  appropriate list and indicate that it doesn't exist in the
	     *  other toolbox.
	     */
	    status = kstrcmp(tmp1, tmp2);
	    if (status == 0)
	    {
	       kinfo(KDEBUG,"Diff'ing files: '%s' '%s'", files1[indx1],
			   files2[indx2]);
	       if (kcms_checkfile(tb1,tb2,files1[indx1],files2[indx2]) == TRUE)
	       {
	          tmp = kstring_copy(temp1, NULL);
	          list1 = karray_insert(list1, KSTRING, files1[indx1], (size_t) num, KLIST_TAIL, TRUE);
	          list2 = karray_insert(list2, KSTRING, files2[indx2], (size_t) num, KLIST_TAIL, TRUE);
	          list = karray_insert(list, KSTRING, tmp, (size_t) num, KLIST_TAIL, TRUE);
		  num++;
	       }
	       indx1++; indx2++;
	    }
	    else if (kstrlen(tmp1) != 0 && (status < 0 || kstrlen(tmp2) == 0))
	    {
	       list1 = karray_insert(list1, KSTRING, files1[indx1],(size_t)num,KLIST_TAIL,TRUE);
	       list2 = karray_insert(list2, KSTRING, NULL,(size_t)num,KLIST_TAIL,TRUE);

	       ksprintf(temp,"\t%s (exists only in '%s')", temp1, tbname1);
	       tmp = kstring_copy(temp, NULL);
	       list = karray_insert(list, KSTRING, tmp, (size_t) num, KLIST_TAIL, TRUE);
	       num++;
	       indx1++;
	    }
	    else
	    {
	       list1 = karray_insert(list1, KSTRING, NULL, (size_t)num,KLIST_TAIL,TRUE);
	       list2 = karray_insert(list2, KSTRING, files2[indx2], (size_t)num, KLIST_TAIL, TRUE);

	       ksprintf(temp,"\t%s (exists only in '%s')", temp2, tbname2);
	       tmp = kstring_copy(temp, NULL);
	       list = karray_insert(list, KSTRING, tmp, (size_t)num, KLIST_TAIL, TRUE);
	       num++;
	       indx2++;
	    }
	}
	kcms_close(obj1);
	kcms_close(obj2);

	if (num == 0)
	{
	   kinfo(KSTANDARD, "\nDiff'ing '%s' for toolboxes '%s' and '%s'\n\n \
\tNo files found to be different", oname, tbname1, tbname2);
	   return(TRUE);
	}

	i = -1;
	value = 0;
	do
	{
	    ksystem("clear");
	    if (i == -1)
	    {
	       kinfo(KSTANDARD, "\nDiff'ing '%s' for toolboxes '%s' and '%s'\n",
		oname, tbname1, tbname2);

	       if (value >= num) value = 0;
	       kchoose(KFORCE, list, (int) num, value+1, NULL, &i,
			"Toolbox: '%s' '%s'", tbname1, tbname2);
	    }
	    value = i;

	    if (i != 0)
	    {
	       if (list1[i-1] != NULL && list2[i-1])
	       {
		   kprintf("\nDiff'ing files: '%s' '%s'\n\n", list1[i-1],
			   list2[i-1]);
	           ksprintf(temp, "diff %s %s | more", list1[i-1], list2[i-1]);
	           ksystem(temp);
		}
		else
		{
		   if (!list1[i-1])
		      kprintf("%s is only available in toolbox '%s'\n",
				list2[i-1], tbname2);
		   else
		      kprintf("%s is only available in toolbox '%s'\n",
				list1[i-1], tbname1);
		}

	        /* CONSTCOND */
	        while (1)
	        {
	           kprintf("\n\nEnter a number of the next file to diff or:\n");
		   kprintf("<   file %s from %s to %s\n", list[i-1],
			tbname1, tbname2);
		   kprintf(">   file %s from %s to %s\n", list[i-1],
			tbname2, tbname1);
		   kprintf("r to REPEAT DIFF   <cr> for FILE LIST   q to \
QUIT...  ");
	           kgets(temp);
	           kstring_cleanup(temp, temp);

		   if (kstrlen(temp) == 0)
		   {
		      i = -1;
		      break;
		   }
	           else if (kstrcmp(temp, "q") == 0)
		   {
		      i = 0;
		      break;
		   }
	           else if (kstrcmp(temp, "r") == 0)
		   {
		      ksystem("clear");
		      kprintf("Diff'ing files: '%s' '%s'\n\n", list1[i-1],
				list2[i-1]);
		      ksprintf(temp, "diff %s %s | more",list1[i-1],list2[i-1]);
	              ksystem(temp);
		   }
	           else if (kstrcmp(temp, ">") == 0)
		   {
		     if (koverwrite(KFORCE, list1[i-1]) == TRUE)
			kcms_copyfile(tb2, tb1, list2[i-1], list1[i-1]);
		   }
	           else if (kstrcmp(temp, "<") == 0)
		   {
		     if (koverwrite(KFORCE, list2[i-1]) == TRUE)
			kcms_copyfile(tb1, tb2, list1[i-1], list2[i-1]);
		   }
	           else if (ksscanf(temp," %d", &value) == 1 && value >= 0
			    && value <= num)
		   {
		      i = value;
		      break;
		   }
	           else
		   {
		      kinfo(KSTANDARD,"\nInvalid selection... '%s'\n\n", temp);
		   }
	        }
	    }
	} while (i != 0);
	kcms_close(tb1);
	kcms_close(tb2);
	return(TRUE);
}
