: # use perl
        eval 'exec perl -S $0 "$@"'
                if $runnning_under_some_shell;

# Khoros: $Id: kmkcontents.pl,v 1.5 1996/04/06 06:09:29 mirage Exp $

# Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
# All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.



#========================================================================
#
#  Program Name: kmkcontents - Make a manual volume table of contents
#
#       Purpose: This perl script generates a table of contents for
#                a set of volumes.  For instance, if a given toolbox
#                has 4 volumes, kmkcontents will make a table of
#		 contents listing each volume.
#
#     Arguments: argument1 - explanation
#                argument2 - explanation
#                argument3 - explanation
#
#   Exit Status: True (1) on success, False (0) otherwise
#
#      Comments: Explanation of algorithm if applicable
#    Written By: Neil Bowers
#          Date: 
#      Verified:
#  Side Effects:
# Modifications:
#
#========================================================================

@khoros_toolbox = ('KHOROS_TOOLBOX');
if (! eval("\$toolboxes = \$ENV{\"KHOROS_TOOLBOX\"}"))
{
   die "The KHOROS_TOOLBOX environment variable must be set.\n";
}

$notfound = 1;
@toolbox_list = split(':', $toolboxes);
foreach $toolbox_file (sort(@toolbox_list))
{
   if ( ! open(TBS,"<$toolbox_file"))
   {
      print "Can not open $toolbox_file.  Please check to make sure your\n";
      die   "KHOROS_TOOLBOX environment variable points to existing files.\n";
   }

   while(<TBS>)
   {
      chop($_);
      next if /^\s*#/;
      @line = m/^([^:]+):([^\;]+)\;?(.*)$/;

      if ($line[1] =~ /kfile:/ || $tbpath =~ /kexec:/)
      {
         print "$line[0] is a remote toolbox.  It is not added to list\n";
         print "because remote toolboxes not supported at this time.\n";
      }
      else
      {
         push(@INC, "$line[1]/repos/perl") if $line[0] eq "SRCMACH";

         if ($line[0] eq "SUPPORT")
         {
            if ( $notfound )
            {
               $support_path = $line[1];
               $notfound = 0;
            }
            else
            {
               print "The SUPPORT toolbox is defined twice.  Please check your\n";
               print "KHOROS_TOOLBOX environment variable and the file(s)\n";
               print "pointed to by it and make sure that all toolboxes are\n";
               die   "only defined once.\n";
            }
         }
      }
   }
}

if ( $notfound )
{
   print "The SUPPORT toolbox is not defined in any of the files pointed to\n";
   die   "by the environment variable KHOROS_TOOLBOX.\n";
}

@INC = ("$support_path/repos/perl", @INC);
require "khoros.pl";

$whatis = 'create a manual table of contents for all volumes';

@kmkcontents_args = (
		     'tb',         'toolbox',  'name of toolbox',
		     '[verbose]',  '',         'verbose reporting'
		    );

# initialize Khoros perl lib
&khoros'initialize('SUPPORT', $whatis, @kmkcontents_args);

@volume_list = &khoros'parse_args(@ARGV);	# parse command line args ';
$toolbox = "\U$opt_tb";			# toolbox name, upper case
if (@volume_list == 0)
{
   ($volname = $toolbox) =~ s/^(.)(.*)/\U$1\E\L$2\E/;
   @volume_list = ($volname);
}

$doc_path = $khoros'toolboxPath{"SUPPORT"};      # get doc toolbox path ';
$tb_path = $khoros'toolboxPath{$toolbox};	# get path to toolbox ';

$manual_home = "$tb_path/manual";		# set path to toolbox manual

&CreateContentsDirectory();

foreach $volume (@volume_list)
{
   &GenerateVolumeContents($toolbox, $volume);
}

&GenerateContentsImakefile();

#
# GenerateVolumeContents() - grab titles from all chapters within volume
#
sub GenerateVolumeContents
{
   local($toolbox, $volume) = @_;
   local($tbpath);
   local(*VOLDIR);
   local($vol_path);
   local(@chapters);
   local($chapter);
   local($mandir);


   print STDERR "Processing volume \"$volume\"\n" if $opt_verbose;

   # full path to manual directory of toolbox
   $mandir = $khoros'toolboxPath{$toolbox}.'/manual'; #';

   # full path to directory for volume
   $vol_path = "$mandir/$volume";

   # get list of sections in the volume
   opendir(VOLDIR, $vol_path) || do
   {
      die "Unable to read volume directory $vol_path: $!\n";
   };
   @chapters = sort grep(/^[^.]/, readdir(VOLDIR));
   closedir(VOLDIR);

   $got_vol_name = 0;

   open(CONTENTS,"> $mandir/contents/$volume.sec");

   foreach $chapter (@chapters)
   {
      next if $chapter eq 'CVS';
      next if -f "$vol_path/$chapter";

      $man_file_name = "$vol_path/$chapter/$chapter.man";
      unless (-f $man_file_name)
      {
	 warn "no .man file found for $chapter: $!\n";
	 next;
      }

      open(MANFILE,"< $man_file_name") || do
      {
	 warn "Unable to read .man file for $chapter ($man_file_name): $!\n";
	 next;
      };

      while (<MANFILE>)
      {
	 if (/^\.\s*chapter\s+(.*)$/)
	 {
	    $arglist = $1;
	    undef @args;
	    while ($arglist =~ /^"([^""]*)"\s*(.*)$/
		   || $arglist =~ /^(\S+)\s*(.*)$/)
	    {
	       push(@args, $1);
	       $arglist = $2;
	    }

	    unless ($got_vol_name)
            {
	       print CONTENTS ".bp\n";
	       print CONTENTS ".ps 18\n";
	       print CONTENTS ".ce\n";
	       print CONTENTS "\\fB$args[1]\\fP\n";
	       print CONTENTS "\\ \n";
	       print CONTENTS ".sp 2\n";
	       print CONTENTS ".ps 16\n";
	       print CONTENTS ".paragraph\n";
	       $got_vol_name = 1;
	    }

            # Chapter stuff
            print CONTENTS ".IP \"\\fBChapter $args[2].\\fP\" 15\n";
            print CONTENTS "$args[3]\n";
            print CONTENTS ".in +0.25i\n";
            print CONTENTS "\\fI$args[4] $args[5]\\fP\n";
            print CONTENTS ".in -0.25i\n";
            print CONTENTS ".sp\n";
	 };

	 #
	 # Chapter is an appendix
	 #
	 if (/\.\s*appendix\s+(.*)$/)
	 {
	    $arglist = $1;
	    undef @args;
	    while ($arglist =~ /^"([^""]*)"\s*(.*)$/
		   || $arglist =~ /^(\S+)\s*(.*)$/)
	    {
	       push(@args, $1);
	       $arglist = $2;
	    }

	    print CONTENTS ".IP \"\\fBAppendix $args[2].\\fP\" 15\n";
	    print CONTENTS "$args[3]\n";
	    print CONTENTS ".in +0.25i\n";
	    print CONTENTS "\\fI$args[4] $args[5]\\fP\n";
	    print CONTENTS ".in -0.25i\n";
	    print CONTENTS ".sp\n";
	 };
      }
      close MANFILE;
   }
   print CONTENTS ".IP \"\\fBGlossary\\fP\" 15\n";
   print CONTENTS ".sp\n";
   print CONTENTS ".IP \"\\fBIndex\\fP\" 15\n";

   close(CONTENTS);
}

#
# CreateContentsDirectory() - create contents directory for toolbox
#
# Check whether the contents directory exists for the toolbox
# If not, then create it
#
sub CreateContentsDirectory
{
   $contents_dir = "$manual_home/contents";
   unless (-d $contents_dir)
   {
      if ($opt_verbose)
      {
	 print STDERR "creating contents directory $contents_dir\n";
      }
      mkdir($contents_dir, 0755) || do
      {
	 die "Unable to create contents directory $contents_dir: $!\n";
      };
   }

   if (-f "$contents_dir/contents.man")
   {
      if ($opt_verbose)
      {
	 print STDERR "creating template contents file contents.man\n";
      }
      rename("$contents_dir/contents.man", "$contents_dir/contents.man.bak");
   }

   open(MANFILE, "> $contents_dir/contents.man") || do
   {
   };

   while (<DATA>)
   {
      print MANFILE;
   }
   print MANFILE ".paragraph\n";
   foreach $volume (@volume_list)
   {
      print MANFILE ".include $volume.sec\n";
   }
   close MANFILE;
   close DATA;
}

sub GenerateContentsImakefile
{
   local($contents_dir);


   if ($opt_verbose)
   {
      print STDERR "creating Imakefile and Makefile ";
      print STDERR "for contents directory\n";
   }
   $contents_dir = "$manual_home/contents";
   system("cd $contents_dir; kgenimake -tb support -manual");
}

#
# Everything after the __END__ line is read from the special DATA filehandle
# This is the template which goes at the top of the contents file
#
__END__
.\" =====================================================================
.\" ==
.\" == DO NOT MODIFY THIS FILE
.\" ==
.\" == This file is autogenerated by kmkcontents.
.\" ==
.\" =====================================================================
.nr  FM 1.0i
.nr  HM 1.0i
.nr  PS 11
.nr  VS 13
.ds LH
.ds RH
.ds CH
.ds LF
.ds RF
.ds CF
.LP
